;
;***********************************************************************\
;                                                                       *
; MSM.INC                                                               *
;                                                                       *
; General definitions for use with the MSM Module.                      *
;                                                                       *
; Written by:   DFS                                                     *
; Date:         November 1990                                           *
;                                                                       *
; Modifications:							*
;									*
; 11-04-94 JCP  -Removed 0 Dup(0).                                      *
;									*
; 17 October 1994 11:19	DGM						*
;	Also modified MLIDStatusFlags (SHARE_INT7) and (SHARE_INT15) to *
;	allow for use of Int 7 & Int 15 shared interrupts, 		*
;	ie. Disable PIC if Int 7/15 & shared, this will prevent Lost 	*
;	interrupts with adapters sharing Int on these values.		*
;									*
;	This would have been a problem if PIC, PnP, PCMCIA, etc. had 	*
;	auto-configured the adapters to these interrupts.		*
;									*
; 19 August 1996 9:58  JRT						*
;	Modified MLIDStatusFlags, POLLSUSPENDED flag added. This	*
;	allows driver to temporarily stop polling and resume it at a	*
;	later time.							*
;                                                                       *
; 09 Sept 1996 01:40  LON						*
;	Added TSM_CONFIG_TABLE and MSM_CONFIG_TABLE structures, etc.	*
;									*
; 18 September 1996 14:36  JRT						*
;	Added timer structure and equates.				*
;									*
; 25 September 1996 AYD 						*
;	Added ExtraConfig structure					*
;									*
; 22 October 1996  12:04  JRT						*
;	Added for second driver ISR support.				*
;									*
; 05 November 1996  16:00  MPK						*
;       Added defines for FAILURE_CRITICAL and FAILURE_FATAL		*
;                                                                       *
; 18 November 1996  08:23  JRT                                          *
;       Added UNLOADING to MLIDStatusFlags                              *
;                                                                       *
; 06 December 1996 16:00  LON						*
;	Added new field, TSMCFG_SystemFlags, to TSM_CONFIG_TABLE        *
;                                                                       *
; 16 January  1996 10:00  LON						*
;       Added new equate,MSMPriorityTxFreeCount, so HSMs can access it. *
;                                                                       *
; 26 February 1997 17:11  JRT                                           *
;       Added MSMHardwareFailure notify type codes.                     *
;                                                                       *
; 05 June 1997 17:48  JRT                                               *
;       Added RETURNING_RESOURCES to MLIDStatusFlags                    *
;                                                                       *
; 22 July 1997 15:00  LON                                               *
;       Added RPhysicalAddress as a negative offset from start of RCB.  *
;                                                                       *
; 30 July 1997 15:00  LON                                               *
;       Added SpareRPhysicalAddress as a negative offset from start     *
;       of Spare ECBs.                                                  *
;                                                                       *
; 03 October 1997 09:10  JRT                                            *
;       Removed absolute path to include of odi.inc.                    *
;       Remove RPhysicalAddress as negative offset from start of RCB.   *
;                                                                       *
; 05 January 1998 LON                                                   *
;       Added the left out field TSMCFG_Reserved to TSM_CONFIG_TABLE    *
;                                                                       *
; 05 March 1998 mcd                                                     *
;       Added MSMFlushWriteCombine macro.                               *
;                                                                       *
;       (c) Copyright 1991 - 1997 Novell, Inc. All rights reserved.     *
;***********************************************************************/
;
;;include \odi32\inc\aspec331\odi.inc
include odi.inc
;

;*******************************************************************\
;  The MPK (Moab) version of the LSL allocates RCBs and saves      *
;  the physical adr of the RCB in the LSL header that is appended  *                         *
;  before the RCB itself. This saves having to call the OS to      *
;  convert a logical adr to a physical.  The LSL uses the include  *
;  file REQBUF.INC to define its equate.      970722 LON           *
;*******************************************************************/
LSLHeaderRCB            struc
  RCBStart              dd      0               ;
LSLHeaderRCB            ends

;;RPhysicalAddress equ    RCBStart - (76)         ; Negative offset from RCB


;*********************************************************************\
;  To speed up obtaining the physical address of the MSM's SpareECBs *
;  the physical adr is appended before the Spare ECB in procedure    *
;  MSMRegisterHardwareOptions. This saves having to call the OS to   *
;  convert a logical adr to a physical. Note this structure must be  *
;  identical to ReceiveBufferStructure except for SparePhysAdr.      *
;                                                    970729 LON      *
;*********************************************************************/
SpareECBStruc           struc
        SparePhysAdr                    dd      ?
	SpareRLink			dd	?
	SpareRBLink			dd	?
	SpareRStatus			dw	?
	SpareRESRAddress		dd	?
	SpareRLogicalID                 dw	?       ; CSpec111 ECB_StackID
	SpareRProtocolID		db	6 dup (?)
	SpareRBoardNumber		dd	?
	SpareRImmediateAddress	db	6 dup (?)
	SpareRDriverWorkSpace	dd	?
	SpareRProtocolWorkspace	db	8 dup (?)
	SpareRPacketLength		dd	?
	SpareRFragmentCount		dd	?       ; always 1 for Rcvs
	SpareRPacketOffset		dd	?
	SpareRPacketSize		dd	?
SpareECBStruc           ends

SpareRPhysicalAddress   equ    0 - (1 * 4)    ; Negative offset from the real
                                              ;   ECB start (RLink) to
                                              ;    SparePhysAdr.
;
;**************************************************************\
;  The following equates are used in the MSMHardwareFailure    *
;  Procedure.                                                  *
;**************************************************************/
FAILURE_CRITICAL	EQU	1
FAILURE_FATAL		EQU	0


DriverAdapterSpace      struc

  DriverAdapterStart    dd      0               ; Use for MSM equates.

DriverAdapterSpace      ends

;***************************************************************\
;                                                               *
; MSMTxFreeCount should be used as an index from ebp by the     *
; driver to manipulate the MSM's MSMTxFreeCount. The driver     *
; should execute the following command at interrupt             *
; time(TxComplete) to increment MSMTxFreeCount:                 *
;                                                               *
;       inc     [ebp].MSMTxFreeCount                            *
;                                                               *
;***************************************************************/
;
MSMTxFreeCount  equ     DriverAdapterStart - (1 * 4)
;
;***************************************************************\
;                                                               *
; The following flag allows the driver to determine whether the *
; driver is shutdown and whether the MSM contains any transmit  *
; packets in its queue. If bit 0 is set, the driver is          *
; temporarily shutdown. The MSM will not attempt to send a      *
; packet to the drivers DriverSend routine if the card is       *
; temporarily shutdown. If bit 1 is set, the MSM has at least   *
; one packet on its queue. The driver should use the following  *
; command to test this flag:                                    *
;                                                               *
;       test    [ebp].MSMStatusFlags, SHUTDOWN                  *
;                                                               *
;***************************************************************/
;
MSMStatusFlags  equ     DriverAdapterStart - (2 * 4)
;
;***************************************************************\
;                                                               *
; MSMVirtualBoardLink Table.                                    *
;                                                               *
;***************************************************************/
;
MSMVirtualBoardLink     equ     DriverAdapterStart - (6 * 4)
        page
;
;***************************************************************\
;                                                               *
; MSMInstanceLink.                                              *
;                                                               *
;***************************************************************/
;
MSMInstanceLink         equ     DriverAdapterStart - (7 * 4)
;
;***************************************************************\
;                                                               *
; MSMInCriticalSection.                                         *
;                                                               *
;***************************************************************/
;
MSMInCriticalSection    equ     DriverAdapterStart - (8 * 4)
;
;***************************************************************\
;                                                               *
; MSMInterruptNumber.                                           *
;                                                               *
;***************************************************************/
;
MSMInterruptNumber      equ     DriverAdapterStart - (9 * 4)
;
;***************************************************************\
;                                                               *
; MSMDefaultVirtualBoard.                                       *
;                                                               *
;***************************************************************/
;
MSMDefaultVirtualBoard  equ     DriverAdapterStart - (10 * 4)
;
;***************************************************************\
;                                                               *
; Physical Node Address.                                        *
;                                                               *
;***************************************************************/
;
MSMPhysNodeAddress      equ     DriverAdapterStart - (16 * 4)
;
;***************************************************************\
;                                                               *
; Max Frame Header Size(Same as [ebx].MLIDLookAheadSize).       *
;                                                               *
;***************************************************************/
;
MSMMaxFrameHeaderSize   equ     DriverAdapterStart - (17 * 4)
;
;***************************************************************\
;                                                               *
; Pointer to RxErrorStatus(For Adapters that fill in the entire *
; receive ECB).                                                 *
;                                                               *
;***************************************************************/
;
MSMRxErrorStatusPtr     equ     DriverAdapterStart - (18 * 4)
;
;***************************************************************\
;                                                               *
; MSMInterrupt2Number.                                          *
;                                                               *
;***************************************************************/
;
MSMInterrupt2Number     equ     DriverAdapterStart - (19 * 4)
;
;***************************************************************\
;                                                               *
; MSMPriorityTxFreeCount should be used as an index from ebp by *
; the driver to initialize the MSM's Priority Send FreeCount.   *
; The diver must initialze this value (and never change it)     *
; before calling MSMRegisterMLID, who then calls EtherTSMInit   *
; which uses the value to allocate TCBs.                        *
;       mov     [ebp].MSMPriorityTxFreeCount, <number>          *
;                                                               *
;***************************************************************/
;
MSMPriorityTxFreeCount  equ     DriverAdapterStart - (42 * 4) ; matches CSPEC111
;
;***************************************************************\
;                                                               *
; TCB(Transmit Control Block) Structure.                        *
;                                                               *
;***************************************************************/
;
TCBStructure    struc
        TCBDriverWS             dd      3 dup (0)       ; Used by MSM or Driver to link TCBs.
        TCBDataLen              dd      ?               ; Total Frag lengths + media length.
        TCBFragStrucPtr         dd      ?               ; Location of ECB fragment struc.
        TCBMediaHeaderLen       dd      ?               ; Length of Media Length.
TCBStructure    ends

TCBMediaHeader  equ     offset TCBMediaHeaderLen + 4    ; Media Header Buffer.
;
;***************************************************************\
;                                                               *
; Fragment Descriptor Structure.                                *
;                                                               *
;***************************************************************/
;
FragmentStructure       struc
        FragmentOffset          dd      ?               ; Location of fragment.
        FragmentLength          dd      ?               ; Length of fragment.
FragmentStructure       ends
        page
;
;***************************************************************\
;                                                               *
; RCB(Receive Control Block) Structure.                         *
;                                                               *
;***************************************************************/
;
RCBStructure    struc
        RCBDriverWS             dd      2 dup (0)       ; Can be freely used by driver.
        RCBReserved             db      52 dup (0)      ; Reserved.
RCBStructure    ends

RCBDataBuffer           equ     RCBReserved + 52
RCBFragmentCount        equ     RCBDriverWS + 48
RCBFragmentOffset1      equ     RCBDriverWS + 48 + 4
RCBFragmentLength1      equ     RCBDriverWS + 48 + 8
;
;***************************************************************\
;                                                               *
; DriverBlockStructure.                                         * 
;                                                               *
;***************************************************************/
;
DriverBlockStructure            struc
        DPSize                          dd      0
        DPStackPointer                  dd      0
        DPModuleHandle                  dd      0
        DPBoardPointer                  dd      0
        DPAdapterPointer                dd      0
        DPConfigTemplate                dd      0
        DPFirmwareSize                  dd      0
        DPFirmwareBuffer                dd      0
        DPNumKeywords                   dd      0
        DPKeywordText                   dd      0
        DPKeywordTextLen                dd      0
        DPProcessKeywordTab             dd      0
        DPAdapterDataSpaceSize          dd      0
        DPAdapterDataSpaceTemplate      dd      0
        DPStatisticsTable               dd      0
        DPEndOfChainFlag                dd      0
        DPSendWantsECBs                 dd      0
        DPMaxMulticast                  dd      0
        DPNeedsBelow16Meg               dd      0
        DPAES                           dd      0
        DPCallBack                      dd      0
        DPISR                           dd      0
        DPMulticastChange               dd      0
        DPPoll                          dd      0
        DPReset                         dd      0
        DPSend                          dd      0
        DPShutdown                      dd      0
        DPTxTimeout                     dd      0
        DPPromiscuousChange             dd      0
        DPStatisticsChange              dd      0
        DPRxLookAheadChange             dd      0
        DPManagement                    dd      0
        DPEnableInterrupt               dd      0
        DPDisableInterrupt              dd      0
        DPISR2		                dd      0
        DPBReserved2                    dd      0       ; unused in ASM HSMs see CHSM spec
        HSMSpecVersionString            dd      0       ; set before MSMRegisterHardwareOptions
        DPDriverPriorityQueuePtr        dd      0
        DPDisableInterrupt2		dd	0
DriverBlockStructure            ends
;
;***************************************************************\
;                                                               *
; MediaBlockStructure.                                          *
;                                                               *
;***************************************************************/
;
MediaBlockStructure     struc
        MPSize                          dd      0
        MPConfigLimits                  dd      0
        MPFrameDescriptTable            dd      0
        MPProtocolIDArray               dd      0
        MPIDArray                       dd      0
        MPHeaderSizeArray               dd      0
        MPSendRoutineArray              dd      0
        MPAdapterDataSpaceSize          dd      0
        MPAdapterDataSpaceTemplate      dd      0
        MPAdjustRecvSize                dd      0
        MPInit                          dd      0
        MPReset                         dd      0
        MPShutdown                      dd      0
        MPDriverSend                    dd      0
        MPGetNextSend                   dd      0
        MPAddMulticast                  dd      0
        MPDeleteMulticast               dd      0
        MPNodeOverride                  dd      0
        MPAdjustNodeAddress             dd      0
        MPSetLookAheadSize              dd      0
        MPPromiscuousChange             dd      0
        MPRegisterReceiveMonitor        dd      0
MediaBlockStructure     ends
;
;***************************************************************\
;                                                               *
; MLIDStatusFlags bit values.                                   *
;                                                               *
;***************************************************************/
;
SHUTDOWN                equ     01h
TXQUEUED                equ     02h
SHARE_INT7		equ	04h	; set if Share Int on 7.
SHARE_INT15		equ	08h	; set if Share Int on 15.
POLLSUSPENDED		equ	10h	; set if polling suspended
UNLOADING               equ     20h     ; set if adapter unloading
RETURNING_RESOURCES     equ     40h     ; set if adapter being released
NEEDS_PHYS_CONTIG_LOG_FRAGS equ 80h     ; set if 3.0 spec suspeced DMA driver
                                        ; that need us to give him physically
                                        ; contifuous logical fragments on 
                                        ; NetWare 5
;
;
;***********************************************************************\
; TSM Configuration Table Definitions.
;***********************************************************************/
;
TSM_TABLE_MAJOR_VER     equ     1
TSM_TABLE_MINOR_VER     equ     0
;
;***********************************************************************\
; TSM Configuration Table Structure definition.
;***********************************************************************/
;
TSM_CONFIG_TABLE        struc
        TSMCFG_TableSize                dd      size TSM_CONFIG_TABLE
        TSMCFG_TableMajorVersion        db      TSM_TABLE_MAJOR_VER
        TSMCFG_TableMinorVersion        db      TSM_TABLE_MINOR_VER
        TSMCFG_ModuleMajorVersion       db      0
        TSMCFG_ModuleMinorVersion       db      0
        TSMCFG_ODISpecMajorVersion      db      ODI_ASPEC_MAJOR_VER
        TSMCFG_ODISpecMinorVersion      db      ODI_ASPEC_MINOR_VER
        TSMCFG_Reserved                 dw      0                  ;LON 980105
        TSMCFG_MaxFrameSize             dd      0
        TSMCFG_SystemFlags              dd      0
TSM_CONFIG_TABLE        ends
;
;
;***********************************************************************\
; MSM Configuration Table Definitions.
;***********************************************************************/
;
MSM_TABLE_MAJOR_VER     equ     1
MSM_TABLE_MINOR_VER     equ     0
;
;***********************************************************************\
; MSM Configuration Table System Flags definitions.
;***********************************************************************/
;
MSM_CFG_SERVER_BIT	equ     40000000h
MSM_CFG_CLIENT_BIT	equ     80000000h
;
;***********************************************************************\
; MSM Configuration Table Structure definition.
;***********************************************************************/
;
MSM_CONFIG_TABLE        struc
        MSMCFG_TableSize                dd      size MSM_CONFIG_TABLE
        MSMCFG_TableMajorVersion        db      MSM_TABLE_MAJOR_VER
        MSMCFG_TableMinorVersion        db      MSM_TABLE_MINOR_VER
        MSMCFG_ModuleMajorVersion       db      0
        MSMCFG_ModuleMinorVersion       db      0
        MSMCFG_ODISpecMajorVersion      db      ODI_ASPEC_MAJOR_VER
        MSMCFG_ODISpecMinorVersion      db      ODI_ASPEC_MINOR_VER
        MSMCFG_Reserved                 dw      0
        MSMCFG_MaxNumberOfBoards        dd      0
        MSMCFG_SystemFlags              dd      0       ; Server or Client
MSM_CONFIG_TABLE        ends
;***********************************************************************
;
;***********************************************************************

ExtraConfig			struc
	ExtraConfigNextLink	dd	0	; Link to the next Extra Config table
	ISRRoutine0		dd	0	; Pointer to the interrupt handler
	ISR0Reserved0		dd	0	; Reserved for use by the MSM
	ISR0Reserved1        	dd	0	; Reserved for use by the MSM
	ISR0Reserved2        	dd	0       ; Reserved for use by the MSM
	ISR0Reserved3        	dd	0       ; Reserved for use by the MSM
	ISRRoutine1		dd	0	; Pointer to the interrupt handler
	ISR1Reserved0        	dd	0	; Reserved for use by the MSM
	ISR1Reserved1        	dd	0       ; Reserved for use by the MSM
	ISR1Reserved2        	dd	0       ; Reserved for use by the MSM
	ISR1Reserved3        	dd	0       ; Reserved for use by the MSM
	
	ExIO_Link 		dd	0
	ExIO_SharingFlags	dw	0	
	ExIO_Slot		dw	0

	ExIO_IOPort0		dw	0
	ExIO_IORange0		dw	0
	ExIO_IOPort1		dw	0
	ExIO_IORange1		dw	0

	ExIO_MemoryDecode0	dd	0
	ExIO_Length0		dw	0
	ExIO_MemoryDecode1	dd	0
	ExIO_Length1		dw	0

	ExIO_Interrupt0		db	0
	ExIO_Interrupt1		db	0
	ExIO_DMALine0		db	0
	ExIO_DMALine1		db	0	

	ExIO_ResourceTag	dd	0
	ExIO_Config		dd	0
	ExIO_CommandString	dd	0
	ExIO_LogicalName	db	18 dup (0)

	ExIO_LinearMemory0	dd	0
	ExIO_LinearMemory1	dd	0

	ExIO_ChannelNumber	dw	0
	ExIO_BusTag		dd	0
	ExIO_ConfigMajorVer	db	0
	ExIO_ConfigMinorVer  	db	0
ExtraConfig			ends
				
;***********************************************************************
; Equates and structure used by MSMScheduleTimer and MSMCancelTimer
;***********************************************************************

AES_TYPE_PRIVILEGED_ONE_SHOT	equ	0
AES_TYPE_PRIVILEGED_CONTINUOUS	equ	1
AES_TYPE_PROCESS_ONE_SHOT	equ	2
AES_TYPE_PROCESS_CONTINUOUS	equ	3

TIMER_STRUCTURE		struc

  TimerNextLink		dd	0	; set and used by MSM
  TimerProcedurePtr	dd	0	; set by HSM (HSM procedure ptr)
  TimerType		dd	0	; set by HSM (timer type)
  TimerInterval		dd	0	; set by HSM (time in milliseconds)
  TimerContext		dd	0	; set and used by MSM
  TimerReserved		db	30 dup (0)	; MSM use only

TIMER_STRUCTURE		ends

;;JRT 970226 begin

; MSMHardwareFailure notify type codes to place in EAX

NOTIFY_FATAL            equ     0
NOTIFY_CRITICAL         equ     1
NOTIFY_DEGRADED         equ     2

;;JRT 970226 end

        page
;
;***********************************************************************\
;                                                                       *
;       Macro Definitions.                                              *
;                                                                       *
;***********************************************************************/
;
Message macro   MessageName, MessageString
        local   StringEnd, StringBegin
MessageName     db      StringEnd - StringBegin
StringBegin     db      MessageString
StringEnd       db      0
        endm

SLOW            macro
        push    eax
        in      al, 61h
        in      al, 61h
        in      al, 61h
        pop     eax

                endm

MSMGetCurrentTime       macro
        extrn   GetCurrentTime: near
        call    GetCurrentTime

                        endm

MSMServiceEvents        macro
        extrn   LSLServiceEvents: near
        call    LSLServiceEvents
                        endm

MSMServiceEventsAndRet  macro
        extrn   LSLServiceEvents: near
        jmp     LSLServiceEvents
                        endm

        page
MSMReturnRCB            macro
        extrn   MSMReturnRcvECB: near
        call    MSMReturnRcvECB
                        endm

MSMGetHardwareBusType   macro
        extrn   GetHardwareBusType: near
        call    GetHardwareBusType
                        endm

MSMAddProtocolID        macro
        extrn   LSLAddProtocolID: near
        call    LSLAddProtocolID
                        endm

MSMRescheduleLast       macro

        extrn   CRescheduleLast: near
        extrn   GetNestedInterruptLevel: near
        local   MSMCanNotReschedule

        call    GetNestedInterruptLevel
        or      eax, eax
        jne     short MSMCanNotReschedule
        call    CRescheduleLast
MSMCanNotReschedule:

                        endm

MSMStartCriticalSection macro
        inc     [ebp].MSMInCriticalSection
                        endm

MSMEndCriticalSection   macro
        dec     [ebp].MSMInCriticalSection
                        endm

MSMGetCriticalStatus    macro
        mov     eax, [ebp].MSMInCriticalSection
                        endm

MSMGetProcessorSpeedRating      macro
        extrn   GetProcessorSpeedRating: near
        call    GetProcessorSpeedRating
                                endm

MSMPSemaphore           macro   SemaphoreNumber
        extrn   CPSemaphore: near
        push    SemaphoreNumber
        call    CPSemaphore
        add     esp, (1 * 4)
                        endm

MSMVSemaphore           macro   SemaphoreNumber
        extrn   CVSemaphore: near
        push    SemaphoreNumber
        call    CVSemaphore
        add     esp, (1 * 4)
                        endm

MSMRealModeInterrupt    macro   InputParmStructure, OutputParmStructure
        extrn   DoRealModeInterrupt: near
        push    offset OutputParmStructure
        push    offset InputParmStructure
        call    DoRealModeInterrupt
        add     esp, (2 * 4)
                        endm

MSMGetRealModeWorkSpace macro   Semaphore, ProtectedAddress, RealModeSeg, RealModeOff, WorkSpaceSize
        extrn   GetRealModeWorkSpace: near
        push    offset WorkSpaceSize
        push    offset RealModeOff
        push    offset RealModeSeg
        push    offset ProtectedAddress
        push    offset Semaphore
        call    GetRealModeWorkSpace
        add     esp, (5 * 4)
                        endm

MSMGetSharedMemoryLinearAddress macro   SharedMemoryPhysical, Size
        extrn   GetSharedMemoryLinearAddress: near
        push    Size
        push    SharedMemoryPhysical
        call    GetSharedMemoryLinearAddress
        add     esp, (2 * 4)
                                endm

MSMReturnSharedMemoryLinearAddress      macro   SharedMemoryLinear
        extrn   ReturnSharedMemoryLinearAddress: near
        push    SharedMemoryLinear
        call    ReturnSharedMemoryLinearAddress
        add     esp, (1 * 4)
                                        endm
MSMDoEndOfInterrupt             macro
        extrn   DoEndOfInterrupt: near
        mov     ecx, [ebp].MSMInterruptNumber
        call    DoEndOfInterrupt
                                endm

MSMEnableHardwareInterrupt      macro
        extrn   EnableHardwareInterrupt: near
        mov     ecx, [ebp].MSMInterruptNumber
        call    EnableHardwareInterrupt
                                endm

MSMDisableHardwareInterrupt     macro
        extrn   DisableHardwareInterrupt: near
        mov     ecx, [ebp].MSMInterruptNumber
        call    DisableHardwareInterrupt
                                endm

MSMCheckHardwareInterrupt       macro
        extrn   CheckHardwareInterrupt: near
        mov     ecx, [ebp].MSMInterruptNumber
        call    CheckHardwareInterrupt
                                endm

MSMReturnNotificationECB        macro
        extrn   LSLHoldRcvEvent: near
        call    LSLHoldRcvEvent
                                endm

MSMFastReturnNotificationECB    macro
        extrn   LSLFastRcvEvent: near
        call    LSLFastRcvEvent
                                endm

MSMAllocRealModeStructuresOnStack  macro
        sub     esp, SIZE OutputParametersStructure
        mov     edi, esp
        sub     esp, SIZE InputParametersStructure
        mov     esi, esp
        push    edi
        push    esi
                                endm

MSMFreeRealModeStructuresOnStack        macro
        add     esp, RealModeStructureSize
        endm

MSMAllocRealModeStructuresOnStack32  macro
        sub     esp, SIZE OutputParametersStructure32
        mov     edi, esp
        sub     esp, SIZE InputParametersStructure32
        mov     esi, esp
        push    edi
        push    esi
                                endm

MSMFreeRealModeStructuresOnStack32      macro
        add     esp, RealModeStructureSize32
        endm

MSMECBPhysToLogFrags  macro     ECB

        mov     edi, [ECB].RESRAddress
        lea     edi, [edi].RFragmentCount
                                endm

MSMTCBPhysToLogFrags  macro     TCB

        mov     edi, [TCB].TCBFragStrucPtr
        mov     edi, [edi-RFragmentCount].RESRAddress   ; get original ECB
        lea     edi, [edi].RFragmentCount
                                endm

MSMFlushWriteCombine  macro
        push    eax
        mov     eax, CR2
        mov     CR2, eax
        pop     eax

                      endm

