;----------------------------------------------------------------------------
;   (C) Unpublished Work of Novell, Inc.  All Rights Reserved.
;
;	THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL, 
;	PROPRIETARY AND TRADESECRET INFORMATION OF NOVELL, INC.  ACCESS 
;	TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
;	NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS 
;	AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO APPROPRIATE 
;	LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED, PRACTICED,
;	PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,
;	ABBRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
;	TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL. 
;	ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD 
;	SUBJECT THE PERPERTRATOR TO CRIMINAL AND CIVIL LIABILITY.
;
;----------------------------------------------------------------------------
;*****************************************************************************
;MSM.INC
; 
;	General definitions for use with the MSM Module
;
;	Written by:	DCD
;	Date:		6/12/90
;
;	ODI Specification Version:	4.00
;
;    Modification History:
;	Date		By	Modification Description
;	---------------	---	------------------------------------------
;	01 August 1994   JWR,WTT  v2.00 Removed Previous history, 
;				See DOS ODI Specifcation Changes Doc for changes.
;
;	17 March 1995  JWR	Checkin New ToolKit. v2.03
;				Added Bit Definitions for MSMKeyWordFlags
;
; 1 April 1996	JWR	New ToolKit. v2.10
;*****************************************************************************
include	ODI.inc
include	NESL.inc

;----------------------------------------------------------------------------
;	Used for delays between I/O accesses to time sensitive ports.
;----------------------------------------------------------------------------
slow	macro
	push	ax
	in	al, 61h
	in	al, 61h
	in	al, 61h
	pop	ax
	endm

;----------------------------------------------------------------------------
;	Used to correctly restore the Interrupt Enable bit in the 
;	flags register if we are running on older versions of the
;	80286 processor.
;----------------------------------------------------------------------------
.popf	macro
	local	PopfLabel, JumpOverIRET

	push	cs
	call	near ptr PopfLabel
	jmp	short JumpOverIRET

PopfLabel:
	iret

JumpOverIRET:
	endm


;----------------------------------------------------------------------------
;	Transmit Control Block (TCB) structure definition.
;----------------------------------------------------------------------------
TCBStruc	struc
	TCBDriverWS		db	6 dup (?) ; Can freely be used by driver
	TCBDataLen		dw	?	  ; Len of frag's and media head added up
	TCBFragStrucPtr		dd	?	  ; Location of a FragStruc
	TCBMediaHeaderLen	dw	?	  ; Length of buffer following this TCB
;;	TCBMediaHeader		db	?	  ; This is the media header
TCBStruc	ends

TCBMediaHeader		equ	TCBMediaHeaderLen+2

;----------------------------------------------------------------------------
;	Fragment Descriptor Structure definition.
;----------------------------------------------------------------------------
FragmentStruc	struc
	FFragmentCount	dw	?		; Number of fragment descriptors following
	FFrag0Address	dd	?		; Location of first frag buffer
	FFrag0Length	dw	?		; Length of first frag buffer
FragmentStruc	ends

;----------------------------------------------------------------------------
;	Receive Control Block (RCB) structure definition.
;----------------------------------------------------------------------------
RCBStruc	struc
	RCBDriverWS		db	8 dup (?)   ; Can freely be used by driver
	RCBReserved		db	36 dup (?)  ; DO NOT TOUCH
	RCBFragCount		dw	?	    ; Number of fragment descriptors following
	RCBFrag0Addr		dd	?	    ; Location of first Rx frag buffer
	RCBFrag0Len		dw	?	    ; Length of first frag buffer
RCBStruc	ends

;----------------------------------------------------------------------------
;	SupportedMedias Structure Definition
;----------------------------------------------------------------------------
SupportedMediasStruc	struc
	SMS_MediaNameOffset	dw	0
	SMS_MediaID		dw	0
	SMS_DefaultPID		db 	6 dup(0)
SupportedMediasStruc	ends
	SMS_NumSupported    equ	(SMS_MediaNameOffset - 2) ; Number of MediaStrucs following.

;----------------------------------------------------------------------------
;	MSMSystemFlags bit definitions.
;----------------------------------------------------------------------------
ATFlag			equ	01h	; AT or later bit.
MCAFlag			equ	02h	; MCA Bus bit.
EISAFlag		equ	04h	; EISA Bus bit
I386Flag		equ	08h	; 386 CPU or greater bit.
PCIFlag			equ	10h	; PCI BIOS bit.
CSFlag			equ	20h	; Card Services is Present.
NESLFlag		equ	80h	; NESL is Present.

;----------------------------------------------------------------------------
;	MSMTxFlags0 bit definitions 
;	(This byte is for a critical section count)
;----------------------------------------------------------------------------
CRITICAL		equ	0FFh	; Set if MLID is in a critical section

;----------------------------------------------------------------------------
;	MSMTxFlags1 bit definitions.
;----------------------------------------------------------------------------
SHUTDOWN		equ	01h	; Set if MLID is shutdown
PERMSHUTDOWN		equ	02h	; Set if MLID is permanently shutdown
TXQUEUED		equ	04h	; Set if Tx queue is NOT empty

;----------------------------------------------------------------------------
;	MSMKeyWordFlags processing Bit defintions.
;----------------------------------------------------------------------------
CASE_SENSITIVE		equ	01h	; Keyword parameter is Case Sensitive.
CASE_SENSITIVE_ACTIVE	equ	80h	; Currently not upper casing letters.
CASE_SENSITIVE_DEACTIVATE equ	NOT CASE_SENSITIVE_ACTIVE ; Turn off bit.

;----------------------------------------------------------------------------
;	MSM ERROR CODES
;----------------------------------------------------------------------------
MSM_SUCCESSFUL		equ	0	; Successfully Initialized Driver, Media, ect.
MSM_PENDING_SUCCESS	equ	1	; Successful, but holding onto passed in resource (ECB)
MSMERR_HSM_FAILED	equ	01h	; The HSM found a problem don't load
MSMERR_TSM_FAILED  	equ	02h	; The TSM found a problem don't load
MSMERR_MSM_FAILED	equ	03h	; The MSM Found a Problem don't Load
MSMERR_CARDNOTPRESENT	equ	04h	; Everything was OK, but the Card is missing.
					; Used by PCMCIA Drivers ...

;----------------------------------------------------------------------------
;	Swap Table Start Address
;----------------------------------------------------------------------------
PSPTableStart	equ	55h

;----------------------------------------------------------------------------
;	DOS INT 21h Functions
;----------------------------------------------------------------------------
STD_OUTPUT	equ	1		; Standard output
ENTER_DOS       equ     21h             ; Dos Interrupt
MULTIPLEX_INT	equ	2Fh		; Multiplex Interrupt

PRINT_CHAR	equ	02h		; Print Char on Standard Output
PRINT_DSTR	equ	09h		; Print $ terminated string
GET_CURRENTDRIVE equ	19h		; Get Current Drive
SET_INTERRUPT	equ	2500h		; Set Interrupt Vector
DOS_VERSION	equ	30h		; Get DOS Version
TERM_AND_STAY	equ	3100h		; Terminate and stay resident
GETINDOSFLAG    equ     34h             ; Get address for Indos Flag
GET_INT_VEC	equ	35h		; Get Interrupt Vector
OPEN_FILE_RO	equ	3D00h		; Open a File for Reading Only
CLOSE_FILE	equ	3Eh		; Close File with Handle
READ_FILE	equ	3Fh		; Read File 
WRITE_DEVICE	equ	40h		; Write File or Device
GET_CURRENTDIR	equ	47h		; Get the Current Directory
FREE_MEMORY     equ     49h             ; Dos Free Memory Function

EXIT_NO_ERROR	equ	4C00h		; Dos Exit no error
ERROR_EXIT_1	equ	4C01h		; Exit with ErrCode set to 1
ERROR_EXIT_2	equ	4C02h		; Exit with ErrCode set to 2
ERROR_EXIT_3	equ	4C03h		; Exit with ErrCode set to 3
ERROR_EXIT_4	equ	4C04h		; Exit with ErrCode set to 4
ERROR_EXIT_5	equ	4C04h		; Exit with ErrCode set to 5

FINDFIRST	equ	4E00h		; Find First File



