ifdef TASM
%PAGESIZE 66,130
else
PAGE 66, 130
endif
;*****************************************************************************
; TOKEN.ASM
;
;	(C) Copyright Unpublished Work of Novell, Inc.  All Rights Reserved.
;
;	THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL, 
;	PROPRIETARY AND TRADESECRET INFORMATION OF NOVELL, INC.  ACCESS 
;	TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
;	NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS 
;	AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO APPROPRIATE 
;	LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED, PRACTICED,
;	PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,
;	ABBRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
;	TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL. 
;	ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD 
;	SUBJECT THE PERPERTRATOR TO CRIMINAL AND CIVIL LIABILITY.
;
;----------------------------------------------------------------------------
;
;	DOS Topology Specific Module (TSM) for 802.5 Token-Ring Media
;
;	Written by:	DCD
;	Date:		6/18/90
;
;	ODI Specification Version:	4.00
;
;Modification History:
;	Date		By	Modification Description
;	---------------	---	------------------------------------------
;	01 August 1994   JWR,WTT  v2.00 Removed Previous history, 
;				See DOS ODI Specifcation Changes Doc for changes.
;
;	19 September 1994 JWR	TxTooBig path missed restoring the TxFreeCounter
;				Snap Protocol ID byte 0 and 1 were swapped.
;
;	09 January 1995   JWR   Checkin New ToolKit. v2.02
;	
;	28 February 1995  JWR	New ToolKit. v2.03
;				TxTooBig error handling failed to load ES:SI correctly.
;				SPD# 91038 -- MaxDataSize Not based on the Base 
;				Config table when FrameData Config table is updated.
;
;	5 April 1995	JWR	New ToolKit. v2.03
;				 1) TxTooBig error handling failed to 
;				    load ES:SI correctly.
;				 2) SPD#91038 -- MaxDataSize Not Based on 
;						the Base ConfigTable when 
;						FrameData config Table is updated.
;				 3) Cleaned out unused code. (FDMODE CODE).
;				 4) Added MSMGetMulticastInfo (IOCTL 15).
;				 5) SPD#94271 -- Checking for Tx size, CX not
;						set to test.  Changed to Load 
;						cx with TCBDataLen.
;				 6) Adjusted MaxMediaFrameSize to account for 
;					DHB overhead (6 bytes).
;				 7) Added SR_Length equate.
;
;	30 Aug. 1995	JWR	v2.04
;				Fixed Source Routing Detection in Tx Path.
;				Corrected the Comment for MSMGetMulticastInfo
;
;	19 Sept.1995	JWR	v2.05 Beta 1
;				SPD# 106153, 106157, 106154 fixed. 106143 NotABug
;				Changed to Test for RII bit on Lookahead SRCAddr. (LSB MODE)
;				Changed Priority Check to be done on FCByte.
;				Changed MAC/Non-MAC frame check to be bit aware.
;
;	25 Oct. 1995	JWR	v2.05 
;				Removed Beta String.
;				Changed the Equate NON_MAC_FRAME to LLC_FRAME
;
; 1 April 1996	JWR	New ToolKit. v2.10
;		SPD# 120688 Multicast Functional Address Attributes not being	
;			flagged corectly in Promiscuous Mode.
;		SPD# 119818 Multicast Packets pass filter after deleted from
;			table in Non-Promiscuous Mode.
;		SPD# 119819 Non-Cannonical Bit not set in TokenTSM in LPacketAttrib
;
; 13 June 1996	JWR  v2.10a
;		SPD#128652 The DestAddrTypeFlag was being setup prior to checking
;			if the Source Route Module was setup.  Moved 5 lines of code
;			to set the DestAddrTypeFlag after checking and maintain stack order.
;			
; 30 Sept. 1996  JWR v2.10b
;		00C0H test was incorrect in the Prepare Multicast function.
;		
; 7 Oct. 1996    JWR v2.10c
;		IBM called and noticed that the SourceRoute Offset field of the
;		LookAhead structure was not initialized correctly for MAC frame 
;		Reception.  This causes strange (sometimes Crashing) behavior.
;		The use of the Snap lookahead structure causes a problem if the
;		frame type is not loaded.  The solution was to skip Informing
;		the SR Module for MAC frames as well as Error Frames.
;
; 6 Dec. 1996   JWR/IDC v2.10d
;		SPD#141162- IBM/Novell Labs found Group Addressing Problem
;		Prepare Multicast now returns pointer to the Configtable so 
;		bx is preloaded.  last 00c0H test was not corrected.
;
; 10 Mar.1996   CKP v2.10e
;               SPD#141162 - Group Addressing Problem also fixed for the 
;               MSMDelTokenGrpAddr label. Same as fix for MSMAddTokenGrpAddr.
;
; 21 Nov. 97	SMN		New ToolKit. v2.12
;
;*****************************************************************************
.8086
%NOINCL				; See listings of Include files in MSM.asm
include	msm.inc

ifdef TASM
%NEWPAGE ;
endif
;----------------------------------------------------------------------------
TokenTSMVersion		equ	'v2.12 (971121) ' ; Set Nov. 21, 1997  SMN

CGroup	group	Code,Init
	assume	cs:CGroup, ds:CGroup, es:nothing, ss:nothing

Code	segment	para public 'CODE'
;----------------------------------------------------------------------------
;	Variables Provided by the TSM.
;----------------------------------------------------------------------------
	public	MSMTxFreeCount
	public	MSMLookAheadSegment
	public	MSMMaxMulticastAddr

ifdef DEBUG		;Internal Use Only. 
	public	MaxVirtBoards
	public	BuildHeaderHandlers
	public	HeaderSizeTable
	public	BoardFlags

	public	TCBFreeList
	public	MulticastTable

	public	LookAhead8022
	public	LookAheadSNAP
endif

;----------------------------------------------------------------------------
;	Functions Provided by the TSM.
;----------------------------------------------------------------------------
	public	MSMSendPacket
	public	MSMSendComplete
	public	MSMGetNextSend
	public	MSMGetRCB
	public	MSMReturnRCB
	public	MSMRcvComplete
	public	MSMRcvCompleteStatus
	public	MSMGetMulticastInfo
	public	MSMAddMulticastAddress
	public	MSMDelMulticastAddress
	public	MSMUpdateMulticast
	public	MSMMediaShutdown		; Unused for TokenRing
	public	MSMMediaReset			; Unused for TokenRing
	public	MSMMediaConfigUpdate		; Called after Hardware Initializes
	public	MSMBuildTransmitControlBlock	; Called for Priority Support by the HSM.


ifdef DEBUG		;Internal Use Only. 
	public	FindAddressInMCTable
	public  FindAddressInMCTableNotLoaded
	public	MSMAddTokenGrpAddr
	public	MSMDelTokenGrpAddr

endif

;----------------------------------------------------------------------------
;	Variables Provided by the MSM
;----------------------------------------------------------------------------
	extrn	LSLSupport		:dword
	extrn	MSMTxMonPtr		:dword
	extrn	MSMTxQueueHead		:word
	extrn	MSMTxQueueTail		:word
	extrn	MSMActiveBoardCount	:word
	extrn	MSMConfigTables		:word
	extrn	MSMActivePromisMask	:word
	extrn	MSMTableIndex		:byte	; Use with Upper Register = 0
	extrn	MSMTxFlags		:word
	extrn	MSMTxFlags0		:byte
	extrn	MSMTxFlags1		:byte
	extrn	MSMMaxActiveTx		:byte
	extrn	MSMPhysNodeAddress	:byte

;----------------------------------------------------------------------------
;	Functions Provided by the MSM
;----------------------------------------------------------------------------
	extrn	MSMEnqueueSend		:near
	extrn	MSMUpdateConfigTables	:near

;----------------------------------------------------------------------------
;	Variables Provided by the HSM
;----------------------------------------------------------------------------
	extrn	DriverStatTable		:byte
	extrn	DriverConfigTable	:byte

;----------------------------------------------------------------------------
;	Functions Provided by the HSM
;----------------------------------------------------------------------------
	extrn	DriverSend		:near
	extrn	DriverMulticastChange	:near
	extrn	DriverPromiscuousChange	:near

;----------------------------------------------------------------------------
;	The following equates are media specific.
;----------------------------------------------------------------------------
MSMMaxMulticastAddr	equ	16		; We support 16 MC addresses MAX
MaxVirtBoards		equ	2		; 8025/8022, 8025/8022/SNAP
MAX_TCB_ALLOCATED	equ	4

MinMediaFrameSize	equ	17		; Absolute min. for TokenRing (AC FC DES SRC DSAP SSAP CTRL)
MaxMediaFrameSize	equ	17954		; Absolute max. for Token-Ring (DHB overhead not added)
MaxFrameHeaderLen	equ	14+(18+12)+3+5	; Max. possible frame header (Should be even)
MinFrameHeaderLen	equ	14+3		; Min Possible Frame Header

SNAPHeaderSize		equ	14+3+5		; Nominal Snap Header Size.
TR8022HeaderSize	equ	14+3		; Nominal 8022 Header Size.
MACHeaderSize		equ	14		; Nominal MAC Header Size
MinMACFrameSize		equ	14+4		; Minimum MAC Frame Len
LLC_FRAME		equ	40h		; LLC type Frame
FC_CONTROL_BITS		equ	3Fh		; Mask off Control Bits in FCbyte.
FC_PRIORITY_BITS	equ	07h		; Mask for Priority Bits.
SR_LENGTH		equ	18+12		; Allow for SR and SRT.

;----------------------------------------------------------------------------
;	Media ID's Supported in this module.
;----------------------------------------------------------------------------
MediaID8022		equ	4		; Well known Media ID's
MediaIDSNAP		equ	11		; defined by ODI specification

;----------------------------------------------------------------------------
;	Media Type Strings
;----------------------------------------------------------------------------
Message	MediaName4	'TOKEN-RING'
Message	MediaName11	'TOKEN-RING_SNAP'

;----------------------------------------------------------------------------
;	802.5/802.2 frame format structure
;----------------------------------------------------------------------------
TokenRingFrameFormat	struc
	ACByte		db	?		;Access Control Byte
	FCByte		db	?		;Frame Control Byte
	DestAddr	db	6 dup (?)
	SrcAddr		db	6 dup (?)
	DSAP		db	?		; 802.2 header
	SSAP		db	?
	Control		db	?
	SNAP		db	5 dup (?)	; SNAP
TokenRingFrameFormat	ends

;----------------------------------------------------------------------------
;	Source Rounting Eqates.
;----------------------------------------------------------------------------
RouteEntryPointOff	equ	size LookAheadStruc ; Holds offset of Config.MRouteHandler
SRLength		equ	SrcAddr+6	; If Source Routing is there, byte 1 = 
						; xxxLLLLL where xxx = Broadcast type.
						;              LLLLL = Length of Field (18 Max)

;----------------------------------------------------------------------------
;	BoardFlags bit definitions.
;----------------------------------------------------------------------------
LOADED		equ	01h		; Set if This Frame Is Loaded.	
EVENIZE		equ	02h		; Set if this frame needs to be evenized
MSWAPFLAG	equ	04h		; Swap Needed if set.

;----------------------------------------------------------------------------
; 802.2 Control Fields LLC Frame Type 
;----------------------------------------------------------------------------
SU_FormatBit		equ	01	;if Bit 0 = 0 then I_Format else S or U_Format
U_FormatBit		equ	02	;if Bit 1 = 1 then U_format 

;----------------------------------------------------------------------------
;	Bit definitions used in LoadedFrameByte
;----------------------------------------------------------------------------
FRAME_IS_8022		equ	01	; Flag this frame is 802.2
FRAME_IS_SNAP		equ	02	; Flag this frame is SNAP
FRAME_IS_MAC		equ	04	; Flag this Frame as A MAC Packet.

;----------------------------------------------------------------------------
;	Some general variables
;----------------------------------------------------------------------------
	even
TCBFreeList		dw	0	;Points to a list of TCB's
DestAddrTypeFlag	dw	0	;	 Defined in ODI.INC
HardwareRcvLength	dw	0	; Used to store the hardware reported
					;  length of a received packet.
MulticastSwapArea	db	6 dup(0);Used if swapping of address is needed.

;----------------------------------------------------------------------------
;	Variables initialized by the HSM
;----------------------------------------------------------------------------
MSMLookAheadSegment	dw	0	; Where the Look Ahead Buffer is.
MSMTxFreeCount		db	?	; Number of Tx resources available

;----------------------------------------------------------------------------
;	MSMGetRCB procedure variables.
;----------------------------------------------------------------------------
MSMRcvProtSkipCount	db	0	; Protocol Skip Count value (MAX = 128)
MSMRcvPacketStatus	dw	0	; Packet Status for MSMGetRCB call.
MSMRcvHeaderLen		dw	0	; Packet Header length for MSMGetRCB call. (Max = 22)

;----------------------------------------------------------------------------
;	The following tables are used to look up media specific information
;	about each logical ODI board number.
;----------------------------------------------------------------------------
	even
BuildHeaderHandlers	dw	MaxVirtBoards dup (0)	; Contains the addr of a routine that builds the media header
HeaderSizeTable		db	MaxBoards dup (0)	; Each boards media header size is here
BoardFlags		db	MaxBoards dup (0)	; Each Boards specific stuff.

	even
FunctionalAddressPrefix db	0C0h,00			; (setup for IOCTL 15)
ActiveFunctionalAddress	db	4 dup (0)		; Current Functional Address.
FunctionalAddressTable	db	32 dup (0)		
MulticastTable		db	(MSMMaxMulticastAddr * 8) dup (0)

	align	4
LookAhead8022		LookAheadStruc	<>
			dw	0		;Reserve room for RouteEntryPointOff field
LookAheadSNAP		LookAheadStruc	<>
			dw	0		;Reserve room for RouteEntryPointOff field
			
;****************************************************************************
;		Source Routing Stub
;****************************************************************************
	even
SRStubRoutine	proc	far
	retf
SRStubRoutine	endp


;******************************************************************************
;                  F R A M E   T R A N S M I T   C O D E
;******************************************************************************
;
;	MSMSendPacket
;
;	Called directly by the LSL when it wants us to send a packet.
;
;	assumes:
;		es:si	-> ECB
;		Interrupts are disabled
;
;	returns:
;		ax	=0 if no initial errors
;			=LSLERR_NO_SUCH_DRIVER if MLID is shut down
;			=LSLERR_PACKET_ERRORED  if Packet is Too BIG
;		z flag	set according to ax.
;		Interrupts disabled
;		DS,BP,SS,SP preserved
;******************************************************************************
;----------------------------------------------------------------------------
;    We are Shutdown, or Already Queued up or in a Critical Section.
;----------------------------------------------------------------------------
CantSendRightNow:
	test	word ptr MSMTxFlags0, (TXQUEUED*100h) or CRITICAL
	jz	ShutDownCase
	jmp short QueueItAndExit

;----------------------------------------------------------------------------
;	Tried to Send one too many, reset and go on.
;----------------------------------------------------------------------------
MLIDIsSaturated:
	mov	MSMTxFreeCount, 0		;Don't go negative

;----------------------------------------------------------------------------
;	Can't send right now, just Queue it up.
;----------------------------------------------------------------------------
QueueItAndExit:
	call	MSMEnqueueSend			;Queue it for later
	xor	ax, ax

;----------------------------------------------------------------------------
;	Most come here to restore the Regs off the stack and exit.
;----------------------------------------------------------------------------
ExitSendPacket:
	pop	ds
	pop	bp
	retf			; Return far to caller

;----------------------------------------------------------------------------
;	If we are in a Shutdown mode, we simply give ECB back 
;----------------------------------------------------------------------------
ShutDownCase:
	mov	ax, LSLERR_NO_SUCH_DRIVER	; Shutdown, set nodriver go figure.
ReturnTooBigECB:
	push	ax				; Save the Error code for later
	mov	es:[si].Status, ax		; Set ECB Status field
	call	es:[si].ESR			; Give ECB back to Caller.
	pop	ax				; AX = Error code
	or	ax, ax				; Set flags accordingly
	jmp short ExitSendPacket

;----------------------------------------------------------------------------
;	MSMSendPacket  Starts HERE:
;----------------------------------------------------------------------------
MSMSendPacket	proc	far
	push	bp				; Must save for ODI spec.
	push	ds
	mov	ax, cs
	mov	ds, ax				; DS = CGroup

;----------------------------------------------------------------------------
;    Check if we are Shutdown, or Already Queued up or in a Critical Section.
;----------------------------------------------------------------------------
	test	word ptr MSMTxFlags0, ((SHUTDOWN or TXQUEUED)*100h) or CRITICAL
	jnz	CantSendRightNow

;----------------------------------------------------------------------------
;	See if the HSM has a free Resource
;----------------------------------------------------------------------------
	dec	MSMTxFreeCount
	js	MLIDIsSaturated

;----------------------------------------------------------------------------
;	Get the Header built
;----------------------------------------------------------------------------
	mov	di, si				; ES:DI -> ECB
	cld					; required for DriverSend
	call	TSMBuildTransmitControlBlock	; get header built

;----------------------------------------------------------------------------
;	Guarentee that the HSM never Gets a Too Big Packet.
;----------------------------------------------------------------------------
	mov	cx, [si].TCBDataLen	
	cmp	cx, DriverConfigTable.MMaxPacketSize
	ja	TXTooBig		; a=> Frame is too large for Configured size.

;----------------------------------------------------------------------------
;	Enter a critical section.
;----------------------------------------------------------------------------
	mov	bx, MLIDSUP_START_CRITICAL_SECTION
	call	LSLSupport
	inc	MSMTxFlags0			;Set critical section flag

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	DriverSend assumes the following
;
;	assumes: si	-> TCB
;		 ds	is set to CGroup
;		 CLD has been executed
;		 Interrupts are disabled
;		 A critical section has been started
;
;	returns: Can destroy all registers
;		 Interrupts must be DISABLED!
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	call	DriverSend			;Send the frame

;----------------------------------------------------------------------------
;	End the Critical section, and return
;----------------------------------------------------------------------------
	pop	ds
	pop	bp
	dec	cs:MSMTxFlags0			;Exit critical section
	mov	bx, MLIDSUP_END_CRITICAL_SECTION
	call	cs:LSLSupport			;Also calls ServiceEvents
	xor	ax, ax				; Set flags, and ax.
	retf					; Return far to caller

;----------------------------------------------------------------------------
;	Packet was too Big to send, Give an Error indication and return it.
;	DS:SI -> TCB
;----------------------------------------------------------------------------
TXTooBig:
	inc	DriverStatTable.MTxTooBig
	inc	MSMTxFreeCount			; Restore -- We didn't take it after all.
	mov	cx, TCBFreeList			; Free up this TCB
	mov	[si].NextLink, cx
	mov	TCBFreeList, si
	les	si, [si].TCBFragStrucPtr	; Get ptr to ECB.fragstruc
	sub	si, FragCount			; ES:SI -> ECB
	mov	ax, LSLERR_PACKET_ERRORED
	jmp short ReturnTooBigECB

MSMSendPacket	endp


ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	MSMSendComplete
;
;	Called by the MLID when it has finished with a transmit event.
;	This routine will assume control of the TCB handed to this routine.
;
;	assumes:
;		ds	is set to CGroup
;		si	-> TCB
;		Interrupts are disabled
;
;	returns:
;		ds,bp,ss,sp preserved
;		CLD preserved
;		Interrupts disabled
;
;****************************************************************************
MSMSendComplete	proc	near
;----------------------------------------------------------------------------
;	Count the Packets transmited
;----------------------------------------------------------------------------
	add	DriverStatTable.MTotalTxPackets+0, 1
	adc	DriverStatTable.MTotalTxPackets+2, 0

;----------------------------------------------------------------------------
;	See if the Transmit Monitor is around
;----------------------------------------------------------------------------
	cmp	word ptr MSMTxMonPtr+2, 00h	; ptr. to Rcv Monitor Tx Routine
	jz	MSMSendCompleteProceed

	call	dword ptr MSMTxMonPtr		; call Tx Monitor Routine

;----------------------------------------------------------------------------
;	Put TCB back on the TCBFreeList
;----------------------------------------------------------------------------
MSMSendCompleteProceed:
	mov	cx, TCBFreeList			;Free up this TCB
	mov	[si].NextLink, cx
	mov	TCBFreeList, si

;----------------------------------------------------------------------------
;	Send the ECB Back to its owner
;----------------------------------------------------------------------------
	les	si, [si].TCBFragStrucPtr	; Get ptr to ECB
	sub	si, FragCount			; ES:SI -> ECB
	mov	bx, MLIDSUP_SEND_COMPLETE	; Hand it to the LSL
	call	LSLSupport
	ret

MSMSendComplete	endp

ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	MSMGetNextSend
;
;	Called by the MLID when it is ready for another transmit 
;	request block.  This routine must check to make sure the HSM
;	can handle another transmit.
;
;	assumes:
;		ds	is set to CGroup
;		Interrupts are disabled
;
;	returns:
;		si	-> TCB if there is one
;		ds	is set to CGroup
;		All other registers destroyed
;		nz flag if TCB NOT available
;		Interrupts disabled
;
;****************************************************************************
;----------------------------------------------------------------------------
;	The HSM has no free Resources.
;----------------------------------------------------------------------------
HSMIsAlreadySaturated:
	mov	MSMTxFreeCount, 0		; Don't go negative

;----------------------------------------------------------------------------
;	Signal NO TCB available.
;----------------------------------------------------------------------------
QueueisEmpty:
	or	al, 1
	ret

;----------------------------------------------------------------------------
;	MSMGetNextSend starts HERE	
;----------------------------------------------------------------------------
MSMGetNextSend	proc	near
;----------------------------------------------------------------------------
;	See if there is one on the Q	
;----------------------------------------------------------------------------
	les	di, dword ptr MSMTxQueueHead
	mov	ax, es
	or	ax, di
	jz	QueueisEmpty

;----------------------------------------------------------------------------
;	We think that there is one
;----------------------------------------------------------------------------
	dec	MSMTxFreeCount
	js	HSMIsAlreadySaturated	; Not ready yet

;----------------------------------------------------------------------------
; 	Keep track of how deep the Queue is.	
;----------------------------------------------------------------------------
	dec	DriverStatTable.MQueueDepth	; Decrement Queue Depth

;----------------------------------------------------------------------------
;	Get an ECB off the MSM's QUEUE
;----------------------------------------------------------------------------
	mov	ax, es:[di].NextLink+0		; Unlink the ECB
	mov	cx, es:[di].NextLink+2
	mov	word ptr MSMTxQueueHead+0, ax
	mov	word ptr MSMTxQueueHead+2, cx
	or	ax, cx				; Did we take the last one?
	jnz	StillMoreOnQueue

;----------------------------------------------------------------------------
;	No ECB is left on the Queue
;----------------------------------------------------------------------------
	and	MSMTxFlags1, not TXQUEUED
	mov	word ptr MSMTxQueueTail+0, offset CGroup:MSMTxQueueHead
	mov	word ptr MSMTxQueueTail+2, cs

;----------------------------------------------------------------------------
;	Build the Header and return
;----------------------------------------------------------------------------
StillMoreOnQueue:
	call	TSMBuildTransmitControlBlock
MSMMediaShutdown:		; We don't need to do anything.
MSMMediaReset:
	xor	ax, ax
	ret

MSMGetNextSend	endp


ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	MSMBuildTransmitControlBlock /TSMBuildTransmitControlBlock
;
;	Takes a ODI ECB and converts into a Transmit Request Block (TCB).
;	It also builds the appropriate media header.
;
;	assumes:	
;		DS	is set to CGroup
;		ES:DI	-> ECB
;		Interrupts are disabled
;
;	returns:
;	(MSMBuildTransmitControlBlock will return si = 0 if no TCB is available.)
;	(TSMBuildTransmitControlBlock will always return a TCB)
;		DS:SI	-> TCB
;		CX	has the hardware frame length
;		DS, SS, SP preserved
;		All other Registers Destroyed.
;		Interrupts are disabled
;
;****************************************************************************
MSMBuildTransmitControlBlock	proc	near
	mov	si, TCBFreeList			; Get a TCB
	or	si, si			    	; Verify that there is one.
	jnz	TSMBuildTransmitControlBlock
	ret					; Return to HSM.

TSMBuildTransmitControlBlock:
;----------------------------------------------------------------------------
;	Get a free TCB
;----------------------------------------------------------------------------
	mov	si, TCBFreeList			; Get a TCB
	mov	ax, [si].NextLink		; Unhook it from the list
	mov	TCBFreeList, ax			; Keep track of who's next.

;----------------------------------------------------------------------------
;	Get the TCB pointers loaded
;----------------------------------------------------------------------------
	lea	ax, [di].FragCount			; AX -> ECB.FragCnt
	mov	word ptr [si].TCBFragStrucPtr+0, ax	; Fill in TCB
	mov	word ptr [si].TCBFragStrucPtr+2, es

	mov	bx, es:[di].BoardNum		; Get the board number
	xor	ax, ax				; Prepare for Zero HeaderLen
	mov	es:[di].Status, ax		; Get this out of the way

;----------------------------------------------------------------------------
;	Check for RAW send packets.  (Allow for Priority settings)
;					FFFFh -> FFF8h Raw Send Priority.
;					FFF7h -> FFF0h Non-Raw Send Priority.
;----------------------------------------------------------------------------
	cmp	es:[di].ProtoNum, 0FFF7h ; Raw send? 
	ja	DoRawSend			; a => Raw Send

	mov	al, HeaderSizeTable[bx]		; Get headerSize for this frame

;----------------------------------------------------------------------------
;	AX = Header Size
;----------------------------------------------------------------------------
DoRawSend:
	mov	[si].TCBMediaHeaderLen, ax	; Set the media header length
	add	ax, es:[di].DataLen		; Get total of frag's and media header
	mov	[si].TCBDataLen, ax		; Set it in the TCB
	cmp	[si].TCBMediaHeaderLen, 0	; See if a Header is needed.
	je	NoHeaderNeeded

;----------------------------------------------------------------------------
;	Check for Bit Swapping needed.
;----------------------------------------------------------------------------
	test	BoardFlags[bx], MSWAPFLAG	; Check on Swap Flag For this Board.
	jz	TCBnoSwapBits

;----------------------------------------------------------------------------
;		Swap the Address
;----------------------------------------------------------------------------
TCBSwapBits:
	push	bx				; Save Board Number
	mov	bx, PSPTableStart		; BitSwapTable @

;1st Word
	mov	ax, word ptr es:[di].ImmAddr+0
	xlat					;Swap it
	xchg	ah, al				;AH gets result, AL is Next look up
	xlat					;Swap it
	xchg	ah, al				;straighten back for save
	mov	word ptr [si].TCBMediaHeader.DestAddr+0, ax

;2nd Word
	mov	ax, word ptr es:[di].ImmAddr+2
	xlat					;Swap it
	xchg	ah, al				;AH gets result, AL is Next look up
	xlat					;Swap it
	xchg	ah, al				;straighten back for save
	mov	word ptr [si].TCBMediaHeader.DestAddr+2, ax

;3rd Word
	mov	ax, word ptr es:[di].ImmAddr+4
	xlat					;Swap it
	xchg	ah, al				;AH gets result, AL is Next look up
	xlat					;Swap it
	xchg	ah, al				;straighten back for save
	mov	word ptr [si].TCBMediaHeader.DestAddr+4, ax
	pop	bx				; Retrieve Board Number

;----------------------------------------------------------------------------
;	Unless this is a RAW Send Guarentee the SR RII bit is clear.
;----------------------------------------------------------------------------
TCBSwapped:
	and	byte ptr [si].TCBMediaHeader.SrcAddr, not 80h	;Clear RII bit in src addr
	mov	cl, HeaderSizeTable[bx]			; Get our header length

;----------------------------------------------------------------------------
;	Setup for building the Headers
;----------------------------------------------------------------------------
	mov	bl, MSMTableIndex[bx]		; Get Virtual Word Index into table.
	push	BuildHeaderHandlers[bx]		; Save offset of head build routine for return auto calling.
	mov	bp, MSMConfigTables[bx]		; DS:BP -> correct config table
	lea	bx, [si].TCBMediaHeader.DSAP	; DS:BX -> place to put 802.2 header
	mov	dx, cs
	cmp	word ptr ds:[bp].MRouteHandler+2, dx ; ROUTE.COM installed?
	jne	DoSourceRoutingStuff

NoHeaderNeeded:
	ret					; Goto header build routine

;----------------------------------------------------------------------------
;	We are marked to not Swap bits.
;	Load the Immediate Address into the Dest Address Spot.
;----------------------------------------------------------------------------
TCBnoSwapBits:					; Allow No Bit Swap to take place.
	mov	ax, word ptr es:[di].ImmAddr+0	;Set the dest. address
	mov	word ptr [si].TCBMediaHeader.DestAddr+0, ax
	mov	ax, word ptr es:[di].ImmAddr+2
	mov	word ptr [si].TCBMediaHeader.DestAddr+2, ax
	mov	ax, word ptr es:[di].ImmAddr+4
	mov	word ptr [si].TCBMediaHeader.DestAddr+4, ax
	jmp short TCBSwapped			; go Clear the SR RII Bit.

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	Build Media Header Routines
;
;	assumes:
;		es:di	-> ECB
;		ds:si	-> TCB
;		ds:bx	-> place to build 802.2 header
;
;	returns:
;		ds:si	-> TCB
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Build8022Header:
	xor	cx, cx				; Clears 
	cmp	byte ptr es:[di].ProtID+0, cl
	jne	Explicit8022Header		; Header is given Explicitly

;----------------------------------------------------------------------------
;	Set up a UI Type 1 frame as default.
;----------------------------------------------------------------------------
	mov	dl, byte ptr es:[di].ProtID+5	; Get DSAP value
	mov	dh, dl				; DSAP=SSAP
	mov	word ptr [bx+0], dx		; DSAP, SSAP
	mov	byte ptr [bx+2], 03h		; Control byte 
	ret

;----------------------------------------------------------------------------
;	Build Header with Explicit values given.
;	Generically done so protocols can use all 5 bytes of the ProtoID field.
;	DS:BX -> where to put the header.
;	ES:DI -> ECB
;	DS:SI -> TCB
;----------------------------------------------------------------------------
Explicit8022Header:				
	mov	bp, si				; DS:BP -> TCB

;----------------------------------------------------------------------------
;	Get Number of Bytes in 802.2 header 0 based (2, or 3)
;----------------------------------------------------------------------------
	mov	cl, byte ptr es:[di].ProtID+0	; Get Type Number.
	cmp	cl, 02				; If 2 then we have already added it in.
	je	DataLenAdjusted			; ne => need to add one more byte.

	inc	[si].TCBMediaHeaderLen		; Add one more for Type 2 frames.
	inc	[si].TCBDataLen

;----------------------------------------------------------------------------
;	Set up pointers to move header over.
;----------------------------------------------------------------------------
DataLenAdjusted:
	lea	si, es:[di].ProtID+6		; Move to end. (but Remember cx is 0 based)
	inc	cx				; Make 1 Based
	sub	si, cx				; Move to start of 8022Header Source
	mov	di, bx				; DS:DI-> to target space.

	mov	bx, ds				; BX:BP ->TCB (Saved off DS before)
	mov	dx, es				; DX:DI -> Target Space
	mov	es, bx				; Get ES:DI -> Target space						; 
	mov	ds, dx				; Get DS:SI -> Sourceof 8022 Header
 rep	movsb					; Move 8022 header.

;----------------------------------------------------------------------------
;	On Exit: 
;		Set DS:SI -> TCB
;----------------------------------------------------------------------------
	mov	si, bp			; BX:SI -> TCB
	mov	ds, bx			; Restore DS:SI -> TCB
	ret

;----------------------------------------------------------------------------
;	Build Snap Packet Header
;----------------------------------------------------------------------------
BuildSNAPHeader:
	mov	word ptr [bx+0], 0AAAAh		; Signal SNAP
	mov	byte ptr [bx+2], 03h
	mov	al, byte ptr es:[di].ProtID+1	; Copy over the SNAP info
	mov	byte ptr [bx+3], al
	mov	ax, word ptr es:[di].ProtID+2
	mov	word ptr [bx+4], ax
	mov	ax, word ptr es:[di].ProtID+4
	mov	word ptr [bx+6], ax
	ret

;----------------------------------------------------------------------------
;	Source Routing Support.
;----------------------------------------------------------------------------
DoSourceRoutingStuff:
	xor	cx, cx				 ; Requesting TX SR info function
	lea	ax, [si].TCBMediaHeader.DestAddr ; DS:AX -> destination address
	call	ds:[bp].MRouteHandler		 ; Call to ROUTE.COM
	jcxz	NoSRInfoThisTime		

;----------------------------------------------------------------------------
;	Add Source Route Info to Header.
;----------------------------------------------------------------------------
	or	byte ptr [si].TCBMediaHeader.SrcAddr, 80h ;Set RII bit
	add	[si].TCBMediaHeaderLen, cx	; Adjust lengths for SR info
	add	[si].TCBDataLen, cx
	push	es				; Save ECB seg value
	push	di				; Save ECB offset value
	mov	ds, dx				; DS:AX -> SR info
	xchg	si, ax				; DS:SI -> SR info
	mov	dx, cs				; DX TCB seg value
	mov	es, dx				; ES:BX -> SR info destination
	mov	di, bx				; ES:DI -> SR info Destination
	shr	cx, 1				; Count will always be even
   rep	movsw					; Store Information
   	mov	bx, di				; CS:BX -> place to build 802.2 header
	mov	ds, dx				; DS:BX -> Place to build 802.2 header.
	xchg	si, ax				; DS:SI -> TCB
	pop	di				; 
	pop	es				; ES:DI -> ECB

NoSRInfoThisTime:
	ret					;Goto header build routine

MSMBuildTransmitControlBlock	endp

ifdef TASM
%NEWPAGE ;
endif
;*******************************************************************************
;                  F R A M E   R E C E P T I O N   C O D E
;*******************************************************************************
;
;	MSMGetRCB
;
;	Called by the MLID when it has a frame waiting for a buffer.
;	The MLID has passed us a pointer to the frame data. (Poss. Errors)
;	1. Check for Proper Size
;	2. Determine Destination Address Type 
;		(Remote or Direct, MC or Direct etc.)
;	3. Parse the Media Header and fill in our LookAheadStructure.
;	4. Pass the frame up to the LSL.
;	5. The LSL determines which Protocol Stack to give the Packet to.
;	6. if an ECB is returned from the LSL, an RCB is passed to the HSM.
;	7. After the HSM has filled in the buffers, the HSM will call 
;	   MSMRcvComplete if when the HSM passed in a valid Frame Length  (bp!=-1)
;	   to MSMGetRCB. If the HSM (bp=-1) didn't know the Frame length 
;	   when it called MSMGetRCB then it will call MSMRcvCompleteStatus 
;	   when the buffers are filled in.
;
;
;	On Entry:
;		AX	Zero		Normal Rx Modes
;			Non-Zero	Rx Frame Errors  (See LPacketAttrib definitions in ODI.INC)
;					PKT_CRC_ERR 	equ 0	CRC Error
;					PKT_ALIGN_ERR	equ 1	CRC/Alignment Error
;
;		ES:SI	-> LookAhead Frame Data
;		DS	is set to CGroup
;		BP	has the hardware reported frame length or
;			-1 for unknown packet length at the time of call.
;		CLD has been executed
;		Interrupts are Enabled.
;
;	On Return:	
;		nz flag	if no buffer for this frame (Discard the frame)
;		ds	is set to CGroup
;		ax	has the number of bytes to skip from the
;			Start of the frame before starting to copy to
;			the RCB buffers.
;		bp	How much data needs to be copied or -1 
;			for unknown packet length at the time this 
;			call was made.
;		es:si	-> Receive Control Block (RCB) if z flag
;		All other registers destroyed
;		CLD has been executed
;		Interrupts Enabled, but have been disabled for a time period
;
;*******************************************************************************
;----------------------------------------------------------------------------
; Make the code think we have a hardware length for the packet.
;	BP must be as large as the largest possible Frame size.
;----------------------------------------------------------------------------
UnknownPacketLength:
	mov	bp, DriverConfigTable.MMaxPacketSize	; BP = Fake Hardware Length
	jmp short UnknownPacketLengthRet		; Check Len during RcvComplete

;----------------------------------------------------------------------------
;	Hardware Reported Length is larger than Media Max.
;----------------------------------------------------------------------------
FrmSizeTooBig:
	inc	DriverStatTable.MRxTooBig		; Inc. Rx Too Big count.
	or  byte ptr MSMRcvPacketStatus, PKT_BIG_ERR	; Set Too Big bit in packet
	jmp short FrmSizeErrorRtn			;  status.

;----------------------------------------------------------------------------
;	Determine if Broadcast or other Multicast packet.
;	Load AX, BX, DX for FindAddressinMcTable.
;----------------------------------------------------------------------------
CheckGroupAddrType:
	mov	byte ptr DestAddrTypeFlag+0, DEST_BROADCAST+DEST_MULTICAST ; Set destination broadcast bit.
	mov	ax, word ptr es:[si].DestAddr+0	; See if a broadcast and 
	mov	bx, word ptr es:[si].DestAddr+2	; Set up for Call to FindAddresinMcTable.
	mov	dx, word ptr es:[si].DestAddr+4	; Get last word of addr.
	mov	cx, dx				; setup for Check for Broadcast
	and	cx, bx				; Lower =-1
	cmp	ax, 00C0h			; if it is a possible Special address.
	je	SpecialCase			; e => we have a C000xxxxxxxx address.

	and	cx, ax				; Include all 48 bits.

SpecialCase:
	inc	cx				; Does it go from -1 to 0?
	jz	BackFromCheckDestAddr

;----------------------------------------------------------------------------
;	See if we can receive this Functional frame.
;	See if we need to examine this Group/Func Addr. eg. Board has a CAM
;----------------------------------------------------------------------------
	mov	byte ptr DestAddrTypeFlag+0, DEST_MULTICAST	; Set MC Bit Only

	cmp	MSMActivePromisMask, 0		; Test if in Promiscuous Mode.
	jnz	SearchMCTable			; Must search table in order to
						;  set dest address type.

;----------------------------------------------------------------------------
;	See if the Adapter is completely filtering the Address
;----------------------------------------------------------------------------
	test byte ptr DriverConfigTable.MFlags+1, HIGH(GROUP_ADDRESSING_BIT+FILTER_MULTICAST_BIT) ; Isolate bits 10 & 9.
	jz	SearchMCTable			; Z - Search Multicast Table. (10+9 = 0)
	jpe	BackFromCheckDestAddr		; Parity Even, NIC has ensured Group/Funct. Address

SearchMCTable:
	call	FindAddressInMCTable		; AX:BX:DX has address
	jz	BackFromCheckDestAddr		; z => Found in Table.

;----------------------------------------------------------------------------
;	Not directed Group/Funct. Address, Mark as Remote and continue.
;----------------------------------------------------------------------------
	mov	byte ptr DestAddrTypeFlag+0, DEST_REMOTE_MULTICAST	; Set destination remote multicast bit.
	cmp	MSMActivePromisMask, 0		; Test if in Promiscuous Mode.
	jnz	BackFromCheckDestAddr		; Pass it on if yes.

	jmp	RefusePacket			; Filter the Packet out if no.

;----------------------------------------------------------------------------
;	MSMGetRCB Code Starts Here
;----------------------------------------------------------------------------
MSMGetRCB	proc	near
;----------------------------------------------------------------------------
;	AX = 	Rx Error Status on entry
;	BP = 	Hardware reported frame length or 
;		-1 unknown packet length at the time of call.
;----------------------------------------------------------------------------
	mov	MSMRcvPacketStatus, ax			; Set initial Error Status
	mov	HardwareRcvLength, bp			; Save for later use.
	cmp	bp, -1					; Is packet length known?
	je	UnknownPacketLength

;----------------------------------------------------------------------------
;	Is reported Frame size Larger than Media can Handle?
;----------------------------------------------------------------------------
	cmp	bp, DriverConfigTable.MMaxPacketSize	; BP = Reported FrameSize
	ja	FrmSizeTooBig			; a => Too Big for configuration.
							
;----------------------------------------------------------------------------
;	Check for Possible Group Address.
;	ES:SI-> look ahead frame data
;----------------------------------------------------------------------------
UnknownPacketLengthRet:
FrmSizeErrorRtn:
	mov	word ptr DestAddrTypeFlag, DEST_DIRECT	; Flag Pkt is for us. (default) (Clears upper Byte also)
	test	byte ptr es:[si].DestAddr, 80h		; Is this a group frame?
	jnz	CheckGroupAddrType

;----------------------------------------------------------------------------
;	See if the NIC is in Promiscuous Mode.
;	If Not Then It is our Directed Packet
;----------------------------------------------------------------------------
	cmp	MSMActivePromisMask, 0		; Test if in Promiscuous Mode.
;	jne	TestForOurNodeAddress		; ne=> need to test
	je	BackFromCheckDestAddr		; e=> don't Need to Test

;----------------------------------------------------------------------------
;	Check to see if this is our Node Address
;----------------------------------------------------------------------------
TestForOurNodeAddress:
	mov	ax, word ptr MSMPhysNodeAddress+4	; Test last word
	cmp	word ptr es:[si].DestAddr+4, ax
	jne	SetDestRemoteUnicastBit			; NZ - Go set dest. remote unicast bit.

	mov	ax, word ptr MSMPhysNodeAddress+2	; Test middle word
	cmp	word ptr es:[si].DestAddr+2, ax
	jne	SetDestRemoteUnicastBit			; NZ - Go set dest. remote unicast bit.

	mov	ax, word ptr MSMPhysNodeAddress+0	; Test first word
	cmp	word ptr es:[si].DestAddr+0, ax
	je	BackFromCheckDestAddr		  ; e=> our address.

;----------------------------------------------------------------------------
;	Set the Remote unicast bit and skip over MC Tests on our return.
;----------------------------------------------------------------------------
SetDestRemoteUnicastBit:
	mov byte ptr DestAddrTypeFlag+0, DEST_REMOTE_UNICAST  ; Signal its a remote unicast
;	jmp short BackFromCheckDestAddr

;----------------------------------------------------------------------------
;	Check if there is any Source routing to take care of.
;----------------------------------------------------------------------------
BackFromCheckDestAddr:
	xor	bx, bx				; Initially no source routing
	test	byte ptr es:[si].SrcAddr, 80h	; RII bit set?
	jz	NoSRInfoPresent

;----------------------------------------------------------------------------
;	Determine the Length of the Source Route field.
;----------------------------------------------------------------------------
SRInfoPresent:	
	mov	bl, byte ptr es:[si].SRLength	; Get length of SR field
	and	bl, 01Fh			; Clear upper 3 Bits.

;----------------------------------------------------------------------------
;	Determine Data Length
;	BP = Frame Len, BX = SR Len
;----------------------------------------------------------------------------
NoSRInfoPresent:
	mov	cx, bp			; CX = Frame Length
	sub	cx, bx			; Minus Source Routing

;----------------------------------------------------------------------------
;	Test for NON-MAC Frames
;	BP=Frame Len, BX = Sr Len, CX = (Frame Len - SR Len)
;----------------------------------------------------------------------------
	mov	al, byte ptr es:[si].FCByte	; Get FCByte loaded for testing.
	and	al, not FC_CONTROL_BITS	   	; Mask off for control bits.
;	jz	HaveMACFrame		   	; Zero flag => MAC frame.
	jnz	HaveNonMacFrame		   	; NZ => LLC or reserved Frame Type.

;----------------------------------------------------------------------------
;	MAC Frame, set PID MACTOK, use SNAP LookAhead
;	BP = Frame Len	 BX = SR Length	  CX = (FrameLen - SR)
;----------------------------------------------------------------------------
HaveMACFrame:
	mov	di, offset CGroup:LookAheadSNAP	; set ProtocolID MACTOK
	mov	word ptr [di].LProtID+0, 'AM'
	mov	word ptr [di].LProtID+2, 'TC'
	mov	word ptr [di].LProtID+4, 'KO'
	add	bx, MACHeaderSize		; Nominal Header size (14)
	sub	bp, bx				; BP = Length of data
	lea	ax, [bx+si]			; AX -> Points MAC Frame Major Vector
	cmp	cx, MinMACFrameSize		; Large enough for MAC frame Vector?
	jb	TooSmallFrame

;----------------------------------------------------------------------------
;	Signal that A MAC Frame Has Been Received.
;----------------------------------------------------------------------------
	mov	cl, FRAME_IS_MAC			    ; Signal that we have a MAC frame
	mov	byte ptr DestAddrTypeFlag+0, DEST_MAC_FRAME ; Set MAC Bit Only
	jmp	FrameTypeDetermined		

;----------------------------------------------------------------------------
;	Make Sure the Frame is NOT from us.
;----------------------------------------------------------------------------
HaveNonMacFrame:
	mov	ax, word ptr es:[si].SrcAddr+4
	cmp	word ptr MSMPhysNodeAddress+4, ax
	je	PossiblyFromUs

;----------------------------------------------------------------------------
;	Non-MAC Frame determined, That is not from US
;	BP=Frame Len, BX = Sr Len, CX = 802.5 Data Length
;----------------------------------------------------------------------------
NotFromUs:
	cmp	cx, 03h			; test if got a valid 802.2 hdr len
	jb	TooSmallFrame		; Must have at least an 802.2 type 1 header.

;----------------------------------------------------------------------------
;	Check for Frame Type
;----------------------------------------------------------------------------
TestFrame:
	mov	ax, word ptr es:[bx+si].DSAP	; Get DSAP and SSAP
	cmp	ax, 0AAAAh 			; Is this a SNAP frame?
	jne	ItIsA8022Frame

;----------------------------------------------------------------------------
;	Snap Frame Processing
;----------------------------------------------------------------------------
ItMaybeSNAP:
	cmp	byte ptr es:[bx+si].Control, 03h ; Test for the 03 Hex
	jne	ItIsA8022Frame			 ; NZ - Its 802.2

;----------------------------------------------------------------------------
;	Frame is a SNAP packet. Load ProtID and continue.
;----------------------------------------------------------------------------
ItIsASNAPFrame:
	mov	di, offset CGroup:LookAheadSNAP
	sub	ax, ax				; ensure 1st byte ProtID is 0.
	mov	ah, byte ptr es:[bx+si].SNAP+0	; Copy 5 bytes of SNAP PID
	mov	word ptr [di].LProtID+0, ax	; Into 6 Byte ProtID Field
	mov	ax, word ptr es:[bx+si].SNAP+1
	mov	word ptr [di].LProtID+2, ax
	mov	ax, word ptr es:[bx+si].SNAP+3
	mov	word ptr [di].LProtID+4, ax
	add	bx, SNAPHeaderSize		; Nominal Header size (14+3+5)
	sub	bp, bx				; BP = Length of data
	lea	ax, [bx+si]			; AX -> Start of Data
	mov	cl, FRAME_IS_SNAP		; Set up Frame Value
	jmp short FrameTypeDetermined

;----------------------------------------------------------------------------
;    Frame Length < 802.5 Header Length + Sr Len + (Min MAC or 802.2 header size)
;----------------------------------------------------------------------------
TooSmallFrame:
	inc	DriverStatTable.MRxTooSmall	; Inc. Rx Too Small Count.
	or	MSMRcvPacketStatus, PKT_RUNT_ERR ; set Runt Packet bit
	jmp short TestFrame

;----------------------------------------------------------------------------
;	Watch out for Receiving our Own BroadCasts/Multicasts.
;----------------------------------------------------------------------------
PossiblyFromUs:
	mov	ax, word ptr es:[si].SrcAddr+2
	cmp	ax, word ptr MSMPhysNodeAddress+2 ; Check if from us.
	jne	NotFromUs		 ; ne => Good Packet To Look At.

	mov	ax, word ptr es:[si].SrcAddr+0
	and	al, not 80h		 ; Clear RII Bit.
	cmp	ax, word ptr MSMPhysNodeAddress+0 ; Check if from us.
	jne	NotFromUs		 ; ne => Good Packet To Look At.

RefusePacket:
	or	al , 1			 ; NZ Flag
	ret				 ; Signal that We Don't Want this packet.

;----------------------------------------------------------------------------
; Frame doesn't fit into a one of the known frame types, or is not supported.
;----------------------------------------------------------------------------
UnSupportedFrame:
	or	MSMRcvPacketStatus, PKT_MALFORM_ERR ; set malformed pkt flag
	mov	ax, si				    ; get offset start of MAC Header
	add	ax, dx				    ; get offset where MAC Header ends
	sub	bp, bp				    ; BP is Neg. Frame Size less than Frame type loaded
	or	MSMRcvPacketStatus, PKT_TYPE_ERR    ; set not ours Status, Board not installed
	jmp short TestLookAheadSize

;----------------------------------------------------------------------------
;	Flag Packet Length as unkown
;----------------------------------------------------------------------------
SetLDataSizeUnknown:
	mov	bp, -1				; Total Packet length Unknown.
	jmp short LDataSizeUnknownRet

;----------------------------------------------------------------------------
;	Flag this packet as having errors
;----------------------------------------------------------------------------
SetDestTypeErrorBit:
	mov	byte ptr DestAddrTypeFlag, DEST_ERRORED ; Exclusive no other bits should be set.
	jmp short SetDestTypeErrorBitRet

;----------------------------------------------------------------------------
;	Frame is completely contained in the LookAhead Data Buffer.
;----------------------------------------------------------------------------
FrameIsLessThanLookAhead:
	mov	dx, bp		; Use packet length instead of Buffer length.
	jmp short LookAheadSizeAdjust

;----------------------------------------------------------------------------
;	It is normal 802.2 Frame
;	Get DI ->Lookahead,
;	Fill in LProtID,
;	Determine 802.2 Type
;	Set up for Packet is 802.2 frame Type I (Default)
;	AX = DSAP SSAP value
;----------------------------------------------------------------------------
ItIsA8022Frame:
	mov	cl, FRAME_IS_8022		; CL = Flag which Frame Type.
	mov	di, offset CGroup:LookAhead8022	; DI = LookAhead Struc.
	mov	[di].LProtID+5, al		; PID is the DSAP
	inc	byte ptr DestAddrTypeFlag+1	; Signal 802.2 frame Type 1 (Default).

;----------------------------------------------------------------------------
;	Check for Which Format
;	Type I = UI-format, and U-Format
;	Type II = S-Format and I-Format
;	Bit 0 & 1 both set, implies U-Format
;----------------------------------------------------------------------------
	test	word ptr es:[bx+si].Control, SU_FormatBit + U_FormatBit	 
	jz	Type2Frame	; Z => I-format (Type 2)
	jpe	FormatAdjusted	; Parity Even => U-format (Type 1)
;	jpo	Type2Frame	; Parity Odd =>I-Format or S-Format (Type 2)

;----------------------------------------------------------------------------
;	Packet is Type 2, Add 1 to BX (Total Header size.)
;----------------------------------------------------------------------------
Type2Frame:		;Type 2 LLC frame is either "S" or "I" format.
	inc	byte ptr DestAddrTypeFlag+1	; Signal type 2 802.2
	inc	bx				; Add one extra for extra control byte.

;----------------------------------------------------------------------------
;	BP = Frame Length
;----------------------------------------------------------------------------
FormatAdjusted:					; Type 1 or 2 LLC now OK.
	add	bx, MinFrameHeaderLen		; Point past the LLC header. (Default Type 1)
	sub	bp, bx				; BP = FrameDataLength
	lea	ax, [si+bx]			; Skip Past the 802.2 header

;----------------------------------------------------------------------------
;	BP = FrameDataLength
;	AX = Ptr to the Start of Data
;	CL = Which Frame Type was Determined.
;	BX = Header size.
;	ES:SI -> HSM LookAhead Data Buffer
;	DS:DI -> LookAhead Structure
;----------------------------------------------------------------------------
FrameTypeDetermined:
	or	bp, bp		 ; test if valid BP
	js	UnSupportedFrame

;----------------------------------------------------------------------------
;	Determine if the Frame type is supported.
;----------------------------------------------------------------------------
	test	cl, FRAME_IS_MAC ; This is a runtime Modified Value
LoadedFrameByte:		 ; Point where the Loaded Bits will be stored
	jz	UnSupportedFrame ; z => Frame not loaded.

;----------------------------------------------------------------------------
;	Test for Priority Frame
;----------------------------------------------------------------------------
	mov 	dl, byte ptr es:[si].FCByte 		; Get Possible Priority
	and	dl, FC_PRIORITY_BITS			; Clear NonPriority Bits
	jz	TestLookAheadSize			; z => Non-Priority Frame

	mov	byte ptr [di].LPriorityLevel, dl	; Set Level
	and	DestAddrTypeFlag+1, RX_PRIORITY_FRAME	; Set Priority Bit
	
;----------------------------------------------------------------------------
;	Check if the Packet is completly contained in the LookAhead Struc.
;----------------------------------------------------------------------------
TestLookAheadSize:
	mov	dx, DriverConfigTable.MLookAheadSize
	cmp	dx, bp				; Is pckt len less than lookahead size?
	ja	FrameIsLessThanLookAhead

;----------------------------------------------------------------------------
;	Load LookAhead Struc
;----------------------------------------------------------------------------
LookAheadSizeAdjust:
	mov	[di].LMediaHeaderPtr+0, si	; Set the offset of look ahead media header
	mov	[di].LookAheadPtr+0, ax		; Set the offset of look ahead data
	mov	[di].LookAheadLen, dx
	cmp	HardwareRcvLength, -1		; Is packet length known?
	je	SetLDataSizeUnknown

;----------------------------------------------------------------------------
;	Set Attributes and check for Errors.
;----------------------------------------------------------------------------
LDataSizeUnknownRet:
	mov	word ptr [di].LStartCopyOffset, 0 ; Clear The Protocol Skip Value jwr
	mov	[di].LDataSize, bp		; Hardware reported length - headers
						;  will be -1 if unknown.
	mov	cx, MSMRcvPacketStatus		; Get packet status bits.
	mov	[di].LPacketAttrib, cx 		; Set status and 802.2 type in lookahead.
	or	cl, cl				; Was There any Errors?
	jnz	SetDestTypeErrorBit		; set bit and come back here

;----------------------------------------------------------------------------
;	Save Frame Header Len on the Stack and 
;	AX = LookAheadPtr
;	SI = MediaHeaderPtr
;	DX = LookAheadLen
;----------------------------------------------------------------------------
SetDestTypeErrorBitRet:	
	sub	ax, si				; Get media header length
	push	ax				; Save frame header length

;----------------------------------------------------------------------------
;	Check if the Address Needs to be swapped.
;----------------------------------------------------------------------------
	mov	bx, [di].LBoardNum		; Get BoardNumber
	test	BoardFlags[bx], MSWAPFLAG	; Need to Swap?
	jz	GetRcbDontSwapAddress

;----------------------------------------------------------------------------
;	Swap the Address and
;	Put the Immediate address in the LookAhead Struc and
;	Save the Immediate Address for stuffing ECB Later
;----------------------------------------------------------------------------
	mov	bx, PSPTableStart		; BitSwapTable @

;1st Word
	mov	ax, word ptr es:[si].SrcAddr+0	; Get the immediate address.
	and	al, not 80h			; Make sure RII bit is zero. (MSB)
	xlat					; Swap it
	xchg	ah, al				; AH gets result, AL is Next look up
	xlat					; Swap it
	xchg	ah, al				; straighten back for save
	mov	word ptr [di].LImmAddress+0, ax	; Store
	push	ax				; First Word of Address

;2nd Word
	mov	ax, word ptr es:[si].SrcAddr+2
	xlat					; Swap it
	xchg	ah, al				; AH gets result, AL is Next look up
	xlat					; Swap it
	xchg	ah, al				; straighten back for save
	mov	word ptr [di].LImmAddress+2, ax ; Store
	push	ax				; Middle word of Address

;3rd Word
	mov	ax, word ptr es:[si].SrcAddr+4
	xlat					; Swap it
	xchg	ah, al				; AH gets result, AL is Next look up
	xlat					; Swap it
	xchg	ah, al				; straighten back for save
	mov	word ptr [di].LImmAddress+4, ax
	push	ax				; Last Word of Address
	jmp short CheckforErrorsReceive		; Done here.

;----------------------------------------------------------------------------
;	Dont Swap the Address
;	Put the Immediate address in the LookAhead Struc and
;	Save the Immediate Address for stuffing ECB Later
;----------------------------------------------------------------------------
GetRcbDontSwapAddress:
	or	MSMRcvPacketStatus, IMM_ADR_MSB_FORM
	or	[di].LPacketAttrib, IMM_ADR_MSB_FORM	; Set Address Format bit.
	mov	ax, word ptr es:[si].SrcAddr+0	; Get the immediate address.
	and	al, not 80h			; Make sure RII bit is zero. (MSB)
	mov	word ptr [di].LImmAddress+0, ax
	push	ax				; First Word of Address

	mov	ax, word ptr es:[si].SrcAddr+2
	mov	word ptr [di].LImmAddress+2, ax
	push	ax				; Middle word of Address

	mov	ax, word ptr es:[si].SrcAddr+4
	mov	word ptr [di].LImmAddress+4, ax
	push	ax				; Last Word of Address

;----------------------------------------------------------------------------
;	Check for Errors before Calling SR module
;	Check for MAC frames Also.  Do not update SR for MAC frames.
;----------------------------------------------------------------------------
CheckforErrorsReceive:
	test	byte ptr DestAddrTypeFlag+0, DEST_ERRORED OR DEST_MAC_FRAME 
	jnz	BackFromSRInform

;----------------------------------------------------------------------------
;	Check if SR Module needs to be called.
;----------------------------------------------------------------------------
;	If the Packet is a Source Routed Frame (i.e. RII bit set), Set the 
;	DestAddrType Flag according to the following Rules:
;		1. Source Route Bit is set if the RII bit is set.
;		2. Source Route Bit is Exclusive:
;		if (SR length > 2) AND (Route.com is not loaded.)
;	Remember that the Address off the wire is always in MSB format.
;	Remember that the RII bit must be cleared prior to calling GetRCB
;----------------------------------------------------------------------------
	cli					; Hold on the Interrupts
	test	byte ptr es:[si].SrcAddr, 80h	; RII bit set?
	jz	SkipSettingBit			; If not set, accept packet

	or	byte ptr DestAddrTypeFlag+0, DEST_SOURCE_ROUTE	; Normal Mark of SR

SkipSettingBit:
	mov	bx, [di].RouteEntryPointOff	; Check to see if SR module is loaded
	mov	ax, cs
	cmp	word ptr [bx+2], ax		; SR module loaded?
	jne	InformSRModule

	test	byte ptr es:[si].SrcAddr, 80h	; RII bit set?
	jz	BackFromSRInform		; If not set, accept packet

	test	es:[si].SrcAddr+6, 1Ch	; If no SR module and SR length > 2 bytes?
	jz	BackFromSRInform	;  z = Set Bit, and Get RCB for Packet.

	mov	byte ptr DestAddrTypeFlag+0, DEST_SOURCE_ROUTE	; Signal Source Route w/o Route.com

;----------------------------------------------------------------------------
;	Set Destination Flag on Stack
;----------------------------------------------------------------------------
BackFromSRInform:
	mov	ax, DestAddrTypeFlag		; Get destination type bits.
	mov	[di].LDestType, ax		; Set destination type bits in lookahead.
	push	ax				; Save DestAddrTypeFlag for ECB

;----------------------------------------------------------------------------
;	See if the LSL can get an ECB from someone.
;----------------------------------------------------------------------------
	mov	bx, MLIDSUP_GET_STACK_ECB	; Attempt to get a Rx buffer
	call	LSLSupport
	cld					; Not guarenteed
	sti					; Allow Interrupts to fire.
	jnz	DiscardTheFrameCleanStack

;----------------------------------------------------------------------------
;	We got one, fill it out and return a Receive Control Block (RCB) to 
;	the MLID so it can disperse the frame data.
;----------------------------------------------------------------------------
;	First Set the Immediate address field in the ECB.
;	Calculate the Skip Values
;----------------------------------------------------------------------------
	pop	word ptr es:[si].DriverWS+0	; Set Dest Type and 802.2 type
	pop	word ptr es:[si].ImmAddr+4	; Set the immediate address
	pop	word ptr es:[si].ImmAddr+2
	pop	word ptr es:[si].ImmAddr+0

;----------------------------------------------------------------------------
;	Determine size of ECB buffers, 
;----------------------------------------------------------------------------
	xor	dx, dx
	mov	cx, es:[si].FragCount		; Number of Frags
	lea	bx, [si].Frag1Addr		; ES:BX -> ECB

TotalRxBuffer:
	add	dx, word ptr es:[bx].FragLen ; Total length of Rx buffers
	add	bx, size FragStruc		; Next Frag Please
	loop	TotalRxBuffer			; One less Frag

;----------------------------------------------------------------------------
;	Initialize some more ECB fields
;----------------------------------------------------------------------------
	mov	word ptr es:[si].DriverWS+2, bp	; Set the FrameDataLen up.
	mov	es:[si].DataLen, dx		; initialize the packet Data length field,
	mov	es:[si].Status, cx		; Clear ECB Status Field (CX = Zero)

;----------------------------------------------------------------------------
;	Get Protocol's Skip Count
;----------------------------------------------------------------------------
	mov	cx, word ptr [di].LStartCopyOffset ; CX = The Protocol Skip Value 

;----------------------------------------------------------------------------
;	Was Packet Size Known at Time of Getting ECB?
;----------------------------------------------------------------------------
	cmp	bp, -1			; Is packet length known?
	je	BackFromOverflow	; No, then skip overflow test.
	
;----------------------------------------------------------------------------
;	Check for Buffer Overflow Conditions
;----------------------------------------------------------------------------
	sub	bp, cx			; Adjust Number of Bytes to Move.
	cmp	dx, bp			; Is there enough room?
	jb	PossibleOverflow	; b => BufferSize < DataLen

	mov	es:[si].DataLen, bp	; Set the packet Data length field,

;----------------------------------------------------------------------------
;	Calculate/Save the Skip Values
;----------------------------------------------------------------------------
BackFromOverflow:
	pop	ax				; Restore the TSM skip value (Frame Header Length)
	mov	MSMRcvHeaderLen, ax		; Save for RxCompleteStatus
	mov	MSMRcvProtSkipCount, cl		; Save for RxCompleteStatus
	add	ax, cx				; Total Skip Count

;----------------------------------------------------------------------------
;	Count the Packet as Received and Return to MLID
;----------------------------------------------------------------------------
	add	DriverStatTable.MTotalRxPackets+0, 1
	adc	DriverStatTable.MTotalRxPackets+2, 0
	xor	cx, cx				; Set Z flag.
	ret

;----------------------------------------------------------------------------
;	Set up to call the SR Module
;	DS:SI -> Source Address
;	DS:CX -> SR info
;	BX = offset to SR module pointer
;	AX = CGroup
;----------------------------------------------------------------------------
InformSRModule:					;Assumes AX = CGroup
	mov	cx, es
	mov	ds, cx
	lea	cx, [si].DSAP			; DS:CX -> SR info (in any)
	add	si, SrcAddr			; DS:SI -> src address
	call	dword ptr cs:[bx]
	mov	ds, ax
	jmp short BackFromSRInform

;----------------------------------------------------------------------------
;	No One wants this frame scrap it off the stack and forget it.
;----------------------------------------------------------------------------
DiscardTheFrameCleanStack:
	add	DriverStatTable.MNoECBsAvailable+0, 1
	adc	DriverStatTable.MNoECBsAvailable+2, 0
	add	sp, 10		; Pop off DestType, Immed. Addr, and Skip value
				; Sets NZ flag appropriately
	ret

;----------------------------------------------------------------------------
;	The BufferSpace is less than data Available,
;	might indicate not be enough room for this packet.
;----------------------------------------------------------------------------
PossibleOverflow:
	mov	bp, dx
	mov	es:[si].Status, LSLERR_RX_OVERFLOW
	jmp	BackFromOverflow		;buffer space provided

MSMGetRCB	endp


ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	MSMReturnRCB
;
;	Called by the driver when it needs to return an allocated RCB that
;	it decided it really didn't need after all.
;
;	assumes:
;		es:si	-> RCB structure
;		ds	is set to CGroup
;		Interrupts are disabled
;
;	returns:
;		bp,ds,ss,sp preserved
;		Interrupts disabled
;
;****************************************************************************
MSMReturnRCB	proc	near
	mov	word ptr es:[si].PrevLink, 0	; Clear the first word for RxStatus.
						; 0 RxStatus, and Cancelled => ReturnRCB 
ChangePacketStatus:
	mov byte ptr es:[si].DriverWS+0, DEST_ERRORED ; Change dest. type as error.
	mov	es:[si].Status, LSLERR_CANCELLED	; Set the ECB's status field.
	jmp short MSMRcvCompleteProceed			; Return the ECB.

MSMReturnRCB	endp

;****************************************************************************
;	MSMRcvCompleteStatus
;
;	Called by the MLID when it has finished dispersing the frame
;	data into the RCB fragment buffers.
;
;	This call passes control of the RCB to us. The HSM should
;	not modify any values in the RCB after calling this routine.
;
;	assumes:
;		es:si	-> RCB structure
;		ax = Status
;		bp = Total Length of Packet
;		ds	is set to CGroup
;		Interrupts are disabled
;
;	returns:
;		ds,ss,sp preserved
;		Interrupts disabled
;
;****************************************************************************
;----------------------------------------------------------------------------
;	Not enough buffers were provided for the full FrameDataLen
;	Indicate overflow condition, and 
;	Reset DataLen with the number that was moved.
;----------------------------------------------------------------------------
SetOverFlowStatus:
	mov	word ptr es:[si].DataLen, bp	; bp = Number of bytes that moved
	mov	word ptr es:[si].Status, LSLERR_RX_OVERFLOW
	jmp short MSMRcvComplete

;----------------------------------------------------------------------------
;	Hardware Reported Length is larger than Media Max.
;----------------------------------------------------------------------------
RcvCmpltFrmSizeTooBig:
	inc	DriverStatTable.MRxTooBig		; Inc. Rx Too Big count.
	or  byte ptr MSMRcvPacketStatus, PKT_BIG_ERR	; Set Too Big bit in packet
	jmp short RcvCmpltFrmSizeErrorRtn		;  status.

;----------------------------------------------------------------------------
;MSMRcvCompleteStatus Starts HERE:
;----------------------------------------------------------------------------
MSMRcvCompleteStatus	proc near
;----------------------------------------------------------------------------
;	Set the Current Status field, (sets up for fall through code later.)
;----------------------------------------------------------------------------
	or	MSMRcvPacketStatus, ax		; Set the Current FrameStatus

;----------------------------------------------------------------------------
;	Check that the Frame Meets current regs.
;	Is reported Frame size Larger than Media can Handle?
;----------------------------------------------------------------------------
	cmp	bp, DriverConfigTable.MMaxPacketSize	; BP = Reported FrameSize
	ja	RcvCmpltFrmSizeTooBig		; a => Too Big
							
;----------------------------------------------------------------------------
;     Determine the Frame size and number bytes that should have been copied.
;----------------------------------------------------------------------------
RcvCmpltFrmSizeErrorRtn:
	mov	ax, MSMRcvHeaderLen		; Get HeaderLen and set AH=0
	sub	bp, ax				; Subtract off the Header Value
	mov	word ptr es:[si].DriverWS+2, bp	; Store Total FrameDataLen
	mov	al, MSMRcvProtSkipCount		; Get Protocol Skip Count
	sub	bp, ax				; BP = Number of Bytes that Moved
	xchg	word ptr es:[si].DataLen, bp	; BP = Total Buffers, DataLen = Bytes to move
	cmp	bp, word ptr es:[si].DataLen	; (BP=TotalBuffers) < (Bytes to move)
	jb	SetOverFlowStatus		; TotalBuffers < FrameDataLen == Overflow

MSMRcvCompleteStatus	endp			; Fall into MSMRcvComplete to finish up.

;****************************************************************************
;	MSMRcvComplete
;
;	Called by the MLID when it has finished dispersing the frame
;	data into the RCB fragment buffers.
;
;	This call passes control of the RCB to us. The driver should
;	not modify any values in the RCB after calling this routine.
;
;	assumes:
;		es:si	-> RCB structure
;		ds	is set to CGroup
;		Interrupts are disabled
;
;	returns:
;		bp,ds,ss,sp preserved
;		Interrupts disabled
;
;****************************************************************************
MSMRcvComplete	proc	near
	mov	ax, MSMRcvPacketStatus
	mov	word ptr es:[si].PrevLink, ax

;----------------------------------------------------------------------------
;	Check for any error condition.
;----------------------------------------------------------------------------
	or	al, al		; Any errors?
	jnz	ChangePacketStatus

;----------------------------------------------------------------------------
;	Give the ECB back to the LSL
;----------------------------------------------------------------------------
MSMRcvCompleteProceed:
	mov	bx, MLIDSUP_HOLD_RECV_EVENT	; Give it to the LSL
 	call	LSLSupport
	ret

MSMRcvComplete	endp

ifdef TASM
%NEWPAGE ;
endif
;*******************************************************************************
;              T O K E N   R I N G   M U L T I C A S T   C O D E
;*******************************************************************************
;	MSMAddMulticastAddress
;
;	Called by a protocol stack to enable reception of a particular
;	multicast address.  In other words, we become a member of a 
;	multicast group.  In this case we become a member of a Functional or
;	Group address group.
;
;	assumes:
;		ax	Board Number
;		es:si	-> 6 byte buffer holding the Group or Functional address
;		ds	is set to CGroup
;		CLD has been executed
;		Interrupts are enabled
;			MSM disables them prior to calling TSM
;
;	returns:
;		ax	=0 if command was successful
;			=LSLERR_BAD_PARAMETER is addr was invalid
;			=LSLERR_OUT_OF_RESOURCES if max. already in use
;		All other registers destroyed
;		Interrupts Preserved
;	Note:
;		If the Address is passed as a C0 00 xx xx xx xx, then it can
;		be assumed to be in Non-Canonical Form Already.
;
;****************************************************************************
MSMAddMulticastAddress	proc	near
;----------------------------------------------------------------------------
;    Place the correct form of the Address in the MulticastSwapArea variable.
;----------------------------------------------------------------------------
	call	PrepareMulticast	; Guarentee the address in proper form.
	jnz	MSMAddTokenGrpAddr	; NZ - Token-Ring Group Address

;----------------------------------------------------------------------------
;	Check for valid Functional Address
;----------------------------------------------------------------------------
	cmp	word ptr [si+0], 00C0h	; Functional Address?
	jne	InvalidAddress

;----------------------------------------------------------------------------
;	Update our Functional Address table.
;----------------------------------------------------------------------------
	mov	di, offset CGroup:FunctionalAddressTable
	mov	ax, word ptr [si+4]	; Get last two bytes of address
	mov	bx, 2			; Two words in functional address

DoSecondWordAdd:
	mov	cx, 16			; 16 bits to do
	
SetupMCTableLoop:
	shl	ax, 1			; Get bit into Carry flag
	adc	byte ptr [di], ch	; Increment if bit is set
	inc	di			; Advance to next slot
	loop	SetupMCTableLoop

	mov	ax, word ptr [si+2]	; Get Middle two bytes of address
	dec	bx
	jnz	DoSecondWordAdd

	jmp short IssueChangeToHSM

;----------------------------------------------------------------------------
;	Add a Group Address if Supported
;----------------------------------------------------------------------------
MSMAddTokenGrpAddr:
	test	[bx].MFlags, GROUP_ADDRESSING_BIT ; Do we support GroupAddresses.
	jz	InvalidAddress			  ; z => No GroupAddress.

;----------------------------------------------------------------------------
;	Check if the address is already in the table.
;----------------------------------------------------------------------------
	call	FindAddressInMCTableNotLoaded	; See if already added (AX:BX:DX not loaded yet)
	jz	MSMAddAddrWasFound		; z => DI -> Points to address

;----------------------------------------------------------------------------
; 	Find an empty slot.
;----------------------------------------------------------------------------
  	mov	di, offset CGroup:MulticastTable
	mov	cx, MSMMaxMulticastAddr		; Get max. possible in HSM

MSMAddrFindEmptySlotLoop:
	cmp	word ptr [di+6], 0		; Slot in use?
	je	MSMAddFoundEmptyMCSlot

	add	di, 8				; Advance to next slot
	loop	MSMAddrFindEmptySlotLoop

;----------------------------------------------------------------------------
;	No slot left to add the Address.
;	Report back the Error
;----------------------------------------------------------------------------
	mov	ax, LSLERR_OUT_OF_RESOURCES	; No free slots
	ret					; PSW set on return

;----------------------------------------------------------------------------
;	The address given is either invalid or not supported.
;	Report back the Error
;----------------------------------------------------------------------------
InvalidAddress:
	mov	ax, LSLERR_BAD_PARAMETER	; Bad Address given
	ret					; PSW set on return

;----------------------------------------------------------------------------
;	Store the Address in the Table.
;----------------------------------------------------------------------------
MSMAddFoundEmptyMCSlot:
	mov	word ptr [di+0], ax		;Save the address
	mov	word ptr [di+2], bx
	mov	word ptr [di+4], dx
	inc	word ptr [di+6]			; Increment use count
	mov	bx, ADD_MULTICAST_ADDRESS	; Indicate that the Address at DS:DI was added

;----------------------------------------------------------------------------
;	Tell the HSM the MC Table has changed.
;----------------------------------------------------------------------------
IssueChangeToHSM:
	call	MSMUpDateMulticastInternal

MSMNoChange:
	xor	ax, ax
	ret					; Finished and out.

MSMAddAddrWasFound:
	inc	word ptr [di+6]			; Increment use count
	jmp short MSMNoChange			; Dont Bother The HSM


MSMAddMulticastAddress	endp


ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	MSMDelMulicastAddress
;
;	Called by a protocol stack to disable reception of a particular
;	multicast address.
;
;	assumes:
;		ax	Board Number
;		es:si	-> 6 byte buffer holding the Group or Functional address
;		ds	is set to CGroup
;		CLD has been executed
;		Interrupts are enabled
;			MSM disables them prior to calling TSM
;
;	returns:
;		ax	=0 if command was successful
;			=LSLERR_BAD_PARAMETER is addr was invalid
;			=LSLERR_ITEM_NOT_PRESENT if addr was not found
;		All other registers destroyed
;		Interrupt State Preserved.
;
;	Note:
;		If the Address is passed as a C0 00 xx xx xx xx, then it can
;		be assumed to be in Non-Canonical Form Already.
;
;****************************************************************************
MSMDelMulticastAddress	proc	near
;----------------------------------------------------------------------------
;    Place the correct form of the Address in the MulticastSwapArea variable.
;----------------------------------------------------------------------------
	call	PrepareMulticast	; Guarentee the address in proper form.
	jnz	MSMDelTokenGrpAddr	; NZ -Token-Ring Group Address

;----------------------------------------------------------------------------
;	Check for valid Functional Address
;----------------------------------------------------------------------------
	cmp	word ptr [si+0], 00C0h	; Functional Address?
	jne	InvalidAddress

;----------------------------------------------------------------------------
;	Update our Functional Address table.
;----------------------------------------------------------------------------
	sub	dx, dx				; DX = Success Code. Init for Success.
	mov	di, offset CGroup:FunctionalAddressTable;
	mov	ax, word ptr [si+4]		; Get last two bytes of address
	mov	bx, 2				; do Two Words.

;----------------------------------------------------------------------------
;	Check each bit of the Functional Address.
;----------------------------------------------------------------------------
DoSecondWordDel:
	mov	cx, 16				;16 bits to do
	
SetupMCTableLoopDel:
	shl	ax, 1				; Get bit into Carry flag
	sbb	byte ptr [di], ch		; Decrement if bit is set
	jnc	DidNotGoBelowZero		; Keep it from underflowing.

	mov	byte ptr [di], ch		; Set to zero, Bit wasn't set before.
	mov	dx, LSLERR_ITEM_NOT_PRESENT	; Indicate an error.

DidNotGoBelowZero:
	inc	di				;Advance to next slot
	loop	SetupMCTableLoopDel

	mov	ax, word ptr [si+2]		;Get Middle two bytes of address
	dec	bx
	jnz	DoSecondWordDel

;----------------------------------------------------------------------------
;	Tell the HSM the MC Table has changed, 
;	Preserve any Error Code in DX
;----------------------------------------------------------------------------
	push	dx				; Preserve possible Error code.
	call	MSMUpDateMulticastInternal
	pop	ax				; Return possible Error code
	ret					; Finished and out.

;----------------------------------------------------------------------------
;	Delete a Group Address if Supported
;----------------------------------------------------------------------------
MSMDelTokenGrpAddr:
        test    [bx].MFlags, GROUP_ADDRESSING_BIT ; Do we support GroupAddresses.
	jz	InvalidAddress			  ; z => No GroupAddress.

	call	FindAddressInMCTableNotLoaded	; See if already added (AX:BX:DX not loaded yet)
	mov	ax, LSLERR_ITEM_NOT_PRESENT	; No change to tell HSM (Possible Error code)
	jnz	AddressWasNotFound

;----------------------------------------------------------------------------
;	Address was found, Decrement count and continue
;----------------------------------------------------------------------------
	dec	word ptr [di+6]			; Adjust use count
	mov	bx, DELETE_MULTICAST_ADDRESS	; Let the HSM Know we are Deleting address at DS:DI
	jz	IssueChangeToHSM

	xor	ax, ax		; Signal Success

AddressWasNotFound:
	ret

MSMDelMulticastAddress	endp

ifdef TASM
%NEWPAGE ;
endif
;*******************************************************************************
;	MSMGetMulticastInfo
;
;	Called by a protocol stack to get pointers to the Multicast Table.
;
;	assumes:	(MSMControlHandler has been called)
;		
;		DS	is set to CGroup
;		CLD has been executed
;		Interrupts are enabled
;				Prior to calling TSM the MSM disables Ints
;					
;
;	returns:
;		ax	=LSL_SUCCESSFUL (0) if command was successful
;		ES:SI -> Multicast Table (Group Address)
;		ES:DI -> Functional Address (Not Used for Ethernet)
;		CX	Size of the Multicast Table.
;	      (MSMControlHandler takes care of any other return values.)
;			
;		Flags set according to AX 
;		DS, BP, SS, SP registers preserved
;		Interrupt State is preserved.
;
;*******************************************************************************
MSMGetMulticastInfo	proc	near
	xor	ax, ax				; setup for return code.
	push	ds
	pop	es				; Get CGROUP
	mov	si, offset CGroup:MulticastTable ; Located in PSP area.
	mov	di, offset CGroup:FunctionalAddressPrefix ; Active Functional Address
	mov	cx, MSMMaxMulticastAddr		; Set for table size.
	ret
MSMGetMulticastInfo	endp

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	PrepareMulticast
;	This function sets up the Multicast Address given from either
;	MSM_ADD_Multicast or from MSM_DELETE_Multicast.
;
;	On Entry:
;		ax	Board Number
;		es:si	-> 6 byte buffer holding the Group or Functional address
;		ds	is set to CGroup
;		CLD has been executed
;		Interrupts are Disabled
;	On Exit:
;		DX = 0
;		si = MulticastSwap Area with Address that was passed in
;			converted to proper format.
;		bx = Pointer to MSMConfigTables[boardnumber]
;
;		ds= es = cs = CGroup
;		Flags:
;			NZ => Group Address
;			Z => Functional Address
;		other registers destroyed.
;		Interrupts unchanged.
;
;~~~~~~~~~~~~~~~~~~~~ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PrepareMulticast	proc near
;----------------------------------------------------------------------------
;	Save Word Board Index Value
;----------------------------------------------------------------------------
	xchg	bx, ax			; BX = Board Number, AX value not needed.

;----------------------------------------------------------------------------
;	Move to Local Data Space for Possible Swap.
;----------------------------------------------------------------------------
	mov	di, offset CGroup:MulticastSwapArea
	mov	ax, word ptr es:[si]
	mov	word ptr [di], ax
	mov	ax, word ptr es:[si+2]
	mov	word ptr [di+2], ax
	mov	ax, word ptr es:[si+4]
	mov	word ptr [di+4], ax

;----------------------------------------------------------------------------
;	check if the Address was supposed to be in LSB or MSB Mode.
;----------------------------------------------------------------------------
	mov	bl, MSMTableIndex[bx]			; BX = Word Index
	mov	bx, MSMConfigTables[bx]			; Get ptr to ConfigTable.
	push	bx					; Preserve around  Swap code
	test	word ptr [bx].MModeFlags, MNonCanonicalBit ; NonCanonical? (MSB?)
	jnz	MCAddressOK				; Already in MSB form

;----------------------------------------------------------------------------
;	Check if the Address is already MSB form
;----------------------------------------------------------------------------
	cmp	word ptr [di], 00C0h	; IS it Non-Canonical Form already?
	je	MCAddressOK		; e = Already Non-Canonical Form.

;----------------------------------------------------------------------------
;	Setup to Swap Address.		; DI and SI = MulticastSwapArea.
;----------------------------------------------------------------------------
	mov	cx, 3
	mov	bx, PSPTableStart    	; BitSwapTable address.
	mov	si, di			; Point SI -> MulticastSwapArea.
	mov	ax, cs
	mov	es, ax			; ES = CGroup

LoopSwapMC:
	lodsw			 	; AX = Byte of Address
	xlat				; Swap AL
	xchg	ah, al			; AH gets result, AL is Next look up
	xlat				; Swap AL
	xchg	ah, al			; straighten back for save
	stosw				; Put Back in Memory
	loop	LoopSwapMC		; Next

;----------------------------------------------------------------------------
;	See if it is a Token-Ring Functional or Group address.
;----------------------------------------------------------------------------
MCAddressOK:
	xor	dx, dx
	mov	si, offset CGroup:MulticastSwapArea	; Point back to the start.
	test	byte ptr [si+2], 80h			; Group bit set?
	pop	bx					; Pointer to MSMConfigTables[boardnumber]
	ret
PrepareMulticast	endp

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	MSMUpdateMulticast
;
;	This routine is assumed to be in the media specific module and will
;	be called by the generic MSM module after a DriverReset IOCTL
;	has been issued to the HSM.  By calling this routine after
;	DriverReset we guarantee the HSM has the proper multicasts
;	setup after a reset.
;
;	MSMUpdateMulticast will not call DriverMulticastChange if in 
;	Promiscuous Mode as this may well affect a number of Multicast 
;	functions in a LAN Controller, eg. 8390.  But will call 
;	DriverPromiscuousChange instead, as this allows HSM's to be re-enabled
;	for Promiscuous Mode if internally reset, eg. after an internal
;	reset the HSM calls MSMUpdateMulticast to ensure correct settings
;
;	assumes:
;		ds	CGroup
;		BX	= Function Indicator
;			 0 => DI not Valid
;			 DELETE_ADD_MULTICAST_ADDRESS
;			 ADD_MULTICAST_ADDRESS
;		DS:DI -> Address in Multicast Table if Function Indictor !=0
;
;		CLD has been executed
;		Interrupts are Disabled
;
;	returns:
;		Everything destroyed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
MSMUpdateMulticast	proc	near
	xor	bx, bx				; Signal that this is not an Add/Delete Multicast call

MSMUpDateMulticastInternal:
;----------------------------------------------------------------------------
;	Set up DX:AX with functional address.
;----------------------------------------------------------------------------
	mov	bp, offset CGroup:FunctionalAddressTable
	push	bx				; Save Multicast Action State
	mov	bx, 2

DoSecondWord1:
	mov	cx, 16				; 16 bits to do
	
SetupMCAddressLoop:
	cmp	ch, byte ptr ds:[bp]		; Sets Carry Bit
	rcl	ax, 1				; Move in carry bit.
	inc	bp				; Advance to next slot.
	loop	SetupMCAddressLoop

	xchg	ax, dx
	dec	bx
	jnz	DoSecondWord1

	pop	bx

;----------------------------------------------------------------------------
;	Store the current Functional Address for faster detection on RX
;----------------------------------------------------------------------------
	mov	word ptr ActiveFunctionalAddress + 0, dx	; Save Address.
	mov	word ptr ActiveFunctionalAddress + 2, ax

;----------------------------------------------------------------------------
;	Check for Promiscuous State.
;----------------------------------------------------------------------------
	mov	cx, MSMActivePromisMask	; Get Current State of Promiscuous Mode
	jcxz	MSMUpdateMulticastCont	; Test if in Promis. Mode cxz =>NO, skip down

;----------------------------------------------------------------------------
;	We are in Promiscuous Mode, 
;----------------------------------------------------------------------------
	jmp	DriverPromiscuousChange	; CX = Not Zero -> IN Promiscuous Mode.

;----------------------------------------------------------------------------
;	Not in Promiscous Mode, 
;	Tell the HSM the Current Group/Functional Address.
;----------------------------------------------------------------------------
MSMUpdateMulticastCont:
	mov	si, offset CGroup:MulticastTable; Group Address Table.
	mov	cx, MSMMaxMulticastAddr		; Max number of Group Addresses.
	jmp	DriverMulticastChange		; Tell the driver about the update

MSMUpdateMulticast	endp

ifdef TASM
%NEWPAGE ;
endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	FindAddressInMCTable
;
;	Called to see if the specified Group or Functional address is present
;	in the filter tables.
;
;	assumes:
;		si	-> to the 6-byte address
;		ds	is set to CGroup
;		CLD has been executed
;		Interrupts are disabled
;
;	returns:
;		z flag 	set if address was found
;		nz flag	is addr not found
;		ds:di	-> Addr entry if Group Address was found
;		CLD preserved
;		CX, DI destroyed
;		AX, BX, DX = Address we were looking for.
;		All other registers preserved
;			Interrupts disabled and not enabled
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FindAddressInMCTableNotLoaded:
	mov	ax, word ptr [si+0]		; Load up the address
	mov	bx, word ptr [si+2]
	mov	dx, word ptr [si+4]

FindAddressInMCTable	proc	near
;----------------------------------------------------------------------------
;	Check if a Functional or Group Address
;----------------------------------------------------------------------------
	test	bx, 80h				 ; Group or Functional?
	jnz	LookForGroupAddress

;----------------------------------------------------------------------------
;	Functional Address Check
;----------------------------------------------------------------------------
	test	word ptr ActiveFunctionalAddress + 0, bx
	jnz	SuccessfulExit

	test	word ptr ActiveFunctionalAddress + 2, dx
	jnz	SuccessfulExit

	jmp short UnSuccessfulExit

;----------------------------------------------------------------------------
;	Check in The GroupAddress Table (Similar to Multicast in Ethernet)
;----------------------------------------------------------------------------
LookForGroupAddress:
	mov	di, offset CGroup:MulticastTable 
	mov	cx, MSMMaxMulticastAddr	; Get max. possible inuse in the table

LookUpAddrLoop:
	cmp	word ptr [di+6], 0	; Entry in use?
	je	NotThisEntry

	cmp	word ptr [di+4], dx	; Check if Address Matches.
	jne	NotThisEntry

	cmp	word ptr [di+2], bx
	jne	NotThisEntry

	cmp	word ptr [di+0], ax
	jne	NotThisEntry

;----------------------------------------------------------------------------
;	Address was Found it is in the Table.
;----------------------------------------------------------------------------
SuccessfulExit:
	xor	cx, cx			; Set Z flag (Addr found).
	ret

;----------------------------------------------------------------------------
;	Address doesn't match this Entry, Move to next one and check again.
;----------------------------------------------------------------------------
NotThisEntry:
	add	di, 8			; Get next table entry.
	loop	LookUpAddrLoop
	
;----------------------------------------------------------------------------
;	Address wasn't Found in the Tables.
;----------------------------------------------------------------------------
UnSuccessfulExit:	
	or	cx, di			; Set NZ flag (Addr not found).
	ret

FindAddressInMCTable	endp


;****************************************************************************
;  MSMMediaConfigUpdate
;	Called at Init time at the end of DriverInit
;		to initially set up the Config table and Media specific stuff.
;	Called at Run time by PCMCIA cards at the end of DriverReset
;		when a Card is inserted to refresh the config table 
;		and Media specific stuff.
;
;	On ENTRY:
;		DS = CS = CGroup
;		Interrupts disabled
;
;	On Return:
;		AX = Error code
;			0 = success,
;			MSMERR_TSM_FAILED => error occured.
;
;		DX = Message Offset of the Error.
;		     only the Init time can the Error Code be used to print
;		     the error message.
;
;		Interrupts disabled
;		SS,SP preserved.
;****************************************************************************
;----------------------------------------------------------------------------
;	Error Exit
;----------------------------------------------------------------------------
HSMForgotExit:
	mov	ax, MSMERR_TSM_FAILED	; DX = MSG offset for init time
	or	ax, ax			; Set appropriate flags.
	ret		

MSMMediaConfigUpdate	proc	near
;----------------------------------------------------------------------------
;	Guarentee that ES is set to CGroup
;----------------------------------------------------------------------------
	push	cs
	pop	es	; CGroup

;----------------------------------------------------------------------------
;	Check that the HSM has set the MSMTxFreeCount
;----------------------------------------------------------------------------
	mov	dx, offset CGroup:ForgotTxFreeCount	; Possible Error
	mov	al, MSMTxFreeCount		; Save a copy of the value
	mov	MSMMaxActiveTx, al
	or	al, al				; Check if even set?
	jz	HSMForgotExit

	mov	dx, offset CGroup:TooManyTCBs	; Possible Error
 	cmp	al, MAX_TCB_ALLOCATED		; Does it exceed Max number of TCBs.
	ja	HSMForgotExit

;----------------------------------------------------------------------------
;	Now check to make sure the HSM set the MSMLookAheadSegment
;----------------------------------------------------------------------------
	mov	ax, MSMLookAheadSegment
	mov	dx, offset CGroup:ForgotLASegment
	or	ax, ax
	jz	HSMForgotExit

;----------------------------------------------------------------------------
;	Initialize the Look Ahead structures Segment Values
;----------------------------------------------------------------------------
	mov	LookAhead8022.LMediaHeaderPtr+2, ax
	mov	LookAhead8022.LookAheadPtr+2, ax

	mov	LookAheadSNAP.LMediaHeaderPtr+2, ax
	mov	LookAheadSNAP.LookAheadPtr+2, ax

;----------------------------------------------------------------------------
;	Set up the Node Address in all Config Tables.
;	Config Table copy may be in either LSB or MSB as Configured.
;----------------------------------------------------------------------------
	mov	bx, MSMActiveBoardCount		; Number of Loaded FrameTypes
	add	bx, bx				; Dword Offset
	
CheckNextBoardsAddress:
	mov	di, MSMConfigTables[bx-2]	; Get first Config Table
	mov	ax, DriverConfigTable.MMaxPacketSize ; Get MAX for Board.
	mov	[di].MMaxPacketSize, ax		; Set Max for This Frame Type
	xor	cx, cx				; Clear CX
	mov	si, [di].MBoardNumber		; SI = Board Number
	mov	cl, HeaderSizeTable[si]		; CX = HeaderSize
	sub	ax, cx				; Subtract Header Size.
	mov	[di].MBestDataSize, ax		; Store Size
	sub	ax, SR_LENGTH			; Take Into Account SR & SRT
	cmp	[di].MFrameID, MediaID8022	; Check for 802.2
	jne	SetWorstDataSize		; Worst case only SR & SRT

	dec	ax				; Except in 802.2 case need to sub one more.

SetWorstDataSize:
	mov	[di].MWorstDataSize, ax		; Store the WorstDataSize.
	mov	bp, di				; BP = Config Table Ptr
	lea	si, word ptr MSMPhysNodeAddress	; Constant for all Frame Types.
	lea	di, word ptr [di].MNodeAddress

	test	word ptr ds:[bp].MModeFlags, MNonCanonicalBit ; NonCanonical?
	jnz	MoveNonCanonical

;----------------------------------------------------------------------------
;	Do The Bit Swap As it is moved up to the Node Address.
;----------------------------------------------------------------------------
	push	bx
	mov	bx, ds:[bp].MBoardNumber	; BX = Board Number
	or	BoardFlags[bx], MSWAPFLAG	; Signal we need to always Swap on this board.
	mov	cx, 6				; 6 bytes to node address
	call	MSMBitSwapAddress
	pop	bx
	jmp short NextBoard

MoveNonCanonical:
	mov	cx, 3
 rep	movsw

NextBoard:
	sub	bl, 2				; Set Index
	jnz	CheckNextBoardsAddress		; Check each Board

;----------------------------------------------------------------------------
;	Setup the source node address in all of the media headers.
;----------------------------------------------------------------------------
	mov	bx, TCBFreeList			;Get ptr first TCB
	lea	si, MSMPhysNodeAddress		;DS:SI -> Physical node address

DoNextTCB:
	push	si
	lea	di, [bx].TCBMediaHeader.SrcAddr	;ES:DI -> src addr field
	movsw					;Set the src address
	movsw
	movsw
	pop	si

;----------------------------------------------------------------------------
;	Setup the AC FC Bytes in Each TCB Header.
;----------------------------------------------------------------------------
	mov	word ptr [bx].TCBMediaHeader.ACByte, 4000h  ; Set AC = 00h and FC = 40h
	mov	bx, [bx+0]		; Next TCB
	or	bx, bx
	jnz	DoNextTCB

	jmp	MSMUpdateConfigTables	; Any Errors will pass back out on ret.

MSMMediaConfigUpdate	endp

Code	ends

;*******************************************************************************
;                  M E D I A   I N I T I A L I Z A T I O N
;*******************************************************************************
Init	segment	para public 'CODE'
	assume	cs:CGroup, ds:CGroup, es:nothing, ss:nothing

	public	InitializeMedia
	public	HSMProvideTCB
	public	SupportedMedias

;----------------------------------------------------------------------------
;	Provided by the MSM
;----------------------------------------------------------------------------
	extrn	MSMAlloc		:near
	extrn	DriverInit		:near
	extrn	MSMEatWhite		:near
	extrn	MSMGetStringLength	:near
	extrn	MSMHexStringToByteArray :near
	extrn	MSMPrintStringZero	:near
	extrn	MSMBitSwapAddress	:near

;----------------------------------------------------------------------------
;	Messages from MSMtxt.inc
;----------------------------------------------------------------------------
	extrn	InvalidMediaAddress	:byte
	extrn	InvalidMediaAddress2	:byte
	extrn	ForgotTxFreeCount	:byte
	extrn	ForgotLASegment		:byte
	extrn	TooManyTCBs		:byte

;Internal Debug
ifdef DEBUG
	public	MediaNodeAddressCheck
	public	PhysNodeInMSBForm, CheckSwapBitAddr
	public	MediaIndexToBoardMap, SetDefaultLBoardNum
endif

;----------------------------------------------------------------------------
;		M E S S A G E   C E N T E R
;		  Media Specific Messages.
;----------------------------------------------------------------------------
		db	'MEDIA = ',TokenTSMVersion		; Must always be present
TokenMsg	db	'Token-Ring',0

;----------------------------------------------------------------------------
;	This table is required for use with the generic MSM module.
;----------------------------------------------------------------------------
SupportedMedias	dw	MaxVirtBoards	 	  ;Num. medias supported

		; Token-Ring
		dw	offset CGroup:MediaName4  ;Default is first one
		dw	MediaID8022		  ;Raw 802.3
		db	0,0,0,0,0,0e0h		  ;Default ProtocolID for IPX

		; Token-Ring_SNAP
		dw	offset CGroup:MediaName11 ;SNAP
		dw	MediaIDSNAP
		db	0,0,0,0,81h,37h		  ;Default ProtocolID for IPX

;----------------------------------------------------------------------------
;	Index by word FrameID Index.
;	Must have each entry up to the Last Entry supported by this Media
;----------------------------------------------------------------------------
MediaToBoardIndexTable	dw	4 dup (0)	; Skip Media 0-3
			dw	TR8022Index	; Index for 802.2 (4)
			dw	6 dup (0)	; Skip Media 5-10
			dw	TRSNAPIndex	; Index for SNAP (11)

;----------------------------------------------------------------------------
;	Index Equates for Accessing the following Tables.
;----------------------------------------------------------------------------
TR8022Index        	equ	0	; Index for 802.5/802.2
TRSNAPIndex        	equ	2	; Index for 802.5/802.2/SNAP

;----------------------------------------------------------------------------
;	Index equates are used to access the Size Table.
;----------------------------------------------------------------------------
MediaHeaderSizeTable	dw	17	; 802.5/802.2
			dw	22	; 802.5/802.2/SNAP

;----------------------------------------------------------------------------
;	Index equates access this table.
;----------------------------------------------------------------------------
MediaTxHandlerTable	dw	offset CGroup:Build8022Header
			dw	offset CGroup:BuildSNAPHeader

;----------------------------------------------------------------------------
;	Index Equates used to access this table 
;	Board number for Frame types, 
;	Default Board number is set to allow error packets to be passed up.
;	Default will be set to the initial logical board number.
;----------------------------------------------------------------------------
MediaIndexToBoardMap		dw	MaxVirtBoards dup (0)	;Holds the board nums for each virtual board

;----------------------------------------------------------------------------
;	Index Equates used to access this table 
;	Bit Table Used to Set the LoadedFrameByte Field
;----------------------------------------------------------------------------
LoadedFrame	dw	FRAME_IS_8022	
		dw	FRAME_IS_SNAP	


ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;	InitializeMedia
;
;	Called by the main MSM to do any initialization of variables
;	and tables that will be used in building and stripping our media 
;	headers. We also call the HSM's DriverInitialize.
;
;	assumes:
;		DS	is set to CGroup
;		CLD has been executed
;		Interrupts are enabled
;
;	returns:
;		ax	=0 if no errors
;			!= 0 if error (DX is valid)
;		dx	-> err message offset if AX != 0
;		DS = ES = CGroup
;		flags set according to AX
;		CLD has been executed
;		Interrupts are enabled.
;
;****************************************************************************
InitializeMedia	proc	near
;----------------------------------------------------------------------------
;	Test Node Address override if any correct for media.
;----------------------------------------------------------------------------
	push	cs
	pop	es
	call	MediaNodeAddressCheck		; Check for Node Address Override.

;----------------------------------------------------------------------------
;	Enable the counter bits that we increment.
;	Prior to the AND Need to invert our logic, set => not used.
;	Remember this 32 bit number high order bits were used first.
;----------------------------------------------------------------------------
	and	word ptr DriverStatTable.MValidCounterMask+2, not 1111001100001100b

;----------------------------------------------------------------------------
;	Initialize the LookAhead Board Number to the First Logical board
;	supported by the HSM as a default
;----------------------------------------------------------------------------
	mov	bx, DriverConfigTable.MBoardNumber ; Get Default Board Num
	mov	bp, (MaxVirtBoards-1)		   ; BP = Num of possible Virt Boards (0 based)

SetDefaultLBoardNum:
	mov	ds:MediaIndexToBoardMap[bp], bx	; Save Real Board Number in look up table
	dec	bp				; count down till done.
	jnz	SetDefaultLBoardNum		; nz => not done

;----------------------------------------------------------------------------
;	Initialize The Media Tables.
;	BP = 0
;----------------------------------------------------------------------------
	mov	cx, MSMActiveBoardCount		; Number of virtual boards

SetupMediaTablesLoop:
	mov	di, ds:MSMConfigTables[bp]	; Get -> first config table
	mov	bx, [di].MBoardNumber		; BX = Board Number

;----------------------------------------------------------------------------
;	Determine Frame type, and Load Table with board number
;----------------------------------------------------------------------------
	mov	si, [di].MFrameID		; Get the media of this virtual board
	add	si, si				; Convert to word index
	mov	si, MediaToBoardIndexTable[si]	; Convert to a Media index

	mov	MediaIndexToBoardMap[si], bx	; Save in look up table 
						; (Used for receives)

;----------------------------------------------------------------------------
;	Initialize the Header size Table.
;----------------------------------------------------------------------------
	mov	ax, MediaHeaderSizeTable[si]	; Get this headers size
	mov	HeaderSizeTable[bx], al		; Initialize table
	or	BoardFlags[bx], LOADED		; Mark Media (Board) Loaded

;----------------------------------------------------------------------------
;	Set the Loaded Frame Bit
;	SI = Frame type Index
;----------------------------------------------------------------------------
	mov	dx, LoadedFrame[si]		 ; Get Loaded Frame Bit
	or	byte ptr LoadedFrameByte - 1, dl ; Mark Loaded Frame in Code.

;----------------------------------------------------------------------------
;	Now adjust the packet size fields in the config table to reflect 
;	the size of this media header.
;	AX = Header Size
;	DI -> Config Table
;	BX = Board Number
;	SI = Frame type Index
;----------------------------------------------------------------------------
	mov	dx, [di].MMaxPacketSize		; DX = Max Packet size
	sub	dx, ax				; Subtract off Length of Media Header
	mov	[di].MBestDataSize, dx		; Set best case size
	sub	dx, 18+12			; Take into account SR & SRT
	mov	[di].MWorstDataSize, dx		; Set Worst Case size
	cmp	si, TR8022Index			; is this Token-Ring frame type?
	jne	Not8022				; ne => SNAP

	dec	[di].MWorstDataSize		; Type 2 is one byte worse.

;----------------------------------------------------------------------------
;	Setup MSMTableIndex table.
;	Save off the BuildHeaderHandlers
;----------------------------------------------------------------------------
Not8022:
	mov	dx, MediaTxHandlerTable[si]	; Get -> Txhandler routine for this media type

	mov	si, bx				; SI = Board Number
	sub	bx, DriverConfigTable.MBoardNumber ;Convert to Virtual value
	add	bx, bx				; Need a word Virtboard index
	mov	MSMTableIndex[si], bl		; Save MSMTableIndex table value
	mov	BuildHeaderHandlers[bx], dx	; Save TxHandler in board lookup table

;----------------------------------------------------------------------------
;	Initialize ModeFlag bits in Config Table.
;----------------------------------------------------------------------------
	or	[di].MModeFlags, MRawSendBit or MLDataSizeBit ; Set raw send bit and lookahead Data Size support field
	
;----------------------------------------------------------------------------
;	Initialize the MRouteHandler field in the config table, 
;	to point to our stub routine which does nothing.
;----------------------------------------------------------------------------
	mov	word ptr [di].MRouteHandler+0, offset CGroup:SRStubRoutine
	mov	word ptr [di].MRouteHandler+2, cs

;----------------------------------------------------------------------------
;	Move to next Config Table.
;----------------------------------------------------------------------------
	add	bp, 2				; Advance to next config table slot
	loop	SetupMediaTablesLoop		; Get both if loaded.

;----------------------------------------------------------------------------
;	Initialize the Board Number Initialized for each LookAhead struc.
;----------------------------------------------------------------------------
	mov	ax, MediaIndexToBoardMap[ TR8022Index ]
	mov	LookAhead8022.LBoardNum, ax

	mov	ax, MediaIndexToBoardMap[ TRSNAPIndex  ]
	mov	LookAheadSNAP.LBoardNum, ax

;----------------------------------------------------------------------------
;	Set Config Route Handler Entry offset in Lookahead structures
;----------------------------------------------------------------------------
	mov	bx, MSMConfigTables[0]		; Get -> first config table
	add	bx, MRouteHandler
	mov	ax, MSMConfigTables[2]		; Get -> second config table
	add	ax, MRouteHandler

	cmp	[bx-MRouteHandler].MFrameID, 4	; Is the 1st one 802.2?
	je	FirstOneIs8022

	xchg	ax, bx				; Swap who gets what.

FirstOneIs8022:
	mov	word ptr LookAhead8022.RouteEntryPointOff, bx
	mov	word ptr LookAheadSNAP.RouteEntryPointOff, ax

;----------------------------------------------------------------------------
;	Allocate and Initialize Max number of TCB structures.
;----------------------------------------------------------------------------
	mov	cx, MAX_TCB_ALLOCATED		; Allocate the Max
	mov	ax, size TCBStruc + MaxFrameHeaderLen
	mov	di, offset CGroup:TCBFreeList

LinkHeaderBufLoop:
	call	MSMAlloc		; Allocate a TCB
	mov	[di], bx		; Link it
	mov	di, bx
	loop	LinkHeaderBufLoop

	mov	[di], cx		; Terminate the list

;----------------------------------------------------------------------------
;	Call the HSM to Initialize the Card.
;----------------------------------------------------------------------------
	push	cs
	pop	es
	call	DriverInit		;is not required to Preserve anything.
	cld
	push	cs
	push	cs
	pop	ds			; CGroup
	pop	es
	ret

InitializeMedia	endp

;****************************************************************************
;	HSMProvideTCB
;	This procedure is only available at DriverInit Time.  The HSM must
;	Provide all TCBs with in CGroup.   The TCB buffer doesn't have to be
;	initialized, but they must be large enough to hold the TCB and the 
;	MAX Media Header. (TCBStruc + MaxFrameHeaderLen)
;
;	On Entry:
;		DS:SI -> TCB 
;		Interrupts are disabled.
;
;	On Exit:
; 		CX is destroyted.
;		Everything else is preserved.
;
;	All the TCB's are initialized when MSMConfigMediaUpdate is Called.
;****************************************************************************
HSMProvideTCB	proc near
	mov	cx, TCBFreeList			; Free up this TCB
	mov	[si].NextLink, cx
	mov	TCBFreeList, si
	ret
HSMProvideTCB	endp

ifdef TASM
%NEWPAGE ;
endif
;****************************************************************************
;
; NAME:		MediaNodeAddressCheck
;
; DESCRIPTION:	If Node Address Override has taken Place then
;		checks the node address in MSMPhysicalNodeAddress 
;		Also fills in the Node Address in the DriverConfigTable 
;		in the form Specified by ModeFlags bit 15 and 14.
;
; ON ENTRY:	AX	N/A
;		BX	N/A
;		CX	N/A
;		DX	N/A
;		BP	N/A
;		SI	N/A
;		DI	N/A
;		CS	CGroup
;		DS	CGroup
;		ES	CGroup
;	
;		NOTE:	cld in effect, Interrupts are enabled
;
; ON RETURN:	AX	Destroyed
;	    	BX	Destroyed
;		CX	Destroyed
;		DX	Destroyed
;		BP	Preserved (Not Used)
;		SI	Destroyed
;		DI	Destroyed
;		CS	CGroup
;		DS	CGroup
;		ES	CGroup
;
;
;		NOTE: Interrupt state is not changed during function.
;		      Interrupt state is preserved.
;	
;
; REMARKS:
;	The DoNodeKeyWord in the Generic MSM.asm sets the LSB/MSB mode bits
;	if nessasary.   
;	Bit 15 signals the support of the particular HSM (Set by the HSM)
;			Bit 15	1 = LSB/MSB supported. 
;				0 = Not Supported (old HSM)
;	Bit 14 describes the form of Node Address in the DriverconfigTable (Set by the MSM)
;			Bit 14 	1 = MSB (Non-Canonical Form)
;				0 = LSB (Canonical Form)
;
; SEE ALSO:
;
;****************************************************************************
;****************************************************************************
MediaNodeAddressCheck	proc	near
	mov	ax, word ptr MSMPhysNodeAddress
	and	ax, word ptr MSMPhysNodeAddress+2
	and	ax, word ptr MSMPhysNodeAddress+4
	inc	ax
	jz	DoNodeExit				; z=>no node override

;----------------------------------------------------------------------------
;	We are Token-Ring make sure it is a Valid address.
;----------------------------------------------------------------------------
CheckTokenRingAddrMode:
	cmp	byte ptr MSMPhysNodeAddress+6, 'L'  ; If In LSB form need to Swap it.
	jne	PhysNodeInMSBForm

;----------------------------------------------------------------------------
; MSMPhysNodeAddress is in LSB form.   For Token Ring it must be Swapped.
;----------------------------------------------------------------------------
PhysNodeInLSBForm:
	mov	si, Offset CGroup:MSMPhysNodeAddress
	mov	di, si					; Do Swap in Place.
	mov	cx, 3
	mov	bx, PSPTableStart			; BitSwapTable @

CheckSwapBitAddr:
	lodsw				; Get a word
	xlat				; Swap it
	xchg	ah, al			; AH gets result, AL is Next look up
	xlat				; Swap it
	xchg	ah, al			; straighten back for save
	stosw				; Store it.
	dec	cx
	jnz	CheckSwapBitAddr	; get all 3 words of Address.

	mov	al, 'M'			; Flag it correctly.
	stosb

;----------------------------------------------------------------------------
;    MSMPhysNodeAddress is in MSB form.   For Token Ring No Action is needed
;----------------------------------------------------------------------------
PhysNodeInMSBForm:
	mov	ax, DriverConfigTable.MFrameID	; Get FrameID
	cmp	ax, MediaID8022
	je	CheckTokenRingAddr

	cmp	ax, MediaIDSNAP
	jne	DoNodeExit	    ; Not a TokenRing Address don't check on it.

;----------------------------------------------------------------------------
;    Check the Address that the Local AdminBit is set and GroupBit not set.
;----------------------------------------------------------------------------
CheckTokenRingAddr:
	test	byte ptr MSMPhysNodeAddress+0, 40h	; Is LA bit set?
	jz	AddressHasBitError

	test	byte ptr MSMPhysNodeAddress+0, 80h	; Is Group bit set?
	jz	DoNodeExit

;----------------------------------------------------------------------------
;	Some one left the Group Bit Set, clear it, and Inform the user.
;	or
;	 Some one left the LA Bit clear, set it, and Inform the user.
;----------------------------------------------------------------------------
AddressHasBitError:
	or	byte ptr MSMPhysNodeAddress+0, 40h	; Set Local Admin Bit.
	and	byte ptr MSMPhysNodeAddress+0, 7Fh	; Clear Group Bit
	mov	dx, offset CGroup: InvalidMediaAddress	; get error message
	call	MSMPrintStringZero
	mov	dx, offset CGroup: TokenMsg		; get Media Name
	call	MSMPrintStringZero
	mov	dx, offset CGroup: InvalidMediaAddress2
	call	MSMPrintStringZero

;----------------------------------------------------------------------------
;	Finished Checking the Address.
;----------------------------------------------------------------------------
DoNodeExit:
	mov	si, offset CGroup:MSMPhysNodeAddress 	     ; DS:SI -> Source
	lea	di, DriverConfigTable.MNodeAddress	     ; ES:DI -> Target
	test	DriverConfigTable.MModeFlags, MNonCanonicalBit ; Check if Swap needed.
	jnz 	NodeMSBMode				     ; nz => no just move it for token ring.

	mov	cx, 6				; Byte Count
	call	MSMBitSwapAddress		; Swap it into The Config Table.
	ret

NodeMSBMode:
	mov	cx, 3					;3 words in address.
 rep	movsw
	ret

MediaNodeAddressCheck	endp
 

Init	ends

	end

