%PAGESIZE 66,130
;*****************************************************************************
; NE32Firm.asm
;	(C) Unpublished Work of Novell, Inc.  All Rights Reserved.
;
;	THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL, 
;	PROPRIETARY AND TRADESECRET INFORMATION OF NOVELL, INC.  ACCESS 
;	TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
;	NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS 
;	AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO APPROPRIATE 
;	LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED, PRACTICED,
;	PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,
;	ABBRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
;	TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL. 
;	ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD 
;	SUBJECT THE PERPERTRATOR TO CRIMINAL AND CIVIL LIABILITY.
;
;----------------------------------------------------------------------------
;*****************************************************************************
;	Title:		Novell NE3200 FirmWare for Client ODI
;
;	Filename:	NE32Firm.asm
;
;	Written by:	Jon Rosdahl
;
;	Date:		05/01/91
;
;	ODI Specification Version:	4.00
;
; Modification History:
;	By	Date	 Ver.	Modification Description
;	---	-------- -----	--------------------------------------
;	JWR	11-02-91 1.01   Added Copyright Notice (MEMO May 23, 1991)
;				Corrected Version Information at end of this file.
;
;	RTY	01-21-91 1.10	Fixed bug; WaitForTranferToHost	was equated
;				to 01h changed to 0Ch.
;
;				Added Code for Promiscuous Mode. The Update Parm
;				Mailbox has two new defined bits bits 4 and 5.
;				if bit 4 is set promiscuous mode will be set.
;				if bit 5 is set promiscuous mode will be disabled.
;
; 	RTY	19 May 1992 
;	Added code to allow for long packets as well as runt packets to be received.
;	changed Receive buffer size to 3040 to allow for long packets.  Only 1514 bytes
;	will be passed to the host machine but the size passed on will be correct.
;
; 	RTY 	07 October 1992  v1.11
;	There was a problem with the transmition of fragmented data.  If the
;	data length was odd a byte would get omitted from the data.  This happened
;	because the source was not getting backed up to match the destination.
;					
;	JWR	09 January 1995  v1.12 Update the Copyright string.
;
;	JWR	01 April 1996  v1.13 Update the Copyright string.
;
;	SMN 21 Nov. 97 1.14		Re-Link Latest ToolKit. v2.12, Update Copyright date
;
; Comments:
;	This FirmWare works with the DOS ODI and the OS/2 Drivers.  The major 
;	difference is where the next packet is put.  In DOS, there is a single
;	buffer that is waiting for Reception.  In OS/2, the driver gets preallocated
;	contiguous buffers for reception.
;
;*****************************************************************************

;*****************************************************************************
;	Include Files containing the Structure Definitions and Major Equates
;*****************************************************************************
.186
include BMICstru.inc			; BMIC Structures (82355)
include 82586def.inc			; 82586 Structures

;*****************************************************************************
;      			MACROS for the BMIC
;*****************************************************************************

;------------------------------------------------------------------------------
;TransferDataToHost	macro
;
; On Entry:	ESI -> 7 byte Host Count/Address(Destination).
;		DI -> local address(Source).
;------------------------------------------------------------------------------
TransferDataToHost	macro
	local	WaitForTransferToHost

	mov	dx, BMIC_Index			; Set index to Transfer
	mov	al, CH0_TBI_BASE_ADDR OR AUTO_INC ; Buffer Interface channel 0.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	ax, di				; AX -> Source.
	shr	ax, 1				; AX = Source Address/2.

	out	dx, al				; Set Source LSB.
	mov	al, ah				; AL = Source MSB.
	out	dx, al				; Set Source MSB.

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_BASE_COUNT OR AUTO_INC	; Base count.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	cx, 7				; Copy 7 bytes.
 rep	outsb

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STROBE			; Strobe register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	out	dx, al				; No data required for strobe.

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STATUS			; Status.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.

WaitForTransferToHost:
	in	al, dx				; AL = current status.
	test	al, TRANSFER_COMPLETE		; Transfer complete?
	jnz	WaitForTransferToHost		; Jump if not.

	and	al, 00011111b			;# Mask out reserved Bits.
	out	dx, al				; Reset Transfer Complete bit.
	
			endm

;------------------------------------------------------------------------------
;TransferDataFromHost	macro
; Uses 	SI, DI, DX, AX, CX
; Destroys AX, DX, CX, SI
; Preserves DI
; On Entry:	SI -> 7 byte Host Count/Address(Source).
;		DI -> local address(Destination).
;------------------------------------------------------------------------------
TransferDataFromHost	macro
	local	WaitForTransferFromHost

	mov	dx, BMIC_Index			; Set index to Transfer
	mov	al, CH0_TBI_BASE_ADDR OR AUTO_INC ; Buffer Interface channel 0.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	ax, di				; AX -> Dest.
	shr	ax, 1				; AX = Dest Address/2.

	out	dx, al				; Set Dest LSB.
	mov	al, ah				; AL = Dest MSB.
	out	dx, al				; Set Dest MSB.

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_BASE_COUNT OR AUTO_INC	; Base count.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	cx, 7
 rep	outsb

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STROBE			; Strobe register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	out	dx, al				; No data required for strobe.

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STATUS			; Status.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.

WaitForTransferFromHost:
	in	al, dx				; AL = current status.
	test	al, TRANSFER_COMPLETE		; Transfer complete?
	jnz	WaitForTransferFromHost		; Jump if not.

	and	al, 00011111b			;# Mask out reserved bits.
	out	dx, al				; Reset Transfer Complete bit.
	
			endm

;------------------------------------------------------------------------------
;ResetThe82586		macro
;	Sets the 82586 for 3 wait states and then back to 0 wait states.
;	Destroys: AX, DX registers.
;------------------------------------------------------------------------------
ResetThe82586		macro
	mov	dx, 0FFA4h			; Configure to 3 wait states.
	mov	ax, 003Fh
	out	dx, ax
	mov	dx, RESET_586_PORT		; Reset 82586.
	in	al, dx
	mov	dx, 0FFA4h			; Configure to 0 wait states.
	mov	ax, 0038h
	out	dx, ax
 			endm

;------------------------------------------------------------------------------
;StatisticsUpdate	macro
;	Increments a counter and sets an update flag.
;	No Registers are destroyed.
;------------------------------------------------------------------------------
StatisticsUpdate	macro	CounterOffset
	push	di
	lea	di, CounterOffset	; point to the Counter.
	add	word ptr [di], 1
	adc	word ptr [di + 2], 0
	inc	CounterUpdateFlag	; Signal that we have changed a counter.
	pop	di	
			endm


;****************************************************************************
;	Firmware Equate Center
;****************************************************************************

;------------------------------------------------------------------------------
;	Fragment Pointers -- Each is comprised of the following 2 fields.
;------------------------------------------------------------------------------	

FragStruc	struc
	FragAddr	dd	?
	FragLen		dw	?
FragStruc	ends
		; 6 bytes per Frag Pointer

;------------------------------------------------------------------------------
;	Buffer Area (Upper 32K) Definition and Location  Equates
;------------------------------------------------------------------------------

MEM_586_Start		equ	08000h		; Start of the Upper 32K
MEM_586_End		equ	0FFF6h		; End of Useable 82586 RAM
MEM_Size		equ	0FFF6h - 07FFFh	; Size of 82586 RAM

PROM_OFFSET		equ	04000h		; PROM Node Address

SCP		equ	0FFF6h				; SCP Offset
ISCP		equ	MEM_586_Start			; ISCP is first in RAM
SCB		equ	ISCP+ size ISCPstruc		; System Control Block for 82586
CmdBlk		equ	SCB + size SCBstruc		; Generic Command Blk -- Don't use Multicast Cmd Here
MCBlk		equ	CmdBlk + size CmdBlkstruc	; Multicast Command Block
TxBlk		equ	MCBlk + size MCBstruc		; Start of Transmit Command And 82586 Buffer Area (Only 1)
RxBlk		equ	TxBlk + size TxBlockStruc 	; Start of Recieve Command and Buffer Area

;------------------------------------------------------------------------------
;	80186 I/O port Definitions
;------------------------------------------------------------------------------

CA_PORT			equ	180h		; 82586 Channel Attention.
RESET_586_PORT		equ	100h		; 82586 Reset port.

;------------------------------------------------------------------------------
;		Error Code Center
;------------------------------------------------------------------------------
; Abend codes.
;
TRANSFER_HANG_ERROR	equ	01h
ODD_ADDRESS_ERROR	equ	02h
INVALID_PACKET_LENGTH	equ	03h
CHANNEL_ONE_HANG	equ	04h
BAD_MEMORY_ERROR	equ	05h
ODD_ADDRESS_DEBUG	equ	06h
TOO_MANY_FRAGMENTS	equ	07h
BAD_TRANSFER_ERROR	equ	08h
BAD_ROM_CHECKSUM	equ	09h
INIT_FAILURE586		equ	0Ah
ABEND_CMD_GIVEN		equ	0Bh

Host_ReQuested		equ    0FFh


;*****************************************************************************
;	Inport/Export Variables.
;____________________________________________________________________________
public	FirmWareStart, FirmWareSize, FirmWareStatsSize
extrn	MAX_TX_RETRIES:ABS		; Number of Retries to the Card.
extrn	DoorBellBits:ABS		; Number of Enabled Interrupt bits

	assume CS:CODE, DS:CODE, ES:CODE
CODE	segment public  para 'ZCODE'

FirmWareStart:				; Start of the Firmware Space
	jmp	FirmStart		; Jump to Firmware Start

	org	100h	       		; Skip Interrupt Table
					; Adapter Parameter Block Starts at 100h.

;****************************************************************************
;	Adapter Parameter Block		; Start at 100h
;****************************************************************************
;	Don't Adjust or move any of the following Block entries:

HostsLocalParamTable	label	dword
NodeAddressPointer	dd	0	; Physical Pointer to DriverConfigTable.NodeAddress
BMFrameSizePtr		dd	0	; Physical Pointer to the Lookahead area
FirmWareStatPtr		dd	0	; Physical Pointer to the Generic Counters
CustomStatCounter	dw	0	; Number of Custom counters
MulticastCount		dw	0	; Number of Multicast Entries Possible in the Table.
MulticastListPtr	dd      0	; Physical Pointer to MulticastTable
AdapterParamBlkSize	equ	$-HostsLocalParamTable ; Size of The Table


;****************************************************************************
;	Counters for the Firmware ... 
;	When updating, the BMIC will copy them over the Host's counters.
;****************************************************************************

	even
FirmWareStats	label	byte
;;;;;	DriverStatTotalTx	dw	2 dup (0)	; 
;;;;;	DriverStatTotalRx	dw	2 dup (0)	; Let the Host update    (MSM)
;;;;;	DriverStatNoECB		dw	2 dup (0)	; these counters.
;;;;;	DriverStatTxTooBig	dw	2 dup (0)	;Not used.
;;;;;	DriverStatTxTooSmall	dw	2 dup (0)	;Not used.
	DriverStatRxOverflow	dw	2 dup (0)	; Rx Error First Counter Used by the Firmware
	DriverStatRxTooBig	dw	2 dup (0)	; MSM Checks for this, but we will do it anyway.
	DriverStatRxTooSmall	dw	2 dup (0)	; Rx Error.
	DriverStatTxMisc	dw	2 dup (0)	; Not Used
	DriverStatRxMisc	dw	2 dup (0)	; If I ever see a Error Bit set on the RFD.
	DriverStatTxRetryCount	dw	2 dup (0)	; Number of Retries
	DriverStatRxCheckSum	dw	2 dup (0)	; CRC errors ect.
	DriverStatRxMismatch	dw	2 dup (0)	; Not Used.

	CustomCounterCount	dw	0		; Filled in at Init time.

	NumberOf82586Resets	dw	2 dup (0)	; number of times 586 reset	
	TxRetryFailure		dw	2 dup (0)	; number of retrys failures
	TxCarrierSenseLost	dw	2 dup (0)	; tx carrier lost
	TxClearToSend		dw	2 dup (0)	; tx error
	TxUnderRun		dw	2 dup (0)	; tx error
	TxDefers		dw	2 dup (0)	; tx error
	TxMaxCollisions		dw	2 dup (0)	; tx error Number of times 16 collions occured on one packet.
	TxNumCollisions		dw	2 dup (0) 	; tx error Actual number of collisions
	RxDMAOverrun		dw	2 dup (0)	; RX errors
	RxNoEOF			dw	2 dup (0)	; Rx errors

FirmWareStatsSize	equ	$-FirmWareStats		; Size of the FirmWare Stat Table

;****************************************************************************
;****************************************************************************

;------------------------------------------------------------------------------
;	Working Variables
;------------------------------------------------------------------------------

	even
NodeOverRide		db	0		; Do we have an OverRide? 0FFh -> Node OverRide
notused			db	0		; Keep Word boundaries
BoardID			db	6 dup (0)	; Desired Immediate Node Address 

SixteenBytesOfZero	db	4  dup (00h)	; Need Zeroes
TwelveZeroPlusSignature db	12 dup (00h)
			dd 	FirmWareRunOK_FLAG ; Running Flag
       even
InitialSCP		db	0		; 0 => 16 bit bus
			db	5 dup (0)	; Reserved Fields
			dw	ISCP		; 24-bit Value to ISCP Physical Address
			dw	0		; last byte is reserved 
	even
InitialISCP		dw	1		; 01h => Busy: 82586 is in the initialization process
			dw	SCB		; Address of the SCB
			dw	2 dup (0)	; SCB-BASE: Base Segment Value of 64-K segment. 

;------------------------------------------------------------------------------
;	Multicast Table Variables
;------------------------------------------------------------------------------
	even
HostMulticastTable	 db	(MAX_MULTICAST_ADDR * 8) dup (0) ; where the host's table is stored

;------------------------------------------------------------------------------
;	Watch Dog Variables
;------------------------------------------------------------------------------
	even
DeadMan			dw	0
DeadManHigh		dw	0		; DeadMan Timers for TimeOuts
CounterUpdateFlag	db	0		; Flag Changes to Counters
ActivityFlag		db	0		; State Machine Flags.

;------------------------------------------------------------------------------
;	Activity Flag Definitions
;------------------------------------------------------------------------------

;Adapter Activity Flags
ActFlgTxWaiting		equ	80h		; Tx has been Sent to 586
ActFlgRxWaiting		equ	40h		; Have Asked the Host for an RCB
ActFlg20		equ	20h		; 
ActFlg10		equ	10h		; 


; 586 Activity Flags
ActFlgRestart		equ	08h		; Have Been Restarting The 586.
ActFlg40		equ	04h		; 
ActFlgFR		equ	02h		; Have Been Processing a Framed Received
ActFlgCX		equ	01h		; Have been Processing a Complete Transmit
ActFlag586		equ	0Fh		; Some Activity has been Registered.
	
No586ActivityFlg	equ	0F0h		; Clears 586 Activities

;------------------------------------------------------------------------------
;	Receive Link List Pointers
;------------------------------------------------------------------------------
	even
RxHead		dw	RxBlk		; Start of Receives
RxTail		dw	RxBlk		; End of List

;****************************************************************************
;****************************************************************************

;------------------------------------------------------------------------------
;		82586  Default Configuration Table
;------------------------------------------------------------------------------

	even
CFGCmdBlk	dw	0			  ; STATUS
		dw	(EL_bit or CMD_CONFIGURE) ; COMMAND
		dw	-1			  ; Link Offset Field == No more commands
CFGParmsTable	db	CFG_ByteCnt		; byte6
		db	CFG_FIFOLIM		; byte7
		db	CFG_SRDYbit		; byte8 (External Synch, Bad Frames Not Saved)
		db	CFG_Preamble_Len or CFG_AddrLoc or CFG_AddrLen	; byte9  (Not LoopBack)	
		db	CFG_LinPrio or CFG_ACR or CFG_BackOffMethod	; byte10 	
		db	CFG_InterFrameSpace	; byte11  	
		db	(CFG_SlotTime Mod 100h)	; byte12  ( low bits are 0)
		db	CFG_Retry_Num or CFG_SlotTime / 100h		; byte13  	
		db	0			; byte14  	
		db	0			; byte15  	
		db	CFG_MinFrame		; byte16
		db	0			; byte17 even out to Words

CFGCmdBlkSize	equ	$-CFGCmdBlk		; Size of Configureation Command

;****************************************************************************
;****************************************************************************

;****************************************************************************
;	Command Sequences for the BMIC	   DO NOT CHANGE ORDER
;****************************************************************************

;----------------------------------------------------------------------------
;	To Host Transfer Sequence Buffer.  DO NOT CHANGE ORDER	      (Generic Buffer)
;----------------------------------------------------------------------------
	even
ToHostTransferSequence	label	byte
ToHostTransferCount	dw	?	; Number of bytes to transfer
			db	AUTO_START or ADAPTER_TO_EISA
ToHostTransferLoWord	dw	?	; Filled in as needed.
ToHostTransferHiWord	dw	?

;----------------------------------------------------------------------------
;	From Host Transfer Command Sequence Buffer.  DO NOT CHANGE ORDER  (Generic Buffer)
;----------------------------------------------------------------------------
	even
FromHostTransferSequence	label	byte
FromHostTransferCount		dw	?	   ; Number of bytes to transfer
	      			db	AUTO_START ; From Host (EISA) to Adapter
FromHostTransferLoWord		dw	?	   ; Fill in As Needed.
FromHostTransferHiWord		dw	?

;----------------------------------------------------------------------------
;	From Host Transfer Parameter Block Sequence.  DO NOT CHANGE ORDER
;----------------------------------------------------------------------------
	even
FromHostTransBlockSequence	label	byte
FromHostTransBlockCount		dw	AdapterParamBlkSize ; number of bytes to transfer
	      			db	AUTO_START	    ; From Host (EISA) to Adapter
FromHostTransBlockLoWord	dw	?		    ; Filled in During FirmWareStart
FromHostTransBlockHiWord	dw	?		    ; Filled in During FirmWareStart


;----------------------------------------------------------------------------
;	Read BMTCB from Host Sequence	DO NOT CHANGE ORDER
;----------------------------------------------------------------------------
	even
OddByteStorage			dw	0		; Keep Transfers on even boundaries.
NumFrags			dw	0		; Number of Fragments to be passed down.

	even
ReadBMTCBFromHostSequence	label	byte
ReadBMTCBTransferCount		dw	?		; Number of bytes to transfer
	      			db	AUTO_START	; From Host (EISA) to Adapter
ReadBMTCBTransLoWord		dw	?		; Filled in During Parameter Update
ReadBMTCBTransHiWord		dw	?		; Filled in During Parameter Update

;----------------------------------------------------------------------------
; Transfer Statistic Counters buffer to HOST Sequence	DO NOT CHANGE ORDER
;----------------------------------------------------------------------------
	even
TransToHostStatCountSequence	label	byte
TransToHostStatCountLocalAddr	dw	?	 ; Local Word Address.
TransToHostStatCountCNT		dw	FirmWareStatsSize
				db	AUTO_START or ADAPTER_TO_EISA
TransToHostStatCountLoWord	dw	?	 ; Filled in During FirmStart Proc.
TransToHostStatCountHiWord	dw	?	 ; Filled in During FirmStart Proc.

;----------------------------------------------------------------------------
;	Transfer LookAhead buffer to HOST Sequence	DO NOT CHANGE ORDER
;----------------------------------------------------------------------------
	even
TransToHostLookAheadSequence	label	byte
TransToHostLookAheadCNT		dw	1514 + 2 ; Enough for Full Packet + FrameLength variable.
				db	AUTO_START or ADAPTER_TO_EISA
TransToHostLookAheadLoWord	dw	?	 ; Filled in during Parameter UpDate
TransToHostLookAheadHiWord	dw	?	 ; FIlled in During Parameter Update

;----------------------------------------------------------------------------

RxStatus	db	0
RxMonStatus	db	0

;****************************************************************************
;	Firmware Start
;****************************************************************************
;	S T A R T   O F   F I R M W A R E   C O D E  
;****************************************************************************
;****************************************************************************
; This routine sets up the 80186 segment registers, stack, tests memory, reads
; host parameters, and sets the node address fields.  It also sets up the 82586
; and BMIC chips. 
;****************************************************************************
	even
FirmStart	proc	near
	cli				; Disable system interrupts
	cld				; set the direction flag

	xor	ax, ax			; Need a zero
	mov	ds, ax			; Ensure all Segment registers set to 0
	mov	es, ax
	mov	ss, ax
	mov	sp, 4000h		; Top of Stack End of 16K segment
	out	80h, al			; Disable the Adapter's PROM 

;----------------------------------------------------------------------------
;	Test Adapter Memory
;----------------------------------------------------------------------------

	mov	ax, 5A5Ah		; Test Pattern
	mov	cx, MEM_Size/2		; Size of 82586 Memory Space
	mov	di, MEM_586_Start	; Start of 82586 Memory
   rep	stosw				; Fill in Memory

	mov	cx, MEM_Size/2		; Size of 82586 Memory Space
	mov	di, MEM_586_Start	; Start of 82586 Memory
   repe	scasw   	
	je	MemoryChecksOUT		; ne => memory error

	mov	bx, BAD_MEMORY_ERROR	; Signal Problem.
	jmp	ABEND			; the Adapter is going to Abend

;----------------------------------------------------------------------------
;Zero out all the MailBox Registers to Tell the Driver I'm Alive, and want the Parameter Block
;----------------------------------------------------------------------------

MemoryChecksOUT:
	mov	dx, BMIC_Index			; Get Index Register port
	mov	al, MAIL_0 OR AUTO_INC		; Point Window to first Mail Box
	out	dx, al

	mov	dx, BMIC_Data			; Put Data in Window
	mov	si, OFFSET TwelveZeroPlusSignature
	mov	cx, 16
   rep  outsb					; Send out 16 bytes.

;----------------------------------------------------------------------------
;	Set up BP Register for the SCB Pointer.
;----------------------------------------------------------------------------	
	
	mov	bp, SCB		; BP -> SCB -- At all Times.

;----------------------------------------------------------------------------
;Wait for The Adapter to pass the physical Address of the Adapter Parameter Block.
;----------------------------------------------------------------------------

	mov	dx, BMIC_Index			; get Index Register port
	mov	al, PARM_UPDATE_MAILBOX		; watch update MailBox
	out	dx, al				; Move Window

	mov	dx, BMIC_Data			; Watch Data Window

WaitForParameterBlock:
	in	al, dx				; AL = Letter from MailBox
	or	al, al				; Still Nothing?
	jz	WaitForParameterBlock		; Wait til it arrives

	mov	NodeOverRide, al		; Save Value (00Fh => Node Address Override) (0FFh => use BoardID)
	mov	dx, BMIC_Index			; Get Index port
	mov	al, (PARAMETER_MAILBOX OR AUTO_INC)
	out	dx, al				; Move Window

	mov	dx, BMIC_Data			; Look in Data Window
	mov	di, OFFSET FromHostTransBlockLoWord ; DI -> Dest
	mov	cx, 4
   rep  insb					; Store Address

;----------------------------------------------------------------------------
;	Read in the Parameter Block
;----------------------------------------------------------------------------

	call	UpdateParameters		; Read it

;----------------------------------------------------------------------------
; 	Read ethernet node address from PROM and save it.
;----------------------------------------------------------------------------

	cmp	NodeOverRide, 00Fh		; 00Fh => Over Ride given
	je	GetHostNodeAddress		; Don't Read Node Address, go get it from host.

	mov	si, PROM_OFFSET			; SI -> 0:4000h
	mov	di, OFFSET BoardID		; DI -> our data section.
	mov	cx, 6				; Read 6 bytes.

CopyBoardIDLoop:
	movsb					; Copy a byte.
	inc	si				; Skip a byte.
	loop	CopyBoardIDLoop			; Copy next byte.

;----------------------------------------------------------------------------
;	Send Node Address to Host 
;----------------------------------------------------------------------------

	mov	ax, word ptr NodeAddressPointer + 0	; AX = Dest Lo
	mov	ToHostTransferLoWord, ax		; Store Dest Lo word
	mov	ax, word ptr NodeAddressPointer + 2	; AX = Dest Hi
	mov	ToHostTransferHiWord, ax		; Store Dest Hi word
	mov	ToHostTransferCount, 6			; Store Count
	mov	di, Offset BoardID			; DI -> Source
	mov	si, OFFSET ToHostTransferSequence 	; SI -> Dest.

	TransferDataToHost				; Macro

	jmp  short ClearMailBoxes			; Carry on

;----------------------------------------------------------------------------
;	Get Node Address from Host
;----------------------------------------------------------------------------

GetHostNodeAddress:
	mov	ax, word ptr NodeAddressPointer + 0	; Lo word
	mov	FromHostTransferLoWord, ax	
	mov	ax, word ptr NodeAddressPointer + 2	; Hi Word
	mov	FromHostTransferHiWord, ax
	mov	FromHostTransferCount, 6		; 6 bytes Min Transfer Size.
	lea	si, FromHostTransferSequence		; SI -> Source
	lea	di, BoardID				; DI -> Dest.

	TransferDataToHost				; Macro
	
;----------------------------------------------------------------------------
; Clear all MailBox registers informing host that it can	
; continue with his initialization.				
;----------------------------------------------------------------------------

ClearMailBoxes:
	mov	dx, BMIC_Index			; Set index to first MailBox
	mov	al, (MAIL_0 OR AUTO_INC)	; register.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	si, OFFSET SixteenBytesOfZero	; SI -> Sixteen zeros.
	mov	cx, 16				; Write 16 bytes.
   rep	outsb

;----------------------------------------------------------------------------
; Fixup statistic sequence buffer addresses.			
;----------------------------------------------------------------------------

	mov	ax, CustomStatCounter		; AX = Count field.
	mov	CustomCounterCount, ax
	mov	ax, offset FirmWareStats	; AX -> counters.
	shr	ax, 1				; Adjust for BMIC. (Addresses Words only)
	mov	TransToHostStatCountLocalAddr, ax	
	mov	ax, word ptr FirmWareStatPtr + 0   ; Place in the Sequence Blocks for Fast Transfers
	mov	TransToHostStatCountLoWord, ax
	mov	ax, word ptr FirmWareStatPtr + 2
	mov	TransToHostStatCountHiWord, ax

;
;***************************************************************\
;                                                               *
; Allow 586 to initialize itself, tell host and jump to main loop.*
;                                                               *
;***************************************************************/
;

	call	Initialize586
	jz	OKGoAhead
	jmp	ABEND				; Didn't Work Then Die!

OKGoAhead:

;----------------------------------------------------------------------------
;	Tell The Host that I am Going In the MainEvent and am Able Bodied
;----------------------------------------------------------------------------
	mov	dx, BMIC_Index			; Set index to MailBox 1
	mov	al, ABEND_IDLE_MAILBOX		; Register
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	al, FIRMWARE_STATE_FLAG		; We are Ready
	out	dx, al

;CAUTION Fall into MAINEvent for now.
;	jmp short MainEvent			; Finally let's get going

FirmStart	endp

;****************************************************************************
;	The MAIN EVENT
;****************************************************************************
;  After the initialization, the 80186 remains in this procedure till 	*
;  an Abend is issued.  at which time the 80186 is put to sleep.	*
;                                                                       *
; 	This is the main polling loop which the processor runs		*
; forever. It will poll the 586 for receives, check the host for the	*
; idle adapter command, check the host for update parameters command	*
; and check the host for another transmit in that order.		*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			(Never returns)			*
;	BX = Not Used							*
;	CX = Not Used							*
;	DX = Not Used							*
; 	BP = SCB							*
;	SI = Not Used							*
;	DI = Not Used							*
;                                                                       *
;***********************************************************************/

MainEvent	proc	near

;----------------------------------------------------------------------------
;		Check For Counter Update Required.
;----------------------------------------------------------------------------
CheckCounters:
	cmp	CounterUpdateFlag, 0		; Counters need updating?
	je	Check586			; Jump if not.

	mov	CounterUpdateFlag, 0		; Clear flag.
	call	UpdateHostStatistics		; Update the host.

;----------------------------------------------------------------------------
;		Check the 82586 for any Events
;----------------------------------------------------------------------------
Check586:
	call	Poll586				; Poll the 82586.
		
;----------------------------------------------------------------------------
;	Check the Host for any MAIL 
;----------------------------------------------------------------------------
	mov	dx, BMIC_Index			  ; Set index to the adapter
	mov	ax, VALID_RCB_MAILBOX OR AUTO_INC ; RCB Ready MailBox. (Clears AH too)
	out	dx, al				  ; It is Byte Port only.

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	in	al, dx				; Read RCB MailBox.
	or	al, al				; Was an RCB issued?
	jnz	MainProcRCBCmd			; Jump if so.

	in	al, dx				; Read Abend/Idle MailBox.
	cmp	al, 'J'				; Something other Than 'J' says a command has come.
	jne	MainAbend_IdleCommand		; Jump if so.

  	in	al, dx				; Read Update Parm MailBox.
	or	al, al				; Was a command issued?
	jnz	UpdateParmCommand		; Jump if so.

	in	al, dx				; Read Valid TCB MailBox.
	or	al, al				; Was a TCB issued?
	jz	CheckCounters			; Jump if not.

;***************************************************************\
;                                                               *
; Valid TCB. Read address of TCB from TCBMAILBOX.		*
;                                                               *
;***************************************************************/

ValidTCB:
	mov	bx, es		; Clear Backup Bytes.
	and	ax, 00FFh	; Clear ah
	dec	al		; Don't count Collection Buffer.
	mov	NumFrags, ax	; Save Number of Frags sent.
	jz	GetNumBytes 	; If only Collection buffer then forget it.

		; Fast Multiply. (Number of Frags * 6)
	shl	ax, 1		; AX * 2
	mov	bx, ax		; Get copy of Number of Extra Frags.
	shl	bx, 1		; BX * 2 = AX * 4
	add	bx, ax		; BX = Number of bytes to back up for frag Pointers. (AX*6)

GetNumBytes:
	in	al, dx		; Read MailBox 4 for TransferByte count hi.
	mov	ah, al
	in	al, dx		; Read MailBox 5 for TransferByte Count Lo.
	jmp	ReadTCBFromHost	; Transfer the TCB

;---------------------------------------------------------------\
; Idle adapter command. Clear MailBox and dispatch.		*
;---------------------------------------------------------------/
MainAbend_IdleCommand:
	cmp	al, Abend_AdapterCmd	; Was an ABEND command Issued?
	jne	IdleAdapter		; Even if it is an unkown command idle the Adapter.

	mov	bl, ABEND_CMD_GIVEN	; Error Code.
	jmp short ABEND			; Abend the adapter.

;---------------------------------------------------------------\
; Lookahead is Available. Mark mailbox and return to MainEvent	*
; 	Clear The Valid RCB MailBox for future use.		*
;---------------------------------------------------------------/
MainProcRCBCmd:
	cmp	al, RCBCOMPLETEOS2		; is the Host in 0S2?
	je	RCBCmdOS2			; if OS2 then RCB @ moves.
	
RCBComplete:
	and	ActivityFlag, NOT ActFlgRxWaiting ; Signal RCB completed
	mov	dx, BMIC_Index			  ; Set index to the adapter
	mov	al, VALID_RCB_MAILBOX		  ; RCB Ready MailBox.
	out	dx, al

	mov	dx, BMIC_Data			; Get Data port window
	mov	ax, es				; Clear AX
	out	dx, al				; Clear Mail Box

	call	ProcessFRInterrupt		; Check to see if we have a new packet.

	jmp short MainEvent

;----------------------------------------------------------------------------
;	In OS2 the RCB is not at a single address.
;	The Address must be updated after each RCBcomplete is given.
;----------------------------------------------------------------------------
RCBCmdOS2:
	mov	dx, BMIC_Index			; Set index to the adapter
	mov	al, MAIL_8 or AUTO_INC		; Get OS2 RCB Address.
	out	dx, al				

	mov	dx, BMIC_Data			; Set window
	mov	cx, 4
	mov	di, OFFSET TransToHostLookAheadLoWord ;BMFrameSizePtr 
   rep	insb					; Read in Address.
	jmp short RCBComplete

;---------------------------------------------------------------\
; Update Parameters command. Clear mailbox and dispatch.	*
;---------------------------------------------------------------/

UpdateParmCommand:
	test	al, Set_Promiscuous + Clear_Promiscuous
	jz	CheckForReset			;Check for Promiscuous change
	call	PromiscuousChange
	jmp	short SetItProm

CheckForReset:
	cmp	al, Reset_AdapterCMD		; Was a Reset Command Given?
	jne	UpDateOnly			; ne => No Must be a MC or UpDate Parameter

SetItProm:
	call	ReStart586			; Reset the 586
	jmp short DoneUpDateParm		; Don't Forget to Clear the MailBoxes.

UpDateOnly:
	call	UpdateParameters		; Update parameters.

DoneUpDateParm:
	mov	dx, BMIC_Index			; Set index to the update
	mov	al, PARM_UPDATE_MAILBOX		; parameter mailbox.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	ax, es				; Clear it.
	out	dx, al
	jmp	MainEvent			; Back to the MAIN EVENT!!

MainEvent	endp


;****************************************************************************
;	Abend  --- Where All Problems come to DIE!!
;****************************************************************************
;	Assume:				     Returns: NEVER, EVER, EVER!!!!
;		BL = Error Number

ABEND	proc	near
	mov	ax, SCBRUC_ABORT+SCBCUC_ABORT
	push	bx			; Save Error Number
	mov	bx, CmdBlk		; Dumb Pointer
	call	Command586NoRestart	; Give 586 what for.
	pop	bx			; Restore Error Number.

	mov	dx, BMIC_Index
	mov	al, (AUTO_INC OR ABEND_IDLE_MAILBOX)
	out	dx, al

	mov	dx, BMIC_Data		; DX = BMIC Data Port.
	mov	al, 'A'			; Abend Flag
	out	dx, al			; Mark Abend
	mov	al, bl
	out	dx, al			; Mark Error CODE
DEAD:
	hlt				; DIE
	jmp short DEAD			; If somhow we wake up die again.

ABEND	endp


;***********************************************************************\
;                                                                       *
; IdleAdapter - Interrupt should already be disabled. Stay in		*
; continuous loop until host Calls for reset.				*
; Jumps to MainEvent After reseting the 586 etc.			*
; IF ABEND comes in before Reset then JMPs to ABEND
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			(Never returns)			*
;	BX = Not Used							*
;	CX = Not Used							*
;	DX = Not Used							*
; 	BP = SCB							*
;	SI = Not Used							*
;	DI = Not Used							*
;                                                                       *
;***********************************************************************/
;
IdleAdapter	proc	near

	ResetThe82586		; MACRO 

	mov	dx, BMIC_Index
	mov	al, ABEND_IDLE_MAILBOX
	out	dx, al

	mov	dx, BMIC_Data		; DX = BMIC Data Port.
	mov	al, IDLE_STATE_FLAG	; Idle Flag
	out	dx, al			; Mark Idle

WaitingIDLE:
	mov	dx, BMIC_Index			   ; Setup to Watch for Reset Call
	mov	al, AUTO_INC OR ABEND_IDLE_MAILBOX ; Look if Abended first.
	out	dx, al

	mov	dx, BMIC_Data		; Look in Window
	in	al, dx		    	; Get Value
	cmp	al, Abend_AdapterCmd	; Have I Been Told to DIE?
	je	ABEND			; e => DIE Sucker!

	in	al, dx			; Get ParameterMailBox for Reset Command.
	test	al, Reset_AdapterCMD	; Have we Been Told to Reset Yet?
	jz	short WaitingIDLE

;----------------------------------------------------------------------------	
;	Count the Number of Resets in Restart586
;----------------------------------------------------------------------------	
	call	ReStart586		; Restart the 586.

;----------------------------------------------------------------------------
;	Tell The Host that I am Going In the MainEvent and am Able Bodied
;----------------------------------------------------------------------------
	mov	dx, BMIC_Index			   ; Set index to MailBox 1
	mov	al, ABEND_IDLE_MAILBOX or AUTO_INC ; Register
	out	dx, al

	mov	dx, BMIC_Data		; DX = BMIC Data Port.
	mov	al, FIRMWARE_STATE_FLAG	; We are Ready
	out	dx, al

	mov	ax, es			; Clear Reset MailBox
	out	dx, al			
	jmp	MainEvent		; Go Back to the MainEvent.

IdleAdapter		endp



;****************************************************************************
; PromiscuousChange - Enable or disable promiscuous mode
;
;	Assume:	AL = 10h enable promiscuous mode
;		AL = 20h disable promiscuous mode.
;		
;	Return: Preserve all
;****************************************************************************

PromiscuousChange	proc near
	push	bx			;save register
	mov	bx, offset CFGCmdBlk	;bx -> Configure command block
	test	al, 10h			;if al = 10h enable
	jz	Disable

	mov	RxMonStatus, 1
	or	[bx].CFGbyte14, CFG_PRMbit ;set Promiscuous mode
	mov	[bx].CFGbyte16, PRM_MinFrame
	or	[bx].CFGbyte8, CFG_SavBF

	jmp	short ExitPromiscuousChange		

Disable:				;disable promiscuous mode.
	mov	RxMonStatus, 0
	and	[bx].CFGbyte14, NOT CFG_PRMbit
	mov	[bx].CFGbyte16, CFG_MinFrame
	and	[bx].CFGbyte8, NOT CFG_SavBF


ExitPromiscuousChange:
	pop	bx			;restore bx
	ret				
PromiscuousChange	endp

;***********************************************************************\
;                                                                       *
; ReadTCBFromHost - We have a TCB ready at the host and a free transmit	*
; buffer on the adapter. Transfer the TCB to the adapter and initiate	*
; the transfer.								*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Number of bytes to Trans.	AX = Destroyed			*
;	BX = Number of Backup Bytes	BX = Destroyed			*
;	CX = Don't Care			CX = Destroyed			*
;	DX = Don't Care			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Don't Care			SI = Destroyed			*
;	DI = Don't Care			DI = Destroyed			*
;									*
;                               Jmps to MainEvent, no ret               *
;                                                                       *
;***********************************************************************/

ReadTCBFromHost		proc	near
;----------------------------------------------------------------------------
;	Move BMTCB to the Adapter
;----------------------------------------------------------------------------
	mov	ReadBMTCBTransferCount, ax	; store it.

	mov	dx, BMIC_Index			; Get Index port
	mov	al, (PARAMETER_MAILBOX OR AUTO_INC)
	out	dx, al				; Move Window

	mov	dx, BMIC_Data			; Look in Data Window
	lea	di, ReadBMTCBTransLoWord 	; DI -> Dest
	mov	cx, 4
   rep  insb					; Store Address

	neg	bx 					; Get how much to back up.
	lea	di, [bx + (TxBlk + TxBMICTransferLoc)]	; DI -> BMTCB location this time
	mov	si, offset ReadBMTCBFromHostSequence

	TransferDataFromHost		; MACRO Call	

;----------------------------------------------------------------------------
;	Load 82586 Transmit Command Block and Buffer Descriptor fields
;----------------------------------------------------------------------------
Load82586TXCMDBLK:
	mov	bx, TxBlk			  ; BX -> 82586 Command Blk
	mov	cx, [bx].BMTCB586FrameLen	  ; Length of frame on wire
	or	cx, TxBufDes_EOFbit		  ; Mark as last Buffer
	mov	[bx].TxBufDesSize, cx		  ; Store Number of bytes in the "Last" (EOF) buffer
	mov	[bx].TxCmdBlkStatus, es		  ; Clear Status Field
	mov	[bx].TxRetryCount, MAX_TX_RETRIES ; Set Retry count.

;----------------------------------------------------------------------------
;	Load Data Buffer if needed.
;----------------------------------------------------------------------------
	cmp	NumFrags, 0	; Is Frame already there, i.e. No Fragments left?
	jne	FragmentedMove	; Have at least one extra frag.

;----------------------------------------------------------------------------
;	Tell NIC to send the Frame
;----------------------------------------------------------------------------
TimeToSendFrame:

	mov	cx, es				; CX = 0.
WaitFor586Loop:
	cmp	[bp].SCBCommand, 0		; Status complete?
	loopne	WaitFor586Loop			; Loop if not.

	jne	ReStart586FromTx		; Jump if never complete.

	or	ActivityFlag, ActFlgTxWaiting	; Watch for it to complete.
	mov	[bp].SCBCommand, SCBCUC_START	; Command 586 to START CU
	mov	[bp].SCBCBL, TxBlk		; Point to Tx packet
	mov	dx, CA_PORT			; Start the 586
	out	dx, al				; Give CA 

;----------------------------------------------------------------------------
; Signal to the Host That We Have Got The TCB DownLoaded, and sent to the 586.
;----------------------------------------------------------------------------
	mov	dx, BMIC_Index			    ; Set index to Valid TCB 
	mov	al, (AUTO_INC OR VALID_TCB_MAILBOX) ; mailbox
	out	dx, al

	mov	dx, BMIC_Data		; DX = BMIC Data port
	mov	ax, es			; Clear all 3 mailboxes
	out	dx, al
	out	dx, al			
	out	dx, al			; Done
	jmp	MainEvent		; Back to the Main Event


ReStart586FromTx:
	call	ReStart586		; 586 might have hung
	jmp	MainEvent		; Pick up the Packet At the MainEvent

;----------------------------------------------------------------------------
;	BX -> TxBlk, Time to Download what isn't already here.
;----------------------------------------------------------------------------
FragmentedMove:
	lea	bp, [bx].BMTCB586FrameLen	; Load BP for Transfer Loop
	lea	di, [bx].TBData+TxCollSizeDef	; End of the Collection Buffer

;----------------------------------------------------------------------------
;	Download each Fragment to TBDATA
;----------------------------------------------------------------------------
NextTCBFragmentPtr:
	sub	bp, 6				; Back up 6 bytes for next pointer
	mov	ax, word ptr [bp].FragAddr + 0	; Lo word
	mov	FromHostTransferLoWord, ax	
	mov	ax, word ptr [bp].FragAddr + 2 	; Hi Word
	mov	FromHostTransferHiWord, ax
	mov	ax, [bp].FragLen
	cmp	ax, 6				; Too Small for BMIC
	jb	TooSmallForBMIC
				 
	test	di, 1				; Are you Odd?
	jnz	AdjustforOdd			; fix you right up

AdjustedForOdd:
	mov	FromHostTransferCount, ax	; Bytes to Transfer
	lea	si, FromHostTransferSequence	; SI -> Source

	TransferDataFromHost			; MACRO CALL	

	cmp	byte ptr OddByteStorage, 0FFh	; Did we have an odd byte to store?
	je	StoreOddByte			; Then go store it.

OddByteStored:
	add	di, [bp].FragLen	; Next open Spot
	dec	NumFrags		; Watch the Number of Frags
	jnz	NextTCBFragmentPtr	; Get Next Fragment Pointer

	mov	bp, SCB			; Restore BP -> SCB
	jmp	TimeToSendFrame

;----------------------------------------------------------------------------
;	BMIC can only transfer 6 words or more.
;----------------------------------------------------------------------------
TooSmallForBMIC:
	mov	ax, 6	      		; BMIC Minimum
	test	di, 1			; Check if on odd boundary too.
	jz	AdjustedForOdd		; If not odd then jump back now.
					; if odd Fall through to fix before jumping back.

;----------------------------------------------------------------------------
;	BMIC can only transfer on Word Boundaries.-- Odd word fix-up area.
;----------------------------------------------------------------------------
AdjustforOdd:
	dec	di		       		; Back up the pointer 1 byte.
	mov	bl, byte ptr [di]	     	; SI = Value we are saving.
	mov	byte ptr OddByteStorage+1, bl	; Get Value to save.
	mov	byte ptr OddByteStorage+0, 0FFh	; Flag hi byte with FF
	inc	ax				; Get one Extra byte
	dec	FromHostTransferLoWord		; Lo word will never be 0
	jmp  short AdjustedForOdd

;----------------------------------------------------------------------------
;	After the Transfer, need to restore the Destroyed byte.
;----------------------------------------------------------------------------
StoreOddByte:
	mov	ax, es
	xchg	ax, OddByteStorage		; Get\Clear Odd byte.
	mov	byte ptr [di], ah		; Restore byte
	inc	di				; Put DI back where it was.
	jmp short OddByteStored			; Back to normal

ReadTCBFromHost		endp

;****************************************************************************
; UpDateParameters -	Up Date the Parameter Table
;	Read the Parameter block from the Host and redo our multicast table.
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Destroyed			*
;	DI = Not Used			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
;

UpDateExit:			; No MC then Exit Here.
	ret

ForceToMaxMulticast:
	mov	cx, MAX_MULTICAST_ADDR		; Make it the Maximum
	jmp  short MulticastBetweenZeroAndMax


;----------------------------------------------------------------------------
;		STARTS HERE
;----------------------------------------------------------------------------
UpdateParameters	proc	near

	mov	si, offset FromHostTransBlockSequence	; SI -> Source
	mov	di, offset HostsLocalParamTable		; DI -> Destination

	TransferDataFromHost				; MACRO CALL

;----------------------------------------------------------------------------
;	Fix up the Parameter Block Addresses into the Sequence Buffers.
;----------------------------------------------------------------------------
;BMFrameSize Ptr ---- ( Look Ahead Buffer area)
	mov	ax, word ptr BMFrameSizePtr + 0
	mov	TransToHostLookAheadLoWord, ax
	mov	ax, word ptr BMFrameSizePtr + 2
	mov	TransToHostLookAheadHiWord, ax
 	
;----------------------------------------------------------------------------
; If First Time Through Multicast table is not ready yet. (Check if Valid Addr.)
;----------------------------------------------------------------------------
	mov	cx, MulticastCount			; CX = Number of Multicasts possible in the Table.
	jcxz	UpDateExit				; Valid Addresses in the Table?

	cmp	cx, MAX_MULTICAST_ADDR			; Jmp if too many given.
	ja	ForceToMaxMulticast			; Jump if so.


;***************************************************************\
;                                                               *
; Read multicast table from host.				*
;                                                               *
;***************************************************************/
MulticastBetweenZeroAndMax:
	mov	dx, BMIC_Index			  ; Set index to channel 0
	mov	al, CH0_TBI_BASE_ADDR OR AUTO_INC ; TBI Base register.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	ax, offset HostMulticastTable	; AX -> Destination.
	shr	ax, 1				; Adjust for BMIC.
	out	dx, al				; Set addr lo.
	mov	al, ah
	out	dx, al				; Set addr hi.

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_BASE_COUNT OR AUTO_INC	; base count register.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	ax, cx				; AX = Addresses.
	shl	ax, 3				; AX = Addresses * 8.
	out	dx, al				; Send high count.
	mov	al, ah
	out	dx, al				; Send lo count.
	mov	al, AUTO_START			; Set auto start and Read
	out	dx, al				; from EISA.

	mov	ax, word ptr MulticastListPtr	; AX = Lo word of addr.
	out	dx, al				; Set addr byte 0.
	mov	al, ah
	out	dx, al				  ; Set addr byte 1.
	mov	ax, word ptr MulticastListPtr + 2 ; AX = High word of addr.
	out	dx, al				  ; Set addr byte 2.
	mov	al, ah
	out	dx, al				  ; Set addr byte 3.

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_STROBE			; strobe register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	out	dx, al				; Strobe channel 0.

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_STATUS			; status register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.

WaitForMulticastTransfer:
	in	al, dx				; AL = Current Status.
	test	al, TRANSFER_COMPLETE		; Transfer complete?
	jnz	short WaitForMulticastTransfer	; Jump if not.

	and	al, 00011111b			; Mask out reserved Bits.
	out	dx, al				; Clear status bit.

TableTransferedFromHost:
	call	SetupForMulticastCommand 	; Set up command block.
	mov	ax, SCBCUC_START+SCBRUC_ABORT	; AX = start command. (CUC = 1)
	call	Command586NoRestart	 	; Issue command.

	xor	cx, cx				; Set loop counter.
WaitForMCCommandLoop:
	test	word ptr [bp], SCBACK_CX	; Command complete?
	jnz	EnableRU			; Jump if so.
	loop	WaitForMCCommandLoop		; Loop if not.
	jmp	ReStart586

EnableRU:
	jmp	ProcessFRInterrupt		; In case we Rx'd something.


UpdateParameters	endp

;
;***********************************************************************\
;                                                                       *
; UpdateHostStatistics - Transfer statistic buffer to the host.		*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Don't Care			AX = Destroyed			*
;	BX = Not Used			BX = Not Used			*
;	CX = Not Used			CX = Not Used 			*
;	DX = Don't Care			DX = Destroyed			*
; 	BP = SCB			BP = Preserved 			*
;	SI = Don't Care			SI = Destroyed			*
;	DI = Not Used			DI = Not Used 			*
;                                                                       *
;***********************************************************************/
;

UpdateHostStatistics	proc	near
;----------------------------------------------------------------------------
;	Gather and Clear SCB Error Counters.
;----------------------------------------------------------------------------
	mov	ax, es				; Clear AX
	xchg	ax, [bp].SCBCRCerrs	
	add	DriverStatRxCheckSum, ax
	mov	ax, es				; Clear AX
	adc	DriverStatRxCheckSum+2, ax

	xchg	ax, [bp].SCBALNerrs	
	add	DriverStatRxCheckSum, ax
	mov	ax, es				; Clear AX
	adc	DriverStatRxCheckSum+2, ax

	xchg	ax, [bp].SCBRSCerrs	
	add	DriverStatRxOverflow, ax
	mov	ax, es				; Clear AX
	adc	DriverStatRxOverflow+2, ax

	xchg	ax, [bp].SCBOVRNerrs
	add	RxDMAOverrun, ax
	adc	RxDMAOverrun+2, 0

;----------------------------------------------------------------------------
;	Move Counters to the Host.
;----------------------------------------------------------------------------
	mov	si, offset TransToHostStatCountSequence	; SI -> Preset sequence.
	mov	dx, BMIC_Index				; Set index to Transfer
	mov	al, CH0_TBI_BASE_ADDR OR AUTO_INC	; Buffer Interface channel 0.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	cx, 2				; Copy 2 bytes.
 rep	outsb

	mov	dx, BMIC_Index			; Set index to channel 0
	mov	al, CH0_BASE_COUNT OR AUTO_INC	; Base count.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data port.

	mov	cx, 7				; Copy 7 bytes.
 rep	outsb

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STROBE			; Strobe register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	out	dx, al				; No data required for strobe.

	mov	dx, BMIC_Index			; Set index to Channel 0
	mov	al, CH0_STATUS			; Status.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.

WaitForStatisticTransfer:
	in	al, dx				; AL = current status.
	test	al, TRANSFER_COMPLETE		; Transfer complete?
	jnz	short WaitForStatisticTransfer	; Jump if not.

	and	al, 00011111b			; Mask out reserved Bits.
	out	dx, al				; Reset Transfer Complete bit.
	ret

UpdateHostStatistics	endp

;***********************************************************************\
;	GetSCBCounters -- Updates the SCB counters but doesn't clear them.
;			  Called just before the SCB is Cleared for a restart.
;***********************************************************************\
GetSCBCounters	proc	near

	mov	ax, [bp].SCBCRCerrs	
	add	DriverStatRxCheckSum, ax
	adc	DriverStatRxCheckSum+2, 0

	mov	ax, [bp].SCBALNerrs	
	add	DriverStatRxCheckSum, ax
	adc	DriverStatRxCheckSum+2, 0

	mov	ax, [bp].SCBRSCerrs	
	add	DriverStatRxOverflow, ax
	adc	DriverStatRxOverflow+2, 0

	mov	ax, [bp].SCBOVRNerrs
	add	RxDMAOverrun, ax
	adc	RxDMAOverrun+2, 0
	ret
	
GetSCBCounters	endp

;***********************************************************************\
;                                                                       *
; SetupForMulticastCommand - Copy multicast table to SCB structure and	*
; initialize the remainder of the command block.			*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Don't Care			AX = Destroyed			*
;	BX = Don't Care			BX = @ MC Command Block		*
;	CX = Don't Care			CX = Destroyed			*
;	DX = Not Used			DX = Preserved			*
; 	BP = SCB			BP = Preserved			*
;	SI = Don't Care			SI = Destroyed			*
;	DI = Don't Care			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
;----------------------------------------------------------------------------
;	Skip Blanks in the Multicast Table
;----------------------------------------------------------------------------
SkipThisEntry:
	add	si, 8			; Skip the entry
	jmp  short Skipped		; Carry on 

;----------------------------------------------------------------------------
;	Set up For Multicast Command STARTS HERE
;----------------------------------------------------------------------------
SetupForMulticastCommand	proc	near
	mov	bx, es				; Clear BX
	mov	si, offset HostMulticastTable	; SI -> Host table.
	mov	di, MCBlk+MCBlist		; DI -> List of MC Addresses
	mov	cx, MulticastCount		; CX = Addresses in table.
	jcxz short MCTableEmpty			; None in the Table yet.

CopyFromHostLoop:
	cmp	word ptr [si+6], 0		; Is this entry in use?
	je	SkipThisEntry			; 0 => not in use

	movsw					; Copy six bytes.
	movsw
	movsw
	lodsw					; Skip two bytes. (Count Field)(wipes out ax)
	inc	bx				; Count it
Skipped:
	loop	CopyFromHostLoop		; Get next address.

MCTableEmpty:
	mov	ax, bx				; Get Count of Addresses
	mov	bx, MCBlk			; BX -> Multicast Command Buffer.
	mov	[bx].CmdBlkStatus, 0
	mov	[bx].CmdBlkCommand, (CMD_MULTICAST OR EL_bit)
	mov	[bx].CmdBlkLink, 0

;----------------------------------------------------------------------------
;	Multiply number of entries by 6 bytes
;----------------------------------------------------------------------------
	shl	ax, 1			;
	mov	cx, ax			; Faster than MUL
	shl	ax, 1			;
	add	ax, cx			; AX = AX * 6

;----------------------------------------------------------------------------
;	Store the Byte Count and return
;----------------------------------------------------------------------------
	mov	[bx].MCBCnt, ax		; Fill in number of bytes in table.
	ret

SetupForMulticastCommand	endp

;***********************************************************************\
;                                                                       *
; 82586 specific code.							*
;                                                                       *
;***********************************************************************/
;
;****************************************************************************
;	Initialize 82586   - Starts the Network Chip 
;****************************************************************************
;***********************************************************************\
;                                                                       *
; Initialize586 - Reset and configure the 82586, initialize the		*
; transmit buffers, initialize the receive buffers and set the dead man	*
; counters.								*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Destroyed			*
;	DI = Not Used			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
Initialize586	proc	near

;---------------------------------------------------------------\
; 1. Reset the 82586.						*
;---------------------------------------------------------------/
	ResetThe82586				; Macro to reset 82586.

;----------------------------------------------------------------\
; 2. Zero out the memory that the 82586 will use for CMD, Tx and Rx buffers.							 *
;----------------------------------------------------------------/
	mov	di, MEM_586_Start		; DI -> 8000h.
	mov	cx, MEM_Size / 2		; CX -> Memory Size in words
	mov	ax, es				; AX =  0000h.
 rep	stosw					; Zero out block.

;---------------------------------------------------------------\
; 3. Configure 586 						*
;---------------------------------------------------------------/
	call	Configure586			; Initializes SCB and CmdBlk as well.
	jnz	InitializeErrExit		; Problems Then Exit-- Let the Calling Routine Handle it.

;---------------------------------------------------------------\
; 4. Set up the Transmit and Receive Buffers and Call RUStart   *
;---------------------------------------------------------------/
	call	InitializeTxBuffers		; Sets up the TxCmdBlock and Tx Buffers
	call	InitializeRxBuffers		; Sets up the RFA and falls through to RUStart for us.
	jnz	InitializeErrExit

;---------------------------------------------------------------\
; 5.  Set deadman counters.					*
;---------------------------------------------------------------/
	mov	DeadMan, 0			; Init dead man counters.
	mov	DeadManHigh, 25
	xor	bx, bx				; Mark of Success. (sets Flag too)
	ret

;---------------------------------------------------------------\
; Errors Exit Here						*
;---------------------------------------------------------------/
InitializeErrExit:
	mov	bx, INIT_FAILURE586		; 586 Failed to Initialize
	or	bx, bx				; Mark of Problem (Guarentee Psw.NZ)
	ret

Initialize586	endp

;
;***********************************************************************\
;                                                                       *
; ReStart586 - Reset the 82586, re-configure it, start the transmit	*
; unit if the transmit list contains pending transmits and re-load	*
; the multicast table.							*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Destroyed			*
;	DI = Not Used			DI = Destroyed			*
;***********************************************************************/

ReStart586	proc	near

;----------------------------------------------------------------------------	
;	Count the Number of Resets
;----------------------------------------------------------------------------	
	StatisticsUpdate NumberOf82586Resets	; Update counters.

;----------------------------------------------------------------------------	
;If I have an TCB waiting on me I must Tell The Host to Return it.
;----------------------------------------------------------------------------	
	test	ActivityFlag, ActFlgTxWaiting	; Any TCB Pending?
	jz	NothingPending			; Tell Host To ForGet any TCB's he has given me.

;----------------------------------------------------------------------------	
;	Tell Host To ForGet any TCB's he has given me.
;----------------------------------------------------------------------------	
	and	ActivityFlag, NOT ActFlgTxWaiting; Clear Flag
	mov	dx, BMIC_Index			; Set index to EISA Doorbell
	mov	al, EISA_DOORBELL_STATUS	; interrupt status register.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	mov	al, TX_COMPLETE_BIT		; Set interrupt bit to forget the TCB's
	out	dx, al	

;----------------------------------------------------------------------------	
;	Call Initialize586 to Get the 586 going again.
;----------------------------------------------------------------------------	
NothingPending:
	call	GetSCBCounters			; Update Any SCB Counters.
	call	Initialize586			; Re-initialize the 586.
	jnz	ReStart586			; nz => Problems try til it is ok.

	and	ActivityFlag, No586ActivityFlg	; Clear 586 activity flag.
	ret

ReStart586	endp


;***********************************************************************\
;                                                                       *
; Poll586 - Read SCB Status to check for transmit or receive interrupt.	*
; If no interrupts from 82586, process dead man timer.			* 
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Don't Care			AX = Destroyed			*
;	BX = Don't Care			BX = Destroyed			*
;	CX = Don't Care			CX = Destroyed			*
;	DX = Don't Care			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Not Used			*
;	DI = Not Used			DI = Not Used			*
;                                                                       *
;***********************************************************************/
;

PollRestart586:
	call	ReStart586		; Attempt to restart it.
	jmp  short CheckDeadMan		; Continue polling.

;----------------------------------------------------------------------------
;Poll586 STARTS HERE
;----------------------------------------------------------------------------
Poll586		proc	near

	mov	cx, es				; Set loop counter. TO MAXimum
WaitForCommandEmpty:
	cmp	[bp].SCBCommand, 0		; Ready for new Command?
	loopne	WaitForCommandEmpty		; Loop if not.
  	jne	PollRestart586			; 586 might be hosed.

	mov	ax, [bp].SCBStatus		; Check the Status
	mov	bx, ax				; Save the Status bits
	and	ax, SCB_IntAckMask		; Strip out the Interrupt bits.
	jz	short CheckDeadMan		; z => No Interrupts

	mov	[bp].SCBCommand, ax
  	mov	dx, CA_PORT			; Tug on the 82586's
	out	dx, al				; channel attention.

	mov	cx, es				; Set loop counter.
WaitForAcknowledge:
	cmp	[bp].SCBCommand, 0		; Command finished.
	loopne	WaitForAcknowledge		; Loop if not.
	jne	PollRestart586			; 586 might be hosed.

	test	bx, SCB_RUneedsAttn		; Was a frame received/or Are we out of Resources?
	jz	short ChkCXbit			; Jump if not.

	push	bx				; Save SCB status.
	call	ProcessFRInterrupt		; Process Rx Interrupt.
	pop	bx

ChkCXbit:
	test	bx, SCB_CUneedsAttn		; Transmit complete?
	jz	short CheckDeadMan		; Jump if not.

	call	ProcessCXInterrupt		; Process Tx Interrupt.

CheckDeadMan:
	test	[bp].SCBStatus, SCBRUS_NoRESOURCE ; Out of Resources?
	jnz	DeadManExpired			  ; nz => trouble try to start the RU

	test	ActivityFlag, ActFlgTxWaiting	; Waiting for transmit to complete?
	jnz	DecDeadMan			; Jump if so.

	test	ActivityFlag, ActFlag586	; Activity recorded from 586 lately?
	jz	DecDeadMan			; Jump if not.

	and	ActivityFlag, No586ActivityFlg	; Clear 586 activity flag.
	mov	DeadMan, 0			; Reset deadman counters.
	mov	DeadManHigh, 25

DecDeadMan:
	dec	DeadMan	     			; Decrement DeadManLow.
	jz	DecDeadManHigh			; Jump if zeroed.
	ret					; Exit.

DecDeadManHigh:
	dec	DeadManHigh			; Decrement DeadManHigh.
	jz	DeadManExpired			; Jump if zeroed, and need to restart the 82586.
	ret					; Exit.

DeadManExpired:
	jmp	ReStart586			; Re-start the 82586, and return from there.
	
Poll586		endp

;
;***********************************************************************\
;                                                                       *
; Configure586 - Initialize 586, Send Config and IA-Setup	 	*
;		 And MC-Setup Commands					*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Destroyed			*
;	DI = Not Used			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
;
Configure586	proc	near
;----------------------------------------------------------------
; 1. Build up initial SCP and ISCP before starting the 82586.	*
;----------------------------------------------------------------
	mov	si, offset InitialSCP		; Build our SCP.
	mov	di, SCP
	mov	cx, SIZE SCPstruc / 2
 rep	movsw

	mov	di, offset ISCP			; Build our ISCP.
	mov	cx, SIZE ISCPstruc / 2
 rep	movsw

;----------------------------------------------------------------
; 2. Issue a Channel Attention to Start the 586.		*
;----------------------------------------------------------------
	mov	dx, CA_PORT			; Tug on the 82586's
	out	dx, al				; channel attention.

;----------------------------------------------------------------
; 3. Send configure command to the 82586.  w/ LoopBack 		*
;----------------------------------------------------------------
IssueConfigCommand:
	call	ConfigureLoopBack		; Send Configure with loopBack bit set.
	jnz	CommandError			; nz => previous Command not Accepted

	mov	cx, es				; Set loop counter.
WaitForConfigLoop:
	test	word ptr [bx], C_BIT		; Command complete?
	jnz	IssueIACommand			; Jump if so.
	loop	WaitForConfigLoop		; Loop if not.
	jmp  short CommandError			; Jump to exit if error

;----------------------------------------------------------------
; 4.	Send IAsetup command to the 82586.			*
;----------------------------------------------------------------
IssueIACommand:				; Set Up For IA Command
	mov	di, CmdBlk		; DI -> Command Block
	mov	ax, es			; AX = 0.
	stosw				; Zero Status Word.
	mov	word ptr [di], (EL_bit or CMD_IASETUP) ; Set up Command word.
	inc	di
	inc	di  			; DI -> CmdBlk.Link Offset
	dec	ax			; AX = FFFFF
	stosw				; Mark Link Offset as FFFFh.
	mov	si, offset BoardID	; SI -> Node Address.
	movsw
	movsw				; Load Node Address (6 bytes)
	movsw

	mov	ax, SCBCUC_START	; AX = start command. (CUC = 1)
	mov	bx, CmdBlk		; BX -> Command Block Buffer.
	call	Command586NoRestart	; Send command.
	jnz	CommandError		; nz => Previous Command was Not Accepted

	mov	cx, es			; Set loop counter.
WaitForIACmdLoop:
	test	word ptr [bx], C_BIT	; Command complete?
	jnz	IssueMCCommand		; Jump if so.
	loop	WaitForIACmdLoop	; Loop if not.
	jmp  short CommandError		; Jump to exit if error

;----------------------------------------------------------------------------
; 5.	Issue MC-Setup Command
;----------------------------------------------------------------------------
IssueMCCommand:
	call	SetupForMulticastCommand	; Set up block. BX -> MC-CmdBlk
	mov	ax, SCBCUC_START		; AX = start command.
	call	Command586NoRestart		; Issue command.
	jnz	CommandError			; Jump if time out.

	mov	cx, es				; Set loop counter.
WaitForMCCmdLoop:
	test	word ptr [bx], C_BIT		; Command complete?
	jnz	ExitConfig			; Jump if so.
	loop	WaitForMCCmdLoop		; Loop if not.

CommandError:
	mov	bx, INIT_FAILURE586		; Some Failure????
	or	bx, bx				; Mark of Trouble
	ret

;----------------------------------------------------------------------------
; 6.	Take 82586 out of LoopBack and Place in Normal Mode.
;----------------------------------------------------------------------------
ExitConfig:
	call	ConfigureNoLoopBack		; Configure Command Normal Mode.
	jnz	CommandError			; nz => previous Command not Accepted

	mov	cx, es				; Set loop counter.
WaitForConfigLoop2:
	test	word ptr [bx], C_BIT		; Command complete?
	jnz	ConFigureExit			; Jump if so.
	loop	WaitForConfigLoop2		; Loop if not.
	jmp  short CommandError			; Jump to exit if error

ConFigureExit:	
	xor	bx, bx				; Mark of Success (And Sets Flag TOO)
	ret

Configure586	endp

;***********************************************************************\
;                                                                       *
; InitializeTxBuffers - Initialize the transmit buffers in the 82586's	*
; buffer space.								*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Preserved			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Preserved			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Preserved			*
;	DI = Not Used			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
;
InitializeTxBuffers	proc	near

	mov	di, TxBlk			; DI -> Tx buffers.
	mov	[di].TxCmdBlkStatus, es		; Clear status.
	mov	[di].TxCmdBlkCmd, (EL_bit OR I_bit OR CMD_TRANSMIT) ;?EOL and interrupt.
	mov	[di].TxCmdBlkLnk, -1		; Phoney -- only one
	lea	bx,[di].TxBufDesSize 		; BX -> TxBufDes
	mov	[di].TxCmdBlkTxBufDes, bx	; Save TxBufDes offset.
	lea	bx, [di].TBData			; Bx -> Transmit Data Buffer
	mov	[di].TxBufDesBuffl, bx		; Save TBData's offset (actually is the physical addr.)
	mov	[di].TxBufDesBuffh, es		; Clear upper bits.
	mov	[di].TxBufDesSize, es		; Clear Size field
	mov	word ptr [di].TxBufDesLnk, -1	; Clear Link Field Meaningless -- only one here
	mov	[di].TxRetryCount, MAX_TX_RETRIES	; Set up for Retries
	ret

InitializeTxBuffers	endp
;
;***********************************************************************\
;                                                                       *
; InitializeRxBuffers - 
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Preserved			*
;	DI = Not Used			DI = Destroyed			*
;                                                                       *
;***********************************************************************/
;
InitializeRxBuffers	proc	near
	mov	ax, MEM_586_End			; AX = Room left over for
	sub	ax, RxBlk			; receive buffers.
	mov	dx, es				; DX = 0 for divide.
	mov	bx, SIZE RxBlockStruc		; BX = Size of each buffer.
	div	bx				; AX = Total Receive buffers possible.
	mov	cx, ax				; CX = Total Receive buffers.
	mov	di, RxBlk			; DI -> Receive Buffers start.
	mov	RxHead, di			; Save as head of list.
	mov	ax, es				; AX = 0000h.

RxInitLoop:
	mov	[di].RFDStatus, ax		; Zero status.
	mov	[di].RFDCommand, ax		; Clear command.
	lea	dx, [di + SIZE RxBlockStruc]	; DX -> Next buffer.
	mov	[di].RFDLink, dx		; Save pointer.
	lea	bx, [di].RBDStatus		; BX -> RBD struct.
	mov	[di].RFDRBD, bx			; Zero status.
	mov	[di].RBDStatus, ax		; Clear RBD status.
	lea	bx, [di].RBuffData		; BX -> Data buffer.
	mov	[di].RBDbufflo, bx		; Save pointer.
	mov	[di].RBDbuffhi, ax		; Zero high word.
	mov	[di].RBDLink, -1		; No Link.
	mov	[di].RBDbuffSize, (RxBufferSize OR 8000h); Size with EOL bit.
	mov	di, dx				; DI -> Next buffer.
	loop	RxInitLoop

	sub	di, SIZE RxBlockStruc		; DI -> Last buffer.
	mov	RxTail, di			; Where is Last buffer.
	mov	[di].RFDCommand, EL_bit		; Set EL bit.
	mov	ax, RxHead			; Make last buffer
	mov	[di].RFDLink, ax		; point to first.
	jmp	RUStart				; Start receive unit, and return from there.

InitializeRxBuffers	endp

;***********************************************************************\
;	ConfigureLoopBack	Configure Command with LoopBack set.	*
;***********************************************************************/
;	Assumes: Nothing	Returns:
;					AX, BX, CX, SI, DI   Destroyed.
;					Jumps to Command586NoRestart

ConfigureLoopBack	proc	near
	mov	si, offset CFGCmdBlk		; SI -> Config Command Defaults
	mov	di, CmdBlk			; DI -> Config command buffer.
	mov	cx, CFGCmdBlkSize		; CX = words to move.
 rep	movsw
	mov	bx, CmdBlk			; BX -> Command Buffer
  	or	[bx].CFGbyte9, CFG_IntLoopBack	; Place in LoopBack Mode.
	mov	ax, SCBCUC_START		; AX = start command. (CUC = 1)
	jmp short Command586NoRestart		; Send Command.

ConfigureLoopBack	endp

;***********************************************************************\
; ConfigureNoLoopBack	-- Makes 82586 configure for Normal Mode.      *
;***********************************************************************/
;	Assumes: Nothing	Returns:
;					AX, BX, CX, SI, DI   Destroyed.
;					Jumps to Command586NoRestart

ConfigureNoLoopBack	proc	near
	mov	si, offset CFGCmdBlk		; SI -> Config Command Defaults
	mov	di, CmdBlk			; DI -> Config command buffer.
	mov	cx, CFGCmdBlkSize		; CX = words to move.
 rep	movsw
	mov	bx, CmdBlk			; BX -> Command Buffer
	mov	ax, SCBCUC_START		; AX = start command. (CUC = 1)
;Fall Through For NOW
;jmp short Command586NoRestart		; Send Command.(Get RET from There).

ConfigureNoLoopBack	endp
; CAUTION DON"T MOVE COMMAND586NoRESTART
;***********************************************************************\
;                                                                       *
; Command586NoRestart - Wait for any previous commands to finish and	*
; start the next one. Return if previous command not cleared.		*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = SCB Command		AX = Preserved			*
;	BX = @ Command Block		BX = Preserved			*
;	CX = Don't Care			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Preserved			*
;	DI = Not Used			DI = Preserved			*
;					Zero flag set if successful.	*
;                                                                       *
;***********************************************************************/
;
Command586NoRestart	proc	near
	mov	cx, es			; Set loop counter.
WaitForPreviousLoop:
	cmp	[bp].SCBCommand, 0	; Command finished?
	loopne	WaitForPreviousLoop	; Loop while not.
	jne	Command586Exit		; nz => 586 might be hosed.

	mov	[bp].SCBCommand, ax	; Set 586 command.
	mov	[bp].SCBCBL, bx		; Set command block pointer.
	mov	dx, CA_PORT		; Tug on the 82586's
	out	dx, al			; channel attention.
Command586Exit:
	ret				; Exit.

Command586NoRestart	endp

;****************************************************************************
;Process CX Interrupt -   Transmit Complete
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Preserved			*
;	DI = Not Used			DI = Preserved			*
;                                                                       *
;***********************************************************************/
;
ProcessCXInterrupt	proc	near
	mov	ax, ds:TxBlk.TxCmdBlkStatus ; Get current TxCmdBlock status

	test	ax, B_BIT		; Busy?
	jnz	IgnoreCX		; Too Busy forget it.

	test	ax, C_BIT		; Command Complete
	jz	IgnoreCX		; Not a Tx Command Completed.

	test	ax, OK_BIT		; Transmit Successful?
	jz	TxError			; z = we have a problem.

	test	ax, TxCmdBlk_DeferTx	; Tx Defered?
	jz	CheckCollCnt		; Jump if haven't

	StatisticsUpdate TxDefers	; update counters

CheckCollCnt:
	and	ax, TxCmdBlk_COLLSNS_CNT
	jz	ErrorReEntry			; No Collisions	
	
	add	TxNumCollisions, ax		; Keep Track of Collisons
	adc	TxNumCollisions + 2, 0		; Catch Wrap Around

ErrorReEntry:
	or	ActivityFlag, ActFlgCX		  ; Signal Somehow We were Called.
	and	ActivityFlag, NOT ActFlgTxWaiting ; Clear Transmit flag
	mov	dx, BMIC_Index			  ; Set index to EISA Doorbell
	mov	al, EISA_DOORBELL_STATUS	  ; interrupt status register.
	out	dx, al
	mov	dx, BMIC_Data			  ; DX = BMIC Data Port.
	mov	al, TX_COMPLETE_BIT		  ; Mark for Tx Complete.
	out	dx, al	

IgnoreCX:
	ret

;----------------------------------------------------------------------------
;	Transmit Error 
;----------------------------------------------------------------------------
TxError:
	test	ax, TxCmdBlk_NoSenseSig		; Carrier Sense lost during tx
	jz	TxErrorCheckClearToSend		; z => try someone else

	StatisticsUpdate TxCarrierSenseLost	; Update Counters

TxErrorCheckClearToSend:
	test	ax, TxCmdBlk_NoClrSndSig	; Loss of Clear-to-Send, Tx stopped
	jz	TxErrorCheckUnderRun		; z => try someone else

	StatisticsUpdate TxClearToSend		; Update Counters

TxErrorCheckUnderRun:
	test	ax, TxCmdBlk_DMAunderRUN	; UnderRuns?
	jz	TxErrorCheckMaxCollisions	; Jump if not

	StatisticsUpdate TxUnderRun		; update counters.

TxErrorCheckMaxCollisions:
	test	ax, TxCmdBlk_MAXCOLLSNS		; 16 collisions?
	jz	TxErrorCheckRetries		; No- Fine Check Retries.

	add	TxNumCollisions, 16		; too many collsions.
	adc	TxNumCollisions + 2, 0		; catch the wrap around.
	StatisticsUpdate TxMaxCollisions	; update counters

TxErrorCheckRetries:				; All errors try again.
	dec	ds:TxBlk.TxRetryCount		; Keep track of how many times
	jnz	TransNeedsRetry			; Jump if can retry

	StatisticsUpdate TxRetryFailure
	jmp	ErrorReEntry			; Go on Acknowledge it and forget it.

TransNeedsRetry:
	mov	ds:TxBlk.TxCmdBlkStatus, es	; Clear Status
	mov	bx, TxBlk			; BX = Command to Start.
	mov	ax, SCBCUC_START		; Start Command
	call	Command586NoRestart		; Tell 586 to try again.

	StatisticsUpdate DriverStatTxRetryCount

	ret					; Finished so Return.

ProcessCXInterrupt	endp

;****************************************************************************

;***********************************************************************\
;                                                                       *
; ProcessFRInterrupt - If frame was received successfully, call receive	*
; routine to transfer to the host.					*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Destroyed			*
;	DI = Not Used			DI = Destroyed			*
;		Jumps to RUSTART or RESTART586 depending on State of RU.*
;***********************************************************************/


;----------------------------------------------------------------------------
;	Need  an RCB for a new packet, the host is slow.
;----------------------------------------------------------------------------
AlreadyWaiting:
 	mov	dx, BMIC_Index			; Set index to EISA Doorbell
	mov	al, EISA_DOORBELL_STATUS	; interrupt status register.
	out	dx, al
	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	or	RxStatus, PKT_WAITING		; Mark Have another RX but no Lookahead location.
	mov	al, RxStatus			; Mark for Rx in LookAhead.
	out	dx, al	
PFRIntExitjmp:
	jmp	PFRIntExit

;----------------------------------------------------------------------------
;	Receive Error Path
;----------------------------------------------------------------------------
TooShort:
	StatisticsUpdate DriverStatRxTooSmall	; update counters
	mov	[si].BMFrameSize, cx		; Store Frame Size before Transfering
	cmp	RxMonStatus, 1			;is RcvMonitor enabled
	je	PktHasBadLengthGoOn
	jmp	MiscError

TooBig:
	StatisticsUpdate DriverStatRxTooBig	; update counters
	mov	[si].BMFrameSize, cx		; Store Frame Size before Transfering
	mov	cx, 1514			; only pass up 1514 bytes.
	cmp	RxMonStatus, 1			;is RcvMonitor enabled
	je	PktHasBadLengthGoOn
	jmp	MiscError
	
;----------------------------------------------------------------------------
;	Process FR Starts Here
;----------------------------------------------------------------------------
ProcessFRInterrupt	proc	near


  	or	ActivityFlag, ActFlgFR		; Signal that we have seen a little activity.
	test	ActivityFlag, ActFlgRxWaiting	; Already waiting for the host to complete an RCB.
;;;	jnz	AlreadyWaiting			; if So can't now, tell Host and exit.
	jnz	PFRIntExitjmp

	mov	RxStatus, 0	;clear Rx errors

	mov	si, RxHead		; SI -> Next Rx Frame
	mov	ax, [si].RFDStatus	; Check Status 
	test	ax, C_BIT		; -- Complete?
	jz	PFRIntExitjmp		; Forget it if not

	test	ax, OK_BIT		; Is it Complete and OK?
	jnz	RxMonitorEnabled	; Paket Good go on.
	jmp	CheckRxError		; Z => Not Good throw pkt away (should never happen)

RxMonitorEnabled:
	mov	cx, [si].RBDStatus	; CX = Status/Length
	and	cx, RBD_LenMask		; Mask out the length bits

;----------------------------------------------------------------------------
;	Check Size of the Packet.
;----------------------------------------------------------------------------
	cmp	cx, 60			; Packet Too Small (60 bytes...64 bytes with CRC)
	jb	TooShort		; Mark the Error

	cmp	cx, 1514		; Maximum Packet Size for Ethernet.
	ja	TooBig			; Mark the Error

;----------------------------------------------------------------------------
;			Send Look Ahead to Host	
;----------------------------------------------------------------------------	
	mov	[si].BMFrameSize, cx		; Store Frame Size before Transfering
PktHasBadLengthGoOn:
	add	cx, 2				; Add a word for the Count Field
	mov	TransToHostLookAheadCNT, cx 	; Store Byte Count in Sequence Buffer
	lea	di, [si].BMFrameSize		; MAC Header in Buffer
	lea	si, TransToHostLookAheadSequence	
		
	TransferDataToHost			; MACRO call

	or	ActivityFlag, ActFlgRxWaiting	; Mark that the Lookahead is busy
 	mov	dx, BMIC_Index			; Set index to EISA Doorbell
	mov	al, EISA_DOORBELL_STATUS	; interrupt status register.
	out	dx, al

	mov	dx, BMIC_Data			; DX = BMIC Data Port.
	or	RxStatus, PKT_IN_LOOKAHEAD	; 
	mov	al, RxStatus			; Mark for Rx in LookAhead.
	out	dx, al	

;----------------------------------------------------------------------------
;	Forget this Packet and return the RFD to the Ring.
;----------------------------------------------------------------------------
ReLinkRFD: 
	mov	si, RxHead			; Get First Rx Buffer
	mov	bx, RxTail	       		; Get Last Rx Buffer
	mov	[si].RFDCommand, EL_bit		; Mark EL bit on new tail
	mov	[si].RFDStatus, es		; Clear Status on new tail
	mov	[si].RBDbuffSize, RxBufferSize OR RBD_EOFbit	;Set Size/EL bit on New Tail.
	mov	[bx].RFDCommand, es    		; Clear EL bit from old tail
	mov	bx, [si].RFDLink		; Get Next FD for Head
	mov	RxTail, si			; Move first to last 
	mov	RxHead, bx			; Store it.

;----------------------------------------------------------------------------
;	Check if the RU unit is OK.
;----------------------------------------------------------------------------
PFRIntExit:
	test	[bp].SCBStatus, SCBRUS_NoRESOURCE ; Out of Resources?
	jz	RUStart				  ; No Then Just Restart us.

	StatisticsUpdate DriverStatRxOverflow	; See if it ever happens.
	jmp	ReStart586 			; Go Restart 586.

ProcessFRInterrupt	endp


CheckRxError:
	cmp	RxMonStatus, 1			;is RcvMonitor enabled
	jne	MiscError			;no? loose packet

	test	ax, RFD_CRCErrbit			;yes? check for crc error.
	jz	CheckAllignError
	StatisticsUpdate DriverStatRxCheckSum	; See if it ever happens.
	or	RxStatus, 10h

CheckAllignError:
	test	ax, RFD_ALNErrbit
	jz	RxMonitorEnabledjmp

	or	RxStatus, 20h
RxMonitorEnabledjmp:
	jmp	RxMonitorEnabled

MiscError:
	StatisticsUpdate DriverStatRxMisc	; See if it ever happens.
	jmp	ReLinkRFD



;***********************************************************************\
;                                                                       *
; RUStart - Start the Receive Unit.					*
;									*
;***********************************************************************<
;                                                                       *
; On Entry:			On Exit:				*
;	AX = Not Used			AX = Destroyed			*
;	BX = Not Used			BX = Destroyed			*
;	CX = Not Used			CX = Destroyed			*
;	DX = Not Used			DX = Destroyed			*
; 	BP = SCB			BP = Preserved			*
;	SI = Not Used			SI = Preserved			*
;	DI = Not Used			DI = Preserved			*
;                                                                       *
;***********************************************************************/
;
RUStart		proc	near
	test	[bp].SCBStatus, SCBRUS_READY	; Receive Unit Ready ?
	jnz	RUStartExit			; Exit if it is.

	mov	bx, RxHead			; BX -> First frame.
	mov	cx, es				; Set loop counter.

RUWaitForCommand:
	cmp	[bp].SCBCommand, 0		; Command finished.
	loopne	RUWaitForCommand		; Loop if not.
	jne	RUStartErrorExit		; 586 might be hosed.

	mov	[bp].SCBRFA, bx			; Set RFA offset.
	mov	[bp].SCBCommand, SCBRUC_START	; Start RU command.

	mov	dx, CA_PORT			; Tug on the 82586's
	out	dx, al				; channel attention.

RUStartExit:
	xor	bx, bx				; Set success flag

RUStartErrorExit:
	ret

RUStart		endp



;----------------------------------------------------------------------------
; Note: Although Not Required the following is included to make the Firmware Code Trackable in the Future
;       It Also looks nice in memory
;----------------------------------------------------------------------------
				; Auto updating of the config table 
FVerMjr	equ	01    		; Firmware Version numbers are controlled by these equates.
FVerMin	equ	14
				; Version program Compatible.
		; VeRsIoN=	; This Version string shows up for the ASM FIle only Not .OBJ
FirmWareSignOnMessage	db	'Netware NE3200 FirmWare  v'
			db	'0' OR (FVerMjr MOD 10),'.'
			db	'0' OR (FVerMin/10),'0' OR (FVerMin MOD 10)
			db	' (971121)',0dh,0ah      ;FixDate will correctly stamp it.
			db	'(c) Copyright 1991 - 1997 Novell, Inc.  '
			db	'All rights reserved.',0

 FirmWareSize	equ	$ - FirmWareStart	; Total Firmware Space to move

 
 CODE	ENDS

 END

