%PAGESIZE 66,130
;*****************************************************************************
; NTR2000.asm
;       (c) Copyright Unpublished Work of Novell, Inc. All Rights Reserved
;
;       THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
;       PROPRIETARY AND TRADESECRET INFORMATION OF NOVELL, INC.  ACCESS
;       TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
;       NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
;       AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO APPROPRIATE
;       LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED, PRACTICED,
;       PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,
;       ABBRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
;       TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL.
;       ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
;       SUBJECT THE PERPERTRATOR TO CRIMINAL AND CIVIL LIABILITY.
;
;------------------------------------------------------------------------------
;
;       Novell NTR2000 Token-Ring MLID for DOS ODI
;
;       Filename:       NTR2000.ASM
;
;       Written by:     JWR/WTT
;
;       Date:           09/21/92        This code was adapted from Novell's
;                                       IBM Token-Ring MLID.
;
; Modification History:
;       By      Date        Ver.           Modification Description
;       ------ ---------   -----   --------------------------------------
;    JWR & WTT 01 Aug 94   2.00    Removed Old History,
;                                  Remade to confirm to ODI SPEC 4.00
;
;       WTT   05 Aug 94    2.00b
;       Fix bug introduced in DriverInit.  Would always fail to
;       initialize if card was located in an EISA machine.  This
;       was due to a problem with the code dealing with the Bus ID
;       and the use of the al register.
;
;     WTT & AYD  29 Aug 94 2.01    Added changes to RPL code for IBM's PnP adapter.
;
;       JWR,WTT 22 Sept. 94 2.02   Re-linked to include latest tool kit.
;
;       AYD,JWR 30 Sept. 94 2.03   Added Changes for IBM's PnP Adapter.
;
;       JWR     09 Jan 95 2.04     Re-linked to include latest tool kit.
;
;       JWR     05 April 95 2.05   Re-linked to include latest tool kit.
;                                  Fix Max Packet Size Calulation to account for 6 byte overhead.
;                                  Adjusted Interrupt State for MSMGetRCB to match spec.
;
;       JWR     03 May 95 2.06a1   SPD #96189
;                                  Adjust for Default\Maximum DHB size of 8-Bit Token-Ring Cards.
;                                  Print out Second Line of Size Too Big in NET.CFG.
;
;       JWR     30 Aug 95 2.06     Re-Link to include Latest ToolKit.
;                                  Corrected Specification Version String.
;
;       JWR     25 Oct 95 2.07     Re-Link to include Latest ToolKit (2.05).
;
;       JWR     4 Jan. 96 2.07      Update CopyRight.

;	AYD	8 March 96 2.09	   IBM Sent RPL fixes.
;
;       JWR     1 April 96 2.10     Re-Link to include Latest ToolKit (2.10).
;
;	AYD	15 July 96 2.11    Incorporated changes from IBM to support
;				   latest Token-Ring cards. Additional range 
;				   of IRQ and 16K page size.
;
;       CKP     11 Mar. 97 2.12    Updated CopyRight and relinked with latest
;                                  token.asm
;
;	SMN 21 Nov. 97 2.12		Re-Link Latest ToolKit. v2.12, Update Copyright date
;
;*****************************************************************************
.8086
include driver.inc

;Local Macros

;A quick way to accomplish either an even or odd byte move

FastMove        macro           ;; Enter with ES:DI -> Dest  |  DS:SI -> Source
                                ;;            CX = # bytes to move
        shr     cx, 1           ; get Num words
   rep  movsw
        adc     cl, cl          ; any odd bytes left?
   rep  movsb                   ; move odd byte if exist
        endm


;******************************************************************************

Init    segment para public 'CODE'

;----------------------------------------------------------------------------
;       Version numbers for sign on message and configuration table
;----------------------------------------------------------------------------

MajorVersion    equ     2
MinorVersion    equ     12              ; Last Release November 21,1997 (v2.12)
;AlphaVersion   equ     1               ; Comment in and out as needed for
;BetaVersion    equ     0               ; Prerelease (Set November 21, 1997)

;----------------------------------------------------------------------------
;       Signon Message
;----------------------------------------------------------------------------
HSMSPEC                 db      'HSM_SPEC_VERSION: 4.00', 0
                        db      'VeRsIoN='
DriverSignOnMessage     db      'Novell NTR2000 Token-Ring MLID  v'
                        db      '0' OR (MajorVersion MOD 10),'.'
                        db      '0' OR (MinorVersion/10)
                        db      '0' OR (MinorVersion MOD 10)

ifdef AlphaVersion
                db      ' Alpha ',((AlphaVersion/10) OR '0')
                db      ((AlphaVersion MOD 10) OR '0') ; Only for Alpha builds.
endif

ifdef BetaVersion
                db      ' BETA ',(BetaVersion + '0')   ; Only for Beta Builds.
endif

                db      ' (971121) ',0Dh,0Ah      ;FixDate will correctly stamp it.
                db      '(c) Copyright 1991 - 1997, by Novell, Inc. '
                db      'All rights reserved.',0


DriverMainSectionText           db      'LINK DRIVER NTR2000',0

Init    ends

Code    segment para public 'CODE'

        public  DriverSendFastPath
        public  FastPathTxComplete
        public  FastPathInit
        public  NICConfigFastPathDone
        public  ProcessISR
        public  ChkAdapterInt
        public  InitNic
        public  NICDirOpenDone
        public  AdapterIsOpen
        public  ChkARBTxDataReq
        public  NICOpenAdapterLoop
        public  ARBReqInt
        public  StartSend

Message NICLongName  'Novell NTR2000 Token-Ring'
Message NICShortName 'NTR2000'

;******************************************************************************
;             D R I V E R   S T A T I S T I C S   T A B L E
;******************************************************************************
        even
DriverStatTable         label   byte
DriverStatMajorVer      db      01
DriverStatMinorVer      db      01
DriverStatGenericCnts   dw      14
DriverStatValidCntsMask dd      00011001011110111111111111111111B
DriverStatTotalTx       dw      2 dup (0)
DriverStatTotalRx       dw      2 dup (0)
DriverStatNoECB         dw      2 dup (0)
DriverStatTxTooBig      dw      2 dup (0)               ;Not Used
DriverStatTxTooSmall    dw      2 dup (0)               ;Not used
DriverStatRxOverflow    dw      2 dup (0)
DriverStatRxTooBig      dw      2 dup (0)
DriverStatRxTooSmall    dw      2 dup (0)               ;Not Used
DriverStatTxMisc        dw      2 dup (0)               ; Bad Correlator value Error
DriverStatRxMisc        dw      2 dup (0)               ;Not Used
DriverStatTxRetryCount  dw      2 dup (0)               ;Not Used
DriverStatRxChecksum    dw      2 dup (0)               ;Not Used
DriverStatRxMismatch    dw      2 dup (0)               ;Not used
DriverQueueDepth        dw      2 dup (0)

;       Custom Diagnostic counters.

     NumberOfCustomCounters     dw      14

        LineErrors              dw      2 dup (0)
        InternalErrors          dw      2 dup (0)
        BurstErrors             dw      2 dup (0)
        ACErrors                dw      2 dup (0)
        AbortDelimeters         dw      2 dup (0)
        LastRingStatus          dw      2 dup (0)
        LostFrames              dw      2 dup (0)
        ResetNICCount           dw      2 dup (0)
        FrameCopiedError        dw      2 dup (0)
        FrequencyError          dw      2 dup (0)
        TokenErrors             dw      2 dup (0)
        AdapterCheckErrCounter  dw      2 dup (0)
        UnknownARBRequests      dw      2 dup (0)
        UnexpectedASBFree       dw      2 dup (0)

;Pointers to the Counter Message Strings
                        dw      offset CGroup:DS1
                        dw      0
                        dw      offset CGroup:DS2
                        dw      0
                        dw      offset CGroup:DS3
                        dw      0
                        dw      offset CGroup:DS4
                        dw      0
                        dw      offset CGroup:DS5
                        dw      0
                        dw      offset CGroup:DS6
                        dw      0
                        dw      offset CGroup:DS7
                        dw      0
                        dw      offset CGroup:DS8
                        dw      0
                        dw      offset CGroup:DS9
                        dw      0
                        dw      offset CGroup:DS10
                        dw      0
                        dw      offset CGroup:DS11
                        dw      0
                        dw      offset CGroup:DS12
                        dw      0
                        dw      offset CGroup:DS13
                        dw      0
                        dw      offset CGroup:DS14
                        dw      0


; Diagnostic Custom Error Counters
Message         DS1     'LineErr'
Message         DS2     'InternalErr'
Message         DS3     'BurstErr'
Message         DS4     'ACError'
Message         DS5     'AbortDelimeter'
Message         DS6     'LastRingStatus'
Message         DS7     'LostFrame'
Message         DS8     'ResetAdapter'
Message         DS9     'FrameCopiedErr'
Message         DS10    'FrequencyErr'
Message         DS11    'TokenErr'
Message         DS12    'AdapterCheckErr'
Message         DS13    'UnknownARBRequest'
Message         DS14    'UnexpectedASBFree'

;******************************************************************************
;               P I O   P o r t   D e f i n i t i o n s
;******************************************************************************
        even
SetupRead1              dw      0        ; + 0
AdapterReset            dw      1        ; + 1
AdptrRelease_SetupRead2 dw      2        ; + 2 Setupread2 & Adapter Reset Latch
AdapterInterruptReset   dw      3        ; + 3 Interrupt Reset and enable
NumIOPorts      equ     ($-SetupRead1)/2 ; Number of ports

;----------------------------------------------------------------------------
;               PIO Port Equates    :Base IO addresses
;----------------------------------------------------------------------------
PrimPortAddr    equ     0A20h
AltPortAddr     equ     0A24h

;******************************************************************************
;            D R I V E R   C O N F I G U R A T I O N   T A B L E
;******************************************************************************
;       Due to the Fact that the Token-Ring Cards are Self-Configurable, Default
;       values have been used sparingly to avoid presenting bogus error statements
;       by the MSM.
;-------------------------------------------------------------------------------
DriverConfigTable       label   byte
Signature               db      'HardwareDriverMLID',8 dup (' ')
CfgTbleMajorVersion     db      01
CfgTbleMinorVersion     db      13
NodeAddress             db      6 dup (0FFh)
ModeFlags               dw      0C009h          ;Real,Multicast,LSB/MSB Configureable
                                                ;MSBModeDefault.
BoardNumber             dw      ?               ;set by MSM
BoardInstance           dw      ?               ;set by MSM
MaxFrameSize            dw      4222            ;Default Size can be overwritten (Should be multiple of 8)
BestDataSize            dw      ?               ;Max - (MAC header[14] + 802.2 UI[3])
WrstDataSize            dw      ?               ;Max - (MAC header[14] + 802.2 UI[3]
                                                ;  + Source Routing[30])
LANCardName             dw      offset CGroup:NICLongName,0
LANShortName            dw      offset CGroup:NICShortName,0
LANFrameName            dd      ?               ;Don't touch MSM will set
Reserved0               dw      0               ; No longer used RESERVED MUST be 0
LANFrameID              dw      ?               ;Don't touch MSM will set
TransportTime           dw      1
SourceRouteHandler      dd      0               ;Don't Touch MSM and Token.asm use
LookAheadSize           dw      18              ; Just enough for IPX
LineSpeed               dw      0               ; Allow DriverInit to Set it
Reserved1               db      6 dup (0)       ; Saved for future uses
PrioritySup             db      0
BusID                   db      BUS_ID_HSM_DEFAULT ; DriverInit to set it.
DriverMajorVersion      db      MajorVersion    ; Set by Equate
DriverMinorVersion      db      MinorVersion    ; Set by Equate
Flags                   dw      0
SendRetries             dw      0               ; Default retry value (Not Supported in Token-Ring)
ConfigTableLink         dd      0               ;Don't Touch it.
SharingFlags            dw      0               ; Share nothing-- be stingy
Slot                    dw      0               ; ignored by ISA but used by MCA
IOAddress1              dw      0000h           ; Leave Zero to detect Override from Net.cfg
IORange1                dw      NumIOPorts      ; only 4 ports are defined for Token-Ring
IOAddress2              dw      0               ;Not used
IORange2                dw      0               ;Not used
MemAddress1             dd      0h              ; Bios/MMIO Address determined in DriverInit
MemRange1               dw      2000h / 16      ; No. of Memory Par. in BIOS/MMIO (8k)
MemAddress2             dd      0h              ; Shared RAM Address determined in DriverInit
MemRange2               dw      0h              ; Size of Ram Determined in DriverInit
IRQLine1                db      0h              ; Irq line will be set by DriverInit
IRQLine2                db      0FFh            ;Not used
DMALine1                db      0FFh            ;Not used
DMALine2                db      0FFh            ;Not used

;******************************************************************************
;               B I O S / M M I O    S T R U C T U R E
;******************************************************************************

; define the MMIOSegment
MMIOStructure   struc
        MMIOSkip        db      1E00h dup (?)           ; OFFSET
        RRReven         db      ?                       ; 1E00h
        RRRodd          db      ?                       ; 1E01h
        WRBReven        db      ?                       ; 1E02h
        WRBRodd         db      ?                       ; 1E03h
        WWOReven        db      ?                       ; 1E04h
        WWORodd         db      ?                       ; 1E05h
        WWCReven        db      ?                       ; 1E06h
        WWCRodd         db      ?                       ; 1E07h
        ISRPeven        db      ?                       ; 1E08h
        ISRPodd         db      ?                       ; 1E09h
        ISRAeven        db      ?                       ; 1E0Ah
        ISRAodd         db      ?                       ; 1E0Bh
        TCReven         db      ?                       ; 1E0Ch
        TCRodd          db      ?                       ; 1E0Dh  : Used by the Adapter's Microcode only
        TVReven         db      ?                       ; 1E0Eh
        TVRodd          db      ?                       ; 1E0Fh
        MMIOSkip1       db       8 dup (?)              ; Reserved 1E10h -> 1E17h
        SRPReven        db      ?                       ; 1E18h
        SRPRodd         db      ?                       ; 1E19h  : Reserverd Register
        MMIOSkip2       db      14 dup (?)              ; Reserved 1E1Ah -> 1E27h
        ISRPResetEven   db      ?                       ; 1E28h  : Even Byte ISRP
        ISRPResetOdd    db      ?                       ; 1E29h  : Odd Byte ISRP
        MMIOSkip3       db      ?                       ; Reserved 1E2A
        ISRAReset       db      ?                       ; 1E2Bh  : Odd  Byte ISRA
        TCRReset        db      ?                       ; 1E2Ch  : Even Byte TCR
        MMIOSkip4       db      27 dup (?)              ; Reserved 1E2Dh -> 1E47h
        ISRPSeteven     db      ?                       ; 1E48h  : Even Byte ISRP
        ISRPSetodd      db      ?                       ; 1E49h  : Odd byte ISRP
        ISRASeteven     db      ?                       ; 1E4Ah  : Even byte ISRA
        ISRASetodd      db      ?                       ; 1E4Bh  : Odd Byte ISRA
        TCRSet          db      ?                       ; 1E4Ch  : Even Byte TCR
        MMIOSkip5       db      51 dup (?)              ; Reserved 1E4Dh -> 1E7Fh
        MMIORes1        db      128 dup (?)             ; Reserved 128 bytes (do not Access) 1E80h -> 1EFFh

;AIP AREA
; even Byte Values valid only

        AIPEncodedAddr  db      24 dup (?)              ; 1F00h -> 1F16h : AIP Encoded Address   (skip odd byte 1F17h)
        AIP1sCompAddr   db      24 dup (?)              ; 1F18h -> 1F2Eh : AIP ones complement   (skip odd byte 1F2Fh)
        AIPChanSig      db      48 dup (?)              ; 1F30h -> 1F5Eh : Channel Identifier (skip odd byte 1F5Fh)
        AIPCheckSum1    db      ?                       ; 1F60h  : 4 bit Checksum of 1F00h -> 1F60h
        MMIOAIPskip1    db      31 dup (?)              ; Reserved 1F61 -> 1F7F
        AIPTestPattern  db      32 dup (?)              ; 1F80h -> 1F9Fh  : Test Pattern
        AIPAdapterType  db       2 dup (?)              ; 1FA0h  : Adapter Type  (skip odd byte 1FA1h)
        AIPDataRate     db       2 dup (?)              ; 1FA2h  : Data Rate
        AIPEarlyTkn     db       2 dup (?)              ; 1FA4h  : Early Token Release
        AIPTotalRam     db       2 dup (?)              ; 1FA6h  : Total Available Shared RAM
        AIPSharedRamPag db       2 dup (?)              ; 1FA8h  : Shared RAM Paging
        AIPDHBat4Mbps   db       2 dup (?)              ; 1FAAh  : DHB size available at 4 Mbps
        AIPDHBat16Mbps  db       2 dup (?)              ; 1FACh  : DHB size available at 16 Mbps
        MMIOAIPskip2    db      66 dup (?)              ; Reserved 1FAEh -> 1FEEh
        AIPCheckSum2    db      ?                       ; 1FF0h  : 4 bit checksum of 1F00h -> 1FF0h
        MMIOAIPskip3    db      15 dup(?)               ; Reserved 1FF1h -> 1FFFh
MMIOStructure   ends


;******************************************************************************
;               Token-Ring Adapter ID Prom (AIP) Signitures
;------------------------------------------------------------------------------
; TRSMC -- Token-Ring Signature for Micro Channel -- MARS63X4518
;                       Hex String =    4D41 5253 3633 5834 3531 3820
;          Located At 1F30 Offset Into The MMIO Segment
; 'x'/16,'x'-'x'/16*16 is the Way IBM created the Hex string value

TRSMC   struc                           ;Token-Ring Signature for Micro Channel
                                        ; hex value
          db    'M'/16,'M'-'M'/16*16    ; 4D
          db    'A'/16,'A'-'A'/16*16    ; 41
          db    'R'/16,'R'-'R'/16*16    ; 52
          db    'S'/16,'S'-'S'/16*16    ; 53
          db    '6'/16,'6'-'6'/16*16    ; 36
          db    '3'/16,'3'-'3'/16*16    ; 33
          db    'X'/16,'X'-'X'/16*16    ; 58
          db    '4'/16,'4'-'4'/16*16    ; 34
          db    '5'/16,'5'-'5'/16*16    ; 35
          db    '1'/16,'1'-'1'/16*16    ; 31
          db    '8'/16,'8'-'8'/16*16    ; 38
                                        ; 20
TRSMC      ends                  ;   Token-Ring Signature for Micro Channel


; TRSPC -- Token-Ring Signature for PC Bus        -- PICO6110990
;                       Hex String =    5049 434F 3631 3130 3939 3020
;          Located At 1F30 Offset Into The MMIO Segment

TRSPC   struc                           ;Token-Ring Signature for PC Bus
                                        ; Hex value
          db    'P'/16,'P'-'P'/16*16    ; 50
          db    'I'/16,'I'-'I'/16*16    ; 49
          db    'C'/16,'C'-'C'/16*16    ; 43
          db    'O'/16,'O'-'O'/16*16    ; 4F
          db    '6'/16,'6'-'6'/16*16    ; 36
          db    '1'/16,'1'-'1'/16*16    ; 31
          db    '1'/16,'1'-'1'/16*16    ; 31
          db    '0'/16,'0'-'0'/16*16    ; 30
          db    '9'/16,'9'-'9'/16*16    ; 39
          db    '9'/16,'9'-'9'/16*16    ; 39
          db    '0'/16,'0'-'0'/16*16    ; 30
                                        ; 20
TRSPC      ends                  ;   Token-Ring Signature for PC Bus


;******************************************************************************
;               The Registers of the Attachment Control Area
;******************************************************************************
;------------------------------------------------------------------------------
;               ACR -- Adapter Control Region Register EQUates
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; define the Adapter Interrupt Status Register (ISRA) bits
;------------------------------------------------------------------------------
SSBFreeBit                      equ     01h     ;Odd
ARBFreeBit                      equ     02h
ASBFreeRequestBit               equ     04h
SRBFreeRequestBit               equ     08h
ResponseInASBBit                equ     10h
CommandInSRBBit                 equ     20h
ISRABridgeReq                   equ     40h
ISRAReservedoddbit              equ     80h


AdapterSoftwareInterruptMaskBit equ     01h     ;Even
AdapterHardwareInterruptMaskBit equ     02h
ISRAReservedevenbit             equ     04h
AdapterProcessorCheckBit        equ     08h
DeadmanTimerHasExpiredBit       equ     10h
AccessInterruptBit              equ     20h
TimerInterruptBit               equ     40h
ISRAInternalParityError         equ     80h

;-------------------------------------------------------------------------
; define the PC Interrupt Status Register (ISRP) bits
;-------------------------------------------------------------------------
                        ; Odd Register
ISRPoddbit1             equ     01h     ; reserved
ISRPoddBridge           equ     02h     ; transmited frame from bridge
SSBResponseBit          equ     04h     ; SSB has response to previous SRB
ARBCommandBit           equ     08h     ; ARB has new command for PC
ASBFreeBit              equ     10h     ; ASB is now ready for a new response
SRBResponseBit          equ     20h     ; the SRB has been processed see return code
AdapterCheckBit         equ     40h     ; Adapter has Unrecoverable error and has closed (Software Check)
ISRPoddbit7             equ     80h     ; reserved

                        ; Even Register
PrimAltAddrBit          equ     01h     ; 0 = Primary adapter, 1 = Alternate Adapter
InterruptBlockedBit     equ     02h     ; For PC System w/ PC I/O bus: Interrupt Block Bit
ISRPAccessErrBit        equ     04h     ; Shared Ram Access violation or illegal MMIO operation
ErrorIntBit             equ     08h     ; Machine Check occurred, Adapter Deadman Timer expire, or Adapter Timer overrun (Hardware Check)
ISRPTimerIntBit         equ     10h     ; TVR-even has Expired
ISRPevenbit5            equ     20h     ; Reserved
InterruptEnableBit      equ     40h     ; 1 = PC will get Ints, 0 = PC won't see ints
NMIIntControlBit        equ     80h     ; used to control error interrupts.  1 => IRQ  0 => CHCK
                                        ; Not supported on Adapter/A and must be set to 1

;-------------------------------------------------------------------------
; define the Timer Control Register (TCR) bits
;-------------------------------------------------------------------------
AdapterTimerBit                 equ     01h     ;Even
TCRReserved                     equ     02h
IBMPCInterlockBit               equ     04h
IBMPCTimerCountStatusBit        equ     08h
IBMPCTimerOverrunStatusBit      equ     10h
IBMPCTimeCountGateBit           equ     20h
IBMPCTimerReloadModeBit         equ     40h
IBMPCTimerInterruptMaskBit      equ     80h

;-------------------------------------------------------------------------
; define the Shared RAM Paging Register (SRPR) bits
;-------------------------------------------------------------------------
MMIOPagingActiveFlag    equ     0C0h            ; Paging is Active Flag

;******************************************************************************
;               S H A R E D   R A M   I n f o r m a t i o n
;******************************************************************************

;-------------------------------------------------------------------------
;       The value of FastPathRAMSize is determined via the following
;
;               Max Frame Size = 17960 bytes
;               Buffer size = 512 bytes
;               Buffer Header = 22 bytes
;               Buffer Data = 490 bytes
;               Control Area = 16 bytes
;
;       17960/(512-22) = 36.65  (Round to 37)
;       37 + 1 = 38             (Add one extra buffer to make up for
;                                crossing a 16K page boundry.  16K paging
;                                is worse case.)
;       38 * 64 = 2432          (There are 64 -- 8 byte blocks in 512 bytes)
;       2432 + 2 = 2434         (Add 2 more blocks for the 16 byte control
;                                area.)
;-------------------------------------------------------------------------
FastPathRAMSize         equ     2434    ; Fast Path RAM Size (# of 8 byte blocks)

SHARED_RAM_8K           equ     00h     ; Shared RAM Size = 08k
SHARED_RAM_16K          equ     04h     ; Shared RAM Size = 16k
SHARED_RAM_32K          equ     08h     ; Shared RAM Size = 32k
SHARED_RAM_64K          equ     0Ch     ; Shared RAM Size = 64k

SharedRamDefault$       equ     0D800h  ; Shared RAM Default Address
SharedRamSizeMask       equ     0Ch     ; Strips out bit for RRRodd for Ramsize

PageSize                dw      4000h   ; Default Page Size is 16 Kbytes (0-3FFFh)
PageBreakMask           db      0C0h    ; Checks for crossing page boundaries
PageSizeMask            db      0FFh    ; Store Mask for Paging if used.
PageActFlg              db      3Fh     ; Used to know if Paging is used (16k default)
PageIncrement           db      40h     ; Value to move to the next page (16k default)
RAMSize                 db      SHARED_RAM_16K  ; Default Size.
DriverMaxActiveTx       db      2       ; Number of Transmits the driver wants to handle

        even
MMIOSegment             dw      ?       ; Segment Value MMIO/Rom Area on The Adapter
SharedRamSegment        dw      ?       ; Segment Value Shared RAM Area on the Adapter


;*************************************************************************
;               D e f a u l t   D e f i n i t i o n s
;*************************************************************************
; Token-Ring Default Definitions
DefaultPortAddr         equ     0A20h   ; Default Port I/O Addr. (Primary Adapter)

; )----->   Misc definitions

SRBSize                 equ     28              ;28 Size of SRB in Shared RAM
SSBSize                 equ     20              ;20 Size of SSB in Shared RAM
ARBSize                 equ     28              ;28 Size Of ARB in Shared RAM
ASBSize                 equ     12              ;12 Size of ASB in Shared RAM

;-------------------------------------------------------------------------
; define Interrupt State Flags  (InterruptStateFlag) bits
;-------------------------------------------------------------------------
IgnoreAllInterrupts     equ     80h
WaitingIntBit           equ     40h
AdapterShutdownBit      equ     20h
DirOpenInProgress       equ     10h                     ; Dir.Open.Adapter has been issued, not yet answered

DIRReadLoginOrder       equ     08h
DIRSetFuncAddr          equ     04h
AdaptHeldReset          equ     02h
SRBBusy                 equ     01h
        even
InterruptStateCopy      dw      ?                       ; Maintain Copy watch it is on an even boundary
InterruptStateFlag      db      IgnoreAllInterrupts + AdapterShutdownBit

;----------------------------------------------------------------------------
; Define DriverStateFlag        (Needed not enough bits in the Intstateflag)
;----------------------------------------------------------------------------

DriverInitFlagBit       equ     01h
MaxFrameFromNetCFGBit   equ     02h

DriverStateFlag         db      DriverInitFlagBit

;******************************************************************************
;               M i s c .   V a r i a b l e s
;******************************************************************************
XmitQueHead     dw      0000            ; Local Queue Ptr for Inprocess TCB's
QueTail         dw     offset CGroup:XmitQueHead
TooBusyPtr      dw      0000            ; Local Queue Ptr for Preprocessed TCB's
FuncAddr        db      00,00,00,00     ; Functional Address  (Multi-casting) (must be a db field)
AdaptRetryCnt   db      2               ; Try at least twice to get the card going

ResetRetryDef   equ     4               ; When reseting during run time give it 4 tries

DirOpenRetCode  dw      ?

;-------------------------------------------------------------------------
;               TCB Work space Definitions    (6 bytes total)
;-------------------------------------------------------------------------
TCBNxtLnk       equ     0       ; offset into the TCB for a NEXT Link ptr to the next TCB

;-------------------------------------------------------------------------
;               AES Event Storage  Location
;-------------------------------------------------------------------------
        even
NICAES          AESECB  <>              ; ESR location to force Read Log

;*************************************************************************
;       S R B   D e f i n i t i o n s   a n d   E q u a t e s
;*************************************************************************
;-------------------------------------------------------------------------
;       SRB Command Definitions
;-------------------------------------------------------------------------

SRBCmdDIRModifyOpenParms        equ     01h             ; DIR.Modify.Open.Parms
SRBCmdDIROpenAdapter            equ     03h             ; DIR.Open.Adapter
SRBCmdDIRCloseAdapter           equ     04h             ; DIR.Close.Adapter
SRBCmdSetFuncAddr               equ     07h             ; DIR.Set.Functional.Address
SRBCmdDIRReadLog                equ     08h             ; DIR.Read.Log
SRBCmdSetBridgeParm             equ     09h             ; SET.Bridge.Parameters
SRBCmdXMTDIRFrame               equ     0Ah             ; XMT.DIR.Frame
SRBCmdXMTIFrame                 equ     0Bh             ; XMT.I.Frame
SRBCmdDIRConfigBridgeRAM        equ     0Ch             ; DIR.Configure.Bridge.RAM
SRBCmdXMTUIFrameSAP             equ     0Dh             ; XMT.UI.Frame.SAP
SRBCmdDLCReset                  equ     0Eh             ; DLC.Reset
SRBCmdDLCOpenSAP                equ     0Fh             ; DLC.Open.SAP
SRBCmdDLCCloseSAP               equ     10h             ; DLC.Close.SAP
SRBCmdDLCStatistics             equ     1Eh             ; DLC.Statistics
SRBCmdDIRInitComplete           equ     80h             ; DIR.Initialize Complete    **
SRBCmdInvalid                   equ     0FFh            ; Invalid Command Code
SRBResponseDataRate             equ     01h             ; 1 ==> 16 Mbps; 0 ==> 04 Mbps ( Warning...
                                                        ; This one Byte is defined as bits 0 -> 7 )
INIT_FAST_PATH                  equ     20h             ; Fast Path Transmit supported

;-------------------------------------------------------------------------
;       SRB Storage Locations           Intel Format Low-High  Order is Critical
;
;       Note:   These addresses are normally returned via DirOpenAdapter and
;               stored in the following variables.  However, when the HSM is
;               included in TOKEN.RPL it does not do the DirOpenAdapter so
;               we need to set the HSM up to match the card when the driver
;               takes over.
;
;-------------------------------------------------------------------------
        even
ASBAddress     dw	0FDF4h          ; Storage for ASB Relative Address (1)
SRBAddress     dw	0FDD8h          ; Storage for SRB Relative Address (2)
ARBAddress     dw	0580h          ; Storage for ARB Relative Address (3)
SSBAddress     dw	0               ; Storage for SSB Relative Address (4)


SRBSaveTCBPtr   dw      ?       ; Temp storage for SI ptr to TCB

; AYD/WLR 7-15-96 Extended Function Adapter, Enhanced Mode Support additions
LOCALFlags      db      0       ; Local flag defined for IFtable
EnhancedMode    EQU     2h
IFtableFound    EQU     1h

;-------------------------------------------------------------------------
; define SRB/SSB  System Request/Status Block
;-------------------------------------------------------------------------

SRBSSBStructure struc
        SRBSSBCommand           db      ?               ;00 Command
        SRBSSBCorrelator        db      ?               ;01 Cmd. Correlate
        SRBSSBRetCode           db      ?               ;02 Return Code
        SRBSSBReserved1         db      ?               ;03 Reserved
        SRBSSBStationID         dw      0000            ;04 Station ID, label here for ref. only
        SRBSSBFuncAddr          dd      00000000        ;06 DIR.Set.Functional.Address Value
SRBSSBStructure ends

;-------------------------------------------------------------------------
;       Token-Ring DIR.Initialize Adapter SRB Response
;-------------------------------------------------------------------------

DIRInitSRBResponse      struc   ; DIR.Initialize SRB Response
        InitSRBCmd      db      0               ;00 Command Code (DIR.Initialize Complete)
        InitSRBStatus   db      00              ;01 DIR.Initialize Status
                        dd      00              ;02 RESERVED
        InitSRBError    db      00, 00          ;06 Bring Up Code
        InitSRBNode     db      00, 00          ;08 Shared RAM Addr. Adapter Node @
        InitSRBLevel    db      00, 00          ;10 Shared RAM Addr. Microcode Level
        InitSRBAdaptAdr db      00, 00          ;12 Shared RAM Addr. Adapter Addresses
        InitSRBParm     db      00, 00          ;14 Shared RAM Addr. Adapter Parameters
        InitSRBMACBuffr db      00, 00          ;16 Shared RAM Addr. Adapter MAC Buffer
DIRInitSRBResponse      ends                    ;18 DIR.Initialize SRB Response

;-------------------------------------------------------------------------
;       Transmit.Dir.Frame --- SRB
;-------------------------------------------------------------------------

XMTSRB struc
        XMTSRBCmd       db      SRBCmdXMTDIRFrame
        XMTSRBCmdCor    db      ?
        XMTSRBRetCode   db      ?
        XMTSRBReserved  db      ?
        XMTSRBStat_ID   dw      ?
XMTSRB  ends

;*************************************************************************
;               STRUCTURES and Storage for ARB
;*************************************************************************

;-------------------------------------------------------------------------
;                          ARB Cmd List
;-------------------------------------------------------------------------

ARBCmdRxData            equ     81h     ;  81h ==> Received.Data
ARBCmdTxData            equ     82h     ;  82h ==> Transmit.Data.Request
ARBCmdDLCStat           equ     83h     ;  83h ==> DLC.Status
ARBCmdRingStatChg       equ     84h     ;  84h ==> Ring.Status.Change

;-------------------------------------------------------------------------
;                          ARB-84 Network Status Bits
;-------------------------------------------------------------------------
; High Byte

SignalLoss      equ     80h             ; 1 => Signal Loss
ARBHardErr      equ     40h             ; 1 => Beacon Frames are being Transmitted or received
ARBSoftErr      equ     20h             ; 1 => Tx Soft Error Report MAC Frame
ARBTxBeacon     equ     10h             ; 1 => Adapter Transmiting Beacon
LobeWireFault   equ     08h             ; 1 => Cable disconnected
AutoRemovError  equ     04h             ; 1 => Auto Remove Error 01
ARBResUpper     equ     02h             ; Reserved
RemoveReceived  equ     01h             ; 1 => Remove Received

; Low Byte

CounterOverflow equ     80h             ; 1 => Counter Overflow
ARBSingleStation equ    40h             ; 1 => We are the only one here
ARBRingRecovery equ     20h             ; 1 => Adapter is Tx or Rx Monitor Contention MAC Frames.
ARBReserved0_4  equ     1Fh             ; Reserved bits

;-------------------------------------------------------------------------
;               ARB -- Adapter Request Block
;-------------------------------------------------------------------------

ARB     struc                                   ; Adapter Request Block
        ARBCommand      db      00              ;00 ARB Command Code
        ARBCorrelator   db      00              ;01 ARB Correlator (Transmit.Data.Request)
        ARBReserved     db      00              ;02 Reserved
                        db      00              ;03 Reserved
        ARBStation      dw      0000            ;04 Station ID
        ARBBufAddr      dw      0000            ;06 DHB Address (Transmit.Data.Request)
                                                ;   1st Rx Buf Addr (Received.Data)
        ARBLanHdrLen    db      00              ;08 Length of Lan Header Field
        ARBDLCHdrLen    db      00              ;09 Length of DLC Header Field
        ARBFrameLen     dw      0000            ;10 Length of the Entire Frame
        ARBNCBType      db      00              ;12 Category of the Message Received
                                                ; (DLC.Status)
        ARBAccesPrio    db      00              ;13 New Access priorirty for SAP/Station
        ARBRemoteAddr   db      6 dup(0)        ;14 Physical Ring Address of Remote Station
        ARBRSapVal      db      00              ;20 Remote Station's Sap_Value
ARB     ends


ARBStatus       equ     ARBBufAddr      ; Current Network Status (Ring.Status.Change)
                                        ; DLC Status indicator   (DLC.Status)
ARBFrmrData     equ     ARBLanHdrLen    ; 5 bytes . Data Sent/received with FRMR response (DLC.Status)

;-------------------------------------------------------------------------
;                       ARB Misc. Variables
;-------------------------------------------------------------------------

ARBSaveArea     db      ARBSize dup (00) ; ARB Image Work area
ARBTCBPtr       dd      00000000h        ; Holds the TCB while DS:SI are busy in the ARB Int.
ARBRCBPtr       dd      00000000h        ; Holds the RCB while ES:SI are Busy in the ARB Int.
ARBRCBmarker    dd      00000000h        ; Point to the Current Frag?Address we're working on in the RCB
ARBRxBufPtr     dd      00000000h        ; Holds the current Rx buffer while ES:DI are busy


;*************************************************************************
;       Structures for the Adapter Receive
;*************************************************************************

;-------------------------------------------------------------------------
;       ARBRxB  --- Shared Ram Receive Buffer for ARB 81 -- Receive Data
;-------------------------------------------------------------------------

ARBRxB  struc                           ; Recieve Buffer for ARB 81 -- Receive Data
;;;     Reserved        dw      0000    ; 00 Reserved, These 2 bytes are automatically skipped by the Adapter
                                        ; Restart the Index counting
        RxBNextBufPtr   dw      0000    ; 00 Next Buffer @
                        db      00      ; 02 Reserved
        RxBRxFS         db      00      ; 03 Receive Frame Status (FS)
        RxBBufLen       dw      0000    ; 04 Length of data in this buffer
        RxBFrameData    db      0       ; 06 Frame Data Starts here.
ARBRxB  ends


;*************************************************************************
;       STRUCTURES for ASB
;*************************************************************************

;-------------------------------------------------------------------------
; ASB -- Adapter Status Block
;-------------------------------------------------------------------------

ASBFrameLength  equ     6                       ;06 ASB Frame Length (ARB 82 - Tx)


ASB     struc                           ; Adapter Status Block
        ASBCommand      db      00      ;00 ASB Command Code
        ASBCorrelator   db      00      ;01 ASB Correlator
        ASBRetCode      db      00      ;02 ASB Return Code
        ASBReserved     db      00      ;03 ASB RESERVED
        ASBStation      dw      0000    ;04 ASB Station ID
        ASBRxBufferPtr  dw      0000    ;06 ASB Receive Buffer @ (ARB 81 - Rx)
        ASBHeaderLen    db      00      ;08 ASB Header Length    (ARB 82 - TX)
        ASBRemoteSAP    db      00      ;09 ASB Remote SAP Value (ARB 82 - Tx)
ASB     ends

;-------------------------------------------------------------------------
;               ASB Equates
;-------------------------------------------------------------------------

ASBNoDataRx     equ     20h             ; Lost Data, or No Buffers available

;*************************************************************************
;               STRUCTURES and VARIABLES for Fast Path
;*************************************************************************

;-------------------------------------------------------------------------
;               Configure Fast Path RAM SRB structure.
;-------------------------------------------------------------------------

ConfigFastPathStructure struc
        FPCommand       db      12h             ; 00 Command
                        db      0               ; 01 reserved
        FPRetcode       db      0               ; 02 Set by adapter on return
                        db      5 dup (0)       ; 03 reserved
        FPRAMSize       dw      0               ; 08 RAM size to Allocate
        FPBufferSize    db      02h, 00h        ; 10 Size of Tx buffers(512)
ConfigFastPathStructure ends

;-------------------------------------------------------------------------
;               Configure Fast Path RAM Return SRB structure.
;-------------------------------------------------------------------------

ConfigFastPathResponse  struc
        FPRCommand      db      12h             ; 00 Command
                        db      0               ; 01 reserved
        FPRRetcode      db      0               ; 02 Return code for this command
                        db      5 dup (0)       ; 03 reserved
        FPRFastPathXmit dw      0               ; 08 Offset to Tx control area
        FPRSRBAddress   dw      0               ; 10 Offset for next SRB addr
ConfigFastPathResponse  ends

;-------------------------------------------------------------------------
;               Fast Path Transmit Control Area structure.
;-------------------------------------------------------------------------

FastPathControlArea     struc
        FPBufferCount           dw      0               ; 00 Number of buffers
        FPFreeQHead             dw      0               ; 02 Offset to first free buffer
        FPFreeQTail             dw      0               ; 04 Offset to last free buffer
        FPAdapterQTail          dw      0               ; 06 offset to next expected
                                                        ;    first buffer of frame.
        FPCBufferSize           dw      0               ; 08 Size in bytes of each buffer
                                                        ;    including the buffer header
        FPCompletionQTail       dw      0               ; 10 Offset to last completed buffer
                                db      4 dup (0)       ; 12 reserved
FastPathControlArea     ends

;-------------------------------------------------------------------------
;               Fast Path Transmit Buffer Structure.                            *
;-------------------------------------------------------------------------

FastPathTxBuffer        struc
        FPTxCommand             db      0ah             ; 00 Transmit command code.
        FPTxCorrelator          db      00h             ; 01 Transmit correlator(0-127)
        FPTxRETCODE             db      00h             ; 02 Return Code for this command
                                db      00h             ; 03 reserved
        FPTxSTATIONID           dw      0000h           ; 04 ID of station to tx this frame
        FPTxFrameLength         dw      0000h           ; 06 Total amount of data in all
                                                        ;    the buffers of the frame
        FPTxHeaderLength        db      00h             ; 08 Length of the Frame Header
        FPTxRSAPValue           db      00h             ; 09 DSAP value
                                dw      0000h           ; 10 reserved
        FPTxLastBuffer          dw      0000h           ; 12 Offset in shared RAM to NEXT.BUFFER
                                                        ;    field of the last buffer of the frame
        FPTxFramePointer        dw      0000h           ; 14 reserved for the adapter
        FPTxNextBuffer          dw      0000h           ; 16 Offset to NEXT.BUFFER field in the next
                                                        ;    buffer in the free tx buffer queue
        FPTxStatus              db      00h             ; 18 reserved for the adapter
        FPTxStrippedFS          db      00h             ; 19 Final status returned from
                                                        ;    frame stripping process
        FPTxBufferLength        dw      0000h           ; 20 Length of the frame data
                                                        ;    in this buffer
        FPTxFrameData           db      00h             ; 22 The frame data to be transmitted
FastPathTxBuffer        ends

;-------------------------------------------------------------------------
;                       Fast Path Misc. Variables
;-------------------------------------------------------------------------
TxPageValue             db      0               ; Fast Path Page.
TxControlAddress        dw      0               ; Fast Path Tx Buffer.

InitStatus              db      0               ; DIRInitStatus value.
TxCorrelator            db      0               ; Fast Path Correlator
CompletionQHead         dw      0               ; Adapter Completion Q Head.

TxLastBuffer            dw      0               ; Fast Path Last Buffer.
TxBufferSpace           dw      0               ; Fast Path Buffer Space.
TxCurrentBuffer         dw      0               ; Fast Path current buffer.
TxCurrentTCB            dw      0               ; Fast Path TCB.
TxFragmentCount         dw      0               ; Fast Path Fragments.

ConfigFastPath  db      12h                     ; 00 Command
                db      00h                     ; 01 reserved
                db      00h                     ; 02 Set by adapter on return
                db      5 dup (0)               ; 03 reserved
ConfigRAMSize   dw      0                       ; 08 RAM size to allocate
                db      02h, 00h                ; 10 Size of Tx buffers(512)

;*************************************************************************
;   STRUCTURES for DIRECT INTERFACE COMMANDS
;*************************************************************************

;-------------------------------------------------------------------------
;       Dir.Open.Reply
;-------------------------------------------------------------------------

DirOpenReply    struc
        OpenReplyCommand        db      ?
        OpenReplySkip1          db      ?
        OpenReplyRetCode        db      ?
        OpenReplySkip3          db      ?
                                dw      ?
        OpenReplyExtendedError  dw      ?
        OpenReplyASBAddress     dw      ?
        OpenReplySRBAddress     dw      ?
        OpenReplyARBAddress     dw      ?
        OpenReplySSBAddress     dw      ?
DirOpenReply    ends


;-------------------------------------------------------------------------
;               Dir.Open.Adapter
;-------------------------------------------------------------------------
; define DIR.Open.Adpater Equates

; Open_options          ; Word Values in Low-High Equates... When used will become High-Low automatically

OpnOptWrapInt           equ     0080h
OpnOptNoHardError       equ     0040h
OpnOptNoSoftError       equ     0020h
OpnOptPassAdaptMACFrm   equ     0010h
OpnOptPassAttenMACFrm   equ     0008h
OpnOptReturnOpenPar     equ     0002h
OpnOptPassBeaconMACFrm  equ     0001h
OpnOptContender         equ     8000h

OpnNumRxBuffer          equ     4       ; Number of Receive Buffers default to 4 (i.e. to be able
                                        ; to handle a 4k data Packet. The Adapter will
                                        ; allocate as many receive buffers as possible, if we ask for
                                        ; more than is available an error will occur...
                                        ; Make sure the number ask for is less than what is possible


OpnDefaultRxBufSize equ 552     ; This is to allow optimization on 2-4k packet sizes, and still
                                ; minimize the congestion in the small packet case.
                                ; (40 = Max Header, 8 = Overhead per Buffer. 74 = Protocol Overhead)
                                ; To better Utilize the Space, The Size of 552 was chosen to Maximize
                                ; Space Utilization, Congestion of Small packets, and Speed Criteria.
                                ; 552 = (2048+52+74+(8*4))/4 and then made a multiple of 8

OpnLen2KTxBuf   equ     2174    ; Transmit packets of 2k data size with max header size of 52 bytes and 74 Protocol header
                                ; if only 8k Ram Available.  2174 = (2048+52+74)

OpnLen4KTxBuf   equ     4222    ; Default size (4096 + 52 + 74) is to allow the following
                                ; 4k data packet + 52 bytes Media Header + 74 bytes Protocol Headers.

OpnLen10KTxBuf  equ     10366   ; Transmit Packets of 10K data size with Max headers of 52, + 74 Protocol bytes
                                ; 10K data + 52 Header + 74 protocol

OpnNumTxBuffer  equ     02      ; Number of Transmit Buffers The Most you should have is 2.
                                ; Default to two, in the hope that a Turbo Token-Ring Card will
                                ; actually use both buffers.  In Reality, only one transmit buffer will
                                ; be used with the current firmware in most adapters available today.

;-------------------------------------------------------------------------
;       DIR.Open.Adapter  Structure
;-------------------------------------------------------------------------

DirOpenAdapterStructure struc                           ; equiv. 386 OPA Structure
        OpenCommand             db      03h                     ; 00 Command Code
        OpenReserved11          db      00                      ; 01 RESERVED
        OpenReturn              db      0FEh                    ; 02 Return Code -- Not Documented in Token-Ring
                                                                ;    Net. Adpt Interface Manual Ch. 7
        OpenReserved25          db      5 dup (?)               ; 03 RESERVED
        OpenOptions             dw      OpnOptNoSoftError       ; 08 Open Options    (Auto High-Low Placement)
        OpenNodeAddress         db      6 dup (?)               ; 10 Node Address
        OpenGroupAddress        db      4 dup (00)              ; 16 Group Address
        OpenFunctionalAddress   db      4 dup (00)              ; 20 Default Functional Address is no Multicasting
        OpenNumRxBuf            db    HIGH OpnNumRxBuffer       ; 24 Number of Receive Buffers (word Value)
                                db    LOW  OpnNumRxBuffer       ;
        OpenLengthRxBuf         db    HIGH OpnDefaultRxBufSize  ; 26 Length of Receive Buffers  (word Value)
                                db    LOW  OpnDefaultRxBufSize
        OpenLengthTxBuf         db    HIGH OpnLen4KTxBuf        ; 28 Length of Tansmit Buffers  (word Value)
                                db    LOW  OpnLen4KTxBuf
        OpenNumTxBuf            db      OpnNumTxBuffer          ; 30 Number of Transmit Buffers
        OpenReserved31          db      00                      ; 31 RESERVED
        OpenMaxSap              db      00                      ; 32 Max Num. of SAPs  was 1,
                                                                ;    Not Supported in This Driver (MSM not supportive)
        OpenMaxSta              db      00                      ; 33 Max Num. of Link Stations
        OpenMaxGSap             db      00                      ; 34 Max Num. of Group SAPs
        OpenMaxGMem             db      00                      ; 35 Max Member Per Group SAP
        OpenT1TickOne           db      05                      ; 36    1 -   2  Seconds
        OpenT2TickOne           db      01                      ; 37    5 -  10  Seconds
        OpenTiTickOne           db      25                      ; 38  200 - 400  Milliseconds
        OpenT1TickTwo           db      25                      ; 39    1 -   2  Seconds
        OpenT2TickTwo           db      10                      ; 40   40 -  80  Milliseconds
        OpenTiTickTwo           db      125                     ; 41  400 - 800  Milliseconds
        OpenProductID           db      'TokenRing ODI v'       ; 42  Product ID with Auto Version Control
                                db      '0' OR MajorVersion,'.','0' OR (MinorVersion/10)  ; (18 characters required)
DirOpenAdapterStructure ends                                    ; 60 DIR.Open.Adapter SRB end point

;*************************************************************************
;       Define Structure Memory Allocations
;*************************************************************************
        even
DIROpenAdapter  DirOpenAdapterStructure <>      ; DIR.Open.Adapter SRB Image


;*************************************************************************
; DriverISR     Interrupt Vector Table  ORDER MUST NOT CHANGE
;
;       Element:        Offset for Service Routine
;                       Interrupt Mask

        even
Dispatch        label   word
        dw      AdapterCheckBit * 100h + ErrorIntBit    ; AdapterCHeck bits
        dw      ASBFreeBit     * 100h                   ; ASB Free
        dw      SRBResponseBit * 100h                   ; SRB Response
        dw      ISRPoddBridge * 100h                    ; Fast Path Tx Complete
        dw      ARBCommandBit  * 100h                   ; ARB Request
        dw      SSBResponseBit * 100h                   ; SSB Response
        dw      ISRPTimerIntBit                         ; TVR Expiration
        dw      0FFFFh                                  ; Dummy Routine

;The Vectors themselves
; starts at
        dw      offset CGroup:ChkAdapterInt             ; Adapter Check (Hardware or Software)
        dw      offset CGroup:ASBFreeInt                ; ASB Free
        dw      offset CGroup:SRBResponseInt            ; SRB Response
        dw      offset CGroup:FastPathTxComplete        ; Fast Path Tx Complete
        dw      offset CGroup:ARBReqInt                 ; ARB Request
        dw      offset CGroup:SSBResponseInt            ; SSB Response
        dw      offset CGroup:TimerInt                  ; Timer Int; Reset
        dw      offset CGroup:Dummy_Rtn                 ; End Of Table; Ignore the Interrupt

;******************************************************************************
;******************************************************************************
;                    D R I V E R   R O U T I N E S
;******************************************************************************
;
;*******************************************************************************
;       DriverManagement          ---Not used .... Stubbed here for the MSM
;       DriverChangeLookAheadSize ---Not used .... Stubbed here for the MSM
;       DriverPoll                ---Not used .... Stubbed here for the MSM
;*******************************************************************************

DriverManagement:
DriverChangeLookAheadSize:
DriverPoll:
        ret

;*******************************************************************************
;       PagingSet -- Sets up the proper page for the Shared Ram
;*******************************************************************************
;==============================================================================
; generate Relative offset for Shared Memory & Update Shared RAM Registers
;
; Entry         AX = Desired Offset, eg. SRBAddress
;
; Exit          ES:DI   Selector : Offset for Relative Shared RAM
;               AH      New Page Register Value
;               BH      Old Page Register Value
;
; Alters AX, BX, ES, DI,   Interrupts -Don't care


PagingSet       proc    near
        mov     es, MMIOSegment         ; get MMIO Segment
        mov     bh, ah                  ; BH = New page
        xchg    bh, es:SRPReven         ; set Current Page, save old one = BH
        and     ah, PageSizeMask        ; get Relative Page
        mov     es, SharedRamSegment    ; get Shared RAM Segment
        mov     di, ax                  ; ES:DI := Shared Ram @ for (address from AX)
        ret
PagingSet       endp

;------------------------------------------------------------------------------
;       DriverPromiscuousChange
;
;       Called by the MSM when it wants us to set/reset Promsicuous Mode
;
;       assumes:        CX      Zero            Disable Promiscuous Mode
;                               Non-Zero        Enable Promiscuous Mode
;                       ds      is set to CGroup
;                               CLD has been executed
;                               Interrupts are disabled
;
;       returns:        Nothing need be preserved
;
; Here only for linker's sake.
;
;------------------------------------------------------------------------------

DriverPromiscuousChange proc    near

        mov     ax, LSLERR_BAD_COMMAND          ;not supported
        or      ax, ax
        ret

DriverPromiscuousChange endp



;*******************************************************************************
;******************************************************************************
;       DriverMulticastChange
;*******************************************************************************
;******************************************************************************
;
;       Called by the MSM when changes have occurred to our
;       multicast address list.
;
;       assumes:        DS :    CGroup
;                       DX:AX   New Token-Ring Functional Address
;                       CLD has been executed
;                       Interrupts are disabled
;
;       returns:        Nothing need be preserved
;

DriverMulticastChange   proc    near
        mov     word ptr FuncAddr, dx                   ; DX: High word Functional address
        mov     word ptr FuncAddr + 2, ax               ; AX: Low  word functional Address
        or      InterruptStateFlag, DIRSetFuncAddr      ;  Set Command Flag
        jmp     ChkNxtSRBCmd                            ; Process Set.Func.Addr and return from there

DriverMulticastChange   endp

;*******************************************************************************
;******************************************************************************
;       DriverShutdown
;*******************************************************************************
;******************************************************************************
;
;       Called by the MSM when it wants us to shut our board down.
;
;       assumes:        ds      is set to CGroup
;                       cx      has the shutdown flag as doc. in ODI spec.
;                               =0 if perm shutdown
;                               !0 if just temp shutdown
;                       CLD has been executed
;                       Interrupts are disabled
;
;       returns:        Nothing need be preserved
;

DriverShutdown  proc    near

;-------------------------------------------------------------------------
;               Shut Down the card --- Disable Interrupts
;-------------------------------------------------------------------------

        mov     InterruptStateFlag, IgnoreAllInterrupts + AdapterShutdownBit  ; Lets forget any outstanding Ints
        mov     CompletionQHead, 0                      ; Disable Fast Path.
        sti                                             ; This will clear outstanding (spurious) Ints
        mov     es, MMIOSegment                         ; ES: MMIO
        mov     byte ptr es:ISRPResetEven, NOT (NMIIntControlBit+InterruptEnableBit)
        and     InterruptStateFlag, NOT (DIRReadLoginOrder + DIRSetFuncAddr + SRBBusy)
        cli                                             ; hold off on any more ints

;-------------------------------------------------------------------------
;               Check to see if this is a permenant thing
;-------------------------------------------------------------------------

        or      cx, cx
        jnz     ClearQBrothers          ; Just Clear the Q's

;-------------------------------------------------------------------------
;                    Cancel outstanding AES events
;-------------------------------------------------------------------------

        mov     si, offset CGroup:NICAES
        push    cs
        pop     es                              ; ES:SI -> AESECB
        mov     bx, MLIDSUP_CANCEL_AES_EVENT
        call    LSLSupport                      ; Cancel it

;-------------------------------------------------------------------------
;               Have the Card Withdraw from the ring and hang himself
;-------------------------------------------------------------------------

        mov     cl, SRBCmdDIRCloseAdapter       ; Close the Adapter
        call    IssueSRBCmd
        mov     dx, AdapterReset                ; Hold the Adapter Still
        or      InterruptStateFlag, AdaptHeldReset
        out     dx, al                          ; AX doesn't matter

DriverShutdown  endp                    ;Falls through to clear the Q Brothers

;-------------------------------------------------------------------------
;               Clear the Q brothers
;-------------------------------------------------------------------------
;       Assumes :       ds:CGroup
;                       Interrupts are Disabled.
;       Uses    :       SI, AL
;       Destroys:      All Registers except DS & BP
;       Resets the MSM TXFreeCount

ClearQBrothers  proc    near

;-------------------------------------------------------------------------
;               Clear the Xmit Q
;-------------------------------------------------------------------------

        mov     si, XmitQueHead         ; get ptr's to TCB

ClrQBroClearXmitQue:
        or      si, si                  ; Error checking
        jz      ClrQBroDoneXmitQue

        push    [si].TCBNxtLnk          ; Save Next Waiting TCB (may be none)
        call    MSMSendComplete         ; Return TCB to MSM
        pop     si                      ; SI: Next TCB (May be none)
        jmp     ClrQBroClearXmitQue     ; Check for more TCB's

ClrQBroDoneXmitQue:                                     ; SI -> 0000
        mov     XmitQueHead, si                         ; Reset the Head to Zero
        mov     TooBusyPtr, si                          ; Reset the Head to zero
        mov     QueTail, offset CGroup:XmitQueHead      ; Reset the Tail to the Beginning
        mov     al, DriverMaxActiveTx                   ; Reset MSMTxFreeCount
        mov     MSMTxFreeCount, al
        ret

ClearQBrothers  endp

;*******************************************************************************
;*******************************************************************************
;       DriverReset
;*******************************************************************************
;*******************************************************************************
;
;       Called by the MSM when it wants us to reset our board.
;
;       assumes:        ds      is set to CGroup
;                       CLD has been executed
;                       Interrupts are disabled
;
;       returns:        Nothing need be preserved
;

DriverReset     proc    near

        inc     ResetNICCount                 ; Keep Track of how often
        mov     AdaptRetryCnt, ResetRetryDef  ; Try a few times before giving up

RstResetNic:
        or      InterruptStateFlag, IgnoreAllInterrupts + AdapterShutdownBit  ; ensure We Are ShutDown
        sti
        mov     es, MMIOSegment                 ; ES -> MMIO area
        mov     byte ptr es:ISRPResetEven, NOT (NMIIntControlBit+InterruptEnableBit)

;-------------------------------------------------------------------------
;               Pull the Adapter Reset Line
;-------------------------------------------------------------------------

        cli                                             ; IF = 0 ==> No Interrupts
        mov     dx, AdapterReset                        ; Adapter Reset port
        or      InterruptStateFlag, AdaptHeldReset      ; Adapter Being Held Reset
        out     dx, al                                  ; Hold The Adapter Reset For A While

;-------------------------------------------------------------------------
;               Set up to Wait For At Least 50 Mills
;-------------------------------------------------------------------------

        mov     bp, 100
        call    WasteTime               ; Waits time in

;-------------------------------------------------------------------------
;               Have the Q's Cleaned
;-------------------------------------------------------------------------

        cli                                             ; Hold Ints while cleaning.
        call    ClearQBrothers

;-------------------------------------------------------------------------
;               Pull the Adapter Release
;-------------------------------------------------------------------------

        mov     dx, AdptrRelease_SetupRead2     ; DX = Release Adapter Reset PIO @
        out     dx, al                          ; Release Adapter Reset   AL = Don't Care
        mov     InterruptStateFlag, (IgnoreAllInterrupts+WaitingIntBit+AdapterShutdownBit)

;-------------------------------------------------------------------------
;               Set up The Adapter for Paging if specified
;-------------------------------------------------------------------------

        mov     al, PageSizeMask ; Get PageSizeMask
        mov     es, MMIOSegment  ; Restore ES: MMIO Segment
        inc     al               ; Default Page Size Mask? (0FFh + 1 = 0)
        jz      RstNoPaging      ; nz => not 16k ram page supported

        mov     byte ptr es:SRPReven, MMIOPagingActiveFlag      ; Set SRPR bits  ES:= MMIO Base

RstNoPaging:

;-------------------------------------------------------------------------
;               Set the RRReven Register (after the Adapter Reset cleared it)
;-------------------------------------------------------------------------

        test    MSMSystemFlags, MCAFlag         ; Check to see if MCA bus (PS2)
        jnz     RstSetISRPeven                  ; on Micro-channel RRReven is RESERVED

        mov     cx, SharedRamSegment            ; CX = Shared Ram Address
        mov     es:RRReven, ch                  ; Write Segment Value out to RRReven. ES:=MMIO Base

;-------------------------------------------------------------------------
;               Set ISRPeven    InitEnable Bit (bit 6)
;-------------------------------------------------------------------------

RstSetISRPeven:
        mov     byte ptr es:ISRPSeteven, NMIIntControlBit+InterruptEnableBit  ; ES:=MMIO Base
        or      InterruptStateFlag, WaitingIntBit                             ; Set the Bit to watch

;-------------------------------------------------------------------------
;       Wait For At Least 72 ticks (4 seconds) for Interrupt to occur
;-------------------------------------------------------------------------

        mov     bp, 4*1000                      ; Wait at least 4 seconds
        call    WasteTime                       ; waits Value in bp
        jz      RstInitCompletedInterrupt

RstErrorResetNICJmp:
        jmp     RstErrorResetNIC                ; Card has Failed to initialize

;-------------------------------------------------------------------------
;               Read WRBR to Get Initial location of SRB
;               After Int has occurred see ISRPodd bit 5
;-------------------------------------------------------------------------

RstInitCompletedInterrupt:               ; Internal Initialization completed, ES:=MMIO base
        mov     bx, word ptr es:WRBReven ; Read the Write Region Base
        xchg    bh, bl                   ; Adjust for IBM Format
        mov     word ptr SRBAddress, bx  ; bx: save Shared RAM SRB Relative @
        mov     es:SRPReven, bh          ; set Current Page   ES:=MMIO Base
        and     bh, PageSizeMask         ; get Relative Page
        mov     dl, ES:AIPTotalRam       ; DL: AIPTotalRam (low nibble valid) While ES is Still MMIO
        mov     es, SharedRamSegment     ; ES: Shared Ram Segment

        cmp     byte ptr es:[bx].InitSRBCmd, SRBCmdDIRInitComplete      ; Initialize Complete ?? ES:BX ->SharedRam SRB
        jz      RstNICInitComplete

        dec     AdaptRetryCnt            ; Try a few times to get it going
        jz      RstErrorResetNICJmp      ; Out of Retries? (jump)

        jmp     RstResetNic              ; Try again

;-------------------------------------------------------------------------
;       Check for any errors in the Bring-up Code field
;-------------------------------------------------------------------------

RstNICInitComplete:                     ; ES : Shared Ram Segment;   BX : SRB Relative Address
        cmp     word ptr es:[bx].InitSRBError, 0000h    ; IF 00 ==> Success
        jnz     RstErrorResetNICJmp                     ; DIR.Initialize FAILED

;-------------------------------------------------------------------------
; Shared RAM Works fine,  --- Check For Zeroing Top 512 Bytes
;-------------------------------------------------------------------------
                                ; Note: Zeroing must take place after Card is Init,
                                ; but before Dir.Open.Adapter Command.
        xor     ax, ax          ; AX = 0000
        and     dl, 0Fh         ; Strip out low nibble from word DX := AIPTotalRam
        sub     dl, 0Bh         ; DL = Top 512 Reserved ??
        jnz     RstChkFastPathSupport

        lea     di, (SIZE DIROpenAdapter)+[bx]  ; ES:DI = Top of SHared Ram
        mov     cx, 512/2                       ; CX = Word Count
 rep    stosw                                   ; Zero Top 512 bytes of Shared RAM

;-------------------------------------------------------------------------
;               Initialize Fast Path if it is supported.
;-------------------------------------------------------------------------

RstChkFastPathSupport:
        test    InitStatus, INIT_FAST_PATH      ; Is Fast Path Supported?
        jnz     RstFastPathInit                 ; Code this way to keep jumps in range.
RstNICOpenAdaptJMP:
        jmp     RstNICOpenAdapt                 ; No - Run in normal mode then.

RstFastPathInit:
        and     InitStatus, NOT INIT_FAST_PATH
        cmp     RAMSize, SHARED_RAM_8K                  ; 8K RAM size?
        je      RstNICOpenAdaptJMP                      ; Jump if so.

        and     cs:InitStatus, NOT INIT_FAST_PATH
        mov     ax, FastPathRAMSize
        xchg    al, ah
        mov     cs:ConfigRAMSize, ax
        lea     si, cs:ConfigFastPath
        mov     es, cs:SharedRamSegment
        mov     ax, word ptr cs:SRBAddress
        and     ah, cs:PageSizeMask
        mov     di, ax
        push    di
        mov     cx, SIZE ConfigFastPathStructure / 02
        rep     movsw
        pop     di
        or      cs:InterruptStateFlag, WaitingIntBit
        push    es
        mov     es, cs:MMIOSegment              ; ES: MMIO base
        mov     byte ptr es:ISRASetodd, CommandInSRBBit
        pop     es

        mov     bp, 3*1000                      ; 3 seconds

RstNICConfigFastPathLoop:
        call    WasteTime                       ; Wait for Fast Path
        jnz     RstNICOpenAdapt

        cmp     es:[di].FPRRetcode, 0           ; Fast Path enabled?
        je      RstNICConfigFastPathDone

        or      cs:InterruptStateFlag, WaitingIntBit
        sub     bp, ax                          ; BP = Time left to wait.
        jnz     RstNICConfigFastPathLoop        ; Still some time left Fine wait some more.

RstNICConfigFastPathDone:
        or      cs:InitStatus, INIT_FAST_PATH

        xor     dx, dx
        mov     ax, MaxTxBuf                    ; Max size TX allowed.
        mov     cx, MaxFrameSize                ; get Max. User Data Size from Config0 Table
        div     cx
        cmp     al, 4                           ; Max allowed by TSM is 4.
        jbe     RstSetDriverMaxActiveTx

        mov     al, 4

RstSetDriverMaxActiveTx:
        mov     DriverMaxActiveTx, al           ; Set number of TX buffers.

        mov     ax, es:[di].FPRFastPathXmit     ; AX = Tx Offset.
        xchg    ah, al                          ; Convert to Intel format.
        mov     cs:TxPageValue, ah                      ; Save Tx page.
        and     ah, cs:PageSizeMask
        mov     cs:TxControlAddress, ax         ; Store offset to TX Control.

        mov     ax, es:[di].FPRSRBAddress       ; AX = offset of new SRB
        xchg    ah, al                          ; Convert to Intel format.
        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     es:SRPReven, ah                 ; Set new page
        mov     word ptr cs:SRBAddress, ax      ; Save offset to SRB

;-------------------------------------------------------------------------
;               Issue DIR.Open.Adapter
;-------------------------------------------------------------------------

RstNICOpenAdapt:
        mov     es, SharedRamSegment    ; ES: Shared Ram Segment
        mov     ax, word ptr SRBAddress ; get Shared RAM SRB Relative @
        and     ah, PageSizeMask        ; get Relative Page
        mov     di, ax                  ; ES:DI = Shared Ram SRB @
        mov     si, offset CGroup:DIROpenAdapter
        mov     cx, size DirOpenAdapterStructure
 rep    movsb                           ; move in the Skeleton --- Go byte at a time Watch out for IBM Byte Swap
                                        ; go Set SRB Interrupt

        mov     es, MMIOSegment                                 ; ES: MMIO base
        or      InterruptStateFlag, WaitingIntBit+SRBBusy+DirOpenInProgress+AdapterShutdownBit
                                                                ; Mark States to
        and     InterruptStateFlag, NOT (IgnoreAllInterrupts)   ; Allow the card to go
                                                                ; But not get in trouble

;-------------------------------------------------------------------------
;               Set ISRPeven    InitEnable Bit (bit 6)
;-------------------------------------------------------------------------

        mov     byte ptr es:ISRASetodd, CommandInSRBBit                      ; Tell Adapter there's work to do
        mov     byte ptr es:ISRPSeteven, NMIIntControlBit+InterruptEnableBit ; ES:=MMIO Base - Enable Int.

;-------------------------------------------------------------------------
;       Allow the Driver ISR to Handle the Open Completion
;       and Ensure to reset any outstanding ARB Commands
;-------------------------------------------------------------------------

        sti                                ; Allow them to happen
        mov     es:ISRASetodd, ARBFreeBit  ; Reset outstanding ARB's
        ret                                ; Wait till Interrupted.... Thanks

;-------------------------------------------------------------------------
;       Error Exit Point
;-------------------------------------------------------------------------

RstErrorResetNIC:
        cli                                     ; Hold off on the ints.
        or      InterruptStateFlag, DIRSetFuncAddr+AdapterShutdownBit   ; Ensure we reset the Functional Address
        mov     es, MMIOSegment                 ; Set up for Clearing ARB's                                                             ; and that we are marked down as dead.
        mov     es:ISRASetodd, ARBFreeBit
        ret

DriverReset     endp

;*********************************************************************
;       WasteTime Procedure to delay some time or until an int occurs
;*********************************************************************
;       Assumes ds:CGroup               Returns: Z - if Int occured
;               bp: Time to wait                NZ - Time Elapsed
;       Wastes: AX, BX, CX, DX          All others Preserved
;       Interrupts are left enabled             AX has Time Elapsed in Milliseconds

WasteTime       proc    near
        mov     bx, MLIDSUP_GET_INTERVAL_MARKER ; Get interval value
        sti
        call    LSLSupport                      ; BX = MLIDSUP_Get_Interval_Marker (5)
        mov     cx, ax                          ; store first Value

WaitingLoop:
        test    InterruptStateFlag, WaitingIntBit
        jz      ExitWaitingLoop

        call    LSLSupport              ; BX = MLIDSUP_Get_Interval_Marker (5)
        sub     ax, cx
        cmp     ax, bp                  ; Have we waited long enough
        jbe     WaitingLoop             ; Wait for Adapter to Settle

        or      bp, bp                  ; Set the NZ Flag => no interrupts

ExitWaitingLoop:
        ret

WasteTime       endp

;*********************************************************************
;               Interrupt Procedures for DriverISR
;*********************************************************************

;*********************************************************************
;       ChkAdapterInt   --- Adapter Check error Handler
;*********************************************************************

ChkAdapterInt   proc    near    ; Adapter Check (Hardware or Software)
        cli
        inc     AdapterCheckErrCounter          ; Count the error
        mov     cl,0Fh                          ; Ensure not permenant
        jmp     DriverReset                     ; Shut us down

ChkAdapterInt   endp


;*********************************************************************
;       ASBFreeInt      ---  Token-Ring ASB Response Interrupt
;*********************************************************************

ASBFreeInt      proc    near                            ; ASB Free
        inc     UnexpectedASBFree                       ; keep count
        mov     ax, word ptr ASBAddress                 ; get Shared RAM ASB @

        call    PagingSet

        mov     byte ptr es:[di].ASBRetCode, 0FFh       ; Reset Return Code
        ret
ASBFreeInt      endp

;*****************************************************************************
;       SRBResponseInterrupt    ---   Token-Ring SRB Response Interrupt
;*****************************************************************************
;       AX: Don't care                  BX: don't care
;       DX: Routine @                   CX: ISRPeven --- CL: Even  CH: Odd
;       SI: Don't Care                  DI:
;       DS: CGroup                      ES: Don't Care
;
;       Return:
;       AX:
;       DX:
;       SI:
;       DS: CGroup

SRBResponseInt  proc   near                     ; SRB Response
        cli                                     ; IF = 0 ==> No Interrupts
        and     InterruptStateFlag, NOT SRBBusy ; Clear SRB bit

;-------------------------------------------------------------------------
;               Set the Shared Ram Page, get DS:SI -> SRB
;-------------------------------------------------------------------------

        mov     ax, word ptr SRBAddress ; AH = Current Page x 16 K (AL)
        mov     es, MMIOSegment         ; get MMIO Segment
        mov     es:SRPReven, ah         ; Set new page
        and     ah, PageSizeMask        ; get Relative Page
        mov     ds, SharedRamSegment    ; get Shared RAM Segment
        mov     si, ax                  ; DS:SI := Shared Ram @ for (address from AX)

;-------------------------------------------------------------------------
;Check the Command Code---- Only SRBCmdDIRReadLog & Dir.Open.Adapter are valid
;-------------------------------------------------------------------------

        mov     al, byte ptr [si].SRBSSBCommand ; AL = Command Code
        cmp     al, SRBCmdDIROpenAdapter        ; Is this Int to open the adapter?
        je      DIROpenAdaptRoutine             ; Not a Open,then check readlog

        cmp     al, SRBCmdDIRReadLog            ; Is this Int to get the counters?
        jne     ParseSRBCommandExit             ; Not a Readlog, then exit

;-------------------------------------------------------------------------
;       DIRReadLogRoutine       DIR.Read.Log --- SRB Interrupt
;-------------------------------------------------------------------------
        ; DS:SI => Shared RAM SRB @
        ; ES: -> MMIOSegment

DIRReadLogRoutine:
        xor     ax, ax                          ; Clear AX
        mov     byte ptr [si] + 11, al  ; ie. reset Counter Byte 05 of SRB to add a 0 to Adapter Error Counters
        xchg    al, byte ptr [si] + 13  ; Get the Rcv Congestion Info
        add     word ptr cs:DriverStatRxOverflow, ax ; Track Congestion as Overflow
        adc     word ptr cs:DriverStatRxOverflow + 2, 0
        add     si, 06                          ; move SI ptr. start of error values
        mov     cx, 11                          ; CX = Loop Counter -- 2 bytes are skipped
        mov     di, offset CGroup:LineErrors    ; ES:DI = Custom Counters

DIRReadLogLoop:
        lodsb                           ; AL = Next Counter Value DS:SI
        add     word ptr cs:[di], ax    ; Update Error Counters
        add     di, 04                  ; update CS:DI by 4 -> next counter
        loop    DIRReadLogLoop          ; Loop for Eleven Counters

        and     cs:InterruptStateFlag, NOT DIRReadLoginOrder    ; reset flag

ParseSRBCommandExit:
        mov     ax, cs
        mov     ds, ax                  ; Restore DS : CGroup

        ;These Commands should never be called, but if they are just return
DLCStatsRoutine:        ; 00 ==> 1E ==> DLC.Statistics
DLCCloseSAPRoutine:     ; 01 ==> 16 ==> DLC.Close.SAP
DLCOpenSAPRoutine:      ; 02 ==> 15 ==> DLC.Open.SAP
DLCResetRoutine:        ; 03 ==> 14 ==> DLC.Reset
        ret


;-------------------------------------------------------------------------
;       Dir.Open.Adapter Error has Occured and we are still in DriverInit.
;-------------------------------------------------------------------------

NotOutofDriverINIT:                                     ; Store Error Code and Return
        mov     al, byte ptr [si].OpenReplyExtendedError + 1    ; Get the Extended Error Code
        mov     byte ptr cs:DirOpenRetCode+1, al        ; Extended Error Code
        mov     cs:InterruptStateFlag, (DIRSetFuncAddr+AdapterShutdownBit+IgnoreAllInterrupts)
        push    cs
        pop     ds                                      ;Restore DS
        ret

;-------------------------------------------------------------------------
;       DIROpenAdaptRoutine     Dir.Open.Adapter --- SRB Interrupt
;-------------------------------------------------------------------------
        ; DS:SI => Shared RAM SRB @
        ; ES: -> MMIO Segment

DIROpenAdaptRoutine:                                    ; 05 ==> 03 ==> DIR.Open.Adapter
                                                        ; DS:SI -> SRBaddress in Shared RAM
        mov     al, byte ptr [si].OpenReplyRetCode      ; Return Code = To be/is in Processing (0FEh)
        cmp     al, 0FEh                                ; 0FEh => in Process please wait
        je      DIROpenAdaptRoutine                     ; Loop till Cmd is finished processing

        and     cs:InterruptStateFlag, NOT DirOpenInProgress ; Mark that DirOpenHas occurred.
        mov     byte ptr cs:DirOpenRetCode, al          ; Save ReturnCode.
        or      al, al                                  ; Return Code = Successfull
        jz      AdapterIsOpen                           ; Z - We have a good Return Code

;-------------------------------------------------------------------------
;       Non Zero => Error Code handler
;-------------------------------------------------------------------------

        test    cs:DriverStateFlag, DriverInitFlagBit   ; Are we still in Driverinit?
        jnz     NotOutofDriverINIT                      ; Jump to a Save ErrorCode and ret if we are.

        cmp     al, 07h                                 ; Check if we have an extended error
        jne     NotExtendedErrorCode

;-------------------------------------------------------------------------
;               Check for Remove From Ring Error  ( 0xAh )
;-------------------------------------------------------------------------

        mov     al, byte ptr [si].OpenReplyExtendedError + 1    ; Get the Extended Error Code
        mov     cl, 4
        shl     al, cl                                  ; Peel off the phases
        cmp     al, 0A0h                                ; 'xA' is a Remove from Ring Code
        jnz     NotExtendedErrorCode

;-------------------------------------------------------------------------
;       Remove From Ring or Worse has been given --> Go Shut Ourselves down
;-------------------------------------------------------------------------

        push    cs
        pop     ds              ; Must set DS to CGroup
        jmp     HSMShutDownMSM  ; Shut ourselves down

;-------------------------------------------------------------------------
;       Non Fatal Error Exit Point
;-------------------------------------------------------------------------

NotExtendedErrorCode:
        cli                                     ; Hold off on the ints.
        or      cs:InterruptStateFlag, (DIRSetFuncAddr+AdapterShutdownBit+IgnoreAllInterrupts)
                                                ; Ensure we reset the Functional Address
                                                ; and that we are marked down as dead.

        push    cs
        pop     ds                              ; Restore DS : CGroup
        jmp     DriverReset                     ; Go Try again to start

;-------------------------------------------------------------------------
;       Store Addresses given from the SRB
;-------------------------------------------------------------------------

AdapterIsOpen:
        lea     si, [si].OpenReplyASBAddress
        lea     di, cs:ASBAddress
        mov     cx, 4                           ; Lets get four Register's Addresses

GetAddresses:
        lodsw
        xchg    ah, al                          ; Allow for IBM format
        mov     cs:[di], ax                     ; Store Address away
        inc     di                              ; move pointer 1 word
        inc     di
        loop    GetAddresses                    ; Get all 4 addresses

        test    cs:InitStatus, INIT_FAST_PATH   ; Fast Path Mode?
        je      NotRunningFastPath

        mov     al, cs:TxPageValue
        mov     es:SRPReven, al                 ; Set new page

        mov     di, cs:TxControlAddress

        push    es
        mov     es, cs:SharedRamSegment         ; ES:DI -> Fast Path Tx Control area.
        mov     ax, es:[di].FPCompletionQTail   ; Save completion Queue Tail.
        mov     cs:CompletionQHead, ax
        mov     cs:TxCorrelator, 0                      ; Initialize correlator.
        pop     es

NotRunningFastPath:

;-------------------------------------------------------------------------
;       Arm on-card timer.  This will allow ucode to remove adapter
;       from the ring in the event of a machine hang or CTRL-ALT-DEL.
;       Set the adapter to interrupt every 2.55 sec (0FFh x 10msec)
;-------------------------------------------------------------------------

        mov     byte ptr es:TVRodd, 0FFh
        mov     byte ptr es:TCReven, IBMPCTimerInterruptMaskBit + IBMPCTimeCountGateBit+IBMPCTimerReloadModeBit

;-------------------------------------------------------------------------
;               Mark the Adapter open for business
;-------------------------------------------------------------------------

        cli                                     ; Hold off on the ints.
        and     cs:InterruptStateFlag, NOT (AdapterShutdownBit + IgnoreAllInterrupts)

;-------------------------------------------------------------------------
;       Set up a Call to the DirReadLog Routine
;-------------------------------------------------------------------------

        or  cs:InterruptStateFlag, WaitingIntBit+DIRSetFuncAddr         ; Clear StateFlag, and Set Flag for Multicasting
        mov     es:ISRASetodd, ARBFreeBit                               ; Ensure to Reset any Outstanding ARB Commands
        jmp     ParseSRBCommandExit

SRBResponseInt  endp            ; Token-Ring SRB Response Interrupt

;*****************************************************************************
;       Adapter TVR Timer Interrupt Service Routine.
;
;       This routine services the adapter's timer interrupt.  This will be
;       called every 2.55 seconds.  We are servicing the interrupts to
;       provide a way to realize that the driver is still present.  If a
;       warm-boot (CTRL-ALT-DEL) or a machine hang occurs, the ucode will
;       use the unserviced timer interrupts as a indication to close the
;       adapter on the ring.  This will prevent the adapter from placing
;       Receive Congestion Error Mac frames while it is inserted on the
;       ring without a driver.
;
;       It is not necessary to reset the ISRPTimerInt bit here as this
;       is always done immediately before exiting the DriverISR routine.
;
;*****************************************************************************

TimerInt        proc    near

        mov     es, MMIOSegment
        mov     byte ptr es:ISRPResetEven, NOT ISRPTimerIntBit  ; Reset Timer
        ret

TimerInt        endp

;*****************************************************************************
;       ARBReqInt --- Token-Ring ARB Request Interrupt
;*****************************************************************************
;       Assume:  DS: CGroup

ChkARBTxDataReqJMP:                             ; Jumping point
        jmp     ChkARBTxDataReq

ARBReqInt       proc    near                    ; Token-Ring ARB Request Interrupt
        cli
        mov     cx, ARBSize                     ; get ARB Size  (# bytes)
        mov     ax, word ptr ARBAddress         ; AH = Current Page x 16 K (AL) for ARB
        mov     es, MMIOSegment                 ; get MMIO Segment
        mov     es:SRPReven, AH                 ; set Current Page
        and     ah, PageSizeMask                ; get Relative Page
        mov     ds, SharedRamSegment            ; get Shared RAM Segment
        mov     si, ax                          ; DS:SI := Shared Ram @ for ARB
        mov     bp, cs                          ; BP:CGroup
        mov     es, bp                          ; ES:CGroup
        mov     al, byte ptr [si].ARBCommand    ; AL = Command Code
        mov     di, offset CGroup:ARBSaveArea   ; ES:DI ARB Save Area
        FastMove                                ; Save the ARB image
        mov     ds, bp                          ; Restore DS: CGroup
        mov     es, MMIOSegment                 ; Restore ES: MMIO
        mov   byte ptr es:ISRASetodd,ARBFreeBit ; Signal the card the ARB is free to use again
        mov     ARBSaveArea.ASBRetCode, cl      ; Return Code = 00
        cmp     al, ARBCmdRxData                ; Check first if it is a receive Data Cmd
        jnz     ChkARBTxDataReqJMP              ; No, skip to check for ARB Next Command

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
; Process ARB 81h -- Receive Data               ; Ints are off (CLI)
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
ReceivedData:
        mov     ax, word ptr ARBSaveArea.ARBBufAddr     ; AX Offset to the Receive Buffer onthe Card
        xchg    ah, al
        mov     es:SRPReven, ah         ; set Current Page
        and     ah, PageSizeMask        ; get Relative Page
        mov     es, SharedRamSegment    ; get Shared RAM Segment
        mov     di, ax                  ; ES:DI := Shared Ram @ for (address from AX)

;-------------------------------------------------------------------------
;               Set Look ahead pointers, and Get the RCB
;-------------------------------------------------------------------------
        inc     di
        inc     di                                      ;Skip 1st.buffer Reserved Field (2 bytes)
        mov     word ptr ARBRxBufPtr+2, es              ;save ES:DI ptr to Buffer
        mov     word ptr ARBRxBufPtr, di
        lea     si, byte ptr es:[di].RxBFrameData       ;ES:SI = Shared RAM Receive Buffer's data
        mov     ax, word ptr ARBSaveArea.ARBFrameLen    ;AX = Frame Length in High-Low Format
        xchg    ah, al                                  ;AX = Entire Frame Length
        mov     bp, ax                                  ;Set up for MSM call with the length
        xor     ax, ax                                  ;No Rx Errors given
        sti
        call    MSMGetRCB                               ;Get Rx Buffer from MSM
        jnz     GiveASBResponseJmp                      ;Z - Load RCB  NZ -> Packet Not Wanted

;-------------------------------------------------------------------------
;               Initalize Registers, setup to load RCB
;-------------------------------------------------------------------------
;       DS - CGroup,                AX - # Bytes to skip, Ints disabled
;       BP - # bytes to be copied,  ES:SI - Ptr. to RCB
;-------------------------------------------------------------------------
        cli                                     ; Hold off the Ints
        mov     word ptr ARBRCBPtr, si          ; keep copy of pointer to RCB
        mov     word ptr ARBRCBPtr + 2, es
        mov     ARBSaveArea.ASBRetCode, 00h     ; ASB Return Code = OK
        lea     di, es:[si].RCBFrag0Addr        ; get ptr. to 1st. Frag
        mov     dx, es:[si].RCBFrag0Len         ; Get the length of the frag
        mov     word ptr ARBRCBmarker, di       ; Keep track where we are in RCB
        mov     word ptr ARBRCBmarker + 2, es   ; and the segment too
        add     word ptr ARBRCBmarker, 6        ; move down list to next Frag
        les     di, es:[di]                     ; ES:DI -> RCB Frag Space
        lds     si, ARBRxBufPtr                 ; DS:SI -> Adapter's buffers
        mov     cx, [si].RxBBufLen              ; CX: # of Bytes to move from this first buffer
        xchg    ch, cl                          ; allow IBM format High-Low
        sub     cx, ax                          ; CX: Total Number of Bytes left to move 1st buffer
        cmp     cx, bp                          ; Check to see if #bytes wanted <= #bytes available in RXbuf
        ja      AdjustMoveCount                 ; if so jump to adjust BP to get only wanted bytes

HaveAdjustedMoveCount:                  ; BP # bytes left to move
        mov     bx, si                  ; save si -> BX: RxB offset
        add     si, ax                  ; add # bytes to skip

;-------------------------------------------------------------------------
;               Have the # bytes to move  (CX)
;-------------------------------------------------------------------------
HaveCounttoMove:
        sub     bp, cx                  ; decrem. BP by CX; BP: # bytes left to get
        lea     si, [si].RxBFrameData   ; get ptr. to start of data

;-------------------------------------------------------------------------
;               Check for overruns on buffer Frags
;-------------------------------------------------------------------------
MoveAdaptersData:
        jcxz    NeedNextAdapterBuffer   ; Z => go get next Rxb

        or      dx, dx                  ; Test if DX Zero => No room left in FRAG
        jz      NeedNextRCBFrag         ; Jump to get the next Frag

        cmp     cx, dx                  ; Test enough room for data
        ja      FillRCBFrag             ; CX > DX ==> no, move dx bytes and get new RCBFrag

;-------------------------------------------------------------------------
;                       All Set to Move the Data
;-------------------------------------------------------------------------
        sub     dx, cx                  ; CX will fit in (dx) RCBFrag
        FastMove                        ; move the data into the Frag

NeedNextAdapterBuffer:
        or      bp, bp                  ; Test if finished copying
        jz      LoadedRCB               ; Got it all we're out of here

;-------------------------------------------------------------------------
;                       Get the Next Rx buffer
;-------------------------------------------------------------------------
        mov     bx, word ptr [bx]                       ;get next buffer

;-------------------------------------------------------------------------
;       perform SharRAMOffsetReg Macro but with DS:SI, BX,
;       ie. speedier leave ES & AX alone, and is set up for next loop through
;-------------------------------------------------------------------------
        xchg    bh, bl                          ; allow IBM format High-Low
        mov     ds, cs:MMIOSegment              ; get MMIO Segment
        mov     byte ptr ds:SRPReven, bh        ; set current Page (Override is Required)
        and     bh, cs:PageSizeMask             ; get relative Page
        mov     ds, cs:SharedRamSegment         ; get Segment Ptr
        mov     si, bx                          ; DS:SI -> Rx Buffer

;-------------------------------------------------------------------------
;               Get the next count from the length of the Rx buffer
;-------------------------------------------------------------------------
        mov     cx, [si].RxBBufLen              ;get # of Bytes to move
        xchg    ch, cl                          ;allow IBM format High-Low
        cmp     cx, bp                          ;Test for size to copy
        jb      HaveCounttoMove

        mov     cx, bp                          ; CX was larger, Adjust -- Only need bp Bytes
        jmp   short HaveCounttoMove

;  ----------- START OF RX JUMPING POINTS----------------
GiveASBResponseJmp:
        jmp   short GiveASBResponse

AdjustMoveCount:                                ; BP is smaller than CX
        mov     cx, bp                          ; Only need the next bp bytes
        jmp  short HaveAdjustedMoveCount

;  --------------- END OF JUMPING POINTS ----------------


;-------------------------------------------------------------------------
;       RCBFrag is nearly full, Complete filling, and get new one
;-------------------------------------------------------------------------
FillRCBFrag:
        sub     cx, dx          ; get difference CX: # bytes too many to move
        xchg    cx, dx          ; DX: #bytes left in Rxb to move, CX: #bytes in RCBfrag to fill
        FastMove                ; move the data
        xchg    cx, dx          ; reset DX, ie. need next frag, CX: #bytes left in Rxb to move

NeedNextRCBFrag:
        les     di, cs:ARBRCBmarker             ; Remember where we were
        mov     dx, es:[di].FragLen             ; get length of new Frag
        les     di, es:[di].FragAddr            ; get place to put data
        add     word ptr cs:ARBRCBmarker, 6     ; move down list to next Frag
        jmp  short MoveAdaptersData


;-------------------------------------------------------------------------
;               RCB is Loaded -- Tell MSM and Send ASB Response
;-------------------------------------------------------------------------
LoadedRCB:
        mov     ax, cs
        mov     ds, ax                  ; Restore DS to CGroup
        les     si, ARBRCBPtr           ; restore base RCB Index ES:SI -> RCB
        call    MSMRcvComplete

;-------------------------------------------------------------------------
;       Every ASB Interrupt That Needs a Response Comes here
;-------------------------------------------------------------------------
GiveASBResponse:                        ; Give ASB Response to The Adapter
                                        ; Set up the Registers once and then enter the loop
        sti                             ; Start your Interrupts
        mov     bx, word ptr ASBAddress ; BH = Current Page x 16 K (BL)
        mov     ah, bh                  ; AH = Current Page Register Value
        and     bh, PageSizeMask        ; get Relative Page
        mov     es, SharedRamSegment    ; get Shared RAM Segment
        mov     cx, ds                  ; Save DS : CGroup
        mov     ds, MMIOSegment         ; DS: MMIO Segment

waitGiveASBResponse:
        test    word ptr ds:ISRPeven, (AdapterCheckBit + ASBFreeBit)*100h + ErrorIntBit  ; Override is needed
        jnz     ARBReqIntExit                           ; Exit if any Errors

        mov     ds:SRPReven, ah                         ; set Current Page -- Override is needed
        cmp     byte ptr es:[bx].ASBRetCode, 0FFh       ; ASB Free ? (i.e. Is the card ready to here from us)
        jnz     waitGiveASBResponse                     ; wait till ASB Free Jump

        mov     di, bx                          ; ES:DI = ASB Shared Ram Address
        mov     ds, cx                          ; Restore DS: CGroup
        mov     si, offset CGroup:ARBSaveArea   ; DS:SI = ASB Skeleton @
        mov     cx, ASBSize/2                   ; ASB Defined to have 12 bytes
   rep  movsw                                   ; Build the ASB Response
        mov     es, MMIOSegment
        mov     byte ptr es:ISRASetodd, ResponseInASBBit
        ret

ARBReqIntExit:
        mov     ds, cx          ; Restore DS: CGroup
        ret

;--------------------Begining Transmit Jumping Points and an Error Counter -----------------

ChkDLCRingStatusJMP:                            ; JUMP POINT
        jmp     ChkRingStatusChg

BadCorrelatorError:
        inc     DriverStatTxMisc
        jmp   short GiveASBResponse


;----------------------End Transmit Jumping Points------------------------

;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;       Check For ARB 82h -- Transmit Data Request
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
ChkARBTxDataReq:
        cmp     al, ARBCmdTxData        ; AL = ARB 82h -- Transmit Data Request ??
        jne     ChkDLCRingStatusJMP     ; jump if NOT Transmit Data Request

;-------------------------------------------------------------------------
;               Get one off the Q
;-------------------------------------------------------------------------
        mov     si, XmitQueHead         ; get ptr's to TCB
        or      si, si                  ; Error checking
        jz      ChkARBDoneXmitQue

        mov     bx, [si].TCBNxtLnk      ; Get NEXT Waiting TCB (may be none)
        mov     XmitQueHead, bx         ; reset hold buffer to either the end, or the next TCB
        or      bx, bx                  ; check to see if Curr-Q is empty
        jnz     ChkARBDoneXmitQue       ; Jump if we don't need to reset the tail

        mov     QueTail, offset CGroup:XmitQueHead      ; Reset the Tail to the Beginning

ChkARBDoneXmitQue:                      ; SI -> TCB
        or      si, si                  ; Set Z if No TCB was Found
        jz      BadCorrelatorError      ; ERROR if End of Queue  (Z-flag if End of Q)

;-------------------------------------------------------------------------
;       Start Loading the ASB Response, and Load the Header into the DHB
;-------------------------------------------------------------------------
                        ; ES: MMIO
                        ; DS: CGroup
                        ; DS:SI = TCB,  Prepare to Move The Data

        mov     ARBSaveArea.ASBCommand, SRBCmdXMTDIRFrame       ;only type of Tx we do.
        mov     ax, word ptr ARBSaveArea.ARBBufAddr     ; get DHB address
        xchg    ah, al                                  ; Switch IBM to Intel Format

;-------------------------------------------------------------------------
;       set the adapter up for the move, i.e. set the page register
;-------------------------------------------------------------------------
        mov     es:SRPReven, ah         ; set Current Page
        and     ah, PageSizeMask        ; get Relative Page
        mov     es, SharedRamSegment    ; get Shared RAM Segment
        mov     di, ax                  ; ES:DI := Shared Ram @ for (address from AX)

;-------------------------------------------------------------------------
;       Prepare the Fill in the Header
;-------------------------------------------------------------------------
        mov     word ptr ARBTCBPtr, si                  ; Save for the end
        mov     word ptr ARBTCBPtr + 2, ds              ; Save For Later
        mov     cx, word ptr [si].TCBDataLen            ; Get the total length
        xchg    ch,cl                                   ; Allow for IBM Format
        mov     word ptr ARBSaveArea.ASBFrameLength, cx ; Fill in the length
        mov     cx, word ptr [si].TCBMediaHeaderLen     ; get Header Len
        lea     si, [si].TCBMediaHeader                 ; DS:SI -> Header (data)

;-------------------------------------------------------------------------
;               Load the TCB   --- Fill in the Header
;-------------------------------------------------------------------------
ifdef RPL
        jmp     HDLdTxBufwithData
endif
        mov     al, cs:PageActFlg
        cmp     cs:PageSizeMask, al     ; see if Paging is Active?
        jnz     HDLdTxBufwithData        ; nz => not active

;-------------------------------------------------------------------------
;       Check For Header to Cross a Page boundary
;-------------------------------------------------------------------------
        mov     bx, cx                   ; Paging is Active, check for page overflow
        lea     ax, word ptr es:[di][bx] ; Where would we be after the move
        test    ah, cs:PageBreakMask     ; Would we go over a page boundary?
        jz      HDLdTxBufwithData        ; z = will not go over the page

;-------------------------------------------------------------------------
;       This Section of Code is for the Moving the Header.
;       The Transmit Buffer is about to cross a page boundary
;       Finish filling to the end of the page, and then try to finish
;-------------------------------------------------------------------------
        mov     ax, cs:PageSize                 ; AX: Size of a page
        sub     ax, di                          ; AX = Number of bytes left on current page
        xchg    ax, cx                          ; CX = Number of Bytes to move this page
                                                ; AX = Total Number of bytes to move
        sub     ax, cx                          ; AX = # Bytes left to load after the move
        FastMove                                ; Load the dhb with cx bytes
        mov     cx, ax                          ; CX = # bytes left to move
        sub     di, cs:PageSize                 ; DI = Start of Next Page
        mov     es, cs:MMIOSegment              ; ES = Restored to MMIO
        mov     al, cs:PageIncrement
        add     byte ptr es:SRPReven, al
        mov     es, cs:SharedRamSegment         ; ES = Restored to SharedRamSegment

HDLdTxBufwithData:
        FastMove                                ; move the Data

        mov     si, word ptr ARBTCBPtr          ; SI -> TCB
        lds     si, [si].TCBFragStrucPtr        ; get ptr. to Frags  DS:SI -> Frag Struct

;-------------------------------------------------------------------------
;               Move Data to Shared RAM Transmit Buffer,
;           DS:SI TCB Frag List ptr. ES:DI place to load Data
;-------------------------------------------------------------------------
        mov     cx, word ptr [si]       ; check # of Fragments
        jcxz    TCBReturn               ; No Frags, THen We must be done.

        inc     si
        inc     si                      ;DS:SI -> Ptr. to 1st. Fragment @ (2 bytes)

MoveDataTxBufferLoop:
        push    cx                      ; Keep track of num of Frags
        push    ds                      ; keep index DS:SI -> TCB Frag List
        push    si
        mov     cx, [si].FragLen        ; get Frag Length
        jcxz    MoveDataTxBufGetNextFrag

        lds     si, [si].FragAddr       ; SI: -> Fragment data

;-------------------------------------------------------------------------
;               Load the TCB   ---  Fill in the DATA
;-------------------------------------------------------------------------
ifdef RPL
        jmp     LoadTxBufwithData
endif
        mov     al, cs:PageActFlg
        cmp     cs:PageSizeMask, al     ; see if Paging is Active?
        jnz     LoadTxBufwithData       ; nz => not active

ChkForCrossPage:
        mov     bx, cx                   ; Paging is Active, check for page overflow
        lea     ax, word ptr es:[di][bx] ; Where would we be after the move
        test    ah, cs:PageBreakMask     ; Would we go over a page boundary?
        jnz     AbouttoCross             ; nz = would go over the mark

LoadTxBufwithData:
        FastMove                         ;move the Data

;-------------------------------------------------------------------------
;       Get the Next Data Fragment from the list
;-------------------------------------------------------------------------
MoveDataTxBufGetNextFrag:
        pop     si
        pop     ds                      ; Restore DS:SI -> TCB Frag List
        pop     cx                      ; Restore Num of Frags
        add     si, 6                   ; get next Fragment
        loop    MoveDataTxBufferLoop    ; CX=CX-1 # Frags.

;-------------------------------------------------------------------------
; Return the TCB to the MSM, and send ARB response
;-------------------------------------------------------------------------
TCBReturn:
        lds     si, cs:ARBTCBPtr        ; DS:SI -> TCB
        call    MSMSendComplete         ; Give TCB back to MSM
        inc     MSMTxFreeCount          ; Free a TCB Count
        jmp     GiveASBResponse

;---------------- ERROR COUNTERS ----------------

UnknownARBRequestsError:
        inc     UnknownARBRequests
        jmp     GiveASBResponse

;---------------- END OF ERROR COUNTERS ----------------

;-------------------------------------------------------------------------
;       This next little section is for the Data Move, and
;       The Transmit Buffer is about to cross a page boundary
;       Finish filling to the end of the page, and then try to finish
;-------------------------------------------------------------------------
AbouttoCross:
        mov     ax, cs:PageSize         ; AX: Size of a page
        sub     ax, di                  ; AX = Number of bytes left on current page
        xchg    ax, cx                  ; CX = Number of Bytes to move this page
                                        ; AX = Total Number of bytes to move
        sub     ax, cx                  ; AX = # Bytes left to load after the move
        FastMove                        ; Load the dhb with cx bytes
        mov     cx, ax                  ; CX = # bytes left to move
        sub     di, cs:PageSize         ; DI = Start of Next Page
        mov     es, cs:MMIOSegment      ; ES: MMIO segment for Register Move
        mov     al, cs:PageIncrement
        add     byte ptr es:SRPReven, al
        mov     es, cs:SharedRamSegment ; ES: Restored SharedRam Segment
        jmp   short ChkForCrossPage


;-------------------------------------------------------------------------
; Check For ARB 84h -- Ring.Status.Change    (ARB 83h should never happen)
;-------------------------------------------------------------------------
ChkRingStatusChg:
        cmp     al, ARBCmdRingStatChg   ; '84h' Ring.Status.Change
        jne     UnknownARBRequestsError ; Not looking like one we know

;-------------------------------------------------------------------------
;       Get the New Status, Place in the Custom Counters for display
;-------------------------------------------------------------------------
        mov     ax, ARBSaveArea.ARBStatus       ; Get the status of the network
        xchg    ah, al                          ; Adjust for IBM Format
        mov     word ptr LastRingStatus, ax     ; Save it away where we all can see it
        test    ah, LobeWireFault+AutoRemovError+RemoveReceived
        jnz     AdapterIsClosed

;-------------------------------------------------------------------------
;       Check if a counter has overflowed... if so read the counters
;-------------------------------------------------------------------------

        test    al, CounterOverflow             ; Check if a Counter has overflowed
        jz      ARB84Exit                       ; Not overflowing then let's quit

        or      InterruptStateFlag, DIRReadLoginOrder   ; Better check the counters
ARB84Exit:                                              ; Check Next SRB is called after return
        ret

;-------------------------------------------------------------------------
;       The Adapter has been closed... Return Outstanding Sends
;-------------------------------------------------------------------------
AdapterIsClosed:                                        ; AH: has Error Code
        mov     InterruptStateFlag, AdapterShutdownBit  ; If the Adapter is Closed then we should too
        test    ah, RemoveReceived                      ; Were We told to Close down?
        jnz     AdaptIsClosedPerm                       ; Then Go Shut Us down

        jmp     DriverReset                             ; Otherwise Get the Card back on line

AdaptIsClosedPerm:
        jmp     HSMShutDownMSM          ; Shoot ourselves in the head

ARBReqInt       endp                    ; Token-Ring ARB Request Interrupt


;*******************************************************************************
;               SSB Response Interrupt
;*******************************************************************************

SSBResponseInt  proc    near                            ; SSB Response
        mov     es, MMIOSegment                         ; ES: MMIOSegment
        mov     byte ptr es:ISRASetodd, SSBFreeBit      ; Ack the SSB int.
        ret                                             ; Get out of here
SSBResponseInt  endp

;*********************************************************************
;               AES Event Handlers
;*********************************************************************
;-------------------------------------------------------------------------
;       NICAESproc
;-------------------------------------------------------------------------
;       Assumes: ES:SI -> AES ECB       Returns:  DS:BP preserved
;                Ints   disabled                  Ints Disable

NICAESproc      proc    far

;-------------------------------------------------------------------------
;                       Schedual DIR.Read.Log
;-------------------------------------------------------------------------
        test    cs:InterruptStateFlag, AdapterShutdownBit ; Are we closed?
        jnz     DontSchedual                            ; Don't schedual a Readlog

        or      cs:InterruptStateFlag, DIRReadLoginOrder ; Set up to enforce the Read

;-------------------------------------------------------------------------
;                       Schedual AES Events
;-------------------------------------------------------------------------
DontSchedual:
        mov     word ptr cs:[si].MSecondValue, 5000     ; Length of time between events = 5 seconds
        mov     bx, MLIDSUP_SCHEDULE_AES_EVENT          ; Line up to schedual the event
        jmp     cs:LSLSupport                           ; Let him give Far Return

NICAESproc      endp


;*******************************************************************************
;******************************************************************************
;       DriverSend
;*******************************************************************************
;******************************************************************************
;
;       This routine will initiate a packet transmission.
;
;       assumes:        DS, CS  CGroup
;                       SI      -> TCB to be transmitted.
;                       CLD has been executed
;                       Interrupts are disabled.  (CLI state)
;                       A critical section has been started
;
;       returns:        Can destroy all registers and flags
;                       Interrupts disabled
;

;Take Care of the Non optimum case first
;-------------------------------------------------------------------------
;               Queue the TCB while we are too busy
;-------------------------------------------------------------------------
ChkNxtSRBCmdJmp:
        jmp     ChkNxtSRBCmd

WayTooBusy:
        cmp     TooBusyPtr, 0000h       ; Check if we have a too busy TCB already
        jnz     CheckShutDownBit        ; If there is one don't move pointer

        mov     TooBusyPtr, si          ; Else Move Que ptr up

CheckShutDownBit:
        test    InterruptStateFlag, AdapterShutdownBit  ;ShutDown?
        jz      ChkNxtSRBCmdJmp                         ; z = SRB is Busy, But Adapter is Alive go return from Jump

        test    InterruptStateFlag, DirOpenInProgress
        jnz     SkipStartSendExit                       ; nz = SRB is Busy, he is trying to open so don't bother him

        jmp     DriverReset                             ; nz = Try to restart the Card

ReturnTCBAdapterClosed:
        cmp     XmitQueHead, 0                  ;Other TCBs queued
        jz      DontClearTransmitQue

        push    si
        call    ClearQBrothers
        pop     si

DontClearTransmitQue:
        inc     MSMTxFreeCount
        jmp     MSMSendComplete                 ;Return TCB

;--------- Procedure Starts Here ------------------------------------------

DriverSend      proc    near
        test    InterruptStateFlag, AdapterShutdownBit  ; Test if adapter closed.
        jnz     ReturnTCBAdapterClosed

        test    InitStatus, INIT_FAST_PATH      ; Is Fast Path Supported?
        jnz     DriverSendFastPath

;-------------------------------------------------------------------------
; Q the TCB and Then Check if Busy
;-------------------------------------------------------------------------
        mov     word ptr [si].TCBNxtLnk, 0000h  ; next ECB @ = 0000 (tail value)
        mov     bx, si                          ; BX : ptr to TCB to Q
        xchg    bx, QueTail                     ; BX: ptr to Last TCB in Q.
                                                ; QueTail: NewLast TCB
        mov     [bx].TCBNxtLnk, si              ; Enqueue to Queue Tail

        test    InterruptStateFlag, DIRReadLoginOrder + DIRSetFuncAddr + SRBBusy + AdapterShutdownBit ; TooBusy?
        jnz     WayTooBusy                      ; z = Ret in StartSend. (Return from there)

DriverSend      endp


;*******************************************************************************
; StartSend - DMA the packet and start the transmit
;*******************************************************************************
;
;       assumes:        DS, CS  CGroup
;                       CLD has been executed
;                       interrupts are disabled   (CLI state)
;

StartSend       proc    near

;-------------------------------------------------------------------------
;       Set the CL Register with the command
;-------------------------------------------------------------------------
mov     cl, SRBCmdXMTDIRFrame           ; CL = XMT.DIR.Frame

;-------------------------------------------------------------------------
;               IssuseSRBCmd:   Issue Command to the SRB
;-------------------------------------------------------------------------
IssueSRBCmd:                                    ; IssueSRBCmd   Issue Command to the SRB
                                                ;       Entry:  CL = SRB Command
                                                ;       Interrupts Disabled
                                                ;       Alters: AX, ES, SI

;-------------------------------------------------------------------------
;       Get the proper Page first
;-------------------------------------------------------------------------
        mov     ax, word ptr SRBAddress         ; AH = Current Page x 16 K (AL)
        mov     es, MMIOSegment                 ; get MMIO Segment
        mov     byte ptr es:SRPReven, ah        ; set Current Page
        and     ah, PageSizeMask                ; get Relative Address
        mov     es, SharedRamSegment            ; get Shared RAM Segment
        mov     di, ax                          ; ES:DI := Shared Ram @ SRBAddress

;-------------------------------------------------------------------------
;       Set  Command and Return Code in SRB and Tell Adapter SRB is ready
;-------------------------------------------------------------------------
        or      InterruptStateFlag, SRBBusy + WaitingIntBit
        mov     byte ptr es:[di].SRBSSBCommand, cl      ; CL = SRB Command Code
        mov     byte ptr es:[di].SRBSSBRetCode, 0FEh    ; Return Code = Not Processed

ifdef RPL
        mov     word ptr es:[di].SRBSSBStationID, 0     ; The IBM ROM uses this field so we need to set
endif                                                   ;  it to zero here.

        mov     es, MMIOSegment                         ; ES: MMIO
        mov     byte ptr es:ISRASetodd, CommandInSRBBit ; Tell Adapter that a Command is in SRB
SkipStartSendExit:                                      ; Label Used if the Dir.Open.Command is in progress
        ret

StartSend       endp

;
;***********************************************************************\
;
; BEGIN_MANUAL_ENTRY( DriverSendFastPath, TOKEN/API/DRVSNDFP )
;
; Name:         DriverSendFastPath
;
; Description:  This routine is call to transmit a packet when the card
;               is configured for fast path. It is responsible for setting
;               up the card, moving the data to the card and initiating
;               the send.
;
; On Entry:     AX      N/A
;               BX      N/A
;               CX      N/A
;               DX      N/A
;               BP      N/A
;               SI      -> to TCB
;               DI      N/A
;               DS      CGroup
;               CS      CGroup
;
;               Note:   CLD has been executed
;                       Interrupts are disabled.
;                       A critical section has been started
;
; On Return:    AX      Destroyed
;               BX      Destroyed
;               CX      Destroyed
;               DX      Destroyed
;               BP      Destroyed
;               SI      Destroyed
;               DI      Destroyed
;
;               Flags:
;
;               Note:   Interrupts disabled.
;
; Remarks:
;
; See Also:     DriverSend, FastPathTxComplete
;
; END_MANUAL_ENTRY
;
;***********************************************************************/
;
DriverSendNotOpen:
        inc     MSMTxFreeCount
        jmp     MSMSendComplete

DriverSendFastPath      proc

        cmp     cs:CompletionQHead, 0           ; Fast Path Opened?
        je      DriverSendNotOpen               ; Jump if not.

        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     al, cs:TxPageValue
        mov     es:SRPReven, al                 ; Set new page

        mov     di, cs:TxControlAddress

        mov     es, cs:SharedRamSegment         ; ES:DI -> Fast Path Tx Control area.
;
; ES:DI -> Fast Path Transmit Control Area. Get First Transmit Buffer.
;
        mov     bx, es:[di].FPFreeQHead         ; Get offset of Q Head.

        mov     cs:TxLastBuffer, bx             ; Save for FPTxLastBuffer.
        xchg    bh, bl                          ; Convert to Intel format.
        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     es:SRPReven, bh                 ; Set new page
        and     bh, cs:PageSizeMask             ; Mask off page bits.
        mov     es, cs:SharedRamSegment         ; ES:DI -> Fast Path Tx Control area.
;
; ES:BX -> First Fast Path Transmit Buffer. Fill out necessary info.
;
        mov     es:[bx-FPTxNextBuffer].FPTxCommand, SRBCmdXMTDIRFrame
        mov     al, cs:TxCorrelator
        mov     es:[bx-FPTxNextBuffer].FPTxCorrelator, al
        inc     al                              ; Set new correlator.
        and     al, 7fh                         ; Set to 0-127.
        mov     cs:TxCorrelator, al
        mov     es:[bx-FPTxNextBuffer].FPTxSTATIONID, 0 ; Set to Direct Station.
        mov     ax, word ptr [si].TCBDataLen    ; AX = Total frame size.
        xchg    ah, al                          ; Convert to IBM format.
        mov     es:[bx-FPTxNextBuffer].FPTxFrameLength, ax

        mov     cs:TxBufferSpace, 512-22        ; Save bytes left in buffer.
        mov     cs:TxCurrentTCB, si             ; Save pointer to TCB.
        mov     cs:TxCurrentBuffer, bx          ; Save Current Tx Buffer.
;
; Copy TCB header to first buffer.
;
        lea     di, es:[bx-FPTxNextBuffer].FPTxFrameData        ; ES:DI -> Buffer Data Area.
        mov     cx, word ptr [si].TCBMediaHeaderLen     ; CX = Media Header Len.
        lea     si, [si].TCBMediaHeader         ; DS:SI -> Media Header.
        sub     cs:TxBufferSpace, cx            ; Update bytes left.

        FastMove                                ; Copy data to Shared RAM.

        mov     si, cs:TxCurrentTCB             ; DS:SI -> TCB
        lds     si, [si].TCBFragStrucPtr        ; get ptr to frags DS:SI -> Frag Struct
        mov     ax, word ptr [si]               ; AX = Fragment Count.
        add     si, 2                           ; BX offset First Frag Desc.
        mov     cs:TxFragmentCount, ax          ; Save fragment count.
        mov     dx, ds
        mov     bp, si

DriverSendFastPathLoop:
        mov     cx, word ptr [si].FragLen       ; CX = Fragment Length.
        lds     si, [si].FragAddr               ; DS:SI -> TCB Frag offset.
        mov     ax, cs:TxBufferSpace            ; AX = Dest space left.

DriverSendFragRet:
        cmp     cx, ax                          ; Do we have enough space?
        jbe     FastPathCont
        jmp     DriverSendFastPathFrag

FastPathCont:
        sub     ax, cx                          ; AX = Space left.
        mov     cs:TxBufferSpace, ax            ; Save for later.

        FastMove                                ; Copy data to Shared RAM.

        mov     ax, cs:TxBufferSpace
        mov     ds, dx
        add     bp, 6                           ; Get Next Descriptor.
        mov     si, bp
        dec     cs:TxFragmentCount              ; Any fragments left?
        jnz     DriverSendFastPathLoop          ; Jump if so.

        mov     ax, 512-22                      ; AX = Max Frame Data Size.
        sub     ax, cs:TxBufferSpace            ; AX = Size of this buffer.
        mov     bx, cs:TxCurrentBuffer
        xchg    ah, al                          ; Convert to IBM format.
        mov     es:[bx-FPTxNextBuffer].FPTxBufferLength, ax     ; Store buffer length.

        mov     cx, es:[bx-FPTxNextBuffer].FPTxNextBuffer       ; CX = offset of next buffer.

        mov     es, cs:MMIOSegment              ; ES: MMIO base
        mov     al, cs:TxPageValue
        mov     es:SRPReven, al                 ; Set new page

        mov     di, cs:TxControlAddress
        mov     es, cs:SharedRamSegment         ; ES:DI -> Fast Path Tx Control area.
        mov     bx, es:[di].FPFreeQHead         ; DX = &Q Head.
        xchg    bh, bl                          ; Convert to Intel format.
        mov     es, cs:MMIOSegment              ; ES: MMIO base
        mov     es:SRPReven, bh                 ; Set new page
        and     bh, cs:PageSizeMask             ; Mask off page bits.

        mov     es, cs:SharedRamSegment         ; ES = Shared RAM Segment
        mov     ax, cs:TxLastBuffer             ; AX = Offset of last buffer.
        mov     es:[bx-FPTxNextBuffer].FPTxLastBuffer, ax       ; Store last buffer.

        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     al, cs:TxPageValue
        mov     es:SRPReven, al                 ; Set new page
        mov     es, cs:SharedRamSegment         ; ES = Shared RAM Segment
        mov     es:[di].FPFreeQHead, cx         ; Update queue head.

        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     byte ptr es:ISRASetodd, ISRABridgeReq   ; Tell the card

        mov     si, cs:TxCurrentTCB
        push    cs
        pop     ds
        call    MSMSendComplete
        ret
;
; Fragment will not fit into Tx buffer. Fill in rest and get new Tx buffer.
;
DriverSendFastPathFrag:
        push    dx
        mov     dx, cx
        sub     dx, ax                  ; DX = Bytes left.
        mov     cx, ax

        FastMove                        ; Copy data to Shared RAM.

        mov     cx, dx                  ; CX = bytes to copy.
        mov     ax, 512-22
        xchg    ah, al                  ; Convert to IBM format.
        mov     es:[bx-FPTxNextBuffer].FPTxBufferLength, ax
        mov     bx, es:[bx-FPTxNextBuffer].FPTxNextBuffer       ; BX = offset of next buffer.

        mov     cs:TxLastBuffer, bx             ; Save for FPTxLastBuffer.
        xchg    bh, bl                          ; Convert to Intel format.
        mov     es, cs:MMIOSegment                      ; ES = BIOS/MMIO.
        mov     es:SRPReven, bh                 ; Set new Window Page.
        mov     es, cs:SharedRamSegment         ; ES = Shared RAM Segment
        and     bh, cs:PageSizeMask             ; Mask off page bits.
        mov     cs:TxCurrentBuffer, bx

        lea     di, es:[bx-FPTxNextBuffer].FPTxFrameData        ; DI -> Buffer Data Area.
        mov     ax, 512-22                      ; AX = Space left.
        mov     cs:TxBufferSpace, ax            ; Save current space left.
        pop     dx
        jmp     DriverSendFragRet               ; Continue copy.

DriverSendFastPath      endp


;*********************************************************************
; ChkNxtSRBCmd  ::  Check for Next SRB Command Code &/OR SRB Busy
;*********************************************************************
;
; Entry         TooBusyPtr has a Pointer to currently waiting TCB
;               DS, CS : CGroup Segment

ChkNxtSRBCmd    proc    near
        cli                                     ; Disable Int.
        test    InterruptStateFlag, (AdapterShutdownBit OR DirOpenInProgress)
                                                ; Are we Alive, or Trying to Ressurrect
        jnz     ChkNxtSRBCmdExit                ; nz => Dead or Ressurrecting, time to exit

        test    InterruptStateFlag, SRBBusy     ; Are We Busy?
        jz      ChkReadLog                      ; z = Not SRBBusy; nz =yes, but go look if done

;-------------------------------------------------------------------------
;               Check if SRB is still processing request
;-------------------------------------------------------------------------
        mov     ax, word ptr SRBAddress                 ; AH = Current Page x 16 K (AL)
        mov     es, MMIOSegment                         ; get MMIO Segment
        mov     bh, ah                                  ; BH = New page
        xchg    bh, es:SRPReven                         ; set Current Page, save old one = BH
        and     ah, PageSizeMask                        ; get Relative Page
        mov     es, SharedRamSegment                    ; get Shared RAM Segment
        mov     di, ax                                  ; ES:DI := Shared Ram @ for SRBaddress
        cmp     byte ptr es:[di].SRBSSBRetCode, 0FFh    ; Cmd in Process ?
        mov     es, MMIOSegment
        mov     es:SRPReven, bh                         ; Restore Old Page Value
        jne     ChkNxtSRBCmdExit                        ; NE => No, need to wait somemore so exit

        jmp   short ChkPreProcessQ                      ; Go see if there are more to send

;-------------------------------------------------------------------------
;               Check the DIR.Read.Log Flag
;-------------------------------------------------------------------------
ChkReadLog:
        test    InterruptStateFlag, DIRReadLoginOrder
        jz      ChkSetFuncAddr                          ; z => Not ready to read log

        mov     cl, SRBCmdDIRReadLog                    ; Cl = DIR.Read.Log
        and     InterruptStateFlag, NOT DIRReadLoginOrder
        jmp     IssueSRBCmd                             ; Go Issuse DIR.Read.Log

;-------------- Exit Point for ChkNxtSRBCmd----------------------

ChkNxtSRBCmdExit:
        ret

;-------------------------------------------------------------------------
; Check the DIR.Set.Functional.Address Flag
;-------------------------------------------------------------------------
ChkSetFuncAddr:
        test    InterruptStateFlag, DIRSetFuncAddr
        jz      ChkPreProcessQ                  ; DIR.Set.Functional.Address Not SET

        and     InterruptStateFlag, NOT DIRSetFuncAddr  ; Clear Flag

;-------------------------------------------------------------------------
;               Get Current Functional Address
;-------------------------------------------------------------------------
        mov     ax, word ptr SRBAddress ; AH = Current Page x 16 K (AL)
        call    PagingSet               ; Entry AX = Offset into Shared RAM
                                        ; ES:DI -> Shared Ram SRB --- BH holds old page Register
        mov     ax, word ptr FuncAddr
        mov     word ptr es:[di].SRBSSBFuncAddr, ax
        mov     ax, word ptr FuncAddr+2
        mov     word ptr es:[di].SRBSSBFuncAddr+2, ax

;-------------------------------------------------------------------------
;                       Set Functional Address
;-------------------------------------------------------------------------

        mov     cl, SRBCmdSetFuncAddr   ; CL = DIR.Set.Functional.Address
        mov     es, MMIOSegment
        mov     es:SRPReven, bh         ; Restore Page Register
        jmp     IssueSRBCmd             ; Go issue DIR.Set.Functional Address

;-------------------------------------------------------------------------
;               The SRB Returned an IN-Process Return Code 'FF'
;               Dequeue the TooBusyQ TCB        (Pre-Process Q)
;-------------------------------------------------------------------------
ChkPreProcessQ:
        and     byte ptr InterruptStateFlag, NOT (SRBBusy) ; Reset SRB Busy Flag
        mov     si, TooBusyPtr                             ; get ptr's to TCB
        or      si, si                                     ; Error Check
        jz      ChkHoldQueue

        mov     bx, [si].TCBNxtLnk ; NEXT Waiting TCB (may be none)
        mov     TooBusyPtr, bx     ; reset hold buffer to either the end, or the next TCB

;-------------------------------------------------------------------------
;       Request to Send the Packet
;-------------------------------------------------------------------------
        test    InitStatus, INIT_FAST_PATH      ; Is Fast Path Supported?
        jnz     ChkNxtSRBCmdExit

        call    StartSend          ; Get the Processing underway

;-------------------------------------------------------------------------
; Check to see if there is room for anymore TCB's, or that the MSM has a new one for us.
;-------------------------------------------------------------------------
ChkHoldQueue:
        cmp     word ptr MSMTxQueueHead + 2, 0000h ; Check to see if there is anything in the MSM Q
        je      ChkNxtSRBCmdExit        ; Exit if nothing there

        call    MSMGetNextSend          ; Check to see if there is anymore we want to do
        jnz     ChkNxtSRBCmdExit        ; Exit if nothing to do

        call    DriverSend              ; there is a new one to start
        jmp     ChkHoldQueue

ChkNxtSRBCmd    endp


;*******************************************************************************
;******************************************************************************
;       DriverISR
;*******************************************************************************
;******************************************************************************
;
;       This routine is called when the NIC interrupts the system.
;
;       assumes:        DS:     CGroup
;                       CLD has been executed
;                       Inside a critical section
;                       Interrupts are disabled
;                       All registers have been preserved
;
;       returns:        DS:     CGroup
;                       All other registers may be destroyed
;                       Interrupts disabled
;
;

;-------------------------------Error Handlers First

UnexpectedBoardInt:                                     ; Isolate Unexpected Interrupt(s)
        and     cx, NOT (0FF00h + ISRPAccessErrBit + ErrorIntBit + ISRPTimerIntBit)
        not     cx
        and     cl, NOT (InterruptEnableBit+NMIIntControlBit)
        mov     word ptr es:ISRPResetEven, cx           ; Reset Interrupt, Set Disabled

UnexpectedInt:

;-------------------------------------------------------------------------
;               EOI for the Error Bunch
;-------------------------------------------------------------------------
DriverISREndJmp:
        mov     al, 20h                 ; Issue EOI to PIC(s)
        cmp     MSMEOIFlag, 0           ; EOI needed on slave PIC?
        je      DontSlave

        out     0A0h, al                ; EOI Slave PIC
        jmp     $+2

DontSlave:
        out     20h, al                 ; Issue EOI to the 1st PIC
        jmp     short ExitDriverISR

;-------------------------------------------------------------------------
;               Slave EOI for the Normal Case
;-------------------------------------------------------------------------
DoSlave:
        out     0A0h, al                ; Issue EOI to the 2nd PIC
        jmp     short BackFromEOI


; Token-Ring Interrupt Service Code
;*******************************************************************************

DriverISR       proc  near                      ; Token-Ring Interrupt Service Code

        test    InterruptStateFlag, AdaptHeldReset
        jnz     DriverISREndJmp                 ; can't look at MMIO if Held Reset

        mov     es, MMIOSegment                 ; ES: MMIO
        mov     cx, word ptr es:ISRPeven        ; get Interrupt (Odd-Even in Reg.)
        mov     InterruptStateCopy, cx          ; keep copy of Interrupt in Odd-Even order (or)
        test    cl, InterruptEnableBit          ; test if my Interrupts supposedly enabled  (Even byte)
        jz      UnexpectedInt                   ; Z - NO, then how here, note it

        test    cx, (0FF00h+ISRPAccessErrBit + ErrorIntBit + ISRPTimerIntBit) ; even byte
        jz      UnexpectedBoardInt              ; Z - NO, then how here

        mov     si, offset CGroup:Dispatch      ; get Interrupt Mask Table

FindInterrupt:
        lodsw                                   ; get Interrupt Mask
        and     ax, cx                          ; Test for Match   (Odd-Even byte)
        jz      FindInterrupt                   ; Look till Find Match or End of Table

;-------------------------------------------------------------------------
;                       Int found now Disable card
;-------------------------------------------------------------------------
FoundInterrupt:
        and     InterruptStateFlag, NOT WaitingIntBit           ; 0 Implies Interrupt Happended
        not     ax                                              ; Turn off everything that was on both even and odd
        and     al, NOT (InterruptEnableBit+NMIIntControlBit)   ; Turn off Card Enable
        mov     word ptr es:ISRPResetEven, ax                   ; Reset Interrupt ( also Resets the ISRPodd)

;-------------------------------------------------------------------------
;                               EOI PIC
;-------------------------------------------------------------------------
EOIController:
        mov     al, 20h                         ; Issue EOI to PIC(s)
        cmp     MSMEOIFlag, 0                   ; EOI needed on slave PIC?
        jne     DoSlave

BackFromEOI:
        out     20h, al                         ; Issue EOI to the 1st PIC
        jmp     $+2                             ; Flush IO queue

        test    InterruptStateFlag, IgnoreAllInterrupts
        jnz     ExitDriverISR                   ; Exit, don't call Second Level

;-------------------------------------------------------------------------
;               ProcessISR:             ; Test all Low Ints & some High Ints
;-------------------------------------------------------------------------
ProcessISR:                                     ; Test all Low Ints & some High Ints
        sti
        mov     al, byte ptr es:SRPReven        ; save Page Ram Register
        push    es                              ; Save Segment Value for later
        push    ax                              ; Save Register Value for later
        push    [si+14]                         ; Save ptr. to Interrupt Vector Table
        call    ChkNxtSRBCmd                    ; Check for SRB In Process Return Code
        pop     dx                              ; get ptr. to Interrupt Vector Table
        call    dx                              ; DX = Routine @
        call    ChkNxtSRBCmd                    ; Check for SRB In Process Return Code
        pop     ax                              ; AL = Page Ram Register
        pop     es                              ; ES: MMIOSegment
        mov     byte ptr es:SRPReven, al        ; restore page register

ExitDriverISR:                                  ; we need to Issue PIO to Re-Arm The Interrupt
        cli                                     ; IF = 0 ==> No Interrupts
        mov     dx, AdapterInterruptReset       ; DX = Interrupt Reset and Enable
        out     dx, al                          ; Re-Arm adapter Interrupt

DriverISREnd:                                           ; If Held reset the Cann't look at MMIO
        test    InterruptStateFlag,AdaptHeldReset       ; if we are shut down we still have to
                                                        ;  allow Dir.Open.Adapter to give an interrupt
        jnz     Dummy_Rtn                               ; z => all is well turn card on

;-------------------------------------------------------------------------
;               Set ISRPeven    InitEnable Bit (bit 6)
;-------------------------------------------------------------------------
        mov     byte ptr es:ISRPSeteven, InterruptEnableBit + NMIIntControlBit  ; Enable the Card

Dummy_Rtn:                                      ; Dummy Call for unknown ISR & ISR EXIT
        ret

DriverISR       endp

;
;***********************************************************************\
;
; BEGIN_MANUAL_ENTRY( FastPathTxComplete, TOKEN/API/FPTXCOMP )
;
; Name:         FastPathTxComplete
;
; Description:  This routine is call for a TX complete when fast path
;               has been enabled.  It is responsible for cleaning up
;               the fast path queues, incrementing MSMTxFreeCount and
;               starting another send if one is waiting.
;
; On Entry:     AX      N/A
;               BX      N/A
;               CX      N/A
;               DX      N/A
;               BP      N/A
;               SI      N/A
;               DI      N/A
;
;               Note:
;
; On Return:    AX      Destroyed
;               BX      Destroyed
;               CX      Destroyed
;               DX      Destroyed
;               BP      Preserved
;               SI      Destroyed
;               DI      Destroyed
;
;               Flags:
;
;               Note:   Interrupts disabled.
;
; Remarks:
;
; See Also:
;
; END_MANUAL_ENTRY
;
;***********************************************************************/
;
FastPathTxComplete      proc

        mov     ax, cs:TxControlAddress
        and     ah, cs:PageSizeMask
        mov     di, ax

        mov     es, cs:SharedRamSegment         ; ES:DI -> Fast Path Tx Control area.
        mov     dx, es:[di].FPCompletionQTail   ; Save completion Queue Tail.
        mov     cx, cs:CompletionQHead

FastTxCompleteLoop:
        cmp     dx, cx
        je      short FastTxNextSend            ; Jump out if so.

        inc     MSMTxFreeCount                  ; Free Tx resource.

        xchg    ch, cl                          ; Convert to Intel format.

        mov     es, cs:MMIOSegment                      ; ES: MMIO base
        mov     es:SRPReven, ch                 ; Set new page
        and     ch, cs:PageSizeMask             ; Mask off page bits.
        mov     di, cx

        push    es
        mov     es, cs:SharedRamSegment
        mov     ax, es:[di]                     ; AX = Tx completed.
        xchg    ah, al                          ; Convert to Intel format.
        pop     es

        mov     es:SRPReven, ah                 ; Set new page.
        and     ah, cs:PageSizeMask             ; Mask off page bits.
        mov     di, ax
        push    es
        mov     es, cs:SharedRamSegment
        mov     cx, es:[di-FPTxNextBuffer].FPTxLastBuffer
        pop     es
        mov     al, cs:TxPageValue                      ; Switch back to Control Page.
        mov     es:SRPReven, al                 ; Set new page.

        mov     di, cs:TxControlAddress
        mov     es, cs:SharedRamSegment
        mov     es:[di].FPFreeQTail, cx         ; Update Free Queue Tail.

        mov     cs:CompletionQHead, cx  ; Update Our Completion Head.
        jmp     short FastTxCompleteLoop

FastTxNextSend:
        call    MSMGetNextSend                  ; Any more TCB's?
        jz      AnotherTCBWaiting               ; Jump if so.
        ret

AnotherTCBWaiting:
        call    DriverSendFastPath
        jmp     short FastTxNextSend

FastPathTxComplete      endp

Code    ends


Init    segment para public 'CODE'

;******************************************************************************
;******************************************************************************
;                    DRIVER   INITIALIZATION Init Segment
;******************************************************************************
        even
ToknrSigMCA     TRSMC   <>              ; Token-Ring MCA Signature
                db      BUS_ID_MCA      ; Flag Signitures Switch end MCA
ToknrSigPC      TRSPC   <>              ; Token-Ring PC Signature
                db      BUS_ID_ISA      ; Flag Signitures End MCA

ifdef RPL
        RPLFlag         db      0               ; Set if in RPL Mode.
endif

        even
SharedRamLookup dw      200h,400h,800h,1000h    ; Possible Ram Sizes in Paragraphs
SharedRamSize32K        equ     800h            ; 32K Shared Ram Size

;-------------------------------------------------------------------------
; DHB4MbpsTable / DHB16MbpsTable        Transmit Buffer Size Look Up Tables
;-------------------------------------------------------------------------
DHB4MbpsTable   label   word    ;AIP Register Value
        dw      4464            ;       1FAA = 0Bh      Reserved - Place holder only
        dw      4464            ;       1FAA = 0Ch      Reserved - Place holder only
        dw      4464            ;       1FAA = 0Dh
        dw      4096            ;       1FAA = 0Eh
        dw      2048            ;       1FAA = 0Fh

DHB16MbpsTable  label   word
        dw      17960           ;       1FAC = 0Bh
        dw      16384           ;       1FAC = 0Ch
        dw      08192           ;       1FAC = 0Dh
        dw      04096           ;       1FAC = 0Eh
        dw      02048           ;       1FAC = 0Fh

DHBXMbpsTableSize       equ     $-DHB16MbpsTable        ; Table Size


;-------------------------------------------------------------------------
; Net.CFG Custom Keyword and Definitions
;-------------------------------------------------------------------------
DriverNumKeywords       equ  2

MaxFrameText            db   'MAX FRAME SIZE'
MaxFrameTextLen         equ  $-MaxFrameText

AltKeywordText          db   'ALTERNATE'
AltKeywordTextLen       equ   $-AltKeywordText

DriverKeywordText       dw      offset CGroup:MaxFrameText
                        dw      offset CGroup:AltKeywordText

DriverKeywordTextLen    dw      MaxFrameTextLen
                        dw      AltKeywordTextLen

DriverProcessKeywordTab dw      offset CGroup:ProcessMaxFrameSize
                        dw      offset CGroup:ProcessAltKeyWord

MaxTxBuf                equ     17954   ; Maximum Transmit buffer Size allowed 16 Mbs case.
MinTxBuf                equ     638     ; 512 + 74 + 30 + 22 we will take care of the
                                        ;  6 bytes of overhead and the adjustment for
                                        ;  multiple of 8 requirement.

;-------------------------------------------------------------------------
;
;                       MESSAGE CENTER
;
;               Information Messages for Initialization
;-------------------------------------------------------------------------
OpenAdapterMsg          db      200,0,'Inserting into the ring.  Please Wait.',0Dh,0Ah,0

ErrFirstTryMsg          db      201,0, 'DIR.OPEN.ADAPTER error code = ',0
ErrFirstTryMsg2         db      '.  Trying again.',0Dh,0Ah

OPENEDCarrigeReturnMsg  db      0Dh
                        db      79 dup (' ')
                        db      0Dh,0                   ;Clear longestline


;-------------------------------------------------------------------------
;       Custom KeyWord Service Routines Messages
;-------------------------------------------------------------------------
FrameTooBigMsg          db      228,0, 'The MAX FRAME SIZE in NET.CFG file is too large (Max = 17954).',0Dh,0Ah,0
FrameTooSmallMsg        db      229,0, 'The MAX FRAME SIZE in NET.CFG file is too small (Min = 638).',0Dh,0Ah,0

;HSM Code Checking Messages.

TooBigMaxFrameSizeMsg   db      231,0,'The MAX FRAME SIZE in the NET.CFG is too big for this adapter.',0
MaxFrameSizeMsg         db      '             The maximum value at this speed for this Adapter is ',0   ; Spaces are needed for alignment.

TooBig8KModeMsg         db      208,0,'The MAX FRAME SIZE is too big for this adapter in 8K Mode.',0

TooBig32KModeMsg        db      209,0,'The MAX FRAME SIZE is too big for this adapter in 32K Mode.',0

;-------------------------------------------------------------------------
;       Error Messages When Exiting Initialization
;-------------------------------------------------------------------------
WrongBusMsg             db      28,0,'The driver cannot load.',0Dh,0Ah
                        db              'This Machine does not have the correct bus for the driver;',0Dh,0Ah
                        db              'or the BUS ID specified in the NET.CFG is incorrect.',0Dh,0Ah,0
NICNotFoundMsg          db      50,0,'The board cannot be found.',0Dh,0Ah,0
RamFailedMsg            db      51,0,'Board RAM failed the memory test. ',0Dh,0Ah,0
DirOpenFailedMsg        db      54,0,'The board did not respond to the initialization command. ',0Dh,0Ah,0
CardNotInSlotMsg        db      65,0,'Board cannot be found at slot/IO port.',0Dh,0Ah
                        db      '             (Please check SLOT and PORT'
                        db      ' values in the NET.CFG file.)',0Dh,0Ah,0
InvalidBusIDMsg         db      96,0,'The driver does not support the selected BUS ID.',0Dh,0Ah
                        db              'Make sure the NET.CFG file contains a valid '
                        db              'parameter for the ',0Dh,0Ah, 'BUS ID keyword.',0Dh,0Ah, 0
BringUpErrorMsg         db      202,0,'Board failed diagnostics.  BRING_UP_ERROR CODE = ',0
BringUpErrorValMsg      db      '   ',,0Dh,0Ah,0


RetryLimitMsg           db      203,0,'Board diagnostics failed to complete.',0Dh,0Ah,0

SharedRamBoundaryErr    db      204,0,'The shared RAM is on incorrect boundary. ',0Dh,0Ah,0

CardIsDeadMsg           db      205,0,'Adapter did not reset during initialization.',0Dh,0Ah,0

SharedRAMOverlapError   db      226,0,'The ROM and shared RAM address range overlap. '
                        db      0Ah,0Dh,'             '         ;(CR,LN,13 spaces).
                        db      '(Please check the ROM switch settings and'
                        db      0Ah,0Dh,'             '         ;(CR,LN,13 spaces).
                        db      'RAM Address [MEM #2] in the NET.CFG file.)',0Dh,0Ah,0

DirOpenIntFailedMsg     db      206,0,'An interrupt failed to occur during initialization.',0Dh,0Ah,0
DirOpenFailedMsg2       db      207,0,'Initialization Error = '
DirOpenFailedEndMsg2    db      'Returned Error Code = '
DirOpenFailCodeMsg2     db      '  ',0Dh,0Ah,0          ;Will be overwriten and overwrite the ones below


;-------------------------------------------------------------------------
; Extended Errors Messages.
;-------------------------------------------------------------------------
Message  DirOpenErrMsg1   ' - Cable Test Failure'               ;'n1 Function Failure'
Message  DirOpenErrMsg2   ' - Signal Loss'                      ;'n2'
Message  DirOpenErrMsg3   '.'   ; - Reserved                    ;'n3'
Message  DirOpenErrMsg4   '.'   ; - Reserved'                   ;'n4'
Message  DirOpenErrMsg5   ' - Timeout... Try again.'            ;'n5' Timeout
Message  DirOpenErrMsg6   '.'   ; - Ring Failure'               ;'n6' (No need to cause alarm.)
Message  DirOpenErrMsg7   ' - Ring Beaconing'                   ;'n7'
Message  DirOpenErrMsg8   ' - Duplicate Node Address Found'     ;'n8' Duplicate Node Address
Message  DirOpenErrMsg9   ' - Parameter Server did not respond' ;'n9' Parameter Request
Message  DirOpenErrMsgA   ' - Remove From Ring Command Received';'nA' Remove Received
Message  DirOpenErrMsgB   '.'   ; - Reserved'                   ;'nB'
Message  DirOpenErrMsgC   '.'   ; - Reserved'                   ;'nC'
Message  DirOpenErrMsgD   '.'   ; - No Monitor Detected for RPL';'nD'
Message  DirOpenErrMsgE   '.'   ; - Monitor Contention Fail for RPL';'nE'
Message  FunctFailHelpMsg  'PROBABLE CAUSE: The cable is disconnected from the board.'
Message  RingBeaconHelpMsg 'PROBABLE CAUSE: The Adapter Data Rate Switch is set at the wrong speed.',0

;-------------------------------------------------------------------------
;       Extended Error Code Tables for Msg building
;-------------------------------------------------------------------------
        even
ExtendedErrorMsgTBL label word
        dw      Offset CGroup:DirOpenErrMsg1    ;'n1'
        dw      Offset CGroup:DirOpenErrMsg2    ;'n2'
        dw      Offset CGroup:DirOpenErrMsg3    ;'n3'
        dw      Offset CGroup:DirOpenErrMsg4    ;'n4'
        dw      Offset CGroup:DirOpenErrMsg5    ;'n5'
        dw      Offset CGroup:DirOpenErrMsg6    ;'n6'
        dw      Offset CGroup:DirOpenErrMsg7    ;'n7'
        dw      Offset CGroup:DirOpenErrMsg8    ;'n8'
        dw      Offset CGroup:DirOpenErrMsg9    ;'n9'
        dw      Offset CGroup:DirOpenErrMsgA    ;'nA'
        dw      Offset CGroup:DirOpenErrMsgB    ;'nB'
        dw      Offset CGroup:DirOpenErrMsgC    ;'nC'
        dw      Offset CGroup:DirOpenErrMsgD    ;'nD'
        dw      Offset CGroup:DirOpenErrMsgE    ;'nE'


;******************************************************************************
;******************************************************************************
;                    DRIVER   INIT
;******************************************************************************
;******************************************************************************
;  Assumes :                Returns:
;       DS,ES,CS -> CGroup      AX = Completion Code
;       CLD is in effect        DX -> ASCIIz Error Message if AX !=0
;       Ints are ENABLED        All other Registers/Flags maybe destroyed
;
;                               Completion Code:  0 => Hardware Operational
;                                             non-zero => DX -> error Msg.
;**********************************************************************
SlotAddressError:
        mov     dx, offset CGroup:CardNotInSlotMsg

InitErrorExitJmp:
        jmp     InitErrorNoInts

DriverInit      proc    near
;-------------------------------------------------------------------------
;       Start off Checking the BUS ID
;-------------------------------------------------------------------------
        mov     dx, offset CGroup:InvalidBusIDMsg       ; Possible Error Message.
        cmp     BusID, BUS_ID_HSM_DEFAULT               ; See if HSM can Choose
        jne     CheckForSupportedBUSIDs                 ; ne => See if the user
                                                        ; specified a useful number

        test    MSMSystemFlags, MCAFlag                 ; Are We on a PS2
        jnz     InitForMCA                              ; nz => We are on PS2
        jmp short InitForISA                            ; Na.. Just do ISA/EISA stuff.

;-------------------------------------------------------------------------
;       Initialize if an EISA bus is present.
;-------------------------------------------------------------------------
InitForEISA:
        mov     dx, offset CGroup:WrongBusMsg
        test    MSMSystemFlags, EISAFlag  ; Do we have an EISA Bus?
        jz      InitErrorExitJmp

        jmp short InitForISA            ; Well, If we have an EISA thats ok, Do ISA

;-------------------------------------------------------------------------
;       Check for Which bus the USER wants to look in.
;-------------------------------------------------------------------------
CheckForSupportedBUSIDs:
        cmp     BusID, BUS_ID_ISA       ; if ISA then do ISA
        je      InitForISA

        cmp     BusID, BUS_ID_EISA
        je      InitForEISA             ; ISA who Cares.

        cmp     BusID, BUS_ID_MCA       ; If MCA then Do MCA
        jne     InitErrorExitJmp

;-------------------------------------------------------------------------
;   Check for PS2 Machine : Scan slots and Fill Slot Info, and IOAddress
;-------------------------------------------------------------------------
        mov     dx, offset CGroup:WrongBusMsg
        test    MSMSystemFlags, MCAFlag  ; Are We PS2
        jz      InitErrorExitJmp

InitForMCA:
        call    ScanSlot        ; Stuffs the Slot Number, and IOAddress1
        jnz     SlotAddressError

;-------------------------------------------------------------------------
;         Check for Adapter Address given from Net.cfg
;-------------------------------------------------------------------------
InitForISA:
ifdef   RPL
; AYD/WLR 7-15-96 Extended Function Adapter, Enhanced Mode Support additions
        call  find_mmio_segment
        or    ax,ax
        jz    none_found
        call  check_for_if_table
        jnz   short IFtable
none_found:
endif
        mov   dx, IOAddress1    ; DX = Adapter PIO Reg @
        or    dx, dx            ; DX = 0000 ?? then no override
        jnz   TryThisOne        ; Try ONLY this Adapter

        mov   dx, PrimPortAddr  ; DX = PRImary Adapter PIO Reg @

;-------------------------------------------------------------------------
;         Release Adapter Reset to Prepare to Read MMIO Area
;-------------------------------------------------------------------------
TryThisOne:
        inc     dx              ; DX = Adapter Reset PIO @
        inc     dx              ; DX = Adapter Release Reset PIO @
        out     dx, al          ; Release Reset JUST in Case:: reg.al don't care
        dec     dx              ; DX = Adapter Reset PIO @
        dec     dx              ; DX = Adapter PIO Reg @   ( SetupRead1 )

;-------------------------------------------------------------------------
;         Decode the Interrupt Level and the ROM/MMIO Domain Address
;-------------------------------------------------------------------------
        in      al, dx            ; AL = ROM/MMIO Domain Jumper:  DX = SetupRead1 port
        mov     bx, 0FC01h        ; BX = Segment Mask (add low bit for bit 19)
        and     bh, al            ; BX = ROM/MMIO Domain Segment * 02
        and     al, 03            ; AL = Interrupt Level
        mov     ah, al            ; AH = Interrupt Level 00, 01, 02, or 03
        shl     al, 01            ; AL = Interrupt Level 00, 02, 04, or 06
        or      al, ah            ; AL = Interrupt Level 00, 03, 06, or 07
        or      al, 02            ; AL = Interrupt Level 02, 03, 06, or 07
        ror     bx, 01            ; BX = Domain @
;AYD 7-15-96 Added label and ror bx before mov to IRQLine
ifdef RPL
IFtable:
endif
        mov     IRQLine1, al      ; AL = Interrupt Level
        mov     MMIOSegment, bx   ; store Segment Value
        mov     es, bx            ; ES = BIOS/MMIO Domain Segment

;-------------------------------------------------------------------------
;       Store Absolute Domain Address for the MSM into the Config0 Table
;-------------------------------------------------------------------------
        mov     cl, 4                        ; shift counter
        shr     bx, cl                       ; BX = Absolute MMIO Domain Address /16
        xor     ax,ax                        ; Clear AX
        xor     cx,cx                        ; Clear CX
        mov     al,bh                        ; Position High Word value
        mov     ch,bl                        ; Position Low Word Value
        mov     word ptr MemAddress1, cx     ; CX = Absolute ROM/MMIO Domain @ Low Word
        mov     word ptr MemAddress1 + 2, ax ; AX = Absolute MMIO Domain @ High Word

;-------------------------------------------------------------------------
;               Check for the Presence of a Token-Ring Card
;-------------------------------------------------------------------------
        mov     si, OFFSET CGroup:ToknrSigMCA   ;DS:SI = Micro-Channel Signature
        cld                             ; makes sure lodsb below increments si.

CheckBusType:           ; Automatically changes from one sig to the next
        xor   ax, ax                    ; AX = 0000
        mov   cx, SIZE TRSMC            ; CX = Size of Signatures are the same
        mov   di, AIPChanSig            ; ES:DI = AIP Area : Channel Identifier @

CheckSigLoop:
        mov     al, byte ptr es:[di]    ; AL = Signature Byte
        and     al, 0Fh                 ; AL = Isolate LOW Order Nibble
        sub     al, [si]                ; Signature Match ??
        inc     si                      ; SI = Next Signature byte @
        inc     di                      ; MMIO Can ONLY Read EVEN Bytes
        inc     di                      ; DI = Next Even Byte MMIO
        or      ah, al                  ; AH = 00 ==> Signature Match
        loop    CheckSigLoop            ; CX = Signature Size

        lodsb                           ; AL = MCA ==> Micro-Channel Sig. End
        jz      CardFound               ; Card FOUND Jump

        cmp     al, BUS_ID_ISA          ; If ISA flag then no card found
        jnz     CheckBusType            ; Go Check for AT Bus

;-------------------------------------------------------------------------
;               Card NOT Found; Check for ALTERNATE Adapter
;-------------------------------------------------------------------------
        cmp     dx, IOAddress1          ; DX = PORT Specified ?? (changed via net.cfg and MSM)
        jz      NoNic                   ; z => Address was given and not correct...exit error

        cmp     dx, AltPortAddr         ; Check to see if we have tried the Alternate already
        mov     dx, AltPortAddr         ; DX = ALTERNATE PIO Reg @
        jnz     TryThisOne              ; nz => try the alternate address this time

NoNic:
        mov     IRQLine1, cl                     ; Clear the Int Line Field
        mov     word ptr MemAddress1, cx         ; Clear the Memory fields
        mov     word ptr MemAddress1 + 2, cx     ; Clear the Memory Fields
        mov     dx, OFFSET CGroup:NICNotFoundMsg ;'Token-Ring Signiture Hardware Failure'
        jmp     InitErrorNoInts                  ; Return to MSM with Error

;-------------------------------------------------------------------------
;         Card FOUND; AL = Micro-Channel Switch Value = 0 (BUS_ID_ISA) or 1 (BUS_ID_MCA)
;-------------------------------------------------------------------------
CardFound:
        test    MSMSystemFlags, EISAFlag ; See if there is an EISA BUS present
        jz      SetBusID                ; if not then AL is the correct BUS ID Value.

        cmp     al, al                  ; Check if we might be in an EISA bus.
        jnz     SetBusID                ; IF we found a Card in the MCA bus thats fine.

        mov     al, BUS_ID_EISA         ; Then it must be an EISA bus.

SetBusID:
        mov     BusID, al               ; Set which Bus ID we are using.
        mov     IOAddress1,dx           ; DX = Adapter PIO Reg @ (Save it)

;-------------------------------------------------------------------------
;           Set PIO Port Addresses in memory
;-------------------------------------------------------------------------
        mov     cx, NumIOPorts
        mov     si, offset CGroup:SetupRead1

SetupPortLoop:
        add     word ptr [si], dx       ; Add in base I/O port
        inc     si
        inc     si                      ; next Defined port (add 2 bytes)
        loop    SetupPortLoop

        mov     bx, 4                   ; CL = Default Shared Ram Factor Mask
        and     bl, dl                  ; CL = Default Shared RAM Factor (4 or 0)
        cmp     al, BUS_ID_MCA          ; AL = either MCA or ISA :Pick The Bus
        jne     CheckDefaultSharedRam   ; AT Bus Jump

;-------------------------------------------------------------------------
;         On The Micro-Channel, the Shared RAM Segment is in A22/A26 PIO
;-------------------------------------------------------------------------
        inc     dx                              ; DX = Adapter Reset PIO @
        inc     dx                              ; DX = SetupRead2 -> Adapter PS/2 Shared Ram PIO @
        in      al, dx                          ; AL = Shared RAM Segment Mask
        and     al, 0FEh                        ; Turn Off Low Order Bit
        xchg    ah, al                          ; AX = Shared RAM Segment
        mov     SharedRamSegment, ax            ; Save Shared Ram Segment
        mov     cl, 4                           ; shift counter
        shr     ax, cl                          ; Shift Address for Disection
        xor     cx, cx                          ; Clear CX
        xchg    cl, ah                          ; CX: High Word
        xchg    ah, al                          ; AX: Low Word
        mov     word ptr MemAddress2, ax        ; Save Shared Ram Segment Low Word
        mov     word ptr MemAddress2 + 2, cx    ; Save Shared RAM Segment high Word
        cmp     IRQLine1, 06                    ; Micro-Channel Interrupt Level = 10 or 11 ??
        jb      CheckRamBoundary                ; Interrupt Level OK
        add     IRQLine1, 04                    ; Set the Micro-Channel Interrupt Level

;-------------------------------------------------------------------------
;                       Check for DEFAULT Shared RAM
;-------------------------------------------------------------------------
CheckDefaultSharedRam:
        mov     ax, word ptr MemAddress2        ; AX = Shared RAM Absolute Low Word
        mov     dx, word ptr MemAddress2 + 2    ; DX = Shared Ram Absolute High Word
        or      ax, dx                          ; AX = 0000 ??
        jz      CheckForISA_PaP                 ; AX = NOT Default Value

        mov     cx, word ptr MemAddress2        ; CX = Shared RAM Absolute Low Word
        mov     ax, word ptr MemAddress2 + 2    ; AX = Shared Ram Absolute High Word
        mov     dh, al                          ; Reverse High Word
        mov     dl, ch                          ; Reverse low word
        mov     cl, 4                           ; Shift Counter
        shl     dx, cl                          ; DX = Shared Ram Segment Value
        mov     SharedRamSegment, dx            ; Save a Copy for use later
        jmp  short CheckRamBoundary             ; Now go check the Boundaries

CheckForISA_PaP:
        test    es:RRReven, 0FFh                ; Check MMIO:RRReven on 0
        jnz     AdapterOpen

        jmp     short SetDefaultSharedRam

AdapterOpen:
        xor     ax,ax
        mov     ah, es:RRReven
        mov     SharedRamSegment,ax

; AYD/WLR 3-18-96 Extended Function Adapter, Enhanced Mode Support additions
; Must not adjust IRQ if value is from IF table
        test    LOCALFlags,IFtableFound
        jnz     SetMemAddress

        cmp     IRQLine1, 06                    ; Interrupt Level = 10 or 11 ??
        jb      SetMemAddress

        cmp     byte ptr es:1FBAh,0Eh           ; If true, ISA P&P. IRQ 2,3,
        jne     SetMemAddress                   ; 10,11 supported

        add     IRQLine1, 04                    ; Interrupt Level for ISA P&P
        jmp     short SetMemAddress

SetDefaultSharedRam:
        mov     ax, SharedRamDefault$           ; AX = Shared RAM Default Address
        sub     ah, bl                          ; AX = D800h for PRI/D400h for ALT
        mov     SharedRamSegment, ax            ; AX = SHared Ram Segment

SetMemAddress:
        mov     cl, 4                           ; Shift counter
        shr     ax, cl                          ; Shift Address for Disection
        xor     cx, cx                          ; Clear CX
        xchg    cl, ah                          ; CX: High Word
        xchg    ah, al                          ; AX: Low Word
        mov     word ptr MemAddress2, ax        ; Save Shared Ram Segment Low Word
        mov     word ptr MemAddress2 + 2, cx    ; Save Shared RAM Segment high Word

;-------------------------------------------------------------------------
;         Validate that Shared RAM is on Correct Boundary
;-------------------------------------------------------------------------
CheckRamBoundary:
        mov     ax, SharedRamSegment            ; AX = Shared RAM Segment
        mov     cx, SharedRamSizeMask           ; CL = Shared RAM Size Mask
        and     cl, es:RRRodd                   ; CX = 00, 04, 08, or 12  ES: MMIO
        shr     cx, 1                           ; CX = 00, 02, 04, or 06  leave doubled for word move
        mov     si, cx                          ; SI = Shared RAM Size Index
        mov  cx, word ptr SharedRamLookup[si]   ; CX = Shared RAM Size (# para.)
        mov  word ptr MemRange2, cx             ; CX = Shared RAM Size (# para.)
        dec     cx                              ; set up Test Mask Value (Ram size -1)
        cmp     ch, 01                          ; Shared Ram 8k??
        jne     RamAddressTest                  ; 8k Shared RAM => e

        mov   DIROpenAdapter.OpenNumTxBuf,ch    ; CH = 01 Transmit Buffer (For 8K cards)

RamAddressTest:
        test    ax,cx                           ; AX = Shared RAM Address
        jz      SharedRamOK                     ; Shared RAM is OK

RamBoundaryError:
        mov     dx, OFFSET CGroup:SharedRamBoundaryErr ;'Shared RAM is on Incorrect Boundary'
        or      ax, 0F0h                               ; return error code
        jmp     InitErrorNoInts                        ; return to MSM errorcode

;-------------------------------------------------------------------------
;         Check for BIOS/MMIO Domain and Shared RAM OVERLAP
;-------------------------------------------------------------------------
SharedRamOK:
; AYD/WLR 7-15-96 Extended Function Adapter, Enhanced Mode Support additions
; skip this check when in Enhanced Mode because they ROM=RAM Segment
        test    LOCALFlags,EnhancedMode
        jnz     ValidRamLoadMSM

        mov     dx, es                  ; DX = BIOS/MMIO Domain @
        mov     bx, ax                  ; BX = Shared RAM Address
        sub     ax, dx                  ; AX = Shared RAM - BIOS/MMIO Domain @
        jnc     SharedRamLarger         ; Shared RAM > BIOS/MMIO Jump

        neg     ax                      ; AX = BIOS/MMIO Domain - Shared RAM
        inc     cx                      ; CX = Actual Shared RAM Size
        cmp     ax, cx                  ; BIOS/MMIO Domain - Shared RAM > Size Shared Ram
        jc      RamOverlapError         ; Invalid Shared RAM Jump

SharedRamLarger:
        cmp     ax, 2000h/10h           ; Shared RAM - BIOS/MMIO > 8k/16 ???
        jnc     ValidRamLoadMSM         ; Invalid SHARED RAM  fall through

RamOverlapError:
        or      ax, 0F0h
        mov     dx, offset CGroup:SharedRAMOverlapError
        jmp     short InitErrorNoInts           ; Return to the MSM with error set

;-------------------------------------------------------------------------
;       Valid Ram, Load MSMLookAheadSegment value as required
;-------------------------------------------------------------------------
ValidRamLoadMSM:
        mov  word ptr MSMLookAheadSegment, bx   ; Tell MSM about our look ahead seg value (Shared Ram)

;-------------------------------------------------------------------------
;       Fill In the Adapter's Encoded Address (Burned In Address)
;               Check for command line node address override.
;-------------------------------------------------------------------------
        mov     ax, word ptr MSMPhysNodeAddress    ; get first word of Address
        and     ax, word ptr MSMPhysNodeAddress +2 ; and it with 2nd word
        and     ax, word ptr MSMPhysNodeAddress +4 ; and it with the 3rd Word : Got all 6 bytes
        cmp     ax, -1                             ; Have we been all F's
        jne     NodeOverride                       ; Not Equal then Use Tabled Value

;-------------------------------------------------------------------------
; Read address from AIP Area.  There was not a Net.CFG override.
;-------------------------------------------------------------------------
        mov     bp, ds                          ; save DS for later (CGroup Segment)
        mov     es, bp                          ; ES: CGroup segment
        mov     ds, dx                          ; DS = MMIO Domain Segment
        mov     ch, 6                           ; Number of bytes to read
        mov     cl, 03*04                       ; num bits to shift nibble from al low to ah high
        mov     di, offset CGroup:MSMPhysNodeAddress ; Restore pointer to NODE field
                                                ; ES:DI => MSMPhysNodeAddress
        mov     si, AIPEncodedAddr              ; DS:SI => AIP Area : NODE ADDRESS

GetSerialNumber:
        lodsb
        shl     ax, cl                  ; AH = MS Nibble
        inc     si                      ; DS:SI =>  Skip odd bytes
        lodsb                           ; get next even byte
        and     al,0Fh                  ; Mask off useless nibble
        or      al,ah                   ; AL = Combined byte
        stosb                           ; ES:DI => MSMPhysNodeAddress
        inc     si                      ; DS:SI => AIP Area, but remember : skip odd bytes
        dec     ch                      ; loop counter
        jnz     GetSerialNumber

        mov     ds, bp                  ; DS = CGroup Segment (restore)

;-------------------------------------------------------------------------
;         Insert the HardWare Interrupt Vector
;-------------------------------------------------------------------------
NodeOverride:
        cli
        call    MSMSetIRQ

;-------------------------------------------------------------------------
;         Issue DIR.INITialize to GET the Adapter ON Line
;-------------------------------------------------------------------------
        call    InitNic                 ; Initialize the Card
        jnz     InitializationError     ; nz = Error in Init. of Card, DX: Message PTR

;-------------------------------------------------------------------------
;                       Initialize AES Event
;-------------------------------------------------------------------------
        cli                                             ; Hold off on ints for now
        push    cs
        pop     es                                      ; ES: CGroup
        mov     si, offset CGroup:NICAES                ; ES:SI -> AESECB block
        mov     word ptr NICAES.AESESR, offset CGroup:NICAESproc ; Store it for now
        mov     word ptr NICAES.AESESR + 2, es          ; Store it for now
        mov     word ptr NICAES.MSecondValue, 2000      ; initial Length of time between events = 2 seconds

;-------------------------------------------------------------------------
;                       Schedual AES Events
;-------------------------------------------------------------------------
        mov     bx, MLIDSUP_SCHEDULE_AES_EVENT  ; Line up to schedule the event
        call    LSLSupport
        mov     al, DriverMaxActiveTx
        mov     MSMTxFreeCount, al              ; Tell MSM we have a clean slate
        xor     ax, ax                          ; Guarentee  Return Code
        and     DriverStateFlag, NOT (DriverInitFlagBit)
                                       ; Clear State Flag as out of DriverInit
        jmp     MSMMediaConfigUpdate            ; No error here, pass on

InitializationError:
        push dx
        call    MSMUnSetIRQ                     ; Remove Interrupt and Vector
        pop dx

InitErrorNoInts:
        mov     ax, MSMERR_HSM_FAILED           ; Return Error
        or      ax, ax                          ; Set appropriate flags
        ret

DriverInit      endp

; AYD/WLR 7-15-96 Extended Function Adapter, Enhanced Mode Support additions
;  entry: none
;  exit: ax - 0 = no adapters found, 1 = adapter found
;        bx - mmio segment if adapters found
find_mmio_segment proc near
        mov     dx,40h                          ;=Segment incrementer      RAR3
        mov     bx,0c7C0h                       ;=Segment search start-40h RAR3

PortQueryOuter:                 ;NIC locater outer loop entry point        RAR3

        add     bx,dx                           ;=MMIO segment to search   RAR3
        cmp     bx,0DE40h                       ;Beyond last possible seg? RAR3
        jge     SearchPortsComplete             ;If yes Jump               RAR3

        mov     es,bx                           ;step to next segment      RAR3
        xor     ax,ax                           ;initialize work reg       RAR3
        mov     cx,SIZE TRSPC                   ;=PC BUS signature length  RAR3
        mov     di,AIPChanSig                   ;=> possible AIP Ch ID     RAR3
;       mov     si,offset ToknrSigPC            ;=> PC BUS signature const RAR3
        mov     si, OFFSET CGroup:ToknrSigPC    ;DS:SI = Micro-Channel Signature

PortQueryInner:                 ;NIC locater inner loop entry point        RAR3

        mov     al,byte ptr es:[di]             ;=possible signature byte  RAR3
        and     al,0Fh                          ;only lo nibble is valid   RAR3
        sub     al, byte ptr [si]               ;compare signature byte    RAR3
        jnz     PortQueryOuter                  ;If not a match - jump     RAR3

        inc     si                              ;=>next signature byte con RAR3
        inc     di                              ;=>next AIP signature byte RAR3
        inc     di                              ;(only even addr's valid)  RAR3
        loop    PortQueryInner                  ;go test next byte if cx>0 RAR3
;************************************************************************* RAR3
;* Having reached this point means that a valid PC (ISA) bus token ring  * RAR3
;* signature has been located and its segment value is contained in the  * RAR3
;* bx register.                                                          * RAR3
;************************************************************************* RAR3
; See if we are in enhanced mode (RAM Segment = MMIO Segment).  If so,
; Don't indicate a card is found.
        xor    ax,ax
        mov    ah,es:RRReven
        cmp    ax,bx
        jne    PortQueryOuter
        mov    ax,1
        jmp    foundtr
SearchPortsComplete:
        xor    ax,ax
foundtr:
        ret
find_mmio_segment endp

;* WLR 3-18-96 Extended Function Adapter, Enhanced Mode Support additions
; On entry bx points to MMIO address
; On exit zf set if IF Table not found. If zf set(IF Table found):
;   al = IRQ
;   dx = I/O address
check_for_if_table proc    near
        mov  es,bx
        mov  MMIOSegment,bx      ; needed in call below to pagingset
        push bx        ; save bx (MMIO Segment)
        xor  ax,ax
        mov  ah,es:RRReven
        sub  ax,bx
        jnz  NOTEnh
        or      LOCALFlags,EnhancedMode
NOTEnh:
ifdef Debug
        call print_DI_msg
endif ;Debug
        ; Disable Interrupts on the adapter
        cli                                ; don't want an IRQ when I write it
        mov  es:ISRPeven, NMIIntControlBit
        mov  es:TCReven, 0           ; 1E0Ch clear timer register
        sti
        ; See if adapter may support IF table (AIP offset 1fbc < 0fh)
        mov  di,1fbch      ; AdapterType Offset in AIP
        mov  al,es:[di]
        cmp  al,0fh
        je   NoIFTable      ; All but type of 0fh can have IF table
        ; Reset the Solo processor on the Adapter
ifdef Debug
        call print_1exx   ; debug
endif ;Debug

        mov  es:RRRodd, 10h
        ; zero mmio:1e0f-1e01
        xor  al,al
        mov  cx,15
        std
        mov  di,TVRodd
rep     stosb
        ; Delay 200 microseconds or more
        xor  cx,cx
;       mov  dx,500h     ;
;       mov  cx,4ch
        mov  dx,2000     ;
        mov  ah,86h      ; wait function for Int 15
        int  15h
        ; See if WRBR is zero.  If it is not, then IF Table exists
        mov  bx,WORD PTR es:WRBReven ; Get WRBR address
ifdef Debug
        call print_1exx
endif ;Debug
        or   bx,bx        ; Set zero flag if no IF table
        jz   NoIFTable
        xchg bl,bh
        push ds
        ; set paging register
        mov  ax,bx
        push es
        push bx
        call PagingSet
        pop  bx
        pop  es

        mov  ah,es:RRReven
        xor  al,al
        mov  ds,ax
        mov  dx,ds:[bx+4]  ; dx = I/O address
        xchg dl,dh
        mov  al,ds:[bx+3]  ; al = IRQ address
        pop  ds
ifdef Debug
        call print_ioint
        call print_if_msg
endif ;Debug
        or   LOCALFlags,IFtableFound
        or   bx,bx         ; Reset Zero Flag (bx is not zero here)
NoIFTable:
        ; See if WRBR is zero.  If it is not, then IF Table exists
        pop  bx            ; bx = MMIOSegment
        ret
check_for_if_table endp

ifdef Debug
; print Disable Interupts message
di_msg db "Disable Interupts", 0Ah, 0Dh
di_msg_len dw 19
print_DI_msg proc near
        push es                       ; Save es, cx, bp
        push cx
        push bp

        push cs
        pop  es                       ; es = cs
        mov  bp, offset CGroup:di_msg ; es:bp points to string to wirte
        mov  cx, di_msg_len           ; length of string to print
        call debug_print              ; print string on display

        pop  bp                       ; Restore bp, cx, es
        pop  cx
        pop  es
        ret                           ; Return
print_DI_msg endp
; debug proc to print IF table found msg
;          12345678901234
if_msg db "Found IF Table", 0Ah, 0Dh
if_msg_len dw 16
print_if_msg proc near
        push es                       ; Save es, cx, bp
        push cx
        push bp

        push cs
        pop  es                       ; es = cs
        mov  bp, offset CGroup:if_msg ; es:bp points to string to wirte
        mov  cx, if_msg_len           ; length of string to print
        call debug_print              ; print string on display

        pop  bp                       ; Restore bp, cx, es
        pop  cx
        pop  es
        ret                           ; Return
print_if_msg endp

; print_ioint proc to print IO address and Interrupt from IF table
; Entry: es: mmio segment
;        dx: I/O address
;        al: Interrupt
; Exit: all registers the same
;
;             1234567890123456   7   8
msg_ioint db "IO:      Int:   ",0ah,0dh
msg_ioint_len dw 18
print_ioint  proc near
        push es                       ; Save es, cx, bp
        push ax
        push bx
        push cx
        push dx
        push bp

; es mmio segment
; di 1exx
; ds = cs (data segment)
; bp = pointer into data to put char register data
; cx count of registers
; dx hex word
; ax, dx char word

        mov  bp, offset CGroup:msg_ioint ; Point to Datat to be printed
        push ax
        call hex2char  ; dx word to ax,dx chars returned
        mov  WORD PTR ds:[bp+6],ax       ; Store I/O address into data msg
        mov  WORD PTR ds:[bp+4],dx
        pop  dx
        call hex2char  ; dx word to ax,dx chars returned
        mov  WORD PTR ds:[bp+14],ax      ; Store interrupt into data msg
; print debug stuff
        push cs
        pop  es                          ; es = cs
        mov  bp, offset CGroup:msg_ioint; es:bp points to string to wirte
        mov  cx, msg_ioint_len           ; length of string to print
        call debug_print                 ; print string on display

        pop  bp
        pop  dx
        pop  cx
        pop  bx
        pop  ax
        pop  es
        ret                           ; Return
print_ioint  endp

; print_1exx proc to print 1E00-1E0A, 1E18, and data at offset in RAM found
; in 1E02
; Entry: es: mmio segment
; Exit: all registers the same
;
;1E00 1E02 1E04 1E06 1E08 1E0A
;                     1         2         3
;            1234567890123456789012345678901234   5  6
msg_1exx db "1E00 1E02 1E04 1E06 1E08 1E0A 1E18",0ah,0dh
data_1exx db 35 dup(' '), 0Ah, 0Dh
msg_1exx_len dw 73
print_1exx   proc near
        push es                       ; Save es, cx, bp
        push ax
        push bx
        push cx
        push dx
        push bp

; es mmio segment
; di 1exx
; ds = cs (data segment)
; bp = pointer into data to put char register data
; cx count of registers
; dx hex word
; ax, dx char word

        mov  di,1e00h
        mov  cx, 7
        mov  bp, offset CGroup:data_1exx ; where data is to update

next_reg: ; read 6 registers and put them into data array
        mov  dx,WORD PTR es:[di] ; Get WRBR address
        call hex2char  ; dx word to ax,dx chars returned
        mov  WORD PTR ds:[bp],ax
        mov  WORD PTR ds:[bp+2],dx
        add  bp,5
        add  di, 2
        cmp  di,01e0ch
        jne  not18
        add  di,12               ; print regiset 1e18 after 1e0a
not18:  loop next_reg
; print debug stuff
        push cs
        pop  es                         ; es = cs
        mov  bp, offset CGroup:msg_1exx ; es:bp points to string to wirte
        mov  cx, msg_1exx_len         ; length of string to print
        call debug_print              ; print string on display

        pop  bp
        pop  dx
        pop  cx
        pop  bx
        pop  ax
        pop  es
        ret                           ; Return
print_1exx   endp

; input dx = word
; output ax,dx = char word
hex2char proc near
        push cx

        push dx                      ; first char
        call onexchar
        mov  ah,dl
        pop  dx
        shr  dx,4

        push dx                      ; Second char
        call onexchar
        mov  al,dl
        pop  dx
        shr  dx,4

        push dx                      ; Third char
        call onexchar
        mov  cl,dl
        pop  dx
        shr  dx,4

        call onexchar                ; Fourth char
        mov  dh,cl

        pop  cx
        ret                           ; Return
hex2char endp

onexchar proc near
        and  dx,000fh
        cmp  dl,09
        jle  digit
        add  dl, 37h
        jmp  cdone
digit:  add dl,30h
cdone:  ret                           ; Return
onexchar endp

; Entry: es:bp - points to string to print
;           cx - length of string to print
;  Exit: All registers preserved
debug_print proc near
        push ax
        push bx
        push dx

        push cx       ; save length of string to print
        mov  ah, 0fh  ; read current video state
        int  10h      ; sets bh to active page
        mov  ah, 03h  ; read cursor position (clobbers cx)
        int  10h      ; sets dx to row, col where cursor is

;       push cs
;       pop  es                 ; es = cs
;       mov  bp, offset CGroup:if_msg  ; es:bp points to string to wirte
        pop  cx                 ; restore length of string to print
        mov  bl, 07h            ; attribute to write
        mov  ax, 1301h          ; write string
        int  10h                ; Write string

        pop  dx
        pop  bx
        pop  ax
        ret
debug_print endp
endif ;Debug


;**********************************************************************
;       Dir.Initialize:=>  InitNic Procedure to Set-up Card
;**********************************************************************
;       Assumes:  DS: CGroup

ifdef RPL
NormalInitNicJMP:
        jmp     NormalInitNic
endif

InitNic proc    near

ifdef RPL
;-------------------------------------------------------------------------
; To save a ring insertion during the RPL process the version of the HSM
; that is being executed from with in the TOKEN.RPL bootstrap does not reset
; and open the adapter.  Since the card is up and running all we need to do
; is set the driver up to match the state of the card.
;-------------------------------------------------------------------------
        mov     es, MMIOSegment         ; ES -> MMIO area
        mov     es:TCReven, 0           ; 1E0Ch clear timer register we don't use it.
        test    es:ISRPeven, 40h        ; test Access Interrupt bit at 1E08h.
        jz      short NormalInitNicJMP  ; If problem open adapter.

        mov     cx, word ptr es:WWOReven; Non zero value at 1E04h means adapter is open.
        jcxz    short NormalInitNicJMP  ; If zero open adapter.

        mov     al, es:RRRodd                   ; read RRR-Odd  ES : MMIO Base
        and     al, 0Ch                         ; get bits 3 & 2 mask off others
        mov     ah, 100h-1                      ; Disable paging for 8K & 64K.
        cmp     al, SHARED_RAM_8K               ; 8K?
        je      short RPLSetPageMask            ; Jump if so.

        cmp     al, SHARED_RAM_64K              ; 64K?
        je      short RPLSetPageMask            ; Jump if so.

        mov     cs:PageSize, 8000h              ; Use 32K Page Size.
        mov     cs:PageBreakMask, 80h
        mov     cs:PageIncrement, 80h
        mov     ah, 80h-1
        cmp     al, SHARED_RAM_32K              ; 32K?
        je      short RPLSetPageMask            ; Jump if so.

        mov     cs:PageSize, 4000h              ; Use 16K Page Size.
        mov     cs:PageBreakMask, 0C0h
        mov     cs:PageIncrement, 40h
        mov     ah, 40h-1                       ; Must be 16K

RPLSetPageMask:
        mov     cs:PageSizeMask, ah             ; Save Page Size Mask.

        mov     al, byte ptr es:WRBReven
        or      al, 10h
        mov     byte ptr cs:SRBAddress+01, al    ; Save SRB page (FDF4h or FFF4h).
        mov     byte ptr cs:ASBAddress+01, al    ; Save ASB value (FDD8h or FFD8h).

;-------------------------------------------------------------------------
; Get arb address from rpl data area at 60:16
;-------------------------------------------------------------------------
        push    es                         ; save es
        push    di                         ; save di
        mov     ax,60h
        mov     es,ax                      ; set es to 60h
        mov     di, 16h                    ; arb offset in rpl data area
        mov     ax, word ptr es:[di]       ; get arb address
        mov     word ptr cs:ARBAddress,ax  ; save arb address
        pop     di                         ; restore di
        pop     es                         ; resotre es

        mov     ax, 240
        mov     MaxFrameSize, ax        ; set config table buffer size
        add     ax, 6                   ; Add in Adapter Overhead
        mov     cx, ax                  ; Get Copy and see if multiple of eight
        and     cx, 0007h               ; Strip off for last nibble
        jcxz    RPLAlreadyMultipleOf8   ; OK Size

        add     ax, 7                   ; Adjust to be nearest multiple of eight above.
        and     al, NOT 07              ; Round down to nearest multiple of eight

RPLAlreadyMultipleOf8:
        mov     si, offset CGroup:DIROpenAdapter
        xchg    ah, al                  ; AX = (High Low Format) Transmit Buffer Size
        mov     word ptr [si].OpenLengthTxBuf, ax       ; AX = DIR.Open.Adapter DHB Size  (high-low format)
        mov     RPLFlag, -1
        mov     es, MMIOSegment                 ; ES -> MMIO area
        jmp     LoadNodeAddress

NormalInitNic:

endif
        mov     InterruptStateFlag, IgnoreAllInterrupts+AdapterShutdownBit+WaitingIntBit
        sti
        mov     es, MMIOSegment                 ; ES -> MMIO area
        mov     byte ptr es:ISRPResetEven, NOT (NMIIntControlBit+InterruptEnableBit)

;-------------------------------------------------------------------------
;       Pull the Adapter Reset Line
;-------------------------------------------------------------------------
PulledReset:
        cli                                             ; IF = 0 ==> No Interrupts
        mov     dx, AdapterReset                        ; Adapter Reset port
        or      InterruptStateFlag, AdaptHeldReset      ; Adapter Being Held Reset
        out     dx, al                                  ; Hold The Adapter Reset For A While

;-------------------------------------------------------------------------
;       Wait For At Least 50 Mills
;-------------------------------------------------------------------------
        mov     bp, 100
        call    WasteTime       ; destroys ax,bx,cx,dx and does STI

;-------------------------------------------------------------------------
;       Pull the Adapter Release
;-------------------------------------------------------------------------
        mov     dx, AdptrRelease_SetupRead2     ; DX = Release Adapter Reset PIO @
        cli                                     ; IF = 0 ==> No Interrupts
        out     dx, al                          ; Release Adapter Reset   AL = Don't Care
        mov     InterruptStateFlag, IgnoreAllInterrupts + WaitingIntBit + AdapterShutdownBit

;-------------------------------------------------------------------------
;               Set up The Adapter for Paging if specified
;-------------------------------------------------------------------------
 SetAdapterPage:
        mov     al, es:RRRodd                   ; read RRR-Odd  ES : MMIO Base
        and     al, 0Ch                         ; get bits 3 & 2 mask off others
        mov     RAMSize, al                     ; Save for Fast Path Init.
;AYD/WLR 3-18-96 Extended Function Adapter, Enhanced Mode Support additions
;add 4K paging stuff in enhanced mode
        test    LOCALFlags,EnhancedMode
        jz      NotEnhP
        mov     byte ptr es:SRPReven, MMIOPagingActiveFlag      ; Set SRPR bits  ES:= MMIO Base
        mov     cs:PageSize, 1000h              ; Use 16K Page Size.
        mov     cs:PageBreakMask, 0F0h
        mov     cs:PageActFlg, 1Fh              ; Used to know if Paging is used
        mov     cs:PageIncrement, 10h
        mov     ah, 10h-1                       ; Must be 4K
        jmp     short SetPageMask               ; Jump if so.

NotEnhP:

        mov     ah, 100h-1                      ; Disable paging for 8K & 64K.
        cmp     al, SHARED_RAM_8K               ; 8K?
        je      short SetPageMask               ; Jump if so.

        cmp     al, SHARED_RAM_64K              ; 64K?
        je      short SetPageMask               ; Jump if so.

        mov     byte ptr es:SRPReven, MMIOPagingActiveFlag      ; Set SRPR bits  ES:= MMIO Base
        mov     cs:PageSize, 8000h              ; Use 32K Page Size.
        mov     cs:PageBreakMask, 80h
        mov     cs:PageActFlg, 7Fh              ; Used to know if Paging is used
        mov     cs:PageIncrement, 80h
        mov     ah, 80h-1
        cmp     al, SHARED_RAM_32K              ; 32K?
        je      short SetPageMask               ; Jump if so.

        mov     cs:PageSize, 4000h              ; Use 16K Page Size.
        mov     cs:PageBreakMask, 0C0h
        mov     cs:PageActFlg, 3Fh              ; Used to know if Paging is used
        mov     cs:PageIncrement, 40h
        mov     ah, 40h-1                       ; Must be 16K

SetPageMask:
        mov     cs:PageSizeMask, ah             ; Save Page Size Mask.

;-------------------------------------------------------------------------
;       Set the RRReven Register (after the Adapter Reset cleared it)
;-------------------------------------------------------------------------
        test    MSMSystemFlags, MCAFlag         ; Check to see if MCA bus (PS2)
        jnz     SetISRPeven                     ; on Micro-channel RRReven is RESERVED

        mov     cx, SharedRamSegment            ; CX = Shared Ram Address
        mov     es:RRReven, ch                  ; Write Segment Value out to RRReven. ES:=MMIO Base

;-------------------------------------------------------------------------
;               Set ISRPeven    InitEnable Bit (bit 6)
;-------------------------------------------------------------------------
SetISRPeven:
        mov     byte ptr es:ISRPSeteven, NMIIntControlBit+InterruptEnableBit  ; ES:=MMIO Base

;-------------------------------------------------------------------------
;       Wait For At Least 72 ticks (4 seconds) for Interrupt to occur
;-------------------------------------------------------------------------
        mov     bp, 4*1000              ; Wait 4 seconds
        call    WasteTime               ; BP holds time to waite
        jz      InitCompletedInterrupt  ; Z= Int has occured

        mov     dx, offset CGroup:CardIsDeadMsg ; The DIR.Initialize FAILED
        jmp     ErrorInitNIC                    ; Go Process Error

;-------------------------------------------------------------------------
;               Read WRBR to Get Initial location of SRB
;               After Int has occurred see ISRPodd bit 5
;-------------------------------------------------------------------------
InitCompletedInterrupt:                  ; Internal Initialization completed, ES:=MMIO base
        mov     bx, word ptr es:WRBReven ; read the Write Region Base high byte (MSB)
        xchg    bh, bl                   ; allow for IBM Format
        mov     word ptr SRBAddress, bx  ; bx: save Shared RAM SRB Relative @
        mov     es:SRPReven, bh          ; set Current Page   ES:=MMIO Base
        and     bh,PageSizeMask          ; get Relative Page
        mov     si, es                   ; Save ES : MMIO
        mov     es, SharedRamSegment     ; ES := Shared Ram Segment
        cmp     byte ptr es:[bx].InitSRBCmd, SRBCmdDIRInitComplete      ; Initialize Complete ??
        jz      NICInitComplete

        dec     AdaptRetryCnt           ; Try at least two times to get it going
        jz      FailedInit              ; Out of Retries

        jmp     InitNic                 ; No, loop to retry init

FailedInit:
        mov     dx, offset CGroup:RetryLimitMsg
        jmp     ErrorInitNIC

;-------------------------------------------------------------------------
;       Check for any errors in the Bring-up Code field
;-------------------------------------------------------------------------
NICInitComplete:                        ; ES : Shared Ram Segment;   BX : SRB Relative Address
        mov     al, byte ptr es:[bx].InitSRBError+01    ; AL = Bring Up Error Code
        or      al, byte ptr es:[bx].InitSRBError       ; AL = 00 ==> Success ??
        jz      LoadDHBTables                           ; DIR.Initialize Passed

        push    ds
        pop     es                                      ; ES: CGroup
        mov     di, offset CGroup:BringUpErrorValMsg
        mov     ah, al                                  ; copy into both registers
        and     ax, 0FF0h                               ; get high nibble in al, and low nible in ah
        mov     cl, 4
        shr     al, cl                                  ; get to correct order
        call    MSMGetHexChar                           ; get high nibble
        stosb
        xchg    al, ah                                  ; swap who's who
        call    MSMGetHexChar                           ; get Low Nibble
        stosb                                           ; add it to the string
        mov     dx, offset CGroup:BringUpErrorMsg
        jmp     ErrorInitNIC                            ; DIR.Initialize FAILED


;-------------------------------------------------------------------------
;       Load DHB Lookup table for correct Data Rate
;-------------------------------------------------------------------------
LoadDHBTables:
        mov     al, byte ptr es:[bx].InitSRBStatus
        mov     InitStatus, al                  ; Save Initialization status for later references

        test    byte ptr es:[bx].InitSRBStatus, SRBResponseDataRate     ; Data Rate = 16 Mbps ??
        mov     bp, es                          ; BP: => saved ES: Shared Ram Segment
        mov     es, si                          ; ES: MMIO base
        mov     dl, byte ptr es:AIPTotalRam     ; Load here, ES is still good for MMIO for ZeroTop512 test
        jz      Set4MbpsBuffers                 ; Data Rate =  4 Mbps

Set16MbpsBuffers:                               ; Set Buffer indexes for 16Mbps
        mov     LineSpeed, 0010h                ; Set Line Speed in Config Table
        mov     al, es:AIPDHBat16Mbps           ; Get Buffer Index for 16Mbps
        lea     di, DHB16MbpsTable              ; get offset to 16 Mbs Table
        jmp     short SetBuffers

Set4MbpsBuffers:                                ; Set Buffer indexes for 4 Mbps
        mov     LineSpeed, 0004h                ; Set Line Speed in Config Table
        mov     al, es:AIPDHBat4Mbps            ; Get Buffer Index for 4Mbps
        lea     di, DHB4MbpsTable               ; get offset to 4 Mbs Table

;-------------------------------------------------------------------------
;       Calculate Table index    ---   Get Maximum Buffer Size
;-------------------------------------------------------------------------
SetBuffers:
        mov     es, bp                          ; restore ES : Shared RAM Segment
        xor     ah, ah                          ; AL = MAX DHB Size Index Available
        and     al, 0Fh                         ; AL = Isolated DHB Size Available (low Nibble)
        sub     al, 16 - (DHBXMbpsTableSize/2)  ; AL = Maximum DHB Table index
        jnc     ValidBufRange                   ; AL = Valid Range Jump

        xor     ax, ax                          ; AX = Maximum DHB value in Table

ValidBufRange:
        shl     ax, 1                           ; allow for word offset (val * 2)
        add     di, ax                          ; DI -> DHB Size
        mov     ax, MaxFrameSize                ; get Max. User Data Size from Config0 Table
        add     ax, 6 + 7                       ; Account for the DHB overhead
                                                ; Adjust to be nearest multiple of eight above.
        and     al, NOT 07                      ; Round down to nearest multiple of eight
        cmp     ax, word ptr [di]               ; Max User Size >= Max Adapt.'s Size ?
        jbe     SetMaxPktSize                   ; if User Size is Smaller, then jump

;-------------------------------------------------------------------------
;       Max Frame Size was too Large for the Adapter at this speed.
;-------------------------------------------------------------------------
        mov     ax, word ptr [di]       ; set to Max Board Frame Size

        test    DriverStateFlag, MaxFrameFromNetCFGBit ; Did User tell us
        jz      SetMaxPktSize           ; NZ => User spec'd size tell him error.

        push    ax                      ; save for setting DHB.
        push    dx                      ; Save    (AIPTotalRam)
        sub     ax, 6                   ; Account for Overhead
        mov     dx, Offset CGroup:TooBigMaxFrameSizeMsg
        call    PrintSizeWarning        ; Tell The World
        pop     dx                      ; Restore  (AIPTotalRam)
        pop     ax                      ; Restore the FULL size.

;-------------------------------------------------------------------------
;       Calculate the Maximum Transmit and Receive Buffer Size and store it.
;-------------------------------------------------------------------------
SetMaxPktSize:
        mov     si, offset CGroup:DIROpenAdapter
        cmp     word ptr [si].OpenNumTxBuf, 01  ; Check to see if 8k card
        jne     Check32KMode                    ; Not 8k then size is ok jump and see about 32K Mode Case

;-------------------------------------------------------------------------
;               8K Mode Case
;-------------------------------------------------------------------------
        cmp     ax, OpnLen2KTxBuf       ; are we trying to be too big
        jb      NumIsOK                 ; Smaller Packet is requested

        mov     ax, OpnLen2KTxBuf       ; Get Transmit Buffer size for the 8K Card = 2k Packet size
                                        ; Tell The World about being too big for 8KMode

        test    DriverStateFlag, MaxFrameFromNetCFGBit ; Did User tell us a Size?
        jz      NumIsOK         ; NZ => User spec'd size tell him error.

        push    dx                              ; Save    (AIPTotalRam)
        mov     dx, Offset CGroup:TooBig8KModeMsg
        call    PrintSizeWarning                ; Tell The World
        pop     dx                              ; Restore  (AIPTotalRam)
        jmp short NumIsOK

;-------------------------------------------------------------------------
;               32K Mode Case
;-------------------------------------------------------------------------
Check32KMode:
        cmp     word ptr MemRange2, SharedRamSize32K
        jne     SizeIsOK                ; Jump to check on the # allowed

        cmp     ax, OpnLen4KTxBuf       ; 32 Mode Max Size = 4K Frames
        jbe     NumIsOK                 ; Less than Max jump to OK

        mov     ax, OpnLen4KTxBuf               ;Get Correct Size
; AYD/WLR 7-15-96
;        push    dx                              ;Save     (AIPTotalRam)
;        mov     dx, offset CGroup:TooBig32KModeMsg
;        call    PrintSizeWarning                ;Tell The World
;        pop     dx                              ;Restore  (AIPTotalRam)
        jmp short NumIsOK

;-------------------------------------------------------------------------
;               Check Number Of Transmit Buffers
;-------------------------------------------------------------------------
SizeIsOK:
        cmp     ax, OpnLen10KTxBuf              ; Is the Frame size > 10K
        jbe     NumIsOK                         ; if ax is less than 10K size jump

        mov     [si].OpenNumTxBuf, 01           ; Only allow 1 Tx buf if over 10K

NumIsOK:
        mov     MaxFrameSize, ax                ; set config table buffer size
        sub     MaxFrameSize, 6                 ; Remove Adapter Overhead
                                                ; Guarantee Equates and User gave multiple of 8
        add     ax, 7                           ; Adjust to be nearest multiple of eight above.
        and     al, NOT 07                      ; Round down to nearest multiple of eight
        xchg    ah, al                          ; AX = (High Low Format) Transmit Buffer Size
        mov     word ptr [si].OpenLengthTxBuf, ax ; AX = DIR.Open.Adapter DHB Size  (high-low format)

;-------------------------------------------------------------------------
;               Perform Shared RAM Read/Write Test
;-------------------------------------------------------------------------
        mov     ax, word ptr SRBAddress ; get Shared RAM SRB Relative @
        and     ah, PageSizeMask        ; get Relative Page
        mov     si, ax                  ; ES:SI : Shared Ram SRB @
        xor     cx, cx                  ; CX = 0000
        mov     ax, 05AFh               ; get work pattern

TestSharedRAMLoop:
        mov     di, si                          ; get start point again
        mov     cl, SIZE DIROpenAdapter/2       ; Shared RAM SRB Size in words for DIR.Open.Adapter
 rep    stosw                                   ; save the bytes
        mov     di, si                          ; get start point again
        mov     cl, SIZE DIROpenAdapter/2       ; Shared RAM SRB size in words for Dir.Open.Adapter
 repz   scasw                                   ; Did it work ??

checksharedMEM:
        jz      ShiftRAMBits                    ; Z - Yes

TestRAMFail:
        mov     dx, offset CGroup:RamFailedMsg  ; Return with the Disgrace of an Error
        jmp     ErrorInitNIC

ShiftRAMBits:
        mov     cl, 2                   ; move 2 bits at a time
        ror     ax, cl                  ; Shift the bytes over; CL: chift counter
        cmp     al, 0AFh                ; all Bits tested ??
        jnz     TestSharedRAMLoop       ; Loop 8 times then drop down

;-------------------------------------------------------------------------
; Shared RAM Works fine,        --- Check For Zeroing Top 512 Bytes
;-------------------------------------------------------------------------
        cmp     RAMSize, SHARED_RAM_8K  ; Check to see if 8k mode.
        je      LoadNodeAddress         ; If 8k then skip zeroing of top
                                        ;  512 bytes.
                                        ; Note: Zeroing must take place after Card is Init,
                                        ; but before Dir.Open.Adapter Command.
        and     dl, 0Fh                 ; Strip out low nibble from word DX := AIPTotalRam
        sub     dl, 0Bh                 ; AL = Top 512 Reserved ??
        jnz     LoadNodeAddress         ; Need NOT Set top 512 To 0000

                                        ; ES:DI = Top of SHared Ram
        xor     ax, ax                  ; AX = 0000
        mov     cx, 512/02              ; CX = Word Count
 rep    stosw                           ; Zero Top 512 of Shared RAM

;-------------------------------------------------------------------------
;               Load Node Address into Dir.Open.Adapter SRB Image
;-------------------------------------------------------------------------
LoadNodeAddress:
        push    cs
        pop     es                               ; ES: CGroup Segment
        mov     di, offset CGroup:DIROpenAdapter ; ES:DI Dir.Open.Adapter
        lea     di, es:[di].OpenNodeAddress      ; ES:DI Dir.Open.Adapter.OpenNodeAddress
        mov     si, offset CGroup:MSMPhysNodeAddress ; DS:SI MSMPhysNodeAddress in Config0
        mov     cx, 03h                          ; 6 bytes = 3 words
 rep    movsw                                    ; open with Burned in Node Address

        mov     AdaptRetryCnt, 2                ; Reset Retry Counter to allow Open.Adapter 2 retries

ifdef RPL
        cmp     RPLFlag, 0
        je      ContinueInitNic

        and     cs:InterruptStateFlag, NOT (AdapterShutdownBit + IgnoreAllInterrupts)
        xor     ax, ax                          ; set zero flag - sucessful.
        mov     dx, offset CGroup:OPENEDCarrigeReturnMsg
        call    MSMPrintStringZero              ; Print out a simple CR which signals us we are open
        ret

ContinueInitNic:
ENDIF

;-------------------------------------------------------------------------
;               Initialize Fast Path if it is supported.
;-------------------------------------------------------------------------
        test    InitStatus, INIT_FAST_PATH      ; Is Fast Path Supported?
        jnz     FastPathInit                    ; Code this way to keep jumps in range.

InvalidRAMSizeForFastPath:
        jmp     NICOpenAdapter                  ; No - Run in normal mode then.

FastPathInit:
        and     InitStatus, NOT INIT_FAST_PATH
        cmp     RAMSize, SHARED_RAM_8K                  ; 8K RAM size?
        je      InvalidRAMSizeForFastPath               ; Jump if so.

        mov     ax, FastPathRAMSize
        xchg    al, ah
        mov     ConfigRAMSize, ax
        lea     si, ConfigFastPath
        mov     es, SharedRamSegment
        mov     ax, word ptr SRBAddress
        and     ah, PageSizeMask
        mov     di, ax
        push    di
        mov     cx, SIZE ConfigFastPathStructure / 02
    rep movsw

        pop     di
        or      InterruptStateFlag, WaitingIntBit
        push    es
        mov     es, MMIOSegment                 ; ES: MMIO base
        mov     byte ptr es:ISRASetodd, CommandInSRBBit
        pop     es
        mov     bp, 3*1000                      ; 3 seconds

NICConfigFastPathLoop:
        call    WasteTime                       ; Wait for Fast Path
        jnz     NICOpenAdapter

        cmp     es:[di].FPRRetcode, 0           ; Fast Path enabled?
        je      NICConfigFastPathDone

        or      InterruptStateFlag, WaitingIntBit
        sub     bp, ax                                  ; BP = Time left to wait.
        jnz     NICConfigFastPathLoop                   ; Still some time left Fine wait some more.

NICConfigFastPathDone:
        or      InitStatus, INIT_FAST_PATH

        xor     dx, dx
        mov     ax, MaxTxBuf                    ; Max size TX allowed.
        mov     cx, MaxFrameSize                ; get Max. User Data Size from Config0 Table
        div     cx
        cmp     al, 4                           ; Max allowed by TSM is 4.
        jbe     SetDriverMaxActiveTx

        mov     al, 4

SetDriverMaxActiveTx:
        mov     DriverMaxActiveTx, al           ; Set number of TX buffers.
        mov     ax, es:[di].FPRFastPathXmit     ; AX = Tx Offset.
        xchg    ah, al                          ; Convert to Intel format.
        mov     TxPageValue, ah                 ; Save Tx page.
        and     ah, PageSizeMask
        mov     TxControlAddress, ax            ; Store offset to TX Control.
        mov     ax, es:[di].FPRSRBAddress       ; AX = offset of new SRB
        xchg    ah, al                          ; Convert to Intel format.
        mov     es, MMIOSegment                 ; ES: MMIO base
        mov     es:SRPReven, ah                 ; Set new page
        mov     word ptr SRBAddress, ax         ; Save offset to SRB
        mov     si, offset CGroup:DIROpenAdapter
        mov     [si].OpenNumTxBuf, 0            ; Free up old Tx buffers

;-------------------------------------------------------------------------
;               Issue DIR.Open.Adapter
;-------------------------------------------------------------------------
NICOpenAdapter:
        mov     dx, offset CGroup:OpenAdapterMsg        ; Print "Inserting...." Message.
        call    MSMPrintStringZero

        mov     es, SharedRamSegment    ; ES := Shared Ram Segment
        mov     ax, word ptr SRBAddress ; get Shared RAM SRB Relative @
        and     ah, PageSizeMask        ; get Relative Page
        mov     di, ax                  ; ES:DI = Shared Ram SRB @
        mov     si, offset CGroup:DIROpenAdapter
        mov     cx, size DirOpenAdapterStructure
 rep    movsb                           ; move in the Skeleton --- Go byte at a time Watch out for IBM Byte Swap
                                        ; go Set SRB Interrupt

;-------------------------------------------------------------------------
;(Must be able to accept ints immediately after Dir.Open.Adapter is done)
;-------------------------------------------------------------------------
        mov     es, MMIOSegment         ; ES: MMIO base
        mov     InterruptStateFlag, (WaitingIntBit OR AdapterShutdownBit OR DirOpenInProgress OR SRBBusy)

;-------------------------------------------------------------------------
;               Set ISRPeven    InitEnable Bit (bit 6)
;-------------------------------------------------------------------------
        mov     byte ptr es:ISRASetodd, CommandInSRBBit               ; Tell Adapter there's work to do
        mov     byte ptr es:ISRPSeteven, NMIIntControlBit+InterruptEnableBit  ; ES:=MMIO Base

;-------------------------------------------------------------------------
;       Wait For At Least 1080 ticks (1 min) for Interrupt to occur
;-------------------------------------------------------------------------
        mov     bp, 60*1000                     ; 60 seconds
NICOpenAdapterLoop:                             ; If wrong Int occures carry on where left off.
        mov     InterruptStateCopy, 0000h       ; reset copy
        call    WasteTime                       ; Destroys ax,bx,cx,dx leaves STI
        jz      NICOpenAdaptIntQuery

        mov     dx, offset CGroup:OPENEDCarrigeReturnMsg
        call    MSMPrintStringZero                      ; Print out a simple CR which signals us we are open
        mov     dx, offset CGroup:DirOpenIntFailedMsg   ; Return with the appropriate error message
        jmp     ErrorInitNIC                            ; Timed Out

;-------------------------------------------------------------------------
;                     Test due to Dir.Open.Adapter Response
;-------------------------------------------------------------------------
NICOpenAdaptIntQuery:
        test    byte ptr InterruptStateFlag, DirOpenInProgress  ; Are we still waiting on the Int.?
        jz      NICDirOpenDone

        or      InterruptStateFlag, WaitingIntBit       ; Wrong int must have occured, try again
        sub     bp, ax                                  ; BP = Time left to wait.
        jnz     NICOpenAdapterLoop                      ; Still some time left Fine wait some more.

FailedOpenJmp:
        mov     dx, offset CGroup:OPENEDCarrigeReturnMsg
        call    MSMPrintStringZero                      ; Print out a simple CR which signals us we are open
        mov     dx, offset CGroup:DirOpenFailedMsg      ; Return an error filled message
        jmp     ErrorInitNIC                            ; Out of Retries

;-------------------------------------------------------------------------
;       The Dir.Open.Adapter has completed and set a return code
;-------------------------------------------------------------------------
NICDirOpenDone:
        cmp     byte ptr DirOpenRetCode, 00h            ; Return Code = Successfull
        jne     NonZeroErrorCode                        ; NE - Dir.Open.Adapter Has An Error For Us.

;-------------------------------------------------------------------------
;       Dir.Open.Adapter has Completed with No Errors
;-------------------------------------------------------------------------

        mov     dx, offset CGroup:OPENEDCarrigeReturnMsg
        call    MSMPrintStringZero                              ; Print out a simple CR which signals us we are open
        ret


;-------------------------------------------------------------------------
;       Decode the Non 00 return Code from Dir.Open.Adapter
;-------------------------------------------------------------------------
NonZeroErrorCode:
        dec     AdaptRetryCnt                           ; Test if Retry Open Adapter
        jz      OpenRetriesDone

        mov     dx, offset CGroup:ErrFirstTryMsg
        call    MSMPrintStringZero
        mov     ah, byte ptr DirOpenRetCode             ; Get the Return Code to Printout
        mov     al,ah                                   ; copy into both registers
        and     ax,0FF0h                                ; get high nibble in al, and low nible in ah
        mov     cl,4
        shr     al,cl                                   ; get to correct order
        call    MSMPrintHexChar                         ; print high nibble
        xchg    al,ah                                   ; swap who's who
        call    MSMPrintHexChar                         ; Print Low Nibble
        mov     dx, offset CGroup:ErrFirstTryMsg2       ; Print Trying MSG
        call    MSMPrintStringZero

        or      InterruptStateFlag, WaitingIntBit
        mov     bp, 10*1000                     ; Wait 10 seconds before retry
        call    WasteTime                       ; Destroys ax,bx,cx,dx leaves STI
        jmp     NICOpenAdapter

OpenRetriesDone:
        mov     dx, offset CGroup:OPENEDCarrigeReturnMsg
        call    MSMPrintStringZero                      ; Print out a simple CR to clear the line

;-------------------------------------------------------------------------
;       Build an Error Message String
;-------------------------------------------------------------------------
        mov     ah, byte ptr DirOpenRetCode             ;Get the Return Code to Printout
        cmp     ah, 07
        jne     NonExtendedError

        xor     bx,bx
        mov     bl, byte ptr DirOpenRetCode+1           ;Get Extended Error Code
        mov     ax, cs
        mov     es, ax                                  ; ES: CGroup
        mov     di, offset CGroup:DirOpenFailedEndMsg2  ; ES:DI : OUTPUT String
        mov     ax, bx                                  ; get copy
        and     bx, 0Fh                                 ; Strip off Phase, BX: Error
        mov     cx, 04
        shr     al, cl                                  ; AX: Error Phase
        mov     bp, ax                                  ; BP: Error Phase
        call    MSMGetHexChar                           ; AL: Error Phase
        stosb                                           ; add it to the string
        mov     al, bl                                  ; AL: Error itself
        call    MSMGetHexChar
        stosb                                           ; add it to the string
        dec     bx
        add     bx,bx                                   ;Allow for word offsets
        mov     si, ExtendedErrorMsgTBL.[bx]            ;SI:Error String
        lodsb                                           ;AL length of string
        mov     cl,al                                   ;CL: Counter
        inc     cl                                      ;add in one for the 0
 FastMove                                               ;Load final string
        cmp     bx, 0                                   ;(1-1)*2 = 0 Adapter Lobe Function Failure (Cable is probably off)
        jne     CheckBeaconing

        dec     di                                      ; Back up over the 0
        mov     ax, 0d0ah                               ; Get CrLf
        stosw
        mov     si, offset CGroup:FunctFailHelpMsg
        lodsb                                           ; Get Length
        mov     cl, al
        inc     cl                                      ; Get that pesky 0
 FastMove
        jmp     short DXisReady

CheckBeaconing:
        cmp     bx, 12                                  ; (7-1)*2 = 12 Ring Beaconing.
        jne     DXisReady

        dec     di                                      ; Back up over the 0
        mov     ax, 0d0ah                               ; Get CrLf
        stosw
        mov     si, offset CGroup:RingBeaconHelpMsg
        lodsb                                           ; Get Length
        mov     cl, al
        inc     cl                                      ; Get that pesky 0
        FastMove

DXisReady:
        mov     dx, offset CGroup:DirOpenFailedMsg2
        jmp     short ErrorInitNIC                              ; Out of Retries

NonExtendedError:
        push    ds
        pop     es                                      ; ES: CGroup
        mov     di, offset CGroup:DirOpenFailCodeMsg2   ; ES:DI : OUTPUT String
        mov     al,ah                                   ; copy into both registers
        and     ax,0FF0h                                ; get high nibble in al, and low nible in ah
        mov     cl,4
        shr     al,cl                                   ; get to correct order
        call    MSMGetHexChar                           ; AL: high nibble
        stosb                                           ; add byte to the string
        xchg    al, ah                                  ; AL: low nibble
        call    MSMGetHexChar
        stosb                                           ; add it to the string
        jmp     DXisReady


ErrorInitNIC:                                   ; DX - Error Value for String
        or      InterruptStateFlag, AdapterShutdownBit
        mov     ax, -1
        or      ax, ax
        ret

InitNic         endp

;******************************************************************************
;               P O S --- Program Option Select EQUates for Micro-Channel
;******************************************************************************
SlotSelectReg   equ     96h     ; channel select
MaxSlots        equ     0Fh     ;
SlotStartVal    equ     08h     ; Bit 3 must be on!!!
PosPriAltBit    equ     001h    ; Pos Reg 3 Pri (0) or Alt (1) Adapter bit
ProgSelect      equ     0100h   ; Program Option Select I/O Port Addr.
POSCardEnable   equ     001h    ; Pos Reg 2 Card Enable Bit
POSCardIDhigh   equ     0E0h    ; POS Reg 1 ID High Byte ; in POS1
POSCardIDlow    equ     001h    ; Pos Reg 0 ID low byte; 01 = 16/4; 00 = Adapter/A
POS0            equ     100H    ; Program Option Select Register (POS) 0
POS1            equ     101H    ; POS 1
POS2            equ     102H    ; POS 2


;**********************************************************************
;         SCANSLOT -- Find Micro-Channel Slot Number
;*****************************************************************
; Scanslot fills in the Slot Number in the Config0 Table with the
; slot with either the first card found (slot = 0, and IOAddress1 = 0)
; or the slot that the IOAddress1 is in (slot = 0, and IOAddress1 != 0).
; Scanslot also fills in the IOAddress1 with the address of the card in
; that slot (IOAddress1 !=0) or Compares the Address to ensure it is correct
; (IOAddress !=0).
;
; Assumes:                      Returns: Ints Enabled
;      The Machine is a PS2             The Slot # and IOAddress1 Fields set
;      Ints are Enabled                 Z-Flag  = successful (doesn't check if the card is really there)
;                                       NZ-Flag = Address/Slots didn't match properly.
; Registers Used:
;       AX, BX, DX


ScanSlot    proc  near                  ;Find Micro-Channel Slot Number

        cli                             ; Turn off ints till done
        mov     bx, PrimPortAddr        ; Set Up for Default port
        mov     ah, byte ptr Slot       ; Get Low byte
        or      ah, ah
        jnz     SlotLooking

        mov     ah, 01                  ;Slots Start With Number 1 ... if not given

SlotLooking:
        mov     al, ah                  ;AL = POS Slot Number
        add     al, MaxSlots-SlotStartVal ;AL = POS Bit 3 MUST Be On
        out     SlotSelectReg, al       ;Select the Slot
        mov     dx, POS0                ;DX = POS REG 00
        in      al, dx                  ;AL = Slot's Low ID Byte
        cmp     al, POSCardIDlow        ;AL = Valid ID ?? for a Token-ring (00 or 01)
        ja      NotThisSlot             ;NOT Token-Ring Card
        inc     dx                      ;DX = POS Reg 01 Port I/O @
        in      al, dx                  ;AL = POS Reg 01
        cmp     al, POSCardIDhigh       ;AL = Valid Token-Ring ID ??
        jnz     NotThisSlot             ;NOT Token-Ring Card
        inc     dx                      ;DX = POS Reg 02 Port I/O @
        in      al, dx                  ;AL = Card Enabled Bit
        and     al, POSCardEnable       ;AL = Card Enabled ??
        jnz     GotASlot                ;Card Enabled

NotThisSlot:
        cmp     Slot, 0         ; Is the Slot Given?
        jnz     SlotError       ; If Slot Was Given then it wasn't there

        inc     ah                              ;AH = Next Slot Number
        cmp     ah, MaxSlots-SlotStartVal+01    ;AH = End of Slots ??
        jna     SlotLooking                     ;Loop for ALL Slots

SlotError:
        mov     al, 0
        out     SlotSelectReg, al       ; DeSelect the Slot and turn Setup Off
        sti                             ; Turn Interrupts Back on
        or      al, 1
        ret                             ; nz ==> Slot NOT Found

;-------------------------------------------------------------------------
;         Token-Ring Card FOUND; Calculate the Port Address
;-------------------------------------------------------------------------
GotASlot:
        inc     dx                      ;DX = POS Reg 03 Port I/O @
        in      al, dx                  ;AL = Primary/Alternate Bit
        and     al, PosPriAltBit        ;AL = 00 ==> Primary; 01 ==> Alternate
        shl     al, 1                   ;AL = 00 ==> Primary; 02 ==> Alternate
        shl     al, 1                   ;AL = 00 ==> Primary; 04 ==> Alternate
        add     al, bl                  ;AL = 20 ==> Primary; 24 ==> Alternate

        cmp     IOAddress1, 0           ; Have We Already set an Address
        jz      SetAddress              ; If 0 then go set Address

        cmp     al, byte ptr IOAddress1 ; AL = Address Match ??
        jnz     NotThisSlot             ; LOOP if NO Match

SetAddress:
        mov     bl, al                  ; BL = 20 ==> Primary; 24 ==> Alternate
        mov     word ptr IOAddress1, bx ; Load IOAddress Variable
        mov     byte ptr Slot, ah       ; Load Slot Variable
        mov     byte ptr BusID, BUS_ID_MCA ; Indicate we are using MCA bus.

        mov     al, 0
        out     SlotSelectReg, al       ; DeSelect the Slot and turn Setup Off
        sti                             ; Turn Interrupts Back on
        ret                             ; z ==> Slot Loaded, IOAddress1 Loaded

ScanSlot    endp                        ; Find Micro-Channel Slot Number

;**********************************************************************
;       ProcessMaxFrameSize --- Set the Max Frame Size in the Table
;**********************************************************************
;       Assume:                                 Returns:
;               DS: CGroup                              Sets Frame Size in Table
;               SI: -> Buffer Holding parms             Ints enabled
;               BX: -> Config Table                     CLD in affect
;               Ints enabled                            Destroys AX, DX, CX
;               CLD in affect
;

ProcessMaxFrameSize     proc    near
        call    MSMEatWhite             ; Point Si at Parameter
        call    MSMAtoI                 ; Convert it to Integer
        jnz     RanOutOfFile            ; Check for end of File

        cmp     ax, MinTxBuf            ; Is the Value too small
        jae     NotTooSmall             ; Jump if not too small

        mov     ax, MinTxBuf            ; Set to the Min
        mov     dx, offset CGroup:FrameTooSmallMsg
        call    MSMPrintStringZero

NotTooSmall:
        cmp     ax, MaxTxBuf            ; Is the Value Too Big?
        jbe     NotTooBig               ; Jump if not too big

        mov     ax, MaxTxBuf            ; Set to the MAX size
        mov     dx, offset CGroup:FrameTooBigMsg
        call    MSMPrintStringZero

NotTooBig:
        mov     MaxFrameSize, ax        ; Stuff into Table
        or      DriverStateFlag, MaxFrameFromNetCFGBit ; Signal Not Defaults.

RanOutOfFile:
        ret

ProcessMaxFrameSize     endp

;******************************************************************************
; PrintSizeWarning ---  Print out Warning Message about the Size of the
;                       Buffers, then a CrLf, then a Line telling the user
;                       what the maximum is to be.
;
;******************************************************************************
;       Assumes:                                Returns: All Registers Preserved
;               DX: String to Print
;               AX: Max Value to print After the MaxFrameSizeMsg String
;-------------------------------------------------------------------------

PrintSizeWarning        proc    near
;-------------------------------------------------------------------------
;       Preserve Destroyed Regs.
;-------------------------------------------------------------------------
        push    ax
        push    dx                      ; Save what is about to Die
        push    es
        push    ds

;-------------------------------------------------------------------------
;       Print out Main String (passed in) and a CRLF.
;       DX = String to print
;-------------------------------------------------------------------------
        call    MSMPrintStringZero
        mov     dx, offset CGroup:MSMNewLine
        call    MSMPrintStringZero

;-------------------------------------------------------------------------
;       Print out the Maximum Message line
;       AX = Maximum Value
;-------------------------------------------------------------------------
        mov     dx, offset CGroup:MaxFrameSizeMsg
        call    MSMPrintStringZero
        call    MSMItoA
        pop     ds                      ; DS was gone, now he has returned
        mov     dx, offset CGroup:MSMNewLine
        call    MSMPrintStringZero      ; protects all regs for me.

;-------------------------------------------------------------------------
;       Restore what has been destroyed
;-------------------------------------------------------------------------
        pop     es
        pop     dx                      ; Restore AX & DX now that we are done.
        pop     ax
        ret                             ; Back to where we were called.
PrintSizeWarning        endp

;******************************************************************************************
;       ProcessAltKeyWord ---   If the Key word ALT is in the NET.CFG File properly
;                               then Set Port to A24 (the Alternate Address)
;******************************************************************************************
;       Assumes: Nothing        Returns: Stuffs a value inConfig table and returns
;
ProcessAltKeyWord       proc    near

        mov     IOAddress1, AltPortAddr ; Set for the Alternate Address
        ret                             ; and return

ProcessAltKeyWord       endp

Init    ends
        end



