/***********************************************************************\
*                                                                       *
*  (c) Copyright Unpublished Work of Novell, Inc. All rights reserved.
*
*	THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*	PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.  ACCESS
*	TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
*	NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
*	AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO
*	APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED,
*	PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED,
*	TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED,
*	LINKED,RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
*	CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
*	THE PRIOR WRITTEN CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF
*	THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
*	CRIMINAL AND CIVIL LIABILITY.
*                                                                       *
************************************************************************/
/**************************************************************************\
*
* Title:		C Ethernet Support Module
*
* Filename:		ethertsm.c
*
* ODI Spec Ver:	1.11
*
* Description:	This module is the C version of ETHERTSM.NLM.  It is
*               single sourced for Portability.  
*
* Modification History:
*
* 8-09-96 TNL	Modified Pipeline Code to check for packet too small and
*				malformed packets.
*				SPD #132048 and 132055.
*
* 8-20-96 TNL	Modified TSM(Fast)SendComplete code to not increment Tx
*				statistics counters when transmitStatus is non-zero.
*				SPD #133041, 133057, and 133071.
*
* 8-28-96 WTT	Added CEtherTSMConfigTable and modified CInitializeEtherTSM
*				to set up the module version fields in the table.  Also added
*				the CEtherTSMGetConfigInfo API.
*
* 10-29-96 MPK	Added code to MediaShutdown, to diferentiate between a standard
*				Shutdown, and one caused by an HSM call to
*				CMSMShutdownMLID, who in turn calls MediaShutdown
*
* 11-04-96 KRA  Added code for remote multicast in MediaPromiscuousChange.
*
* 11-12-96 WTT	CEtherTSMProcessGetRCB was blowing away the
*				ECB_Fragment[0].FragmentLength field in the ECB when it was
*				building the look ahead buffer.
*				SPD #139529
*
* 11-15-96 WTT	Added code to CEtherTSMGetRCB check and make sure that an
*				ecb was returned in the lookahead structure when
*				CLSL_GetStackECB returned ODISTAT_SUCCESSFUL before tring to
*				use the pointer.
*				SPD #137522
*
* 11-19-96 KRA	In MediaReset and MediaShutdown, if there are ecbs queued up
*				and returned to the LSL, decrement the Qeueu depth counter.
*
* 12-05-96 WTT	Changed MediaAddMulticast, MediaDeleteMulticast and 
*				FindAddressInMCTable to conform to spec.  They were using 
*				UINT8 pointer and should have been using NODE_ADDR pointer 
*				for	the mcAddress.
*				SPD #143895
*
* 12-19-96 WTT	Reversed the change made to CEtherTSMProcessGetRCB on 11-12-96.
*				for SPD#139529.  According to specification and code the 
*				adjustment to the length field needs to be made for HSMs that 
*				pre-allocate RCBs.  To fix the side effects of doing this 
*				CLSL_ReturnECB will be changed to restore the fragment 
*				address and length fields before adding the ECB back to the 
*				ECB pool it came from.
*
* 12-19-96 WTT	Changed MediaSend to check shutdown bit for logical board 
*				instead of adapter.
*
* 01-02-96 WTT	Changed CEtherTSMProcessGetRCB and CEtherTSMGetRCB to check 
*				shutdown bit for logical board and set rcvStatus to
*				PAE_NOT_ENABLED_BIT if the logical board is shutdown.
*
* 01-14-97 MPK	Changed CEtherTSMBuildTransmitControlBlock so that ECB 
*			   	Drivers will return quicker, as they don't need this process.
*				However, I will provide Physical addresses to those ECB
*				Aware adapters requiring them.
*				SPD #146436
*
* 01-17-97 MPK	Changed CancelPrioritySEND, since BuildTransmitControlBlock is now
*			   	Responsible for making the ECB physical (if necessary,
*				the checking for Physical and return Physical is no
*				longer required in CancelPrioritySend, since the ECB will
*				never have been made physical before calling CancelPrioritySend.
*
* 03-06-97 MPK	Changed MediaNodeOverride to check for the proper range value of the 
*			   	NOdeAddress Override to make sure we are not over or undersize.
*				Also to report a message and reset to unknown value if so.
*				SPD 151815
*
* 07-29-97 WTT	With the recent changes to LSLC32 and CMSM for the SPDs listed 
*				below was able to remove the check to see if we were using 
*				a spare ECB and the subsquent call to CMSMReturnSpareECB in 
*				CEtherTSMProcessGetRCB.
*				SPD 162312, 146492
*
* 08-29-97 WTT	Changed the code to use the new define NON_STACKID_RAW_SEND_MASK
*				when checking to see if we have a raw send.
*
\**************************************************************************/
/* Fake type definition to keep .h files happy. _DRIVER_DATA_ is defined	*/
/* in the HSM and not accessed directly by the MSM or TSM.						*/
typedef	struct	_DRIVER_DATA_ DRIVER_DATA;

#define	_cdecl

#include <portable.h>
#include <odi.h>
#include <odi_nbi.h>
#include <parser.h>
#include <cmsm.h>
#include <ethertsm.h>
#include <cmsmstr.h>
#include <splat.h>
#include <ethertsm.txt>

/**************************************************************************\
**
**    Global declarations
**
\**************************************************************************/
MEON_STRING	CODISPEC[] = ETHERTSM_ODI_SPECVER_MSG;
MEON			**DriverMessages = NULL;

/* Ethernet8022DescriptLen, Ethernet8023DescriptLen,					*/
/*	EthernetIIDescriptLen, EthernetSNAPDescriptLen are					*/
/*	supplied for CMSMOldParseDriverParameters.  Order is critical.	*/

static UINT8			Ethernet8022Index =	E8022_INTERNALID;
static UINT8			Ethernet8022DescriptLen = E8022_DESCRIPT_LEN;
static MEON_STRING	Ethernet8022Descript[] = { ETHERTSM_FRAME_8022_TXTMSG };
static UINT8			Ethernet8023Index =	E8023_INTERNALID;
static UINT8			Ethernet8023DescriptLen = E8023_DESCRIPT_LEN;
static MEON_STRING	Ethernet8023Descript[] = { ETHERTSM_FRAME_8023_TXTMSG };
static UINT8			EthernetIIIndex	=	EII_INTERNALID;
static UINT8			EthernetIIDescriptLen = EII_DESCRIPT_LEN;
static MEON_STRING	EthernetIIDescript[] = { ETHERTSM_FRAME_EII_TXTMSG };
static UINT8			EthernetSNAPIndex =	ESNAP_INTERNALID;
static UINT8			EthernetSNAPDescriptLen = ESNAP_DESCRIPT_LEN;
static MEON_STRING	EthernetSNAPDescript[] = { ETHERTSM_FRAME_SNAP_TXTMSG };

MEON_STRING		*EtherTSMFrameDescriptTable[] =
{
	Ethernet8022Descript,
	Ethernet8023Descript,
	EthernetIIDescript,
	EthernetSNAPDescript
};

static UINT8 MinAddress[] = { 1, 0, 0, 0, 0, 0 };
static UINT8 MaxAddress[] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE };
static UINT8 BroadCast[]  = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };

TSM_CONFIG_LIMITS		EtherTSMConfigLimits =
{
	(UINT8 *)&MinAddress,	/* MinNodeAddress */
	(UINT8 *)&MaxAddress,	/* MaxNodeAddress */
	0,								/* MinRetries 		*/
	255,							/* MaxRetries 		*/
	4,								/* NumberFrames 	*/
};

static PROT_ID EtherTSMProtocolIDArray[] =
{
	0, 0, 0, 0, 0, 0xE0,					/* 802.2 */
	0, 0, 0, 0, 0, 0,					   /* 802.3 */
	0, 0, 0, 0, 0x81, 0x37,				/* E_II */
	0, 0, 0, 0, 0x81, 0x37				/* 802.2 w/SNAP */
};

static UINT8	EtherTSMIDArray[] =
{
	E8022_ID,
	E8023_ID,
	EII_ID,
	ESNAP_ID,
};

static UINT8	EtherTSMHeaderSizeArray[] =
{
	SIZE_E8022,
	SIZE_E8023,
	SIZE_EII,
	SIZE_ESNAP,
};

UINT32	(*EtherTSMSendRoutineArray[])(ECB *, TCB *) =
{
	(UINT32 (*)(ECB *, TCB *))CMediaSend8022Over8023,
 	(UINT32 (*)(ECB *, TCB *))CMediaSendRaw8023,
 	(UINT32 (*)(ECB *, TCB *))CMediaSendEthernetII,
 	(UINT32 (*)(ECB *, TCB *))CMediaSend8022Snap
};

GROUP_ADDR_LIST_NODE	MCPlaceHolder;

MEDIA_DATA MediaDataTemplate =
{
	MAX_MULTICAST,				/* MaxMulticastAddresses		*/
 	&MCPlaceHolder,			/* MulticastAddressList			*/
	0,								/* MulticastAddressCount		*/
	NULL,							/* TCBHead							*/
	NULL,							/* TCBList							*/
	NULL,							/* NewStatsPtr						*/
	0,								/* PromiscuousMode				*/
	{0},							/* PromiscuousCounters			*/
	0,								/* RxStatus							*/
};

static TSM_PARM_BLOCK	MediaParameters =
{
   sizeof(TSM_PARM_BLOCK),							/* MediaParameterSize			*/
   &EtherTSMConfigLimits,					 		/* MediaConfigLimits				*/
   (MEON **)&EtherTSMFrameDescriptTable,		/* MediaFrameDescriptTable 	*/
   (UINT8 (*)[])&EtherTSMProtocolIDArray,		/* MediaProtocolIDArray			*/
   EtherTSMIDArray,							 		/* MediaIDArray					*/
   EtherTSMHeaderSizeArray,				 		/* MediaHeaderSizeArray			*/
   (UINT32 (**)())&EtherTSMSendRoutineArray,	/* MediaSendRoutineArray 		*/
   sizeof( MEDIA_DATA ),					 		/* MediaAdapterDataSpaceSize	*/
   &MediaDataTemplate,						 		/* MediaAdapterPtr				*/
   MediaAdjust,								 		/* MediaAdjustPtr					*/
   (ODISTAT (*)())MediaInit,				 		/* MediaInitPtr					*/
   MediaReset,											/* MediaResetPtr					*/
   MediaShutdown,										/* MediaShutdownPtr				*/
   MediaSend,									 		/* MediaSendPtr					*/
   (void *(*)())CEtherTSMGetNextSend,	  		/* MediaGetNextSendPtr			*/
   MediaAddMulticast,						 		/* MediaAddMulticastPtr			*/
   MediaDeleteMulticast,					 		/* MediaDeleteMulticastPtr		*/
   MediaNodeOverride,						 		/* MediaNodeOverridePtr			*/
   MediaAdjustNodeAddress,							/* MediaAdjustNodeAddressPtr	*/
   MediaSetLookAheadSize,					 		/* MediaSetLookAheadSizePtr	*/
   MediaPromiscuousChange,							/* MediaPromiscuousChangePtr	*/
   MediaRegisterMonitor,							/* MediaRegisterMonitorPtr		*/
   MediaGetParameters,								/* MediaGetParametersPtr		*/
   MediaGetMulticastInfo							/* MediaGetMulticastInfoPtr		*/
};

struct
{
   UINT32		OptionCount;
  	MEON_STRING	*StrOptVal[4];
} FrameOptionList = { 4, NULL};


DRIVER_OPTION  NodeOption =
{
   NULL,
   ETHERTSM_NODE_EQUAL_TXTMSG,
   0,
   0,
   0,
   CUSTOMPARAM,
   OPTIONALPARAM | STRINGPARAM,
   {'\0'}
};


DRIVER_OPTION	FrameOption =
{
	&NodeOption,
	ETHERTSM_FRAME_EQUAL_TXTMSG,
	(PARAMETER_OPTIONS *)&FrameOptionList,
	0,
	0,
   FRAMEPARAM,
   ENUMPARAM,
	{'\0'}
};

CTSM_CONFIG_TABLE CEtherTSMConfigTable =
{
	sizeof(CTSM_CONFIG_TABLE),	/* CTSMCFG_TableSize */
	CTSM_TABLE_MAJOR_VER,		/* CTSMCFG_TableMajorVersion */
	CTSM_TABLE_MINOR_VER,		/* CTSMCFG_TableMinorVersion */
	NULL,						/* CTSMCFG_ModuleMajorVersion - filled in by MediaInit */
    NULL,						/* CTSMCFG_ModuleMinorVersion - filled in by MediaInit */
    ODI_SPEC_MAJOR_VER,			/* CTSMCFG_ODISpecMajorVersion */
    ODI_SPEC_MINOR_VER,			/* CTSMCFG_ODISpecMinorVersion */
	NULL,						/* CTSMCFG_Reserved */
    MAX_PACKET_LENGTH,			/* CTSMCFG_MaxFrameSize - defined in ETHERTSM.H */
	NULL,						/* CTSMCFG_SystemFlags - filled in at Init Time */
};

/**************************************************************************\
**
**    Procedures
**
\**************************************************************************/

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaInit(
*								DRIVER_DATA				*driverData,
*								FRAME_DATA				*frameData);
*
*	Description:
*		This function initializes the media specific variables of the
*		adapter data space. It will build a v4.0 style stat table if the
*		HSM is using a v3.0 table. It will also allocate memory for
*		a multicast table and a TCB pool. This function is called by
*		CMSMRegisterMLID.
*
*	Parameters in:
*		driverData -	Pointer to CHSMs adapter data space.
*		FrameData -		to get Pointer to default configuration table.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			If media data space initialized
*		ODISTAT_OUT_OF_RESOURCES	If unable to allocate memory
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
ODISTAT	MediaInit(
						DRIVER_DATA		*driverData,
 						FRAME_DATA		*frameData)
{
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	TCB								*tcbPtr;
	UINT32							i;
	UINT32							bufferSize;

#ifdef IAPX386
	MSTATS                  *statsTable;
   MLID_STATS_TABLE			*newStats;
	StatTableEntry          *tableEntry;
	MEON                    *customStrings;
	UINT32                  *customCounters, numCustomCounters;
	UINT32                  bitMask, j;
#endif

	Ethernet8022DescriptLen = Ethernet8022DescriptLen;	/* resolve complier warnings.*/
	Ethernet8023DescriptLen = Ethernet8023DescriptLen;	/* resolve complier warnings.*/
	EthernetIIDescriptLen   = EthernetIIDescriptLen;	/* resolve complier warnings.*/
	EthernetSNAPDescriptLen = EthernetSNAPDescriptLen;	/* resolve complier warnings.*/

	/* Setup Local Variables */
	configTable = &frameData->ConfigTable;

	/*Set the Maximum frame header size to SNAP + 18 for lookahead Data.*/
	DADSP_TO_CMSMADSP(driverData)->CMSMMaxFrameHeaderSize = SIZE_ESNAP +
																		DEFAULT_LOOK_AHEAD_SIZE;
	mediaData = DADSP_TO_CMSMADSP(driverData)->CMSMMediaDataPtr;

#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
		/* Get Valid pointer to HSM Stat Table */
		statsTable = (MSTATS *) DADSP_TO_CMSMADSP (driverData)->CMSMStatsPtr;
	
		/********************************************************************\
		**
		**  We only Support from version 3.00 of the Statistics Table
		**  Check the Statistics table Version for 3.00
		**
		\********************************************************************/
		if ( statsTable->MSTATS_MajorVer >= 0x03 )
		 {
			if( statsTable->MSTATS_MajorVer == 0x03 )
			 {
				/**************************************************************\
				**
				**  We have a valid OLD ASSEMBLY style Statistics Table.
				**  Set mask field for EtherTSM support.
	 			**
 				\**************************************************************/
 				statsTable->MSTATS_ValidMask0 &= STATISTICSMASK;
	
				/**************************************************************\
				**
	 			**  Allocate memory for Generic and, Media specific table entries.
				**  We need 2 less than NumGenericCounters says because of 2 64
 				**  bit counters.
 				**
 				\**************************************************************/
	  			customCounters    = &statsTable->MSTATS_Generic.Gen_TotalTxPackets;
	  			customCounters   += statsTable->MSTATS_NumGenericCounters;
   			numCustomCounters = *(UINT16 *) customCounters;

				bufferSize = sizeof(MSTATS) + (sizeof(StatTableEntry) *
					 ((statsTable->MSTATS_NumGenericCounters - 2) + numCustomCounters));

				mediaData->NewStatsPtr =
								(MLID_STATS_TABLE *) CMSMAlloc( driverData, bufferSize );
	
				if (!mediaData->NewStatsPtr)
				{
					CMSMPrintString( configTable,	MSG_TYPE_INIT_ERROR,
							  ETHERTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );
					return( ODISTAT_OUT_OF_RESOURCES );	/* Could not allocate Memory. */
				}
	
				/**************************************************************\
				**
				**  Write AsmStatsPtr with pointer to "ASSEMBLY" style table.
				**  Overwrite StatsPtr with pointer to "C" style table.
 				**
	 			\**************************************************************/
				DADSP_TO_CMSMADSP(driverData)->CMSMStatsPtr = mediaData->NewStatsPtr;
	         newStats = mediaData->NewStatsPtr;

   	      /**************************************************************\
	         **
      	   **  Prepare the NEW "C" table header, then strings.
         	**
	         \**************************************************************/
	 			newStats->MStatTableMajorVer  = 0x04;
				newStats->MStatTableMinorVer  = 0x00;
 				tableEntry = (StatTableEntry *) (newStats + 1);
				newStats->MGenericCountsPtr   = tableEntry;
				newStats->MNumGenericCounters = MLID_Q_DEPTH + 1;
	
				/* Set up the Counter Strings */
	  			BuildASMStatStrings (tableEntry);
	
         	/**************************************************************\
	         **
	         **  Now prepare the generic and media counter fields.
         	**    Ethernet has only 29 Counters from the Assembly Spec.
          	\**************************************************************/
				bitMask = statsTable->MSTATS_ValidMask0;
	
	     		for (i = 0, j = 0; i < statsTable->MSTATS_NumGenericCounters; ++i)
	    		 {
					/* TotalTxOKbyte,TotalRxOKbyte */
	    			if (i == 14 || i == 16 )
					 {
						/*We have the high part of a 64 bit counter.*/
						(tableEntry -1)->StatUseFlag = ODI_STAT_UINT64;
					 }
					else
					 {
						/****************************************
							Check if the HSM uses this Counter.
							True (1) => Not used
							False (0) => UINT32 Stat Counter
						*****************************************/
						/* Force RxTooSmall and TxMiscCount to always be used	*/
						if( i == 7 || i == 8 )
							tableEntry->StatUseFlag = ODI_STAT_UINT32;
						else if (bitMask & (0x80000000 >> j))
	                  tableEntry->StatUseFlag = ODI_STAT_UNUSED;
						else
	                  tableEntry->StatUseFlag = ODI_STAT_UINT32;
	
						tableEntry->StatCounter
							= &(((void **)&statsTable->MSTATS_Generic.Gen_TotalTxPackets)[i]);
						++j;
						++tableEntry;
					 } /* end Else normal Counter setup. */
				 } /* end for loop for Generic counters */
	
				/**************************************************************\
				**
				**  Now initialize other media fields.
				**
	 			\**************************************************************/
				newStats->MNumMediaCounters   = ETHERNETSTATSCOUNT;
				newStats->MMediaCountsPtr     = tableEntry - ETHERNETSTATSCOUNT;

				/**************************************************************\
				**
				**  Now prepare the custom counters.
				**
				\**************************************************************/
				customCounters = (UINT32 *) ((UINT8 *)customCounters + 2);
	
	 			/*Setup pointer to the Strings for the custom counters.*/
 				customStrings  = (UINT8 *) customCounters [numCustomCounters];
				customStrings += 2;

				newStats->MNumCustomCounters = numCustomCounters;
				newStats->MCustomCountersPtr = numCustomCounters ? tableEntry:NULL;

				for (i = 0; i < numCustomCounters; ++i)
 				 {
 					tableEntry->StatUseFlag = ODI_STAT_UINT32;
					tableEntry->StatCounter = customCounters;
 					tableEntry->StatString  = customStrings;
	
					while (*customStrings != NULL)
						++customStrings;
	
					/* Point at the start of the next string. */
					++customStrings;
	
					/* Move to next Counter */
					++customCounters;
					++tableEntry;
				 } /* End Update Custom Strings  */
			 }	/* End Supported Old Style Assembly */
			else    /* Setup Generic and Media strings for C drivers */
			 {
				/**************************************
					C version of Stats Table, needs
					TOPOLOGY specific components setup
				***************************************/
				newStats = DADSP_TO_CMSMADSP(driverData)->CMSMStatsPtr;
	
				/* build generic stats strings */
				tableEntry = newStats->MGenericCountsPtr;
				tableEntry = CMSMBuildGenericStatStrings( tableEntry );
		
				/* build media stats strings */
				tableEntry = newStats->MMediaCountsPtr;
				BuildTSMStatStrings (tableEntry);
			 }      /* End String changes and Fixups in TSM */
	  	 }	 /* Valid Stats Version */
    else
	    {
	   	/* Wrong version of Stats Table */
	   	/* Error out. */
	   	CMSMPrintString(	configTable, MSG_TYPE_INIT_WARNING,
									ETHERTSM_STATS_VERSION_ERROR_MSG,
     								NULL, NULL);
	
	    	return( ODISTAT_OUT_OF_RESOURCES );
		 }	/* end wrong version of Stats Table */
	
#endif			/* IAPX386	*/

	/*Allocate multicast table*/
   if (DADSP_TO_CMSMADSP(driverData)->CMSMDPBlockPtr->DriverMaxMulticast)
	 {
		if ( (mediaData->MulticastAddressList = CMSMAlloc(
						driverData,
						mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE))) == NULL)
		 {
			CMSMPrintString( configTable, MSG_TYPE_INIT_WARNING,
 								  ETHERTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );
			return( ODISTAT_OUT_OF_RESOURCES );
		 }

		CMSMmemset(mediaData->MulticastAddressList, NULL,
				(mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE)) );
	 }	/* End Allocate Space for Multicast Table */

	/* Allocate and initialize memory for TCBs */
	bufferSize = sizeof(TCB) * DADSP_TO_CMSMADSP(driverData)->CMSMTxFreeCount;
	bufferSize += sizeof(TCB) * DADSP_TO_CMSMADSP(driverData)->CMSMPriorityTxFreeCount;

	if ( (mediaData->TCBHead = (TCB *)CMSMAlloc(driverData,bufferSize)) == NULL)
	 {
		CMSMPrintString( configTable,	MSG_TYPE_INIT_ERROR,
							  ETHERTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );
  		return( ODISTAT_OUT_OF_RESOURCES );
    }

	/* Initialize the Buffer */
	CMSMmemset( mediaData->TCBHead, 0, bufferSize );
	mediaData->TCBList = mediaData->TCBHead;
	tcbPtr = mediaData->TCBHead;

	/*Fill out fields in TCB and link into list.*/
	for( i = DADSP_TO_CMSMADSP(driverData)->CMSMTxFreeCount +
				DADSP_TO_CMSMADSP(driverData)->CMSMPriorityTxFreeCount;
		  i != 0; i--)
	 {
		tcbPtr->TCB_DriverWS[0] = (UINT32)(tcbPtr + 1);

		COPY_ADDR( &tcbPtr->TCB_MediaHeader.EtherMedia.MH_Source,
					  &configTable->MLIDCFG_NodeAddress );
		++tcbPtr;
	 }	 /* End Filling in Node Address into TCBs */

	 /* Mark the Last TCB as the End */
	--tcbPtr;
	tcbPtr->TCB_DriverWS[0] = NULL;
	return( ODISTAT_SUCCESSFUL );

}	/*End MediaInit*/


#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
void	BuildASMStatStrings( StatTableEntry		*tableEntry )
 {
	/* Calling the CMSM for the generic statistics */
	tableEntry = CMSMBuildGenericStatStrings( tableEntry );

	/* now moving on to Topology specific statistics*/
	BuildTSMStatStrings( tableEntry );
  	return;
 }	/* End BuildASMStatStrings */

#endif		/* End ifdef IAPX386 */

void  BuildTSMStatStrings( StatTableEntry	*tableEntry )
 {
	/* Ethernet Media Specific Statistics Counters */
	(tableEntry++)->StatString = CETHER_SINGLECOL_STR;
	(tableEntry++)->StatString = CETHER_MULTICOL_STR;
	(tableEntry++)->StatString = CETHER_DEFERRED_STR;
	(tableEntry++)->StatString = CETHER_LATECOL_STR;
	(tableEntry++)->StatString = CETHER_EXCESSCOL_STR;
	(tableEntry++)->StatString = CETHER_CARRSENSE_STR;
	(tableEntry++)->StatString = CETHER_TXABORT_STR;
	(tableEntry++)->StatString = CETHER_RXABORT_STR;
  	return;

 }	/* End BuildASMStatStrings */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaAdjust(FRAME_DATA			*frameData);
*
*	Description:
*		This function initializes MLIDCFG_MaxFrameSize to its current value
*		or the maximum ethernet packet size, whichever is smaller. It then
*		must initialize MLIDCFG_BestDataSize and MLIDCFG_WorstDataSize
*		according to the MediaID.
*
*	Parameters in:
*		frameData -		Pointer to MSM/TSM Frame Data Space
*
*	Return Value:
*		ODISTAT_SUCCESSFUL			Frame Size values initialized.
*
************************************************************************/
ODISTAT	MediaAdjust(FRAME_DATA		*frameData)
{
	CONFIG_TABLE		*configTable;

	/* Initialize Locals */
	configTable = &frameData->ConfigTable;

	/* If the Frame Size is specified and Less than Max do nothing. */
	if( (configTable->MLIDCFG_MaxFrameSize) &&
		 (configTable->MLIDCFG_MaxFrameSize <= MAX_PACKET_LENGTH));
	else
		configTable->MLIDCFG_MaxFrameSize = MAX_PACKET_LENGTH;

	configTable->MLIDCFG_BestDataSize =
					configTable->MLIDCFG_MaxFrameSize - frameData->MediaHeaderSize;
	
	if (frameData->InternalMediaID == E8022_INTERNALID)
		configTable->MLIDCFG_WorstDataSize = configTable->MLIDCFG_BestDataSize - 1;
	else
		configTable->MLIDCFG_WorstDataSize = configTable->MLIDCFG_BestDataSize;

	/* Set the Raw Send bit in MLIDCFG_ModeFlags. */
	configTable->MLIDCFG_ModeFlags |= MM_RAW_SENDS_BIT;
	return( ODISTAT_SUCCESSFUL );

}	/* End MediaAdjust */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaReset(
*							PDRIVER_DS			driverData,
*							FRAME_DATA			*frameData,
*							OPERATION_SCOPE   operationScope
*							);
*
*	Description:
*		This function returns any sends queued up, and calls the HSM's
*		Driver Reset function.
*
*	Parameters in:
*		driverData -		Pointer to CHSMs Adapter Data Space.
*		frameData -			Pointer to MSM/TSM Frame Data Space.
*     operationScope -	OP_SCOPE_ADAPTER or OP_SCOPE_LOGICAL_BOARD
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			TSM/HSM reset successfully.
*		ODISTAT_FAIL					HSM's Driver Reset failed.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmReset from NT's ethertsm.c
*
************************************************************************/
ODISTAT	MediaReset(
							DRIVER_DATA		*driverData,
							FRAME_DATA		*frameData,
							OPERATION_SCOPE	operationScope)
{
	SHARED_DATA		*sharedData;
	ECB				*ecb;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Local Variables. */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	statTable = sharedData->CMSMGenericStatsPtr;

	/* Check if any sends are outstanding. */
	while ((ecb = sharedData->CMSMSendListHead) != NULL)
	 {
		/* Return ECBs to LSL */
		CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
		(statTable->Gen_Qdepth)--;
		sharedData->CMSMSendListHead = ecb->ECB_NextLink;
		ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		CLSL_FastSendComplete(ecb);
	 }

	/* Initialize queue and exit */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
	ecb = (ECB *)&sharedData->CMSMSendListHead;
	sharedData->CMSMSendListTail = ecb;
	sharedData->CMSMStatusFlags &= ~TXQUEUED;
	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

	return( sharedData->CMSMDPBlockPtr->DriverResetPtr(
					driverData,
					&frameData->ConfigTable,
					operationScope) );

}	/* End MediaReset */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaShutdown(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData,
*									UINT32					shutdownType,
*									OPERATION_SCOPE		operationScope
*									);
*
*	Description:
*		This function returns any sends queued up, returns any memory
*		allocated if a permanent shutdown and calls the HSM's
*		Driver Shutdown function.
*
*	Parameters in:
*		driverData -		Pointer to CHSMs Adapter Data Space.
*		frameData -			Pointer to MSM/TSM frame data space.
*		shutdownType -		SHUTDOWN_PERMANENT or SHUTDOWN_PARTIAL
*
*     operationScope -	OP_SCOPE_ADAPTER or OP_SCOPE_LOGICAL_BOARD
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			TSM/HSM shutdown successfully.
*		ODISTAT_FAIL					HSM's Driver shutdown failed.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmShutdown in NT's ethertsm.c
*
************************************************************************/
ODISTAT	MediaShutdown(
							DRIVER_DATA				*driverData,
							FRAME_DATA				*frameData,
							UINT32					shutdownType,
							OPERATION_SCOPE		operationScope)
{
	SHARED_DATA			*sharedData;
	MEDIA_DATA			*mediaData;
	ECB					*ecb;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Set up Local Variables */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData  = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/*Check if any sends are outstanding.*/
	while( (ecb = sharedData->CMSMSendListHead) != NULL )
	 {
		/*Return ECBs to LSL*/
		CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
		(statTable->Gen_Qdepth)--;
		sharedData->CMSMSendListHead = ecb->ECB_NextLink;
		ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		CLSL_FastSendComplete(ecb);
	 }

	/* Initialize queue and exit */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
	ecb = (ECB *)&sharedData->CMSMSendListHead;
	sharedData->CMSMSendListTail = ecb;
	sharedData->CMSMStatusFlags &= ~TXQUEUED;
	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

	/* Check if permanent shutdown */
	if (shutdownType == SHUTDOWN_PERMANENT)
	{
		/*This is a permanent shutdown free any memory that was allocated*/
		/*by this module and exit.*/

#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
		/*Free the EXTRA Statistics Table ("C" or ASSEMBLY).*/
		if (mediaData->NewStatsPtr)
      {
			CMSMFree (driverData, mediaData->NewStatsPtr);
			mediaData->NewStatsPtr = NULL;
		}
#endif

 		if ( mediaData->MulticastAddressList
			&& mediaData->MulticastAddressList != &MCPlaceHolder)
		{
			CMSMFree(driverData, mediaData->MulticastAddressList);
			mediaData->MulticastAddressList = NULL;
		}
		if ( mediaData->TCBList )
		{
			CMSMFree(driverData, mediaData->TCBList);
			mediaData->TCBList = NULL;
		}
	}

	    return( sharedData->CMSMDPBlockPtr->DriverShutdownPtr(
				driverData,
				&frameData->ConfigTable,
				shutdownType,
				operationScope) );

}	/* End MediaShutdown */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaNodeOverride(FRAME_DATA	*frameData,
*											MEON			mode);
*
*	Description:
*		This function will verify that the node override was of the correct
*		format and convert to canonical format if necissary.
*
*	Parameters in:
*		frameData -	Pointer to MSM/TSM frame data space.
*		mode -		'M' or 'm' if entered with msb(non-canonical) suffix
*						'L' or 'l' if entered with lsb(canonical) suffix
*
*	Values returned:
*		ODISTAT_SUCCESSFUL	Config Table NodeAddress in correct format
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmNodeOverride in NT's ethertsm.c
*
************************************************************************/
ODISTAT	MediaNodeOverride( FRAME_DATA	*frameData,
									 MEON			mode)
{
	CONFIG_TABLE	*configTable;
	UINT8				addressbyte;
	UINT32		i,j;

	configTable = &frameData->ConfigTable;

	/*check if M option was used.*/
	if (mode == 'M' || mode == 'm')
	{
		/*Swap address to LSB mode.*/
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[0];
		configTable->MLIDCFG_NodeAddress.nodeAddress[0] = MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[1];
		configTable->MLIDCFG_NodeAddress.nodeAddress[1] = MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[2];
		configTable->MLIDCFG_NodeAddress.nodeAddress[2] = MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[3];
		configTable->MLIDCFG_NodeAddress.nodeAddress[3] = MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[4];
		configTable->MLIDCFG_NodeAddress.nodeAddress[4] = MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[5];
		configTable->MLIDCFG_NodeAddress.nodeAddress[5] = MSMBitSwapTable[addressbyte];
	}

/* MPK 970306  range checking of node address */
	/* see how long we are equal to the max address size */
	addressbyte = 1;	/* need this for flag purposes later */
	i=0;
	while ((i < 5) 
		&& 
	(configTable->MLIDCFG_NodeAddress.nodeAddress[i] == MaxAddress[5-i]))
	    i++;	/*over the entire address */

	/* see how long we are equal to the min address size */
	j=0;
	while ((j < 5) 
		&& 
	(configTable->MLIDCFG_NodeAddress.nodeAddress[j] == MinAddress[5-j]))
	     j++;	/*over the entire address */


	/* now: i = index where the node different from the max or is completely equal
	        j = index where the node different from the min or is completely equal
	   are we in range, or did we jump out?  */
	if (((UINT8 *)configTable->MLIDCFG_NodeAddress.nodeAddress[i] > (UINT8 *)MaxAddress[5-i])
					||
	    ((UINT8 *)configTable->MLIDCFG_NodeAddress.nodeAddress[j] < (UINT8 *)MinAddress[5-j]))
	  { 			/* we are either over or under range here */
	    addressbyte = 0;    /* flag that we can do no more */
	    CMSMPrintString( configTable,	
			MSG_TYPE_INIT_WARNING,
			ETHERTSM_INVALID_NODE_ERROR_MSG,	
			NULL,	
			NULL );
	    for (i=0;i<6;i++)	/*over the entire address reset to ff <no override> */
		configTable->MLIDCFG_NodeAddress.nodeAddress[i] = 0xFF;
	  }
/* END MPK 970306 */


	/*Check group and locally administered bits.*/
	if ((configTable->MLIDCFG_NodeAddress.nodeAddress[0] & GROUPBIT)
		 && (addressbyte))
	 {
		/*Clear group bit and warn user.*/
		configTable->MLIDCFG_NodeAddress.nodeAddress[0] &= ~GROUPBIT;
		CMSMPrintString( configTable,	
				MSG_TYPE_INIT_WARNING,
				ETHERTSM_GRP_BIT_OVERRIDE_MSG,	
				NULL,	
				NULL );
	 }

	if ((!(configTable->MLIDCFG_NodeAddress.nodeAddress[0] & LOCALBIT))
		&& (addressbyte))
	 {
		/*Set locally administered bit and warn user.*/
		configTable->MLIDCFG_NodeAddress.nodeAddress[0] |= LOCALBIT;
		CMSMPrintString( configTable,	
				MSG_TYPE_INIT_WARNING,
 				ETHERTSM_LOCAL_BIT_OVERRIDE_MSG,	
				NULL,	
				NULL );
	 }
	return( ODISTAT_SUCCESSFUL );

}	/* End MediaNodeOverride	*/

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaAdjustNodeAddress(FRAME_DATA	*frameData);
*
*	Description:
*		Since Non-canonical Node Addressing is not supported under Ethernet,
*		this function just forces the Config Table Mode Flags to Canonical
*		format. PhysNodeAddress was filled in during CMSM_RegisterMLID.
*	
*	Parameters in:
*		frameData -		Pointer to MSM/TSM frame data space.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Config Table Mode Flags LSB/MSB bits set
*
*	Implied parameters:
*		NONE
*
************************************************************************/
ODISTAT	MediaAdjustNodeAddress(	FRAME_DATA	*frameData )
{
	/* NonCannonical Node Addresses */
	frameData->ConfigTable.MLIDCFG_ModeFlags |= MM_PHYS_NODE_ADDR_BIT;
	frameData->ConfigTable.MLIDCFG_ModeFlags &= ~MM_NONCANONICAL_BIT;
	return( ODISTAT_SUCCESSFUL );

}	/* End MediaAdjustNodeAddress */

/***********************************************************************\
*
*	Name:
*		ODISTAT MediaSetLookAheadSize(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData;
*									UINT32					size);
*
*	Description:
*		Update MLIDCFG_LookAheadSize to requested size, unless size is
*		greater than maximum look ahead size(128), or MLIDCFG_LookAheadSize
*		is already at least the requested size. We'll also need to update
*		DADSP_TO_CMSMADSP(driverData)->CMSMMaxFrameHeaderSize.
*
*	Parameters in:
*		frameData -						Pointer to MSM/TSM frame data space.
*		driverData -					Pointer to HSM adapter data space.
*		size -							Requested look ahead size.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Look Ahead Size successfully updated
*		ODISTAT_BAD_PARAMETER		size > 128
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmSetLookAheadSize in NT's ethertsm.c
*
************************************************************************/
ODISTAT	MediaSetLookAheadSize(
							DRIVER_DATA					*driverData,
							FRAME_DATA					*frameData,
							UINT32						size)
{
	SHARED_DATA				*sharedData;
	CONFIG_TABLE			*configTable;
	ODISTAT					odiStat = ODISTAT_SUCCESSFUL;		/*assume successful status.*/
	UINT32					i;

	sharedData = DADSP_TO_CMSMADSP(driverData);
	if( (size > frameData->ConfigTable.MLIDCFG_LookAheadSize) &&
		(size <= 128) )
	 {
		/*Set size for all virtual boards associated with this physical board.*/
		for (i = 0; i < 4; ++i)
		 {
			if (configTable = sharedData->CMSMVirtualBoardLink[i])
			 {
				configTable->MLIDCFG_LookAheadSize = (UINT16)size;
			 	sharedData->CMSMMaxFrameHeaderSize = size + sizeof(ETHER_MEDIA_HEADER);
			 }	/* Ok to update MaxFrameHeaderSize */
		 } /* Loop for each frame type */

		/*Call HSM if it has a SetLookAheadRoutine*/
		if (sharedData->CMSMDPBlockPtr->DriverRxLookAheadChangePtr != NULL)
		 {
			odiStat = sharedData->CMSMDPBlockPtr->DriverRxLookAheadChangePtr(
							driverData,
							(CONFIG_TABLE *)&frameData->ConfigTable);
		 }
	 }
	else
	 {
		if( size > 128 )
			odiStat = ODISTAT_BAD_PARAMETER;
	 }
	return( odiStat );
}	/* End MediaSetLookAheadSize */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaAddMulticast(
*									DRIVER_DATA		*driverData,
*									NODE_ADDR		*mcAddress);
*
*	Description:
*		This function adds the multicast address to the MLIDs multicast
*		address table. If its a new address, the HSM's DriverMulticastChange
*		function will get called. If the multicast address has already
*		added this address, we will only increment its EntryUsed Flag,
*
*	Parameters in:
*		driverData						HSMs adapter data space
*		mcAddress						Pointer to multicast node address
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Multicast address successfully added
*		ODISTAT_BAD_PARAMETER		Invalid multicast address
*		ODISTAT_OUT_OF_RESOURCES	Multicast table full
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
ODISTAT	MediaAddMulticast(
							DRIVER_DATA			*driverData,
							NODE_ADDR			*mcAddress)
{
	GROUP_ADDR_LIST_NODE		*mcEntry, *newMulticast;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	ODISTAT						odiStat = ODISTAT_SUCCESSFUL;
	UINT32						i;

	/* Setup the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;

	/*Check if valid Multicast Address.*/
	if ((mcAddress->nodeAddress[0] & MULTICASTBIT) == 0)
	{
		/*Address is not a valid Multicast*/
		return ODISTAT_BAD_PARAMETER;
	}
		
	/*Driver supports Multicast.*/
	/*See if table can hold address*/
	
	if( (mcEntry = FindAddressInMCTable( sharedData, mcAddress )) == NULL )
	 {
		/*Address is not in Table*/
		/*Search for an empty slot.*/
		mcEntry = mediaData->MulticastAddressList;
		for (i = 0; i < mediaData->MaxMulticastAddresses; ++i)
		 {
			/*Check if entry is in use.*/
			if (mcEntry->GRP_ADDR_COUNT == 0)
				break;
			++mcEntry;
		 }

		if( i >= sharedData->CMSMDPBlockPtr->DriverMaxMulticast )
			return( ODISTAT_OUT_OF_RESOURCES );

		if (i >= mediaData->MaxMulticastAddresses)
		 {
			/*Table is full.*/
			newMulticast = CMSMAlloc(driverData,
											(mediaData->MaxMulticastAddresses + 32)
											* sizeof(GROUP_ADDR_LIST_NODE));
			if( newMulticast == NULL )
				return( ODISTAT_OUT_OF_RESOURCES );

			CMSMmemset( newMulticast, 0, ((mediaData->MaxMulticastAddresses + 32)
					* sizeof(GROUP_ADDR_LIST_NODE)) );

			CMSMmemcpy( newMulticast,
					mediaData->MulticastAddressList,
					(mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE)));

			CMSMFree(driverData, mediaData->MulticastAddressList);
			mediaData->MulticastAddressList = newMulticast;
			mediaData->MaxMulticastAddresses += 32;
			return ( MediaAddMulticast(driverData, mcAddress) );
		 }
	 }

	/*Address has been found or an empty location was returned.*/
	if( !(mcEntry->GRP_ADDR_COUNT) )
	 {
		/*Add address to table*/
		COPY_ADDR( mcEntry->GRP_ADDR.nodeAddress, mcAddress->nodeAddress );
		mediaData->MulticastAddressCount++;
	 }
	/*Increment Entry Used count.*/
	++(mcEntry->GRP_ADDR_COUNT);

	/*Inform driver of Multicast Change.*/
	if( (mediaData->PromiscuousMode == 0) ||
		 (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) )
		odiStat = CEtherTSMUpdateMulticast(driverData);

	return( odiStat );
}	/* End MediaAddMulticast */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaDeleteMulticast(
*									DRIVER_DATA		*driverData,
*									NODE_ADDR		*mcAddress);
*
*	Description:
*		This function deletes the multicast address from the MLIDs multicast
*		address table. If another process has added this multicast address,
*		we will only decrement its EntryUsed Flag. If the GRP_ADDR_COUNT flag equals
*		zero, no other process is expecting this multicast address, the address
*		will get deleted and the HSM's DriverMulticastChange function will
*		get called.
*
*	Parameters in:
*		driverData						HSMs adapter data space
*		mcAddress						Pointer to multicast node address
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Instance of Multicast address deleted
*		ODISTAT_BAD_PARAMETER		Invalid multicast address
*		ODISTAT_ITEM_NOT_PRESENT	Multicast address not in table
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*
************************************************************************/
ODISTAT	MediaDeleteMulticast(
							DRIVER_DATA					*driverData,
							NODE_ADDR					*mcAddress)
{
	GROUP_ADDR_LIST_NODE			*mcEntry;
	SHARED_DATA						*sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;
	UINT32							count;

	/* Setup Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;

	/*Check if valid Multicast Address.*/
	if ((mcAddress->nodeAddress[0] & MULTICASTBIT) == 0)
	 {
		/*Address is not a valid Multicast*/
		return ODISTAT_BAD_PARAMETER;
	 }
		
	/*Driver supports Multicast.*/
	/*See if Entry is in table*/
	if( (mcEntry = FindAddressInMCTable( sharedData, mcAddress)) == NULL )
	 {
		/*Address is not in Table*/
		return ODISTAT_ITEM_NOT_PRESENT;
	 }

	/*Address has been found.*/
	/*Decrement Entry Used count.*/
	--(mcEntry->GRP_ADDR_COUNT);
	if (mcEntry->GRP_ADDR_COUNT == 0)
	 {
		/* Collapse Muticast Table */
		count = (UINT32)((UINT8 *)mcEntry - (UINT8 *)mediaData->MulticastAddressList);
		count = (mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE)) - count;
		count -= sizeof(GROUP_ADDR_LIST_NODE);
		CMSMmemcpy(mcEntry, mcEntry + 1, count);

		/* Now Zero out the last Address in the table. */
		CMSMmemset(
				mediaData->MulticastAddressList + (mediaData->MaxMulticastAddresses - 1),
				0,	sizeof(GROUP_ADDR_LIST_NODE));

		mediaData->MulticastAddressCount--;

		if ( (mediaData->PromiscuousMode == 0) ||
  			  (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) )
			/*Inform driver of Multicast Change.*/
			odiStat = CEtherTSMUpdateMulticast(driverData);

	 }	/* Address was deleted. */
	return( odiStat );
}	/* End MediaDeleteMulticast */

/***********************************************************************\
*                                                                       *
*	Name:
*		GROUP_ADDR_LIST_NODE	*FindAddressInMCTable(
*									SHARED_DATA		*sharedData,
*									NODE_ADDR		*mcAddress);
*
*	Description:
*	                This function checks to see if a multicast address has
*					been added to the multicast address table.
*
*	Parameters in:
*		sharedData						MSM/TSM/HSM shared adapter data space
*		mcAddress						Pointer to multicast node address
*
*	Values returned:
*		Pointer to Multicast address entry if found.
*		Otherwise NULL.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
GROUP_ADDR_LIST_NODE *FindAddressInMCTable(
				SHARED_DATA					*sharedData,
				NODE_ADDR					*mcAddress)
{
	GROUP_ADDR_LIST_NODE			*mcEntry;
	MEDIA_DATA						*mediaData;
	UINT32							i;
	UINT32							maxAddresses;

	mediaData = sharedData->CMSMMediaDataPtr;
	mcEntry = mediaData->MulticastAddressList;

	maxAddresses = (sharedData->CMSMDPBlockPtr->DriverMaxMulticast <
						 mediaData->MaxMulticastAddresses) ?
					  	 sharedData->CMSMDPBlockPtr->DriverMaxMulticast :
					  	 mediaData->MaxMulticastAddresses;

	for (i = 0; i < maxAddresses; ++i)
	{
		/* Check if entry is in use.*/
		if (mcEntry->GRP_ADDR_COUNT != 0)
		{
			/*Check to see if address matches*/
			if (ADDR_EQUAL(mcEntry->GRP_ADDR.nodeAddress, mcAddress->nodeAddress))
			{
				return mcEntry;
			}
		}
		else break;
		++mcEntry;
	}
	return (GROUP_ADDR_LIST_NODE *)NULL;
} /*End FindAddressInMCTable*/

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT	CEtherTSMUpdateMulticast(
*									DRIVER_DATA			*driverData);
*
*	Description:
*		This function gives the current multicast address table to the
*		HSMs DriverMulticastChange function so that the HSM can update its
*		multicast registers. It is called by MediaAddMulticast and
*		MediaDeleteMulticast. It may also be called by the HSM if it
*		has just reset its adapter and need its multicast address registers
*		updated.
*
*	Parameters in:
*		driverData						HSMs adapter data space
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			HSMs Multicast Address registers updated
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmUpdateMulticast in NT's ethertsm.c
*
************************************************************************/
ODISTAT	CEtherTSMUpdateMulticast(
				DRIVER_DATA			*driverData)
{
	SHARED_DATA						*sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;
	UINT32							functionalAddress = NULL;

	 /* Initialize Local Variables. */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;

	if( sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr &&
			mediaData->PromiscuousMode )
    {
		odiStat = sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr(
							driverData,
							(CONFIG_TABLE *)sharedData->CMSMDefaultVirtualBoard,
							mediaData->PromiscuousMode);

		if ( (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) &&
	    	  (sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr) )

			odiStat = sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
							driverData,
							sharedData->CMSMDefaultVirtualBoard,
							mediaData->MulticastAddressList,
							mediaData->MulticastAddressCount,
							functionalAddress);

    }
	else if (sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr)
    {
			odiStat = sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
							driverData,
							sharedData->CMSMDefaultVirtualBoard,
							mediaData->MulticastAddressList,
							mediaData->MulticastAddressCount,
							functionalAddress);
    }

	return( odiStat);
}	/*End	CEtherTsmUpdateMulticast*/

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaPromiscuousChange(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData,
*									UINT32					promiscuousState,
*									UINT32					*promiscuousMode);
*
*	Description:
*		This function enables/disables promiscuous mode depending on
*		promiscuousState.
*
*	Parameters in:
*		frameData -				Pointer to Frame Data Space
*		driverData -			HSMs adapter data space
*		promiscuousState -	PROM_STATE_OFF to disable promiscuous mode
*									PROM_STATE_ON  to enable  promsicuous mode
*
*		promiscuousMode -		PROM_MODE_QUERY     to query promiscuous state
*									PROM_MODE_MAC       to enable all MAC reception
*									PROM_MODE_NON_MAC   to enable all Non-MAC reception
*									PROM_MODE_SMT		  to enable all SMT Frames (FDDI)
*									PROM_MODE_MACANDNON to enable MAC and Non-MAC
*	Parameters out:
*		promiscuousMode -		The current promiscuous mode will be returned
*									to the location pointed to by this field.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Promiscuous Mode enabled/disabled
*		ODISTAT_BAD_COMMAND			Promiscuous Mode not supported by MLID
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*
************************************************************************/
ODISTAT	MediaPromiscuousChange(
								DRIVER_DATA		*driverData,
								FRAME_DATA		*frameData,
								UINT32			promiscuousState,
								UINT32			*promiscuousMode )
{
	SHARED_DATA						*sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;
	UINT32							i;

		/* Initialize Local Variables. */
		configTable = &frameData->ConfigTable;
		sharedData = DADSP_TO_CMSMADSP(driverData);
		mediaData = sharedData->CMSMMediaDataPtr;

		/* Check if promiscuous mode supported by this mlid. */
		if ((configTable->MLIDCFG_ModeFlags & MM_PROMISCUOUS_BIT) &&
			(sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr) != NULL)
		 {
			/* The MLID supports promiscuous mode */
			if (*promiscuousMode != PROM_MODE_QUERY)
			 {
				/* We need to change the promiscuousMode */
				if (promiscuousState == PROM_STATE_ON)
				 {
					/* We are Enabling promiscuous mode */
					for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
					 {
						mediaData->PromiscuousCounters[i] += ((*promiscuousMode >> i) & 0x01);
					 }
				 }	/* End Enable Promiscuous Mode */
				else		/* PROM_STATE_OFF */
				 {
					/* We are Disabling promiscuous mode */
					for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
					 {
						if (mediaData->PromiscuousCounters[i])
							mediaData->PromiscuousCounters[i] -= ((*promiscuousMode >> i) & 0x01);
					 }
				 }	/* End Disable Promiscuous Mode */

				/**************************************/
				/* Determine the new promiscuous Mode */
				/**************************************/
         	mediaData->PromiscuousMode = 0;			/* Initialize mode */
				for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
				 {
					if (mediaData->PromiscuousCounters[i])
						mediaData->PromiscuousMode = (mediaData->PromiscuousMode | (1 << i));
				 }

				/* Check to see if the remote multicast bit is set. */
				if (mediaData->PromiscuousMode & PROM_MODE_RMC)
				{
					/* Find out if this driver supports remote multicase.*/
					if (sharedData->CMSMDPBlockPtr->DriverParameterSize <
						(UINT32)&(((DRIVER_PARM_BLOCK *)0)->HSMSpecVersionStringPtr))
					{
						/* This driver does not have the spec string so it must not */
						/* be able to support remote multicast. */
						mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
					}
					else
					{
						/* The driver has the spec string.  Check to see if it */
                  /* supports remote multicast.  Must be at least cspec 1.11 */
						/* or asm spec 3.31. */
						if (configTable->MLIDCFG_ModeFlags & MM_C_HSM_BIT)
						{
							if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MAJOR_VERSION_POS]
								< '2')
							{
								if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MINOR_VERSION_TENS_POS]
                           < '2')
								{
									if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MINOR_VERSION_ONES_POS]
										< '1')
									{
                              /* This is less than c spec 1.11. */
										mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
									}
								}
							}
						}
						else
						{
                     if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MAJOR_VERSION_POS]
                        < '4')
							{
                        if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MINOR_VERSION_TENS_POS]
                           < '4')
								{
									if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MINOR_VERSION_ONES_POS]
                              < '1')
									{
										/* This is less than a spec 3.31. */
										mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
									}
								}
							}
						}
					}
				}

				/*********************************/
				/* Inform the Stacks of a change */
				/*********************************/
				CLSL_ControlStackFilter(
							configTable->MLIDCFG_BoardNumber,
							PSTK_PROMISCUOUS_STATE,
							DT_PROMISCUOUS,
							&mediaData->PromiscuousMode,
							NULL);

				/*************************/
				/* Inform MLID of change */
				/*************************/
				odiStat = sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr(
							driverData,
							configTable,
							mediaData->PromiscuousMode);

				/***************************************************************/
				/* If promiscuous mode is being disabled, make sure that the	*/
				/* the multicast addresses get updated too.                  	*/
				/* Disabling promiscuous mode may destroy the multicast table	*/
				/* on the card. 																*/
				/***************************************************************/
				if (mediaData->PromiscuousMode == 0)
				 {
					/* Check that there are addresses to be updated. */
					if (mediaData->MulticastAddressCount != 0)
					 {
						odiStat =
							sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
								driverData,
								sharedData->CMSMDefaultVirtualBoard,
								mediaData->MulticastAddressList,
								mediaData->MulticastAddressCount,
								0);
					 }
				 } /* End Promiscuos Disabled, so update Multicast tables. */
			 }	/* End Adjust PromiscuosMode */

			/****************************************/
			/* Return the current promiscuous mode. */
			/****************************************/
			*promiscuousMode = mediaData->PromiscuousMode;
			return( odiStat );
		 } /* End HSM Supports Promiscuous Mode */

	/*********************************************/
	/* Set Mode to 0 because we don't support it.*/
	/*********************************************/
	*promiscuousMode = 0;
	return( ODISTAT_BAD_COMMAND );

}	/* End MediaPromiscuousChange */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaRegisterMonitor(
*									FRAME_DATA				*frameData,
*									void                 (*TxMonRoutine)(TCB *),
*									BOOLEAN					MonitorState);
*
*	Description:
*		This function allows a monitor protocol stack to Register/Deregister
*		its Transmit Monitor Routine. The routine will be called by the TSM
*		after a packet has been transmited with a pointer to the TCB. Only one
*		transmit monitor may be registered per logical board.
*
*	Parameters in:
*		frameData -				Pointer to Frame Data Space
*		TxMonRoutine -			Pointer to transmit monitor function
*		MonitorState -			TRUE if registering function
*									FALSE if deregistering function
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Routine registered/deregisteed
*		ODISTAT_OUT_OF_RESOURCES	Another transmit routine already registered
*		ODISTAT_NO_SUCH_HANDLER		Tried to disable another processes routine
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
ODISTAT	MediaRegisterMonitor(
				FRAME_DATA					*frameData,
				void                    (*TxMonRoutine)(TCB *),
				BOOLEAN						MonitorState)
{
	UINT32				status;

	for (;;)
	 {
		if (MonitorState == TRUE)
		 {
			/*We need to enable the Monitor.*/
			/*Check to see if already enabled.*/
			if (frameData->TransmitRoutine)
			 {
				/*We are already enabled.*/
				status = ODISTAT_OUT_OF_RESOURCES;
				break;
			 }
			frameData->TransmitRoutine = TxMonRoutine;

			status = ODISTAT_SUCCESSFUL;
			break;
		 }

		/*Disable the Monitor.*/
		if ( TxMonRoutine == frameData->TransmitRoutine )
		 {
			frameData->TransmitRoutine = NULL;
			status = ODISTAT_SUCCESSFUL;
			break;
		 }

		/*An imposter tried to disable the Monitor.*/
		status = ODISTAT_NO_SUCH_HANDLER;
		break;

	 }	/* End of Infinite for loop */
	return( status );

}	/* End MediaRegisterMonitor */


/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaGetParamters(
*           CONFIG_TABLE      *configTable);
*
*	Description:
*		This function allows the TSM to parse for custom parameters
*     The TSM must parse for FRAME and NODE.
*
*	Parameters in:
*		hsmParameters        Pointer to the CHSMs parameterblock.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Routine fetched required parameters
*		ODISTAT_FAIL               Parameters could not be obtained.
*
*  Notes:
*
*
************************************************************************/
ODISTAT	MediaGetParameters(
            CONFIG_TABLE      *configTable)
{
   DRIVER_DATA *driverData;
   FRAME_DATA  *frameData;
   ODISTAT     status;
   UINT32      frameIndex;
   MEON_STRING *whiteSpace = ETHERTSM_WHITESPACE_MSG;

	frameData = CONFIG_TO_MSM(configTable);

   /* If MLIDCFG_FrameID is set To UNUSED This is virtual board loading. */
	/* The driverData was saved in MLIDCFG_FrameTypeString. */
   if ( (configTable->MLIDCFG_FrameID & 0xFF00) == (UINT16)(UNUSED & 0xFF00))
		driverData = (DRIVER_DATA *)configTable->MLIDCFG_FrameTypeString;
	else driverData = NULL;

   if (driverData != NULL)
    {
      FrameOptionList.OptionCount = 0;

      for (frameIndex = 0; frameIndex < 4; ++frameIndex)
       {
         if(DADSP_TO_CMSMADSP(driverData)->CMSMVirtualBoardLink[frameIndex] == NULL )
         {
            FrameOptionList.StrOptVal[FrameOptionList.OptionCount]
               = EtherTSMFrameDescriptTable[frameIndex];
            ++FrameOptionList.OptionCount;
         }
       }
      FrameOption.Flags = ENUMPARAM | REQUIREDPARAM;
    }
   else
    {
      FrameOptionList.OptionCount = 4;

      for (frameIndex = 0; frameIndex < 4; ++frameIndex)
      {
         FrameOptionList.StrOptVal[frameIndex]
            = EtherTSMFrameDescriptTable[frameIndex];
      }
      FrameOption.Flags = STRINGPARAM;
    }

   status = CMSMParseCTSMParameters(configTable, &FrameOption);

   if (!(status == ODISTAT_SUCCESSFUL || status == ODISTAT_ITEM_NOT_PRESENT))
      return( status );
   else if (FrameOption.String[0] != '\0')
    {
		/* Place a NULL in the FrameOption.String so that the CMSMstrcmp will */
		/* compare the correct piece of the string. */
		FrameOption.String[CMSMstrcspn(FrameOption.String, whiteSpace)] = '\0';

      for (frameIndex = 0; frameIndex < 4; ++frameIndex)
      {
         if (CMSMstrcmp(FrameOption.String, EtherTSMFrameDescriptTable[frameIndex]) == 0)
         {
				break;
         }
      }
      if (frameIndex >= 4)
         return ODISTAT_FAIL;
    }
   else
		frameIndex = 0; /* Use default frame. */

   configTable->MLIDCFG_FrameTypeString
		= EtherTSMFrameDescriptTable[frameIndex];
   frameData->InternalMediaID = frameIndex;
   frameData->MediaHeaderSize
		= frameData->MPBlock->MediaHeaderSizeArray[frameData->InternalMediaID];
   configTable->MLIDCFG_FrameID
   	= frameData->MPBlock->MediaIDArray[frameData->InternalMediaID];

   /* Now lets check to see if the Node was entered. */
   if (NodeOption.String[0] != '\0')
   {
      /* We have a Node Address Override convert string to HEX. */
      CMSMASCIINodeToHex(
                  configTable,
                  NodeOption.String);
   }

   return( ODISTAT_SUCCESSFUL );
}	/* End MediaGetParameters */

/***********************************************************************\
*
*	Name:
*		ODISTAT MediaGetMulticastInfo(
*            struct _DRIVER_DATA_    *driverData,
*            ECB                     *MulticastInfoECB);
*
*   Description:  This function returns the Group(Multicast) and/or
*                 Functional addresses the MLID is using.
*
*   Input Parameters:
*
*            driverData        Pointer to the drivers data space.
*
*            MulticastInfoECB  Pointer to an ECB to return info in.
*
*   Values returned:
*
*     ODISTAT_SUCCESSFUL       Command was successfully executed.  The
*                              ECB is returned to the caller.
*
*     ODISTAT_RESPONSE_DELAYED The requested operation was successfully
*                              started but will complete asynchronously.
*                              MulticastInfoECB event service routine
*                              will be called with the cpmpletion code
*                              when the command has finished execution.
*
*     ODISTAT_BAD_COMMAND      GetMulticastInfo is not supported by
*                              the MLID.
*
*     ODISTAT_OUT_OF_RESOURCES The buffer presented in the MulticastInfoECB
*                              to transfer the Group(Multicast) addresses
*                              used by the MLID into was insufficient.  The
*                              field MulticastInfoECB ECB_DataLength contains
*                              the size the buffer should be to transfer
*                              Group(Multicast) addresses used by the MLID.
*
*
*   Input/Output Parameters:
*
*				  MulticastInfoECB
*                      Pointer to an ECB in which to return Group
*                      (Multicast) and/or Functional address
*                      information.  The MulticastInfoECB contains a
*                      single fragment into which to copy Group
*                      (Multicast) address information:
*
*                      On Entry:
*
*                         ECB_FragmentCount
*                                   Is equal to 1.
*
*                         ECB_FragmentAddress
*                                   Points to the buffer to copy Group
*                                   (Multicast) address structures.
*
*                         ECB_FragmentLength
*                                   The size of the buffer to copy Group
*                                   (Multicast) address structures.
*
*
*                      On Exit:
*
*                         ECB_ImmediateAddress
*                                   Contains the Functional address currently
*                                   used by the MLID,  zero if the MLID does
*                                   (Multicast) address structures.
*                                   not use Functional addresses (e.g.,
*                                   Ethernet, FDDI), or the Functional address
*                                   is currently unused.
*
*                         ECB_DataLength
*
*                                   Contains the number of bytes transferred
*                                   into the buffer indicated by
*                                   ECB_FragmentAddress.  If the MLID does not
*                                   use Group addresses the ECB_DataLength
*                                   field will be set to zero.
*
*                                   If ODISTAT_OUT_OF_RESOURCES is returned.
*                                   This field will contain the size of the
*                                   buffer required to return the Multicast
*                                   address information.
*
*                         ECB_FragmentAddress
*
*                                   Points to the buffer passed in on entry.
*                                   The buffer has been filled in with a
*                                   series of Group (Multicast) address
*                                   structures used by the MLID.  If the MLID
*                                   does not use Group (Multicast) addresses
*                                   or there are non active the ECB_DataLength
*                                   field will be set to zero.  The address
*                                   structure is defined as follows:
*
*                                   typedef struct _GROUP_ADDR_LIST_NODE_
*                                   {
*                                       NODE_ADDR     GRP_ADDR;
*                                       UINT16        GRP_ADDR_COUNT;
*                                   }GROUP_ADDR_LIST_NODE;
*
*                         ECB_FragmentLength
*
*                                   Unchanged.
*
*
*   Notes:
*
*     The MulticastInfoECb's ESR will only be called if the function
*     returns ODISTAT_RESPONSE_DELAYED, any other return code implies
*     that the processing of the ECB is complete and the ECB contains
*     valid information (e.g., ODISTAT_OUT_OF_RESOURCES implies the
*     ECB_DATALength field contains the size of the buffer required
*     to return appropriate information.
*
************************************************************************/
ODISTAT	MediaGetMulticastInfo(
								struct _DRIVER_DATA_    *driverData,
										ECB               *MulticastInfoECB )
{
	GROUP_ADDR_LIST_NODE	*mcEntry;
	SHARED_DATA		  		*sharedData;
	MEDIA_DATA				*mediaData;
	void						*dataPtr;
	UINT32					i, bufSizeNeeded;

	/* Initialize Local Variables */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* initialize fields */
	MulticastInfoECB->ECB_DataLength = 0;
	CMSMmemset( MulticastInfoECB->ECB_ImmediateAddress.nodeAddress,
					0, ADDR_SIZE );


	/* Calculate the size of buffer needed. */
	for (i = 0, bufSizeNeeded = 0; i < mediaData->MaxMulticastAddresses; i++)
	 {
		/*Check if entry is in use.*/
		if (mcEntry->GRP_ADDR_COUNT != 0)
		 {
		 	bufSizeNeeded += sizeof(GROUP_ADDR_LIST_NODE);
		 }
		++mcEntry;
	 }

	/* Make sure there is enough room to copy data */
	if (MulticastInfoECB->ECB_Fragment[0].FragmentLength < bufSizeNeeded)
	 {
		MulticastInfoECB->ECB_DataLength = bufSizeNeeded;
		return( ODISTAT_OUT_OF_RESOURCES );
	 }

	/* Get data for MulticastInfoECB */
	dataPtr = MulticastInfoECB->ECB_Fragment[0].FragmentAddress;
	mcEntry = mediaData->MulticastAddressList;

	for (i = 0; i < mediaData->MaxMulticastAddresses; i++)
	 {
		/*Check if entry is in use.*/
		if (mcEntry->GRP_ADDR_COUNT != 0)
		 {
			/* Copy Entry into MulticastInfoECB */

			CMSMmemcpy(dataPtr, mcEntry,sizeof(GROUP_ADDR_LIST_NODE));
			dataPtr = (UINT8 *)dataPtr + sizeof(GROUP_ADDR_LIST_NODE);
		
  			/* Update amount of data copied. */
  			MulticastInfoECB->ECB_DataLength =
  				MulticastInfoECB->ECB_DataLength +
  				sizeof(GROUP_ADDR_LIST_NODE);
  		 }
		++mcEntry;
	 }
    return( ODISTAT_SUCCESSFUL );

}	/* End MediaGetMulticastInfo */

/***********************************************************************\
*                                                                       *
*	Name:
*		void	MediaSend(
*							ECB					*ecb,
*							CONFIG_TABLE		*configTable);
*
*	Description:
*		This function converts the ECB to a TCB, and passes the TCB to
*		the HSMs DriverSend routine. Converting to TCB means building a
*		Media Access Header and verifying contiguous, phyiscally addressed
*		fragments if the HSM is expecting it.
*
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		configTable -		Pointer to configuration table.
*
*	Values returned:
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	MediaSend(ECB					*ecb,
					CONFIG_TABLE		*configTable)
{
	DRIVER_DATA					*driverData;
	FRAME_DATA					*frameData;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	GENERIC_STATS				*statTable;
	TCB							*tcb;
   void			        	   *physTcb;		
	UINT32						pktSize;
	UINT32						oldState;

	/* Initialize the Local Variables */
	frameData = CONFIG_TO_MSM(configTable);
	driverData = frameData->DriverData;
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	/*Check if logical board is shutdown or if we already have */
	/*transmits waiting on the queue.*/

	if((sharedData->CMSMStatusFlags & TXQUEUED) ||
				(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
	 {
		/*Either the board is shutdown or */
		/*transmits are on the queue.*/

		if(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT)
		 {
			CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

			/* The board is shutdown, */
			/* Set the status and return the ECB. */
			ecb->ECB_Status = (UINT16)ODISTAT_MLID_SHUTDOWN;
			CLSL_FastSendComplete( ecb );
		 }
		else
		 {
			/* Transmits are on the queue. */
			/* Queue this send. */
			CMSMEnqueueSend( frameData, driverData, ecb );
			CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		 }
		return;
	 } /* End:Card is ShutDown or Tx Already Q'd */

	/* If we are not in a critical section we can send this */
	/* packet imediatly to the card. */
	if(sharedData->CMSMInCriticalSection)
	 {
		/*We are in a critical section*/
		/*Queue the send.*/
		CMSMEnqueueSend( frameData, driverData, ecb );
	 }
	else
	 {
		/* We are not in a critical section */
		/* Make sure the card has resources to process this send. */
		if( sharedData->CMSMTxFreeCount > 0 )
		 {
			/* We have resources.*/
			/* This is here for DMA, or BusMaster boards */
			/* Check if DriverUsesDMA or DriverSupportsPhysFrags bits are Set. */
			/* If set get a Physically contiguous ECB */
			if ( sharedData->CMSMNeedContiguousECB )
				ecb = CMSMGetPhysicalECB( driverData, ecb );

			/* Disable the cards interrupts. */
			if ( sharedData->CMSMAdapterIntDisabled == 0 )
 				if ( sharedData->CMSMDPBlockPtr->DriverDisableInterruptPtr )
				{
#ifdef TRACE_OUT
	CMSMOutMEON('S',0);
#endif
					sharedData->CMSMDPBlockPtr->DriverDisableInterruptPtr( driverData, FALSE );
				}

			++sharedData->CMSMAdapterIntDisabled;

			if( sharedData->CMSMDPBlockPtr->DriverSendWantsECBs != 0 )
			 {
				/* Driver Wants ECBs */
				/* check that the packet is not to big. */
				if( (ecb->ECB_DataLength <= configTable->MLIDCFG_BestDataSize) ||
					 ((( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK ) &&
					  (ecb->ECB_DataLength <= configTable->MLIDCFG_MaxFrameSize)) )
				 {
					/*Bump Queue Depth*/
					++(statTable->Gen_Qdepth);
					--(sharedData->CMSMTxFreeCount);
					++sharedData->CMSMInCriticalSection;

					CMSMReleaseProtection(
									sharedData->CMSMProtectionObject,
									oldState);

					/* Give the ECB to the HSM to process. */
					sharedData->CMSMDPBlockPtr->DriverSendPtr(
									driverData,
									configTable,
									(TCB *)ecb,
									0,
									CMSMGetPhysical(ecb));

					/* Check to see if anything got queued up while we	*/
					/* were in DriverSend; if so service the queue now	*/
				   if (sharedData->CMSMStatusFlags & TXQUEUED)
				    {
				   	do
				       {
							/* CEtherTSMGetNextSend acquires his own protection. */
							/* No need to do it here. */
				      	tcb = CEtherTSMGetNextSend(
				                  driverData,
  					               &configTable,
					               &pktSize,
				                  &physTcb);

				         if( tcb )
							 {
				         	sharedData->CMSMDPBlockPtr->DriverSendPtr (
				                     driverData,
				                     configTable,
				                     tcb,
				                     pktSize,
				                     physTcb);
							 }
				     	 }
						while (tcb && (sharedData->CMSMStatusFlags & TXQUEUED));
   				 }	/* End at least one TCB was Q'd */
					
					CMSMAcquireProtection(
									sharedData->CMSMProtectionObject,
									&oldState);

					--sharedData->CMSMInCriticalSection;
				 } /* Packet was ok to send */
				else
				 {
					++(statTable->Gen_TxTooBig);
					/*If we are a DMA card then we need to return the ECB to the pool*/
					if (sharedData->CMSMNeedContiguousECB)
						ecb = CMSMReturnPhysicalECB(driverData, ecb);

					/*Set ECB status.*/
					ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
					CLSL_FastSendComplete(ecb);
				 }
			 }
			else
			 {
				/* Get a TCB and create the MAC Header. */
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];
				pktSize = frameData->SendRoutine( sharedData, ecb,	tcb );
				ecb->ECB_DriverWorkspace.DWs_pval = (UINT32 *) frameData;

				if (sharedData->CMSMNeedContiguousECB)
				 {
					tcb->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
					tcb->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&tcb->TCB_MediaHeader );
				 }
						
				/* check that the packet is not to big. */
				if( pktSize <= configTable->MLIDCFG_MaxFrameSize )
				 {
					/* Bump Queue Depth */
					--(sharedData->CMSMTxFreeCount);
					++(statTable->Gen_Qdepth);
					++sharedData->CMSMInCriticalSection;

					CMSMReleaseProtection(
									sharedData->CMSMProtectionObject,
									oldState);

					/* Give the TCB to the HSM to process. */
					sharedData->CMSMDPBlockPtr->DriverSendPtr(
									driverData,
									configTable,
									tcb,
									pktSize,
									NULL);

					/* Check to see if anything got queued up while we	*/
					/* were in DriverSend; if so service the queue now	*/
				   if (sharedData->CMSMStatusFlags & TXQUEUED)
				    {
				   	do
				       {
							/* CEtherTSMGetNextSend acquires his own protection. */
							/* No need to do it here. */
				      	tcb = CEtherTSMGetNextSend(
				                  driverData,
  					               &configTable,
					               &pktSize,
				                  &physTcb);

				         if (tcb)
							 {
				         	sharedData->CMSMDPBlockPtr->DriverSendPtr (
				                     driverData,
				                     configTable,
				                     tcb,
				                     pktSize,
				                     physTcb);
							 }
				     	 }
				      while (tcb && (sharedData->CMSMStatusFlags & TXQUEUED));
   				} /* Something was Queued up. */
					
					CMSMAcquireProtection(
									sharedData->CMSMProtectionObject,
									&oldState);

					--sharedData->CMSMInCriticalSection;
				 }
				else
				 {
					++(statTable->Gen_TxTooBig);
					/* If we are a DMA card then we need to return the ECB to the pool */
					if (sharedData->CMSMNeedContiguousECB)
						ecb = CMSMReturnPhysicalECB(driverData, ecb);

         		/* Return TCB and link into TCB list. */
         		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		         mediaData->TCBHead = tcb;

					/* Set ECB status. */
					ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
					CLSL_FastSendComplete( ecb );
				 } /* Packet turned out to be too big */
			 }	/* Driver Wanted TCB's */

			/* Reenable the cards interrupts. */
			--sharedData->CMSMAdapterIntDisabled;
			if ( sharedData->CMSMAdapterIntDisabled == 0 )
				if (sharedData->CMSMDPBlockPtr->DriverEnableInterruptPtr)
			  	{
#ifdef TRACE_OUT
	CMSMOutMEON('s',0);
#endif
						sharedData->CMSMDPBlockPtr->DriverEnableInterruptPtr(
												driverData);
				}
		 }
		else
		 {
			/* We don't have any transmit resources available. */
			/* Queue the send. */
			CMSMEnqueueSend( frameData, driverData, ecb );
		}
	} /* Not in Critical Section, and Sent the Frame. */

	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

#ifdef OS_NIOS
	CLSL_ServiceEvents();
#endif

 	return;

}	/* End MediaSend */

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32	CMediaSendRaw8023(
*							ECB					*ecb,
*							TCB					*tcb);
*
*	Description:
*		This function constructs a Raw 802.3 media header. It will fill
*		in all the TCB fields.
*
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
*
*	Values returned:
*		Packet Size(Padded and evenized).
*
*	Implied parameters:
*
*  Notes:
*
*
************************************************************************/
UINT32 CMediaSendRaw8023(
				SHARED_DATA	*sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	UINT32	pktSize;
	UINT16	evenSize;

	/* Bag compiler warnings */
   sharedData = sharedData;

	/*set up fragment structure.*/
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
	
	/* Get size of data packet. */
	/* and evenize it if odd length. */
	pktSize = ecb->ECB_DataLength;
	evenSize = (UINT16)(pktSize + (pktSize & 0x00000001));

	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		pktSize += SIZE_E8023;

		/* fill in MAC header. */
		COPY_ADDR( &tcb->TCB_MediaHeader.EtherMedia.MH_Destination,
					  &ecb->ECB_ImmediateAddress );
		tcb->TCB_MediaHeader.EtherMedia.MH_Length = VALUE_TO_HILO_UINT16( evenSize );
		tcb->TCB_MediaHeaderLen = SIZE_E8023;
	 }

	/* Fill in DataLen and Board Number */
	tcb->TCB_DataLen = pktSize;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;

   /* evenize, and pad to 60 bytes. */
	pktSize += (pktSize & 0x00000001);

	if ( pktSize < MIN_PKT_SIZE )
		pktSize = MIN_PKT_SIZE;

	return( pktSize );			

}	/* End CMediaSendRaw8023 */

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32	CMediaSend8022Over8023(
*							SHARED_DATA			*sharedData,
*							ECB					*ecb,
*							TCB					*tcb);
*
*	Description:
*		This function constructs an 802.2 media header. It will fill
*		in all the TCB fields.
*
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
*
*	Values returned:
*		Packet Size(Padded).  PktSize
*
*	Implied parameters:
*
*  Notes:
*
*  Origin:	CEtherTsmSend8022 in NT's ethertsm.c
*
************************************************************************/
UINT32 CMediaSend8022Over8023(
				SHARED_DATA	*sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	ETHER_MEDIA_HEADER	*machdr;
	UINT32					pktSize;
	UINT16					lengthField, extrabytes=0;
	UINT8						*pprotocolID, *pheader, loopcount, i;

	/* Bag Compiler Warnings. */
   sharedData = sharedData;

	/* set up fragment structure. */
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK*)(&ecb->ECB_FragmentCount);
	
	/* Add MAC header length If not raw send. */
	pktSize = ecb->ECB_DataLength;
	lengthField = (UINT16)(pktSize + (SIZE_E8022 - SIZE_E8023));

	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		tcb->TCB_MediaHeaderLen = SIZE_E8022;
		pktSize += SIZE_E8022;

		/* fill in MAC header. */
		machdr = &(tcb->TCB_MediaHeader);
		COPY_ADDR( &machdr->MH_Destination, &ecb->ECB_ImmediateAddress );

		/*If PID[0] = 0  then PID[5] = DSAP.*/
		/*if PID[0] = 1  then PID[4] = DSAP, SSAP */
		/*if PID[0] = 2  then PID[3] = DSAP, SSAP, CTL0.*/
		/*if PID[0] = 3  then PID[2] = DSAP, SSAP, CTRL0, CTRL1 */
		/*if PID[0] = 4  then PID[1] = DSAP, SSAP, CTRL0, CTRL1, First byte of Data. */
		if( ecb->ECB_ProtocolID.protocolID[0] == 0 )
		 {
			/* Nominal LLC Header */
			machdr->MH_DSAP  = ecb->ECB_ProtocolID.protocolID[5];
			machdr->MH_SSAP  = ecb->ECB_ProtocolID.protocolID[5];
			machdr->MH_Ctrl0 = 03;
		 }
		else
		 {
			/* explicit Header Given */
			extrabytes = (UINT16)(ecb->ECB_ProtocolID.protocolID[0]) - 2;
			lengthField += extrabytes;
			pktSize += extrabytes;
			tcb->TCB_MediaHeaderLen += extrabytes;

			loopcount = extrabytes + 3;
			pprotocolID = &(ecb->ECB_ProtocolID.protocolID[6 - loopcount]);
			pheader = &(machdr->MH_DSAP);

			/* Loop Count is 1 based. */
			for( i=0; i < loopcount; ++i )
				pheader[i] = pprotocolID[i];

		 }	/*  End Explicit Header */

		machdr->MH_Length = VALUE_TO_HILO_UINT16( lengthField );

	 }/* End Non-Raw Send Packet */

	/* Fill in DataLen and Board Number */
	tcb->TCB_DataLen = pktSize;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;

   /* Pad to 60 bytes.*/
	if ( pktSize < MIN_PKT_SIZE )
		pktSize = MIN_PKT_SIZE;

	return( pktSize );
}	/* End CMediaSend8022Over8023 */

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32 CMediaSend8022Snap(
*				SHARED_DATA	*sharedData,
*				ECB 			*ecb,
*				TCB 			*tcb)
*
*	Description:
*		This function constructs a SNAP media header. It will fill
*		in all the TCB fields.
*
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
*
*	Values returned:
*		Packet Size(Padded).
*
*  Notes:
*
************************************************************************/
UINT32 CMediaSend8022Snap(
				SHARED_DATA	*sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	UINT32	pktSize;
	UINT16	lengthField;

	/* Bag Annoying Compiler Warning */
   sharedData = sharedData;

	/*set up fragment structure.*/
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
	
	/*Get size of data packet.*/
	pktSize = ecb->ECB_DataLength;
	lengthField = (UINT16)(pktSize + (SIZE_ESNAP - SIZE_E8023));

	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		pktSize += SIZE_ESNAP;

		/*fill in MAC header.*/
		COPY_ADDR( &tcb->TCB_MediaHeader.EtherMedia.MH_Destination,
			        &ecb->ECB_ImmediateAddress );
		tcb->TCB_MediaHeader.EtherMedia.MH_Length = VALUE_TO_HILO_UINT16(lengthField);

		/* Set these Values a Byte at a time to maintain Portability */
		tcb->TCB_MediaHeader.EtherMedia.MH_DSAP  = 0xAA;
		tcb->TCB_MediaHeader.EtherMedia.MH_SSAP  = 0xAA;
		tcb->TCB_MediaHeader.EtherMedia.MH_Ctrl0 = 0x03;

		tcb->TCB_MediaHeader.EtherMedia.MH_SNAP[0] =  ecb->ECB_ProtocolID.protocolID[1];
		COPY_UINT32( &(tcb->TCB_MediaHeader.EtherMedia.MH_SNAP[1]),
						 &(ecb->ECB_ProtocolID.protocolID[2]));
		
		tcb->TCB_MediaHeaderLen = SIZE_ESNAP;
	}

	/* Fill in DataLen and Board Number */
	tcb->TCB_DataLen = pktSize;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;

   /* Pad to 60 bytes.*/
	if( pktSize < MIN_PKT_SIZE )
		pktSize = MIN_PKT_SIZE;

	return( pktSize );
}	/* End CMediaSend8022Snap */

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32	CMediaSendEthernetII(
*				SHARED_DATA *sharedData,
*				ECB					*ecb,
*				TCB					*tcb);
*
*	Description:
*		This function constructs an Ethernet II media header. It will fill
*		in all the TCB fields.
*
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
*
*	Values returned:
*		Packet Size(Padded and evenized).
*
*	Implied parameters:
*
*  Notes:
*
*  Origin:	CEtherTsmSendEII in NT's ethertsm.c
*
************************************************************************/
UINT32 CMediaSendEthernetII(
				SHARED_DATA *sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	UINT32	pktSize;

   sharedData = sharedData;

	/*set up fragment structure.*/
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
	
	/* Add MAC header length If not raw send. */
	pktSize = ecb->ECB_DataLength;

	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		pktSize += SIZE_EII;

		/*fill in MAC header.*/
		COPY_ADDR( &tcb->TCB_MediaHeader.EtherMedia.MH_Destination,
			        &ecb->ECB_ImmediateAddress );
		COPY_UINT16( &tcb->TCB_MediaHeader.EtherMedia.MH_Length,
						 &(ecb->ECB_ProtocolID.protocolID[4]) );
			
		tcb->TCB_MediaHeaderLen = SIZE_EII;
	 }	/* End Non-RAW_SEND packet */
	
	/* Fill in DataLen and Board Number */
	tcb->TCB_DataLen = pktSize;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;

   /* evenize, and pad to 60 bytes.*/
	pktSize += ( pktSize & 0x00000001 );
	if( pktSize < MIN_PKT_SIZE )
		pktSize = MIN_PKT_SIZE;

	return( pktSize );
}	/* End CMediaSendEthernetII */

/***********************************************************************\
*                                                                       *
*	Name:
*		TCB	*CEtherTSMGetNextSend(
*							DRIVER_DATA			*driverData,
*							CONFIG_TABLE		**configTable,
*							UINT32				*pktSize,
*							void					**physicalEcb);
*
*	Description:
*		This function returns a TCB if the TSM has one queued up.
*
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*		configTable - 			Pointer to configuration table
*
*	Parameters out:
*		pktSize -				Pointer to hold packet size(padded/evenized)
*		physicalEcb -			Pointer to hold physical ECB
*
*	Values returned:
*		Pointer to TCB if we had one queued
*		Otherwise NULL
*
*	Implied parameters:
*
*  Notes:
*
*  Origin:	CEtherTsmGetNextSend in NT's ethertsm.c
*
************************************************************************/
TCB	*CEtherTSMGetNextSend(
							DRIVER_DATA			*driverData,
							CONFIG_TABLE		**configTable,
							UINT32				*pktSize,
							void					**physicalEcb)
{
	FRAME_DATA					*frameData;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	TCB							*tcb = NULL;
	ECB							*ecb;
	UINT32						status = ODISTAT_ITEM_NOT_PRESENT;
	UINT32						oldState;	

	sharedData = DADSP_TO_CMSMADSP(driverData);

	/* Protect Our Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	/* Check if ECB on the QUEUE. */
	if( (ecb = sharedData->CMSMSendListHead) != NULL )
	 {
		if( sharedData->CMSMTxFreeCount > 0 )
		 {
			status = ODISTAT_SUCCESSFUL;

			/* Get ECB off of queue.  Check if more ECBs on queue. */
			--(sharedData->CMSMTxFreeCount);
			sharedData->CMSMSendListHead = ecb->ECB_NextLink;
			frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;
			*configTable = &frameData->ConfigTable;

			if( sharedData->CMSMSendListHead == NULL )
			 {
				/*No more ECBs on the queue. Set up queue tail.*/
				sharedData->CMSMSendListTail = (ECB *)&sharedData->CMSMSendListHead;
				sharedData->CMSMStatusFlags &= ~TXQUEUED;
			 }

			/* Check if we need to get physical addresses for DMA cards */
			if (sharedData->CMSMNeedContiguousECB)
				ecb = CMSMGetPhysicalECB( driverData, ecb );
			
			if(sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0)
			 {
				mediaData = sharedData->CMSMMediaDataPtr;
				*physicalEcb = NULL;

				/* Get a TCB and call Media Send routine to fill it out. */
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];
				*pktSize = frameData->SendRoutine( sharedData, ecb, tcb );

				/* if the adapter needs physical frags then set the DriverWs + 8 */
				/* to point to the Media Header */
				if( sharedData->CMSMNeedContiguousECB )
				 {
					tcb->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
					tcb->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&tcb->TCB_MediaHeader);
				 }
			 }	/* Driver Wants TCBs */
			else
			 {
				/* Save the Frame Data Space for latter. */
				ecb->ECB_PreviousLink = (ECB *)frameData;
				*physicalEcb = CMSMGetPhysical(ecb);
				tcb = (TCB *)ecb;
			 }  /* Driver Wants ECBs */
		 }	/* TxFreeCount Greater than 0 */
	 } /* End ECB on Q */

	CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState);
	return( tcb );

}	/* End CEtherTSMGetNextSend */

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMFastSendComplete(
*							DRIVER_DATA			*driverData,
*                    TCB               *tcb,
*                    UINT32            transmitStatus);
*
*	Description:
*		This function returns a TCB to the TSM. The TSM will extract the
*		ECB from the TCB, return the TCB to the TCB pool, and return
*		the ECB to the LSL. The LSL will give the ECB back to its owner
*		by calling its ESR routine.
*
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*		tcb - 					Pointer to Transmit Control block.
*     transmitStatus -     Flag used to indicate if packet was acually sent.
*                          non-zero means packet undeliverable.
*
*	Values returned:
*
*	Implied parameters:
*
*  Notes:
*
************************************************************************/
void	CEtherTSMFastSendComplete(
										DRIVER_DATA			*driverData,
										TCB					*tcb,
										UINT32				transmitStatus)
{
	FRAME_DATA						*frameData;
	SHARED_DATA						*sharedData;
	MEDIA_DATA						*mediaData;
	GENERIC_STATS					*statsTable;
	ECB								*ecb, *nullEcb = NULL;
	UINT32							count, oldState, i;	

	/* Setup Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	statsTable = sharedData->CMSMGenericStatsPtr;
	--(statsTable->Gen_Qdepth);

	/* Is HSM ECB aware? */
 	if(sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0)
	 {
		/* HSM is not ECB aware. */
		/* Set count to hold the Total Bytes transmitted. */
		count = tcb->TCB_DataLen;

		/* Get ptr to ECB. */
		ecb = (ECB *)((UINT32)(tcb->TCB_FragBlockPtr) -
									(UINT32)(&(nullEcb->ECB_FragmentCount)));

		/* Get Frame Data Pointer */
		frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;
		
		/*If we are a DMA card then we need to return the ECB to the pool*/
		if ( sharedData->CMSMNeedContiguousECB )
		{
			ecb = CMSMReturnPhysicalECB( driverData, ecb );

			/* Adjust TCB to use logical fragment list	*/
			tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
		}
		
		/*call The Transmit monitor*/
		if ( frameData->TransmitRoutine )
			frameData->TransmitRoutine( tcb );

		/*Return TCB and Link into TCB list.*/
		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		mediaData->TCBHead = tcb;

	 } /* End HSM wants TCBs */
	else
	 {
		/* HSM is ECB aware. CurTCB is really an ECB. */
		ecb = (ECB *)tcb;

		/* If we are a DMA card then we need to return the ECB to the pool */
		if (sharedData->CMSMNeedContiguousECB)
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

		/* Find correct FrameData */
		frameData = NULL;
		for (i = 0; i < 4; i++)
			if( (sharedData->CMSMVirtualBoardLink[i])->MLIDCFG_BoardNumber ==
					ecb->ECB_BoardNumber)
				{
					frameData = CONFIG_TO_MSM(sharedData->CMSMVirtualBoardLink[i]);
					break;
				}

		/*call The Transmit monitor*/
		if (frameData)
			if (frameData->TransmitRoutine)
			 {
				/* Get a TCB and call Media Send routine to fill it out. */
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];
				frameData->SendRoutine( sharedData, ecb, tcb );
				frameData->TransmitRoutine( tcb );

				/* Return TCB and Link into TCB list. */
				tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
				mediaData->TCBHead = tcb;

			 }	/* End Call the TxMonitor */

		count = ecb->ECB_DataLength;

		/******************************************************************
			Check if Not raw send Add in Header bytes.--
			Raw Send Priority StackID values increase with lessor priority
		*******************************************************************/
		if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) < NON_STACKID_RAW_SEND_MASK )
		 {
			count += frameData->MediaHeaderSize;
			if( ecb->ECB_ProtocolID.protocolID[0] == 0x03 )
				++count;
		 }
	 }/* HSM Wants ECBs */

	/*Set ECB status and update Tx statistics.*/
	if (transmitStatus)
		ecb->ECB_Status = ODISTAT_CANCELED;
	else
	{
		++(statsTable->Gen_TotalTxPackets);

		statsTable->Gen_TotalTxOkByteLow += count;

		/* Check for overflow in the lower 32 bits. */
   	if (statsTable->Gen_TotalTxOkByteLow < count)
			statsTable->Gen_TotalTxOkByteHigh++;

		statsTable->Gen_TotalGroupAddrTx +=
									 (ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x01);

		ecb->ECB_Status = ODISTAT_SUCCESSFUL;
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
  	CLSL_FastSendComplete(ecb);
	return;

}	/* End CEtherTSMFastSendComplete */

/***********************************************************************\
*                                                                       *
*	Name:
*		void	*CEtherTSMSendComplete(
*							DRIVER_DATA			*driverData,
*							TCB					*tcb,
                     UINT32            transmitStatus);
*
*	Description:
*		This function returns a TCB to the TSM. The TSM will extract the
*		ECB from the TCB, return the TCB to the TCB pool, and return
*		the ECB to the LSL. The LSL will queue up the ECB until
* 		LSLServiceEvents is called. The HSM will have to initiate this
*		by calling MSMServiceEvents when it is safe for the LSL to give
*		the queued ECBs back to their owners by calling their ESR routines.
*
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*		tcb - 					Pointer to Transmit Control block.
*     transmitStatus -     Flag used to indicate if packet was acutally sent.
*                          Non-zero means packet undeliverable.
*
*	Values returned:
*
*	Implied parameters:
*
*  Notes:
*
************************************************************************/
void	CEtherTSMSendComplete(
							DRIVER_DATA			*driverData,
							TCB					*tcb,
							UINT32				transmitStatus)
{
	FRAME_DATA			*frameData;
	SHARED_DATA			*sharedData;
	MEDIA_DATA			*mediaData;
	GENERIC_STATS		*statsTable;
	ECB					*ecb, *nullEcb = NULL;
	UINT32				count, oldState, i;

	/* Setup the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject, &oldState );

	statsTable = sharedData->CMSMGenericStatsPtr;
	--(statsTable->Gen_Qdepth);

	/* Is HSM ECB aware? */
	if( sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0 )
	 {
		/* HSM is not ECB aware. */
		/* Set count to hold the Total Bytes transmitted. */
		count = tcb->TCB_DataLen;

		/*Get ptr to ECB.*/
		ecb = (ECB *)((UINT32)(tcb->TCB_FragBlockPtr) -
									(UINT32)(&(nullEcb->ECB_FragmentCount)));

		/* Get Frame Data Pointer */
		frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;

		/*If we are a DMA card then we need to return the ECB to the pool*/
		if (sharedData->CMSMNeedContiguousECB)
		{
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

			/* Adjust TCB to use logical fragment list */
			tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
		}

		/*call The Transmit monitor*/
		if( frameData->TransmitRoutine )
			frameData->TransmitRoutine( tcb );

		/*Return TCB and Link into TCB list.*/
		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		mediaData->TCBHead = tcb;
	 }	/* End HSM wanted TCBs */
	else
	 {
		/*HSM is ECB aware. CurTCB is really an ECB.*/
		ecb = (ECB *)tcb;

		/*If we are a DMA card then we need to return the ECB to the pool*/
		if( sharedData->CMSMNeedContiguousECB )
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

		/* Find correct FrameData */
		frameData = NULL;
		for (i = 0; i < 4; i++)
			if( (sharedData->CMSMVirtualBoardLink[i])->MLIDCFG_BoardNumber ==
					ecb->ECB_BoardNumber )
			 {
				frameData = CONFIG_TO_MSM(sharedData->CMSMVirtualBoardLink[i]);
				break;
			 }

		/* call The Transmit monitor */
		if (frameData)
			if( frameData->TransmitRoutine )
			 {
				/*Get a TCB and call Media Send routine to fill it out.*/
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];
				frameData->SendRoutine( sharedData, ecb, tcb );
				frameData->TransmitRoutine( tcb );

				/*Return TCB and Link into TCB list.*/
				tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
				mediaData->TCBHead = tcb;
			 }	/* End call the TxMonitor */

		count = ecb->ECB_DataLength;

		/******************************************************************
			Check if Not raw send.--
			Raw Send Priority StackID values increase with lessor priority
		*******************************************************************/
		if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) < NON_STACKID_RAW_SEND_MASK )
		 {
			count += frameData->MediaHeaderSize;
			if(ecb->ECB_ProtocolID.protocolID[0] == 0x03)
				++count;
		 }
	 }

	if (transmitStatus)
		ecb->ECB_Status = ODISTAT_CANCELED;
	else
	{
		++(statsTable->Gen_TotalTxPackets);

		statsTable->Gen_TotalTxOkByteLow += count;

		/* Check for overflow in the lower 32 bits. */
   	if (statsTable->Gen_TotalTxOkByteLow < count)
			statsTable->Gen_TotalTxOkByteHigh++;

		statsTable->Gen_TotalGroupAddrTx +=
									 (ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x01);

		ecb->ECB_Status = ODISTAT_SUCCESSFUL;
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
  	CLSL_SendComplete( ecb );
	return;

}	/* End CEtherTSMSendComplete */

/***********************************************************************\
*                                                                       *
*	Name:
*		RCB	*CEtherTSMFastProcessGetRCB(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						newRcbSize);
*
*	Description:
*		This routine is called by HSM's that have completely filled
*		in a pre-allocated RCB and is ready to be passed the LSL/
*		protocol stack. If this function can pass back a new RCB
*		to use, it will. This call will give the packet to the protocol
*		stack if one wants it and could take a while to return.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Receive Control Block
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		newRcbSize -		Size of new RCB to be returned
*
*	Values returned:
*		New Receive Control Block for HSM to reuse if we can get one,
*		otherwise NULL.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
RCB *CEtherTSMFastProcessGetRCB(
							DRIVER_DATA		*driverData,
							RCB				*rcb,
							UINT32 			pktSize,
							UINT32			rcvStatus,
							UINT32			newRcbSize)
{
	RCB * retRCB;

	retRCB = CEtherTSMProcessGetRCB(
					driverData,
					rcb,
					pktSize,
					rcvStatus,
					newRcbSize);
#ifdef OS_NIOS
	CLSL_ServiceEvents();
#endif

	return( retRCB );

}	/*End CEtherTSMFastProcessGetRCB*/

/***********************************************************************\
*                                                                       *
*	Name:
*		RCB	*CEtherTSMProcessGetRCB(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						newRcbSize);
*
*	Description:
*		This routine is called by HSM's that have completely filled
*		in a pre-allocated RCB and is ready to be passed the LSL/
*		protocol stack. If this function can pass back a new RCB
*		to use, it will.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Receive Control Block
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		newRcbSize -		Size of new RCB to be returned
*
*	Values returned:
*		New Receive Control Block for HSM to reuse if we can get one,
*		otherwise NULL.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*                                                                       *
************************************************************************/
RCB *CEtherTSMProcessGetRCB(
				DRIVER_DATA				*driverData,
				RCB			 			*rcb,
				UINT32					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize)
{
	FRAME_DATA					*frameData = NULL;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statsTable;
	ETHER_MEDIA_HEADER		*lookAheadData;
	void							*physicalAddress;
	UINT32						receptiontype = 0;
	LOOKAHEAD					lkAheadStruc;
	UINT16						lengthField;

	/* Initialize Local Variables */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	statsTable = sharedData->CMSMGenericStatsPtr;

	lookAheadData = (ETHER_MEDIA_HEADER *)(rcb + 1);

     /* Set the value to the number of bytes to skip to the begining
        of the DA field. In some busmaster implementations,
        this value (in protocol work space) can be as high as 128 bytes
        for chips which have poor alignment capabilities.*/
  lookAheadData = (ETHER_MEDIA_HEADER *)((UINT32)lookAheadData +
						(UINT32)((ECB *)rcb)->ECB_ProtocolWorkspace.PWs_i32val[0]);
  ((ECB *)rcb)->ECB_ProtocolWorkspace.PWs_i32val[0] = 0;

	/***************************************************************/
	/* Determine the frame type of the packet:                     */
	/*                                                             */
	/*	EII:	DA, SA, PROTCL, D A T A				(PROTCL > MAXLENGTH) */
	/*	802.3:	DA, SA, LENGTH, 0xFF 0xFF D A T A                  */
	/*	802.2:	DA, SA, LENGTH, DSAP, SSAP, 0x03, D A T A          */
	/*	SNAP:	DA, SA, LENGTH, 0xAA, 0xAA, 0x03, PID, D A T A        */
	/***************************************************************/
		lengthField = GET_HILO_UINT16( &lookAheadData->MH_Length );

		if( lengthField <= (MAX_PACKET_LENGTH - SIZE_E8023) )
		 {
			/* Check for Malformed. Non E_II type frame */
			if ( lengthField > pktSize - SIZE_E8023 )
		    {
		      /* looks like a malformed packet */
	   	   rcvStatus |= PAE_MALFORMED_BIT;
				statsTable->Gen_RxMiscError++;
		    }

			if(*(UINT16 *)(&(lookAheadData->MH_DSAP)) != 0xFFFF)
			 {
				if( ((*(UINT16 *)&lookAheadData->MH_DSAP) == 0xAAAA) &&
					 (lookAheadData->MH_Ctrl0 == 0x03) )
				 {
					/* type is SNAP */

					/* Pass Pad Bytes up if in PromiscousMode. */

					if( mediaData->PromiscuousMode )
						lengthField = pktSize - SIZE_ESNAP;

					else
						lengthField -= 8;

					/* Get frame data space and copy 5 bytes following the */
					/* DSAP, SSAP and Ctl0 to PID space. */

					if(configTable = sharedData->CMSMVirtualBoardLink[ESNAP_INTERNALID])
					 {
						if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
						{
							frameData = CONFIG_TO_MSM(configTable);
							lkAheadStruc.LkAhd_ProtocolID.protocolID[0] = 0;
							lkAheadStruc.LkAhd_ProtocolID.protocolID[1] = lookAheadData->MH_SNAP[0];
							COPY_UINT32( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[2]),
															 &(lookAheadData->MH_SNAP[1]) );
						}

					 } /* End of frame loaded and not shutdown */
				 }	/* Type is Snap */
				else
				 {
					/* type is 802.2 */

					/* Pass Pad Bytes up if in PromiscousMode. */

					if( mediaData->PromiscuousMode )
						lengthField = pktSize - SIZE_E8022;

					else
						lengthField -= 3;


					/*Get frame data space and copy DSAP field to PID.*/

					if(configTable = sharedData->CMSMVirtualBoardLink[E8022_INTERNALID])
					{
						if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
						{
							*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
							lkAheadStruc.LkAhd_ProtocolID.protocolID[4] = 0;
							lkAheadStruc.LkAhd_ProtocolID.protocolID[5] = lookAheadData->MH_DSAP;
							frameData = CONFIG_TO_MSM(configTable);
							frameData->MediaHeaderSize = SIZE_E8022;		/*default to type 1*/

							if(!(lookAheadData->MH_Ctrl0 & S_OR_U_FORMAT) || !(lookAheadData->MH_Ctrl0 & U_FORMAT))
							{
								++(frameData->MediaHeaderSize);				/*Change to type 2*/
								--lengthField;
								receptiontype |= DT_8022_TYPE_II;
							}

							else
								receptiontype |= DT_8022_TYPE_I;

						}
					}/* End of frame loaded and not shutdown */
				 }	/* type is 802.2 */
			 }	/* Packet isn't a 802.3 type */
			else
			 {
				/*type is 802.3*/
				if( mediaData->PromiscuousMode )
					lengthField = pktSize - SIZE_E8023;

				/*Get frame data space.*/
				if (configTable = sharedData->CMSMVirtualBoardLink[E8023_INTERNALID])
				{
					if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
					{
						frameData = CONFIG_TO_MSM(configTable);
						*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
						*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
					}
				} /* End of frame loaded and not shutdown */

			 } /* Packet is 802.3 type */
		 } /* Packet has 802.3 header */
		else
		 {
			/*type is EII*/

			lengthField = pktSize - SIZE_EII;

			/*Get frame data space and copy type field into PID.*/

			if(configTable = sharedData->CMSMVirtualBoardLink[EII_INTERNALID])
			 {
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
					frameData = CONFIG_TO_MSM(configTable);
					*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
					COPY_UINT16( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]),
								 &lookAheadData->MH_Length );
				}
			 } /* End of frame loaded and not shutdown */
		 } /* Type is EII */

		if(frameData == NULL)
		 {
			configTable = sharedData->CMSMDefaultVirtualBoard;
			frameData = CONFIG_TO_MSM(configTable);
			rcvStatus |= PAE_NOT_ENABLED_BIT;
			statsTable->Gen_RxMiscError++;

			/* zero out Protocol ID field */

			*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
			*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
		 }

		/*We have determined the frame type.*/
		/*Make sure the length is not negative*/
		if( lengthField > pktSize )
		 {  	
			/*Looks like a malformed packet */
			lengthField = 3;
			rcvStatus |= PAE_MALFORMED_BIT;
			statsTable->Gen_RxMiscError++;
		 }

		/****************************************/
		/* Check the destination address.		 */
		/* Check if multicast bit set.			 */
		/****************************************/
		if( lookAheadData->MH_Destination.nodeAddress[0] & MULTICASTBIT )
		 {
			statsTable->Gen_TotalGroupAddrRx++;

			/*Check if BroadCast*/
			if (ADDR_EQUAL(&lookAheadData->MH_Destination, &BroadCast))
	        receptiontype |= (ECB_BROADCAST | ECB_MULTICAST);
			else
			 {
				/* We Don't need to filter...
					if Not in PromiscuousMode AND HardwareFiltering is on.(Both Bits.) */
				if ( (!mediaData->PromiscuousMode) &&
					  (configTable->MLIDCFG_Flags & (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)) ==
					  (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)
					)
				 {
					receptiontype |= ECB_MULTICAST;
				 }
				else
				 {
					/*Check if multicast is in our table.*/
					if( FindAddressInMCTable(sharedData, (NODE_ADDR *)lookAheadData->MH_Destination.nodeAddress) )
						receptiontype |= ECB_MULTICAST;
					else
					 {
						receptiontype |= ECB_MULTICASTREMOTE;

						/* If not in Promiscous Mode filter out the Packet */
 						if( !mediaData->PromiscuousMode )
							return( rcb );
					 } /* Remote Multicast, Drop if not in Promiscuous Mode. */
				 }	/* Need to check if in Table */
			 } /* Packet is a Multicast--Not Broadcast */
		 }	/* Packet has a Group Address */
		else
		 {
			if( mediaData->PromiscuousMode )
			 {
				/* Promiscous Mode Check Dest Address */
				if (ADDR_EQUAL(&lookAheadData->MH_Destination,
									&configTable->MLIDCFG_NodeAddress) )
					receptiontype |= ECB_UNICASTDIRECT;
				else
					receptiontype |= ECB_UNICASTREMOTE;
			 }
			else
				receptiontype |= ECB_UNICASTDIRECT;
		 }	/* Packet has a Unicast Address */

	  	/* Make sure that packet is not too small. */
   	if( pktSize < MIN_PKT_SIZE )
		 {
			rcvStatus |= PAE_RUNT_PACKET_BIT;
			if (pktSize < MIN_RUNT_SIZE)
				rcvStatus |= PAE_NOT_ENABLED_BIT;

			++(statsTable->Gen_RxTooSmall);
		 }
	
	   /* Make sure that packet is not to large if it is not a pipelined adapter. */
		if( (pktSize > MAX_PACKET_LENGTH) && (pktSize != UNUSED) )
		 {
	   	rcvStatus |= PAE_TOO_BIG_BIT;	
			statsTable->Gen_RxTooBig++;
		 }

		/* Watch for an Error Packet */
		if( (UINT8)rcvStatus )
			receptiontype = ECB_GLOBALERROR;		/* This bit is exclusive */
		else
			/* Bump counters only for good packets. */
		++(statsTable->Gen_TotalRxPackets);

		/*fill in the lookahead structure.*/
		lkAheadStruc.LkAhd_MediaHeaderLen = frameData->MediaHeaderSize;
		lkAheadStruc.LkAhd_MediaHeaderPtr = (UINT8 *)lookAheadData;
		lkAheadStruc.LkAhd_DataLookAheadPtr	= (UINT8 *)(lookAheadData) + frameData->MediaHeaderSize;
		((ECB *)rcb)->ECB_Fragment[0].FragmentAddress	= lkAheadStruc.LkAhd_DataLookAheadPtr;
		lkAheadStruc.LkAhd_DataLookAheadLen 				= lengthField;
		lkAheadStruc.LkAhd_FrameDataSize 					= lengthField;
		((ECB *)rcb)->ECB_DataLength 							= lengthField;
		((ECB *)rcb)->ECB_DriverWorkspace.DWs_i16val[1] = lengthField;
		((ECB *)rcb)->ECB_Fragment[0].FragmentLength = lengthField;
		lkAheadStruc.LkAhd_BoardNumber = configTable->MLIDCFG_BoardNumber;
		((ECB *)rcb)->ECB_BoardNumber = configTable->MLIDCFG_BoardNumber;
		COPY_ADDR( &((ECB *)rcb)->ECB_ProtocolID,
			        &(lkAheadStruc.LkAhd_ProtocolID) );
		lkAheadStruc.LkAhd_PadAlignBytes1 = 0;
		lkAheadStruc.LkAhd_PadAlignBytes2 = 0;
		lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
		lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
		lkAheadStruc.LkAhd_PriorityLevel = 0;
		lkAheadStruc.LkAhd_Reserved = NULL;

		COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress, &lookAheadData->MH_Source );
		COPY_ADDR( &((ECB *)rcb)->ECB_ImmediateAddress, &lookAheadData->MH_Source );
		lkAheadStruc.LkAhd_PktAttr = rcvStatus;
		((ECB *)rcb)->ECB_PreviousLink = (ECB *)rcvStatus;
		lkAheadStruc.LkAhd_DestType = receptiontype;
		((ECB *)rcb)->ECB_DriverWorkspace.DWs_i16val[0]
										= VALUE_TO_LOHI_UINT16((UINT16)receptiontype);
		lkAheadStruc.LkAhd_PreFilledECB = (ECB *)rcb;
	
		statsTable->Gen_TotalRxOkByteLow += lengthField + lkAheadStruc.LkAhd_MediaHeaderLen;
		/* statsTable->Gen_TotalRxOkByteLow += lkAheadStruc.LkAhd_MediaHeaderLen; */

	/* Check for overflow in the lower 32 bits. */
	if (statsTable->Gen_TotalRxOkByteLow < (lengthField + lkAheadStruc.LkAhd_MediaHeaderLen))
			statsTable->Gen_TotalRxOkByteHigh++;

	/*See if the packet was not Wanted.*/
	if( CLSL_GetStackECB( &lkAheadStruc ) == ODISTAT_OUT_OF_RESOURCES )
		statsTable->Gen_NoECBs++;	/* no one wanted it, count it */

	/* Get a new RCB to return to the HSM (The LSL always keeps it) */
	rcb = CMSMAllocateRCB(driverData, newRcbSize, &physicalAddress);
	return( rcb );
}	/*End CEtherTSMProcessGetRCB*/

/***********************************************************************\
*                                                                       *
*	Name:
*		RCB	*CEtherTSMGetRCB(
*					DRIVER_DATA					*driverData,
*					UINT8                   *packetHdr,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						*startByte,
*					UINT32						*numBytes);
*
*	Description:
*		This routine is called by HSM's that have read in at least the
*		look ahead portion of the received packet. If a protocol stack
*		wants the packet, an RCB is returned with a fragment list for
*		which points to the buffers that the HSM should place the rest
*		of the packet.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		packetHdr -	      first look ahead bytes of the packet
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		startByte -			Where TSM will store bytes how many bytes
*								from the beginning of the packet that the
*								HSM should skip before copying the remainder
*								of the packet.
*		numBytes -			Remaining number of bytes for HSM to copy
*								into fragment list.
*
*	Values returned:							
*		RCB with fragment list to fill in if accepted
*		NULL if no protocol stack wanted it. HSM should discard packet.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
RCB *CEtherTSMGetRCB(
              DRIVER_DATA       *driverData,
              UINT8             *packetHdr,
              UINT32            pktSize,
              UINT32            rcvStatus,
              UINT32            *startByte,
              UINT32            *numBytes)
{
	FRAME_DATA				*frameData = NULL;
	SHARED_DATA				*sharedData;
	MEDIA_DATA				*mediaData;
	CONFIG_TABLE			*configTable;
	GENERIC_STATS			*statsTable;
	ECB						*ecb;
	ETHER_MEDIA_HEADER	*lookAheadData;
	FRAGMENT_STRUCT		*fragptr;
	UINT32					receptiontype = 0;
	UINT32					fragCount;
	UINT32					sumOfFragBytes;
	LOOKAHEAD				lkAheadStruc;
	UINT16					lengthField;

		/* Initialize the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	statsTable = sharedData->CMSMGenericStatsPtr;
	lookAheadData = (ETHER_MEDIA_HEADER *)packetHdr;

	/***************************************************************/
	/*Determine the frame type of the packet.								*/
	/*	EII:	DA, SA, PROTCL, D A T A				(PROTCL > MAXLENGTH) */
	/*	802.3:	DA, SA, LENGTH, 0xFF 0xFF D A T A 						*/
	/*	802.2:	DA, SA, LENGTH, DSAP, SSAP, 0x03, D A T A				*/
	/*	SNAP:	DA, SA, LENGTH, 0xAA, 0xAA, 0x03, PID, D A T A			*/
	/***************************************************************/
	lengthField = GET_HILO_UINT16( &lookAheadData->MH_Length );

	if( lengthField <= (MAX_PACKET_LENGTH - SIZE_E8023) )
	 {
		if ( lengthField > pktSize - SIZE_E8023 )
	    {
	      /* looks like a malformed packet */
	      rcvStatus |= PAE_MALFORMED_BIT;
			statsTable->Gen_RxMiscError++;
	    }

		if( *(UINT16 *)(&(lookAheadData->MH_DSAP)) == 0xFFFF )
		 {
			/* Type is 802.3 */

			if( mediaData->PromiscuousMode )
			 {
				if( pktSize != UNUSED )
					lengthField = pktSize - SIZE_E8023;
			 }	/* In Promiscous Mode Pass up Pad bytes. */

			/* Get frame data space. */
			if (configTable = sharedData->CMSMVirtualBoardLink[E8023_INTERNALID])
			{
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
					frameData = CONFIG_TO_MSM(configTable);
					*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
					*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
				}
			}/* End of frame loaded and not shutdown */

		 } /* Type is 802.3 */
		else
		 {
			if( ((*(UINT16 *)&lookAheadData->MH_DSAP) == 0xAAAA) &&
				 (lookAheadData->MH_Ctrl0 == 0x3) )
			 {
				/*type is SNAP*/
				if( mediaData->PromiscuousMode )
				 {
					if( pktSize != UNUSED )
						lengthField = pktSize - SIZE_ESNAP;
					else
						lengthField -= 8;  /* 3 (802.2 header) + 5 (snap) */
				 }
				else
					lengthField -= 8;  /* 3 (802.2 header) + 5 (snap) */

				/*Get frame data space and copy 5 bytes following*/
				/*DSAP, SSAP and Ctl0 to PID.*/

				if(configTable = sharedData->CMSMVirtualBoardLink[ESNAP_INTERNALID])
				 {
				 	if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
					{
						frameData = CONFIG_TO_MSM(configTable);
					 	lkAheadStruc.LkAhd_ProtocolID.protocolID[0] = 0;
						lkAheadStruc.LkAhd_ProtocolID.protocolID[1] = lookAheadData->MH_SNAP[0];
						COPY_UINT32( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[2]),
														 &(lookAheadData->MH_SNAP[1]) );
					}
				 }/* End of frame loaded and not shutdown */
			 } /* End of Type is Snap */
			else
			 {
				/*type is 802.2*/
				if( mediaData->PromiscuousMode )
				 {
					if( pktSize != UNUSED )
						lengthField = pktSize - SIZE_E8022;
					else
						lengthField -= 3; /* 802.2 header */
				 }
				else
						lengthField -= 3; /* 802.2 header */

				/*Get frame data space and copy DSAP field to PID.*/
				if(configTable = sharedData->CMSMVirtualBoardLink[E8022_INTERNALID])
				 {
					if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
					{
						frameData = CONFIG_TO_MSM(configTable);
						*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
						lkAheadStruc.LkAhd_ProtocolID.protocolID[4] = 0;
						lkAheadStruc.LkAhd_ProtocolID.protocolID[5] = lookAheadData->MH_DSAP;
						frameData->MediaHeaderSize = SIZE_E8022;		/*default to type 1*/

						if((!(lookAheadData->MH_Ctrl0 & S_OR_U_FORMAT)) || (!(lookAheadData->MH_Ctrl0 & U_FORMAT)))
						{
							++(frameData->MediaHeaderSize);				/*Change to type 2*/
							--lengthField;
							receptiontype |= DT_8022_TYPE_II;
						}

						else 
							receptiontype |=  DT_8022_TYPE_I;
					}
				 }/* End of frame loaded and not shutdown */
			 }/* Type is 802.2 */
		 } /* Type Not 802.3 only */
	 }	/* has 802.3 header */
	else
	 {
		/*type is EII*/
		/*Get frame data space and copy type field into PID.*/
		if( pktSize != UNUSED )
			lengthField = pktSize - SIZE_EII;
		else
			lengthField = MAX_PACKET_LENGTH - SIZE_EII;

		if( configTable = sharedData->CMSMVirtualBoardLink[EII_INTERNALID] )
		 {
			if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
			{
				frameData = CONFIG_TO_MSM(configTable);
				*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
				COPY_UINT16( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]),
								 &(lookAheadData->MH_Length) );
			}
		 }/* End of frame loaded and not shutdown */
	 }	/* Type is EII */

	if( frameData == NULL )
	 {
	    configTable = sharedData->CMSMDefaultVirtualBoard;
		frameData = CONFIG_TO_MSM( configTable );
      	rcvStatus |= PAE_NOT_ENABLED_BIT;
		statsTable->Gen_RxMiscError++;

		/* zero out Protocol ID field */

		*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
		*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
    }

	/****************************************/
	/* We have determined the frame type.   */
	/* Make sure the length is not negative */
	/****************************************/
	if ( lengthField > pktSize )
    {
      /* looks like a malformed packet */
		lengthField = 3;
      rcvStatus |= PAE_MALFORMED_BIT;
		statsTable->Gen_RxMiscError++;
    }

	/****************************************/
	/* Check the destination address.       */
	/* Check if multicast bit set.          */
	/****************************************/
	if(lookAheadData->MH_Destination.nodeAddress[0] & MULTICASTBIT)
	 {
		++(statsTable->Gen_TotalGroupAddrRx);

		/*Check if BroadCast*/
		if (ADDR_EQUAL(&lookAheadData->MH_Destination, &BroadCast))
			receptiontype |= (ECB_BROADCAST | ECB_MULTICAST);
		else
		 {
			/* We need to filter.
				if Not in PromiscuousMode AND HardwareFiltering is on. (Both Bits ON) */
				if ( (!mediaData->PromiscuousMode) &&
					  (configTable->MLIDCFG_Flags & (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)) ==
					  (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)
					)
				 {
					receptiontype |= ECB_MULTICAST;
				 }
				else
				 {
					/*Check if multicast is in our table.*/
					if (FindAddressInMCTable(sharedData, (NODE_ADDR *)lookAheadData->MH_Destination.nodeAddress))
						receptiontype |= ECB_MULTICAST;
					else
					 {
						receptiontype |= ECB_MULTICASTREMOTE;

						/* If not in Promiscous Mode filter out the Packet */
 						if( !mediaData->PromiscuousMode )
							return( (RCB *)NULL );  /* Null tells the HSM to discard */

					 } /* Remote Multicast, Drop if not in Promiscuous Mode. */
				 }	/* Need to check if in Table */
			 } /* Packet is a Multicast--Not Broadcast */
		 }	/* Packet has a Group Address */
		else
		 {
			if( mediaData->PromiscuousMode )
			 {
				/* Promiscous Mode Check Dest Address */
				if (ADDR_EQUAL(&lookAheadData->MH_Destination,
									&configTable->MLIDCFG_NodeAddress) == 0)
					receptiontype |= ECB_UNICASTREMOTE;
				else
					receptiontype |= ECB_UNICASTDIRECT;
			 }
			else
				receptiontype |= ECB_UNICASTDIRECT;
		 }	/* Packet has a Unicast Address */

	if( pktSize != UNUSED)
 	 {
	  	/*Make sure that packet is not too small. */
   	if ( pktSize < MIN_PKT_SIZE )
		 {
			rcvStatus |= PAE_RUNT_PACKET_BIT;
			if (pktSize < MIN_RUNT_SIZE)
				rcvStatus |= PAE_NOT_ENABLED_BIT;

			++(statsTable->Gen_RxTooSmall);
		 }

	   /* Make sure that packet is not to large if it is not a pipelined adapter. */
		if( pktSize > MAX_PACKET_LENGTH )
		 {
		   rcvStatus |= PAE_TOO_BIG_BIT;	
			++(statsTable->Gen_RxTooBig);
		 }
			/**************************************************/
			/* Only bump counters if not a pipelined adapter. */
			/* And it has no errors yet discovered.			  */
			/**************************************************/
			if( (UINT8)rcvStatus )
				receptiontype = ECB_GLOBALERROR;	/* This bit is exclusive */
			else
			{
				++(statsTable->Gen_TotalRxPackets);

				/*add to count header size*/
				statsTable->Gen_TotalRxOkByteLow += ( lengthField +
									frameData->MediaHeaderSize );

				/* Check for overflow in the lower 32 bits. */
				if ( statsTable->Gen_TotalRxOkByteLow <
					  (lengthField + frameData->MediaHeaderSize) )
					statsTable->Gen_TotalRxOkByteHigh++;
			}

		/* Start Fillin in lookAhead Structure */

		lkAheadStruc.LkAhd_FrameDataSize = lengthField;

	 }	/* Not Pipeline Adapter */
	else
	 {
		if ( (UINT8)rcvStatus )
			receptiontype = ECB_GLOBALERROR;	/* This bit is exclusive */

		/* Start Fillin in lookAhead Structure */
		lkAheadStruc.LkAhd_FrameDataSize = pktSize;
	 }/* PipeLine Adapter */

	/*fill in the lookahead structure.*/
	lkAheadStruc.LkAhd_MediaHeaderLen	= frameData->MediaHeaderSize;
	lkAheadStruc.LkAhd_MediaHeaderPtr	= (UINT8 *)lookAheadData;
	lkAheadStruc.LkAhd_DataLookAheadPtr = (UINT8 *)(lookAheadData) + frameData->MediaHeaderSize;

	if( lengthField > configTable->MLIDCFG_LookAheadSize )
		lkAheadStruc.LkAhd_DataLookAheadLen = configTable->MLIDCFG_LookAheadSize;
	else
		lkAheadStruc.LkAhd_DataLookAheadLen = lengthField;

	COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress, &lookAheadData->MH_Source );
	lkAheadStruc.LkAhd_BoardNumber = configTable->MLIDCFG_BoardNumber;
	mediaData->RxStatus = rcvStatus;
	lkAheadStruc.LkAhd_PktAttr	= rcvStatus;
	lkAheadStruc.LkAhd_DestType = receptiontype;
	lkAheadStruc.LkAhd_PadAlignBytes1 = 0;
	lkAheadStruc.LkAhd_PadAlignBytes2 = 0;
	lkAheadStruc.LkAhd_PreFilledECB = NULL;
	lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
	lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
	lkAheadStruc.LkAhd_PriorityLevel = 0;
	lkAheadStruc.LkAhd_Reserved = NULL;

	/****************************************/
	/* See if anyone wants the packet.      */
	/****************************************/
	if( CLSL_GetStackECB(&lkAheadStruc) == ODISTAT_SUCCESSFUL )
	{
		ecb = lkAheadStruc.LkAhd_ReturnedECB;

		if(ecb) /* Protocol Stack return an ECB */
		{
			ecb->ECB_Status = (UINT16)ODISTAT_SUCCESSFUL;

			/********************************************************/
			/* Calculate the sum of all the fragment buffers sizes. */
			/********************************************************/
			sumOfFragBytes = 0;
			fragCount = ecb->ECB_FragmentCount;
			fragptr = ecb->ECB_Fragment;
			while (fragCount)
			{
				sumOfFragBytes += fragptr->FragmentLength;
				fragptr++;
				fragCount--;
			}

			*startByte = lkAheadStruc.LkAhd_FrameDataStartCopyOffset;
			*startByte += frameData->MediaHeaderSize;
			*numBytes = lkAheadStruc.LkAhd_FrameDataBytesWanted;
			if (pktSize != UNUSED) /* The packet has been fully received. */
			{
				/*******************************************************************/
				/* Check if the number of bytes requested by the protocol stack is */
				/* greater than the actual number of bytes in the frame.           */
				/*******************************************************************/
				if ( *numBytes > (lkAheadStruc.LkAhd_FrameDataSize - *startByte +
										frameData->MediaHeaderSize) )
				*numBytes = lkAheadStruc.LkAhd_FrameDataSize - *startByte +
										frameData->MediaHeaderSize;

				/* Check for possible overflow. */
				if (*numBytes > sumOfFragBytes)
				{
					*numBytes = sumOfFragBytes;
					ecb->ECB_Status = (UINT16)ODISTAT_RX_OVERFLOW;
				}

				/*set Data length field in ECB.*/
				ecb->ECB_DataLength = *numBytes;
				ecb->ECB_DriverWorkspace.DWs_i16val[0]
				        = VALUE_TO_LOHI_UINT16((UINT16)receptiontype);
				ecb->ECB_DriverWorkspace.DWs_i16val[1] = lengthField;
			}	/* End Packet Fully received */

			else /* pipelined */
			{
				ecb->ECB_DataLength = 0;
				/* Check for possible overflow.  If the number of bytes wanted is */
				/* <= the sumOfFragBytes either the protocol stack wants the packet */
				/* and the packet fits perfectly in the fragments or the protocol */
				/* wants only the number of bytes requested even if the frame data */
				/* is greater than the nubmer of bytes wanted.  It is only overflow */
				/* if the number of bytes wanted is greater than the sumOfFragbytes.*/
				if( *numBytes > sumOfFragBytes )
				{
					*numBytes = sumOfFragBytes;

					/* Set ECB_DataLength field bit 30 to protocol wants all data. */
					ecb->ECB_DataLength |= PSTK_WANTS_FRAME;
				}

				/* Save bytes to skip in bits b23-b16 of the ECB DataLength field. */
				ecb->ECB_DataLength |= (((*startByte) << 16) & HSM_TOLD_TO_SKIP);

				/* Save bytes to copy in bits b15-b0 of the ECB DataLength field. */
				ecb->ECB_DataLength |= ((*numBytes) & HSM_TOLD_TO_COPY);
				
				/* Save the lkahd destination type in first part of DriverWorkspace */
				ecb->ECB_DriverWorkspace.DWs_i16val[0]
						  = VALUE_TO_LOHI_UINT16((UINT16)receptiontype);

				/* If we're in Prom mode or if its and EII packet we just store	*/
				/* the MediaHeaderLen in DriverWorkspace. This will get adjusted	*/
				/* at RcvCompleteStatus time when we HSM reports packet length		*/
			
				if( (mediaData->PromiscuousMode)
					 || (GET_HILO_UINT16(&lookAheadData->MH_Length) >
				 		(MAX_PACKET_LENGTH - SIZE_E8023 ) ) )
				{	/* OR in flag to make RcvCompleteStatus aware of this case */
					ecb->ECB_DriverWorkspace.DWs_i16val[1]
				 			= (UINT16)(lkAheadStruc.LkAhd_MediaHeaderLen | 0x8000);

				}
				else	/* Store true data length (minus pad if any) now, this won't */
				{		/* get adjusted at RcvCompleteStatus time.	*/

 					ecb->ECB_DriverWorkspace.DWs_i16val[1]	= lengthField;
				}

			}	/* End Piplined Adapter  */

			/* Mark Ecb as having gone through look ahead. */
			ecb->ECB_DataLength |= HSM_HAS_BEEN_IN_GETRCB;

			return( (RCB *)ecb );
		}

		else /* Protocol Stack got everything it needed from the lookahead. */
		{
		   return( (RCB *)NULL );
		}
	}	/* End of packet wanted */

	else  /* The packet was not wanted. */
	 {
		++(statsTable->Gen_NoECBs);

		/* If the frame came from a pipeline adapter and was not wanted,  */
		/* this is the only chance we have to bump any stat counters. */

		if (pktSize == UNUSED)
		 {
			/* bump Recieve count */

			if ( !((UINT8)rcvStatus) )
				++(statsTable->Gen_TotalRxPackets);

			/*add to the count the header size + data length */

			statsTable->Gen_TotalRxOkByteLow += (lengthField + lkAheadStruc.LkAhd_MediaHeaderLen);

			/* Check for overflow in the lower 32 bits. */
         	if ( statsTable->Gen_TotalRxOkByteLow < (lengthField + lkAheadStruc.LkAhd_MediaHeaderLen))
				  	statsTable->Gen_TotalRxOkByteHigh++;

		 } /* End of Pipeline Adapter Counter Fest */
	 } /* No Ecb for this RCB */

   return( (RCB *)NULL );  /* Null tells the HSM to discard */

}	/*End CEtherTSMGetRCB*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMFastRcvComplete(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb);
*
*	Description:
*		This routine is called by HSM's that call CEtherTSMGetRCB and have
*		filled out the receive fragment list returned by CEtherTSMGetRCB.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Pointer to receive control block
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	CEtherTSMFastRcvComplete(
				DRIVER_DATA		*driverData,
				RCB				*rcb)
{
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	ECB							*ecb;

	ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
		ecb->ECB_DataLength &= ~HSM_HAS_BEEN_IN_GETRCB;

		sharedData = DADSP_TO_CMSMADSP(driverData);
		mediaData = sharedData->CMSMMediaDataPtr;
		ecb->ECB_PreviousLink = (ECB *)mediaData->RxStatus;

		/* ECB_Status was set in GetRCB */
		/* ecb->ECB_Status = ODISTAT_SUCCESSFUL; */

		CLSL_FastHoldEvent(ecb);
	 }
	else	/* This ECB came from a ecb aware adapter. */
	 {
		CEtherTSMECBRcvComplete(driverData, ecb);
#ifdef OS_NIOS
		CLSL_ServiceEvents();
#endif
	 }
	return;
}	/*End	CEtherTSMFastRcvComplete*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMFastRcvCompleteStatus(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						packetLength,
*					UINT32						packetStatus);
*
*	Description:
*		This routine is called by pipelined HSM's that call CEtherTSMGetRCB
*		with an unknown packet size/status. Since the packet has been
*		fully received off of the wire, the HSM can now give us the full
*		size of the packet and the error status.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Pointer to receive control block
*		packetLength -		Length of entire packet, including media header
*		packetStatus -		Error status of received packet
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	CEtherTSMFastRcvCompleteStatus(
				DRIVER_DATA		*driverData,
				RCB 				*rcb,
				UINT32			packetLength,
				UINT32			packetStatus)
{
	ECB							*ecb;

	/* Set up Locals */
	ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
		/* Fix up all the necessary ECB fields to complete the receive. */
		CEtherTSMPipelineRcvComplete(driverData, ecb, packetLength, packetStatus);
		CLSL_FastHoldEvent(ecb);
    }
	else
	 {
		/* This is an old style ecb aware adapter. */
		/* Setup for calling the nominal ECBRcvComplete Function */
		ecb->ECB_PreviousLink = (ECB *)(packetStatus);
		CEtherTSMECBRcvComplete(driverData, ecb);
#ifdef OS_NIOS
		CLSL_ServiceEvents();
#endif

	} /* Old style ecb aware adapter. */

	return;
}	/*End	CEtherTSMFastRcvCompleteStatus*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMRcvComplete(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb);
*
*	Description:
*		This routine is called by HSM's that call CEtherTSMGetRCB and have
*		filled out the receive fragment list returned by CEtherTSMGetRCB.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Pointer to receive control block
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	CEtherTSMRcvComplete(
				DRIVER_DATA		*driverData,
				RCB				*rcb)
{
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	ECB							*ecb;

	ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
		sharedData = DADSP_TO_CMSMADSP(driverData);
		mediaData = sharedData->CMSMMediaDataPtr;

		ecb->ECB_DataLength &= ~HSM_HAS_BEEN_IN_GETRCB;
		ecb->ECB_PreviousLink = (ECB *)mediaData->RxStatus;

		/* ECB_Status was set in GetRCB */
		/* ecb->ECB_Status = ODISTAT_SUCCESSFUL; */

		CLSL_HoldEvent(ecb);
	 }
	else	/* This ECB came from an ecb aware adapter. */
	 {
		CEtherTSMECBRcvComplete(driverData, ecb);
	 }

	return;
}	/*End	CEtherTSMRcvComplete*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMRcvCompleteStatus(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						packetLength,
*					UINT32						packetStatus);
*
*	Description:
*		This routine is called by pipelined HSM's that call CEtherTSMGetRCB
*		with an unknown packet size/status. Since the packet has been
*		fully received off of the wire, the HSM can now give us the full
*		size of the packet and the error status.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Pointer to receive control block
*		packetLength -		Length of entire packet, including media header
*		packetStatus -		Error status of received packet
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*                                                                       *
************************************************************************/
void	CEtherTSMRcvCompleteStatus(
				DRIVER_DATA		*driverData,
				RCB 				*rcb,
				UINT32			packetLength,
				UINT32			packetStatus)
{
	ECB							*ecb;

	/* Setup Locals */
	ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
		/* Fix up all the necessary ECB fields to complete the receive. */
		CEtherTSMPipelineRcvComplete(driverData, ecb, packetLength, packetStatus);
		CLSL_HoldEvent(ecb);
	 }
	else
	 {
		/* This is an old style ecb aware adapter. */
		/* Setup for calling the nominal ECBRcvComplete Function */
		ecb->ECB_PreviousLink = (ECB *)(packetStatus);
		CEtherTSMECBRcvComplete(driverData, ecb);

	 } /* Old style ecb aware adapter. */
	return;

}	/*End	CEtherTSMRcvCompleteStatus*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMECBRcvComplete(
*					DRIVER_DATA					*driverData,
*					ECB							*ecb,
*
*	Description:
*		This routine is called from CEtherTSMFastRcvComplete or
*		CEtherTSMRcvComplete for an ECB aware adapter.  A lookahead has
*		not been created an presented to the LSL.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		ecb -					Pointer to a completely filled out ecb
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*                                                                       *
************************************************************************/
void	CEtherTSMECBRcvComplete(
						DRIVER_DATA		*driverData,
						ECB 				*ecb)
{
	SHARED_DATA				*sharedData;
	GENERIC_STATS			*statsTable;
	MEDIA_DATA				*mediaData;
	ETHER_MEDIA_HEADER	*macData;
	LOOKAHEAD				lkAheadStruc;

	/* Set up locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
   statsTable = sharedData->CMSMGenericStatsPtr;

	/*fill in the lookahead structure.*/
   lkAheadStruc.LkAhd_PreFilledECB = ecb;
	lkAheadStruc.LkAhd_MediaHeaderPtr = (UINT8 *)&(ecb->ECB_Fragment[1]);
	macData = (ETHER_MEDIA_HEADER *)lkAheadStruc.LkAhd_MediaHeaderPtr;

	/* Check to see if physical fragments are being used. */
	if (((DADSP_TO_CMSMADSP(driverData)->CMSMDefaultVirtualBoard->MLIDCFG_ModeFlags) & MM_FRAGS_PHYS_BIT))
		lkAheadStruc.LkAhd_MediaHeaderLen =
			(UINT32)(((UINT32)(ecb->ECB_Fragment[0]).FragmentAddress) -
					  ((UINT32)CMSMGetPhysical(lkAheadStruc.LkAhd_MediaHeaderPtr)));
	else
		lkAheadStruc.LkAhd_MediaHeaderLen =
				(UINT32)(((UINT32)(ecb->ECB_Fragment[0]).FragmentAddress) -
							((UINT32)lkAheadStruc.LkAhd_MediaHeaderPtr));

	lkAheadStruc.LkAhd_DataLookAheadPtr =
							(UINT8 *)( (UINT32)(lkAheadStruc.LkAhd_MediaHeaderPtr) +
										  lkAheadStruc.LkAhd_MediaHeaderLen );

	/* Adjust fragment address past the media header. */
	ecb->ECB_Fragment[0].FragmentAddress = lkAheadStruc.LkAhd_DataLookAheadPtr;

	lkAheadStruc.LkAhd_DataLookAheadLen = ecb->ECB_DataLength;
	lkAheadStruc.LkAhd_FrameDataSize = ecb->ECB_DataLength;
	lkAheadStruc.LkAhd_BoardNumber = ecb->ECB_BoardNumber;
	COPY_UINT32( &lkAheadStruc.LkAhd_ProtocolID, &ecb->ECB_ProtocolID );
	COPY_UINT16( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]),
					 &ecb->ECB_ProtocolID.protocolID[4] );
	COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress, &ecb->ECB_ImmediateAddress );
	lkAheadStruc.LkAhd_PadAlignBytes1 = 0;
	lkAheadStruc.LkAhd_PadAlignBytes2 = 0;
	lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
	lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
	lkAheadStruc.LkAhd_PriorityLevel = 0;
	lkAheadStruc.LkAhd_Reserved = NULL;
	lkAheadStruc.LkAhd_PktAttr = (UINT32)ecb->ECB_PreviousLink;
	lkAheadStruc.LkAhd_DestType =
					       GET_LOHI_UINT16(&ecb->ECB_DriverWorkspace.DWs_i8val[0]);

		/* Check to see if we need to filter/fix-up DestType bits	*/
	if ( (mediaData->PromiscuousMode) &&
		  (!(sharedData->CMSMDefaultVirtualBoard->MLIDCFG_Flags
		  	  & MF_SOFT_FILT_GRP_BIT)) &&
		  (!(lkAheadStruc.LkAhd_DestType & ECB_BROADCAST)) &&
		  (lkAheadStruc.LkAhd_DestType & (ECB_MULTICAST | ECB_MULTICASTREMOTE))
		)
	{	/* We need to do additional filterering */

		if (FindAddressInMCTable(sharedData,
										 (NODE_ADDR *)macData->MH_Destination.nodeAddress))
		{
			lkAheadStruc.LkAhd_DestType |= ECB_MULTICAST;
			lkAheadStruc.LkAhd_DestType &= ~ECB_MULTICASTREMOTE;
		}
		else
		{
			lkAheadStruc.LkAhd_DestType |= ECB_MULTICASTREMOTE;
			lkAheadStruc.LkAhd_DestType &= ~ECB_MULTICAST;
		}
	}

		/*Bump Counters*/
	if( !((UINT16)ecb->ECB_PreviousLink) )
		++(statsTable->Gen_TotalRxPackets);

	/* add to count header size */
	statsTable->Gen_TotalRxOkByteLow += ecb->ECB_DataLength +
	                                     lkAheadStruc.LkAhd_MediaHeaderLen;

  	/* Check for overflow in the lower 32 bits. */
   if (statsTable->Gen_TotalRxOkByteLow < (ecb->ECB_DataLength
											      + lkAheadStruc.LkAhd_MediaHeaderLen))
  	{
  		statsTable->Gen_TotalRxOkByteHigh++;
  	}

   statsTable->Gen_TotalGroupAddrRx +=
								ecb->ECB_DriverWorkspace.DWs_i8val[0] & ECB_MULTICAST;
			
	/* Update ECB DestType Field in DriverWorkspace */
	/* Save the lkahd destination type in first part of DriverWorkspace */
	ecb->ECB_DriverWorkspace.DWs_i16val[0] =
											 VALUE_TO_LOHI_UINT16((UINT16)lkAheadStruc.LkAhd_DestType);

	/* Update ECB fields to ensure set correctly */
	ecb->ECB_DriverWorkspace.DWs_i16val[1] = ecb->ECB_DataLength;

	if (CLSL_GetStackECB(&lkAheadStruc) == ODISTAT_OUT_OF_RESOURCES)
		++(statsTable->Gen_NoECBs);

	return;
}	/*End	CEtherTSMECBRcvComplete*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CEtherTSMPipelineRcvComplete(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						packetLength,
*					UINT32						packetStatus);
*
*	Description:
*		This routine is called by one of the RcvCompleteStatus routines for
*		ecbs coming from pipelined HSM's with an unknown packet size/status.
*		Since the packet has been fully received off of the wire, the HSM can
*		now give us the full size of the packet and the error status.
*
*	Parameters in:
*		driverData -		CHSM adapter data space
*		rcb -					Pointer to receive control block
*		packetLength -		Length of entire packet, including media header
*		packetStatus -		Error status of received packet
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	CEtherTSMPipelineRcvComplete(
				DRIVER_DATA		*driverData,
				ECB 				*ecb,
				UINT32			packetLength,
				UINT32			packetStatus)
{
	SHARED_DATA					*sharedData;
 	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statsTable;
	UINT32						bytesToCopy;
	UINT32						skipBytes;

	/* Set up Locals. */
	sharedData = DADSP_TO_CMSMADSP(driverData);
  	mediaData = sharedData->CMSMMediaDataPtr;
	statsTable = sharedData->CMSMGenericStatsPtr;
   configTable = sharedData->CMSMDefaultVirtualBoard;

	/* Update Status from what we found before. */
   packetStatus |= mediaData->RxStatus;

	/* check for Packet Too Big. */
	if( packetLength > configTable->MLIDCFG_MaxFrameSize )
	 {
		packetStatus |= PAE_TOO_BIG_BIT;	
	   statsTable->Gen_RxTooBig++;
	 }

  	/* check for Packet Too Small. */
  	if ( packetLength < MIN_PKT_SIZE )
	 {
		packetStatus |= PAE_RUNT_PACKET_BIT;
		if (packetLength < MIN_RUNT_SIZE)
			packetStatus |= PAE_NOT_ENABLED_BIT;

		++(statsTable->Gen_RxTooSmall);
	 }

	/* Adjust the total frame data size if GetRCB has set high bit which	*/
	/* means we're in promiscuous mode or it's an EII frame */
	if (ecb->ECB_DriverWorkspace.DWs_i16val[1] & 0x8000)
	 {
		ecb->ECB_DriverWorkspace.DWs_i16val[1] = packetLength -
									 (ecb->ECB_DriverWorkspace.DWs_i16val[1] & 0x7FFF);
	 }
	else if (ecb->ECB_DriverWorkspace.DWs_i16val[1] > packetLength)
    {
      /* looks like a malformed packet */
      packetStatus |= PAE_MALFORMED_BIT;
		statsTable->Gen_RxMiscError++;
    }


	/*bump stat counts*/
   /* statsTable->Gen_TotalGroupAddrRx was incremented in GetRCB */
	/* count all Bytes received --even for Error Packets? */
	statsTable->Gen_TotalRxOkByteLow += packetLength;

  	/* Check for overflow in the lower 32 bits. */
   if (statsTable->Gen_TotalRxOkByteLow < packetLength)
  		statsTable->Gen_TotalRxOkByteHigh++;

	/* Only count the Good packets received */
	if( !((UINT8)packetStatus) )
	 {
		++(statsTable->Gen_TotalRxPackets);
		ecb->ECB_Status = ODISTAT_SUCCESSFUL; /* assume success */

		/* Check for Possible Overflow */
		skipBytes = (ecb->ECB_DataLength & HSM_TOLD_TO_SKIP) >> 16;
		bytesToCopy = ecb->ECB_DataLength & HSM_TOLD_TO_COPY;

		if ((skipBytes + bytesToCopy) >= packetLength)
		 {
			if (skipBytes > packetLength)
				ecb->ECB_DataLength = 0;	/* packet too small */
			else
				ecb->ECB_DataLength = packetLength - skipBytes;
		 }	
		else /* may be an overflow packet */
		 {
			if (ecb->ECB_DataLength & PSTK_WANTS_FRAME)
			 {
				/* The protocol stack wanted the entire frame but did not supply */
				/* enough data buffer space. */
				ecb->ECB_Status = ODISTAT_RX_OVERFLOW;
			 }
			/* Clear bits 31 through 16. */
			ecb->ECB_DataLength &= HSM_TOLD_TO_COPY;
	 	 }	/* end of Maybe Overflow */
	 } /* Successful packets */
	else
	 {
		ecb->ECB_Status = ODISTAT_CANCELED; /* Indicate Failure */
		ecb->ECB_DriverWorkspace.DWs_i16val[0] = ECB_GLOBALERROR;

		/* Clear bits 31 through 16. */
		ecb->ECB_DataLength &= HSM_TOLD_TO_COPY;
		
	 }	/* End Error Packets */

	/* Set ECB packetStatus */
	ecb->ECB_PreviousLink = (ECB *)packetStatus;
	return;
} /* End CEtherTSMPipelineRcvComplete */

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32 CEtherTSMGetHSMIFLevel(void);
*
*	Description:
*		This routine will return the TSM<->HSM interface version level.
*		Current Version Level is 1.11.
*
*	Parameters in:
*
*	Values returned:
*		HSM Interface level
*
*	Implied parameters:
*		NONE
*
*  Notes:
*                                                                       *
************************************************************************/
UINT32	CEtherTSMGetHSMIFLevel()
{
	return( 111 );
}	/*End CEtherTSMGetHsmIfLevel*/

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT CEtherTSMRegisterHSM(
*					DRIVER_PARM_BLOCK			*hsmParameters,
*					CONFIG_TABLE				**configTable)
*
*	Description:
*		This routine is called by HSM's at the beginning of DriverInit.
*		It must be called at init time.
*
*	Parameters in:
*		hsmParameters -	Pointer to the Adapter Data Space.
*		configFDSP -		Location to return config table.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL	If TSM/MSM registers HSM successfully
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*                                                                       *
************************************************************************/

ODISTAT	CEtherTSMRegisterHSM(DRIVER_PARM_BLOCK	*hsmParameters,
										CONFIG_TABLE			**configTable)
{

	return( CMSMRegisterTSM( hsmParameters, &MediaParameters, configTable) );

}	/*End CEtherTSMRegisterCHSM*/

/***********************************************************************\
*                                                                       *
*	Name:
*		UINT32 CInitializeEtherTSM(
*					struct	LoadDefinitionStructure *moduleHandle,
*					struct	ScreenSruct *screenID)
*
*	Description:
*		This routine will be called by the OS when it
*		is being loaded.
*
*	Parameters in:
*		*moduleHandle -	Handle to this module.
*		*screenID -			Handle to output screen(usually console).
*
*	Values returned:
*		0	If module initialized successfully
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*
*                                                                       *
************************************************************************/
UINT32 CInitializeEtherTSM( struct _MODULE_HANDLE_	*moduleHandle,
									 struct _SCREEN_HANDLE_	*screenID)
{
	UINT32 		messageCount=0;
	UINT32 		languageID;
	UINT8  		*helpFile;
	UINT32      majorVersion, minorVersion;

	screenID = screenID;
	Ethernet8023Index = Ethernet8023Index;
	EthernetIIIndex   = EthernetIIIndex;
	Ethernet8022Index = Ethernet8022Index;
	EthernetSNAPIndex = EthernetSNAPIndex;

	/* Initialize fields in the TSM's Configuration Table */

	CMSMGetNLMVersionInfo ( moduleHandle, &majorVersion,
   	                     &minorVersion, NULL, NULL, NULL, NULL, NULL);
   	CEtherTSMConfigTable.CTSMCFG_ModuleMajorVersion = majorVersion;
   	CEtherTSMConfigTable.CTSMCFG_ModuleMinorVersion = minorVersion;

#ifdef SVR
	CEtherTSMConfigTable.CTSMCFG_SystemFlags = CEtherTSMConfigTable.CTSMCFG_SystemFlags | CTSM_CFG_SERVER_BIT;
#else
	CEtherTSMConfigTable.CTSMCFG_SystemFlags = CEtherTSMConfigTable.CTSMCFG_SystemFlags | CTSM_CFG_CLIENT_BIT;
#endif


	return CMSMReturnMessageInformation (
					moduleHandle,
					&DriverMessages,
					&messageCount,
					&languageID,
					&helpFile);

}	/* End CInitializeEtherTSM */

/***********************************************************************\
*                                                                       *
*	Name:
*		void CRemoveEtherTSM(void)
*
*	Description:
*		This routine will be called by the OS when it
*		is being unloaded.  It calls UnRegisterEventNotification
*		with ExitToDOSID as the parameter and returns.
*
*	Parameters in:
*		NONE
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*
************************************************************************/
void CRemoveEtherTSM( void )
{
	return;
}	/* End CRemoveEtherTSM */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT CEtherTSMBuildTransmitControlBlock(
*							DRIVER_DATA		*driverData,
*							ECB				*ecb,
*							TCB				**tcb
*						UINT32			*pktSize);
*
*	Description:
*			This routine is called by the CHSM when it is ready to send a
*	priority Packet that has been queued by a call to DriverPriorityQueueSupport.
*	The CHSM calls this routine to convert an ECB to a TCB.
*	
*
*	Parameters in:
*		driverData	-		CHSM adapter data space
*		ecb			-		Pointer to a Transmit ECB
*
*	Parameters returned:
*		tcb			-     Pointer to a Pointer to the TCB to send.
*		pktSize		-		Padded Length to send.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL
*							TCB pointer is valid. The CHSM should transmit the TCB.
*
*		ODISTAT_OUT_OF_RESOURCES
*							A TCB was not available.  The CHSM must not call this
*							routine with more outstanding TCB's than it set in the
*							CMSMPriorityTxFreeCount variable. The ECB is returned
*							to the CHSM.  The CHSM must either call this function
*							again after a TCB resource is available, or return the
*							ECB via the <CTSM>CancelPrioritySend.
*
*		ODISTAT_PACKET_UNDELIVERABLE
*							A TCB was available, but the ECB created a packet that
*							was too large for the media.  The ECB was returned to
*							the LSL and a TCB was not allocated.
*
*
*  Notes:
*			The CHSM should be aware of the number of TCBs available to the MLID
*			for Priority Sends.  The CTSM allocates a number of TCBs based on
*			the sum of the CMSMTxFreeCount and the CMSMPriorityTxFreeCount.
*			The CHSM must not have more outstanding PriorityTCB's than the CHSM
*			set in CMSMPriorityTxFreeCount during DriverInit. If the CHSM makes
*			this call when there are no TCB's available, an
*			ODISTAT_OUT_OF_RESOURCES is returned.
*
*			The CHSM need not do size checking on the resultant TCB.  If the
*			packet generated is too large for the Media, this function returns
*			ODISTAT_PACKET_UNDELIVERABLE after it returns the ECB to the LSL
*			and does not return a TCB to the CHSM.
*
*			The CHSM should not change the CMSMTxFreeCount for any TCB obtained
*			for a Priority transmit.  An internal counter for Priority support
*			resources should be maintained by the CHSM.
*			Pktsize provides the same Information as DriverSend received.
*			it is the "padded" size that should be transmitted.
*
*			If the HSM needs to cancel this ECB it should use
*			CEtherTSMSendComplete or CEtherTSMFastSendComplete.
*
************************************************************************/
ODISTAT CEtherTSMBuildTransmitControlBlock(
						DRIVER_DATA		*driverData,
						ECB				*ecb,
						TCB				**tcb,
						UINT32			*pktSize)
{
	FRAME_DATA					*frameData;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Set Up Local variables */
	frameData  = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;
	configTable = &frameData->ConfigTable;
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData  = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/* are we ECB aware? */
	if (sharedData->CMSMDPBlockPtr->DriverSendWantsECBs & 1)
	{  /* Yes, so not much to do */

	    /* Does CHSM want physical addresses? */

	    if (sharedData->CMSMNeedContiguousECB)
		{
			/* Protect Data */
			CMSMAcquireProtection( sharedData->CMSMProtectionObject, &oldState );

	    	*tcb = (TCB *)CMSMGetPhysicalECB (driverData, ecb);

			CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
		}
	    else *tcb = (TCB *)ecb;
	    /* All done withe the ECB aware case */
	    return (ODISTAT_SUCCESSFUL);
	}	/* End ECB Aware case */

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	/* Do we have a free TCB? */
	if( mediaData->TCBHead )
	{

		/* Does CHSM want physical addresses */

	    if (sharedData->CMSMNeedContiguousECB)
		{
	    	ecb = CMSMGetPhysicalECB (driverData, ecb);
		}

		/* Get a TCB and create the MAC Header. */
		*tcb = mediaData->TCBHead;
		mediaData->TCBHead = (TCB *)((*tcb)->TCB_DriverWS[0]);
		*pktSize = frameData->SendRoutine( sharedData, ecb,	*tcb );

		/* Check if Need ???? */
		if (sharedData->CMSMNeedContiguousECB)
		{
			(*tcb)->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
			(*tcb)->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&((*tcb)->TCB_MediaHeader) );
		}
						
		/* check that the packet is not to big. */
		if( *pktSize <= configTable->MLIDCFG_MaxFrameSize )
		{
			/* Bump Queue Depth */
			++(statTable->Gen_Qdepth);
		}
		else
		{
     		/* Return TCB and link into TCB list. */
	    	(*tcb)->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
   	    	mediaData->TCBHead = *tcb;
			++(statTable->Gen_TxTooBig);

			/* Set ECB status. */
			ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
			CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

			CLSL_FastSendComplete( ecb );
			*tcb = NULL;
			return( ODISTAT_PACKET_UNDELIVERABLE );
		 } /* Packet turned out to be too big */
	 } /* A free Resource was available. */
	else
	{
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		*tcb = NULL;
		return( ODISTAT_OUT_OF_RESOURCES );
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
	return( ODISTAT_SUCCESSFUL );
}	/* BuildTransmitControlBlock */

/***********************************************************************\
*                                                                       *
*   Name:
*       void CEtherTSMCancelPrioritySend(
*									 DRIVER_DATA		*driverData,
*                           ECB             *ecb);
*
*   Description:
*           This routine is called by the CHSM when it is canceling an ECB
*           originally accepted to transmit via DriverPriorityQueueSupport
*           The CHSM calls this routine to cancel/return an ECB that has
*           not been Sent.
*
*
*   Parameters in:
*       ecb         -       Pointer to a Transmit ECB
*
*   Values returned:
*       None
*
*  Notes:
*
*
************************************************************************/
void CEtherTSMCancelPrioritySend( DRIVER_DATA		*driverData,
                            ECB             *ecb)
{
	SHARED_DATA					*sharedData;
	GENERIC_STATS				*statTable;

	/* Setup the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	statTable = sharedData->CMSMGenericStatsPtr;

	++(statTable->Gen_TxMiscError);
    ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
    CLSL_FastSendComplete( ecb );
    return;

} /* End CEtherTSMCancelPrioritySend */


/***********************************************************************\
*                                                                       *
*   Name:
*      ODISTAT     CEtherTSMGetConfigInfo(
*                  		void 	*configInfo,
*                  		UINT32  *nBytes);
*
*   Description:
*                 	Allows a C HSM to get the configuration information
*					for the C TSM, including specification and module
*					versions.
*
*   Parameters in:
*					configInfo
*                    	A pointer to a buffer used to receive the returned
*                    	configuration information.  The caller needs to be
*						sure that the buffer is at least nBytes bytes long.
*
*                   nBytes
*                       Pointer to the requested number of bytes to be returned into the
*						buffer.
*
*   Values returned:
*					ODISTAT_SUCCESSFUL
*						The configuration information of nBytes was
*						successfully returned in the buffer.
*
*					ODISTAT_BAD_PARAMETER
*
*						The nBytes requested was larger than the actual
*						configuration information available.
*
*   Implied parameters:
*                   nBytes
*                       Pointer to the number of bytes returned in the
*						buffer.
*
*   Notes:
*					The configuration information is returned in the format
*					defined by CTSM_CONFIG_TABLE.
*
*
************************************************************************/

ODISTAT	CEtherTSMGetConfigInfo (
         void 	*configInfo,
         UINT32 *nBytes)
{
	UINT32	configTableSize;

	configTableSize = sizeof(CTSM_CONFIG_TABLE);

	if (*nBytes > configTableSize)
	{
		CMSMmemcpy(configInfo, &CEtherTSMConfigTable, configTableSize);

		*nBytes = configTableSize;

		return ( ODISTAT_BAD_PARAMETER );
	}

	CMSMmemcpy(configInfo, &CEtherTSMConfigTable, *nBytes);

	return ( ODISTAT_SUCCESSFUL );

}  /* End CEtherTSMGetConfigInfo */
