/***********************************************************************\
*
*   (c) Unpublished Work of Novell, Inc.  All rights reserved.
*
*   THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*   PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.  ACCESS
*   TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
*   NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
*   AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO
*   APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED,
*   PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED,
*   TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED,
*   LINKED,RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
*   CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
*   THE PRIOR WRITTEN CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF
*   THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
*   CRIMINAL AND CIVIL LIABILITY.
*                                                                      
***********************************************************************
*
*  Name:          FDDITSM.h
*
*  Description:   C version of the FDDI Structures
*
*  ODI Spec Ver   1.11
*
*  Modification History:
*
*  08-29-96 WTT		Added proto type for CFDDITSMGetConfigInfo.
*
*  12-05-96 WTT		Changed proto types for MediaAddMulticast, 
*					MediaDeleteMulticast and FindAddressInMCTable to 
*					conform to spec.  They were using UINT8 pointer and 
*					should have been using NODE_ADDR pointer for
*					the mcAddress.
*					SPD #143897
*
*\***********************************************************************/
#ifndef FDDITSM_H

/**************************************************************************\
**
**      Defines
**
\**************************************************************************/
/*These are used for indexing into a specific Frame-Type.		*/
#define  FDDI_8022_INTERNALID  0
#define  FDDI_SNAP_INTERNALID  1

/*These are the NOVELL assigned Frame IDs.					*/
#define	FDDI_8022_ID          20
#define	FDDI_SNAP_ID          23

/* This defines the size of each Frame-Types MAC Header.	*/
#define	SIZE_FDDI			16 
#define	SIZE_FDDISNAP		21 
#define	SIZE_MACFDDI		13

/*These are used for supporting Assembly HSMs.					*/
#define	FDDI_DESCRIPT_LEN				10
#define	FDDI_SNAP_DESCRIPT_LEN		9

/*PktHdr0 Constants.*/
#define	TOKEN_TYPE_MASK			0xCF 	/*Token type mask.*/
#define	TOKEN_TYPE_NONE			0x00	/*No token is required.*/
#define	TOKEN_TYPE_RESTRICT		0x10	/*Restricted Token used.*/
#define	TOKEN_TYPE_UNRESTRICT	0x20	/*Unrestricted Token used.*/
#define	TOKEN_TYPE_BOTH			0x30	/*Restricted & Unrestricted.*/

#define	SYNCH_FRAME					0x08	/*Synchronous frame TX.*/
#define	IMM_MODE						0x04	/*Ignore ring_operation.*/
#define	SEND_FIRST					0x02	/*Frame is first sent.*/
#define	BEACON_FRAME				0x01	/*Send only in tx_beacon state.*/

/*PktHdr1 Equates.*/
#define	SEND_LAST					0x40	/*Release token after this*/
													/*frame is sent.*/
#define	APPEND_CRC					0x20	/*Adapter append CRC.*/

#define	TOKEN_SEND_MASK			0xE7	/*Token send mask.*/
#define	TOKEN_SEND_NO_TOKEN		0x00	/*No token released.*/
#define	TOKEN_SEND_UNRESTRICT	0x08	/*Unrestricted token released.*/
#define	TOKEN_SEND_RESTRICT		0x10	/*Restricted token released.*/
#define	TOKEN_SEND_SAME			0x18	/*Save as receive released.*/

#define	EXTRA_FS_MASK				0xF8	/*Extra Frame State mask.*/
#define	EXTRA_NONE					0x00	/*TR RR II II.*/
#define	EXTRA_RR						0x01	/*TR RR RR II.*/
#define	EXTRA_SR						0x05	/*TR RR SR II.*/
#define	EXTRA_RS						0x01	/*TR RR RS II.*/
#define	EXTRA_SS						0x06	/*TR RR SS II.*/
#define	EXTRA_RT						0x03	/*TR RR RT II.*/
#define	EXTRA_ST						0x07	/*TR RR ST II.*/

/* This is the Maximum size a header can be	*/
#define	MAX_MAC_HEADER		sizeof (FDDI_MEDIA_HEADER) + SOURCE_MAX_SIZE	

/* This is the Maximum size a packet can be	*/
#define MAX_PACKET_LENGTH		4500 - 9

/* FrameControl bits */							/* All Defines include 48Bit Address Bit. */
#define	FC_FRAME_TYPE_BITS		0x70		/* Mask for type bits.(Ignore Synch bit.) */
#define	FC_DATA_TYPE_BITS			0x30		/* Mask off 2 low bits for data/Mac detection. */
#define	FC_SYNC_FRAME				0x80		/* Synchronous Frame Bit. */
#define	FC_MAC_FRAME				0xC0		/* MAC frames CX 						*/
#define	FC_SMT_FRAME				0x40		/* Station Management Frame 4X	*/
#define  FC_LLC_FRAME				0x50		/* LLC Frame Control Bit 5X		*/
#define	FC_IMP_FRAME				0x60		/* Reserved for Implementor 6X	*/
#define	FC_RES_FRAME				0x70		/* Reserved for Future Standards 7X */
#define	FC_TX_BYTE					0x51		/* 48Bit addr. Asynch. Nonimal Data traffic. */


#define	SOURCE_ROUTING_BIT		0x01		/* Routing field exists. */
#define	SOURCE_SIZE_MASK			0x1F		/* Routing Size Mask. */
#define	TX_8022_SNAP				0x03AAAAh /* SNAP DSAP, SSAP, CTRL0. */
#define	SNAP_INFO_SIZE				5			/* SNAP header info size. */
#define 	SOURCE_MAX_SIZE			30

#define MULTICASTBIT			0x01
#define GROUPBIT				0x01
#define LOCALBIT				0x02
#define BROADCAST				0xFFFF

/* These bits define 802.2 types.				*/
#define S_OR_U_FORMAT		0x01
#define U_FORMAT				0x02

/*v3.00 Assembly Statistics Table */
#define STATISTICSMASK		0x0D2813FF
#define FDDISTATSCOUNT		10
#define LONGCOUNTER			0x00
#define LARGECOUNTER			0x01

#define MAX_MULTICAST		32

/* ECB_DataLength bit assignmets for pipeline and ecb aware adapters. */
#define HSM_HAS_BEEN_IN_GETRCB	0x80000000		/* bit 31 */
#define PSTK_WANTS_FRAME			0x40000000		/* bit 30 */
#define HSM_TOLD_TO_SKIP			0x007F0000		/* bits 23 through 16 */
#define HSM_TOLD_TO_COPY			0x0000FFFF		/* bits 15 through 0 */

#define	NUMBER_OF_PROMISCUOUS_COUNTERS	32

/**************************************************************************\
**
**      TypeDefs
**
\**************************************************************************/

typedef struct	_MEDIA_DATA_	{
	UINT32					MaxMulticastAddresses;
	GROUP_ADDR_LIST_NODE	*MulticastAddressList;
	UINT32					MulticastAddressCount;
	TCB						*TCBHead;    
	TCB						*TCBList;    
	MLID_STATS_TABLE		*NewStatsPtr;
	UINT32					PromiscuousMode;
	UINT8						PromiscuousCounters[NUMBER_OF_PROMISCUOUS_COUNTERS];
	UINT32					RxStatus;
} MEDIA_DATA;

/**************************************************************************\
**
**      Prototype Definitions
**
\**************************************************************************/
UINT32   CInitializeFDDITSM (
            MODULE_HANDLE    *moduleHandle,
            SCREEN_HANDLE    *screenID);

void     CRemoveFDDITSM ( void );

ODISTAT	CFDDITSMRegisterHSM(
				DRIVER_PARM_BLOCK	   *DriverParameterBlock,
				CONFIG_TABLE			**configTable);

GROUP_ADDR_LIST_NODE *FindAddressInMCTable(
				SHARED_DATA			   *sharedData,
				NODE_ADDR				*mcAddress);

ODISTAT	MediaAdjust(
				FRAME_DATA				*frameData);

ODISTAT	MediaInit(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData);

ODISTAT	MediaReset(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				OPERATION_SCOPE		operationScope);

ODISTAT	MediaShutdown(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					shutdownType,
				OPERATION_SCOPE		operationScope);

void	   MediaSend(
				ECB						*ecb,
				CONFIG_TABLE			*configTable);

ODISTAT	MediaAddMulticast(
				DRIVER_DATA			   *driverData,
				NODE_ADDR				*McAddress);

ODISTAT	MediaDeleteMulticast(
				DRIVER_DATA			   *driverData,
				NODE_ADDR				*McAddress);

ODISTAT	MediaNodeOverride(
				FRAME_DATA				*frameData,
				MEON						mode);


ODISTAT	MediaAdjustNodeAddress(
				FRAME_DATA				*frameData);


ODISTAT	MediaSetLookAheadSize(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					size);

ODISTAT	MediaPromiscuousChange(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					PromiscuousState,
				UINT32					*PromiscuousMode);

ODISTAT	MediaRegisterMonitor(
				FRAME_DATA				*frameData,
				void						(*TXMonRoutine)(TCB *),
				BOOLEAN					MonitorState);

ODISTAT	MediaGetParameters(
            CONFIG_TABLE      *configTable);

ODISTAT	MediaGetMulticastInfo(
             DRIVER_DATA    *driverData,
             ECB            *MulticastInfoECB);

ODISTAT	CFDDITSMUpdateMulticast(
				DRIVER_DATA			   *driverData);

UINT32   CMediaSend8022(
            SHARED_DATA *sharedData,
				ECB         *ecb,
				TCB         *tcb);
	
UINT32   CMediaSendSNAP(
            SHARED_DATA *sharedData,
				ECB         *ecb,
				TCB         *tcb);
	
TCB		*CFDDITSMGetNextSend(
				DRIVER_DATA			   *driverData,
				CONFIG_TABLE			**configTable,
				UINT32					*lengthToSend,
				void						**TCBPhysicalPtr);

void		CFDDITSMFastSendComplete(
				DRIVER_DATA			   *driverData,
				TCB						*tcb,
            UINT32               transmitStatus);

void		CFDDITSMSendComplete(
				DRIVER_DATA			   *driverData,
            TCB                  *tcb,
            UINT32               transmitStatus);

RCB		*CFDDITSMGetRCB(
				DRIVER_DATA          *driverData,
				UINT8                *packetHdr,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32 					*startByte,
				UINT32 					*numBytes);

RCB 		*CFDDITSMProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize);

RCB 		*CFDDITSMFastProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize);

void		CFDDITSMFastRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb);

void		CFDDITSMFastRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus);

UINT32	CFDDITSMGetHSMIFLevel();

void		CFDDITSMRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb);

void		CFDDITSMRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus);

void		CFDDITSMECBRcvComplete(
				DRIVER_DATA				*driverData,
				ECB 						*ecb);

void		CFDDITSMPipelineRcvComplete(
				DRIVER_DATA				*driverData,
				ECB 						*ecb,
				UINT32					packetLength,
				UINT32					packetStatus);

ODISTAT CFDDITSMBuildTransmitControlBlock(
						DRIVER_DATA		*driverData,
						ECB				*ecb,
						TCB				**tcb,
						UINT32			*pktSize);


void CFDDITSMCancelPrioritySend(
							DRIVER_DATA		*driverData,
							ECB				*ecb);

ODISTAT	CFDDITSMGetConfigInfo (
         void 	*configInfo,
         UINT32 *nBytes);


#ifdef IAPX386
void     BuildASMStatStrings(
				StatTableEntry		   *tableEntry);
#endif


#define FDDITSM_H
#endif /* FDDITSM_H */
