
;****************************************************************************
; ODI.INC
;
;  Copyright Unpublished Work of Novell, Inc. All Rights Reserved
;  
;  THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
;  PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.   
;  ACCESS TO THIS WORK IS RESTRICTED TO (i) NOVELL EMPLOYEES  
;  WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE  
;  OF THEIR ASSIGNMENTS AND (ii) ENTITIES OTHER THAN NOVELL   
;  WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.      
;  NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,
;  DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED,      
;  CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,  
;  TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF
;  NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION
;  COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
;****************************************************************************
;
; Title:		32 Bit Assembly version of the C ODI Header File
;
; Filename:    	ODI.inc
;
; ODI Spec Ver:	C 1.11
;
; Description:	This file provides C ODI specification definition in assembly
;				format.
;
; Modification History:
; 
; 12-02-96 WTT	Updated LSL configuration table structure to spec 1.11
;				
;*****************************************************************************
;
;       DOS/Windows Client-32 General ODI Definitions
;       
;*****************************************************************************

.386p

ODISPEC_MAJOR_VER	equ		1	
ODISPEC_MINOR_VER	equ		11


;! Well Defined Sizes

PID_SIZE                equ     06h             ; Number of Octets in Protocol Identifier
ADDR_SIZE               equ     06h             ; Number of Octets in Address
MAXLOOKAHEADSIZE        equ     128             ; Max. LookAhead Data Size

;define assumed Maximum Media Header Size that we'll encounter
;assume that it will be Token-Ring (with SRT)
; AC, FC, Dest[6], Source[6], SRFields[30], 802.2UI[3], SNAP[5] = 52

MAXMEDIAHEADERSIZE      equ     52


DefaultNumECBs          equ     00h
DefaultECBSize          equ     1518                            ; not including ECB Structure
MinECBSize              equ     512+74+MAXMEDIAHEADERSIZE       ; Max. ECB size < 64K, Min ECB Size 512+74+52


MAXSTACKNAMELENGTH      equ     65
MAXNAMELENGTH           equ     64

MAXMULTICASTS           equ     20              ;MAXIMUM number of Multicast addresses

MAX_FRAG_COUNT          equ     16              ;Maximum number of Fragment 
						;structures in a Fragment List

;! Chained Protocol Stack position values

CHNPOS_FIRST_MUST       equ     0000h   ; Must be First
CHNPOS_FIRST_NEXT       equ     0001h   ; Next available First
CHNPOS_LOAD_ORDER       equ     0002h   ; dependent on Load Order
CHNPOS_LAST_NEXT        equ     0003h   ; Next available Last
CHNPOS_LAST_MUST        equ     0004h   ; Must be Last

CHNPOS_MAX_POSIT        equ     0004h   ; Maximum possible Chain position

CHAINTYPE_TX            equ     0000h   ; Transmit type Chain Protocol Stack
CHAINTYPE_RX            equ     0001h   ; Receive type Chain Protocol Stack

;!


; Macro used for generating length preceded, 0 terminated strings.
;
; Example:      Message CustomString1   'TotalTxCount'

Message macro   name,stringg
	local   tail
name    db      (tail-2) - name
	db      stringg, 0
tail    equ     $
	endm


;*****************************************************************************
; GENERAL DEFINITIONS
;*****************************************************************************

;! ECB FRAGMENTSTRUC

FRAGMENTSTRUC   struc
	FragmentAddress dd      0
	FragmentLength  dd      0

FRAGMENTSTRUC   ends
;!

;! AESECB

AESECB  struc

	AES_Link                dd      0
	AES_MSecondValue        dd      0
	AES_Status              dw      0
	AES_ESR                 dd      0
	AES_Reserved            dd      0
	AES_ResourceObj         dd      0
	AES_Context             dd      0
AESECB  ends
;!

;! ECB

ECB     struc
	ECB_NextLink            dd      0
	ECB_PreviousLink        dd      0
	ECB_Status              dw      0
	ECB_ESR                 dd      0
	ECB_StackID             dw      0
	ECB_ProtocolID          db      PID_SIZE dup (0)
	ECB_BoardNumber         dd      0
	ECB_ImmediateAddress    db      ADDR_SIZE dup (0)
	ECB_DriverWorkspace     db      4 dup (0)
	ECB_ProtocolWorkspace   db      8 dup (0)
	ECB_DataLength          dd      0
	ECB_FragmentCount       dd      0
	ECB_Fragment1Addr       dd      0
	ECB_Fragment1Len        dd      0
ECB     ends

;ECB_PacketEnvelope should only be used with LSL Buffers,ECB_FragmentCount is 1

ECB_PacketEnvelope              equ     byte ptr ECB_Fragment1Len + 4
;!

;! ECBD

ECBD    struc
	NextLink        dd      0
	PrevLink        dd      0
	Status          dw      0
	ESR             dd      0
	ProtoNum        dw      0
	ProtID          db      6 dup (0)
	BoardNum        dd      0
	ImmAddr         db      6 dup (0)
	DriverWS        db      4 dup (0)
	ProtocolWS      db      8 dup (0)
	DataLen         dd      0
	FragCount       dd      0
	Frag1Addr       dd      0
	Frag1Len        dd      0
ECBD    ends

; ECB fields defined for receive ECBs when the ESR is invoked.

RxECBPacketAttributes   equ     PrevLink
RxECBDestinationType    equ     byte ptr DriverWS+0
RxECBFrameDataSize      equ     word ptr DriverWS+2

; Stack ID Definitions

ECB_RAWSEND             equ     0FFFFh  ; Raw Send, ie. ECB includes MAC Header
					; implies MLID should not build MAC header.

; Rx Destination Address Type (First byte of ECB_DriverWorkspace).
; These should be the same as their DT_xxxxx bit flag counterparts.

ECB_MULTICAST           equ     0001h   ;Multicast Dest. address (Group Address)
ECB_BROADCAST           equ     0002h   ;Broadcast Dest. address
ECB_UNICASTREMOTE       equ     0004h   ;Remote Unicast Dest. address
ECB_MULTICASTREMOTE     equ     0008h   ;Unsupported Multicast address
ECB_SOURCE_ROUTE        equ     0010h   ;Source Routed packet

ECB_ERRORED             equ     0020h   ;Set if packet contains errors (exculsive)
ECB_GLOBALERROR         equ     0020h   ;NB. If set all other bits should be reset.

ECB_MAC_FRAME           equ     0040h   ;MAC/SMT frames. (ie. NON-DATA Frame)
ECB_UNICASTDIRECT       equ     0080h   ;Unicast for this workstation

ECB_MASK                equ     00FFh   ;mask off allowable Destination Address Types

ECB_TYPE_I              equ     0100h   ;Set if packet is 802.2 Type I
ECB_TYPE_II             equ     0200h   ;Set if packet is 802.2 Type II
ECB_RX_PRIORITY         equ     0400h   ;Set if packet has priority value other than base values
ECB_PROMISCUOUS         equ     ECB_ERRORED OR ECB_MULTICAST OR ECB_BROADCAST OR ECB_UNICASTREMOTE OR ECB_MULTICASTREMOTE OR ECB_SOURCE_ROUTE OR ECB_MAC_FRAME OR ECB_RX_PRIORITY


; ECBFragStruc

FragStruc       struc
	FragAddr        dd      0
	FragLen         dd      0
FragStruc       ends


;! LOOKAHEAD

LOOKAHEAD       struc
	LkAhd_PreFilledECB              dd      0
	LkAhd_MediaHeaderPtr            dd      0
	LkAhd_MediaHeaderLen            dd      0
	LkAhd_DataLookAheadPtr          dd      0
	LkAhd_DataLookAheadLen          dd      0
	LkAhd_BoardNumber               dd      0
	LkAhd_PktAttr                   dd      0
	LkAhd_DestType                  dd      0
	LkAhd_FrameDataSize             dd      0
	LkAhd_PadAlignBytes1            dw      0
	LkAhd_ProtocolID                db      PID_SIZE dup (0)
	LkAhd_PadAlignBytes2            dw      0
	LkAhd_ImmediateAddress          db      ADDR_SIZE dup (0)
	LkAhd_FrameDataStartCopyOffset  dd      0
	LkAhd_FrameDataBytesWanted      dd      0
	LkAhd_ReturnedECB               dd      0
	LkAhd_Priority                  dd      0
	LkAhd_Reserved1                 dd      0
LOOKAHEAD       ends
;!

;! Packet Attributes Bit Values
									
PAE_CRC_BIT             equ     00000001h
PAE_CRC_ALIGN_BIT       equ     00000002h
PAE_RUNT_PACKET_BIT     equ     00000004h
PAE_TOO_BIG_BIT         equ     00000010h
PAE_NOT_ENABLED_BIT     equ     00000020h
PAE_MALFORMED_BIT       equ     00000040h
PA_NO_COMPRESS_BIT      equ     00004000h       ;Do not compress received packet
PA_NONCAN_ADDR_BIT      equ     00008000h

PAE_ERROR_MASK          equ     PAE_CRC_BIT OR PAE_CRC_ALIGN_BIT OR PAE_RUNT_PACKET_BIT OR\
				PAE_TOO_BIG_BIT OR PAE_NOT_ENABLED_BIT OR PAE_MALFORMED_BIT

;!


;! Destination Type Bit Values

DT_MULTICAST            equ     000000001h
DT_BROADCAST            equ     000000002h
DT_REMOTE_UNICAST       equ     000000004h
DT_REMOTE_MULTICAST     equ     000000008h
DT_SOURCE_ROUTE         equ     000000010h
DT_ERRORED              equ     000000020h      ;Mutually exclusive with other bits
DT_MAC_FRAME            equ     000000040h      ;MAC/SMT frames. (ie. NON-DATA Frame)
DT_DIRECT               equ     000000080h
DT_MASK                 equ     0000000FFh      ;mask off allowable Destination Address Types (Dword Mask)
DT_MASK_B               equ     0FFh            ;mask off allowable Destination Address Types (Byte Mask)

DT_8022_TYPE_I          equ     000000100h      ;Set if packet is 802.2 Type I
DT_8022_TYPE_II         equ     000000200h      ;Set if packet is 802.2 Type II
DT_8022_BYTES_BITS      equ     000000300h
DT_RX_PRIORITY          equ     000000400h      ;Set if packet has priority value other than base values
DT_PROMISCUOUS          equ     DT_ERRORED or DT_DIRECT or DT_MULTICAST or DT_BROADCAST or DT_REMOTE_UNICAST or DT_REMOTE_MULTICAST or DT_SOURCE_ROUTE or DT_MAC_FRAME or DT_RX_PRIORITY

;!


;! LSL System Error Codes

LSLERR_BAD_COMMAND      equ     0FFFFFF81h
LSLERR_BAD_PARAMETER    equ     0FFFFFF82h
LSLERR_DUPLICATE_ENTRY  equ     0FFFFFF83h
LSLERR_FAIL             equ     0FFFFFF84h
LSLERR_ITEM_NOT_PRESENT equ     0FFFFFF85h
LSLERR_NO_MORE_ITEMS    equ     0FFFFFF86h
LSLERR_NO_SUCH_DRIVER   equ     0FFFFFF87h
LSLERR_NO_SUCH_HANDLER  equ     0FFFFFF88h
LSLERR_OUT_OF_RESOURCES equ     0FFFFFF89h
LSLERR_RX_OVERFLOW      equ     0FFFFFF8Ah
LSLERR_CANCELED         equ     0FFFFFFFCh
LSLERR_CANCELLED        equ     0FFFFFFFCh

ODISTAT_SUCCESSFUL              equ     000000000h
ODISTAT_RESPONSE_DELAYED        equ     000000001h
ODISTAT_SUCCESS_TAKEN           equ     000000002h

ODISTAT_BAD_COMMAND             equ     0FFFFFF81h
ODISTAT_BAD_PARAMETER           equ     0FFFFFF82h
ODISTAT_DUPLICATE_ENTRY         equ     0FFFFFF83h
ODISTAT_FAIL                    equ     0FFFFFF84h
ODISTAT_ITEM_NOT_PRESENT        equ     0FFFFFF85h
ODISTAT_NO_MORE_ITEMS           equ     0FFFFFF86h
ODISTAT_NO_SUCH_DRIVER          equ     0FFFFFF87h
ODISTAT_NO_SUCH_HANDLER         equ     0FFFFFF88h
ODISTAT_OUT_OF_RESOURCES        equ     0FFFFFF89h
ODISTAT_RX_OVERFLOW             equ     0FFFFFF8Ah
ODISTAT_CANCELED                equ     0FFFFFFFCh
ODISTAT_CANCELLED               equ     0FFFFFFFCh
;!


; C LSL Configuration Table System Flags definitions.

CLSL_CFG_SERVER_BIT equ	40000000h
CLSL_CFG_CLIENT_BIT	equ 80000000h

;! StatTableEntry

; StatTableEntry Definitons
;   StatUseFlag Determines how StatCounter is defined:

ODI_STAT_UNUSED         equ     0FFFFFFFFh      ; Statistics Table Entry not in use.
ODI_STAT_UINT32         equ     00000000h       ; Statistics Table Entry UINT32 Counter.
ODI_STAT_UINT64         equ     00000001h       ; Statistics Table Entry UINT64 Counter.
ODI_STAT_MEON_STRING    equ     00000002h       ; Statistics Table Entry Counter is a MEON_STRING.
ODI_STAT_UNTYPED        equ     00000003h       ; Statistics Table Entry Counter is a UINT32 length preceded array UINT8.
ODI_STAT_RESETABLE      equ     80000000h       ; Statistics Table Entry Counter is resetable by 
												;   external entity. This define is only used by 
												;   management agents and other modules that
												;	are using STAT_TABLE definintions from this
												;	header file.  ODI_STAT_RESETABLE is not a
												;	part of the ODI Specification and is not allowed 
												;	for MLID stats counters.  ODI considers 
												;	0x80000000 a reserved bit to avoid conflicts with 
												;	other modules in the system.


; *StatCounter  pointer as defined by the StatUseFlag.
; *StatString   pointer to a MEON String, describing the statistics counter.

StatTableEntry  struc
	StatUseFlag     dd      0
	StatCounter     dd      0
	StatString      dd      0
StatTableEntry  ends
;!


;*****************************************************************************
; LSL DEFINITIONS
;*****************************************************************************

;! Definitions for Information Block for passing API's, eg. Function Lists

INFO_BLOCK      struc
	NumberOfAPIs    dd      0
	SupportAPIArray dd      0
INFO_BLOCK      ends

;!


;! Definitions for Link Support Layer (LSL) 

LogBrdStatTableEntry    struc
	LogBrd_TransmittedPackets       dd      0
	LogBrd_ReceivedPackets          dd      0
	LogBrd_UnclaimedPackets         dd      0
	LogBrd_Reserved                 dd      0
LogBrdStatTableEntry    ends


LSL_ConfigTable struc
	LConfigTableMajorVer    		dw  0
	LConfigTableMinorVer    		dw	0
	LSLLongName             		dd  0       ; -> ASCIIZ
	LSLShortName            		dd  0       ; -> ASCIIZ
	LSLMajorVer             		dw  0
	LSLMinorVer             		dw  0
	LMaxNumberOfBoards      		dd  0
	LMaxNumberOfStacks      		dd  0
	LConfigTableReserved0   		dd  0
	LConfigTableReserved1   		dd  0
	LConfigTableReserved2   		dd  0
	LConfigODISpecMajorVer			db	0
	LConfigODISpecMinorVer			db	0
	LConfigTableReserved3   		dw  0
	LConfigSystemFlags				dd	0
	LConfigSmallECBCount			dd	0
	LConfigMediumECBCount			dd	0
	LConfigLargeECBCount			dd	0
	LConfigXLargeECBCount			dd	0
	LConfigHugeECBCount				dd	0
	LConfigSmallECBBelow16Count		dd	0
	LConfigMediumECBBelow16Count	dd	0
	LConfigLargeECBBelow16Count		dd	0
	LConfigXLargeECBBelow16Count	dd	0
	LConfigHugeECBBelow16Count 		dd	0
	LConfigSmallECBMinCount			dd	0
	LConfigMediumECBMinCount		dd	0
	LConfigLargeECBMinCount			dd	0
	LConfigXLargeECBMinCount		dd	0
	LConfigHugeECBMinCount			dd	0
	LConfigSmallECBMaxCount			dd	0
	LConfigMediumECBMaxCount		dd	0
	LConfigLargeECBMaxCount			dd	0
	LConfigXLargeECBMaxCount		dd	0
	LConfigHugeECBMaxCount			dd	0
	LConfigSmallECBSize				dd	0
	LConfigMediumECBSize			dd	0
	LConfigLargeECBSize				dd	0
	LConfigXLargeECBSize			dd	0
	LConfigHugeECBSize				dd	0

LSL_ConfigTable ends


LSL_StatsTable  struc
	LStatTableMajorVer      dw      0
	LStatTableMinorVer      dw      0
	LNumGenericCounters     dd      0
	LGenericCountsPtr       dd      0       ; -> array of StatTableEntries
	LNumLogBrds             dd      0
	LLogBrdsCountsPtr       dd      0       ; -> array of StatTableEntries
	LNumCustomCounters      dd      0
	LCustomCountersPtr      dd      0       ; -> array of StatTableEntries
LSL_StatsTable  ends

;!



;*****************************************************************************
; PROTOCOL STACK DEFINITIONS
;*****************************************************************************

;! Definitions for Protocol Stack Configuration And Statistics Tables

PS_ConfigTable  struc
	PConfigTableMajorVer    dw      0
	PConfigTableMinorVer    dw      0
	PProtocolLongName       dd      0       ; -> ASCIIZ
	PProtocolShortName      dd      0       ; -> ASCIIZ
	PProtocolMajorVer       dw      0
	PProtocolMinorVer       dw      0
PS_ConfigTable  ends


PS_StatsTable   struc
	PStatTableMajorVer      dw      0
	PStatTableMinorVer      dw      0
	PNumGenericCounters     dd      0
	PGenericCountsPtr       dd      0       ; -> array of StatTableEntries
	PNumCustomCounters      dd      0
	PCustomCountersPtr      dd      0       ; -> array of StatTableEntries
PS_StatsTable   ends
;!

;*****************************************************************************
; MLID DEFINITIONS
;*****************************************************************************

;! Definitions for MLID Configuration, Statistics Tables and Misc. structures

MLID_ConfigTable        struc
	MLIDCFG_Signature       db      26 dup (0)
	MLIDCFG_MajorVersion    db      0
	MLIDCFG_MinorVersion    db      0
	MLIDCFG_NodeAddress     db      ADDR_SIZE dup (0)
	MLIDCFG_ModeFlags       dw      0
	MLIDCFG_BoardNumber     dw      0
	MLIDCFG_BoardInstance   dw      0
	MLIDCFG_MaxFrameSize    dd      0
	MLIDCFG_BestDataSize    dd      0
	MLIDCFG_WorstDataSize   dd      0
	MLIDCFG_CardName        dd      0       ; -> ASCIIZ
	MLIDCFG_ShortName       dd      0       ; -> ASCIIZ
	MLIDCFG_FrameTypeString dd      0       ; -> ASCIIZ
	MLIDCFG_Reserved0       dw      0
	MLIDCFG_FrameID         dw      0
	MLIDCFG_TransportTime   dw      0
	MLIDCFG_SourceRouting   dd      0
	MLIDCFG_LineSpeed       dw      0
	MLIDCFG_LookAheadSize   dw      0
	MLIDCFG_Reserved1       db      2 dup (0)
	MLIDCFG_PrioritySup     dw      0
	MLIDCFG_Reserved2       dd      0
	MLIDCFG_DriverMajorVer  db      0
	MLIDCFG_DriverMinorVer  db      0
	MLIDCFG_Flags           dw      0
	MLIDCFG_SendRetries     dw      0
	MLIDCFG_DriverLink      dd      0
	MLIDCFG_SharingFlags    dw      0
	MLIDCFG_Slot            dw      0
	MLIDCFG_IOPort0         dw      0
	MLIDCFG_IORange0        dw      0
	MLIDCFG_IOPort1         dw      0
	MLIDCFG_IORange1        dw      0
	MLIDCFG_MemoryAddress0  dd      0
	MLIDCFG_MemorySize0     dw      0
	MLIDCFG_MemoryAddress1  dd      0
	MLIDCFG_MemorySize1     dw      0
	MLIDCFG_Interrupt0      db      0
	MLIDCFG_Interrupt1      db      0
	MLIDCFG_DMALine0        db      0
	MLIDCFG_DMALine1        db      0
	MLIDCFG_ResourceTag     dd      0
	MLIDCFG_Config          dd      0
	MLIDCFG_CommandString   dd      0
	MLIDCFG_LogicalName     db      18 dup (0)
	MLIDCFG_LinearMemory0   dd      0
	MLIDCFG_LinearMemory1   dd      0
	MLIDCFG_ChannelNumber   dw      0
	MLIDCFG_DBusTag         dd      0
	MLIDCFG_DIOConfigMajorVer db    0
	MLIDCFG_DIOConfigMinorVer db    0
MLID_ConfigTable        ends


MLID_StatsTable struc
		MStatTableMajorVer      dw      0
		MStatTableMinorVer      dw      0
		MNumGenericCounters     dd      0
		MGenericCountsPtr       dd      0       ; -> array of StatTableEntries
		MNumMediaCounters       dd      0
		MMediaCountsPtr         dd      0       ; -> array of StatTableEntries
		MNumCustomCounters      dd      0
		MCustomCountersPtr      dd      0       ; -> array of StatTableEntries
MLID_StatsTable ends


MLID_Reg        struc
	MLIDSendHandler         dd      0
	MLIDControlHandler      dd      0
	MLIDSendContext         dd      0
	MLIDResourceObj         dd      0
	MLIDModuleHandle        dd      0
MLID_Reg        ends

;!


;! MLID Control Commands

MLID_GET_CONFIGURATION          equ     0
MLID_GET_STATISTICS             equ     1
MLID_ADD_MULTICAST              equ     2
MLID_DELETE_MULTICAST           equ     3
MLID_SHUTDOWN                   equ     5
MLID_RESET                      equ     6
MLID_SET_LOOK_AHEAD             equ     9
MLID_PROMISCUOUS_CHANGE         equ     10
MLID_REGISTER_TX_MONITOR        equ     11
MLID_MANAGEMENT                 equ     14
;!


;! MLIDAsmConfigTableStruc (Assembly)

MLIDAsmConfigTableStruc struc
	MAsmSignature           db      'HardwareDriverMLID',8 dup (' ')
	MAsmConfigTableMajorVer db      ?
	MAsmConfigTableMinorVer db      ?
	MAsmNodeAddress         db      6 dup (?)
	MAsmModeFlags           dw      ?
	MAsmBoardNumber         dw      ?
	MAsmBoardInstance       dw      ?
	MAsmMaxPacketSize       dd      ?
	MAsmBestDataSize        dd      ?
	MAsmWorstDataSize       dd      ?
	MAsmCardLongName        dd      ?               ; -> Len preceded string
	MAsmCardShortName       dd      ?               ; -> Len preceded string
	MAsmFrameString         dd      ?               ; -> Len preceded string
	MAsmReserved0           dw      0               ;Must be set to 0
	MAsmFrameID             dw      ?
	MAsmTransportTime       dw      ?
	MAsmRouteHandler        dd      ?               ;Only for Token-Ring
	MAsmLineSpeed           dw      ?               ;In Mbps, if MSb set then Kbps
	MAsmLookAheadSize       dw      ?
	MAsmReserved1           db      2 dup (0)       ;Must be set to 0
	MAsmPrioritySup         dw      0
	MAsmBusTag              dd      0
	MAsmMLIDMajorVer        db      ?
	MAsmMLIDMinorVer        db      ?
	MAsmFlags               dw      ?
	MAsmSendRetries         dw      ?
	MAsmLink                dd      ?
	MAsmSharingFlags        dw      ?
	MAsmSlot                dw      ?
	MAsmIOAddress1          dw      ?
	MAsmIORange1            dw      ?
	MAsmIOAddress2          dw      ?
	MAsmIORange2            dw      ?
	MAsmMemoryAddress1      dd      ?
	MAsmMemorySize1         dw      ?
	MAsmMemoryAddress2      dd      ?
	MAsmMemorySize2         dw      ?
	MAsmIntLine1            db      ?
	MAsmIntLine2            db      ?
	MAsmDMALine1            db      ?
	MAsmDMALine2            db      ?
	MAsmIOResourceTag       dd      ?
	MAsmLSLConfigPtr        dd      ?
	MAsmCommandLineParms    dd      ?
	MAsmLogicalName         dd      ?
	MAsmLinearMemory0       dd      ?
	MAsmLinearMemory1       dd      ?
	MAsmChannelNumber       dw      ?
	MAsmIOReserved          db      6 dup (?)
MLIDAsmConfigTableStruc ends


; MModeFlags Bit Values

MM_CANONICAL_BITS       equ     0C000h
MM_PHYS_NODE_ADDR_BIT   equ     08000h  ;Set if MLID utilizes Physical Node Address
MM_NONCANONICAL_BIT     equ     04000h  ;Set if Config Node Address Non-Canonical

MM_PROMISCUOUS_BIT      equ     02000h
MM_FRAGS_PHYS_BIT       equ     01000h  ;Set if HSM wants Frags with Physical Addresses
MM_C_HSM_BIT            equ     00800h  ;Set if HSM written in C.
MM_FRAG_RECEIVES_BIT    equ     00400h
MM_SMP_BIT              equ     00100h  ;Set if MLID is SMP enabled.
MM_DATA_SZ_UNKNOWN_BIT  equ     00080h
MM_RAW_SENDS_BIT        equ     00040h

MM_CSL_COMPLIANT_BIT    equ     00010h  ;Set if MLID is CSL compliant
;;MM_POINT_TO_POINT_BIT equ     00010h  ;removed

MM_MULTICAST_BIT        equ     00008h
MM_DEPENDABLE_BIT       equ     00004h
MM_USES_DMA_BIT         equ     00002h
MM_REAL_DRV_BIT         equ     00001h



; MFlags Bit Values

MF_EISA_BIT             equ     00001h
MF_ISA_BIT              equ     00002h
MF_MCA_BIT              equ     00004h
MF_HUB_MANAGEMENT_BIT   equ     00100h
MF_SOFT_FILT_GRP_BIT    equ     00200h
MF_GRP_ADDR_SUP_BIT     equ     00400h
MF_MULTICAST_TYPE_BITS  equ     00600h
MF_RECONFIG_BIT         equ     00800h
MF_PRIORITYSUP_BIT      equ     01000h


; MSharingFlags Bit Values

MS_SHUTDOWN_BIT         equ     00001h
MS_SHARE_PORT0_BIT      equ     00002h
MS_SHARE_PORT1_BIT      equ     00004h
MS_SHARE_MEMORY0_BIT    equ     00008h
MS_SHARE_MEMORY1_BIT    equ     00010h
MS_SHARE_IRQ0_BIT       equ     00020h
MS_SHARE_IRQ1_BIT       equ     00040h
MS_SHARE_DMA0_BIT       equ     00080h
MS_SHARE_DMA1_BIT       equ     00100h
MS_HAS_CMD_INFO_BIT     equ     00200h
MS_NO_DEFAULT_INFO_BIT  equ     00400h
;!


;! MLIDStatTableStruc   Assembly version

MLIDAsmStatTableStruc   struc
	MAsmStatTableMajorVer   db      ?
	MAsmStatTableMinorVer   db      ?
	MAsmNumGenericCounters  dw      ?
	MAsmValidCounterMask0   dd      ?
	MAsmTotalTxPackets      dd      ?
	MAsmTotalRxPackets      dd      ?
	MAsmNoECBsAvailable     dd      ?
	MAsmTxTooBig            dd      ?
	MAsmTxTooSmall          dd      ?
	MAsmRxOverflow          dd      ?
	MAsmRxTooBig            dd      ?
	MAsmRxTooSmall          dd      ?
	MAsmTxMiscError         dd      ?
	MAsmRxMiscError         dd      ?
	MAsmTxRetryCount        dd      ?
	MAsmRxChecksumError     dd      ?
	MAsmRxMismatchError     dd      ?
	MAsmTotalTxOKByteLow    dd      ?
	MAsmTotalTxOKByteHigh   dd      ?
	MAsmTotalRxOKByteLow    dd      ?
	MAsmTotalRxOKByteHigh   dd      ?
	MAsmTotalGroupAddrTx    dd      ?
	MAsmTotalGroupAddrRx    dd      ?
	MAsmAdapterReset        dd      ?
	MAsmAdapterOprTimeStamp dd      ?
	MAsmQDepth              dd      ?
	MAsmMediaStat0          dd      ?
	MAsmMediaStat1          dd      ?
	MAsmMediaStat2          dd      ?
	MAsmMediaStat3          dd      ?
	MAsmMediaStat4          dd      ?
	MAsmMediaStat5          dd      ?
	MAsmMediaStat6          dd      ?
	MAsmMediaStat7          dd      ?
	MAsmMediaStat8          dd      ?
	MAsmMediaStat9          dd      ?
	MAsmValidCounterMask1   dd      ?
	MAsmMediaStat10         dd      ?
	MAsmMediaStat11         dd      ?
	MAsmMediaStat12         dd      ?
	MAsmMediaStat13         dd      ?
	MAsmNumCustomCounters   dw      ?
MLIDAsmStatTableStruc   ends


; Token-Ring Media Specific Counters

TRACErrorCount                  equ     MediaStat0
TRAbortDelimiterCount           equ     MediaStat1
TRBurstErrorCount               equ     MediaStat2
TRFrameCopiedErrorCount         equ     MediaStat3
TRFrequencyErrorCount           equ     MediaStat4
TRInternalErrorCount            equ     MediaStat5
TRLastRingStatus                equ     MediaStat6
TRLineErrorCount                equ     MediaStat7
TRLostFrameCount                equ     MediaStat8
TRTokenErrorCount               equ     MediaStat9
TRUpstreamNodeHighDword         equ     MediaStat10
TRUpstreamNodeLowWord           equ     MediaStat11
TRLastRingID                    equ     MediaStat12
TRLastBeaconType                equ     MediaStat13

; Ethernet Media Specific Counters

ETxOKSingleCollisionsCount      equ     MediaStat0
ETxOKMultipleCollisionsCount    equ     MediaStat1
ETxOKButDeffered                equ     MediaStat2
ETxAbortLateCollision           equ     MediaStat3
ETxAbortExcessCollision         equ     MediaStat4
ETxAbortCarrierSense            equ     MediaStat5
ETxAbortExcessiveDeferral       equ     MediaStat6
ERxAbortFrameAligment           equ     MediaStat7

; RXNet Media Specific Counters

RNoResponseToFreeBufferEnquiry  equ     MediaStat0
RNetworkReconfigurationCount    equ     MediaStat1
RInvalidSplitFlagInPacketFrag   equ     MediaStat2
ROrphanPacketFragmentCount      equ     MediaStat3
RReceivePacketTimeout           equ     MediaStat4
RFreeBufferEnquiryNAKTimeout    equ     MediaStat5
RTotalTxPacketFragmentsOK       equ     MediaStat6
RTotalRxPacketFragmentsOK       equ     MediaStat7


; FDDI Media Specific Counters

FConfigurationState             equ     MediaStat0
FUpstreamNodeLow                equ     MediaStat1
FUpstreamNodeHigh               equ     MediaStat2
FDownstreamNodeLow              equ     MediaStat3
FDownstreamNodeHigh             equ     MediaStat4
FFrameErrorCount                equ     MediaStat5
FFrameLostCount                 equ     MediaStat6
FRingManagementState            equ     MediaStat7
FLCTFailureCount                equ     MediaStat8
FLemRejectCount                 equ     MediaStat9
FLConnectionState               equ     MediaStat10

;!


;! Definitions for Bound Protocol Stacks

PS_BoundNode    struc
	ProtocolName            dd      0
	ProtocolReceiveHandler  dd      0
	ProtocolControlHandler  dd      0
	ProtocolResourceObj     dd      0
PS_BoundNode    ends


; Definitions for PreScan Rx/Tx and Default Chained Protocol Stacks

PS_ChainedNode  struc
	StackChainLink                  dd      0
	StackChainBoardNumber           dd      0
	StackChainPositionRequested     dd      0
	StackChainHandler               dd      0
	StackChainControl               dd      0
	StackChainFilter                dd      0
	StackChainContext               dd      0
	StackChainResourceObj           dd      0
PS_ChainedNode  ends


;! Useful macros for writing "C" callable procs.

DoOdiCPush      macro
	push    ebp
	push    ebx
	push    esi
	push    edi

	endm

DoOdiCPop       macro
	pop     edi
	pop     esi
	pop     ebx
	pop     ebp

	endm

;!

;! Macro Definitions to ease access to Control Procedures for Stacks and MLIDs

;Protocol Stack Control Functions

PSTK_GET_CONFIGURATION  equ     0
PSTK_GET_STATISTICS     equ     1
PSTK_BIND               equ     2
PSTK_UNBIND             equ     3
PSTK_MLID_DEREGISTER    equ     4
PSTK_PROMISCUOUS_STATE  equ     5
PSTK_SFTIIIEXCHANGE     equ     6
PSTK_GET_PROTOCOL_STRG  equ     7


; Macro's to obtain Protocol Stack's Configuration Table
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               ECX     Stack Identifier
;
; returns:      EAX     Ptr. to Stacks Configuration Table
;

PStkCntl_GetConfig      macro
		DoOdiCPush
		push    ecx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_GET_CONFIGURATION * 4)]
		add     esp, 1 * 4
		DoOdiCPop
		endm
		

; Macro's to obtain Protocol Stack's Statistics Table
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               ECX     Stack Identifier
;
; returns:      EAX     Ptr. to Stacks Statistics Table
;

PStkCntl_GetStats       macro
		DoOdiCPush
		push    ecx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_GET_STATISTICS * 4)]
		add     esp, 1 * 4
		DoOdiCPop
		endm


; Macro's to bind Protocol Stack's to specified Board
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to User defined String
;               EBX     Board to Bind to.
;               ECX     Stack Identifier
;
; returns:      EAX     ODI Status of call.
;

PStkCntl_Bind   macro
		DoOdiCPush
		push    ecx
		push    edi
		push    ebx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_BIND * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm



; Macro's to notify Protocol Stack that MLID it was bound has deregistered
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EBX     Board #
;               ECX     Stack Identifier
;
; returns:      none
;

PStkCntl_MLIDDeReg      macro
		DoOdiCPush
		push    ecx
		push    ebx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_MLID_DEREGISTER * 4)]
		add     esp, 2 * 4
		DoOdiCPop
		endm


; Macro's to unbind a Protocol Stack's from a specified Board
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to User defined String
;               EBX     Board to unbind from.
;               ECX     Stack Identifier
;
; returns:      EAX     ODI Status of call.
;

PStkCntl_Unbind macro
		DoOdiCPush
		push    ecx
		push    edi
		push    ebx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_UNBIND * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to inform Protocol Stack's of Promiscuous State change of board
;                                                               bound to.
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EBX     Board #.
;               EDX     Promiscuous Mask.
;               ECX     Stack Identifier
;
; returns:      EAX     ODI Status of call.
;

PStkCntl_PromiscState   macro
		DoOdiCPush
		push    ecx
		push    edx
		push    ebx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_PROMISCUOUS_STATE * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to for Protocol Stack's SFT III Exchange
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to buffer to place Protocol String 
;               EBX     Board to obtain Protocol String for.
;               ECX     Stack Identifier
;
; returns:      EAX     ODI Status of call.
;

PStkCntl_SFTIIIXhge     macro

		mov     eax, ODISTAT_BAD_COMMAND
		endm


; Macro's to obtain a Protocol Stack's specific string for specified Board
;
; assumes:      ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to buffer to place Protocol String 
;               EBX     Board to obtain Protocol String for.
;               ECX     Stack Identifier
;
; returns:      EAX     ODI Status of call.
;

PStkCntl_GetProtString  macro
		DoOdiCPush
		push    ecx
		push    edi
		push    ebx
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (PSTK_GET_PROTOCOL_STRG * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm

;!


;!
; Macro's to obtain MLID's Configuration Table
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. memory to place ptr to Config Table
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code
;


MLIDCntl_GetConfig      macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_GET_CONFIGURATION * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm

; Macro's to obtain MLID's Statistics Table
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. memory to place ptr to Stats Table
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code
;


MLIDCntl_GetStats       macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_GET_STATISTICS * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm
		

; Macro's to add Multicast Address to MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to 6 byte Multicast Address to add
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_AddMulti       macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_ADD_MULTICAST * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to delete Multicast Address to MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ptr. to 6 byte Multicast Address to delete
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_DelMulti       macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_DELETE_MULTICAST * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to ShutDown MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     ShutDown Type
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_Shutdown       macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_SHUTDOWN * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to Reset MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_Reset  macro
		DoOdiCPush
		push    ecx
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_RESET * 4)]
		add     esp, 2 * 4
		DoOdiCPop
		endm


; Macro's to Set the Look Ahead size for an MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     LookAhead size requested
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_SetLookAheadSize       macro
		DoOdiCPush
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_SET_LOOK_AHEAD * 4)]
		add     esp, 3 * 4
		DoOdiCPop
		endm


; Macro's to change the Promiscuous Mode of an MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     Ptr to Promiscuous State (Bits)
;               EDX     Promiscuous Mode (Enable/Disable)
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_PromisChange   macro
		DoOdiCPush
		push    ecx
		push    edx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_PROMISCUOUS_CHANGE * 4)]
		add     esp, 4 * 4
		DoOdiCPop
		endm


; Macro's to register a Transmit Monitor for an MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     Ptr to Transmit Mointor
;               EDX     Monitor State
;               ECX     ptr. to AsyncECB (usually NULL)
;
; returns:      EAX     Completion Code

MLIDCntl_RegMon macro
		DoOdiCPush
		push    edx
		push    ecx
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_REGISTER_TX_MONITOR * 4)]
		add     esp, 4 * 4
		DoOdiCPop
		endm


; Macro's to pass a Management ECB to an MLID
;
; assumes:      EAX     Board Number
;               ESI     ptr. INFO_BLOCK API Array for Protocol Stack Control
;               EDI     Ptr to Management ECB
;
; returns:      EAX     Completion Code

MLIDCntl_Management     macro
		DoOdiCPush
		push    edi
		push    eax
		mov     eax, [esi + 4]                  ;get ptr. to API Array
		call    [eax + (MLID_MANAGEMENT * 4)]
		add     esp, 2 * 4
		DoOdiCPop
		endm

;!

