/***********************************************************************\
*
*   (c) Unpublished Work of Novell, Inc.  All rights reserved.
*
*   THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*   PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.  ACCESS
*   TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
*   NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
*   AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO
*   APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED,
*   PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED,
*   TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED,
*   LINKED,RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
*   CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
*   THE PRIOR WRITTEN CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF
*   THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
*   CRIMINAL AND CIVIL LIABILITY.
*                                                                      
***********************************************************************
*
*  Name:          ODI32ASM.H
*
*  Description:   
*
*  ODI Spec Ver   1.10
*
*  Origin:        
*
*  Created by:    DCD, DGM
*
*  Modification History:
*
*	26 August 1993 14:17	DGM
*		Updated LookAhead Structure. Updated Packet Attributes & Destination Type 
*		bits.  Updated all tables for ** pointers in Config & Stat Tables. Also
*		updated misc. bit definitions.
*
* 6 December 1993 10:53	DGM
*		Removed somemore old definitions.
*		StatTableEntryStruc, LSLLogicalBoardStatStruc, ProtocolControlProcsStruc, 
*		BoundStack & ChainedStackStruc.
*
*		Updated to ANSI "C" Beta 3 spec., 
*									eg. MLID_Reg, MLID IOCtls, CLSL_BindProtocolToBoard.
*
*		Removed use of old server CLSLXXXXX calls, all function calls should
*		now be to CLSL_XXXXXX etc.
*
* 16 December 1993 11:07	DGM
*		Swapped positions of MMLIDCFG_Reserved1 & MLIDCFG_BusTag, made the 
*		reserved field two UINT8s and changed BusTag from a UINT16 to a UINT32.
*
* 20 April 1994 9:11	DGM
*		Added the field 'UINT32 AES_Reserved' to the definition for the structure
*		AESECB.  This now makes the definition overlay those of the assembly 
*		NetWare Server v3.x and v4.x platforms.
*
* 13 May 1994 7:44	DGM
*		Replaced use of CLSL_RegisterPreScanRxChain & 
*		CLSL_RegisterPreScanTxChain by CLSL_RegisterPreScanChain.  Also
*		replaced use of CLSL_DeRegisterPreScanRxChain & 
*		CLSL_DeRegisterPreScanTxChain by CLSL_DeRegisterPreScanChain.
*
*		Also adjusted the macro definition for the Protocol Stack Control 
*		Services for GetProtocolStringForBoard.
*
*		Also added DT_MAC_FRAME & ECB_MAC_FRAME to identify frames whose 
*		Destination type is non-data frames, eg. MAC Frames in Token-Ring 
*		or FDDI.  Also updated DT_PROMISCUOUS with DT_MAC_FRAME and 
*		ECB_PROMISCUOUS with ECB_MAC_FRAME.
*
*		Replaced use of DT_SROUTE with DT_SOURCE_ROUTE and ECB_NOSOURCEROUTE		
*		with ECB_SOURCE_ROUTE to reflect altered definitions for these bits.
*
* 17 May 1994 13:02	dgm
*		Altered the definition of <typedef UINT8 PROT_ID [PID_SIZE];> & 
*		<typedef UINT8 NODE_ADDR [ADDR_SIZE];> to be structures
*
*			typedef	struct	_PROT_ID_ {
*						UINT8		protocolID[PID_SIZE];
*						} PROT_ID;
*
*			typedef struct	_NODE_ADDR_ {
*						UINT8	nodeAddress[ADDR_SIZE];
*						} NODE_ADDR;
*
*		Also updated structures LOOKAHEAD, ECB, MLID_ConfigTable and MLID Control
*		Macro's to allow for these definitions.
*
* 31 May 1994 12:28	DGM
*		Updated Protocol Control Service Macros to allow for StackIdentifier.
*
*	8 June 1994 13:21	DGM
*		Added MM_SMP_BIT equ 00100h, Set if MLID is SMP enabled for 
*		MLIDCFG_ModeFlags.
*
*	10 June 1994 15:41	DGM
*		Corrected declearation for CLSL_GetStackIDFromName, should have had
*		2 parmeters.
*
*  18 Aug 1995  JWR
*	   			Type modHandle in MLIDAsmConfigTableStruc changed to void *
*
\***********************************************************************/


/*****************************************************************************
*
*	DOS/Windows Client-32 General ODI Definitions
*	
*****************************************************************************/

#ifndef	_ODI_H
#define	_ODI_H	1


include	"odi.h"

/****************************************************************************
* GENERAL DEFINITIONS
****************************************************************************/

/* Old LSL System Error Codes
*/

#define	LSLERR_BAD_COMMAND			0xFFFFFF81
#define	LSLERR_BAD_PARAMETER			0xFFFFFF82
#define	LSLERR_DUPLICATE_ENTRY		0xFFFFFF83
#define	LSLERR_FAIL						0xFFFFFF84
#define	LSLERR_ITEM_NOT_PRESENT		0xFFFFFF85
#define	LSLERR_NO_MORE_ITEMS			0xFFFFFF86
#define	LSLERR_NO_SUCH_DRIVER		0xFFFFFF87
#define	LSLERR_NO_SUCH_HANDLER		0xFFFFFF88
#define	LSLERR_OUT_OF_RESOURCES		0xFFFFFF89
#define	LSLERR_RX_OVERFLOW			0xFFFFFF8A
#define	LSLERR_CANCELED				0xFFFFFFFC
#define	LSLERR_CANCELLED				0xFFFFFFFC


/*Set PRAGMA to pack these structures
*/

#pragma	pack(1)

/* Old FragStruc
*/

typedef struct FragStruc
{
	void		*FragAddr;
	UINT32	FragLen;

}Frag, FragT, *FragP;


/* Old ECBStruc
*/

typedef struct ECBStruc
{
	struct	ECBStruc *NextLink;
	struct	ECBStruc *PrevLink;
	UINT16	Status;
	void		(*ESR)( struct ECBStruc *);			/* Send & Receive	*/
	UINT16	ProtoNum;									/* Send	*/
	UINT8		ProtID[6];									/* Send	*/
	UINT32	BoardNum;									/* Send	*/
	UINT8		ImmAddr[6];									/* Send	*/
	UINT8		DriverWS[4];
	UINT8		ProtocolWS[8];
	UINT32	DataLen;										/* Send	*/
	UINT32	FragCount;									/* Send & Receive	*/
	void		*Frag1Addr;									/* Send & Receive	*/
	UINT32	Frag1Len;									/* Send & Receive	*/

}ECBD;

#define RxECBPacketAttributes ((UINT32)PrevLink);
#define RxECBDestinationType (*((UINT8) *)&DriverWS[0]));
#define RxECBFrameDateSize (*((UINT16 *)&DriverWS[2]));


/****************************************************************************
* Old MLID DEFINITIONS
****************************************************************************/

/* Assembly MLIDAsmConfigTableStruc
*/

typedef struct MLIDAsmConfigTableStruc
{
	UINT8			MAsmSignature[26];
	UINT8			MAsmConfigTableMajorVer;
	UINT8			MAsmConfigTableMinorVer;
	UINT8			MAsmNodeAddress[6];
	UINT16		MAsmModeFlags;
	UINT16		MAsmBoardNumber;
	UINT16		MAsmBoardInstance;
	UINT32		MAsmMaxPacketSize;
	UINT32		MAsmBestDataSize;
	UINT32		MAsmWorstDataSize;
	UINT8			*MAsmCardLongName;			/* -> Len preceded string	*/
	UINT8			*MAsmCardShortName;			/* -> Len preceded string	*/
	UINT8			*MAsmFrameString;				/* -> Len preceded string	*/
	UINT16		MAsmReserved0;					/* Must be set to 0	*/
	UINT16		MAsmFrameID;
	UINT16		MAsmTransportTime;
	void			(*MAsmRouteHandler)();
	UINT16		MAsmLineSpeed;					/* In Mbps, if MSb set then Kbps	*/
	UINT16		MAsmLookAheadSize;
	UINT16		MAsmBusTag;
	UINT16		MAsmPrioritySup;
	UINT8			MAsmReserved1[4];
	UINT8			MAsmMLIDMajorVer;
	UINT8			MAsmMLIDMinorVer;
	UINT16		MAsmFlags;
	UINT16		MAsmSendRetries;
	UINT32		MAsmLink;
	UINT16		MAsmSharingFlags;
	UINT16		MAsmSlot;
	UINT16		MAsmIOAddress1;
	UINT16		MAsmIORange1;
	UINT16		MAsmIOAddress2;
	UINT16		MAsmIORange2;
	UINT32		MAsmMemoryAddress1;
	UINT16		MAsmMemorySize1;
	UINT32		MAsmMemoryAddress2;
	UINT16		MAsmMemorySize2;
	UINT8			MAsmIntLine1;
	UINT8			MAsmIntLine2;
	UINT8			MAsmDMALine1;
	UINT8			MAsmDMALine2;
	void			*MAsmIOResourceTag;
	UINT32		MAsmLSLConfigPtr;
	UINT8			*MAsmCommandLineParms;
	UINT8			*MAsmLogicalName;
	UINT32		MAsmLinearMemory0;
	UINT32		MAsmLinearMemory1;
	UINT16		MAsmChannelNumber;
	UINT8			MAsmIOReserved[6];
}MLIDAsmConfigTable;

/* MLIDStatTableStruc	*/

typedef struct MLIDAsmStatTableStruc
{
	UINT8		MAsmStatTableMajorVer;
	UINT8		MAsmStatTableMinorVer;
	UINT16	MAsmNumGenericCounters;
	UINT32	MAsmValidCounterMask0;
	UINT32	MAsmTotalTxPackets;
	UINT32	MAsmTotalRxPackets;
	UINT32	MAsmNoECBsAvailable;
	UINT32	MAsmTxTooBig;
	UINT32	MAsmTxTooSmall;
	UINT32	MAsmRxOverflow;
	UINT32	MAsmRxTooBig;
	UINT32	MAsmRxTooSmall;
	UINT32	MAsmTxMiscError;
	UINT32	MAsmRxMiscError;
	UINT32	MAsmTxRetryCount;
	UINT32	MAsmRxChecksumError;
	UINT32	MAsmRxMismatchError;
	UINT32	MAsmTotalTxOKByteLow;
	UINT32	MAsmTotalTxOKByteHigh;
	UINT32	MAsmTotalRxOKByteLow;
	UINT32	MAsmTotalRxOKByteHigh;
	UINT32	MAsmTotalGroupAddrTx;
	UINT32	MAsmTotalGroupAddrRx;
	UINT32	MAsmAdapterReset;
	UINT32	MAsmAdapterOprTimeStamp;
	UINT32	MAsmQDepth;
	UINT32	MAsmediaStat0;
	UINT32	MAsmediaStat1;
	UINT32	MAsmediaStat2;
	UINT32	MAsmediaStat3;
	UINT32	MAsmediaStat4;
	UINT32	MAsmediaStat5;
	UINT32	MAsmediaStat6;
	UINT32	MAsmediaStat7;
	UINT32	MAsmediaStat8;
	UINT32	MAsmediaStat9;
	UINT32	MAsmValidCounterMask1;
	UINT32	MAsmediaStat10;
	UINT32	MAsmediaStat11;
	UINT32	MAsmediaStat12;
	UINT32	MAsmediaStat13;
	UINT16	MAsmNumCustomCounters;

}MLIDAsmStatTable;


/* Token-Ring Media Specific Counters	*/

#define	TRACErrorCount		   			MediaStat0
#define	TRAbortDelimiterCount  			MediaStat1
#define	TRBurstErrorCount	   			MediaStat2
#define	TRFrameCopiedErrorCount			MediaStat3
#define	TRFrequencyErrorCount  			MediaStat4
#define	TRInternalErrorCount   			MediaStat5
#define	TRLastRingStatus	   			MediaStat6
#define	TRLineErrorCount	   			MediaStat7
#define	TRLostFrameCount	   			MediaStat8
#define	TRTokenErrorCount	   			MediaStat9
#define	TRUpstreamNodeHighDword			MediaStat10
#define	TRUpstreamNodeLowWord  			MediaStat11
#define	TRLastRingID		   			MediaStat12
#define	TRLastBeaconType	   			MediaStat13

/* Ethernet Media Specific Counters	*/

#define	ETxOKSingleCollisionsCount		MediaStat0
#define	ETxOKMultipleCollisionsCount	MediaStat1
#define	ETxOKButDeffered					MediaStat2
#define	ETxAbortLateCollision			MediaStat3
#define	ETxAbortExcessCollision			MediaStat4
#define	ETxAbortCarrierSense				MediaStat5
#define	ETxAbortExcessiveDeferral		MediaStat6
#define	ERxAbortFrameAligment			MediaStat7

/* RXNet Media Specific Counters	*/

#define	RNoResponseToFreeBufferEnquiry MediaStat0
#define	RNetworkReconfigurationCount  MediaStat1
#define	RInvalidSplitFlagInPacketFrag	MediaStat2
#define	ROrphanPacketFragmentCount		MediaStat3
#define	RReceivePacketTimeout			MediaStat4
#define	RFreeBufferEnquiryNAKTimeout	MediaStat5
#define	RTotalTxPacketFragmentsOK		MediaStat6
#define	RTotalRxPacketFragmentsOK		MediaStat7


/*	FDDI Media Specific Counters	*/

#define	FConfigurationState				MediaStat0
#define	FUpstreamNodeLow					MediaStat1
#define	FUpstreamNodeHigh					MediaStat2
#define	FDownstreamNodeLow				MediaStat3
#define	FDownstreamNodeHigh				MediaStat4
#define	FFrameErrorCount					MediaStat5
#define	FFrameLostCount					MediaStat6
#define	FRingManagementState				MediaStat7
#define	FLCTFailureCount					MediaStat8
#define	FLemRejectCount					MediaStat9
#define	FLConnectionState					MediaStat10


/*--------------------------------------------------------------------------*/

#pragma	pack()


/* CLSLGetMLIDAttributes Return Bits
*/

#define	MA_PACKET_ATTRIBUTES_BIT	0x00000001

#endif	/* _ODI_H	*/

/*========[ Function Prototypes ]================*/

/****************************************************************************
* LSL "C" FUNCTION PROTOTYPES
****************************************************************************/

/*SINT32	CLSLGetMLIDAttributes(	UINT32	boardNumber);	*/
void		LSLServiceEvents (void);


/*========[ Global Variables ]================*/
