/*--------------------------------------------------------------------------*
 *  Copyright Unpublished Work of Novell, Inc. All Rights Reserved
 *
 *  THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 *  PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.
 *  ACCESS TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES
 *  WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 *  OF THEIR ASSIGNMENTS AND (ii) ENTITIES OTHER THAN NOVELL
 *  WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.
 *  NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,
 *  DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED,
 *  CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
 *  TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF
 *  NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION
 *  COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
 *--------------------------------------------------------------------------*/
/*****************************************************************************
 *
 * Title:			 Environment Mapping Prototypes
 *
 * Filename:		 private.h
 *
 * ODI Spec Ver:	 1.11
 *
 * Description:		Derived from private.h in original PIN code.
 *
 * Modification History:
 *
 * 09-03-96 KRA		Added #define of CMSMmemcpy for NT.
 *
 * 12-13-96 KRA     Moved COMMAND_STRUCT to a common area so that NT can
 *					also see it.
 *
 * 01-08-97 WTT     Added defines for memory allocation routines.  Some of
 *					these defines were previously located in generic.c
 *
 * 03-04-97 KRA     Removed the MSG macros for the NT case.
 *
 * 01-15-98 KRA		Added #define of CMSMmemset for NT.  Made
 *							MAX_LOGICAL_NAME_SZ available in all cases, not just NT.
 *
 ****************************************************************************/

typedef	struct	_COMMAND_STRUCT_
{
	struct _COMMAND_STRUCT_	*CommandLineNext;
	MEON							*CommandLineBuffer;
} COMMAND_STRUCT;

/****************************************************************\
	
	Defines used by memory allocation routines.

\****************************************************************/

#define CMSM_PAGE_SIZE     		4096
#define CMSM_PAGE_SHIFT    		12
#define	CMSM_MAX_ALLOC_SIZE		0x3FFFFFFF-(CMSM_PAGE_SIZE*2)
#define CMSM_BYTES_TO_PAGES(n)	((n+CMSM_PAGE_SIZE-1) >> CMSM_PAGE_SHIFT)
#define SIXTEEN_MEG				16384000
#define BELOW_16MEG_NOT_REQ		0x80000000


#define  MAX_LOGICAL_NAME_SZ        17 /* config table LogicalName field size */
													/* less the null terminator */

#ifndef OS_NT
/****************************************************************\
*                                                              	*
* ResourceTagStructure - Fields accessable from RTag, which		*
* is allocated by AllocateResourceTag, and passed to OS				*
* allocation routines so that system resources can be tracked.	*
* The HSM is not to modify any of the fields in this structure.	*
*                                                              	*
\****************************************************************/

typedef struct _RTAG_HANDLE_
{
	struct	ResourceTagStructure *RTLink;
	struct	LoadDefinitionStructure *RTModule;
	UINT32	RTSignature;
	UINT32	RTResourceCount;
	UINT32	RTCustomData[4];
	MEON		RTDescriptionString[1];
}RTAG_HANDLE;

ECB		*CMSMReturnSpareECB(
				const struct _DRIVER_DATA_	*driverData,
				ECB								*ecb);

UINT32		ReturnMessageInformation(
							MODULE_HANDLE *moduleHandle,
							UINT8		 ***messageTable,
							UINT32	 *messageCount,
							UINT32	 *languageID,
							UINT8		 **helpFile);

UINT32      GetNLMVersionInfo(
               MODULE_HANDLE     *moduleHandle,
               UINT32            *majorVersion,
               UINT32            *minorVersion,
               UINT32            *revision,
               UINT32            *year,
               UINT32            *month,
               UINT32            *day,
               MEON              *copyright);

typedef	struct _ASM_Strings_
{
      UINT8          Asm_CardName [100];
      UINT8          Asm_ShortName [10];
      UINT8          Asm_FrameStrg [40];
}
ASM_STRINGS;


extern   AsmAES_Glue ();
extern   Asm_ShutdownAES_Glue ();

extern   AsmDriverAES ();
extern   AsmDriverCallBack ();
extern   AsmDriverISR ();
extern   AsmDriverMulticastChange ();
extern   AsmDriverPoll ();
extern   AsmDriverReset ();
extern   AsmDriverSend ();
extern   AsmDriverShutdown ();
extern   AsmDriverTxTimeout ();
extern   AsmDriverPromiscuousChange ();
extern   AsmDriverStatisticsChange ();
extern   AsmDriverLookAheadChange ();
extern   AsmDriverManagement ();
extern   AsmDriverEnableInterrupt ();
extern   AsmDriverDisableInterrupt ();
extern   AsmDriverISR2 ();
extern   AsmDriverPriorityQueue ();
extern   AsmDriverDisableInterrupt2 ();

/************************************************************************/
/* Client 32 Section																		*/
/************************************************************************/

#ifdef OS_NIOS

#define	MIN_BOARD_NUMBER 0

#include <nios.h>
#include <nstdlib.h>

modHandle  NiosGetAddressOwner(void *address);
void	CancelInterruptTimeCallBack(void);
void	CRescheduleLast(void);
void	ScheduleInterruptTimeCallBack(void);

#define CMSMmemcmp(ptr1,ptr2,len) NiosMemCmp((ptr1),(ptr2),(len))
#define CMSMmemcpy(dst,src,len) NiosMemCpy((dst),(src),(len))
#define CMSMmemset(ptr,val,len) NiosMemSet((ptr),(val),(len))

#define CYieldWithDelay() CRescheduleLast()
#define Disable() NiosCli()
#define DisableAndRetFlags() NiosPushfdCli()
#define Enable() NiosSti()
#define RetFlags() NiosPushfd()
#define SetFlags(Flags) NiosPopfd((Flags))
#define GetServerPhysicalOffset() (void *)NiosGetPhysLinearStart()
#define CMSMReturnMessageInformation(	\
							moduleHandle,	\
							messageTable,	\
							messageCount,	\
							languageID,		\
							helpFile)		\
					ReturnMessageInformation( \
							(moduleHandle),	\
							(messageTable),	\
							(messageCount),	\
							(languageID),		\
							(helpFile))
#define CMSMGetNLMVersionInfo(	\
               		moduleHandle,	\
							majorVersion,	\
               		minorVersion,	\
      					revision,		\
							year,	   		\
               		month,   		\
               		day,	   		\
               		copyright)		\
				GetNLMVersionInfo( \
               		(moduleHandle),	\
					(majorVersion),	\
               		(minorVersion),	\
               		(revision),		\
					(year),			\
               		(month),		\
               		(day),			\
               		(copyright))		

/*****************************************************************************
*
*	void	CCancelInterruptTimeCallBack(struct _INT_AES_  *TimerNode);
*
* #include <nios.h>
*
* Cancels a callback event.
*
* This is an inline function.
*
* Returns:	Nothing
*
* See Also:
*
****************************************************************************/
#ifdef _WATCOMC_
void	CCancelInterruptTimeCallBack(struct _INT_AES_  *TimerNode);
#pragma aux CCancelInterruptTimeCallBack=		\
  "pushad"                         \
  "call CancelInterruptTimeCallBack" \
  "popad"                         \
	parm [edx]											\
	modify exact [edx];
#endif
/*****************************************************************************
*
*	void	CScheduleInterruptTimeCallBack(struct _INT_AES_  *TimerNode);
*
* #include <nios.h>
*
* Schedules a callback event.
*
* This is an inline function.
*
* Returns:	Nothing
*
* See Also:
*
****************************************************************************/
#ifdef _WATCOMC_
void	CScheduleInterruptTimeCallBack(struct _INT_AES_  *TimerNode);
#pragma aux CScheduleInterruptTimeCallBack=		\
  "pushad"                          \
  "call ScheduleInterruptTimeCallBack" \
  "popad"                           \
	parm [edx]											\
	modify exact [edx];
#endif
/*****************************************************************************
*
* UINT32	GetFileServerMinorVersionNumber(void)
*
* Returns Minor Version.
*
* This is an inline function.
*
* Returns:	Nothing
*
* See Also:
*
****************************************************************************/
#ifdef _WATCOMC_
UINT32	GetFileServerMinorVersionNumber(void);
#pragma aux GetFileServerMinorVersionNumber=		\
	"mov	eax, 0";
#endif
/*****************************************************************************
*
* UINT32	GetFileServerMajorVersionNumber(void)
*
* Returns Major Version.
*
* This is an inline function.
*
* Returns:	Nothing
*
* See Also:
*
****************************************************************************/
#ifdef _WATCOMC_
UINT32	GetFileServerMajorVersionNumber(void);
#pragma aux GetFileServerMajorVersionNumber=		\
	"mov	eax, 4";
#endif

#ifdef _WATCOMC_
void EnterDebugger (void);
#pragma aux EnterDebugger=	\
	"int	1";
#endif

#endif

/************************************************************************/
/* NetWare Server Section																*/
/************************************************************************/

#ifdef SVR

#define	MIN_BOARD_NUMBER 1

void        CYieldWithDelay (void);
UINT32      DisableAndRetFlags(void);
void        Enable(void);
void        *GetServerPhysicalOffset(void);
void        Disable (void);
UINT32      RetFlags(void);
void        SetFlags(UINT32 flag);

UINT32      CCmpB(
               void              *address1,
               void              *address2,
               UINT32            numberOfBytes);

void        CMovB(
               void              *sourceAddress,
               void              *destinationAddress,
               UINT32            numberOfBytes);

void        CSetB(
               UINT8             value,
               void              *address,
               UINT32            numberOfBytes);


#define CMSMmemcmp(ptr1,ptr2,len) CCmpB((ptr1),(ptr2),(len))
#define CMSMmemcpy(dst,src,len) CMovB((src),(dst),(len))
#define CMSMmemset(ptr,val,len) CSetB((val),(ptr),(len))

#define CMSMReturnMessageInformation(	\
		moduleHandle,	\
		messageTable,	\
		messageCount,	\
		languageID,	\
		helpFile)	\
	ReturnMessageInformation( \
		(moduleHandle),	\
		(messageTable),	\
		(messageCount),	\
		(languageID),	\
		(helpFile))

#define CMSMGetNLMVersionInfo(	\
               		moduleHandle,	\
					majorVersion,	\
               		minorVersion,	\
            		revision,		\
					year,	   		\
               		month,   		\
               		day,	   		\
               		copyright)		\
				GetNLMVersionInfo( \
               		(moduleHandle),	\
					(majorVersion),	\
               		(minorVersion),	\
               		(revision),		\
					(year),			\
               		(month),		\
               		(day),			\
               		(copyright))		

#endif
#endif /* ifndef OS_NT */

/************************************************************************/
/* NT Client Section 																	*/
/************************************************************************/

#ifdef OS_NT

/* Version information stored in LoadedModuleStruct.version. */
/* Information found after the C32 load header "VeRsIoN#" stamp. */

typedef int INT32;

typedef	struct _VERSION_INFO_
{
	UINT32   majorVersion;
	UINT32   minorVersion;
	UINT32   revision;
	UINT32   year;
	UINT32   month;
	UINT32   day;
} VERSION_INFO;

/* Debug Record List node structure.  Lists are always sorted according */
/* to public address. */

typedef	struct _DEBUG_RECORD_
{
	struct	_DEBUG_RECORD_	*link;
	PMEON    					debugName;
	UINT32   					address;
} DEBUG_RECORD;

/* moduleHandle.loadAttributes bit definitions. */
#define	MODULE_REENTRANT_FLAG		0x00000001
#define	MODULE_MULTIPLE_LOAD_FLAG	0x00000002
#define	MODULE_CANNOT_UNLOAD			0x20000000
#define	MODULE_DEBUG_VERSION			0x40000000


#define		CMSMReturnSpareECB(driverData, ecb) ecb

#ifdef _WATCOMC_
void CMSMmemcpy(
            void                 *destAddrPtr,
            void                 *srcAddrPtr,
            UINT32               count);

#pragma aux CMSMmemcpy parm [EDI] [ESI] [ECX]

#pragma aux CMSMmemcpy = \
   0xF3 0xA4                           /* rep   movsb                     */ \
   parm [EDI] [ESI] [ECX] \
   modify exact [ESI EDI ECX];

void CMSMmemset(
         void                    *memAddrPtr,
         UINT8                   value,
         UINT32                  count);

#pragma aux CMSMmemset parm [EDI] [EAX] [ECX];

#pragma aux CMSMmemset = \
   0xF3 0xAA                           /* rep   stosb                   */ \
   parm [EDI] [EAX] [ECX] \
   modify exact [EDI ECX];

/* end of _WATCOMC_ */
#else
#define CMSMmemcpy(_dest, _src, _nbytes)			\
	RtlMoveMemory((_dest), (_src), (_nbytes))

#define CMSMmemset(_dest, _fill, _nbytes)			\
	RtlFillMemory((_dest), (_nbytes), (_fill))
#endif

void CMSMBusTypeToNTBusType(UINT32 *busType);

UINT32 _cdecl CMSMReturnMessageInformation(
							MODULE_HANDLE *moduleHandle,
							UINT8 	***messageTable,
							UINT32	 *messageCount,
							UINT32	 *languageID,
							UINT8		 **helpFile);

UINT32 _cdecl CMSMGetNLMVersionInfo(
             	  MODULE_HANDLE     *moduleHandle,
               	UINT32            *majorVersion,
	               UINT32            *minorVersion,
   	            UINT32            *revision,
      	         UINT32            *year,
         	      UINT32            *month,
            	   UINT32            *day,
               	MEON              *copyright);

ECB	*  _cdecl CLSLGetSizedECB(UINT32 EcbDataSize, void *pResourceObj,
                  BOOLEAN below16);

ODISTAT _cdecl CLSLReturnECB(ECB *ecb);

ODISTAT _cdecl CLSLCancelEvent(ECB *ecb);

void  _cdecl CLSLScheduleAESEvent(AES_ECB *aesEcb);

ODISTAT _cdecl CLSLCancelAESEvent(AES_ECB *aesEcb);

UINT32   _cdecl CLSLGetIntervalMarker();

LSL_CONFIG_TABLE	*  _cdecl CLSLGetLSLConfiguration();

UINT32   _cdecl CLSLStartCriticalSection(UINT32 BoardNumber);

UINT32   _cdecl CLSLEndCriticalSection(UINT32 BoardNumber);

UINT32   _cdecl CLSLGetCriticalSectionStatus();

ODISTAT _cdecl CLSLGetStackECB(LOOKAHEAD	*LookAheadBuf);

void	_cdecl CLSLFastSendComplete(ECB *ecb);

void	_cdecl CLSLSendComplete(ECB *ecb);

void	_cdecl CLSLHoldEvent(ECB *ecb);

void	_cdecl CLSLFastHoldEvent(ECB	*ecb);

UINT32   _cdecl CLSLGetMaxECBBufferSize();

void	_cdecl CLSLServiceEvents();

ODISTAT _cdecl CLSLControlStackFilter(
            UINT32   BoardNumber,
            UINT32   Function,
            UINT32   Mask,
            void    	*Parameter1,
            void    	*Parameter2);

#endif
