/***********************************************************************\
*
*   (c) Unpublished Work of Novell, Inc.  All rights reserved.
*
*   THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*   PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.  ACCESS
*   TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
*   NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
*   AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO
*   APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED,
*   PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED,
*   TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED,
*   LINKED,RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
*   CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
*   THE PRIOR WRITTEN CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF
*   THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
*   CRIMINAL AND CIVIL LIABILITY.
*                                                                      
***********************************************************************
*
*  Name:          TOKENTSM.h
*
*  Description:   C version of the Token-Ring Structures
*
*  ODI Spec Ver   1.11
*
*  Modification History:
*
*  08-29-96 WTT		Added proto type for CTokenTSMGetConfigInfo.
*
*  12-05-96 WTT		Changed proto types for MediaAddMulticast, 
*					MediaDeleteMulticast and FindAddressInMCTable to 
*					conform to spec.  They were using UINT8 pointer and 
*					should have been using NODE_ADDR pointer for
*					the mcAddress.
*					SPD #143898
*
\***********************************************************************/
#ifndef TOKENTSM_H

/**************************************************************************\
**
**      Defines
**
\**************************************************************************/
/*These are used for indexing into a specific Frame-Type.		*/
#define  TOKEN_INTERNALID  0
#define  TSNAP_INTERNALID  1

/*Brouter Request Definitions.                           		*/
#define  BROUTER_SUPPORT               0x00
#define  SELECT_T_BRIDGING             0x01
#define  SELECT_SR_BRIDGING            0x02
#define  SELECT_PROMISCUOUS_MODE       0x03
#define  UPDATE_ADDRESS_FILTER_TABLE   0x04

#define  PMT_NONE                      0
#define  PMT_TRANSPARENT               1
#define  PMT_SOURCE_ROUTINE            2
#define  PMT_TSR_TRANSPARENT           3

#define  FTA_CLEAR_TABLE               0
#define  FTA_ADD_ADDRESSES             1
#define  FTA_DELETE_ADDRESSES          2

/*These are the NOVELL assigned Frame IDs.					*/

#define	TOKEN_ID          4
#define	TSNAP_ID          11

/* This defines the size of each Frame-Types MAC Header.	*/
#define	SIZE_TOKEN			17
#define	SIZE_TOKENSNAP		22
#define  SIZE_MACTOK       14

/*These are used for supporting Assembly HSMs.					*/
#define	TOKEN_DESCRIPT_LEN 10
#define	TSNAP_DESCRIPT_LEN 15

/* This is the Maximum size a header can be	*/
#define MAX_MAC_HEADER    sizeof(TOKEN_MEDIA_HEADER) + SOURCE_MAX_SIZE /* SNAP hdr is max  */

/* This is the Maximum size a packet can be	*/
#define MAX_PACKET_LENGTH	17960 - 6

#define MULTICASTBIT			0x80
#define GROUPBIT				0x80
#define LOCALBIT				0x40
#define BROADCAST				0xFFFFFFFF

/* These bits define 802.2 types.				*/
#define S_OR_U_FORMAT		0x01
#define U_FORMAT				0x02

/* TxAccessControl Equates                                                */
#define  AC_PRIORITY_MASK        0xE0     /* AC E0 Priority Ind Mask      */
#define  AC_BUSY_FRAME           0x10     /* AC 10 1==> Busy Frame        */
#define  AC_ACTIVE_MONITOR       0x08     /* AC 08 1==> Active Monitor    */
#define  AC_RESERVATION_MASK     0x07     /* AC 07 Priority Res Mask      */

/* TxFrameControl Equates                                                 	*/
#define	FC_FRAME_TYPE_BITS	0xC0		/* FC C0 ==> Frame Type bits.			*/
#define  FC_MAC_FRAME			0x00     /* FC 00 0==> MAC Fame					*/
#define  FC_LLC_FRAME			0x40		/* FC 40 1==> LLC Frame					*/
#define	FC_UNDEF_FRAME			0xC0		/* FC C0 1X ==> Undefined/Reserved 	*/
												/* FC 30 xxRRxxxx RR==> Reserved 		*/

						/* FC 0F Control Bits, Valid for MAC frames.
						   For NON-MAC frames should be ignored.    */
#define  FC_RING_POLL_RESPONSE   0x06     /* FC 06 Ring Poll Response		*/
#define  FC_RING_POLL_REQUEST    0x05     /* FC 05 Ring Poll Request			*/
#define  FC_RING_PURGE           0x04     /* FC 04 Ring Purge					*/
#define  FC_MONITOR_CONTENTION   0x03     /* FC 03 Monitor Contention		*/
#define  FC_BEACON               0x02     /* FC 02 Beacon						*/
#define  FC_EXPRESS_MAC          0x01     /* FC 01 Xpress MAC					*/
#define  FC_MAC_FIELD_MASK       0x0F     /* FC 0F MAC Field Mask				*/

/* TxRoutingInfo -- 14 Routing Information Field EQUates                  */

#define  SOURCE_DIRECTION_IND    0x80     /*    Direction Indicator       */
#define  SOURCE_ROUTING_BIT      0x80     /* 80 TxSourceNodeAddress Bit   */
#define  SOURCE_GENERAL_BROAD    0x80     /* 80 General Broadcast  Mask   */
#define  SOURCE_LIMITED_BROAD    0xC0     /* C0 Limited Broadcast  Mask   */
#define  SOURCE_LARGEST_FRAME    0x70     /* 70 Largest Frame Size Maks   */
#define  SOURCE_SIZE_MASK        0x1F     /*    Routing Size Mask         */
#define	SOURCE_MAX_SIZE			30			/*  Max Source Route field size */

#define SNAP_INFO_SIZE		5

/*v3.00 Assembly Statistics Table */
#define STATISTICSMASK		0x0D380000
#define TOKENSTATSCOUNT		13
#define LONGCOUNTER			0x00
#define LARGECOUNTER			0x01

#define MAX_MULTICAST		32

/* ECB_DataLength bit assignmets for pipeline and ecb aware adapters. */
#define HSM_HAS_BEEN_IN_GETRCB	0x80000000		/* bit 31 */
#define PSTK_WANTS_FRAME			0x40000000		/* bit 30 */
#define HSM_TOLD_TO_SKIP			0x007f0000		/* bits 23 through 16 */
#define HSM_TOLD_TO_COPY			0x0000ffff		/* bits 15 through 0 */

/**************************************************************************\
**
**      TypeDefs
**
\**************************************************************************/
/*Brouter Request Type Definitions.                      		*/

typedef struct	_HUB_ECB_
{
	struct   _HUB_ECB_   *HECB_NextLink;
	struct   _HUB_ECB_   *HECB_PreviousLink;
	UINT16               HECB_Status;
	void                 (*HECB_ESR)(struct _HUB_ECB_ *);
	UINT16               HECB_StackID;
	UINT8                HECB_ProtocolID [6];
   UINT16               HECB_BrouterRequestCode;
}
HUB_ECB;

typedef struct	_HUB_BR_ECB_
{
   HUB_ECB     BR_ECB;
   void        *BR_BrouterSupportStatus;
   UINT8       BR_FilterTableSize;
}
HUB_BR_ECB;

typedef struct	_HUB_TB_ECB_
{
   HUB_ECB     TB_ECB;
   void        *TB_ServiceHandler;
}
HUB_TB_ECB;

typedef struct	_HUB_SR_ECB_
{
   HUB_ECB     SR_ECB;
   void        *SR_ServiceHandler;
   UINT8       SR_PartitionSize;
   UINT8       SR_FilterSTE;
   UINT16      SR_MaximumRDSize;
   UINT16      SR_RingInNumber;
   UINT16      SR_RingOutNumber;
   UINT16      SR_BridgeNumber;
}
HUB_SR_ECB;

typedef struct	_HUB_PM_ECB_
{
   HUB_ECB     PM_ECB;
   void        *PM_PromiscuousModeType;
}
HUB_PM_ECB;

typedef struct	_HUB_UA_ECB_
{
   HUB_ECB     UA_ECB;
   UINT8       UA_FilterTableAction;
   UINT8       UA_FilterTableAddresses;
   UINT8       UA_FilterTableAddress1;
}
HUB_UA_ECB;

typedef	struct	_FUNCTIONAL_TABLE_
{
	UINT8		FunctionalBit[32];
	UINT32	ActiveFunctionalBits;
}
FUNCTIONAL_TABLE, *PFUNCTIONAL_TABLE;

typedef struct	_M_ADAPTER_DS_	
{
	UINT32	MADS_MulticastCount;
}
M_ADAPTER_DS;

#define	NUMBER_OF_PROMISCUOUS_COUNTERS	32

typedef struct	_MEDIA_DATA_	{
	UINT32					MaxMulticastAddresses;
	GROUP_ADDR_LIST_NODE	*MulticastAddressList;
	UINT32					MulticastAddressCount;
   FUNCTIONAL_TABLE		FunctionalAddressList;
	TCB						*TCBHead;    
	TCB						*TCBList;    
	MLID_STATS_TABLE		*NewStatsPtr;
	UINT32					PromiscuousMode;
	UINT8						PromiscuousCounters[NUMBER_OF_PROMISCUOUS_COUNTERS];
	UINT32					RxStatus;
} MEDIA_DATA;

/**************************************************************************\
**
**      Prototype Definitions
**
\**************************************************************************/
UINT32   CInitializeTokenTSM (
            struct _MODULE_HANDLE_    *moduleHandle,
            SCREEN_HANDLE    *screenID);

void     CRemoveTokenTSM ( void );

ODISTAT	CTokenTSMRegisterHSM(
				DRIVER_PARM_BLOCK	   *DriverParameterBlock,
				CONFIG_TABLE			**configTable);

ODISTAT FindAddressInMCTable(
				SHARED_DATA			   *sharedData,
				NODE_ADDR				*mcAddress,
				GROUP_ADDR_LIST_NODE **mcEntry);

ODISTAT	MediaAdjust(
				FRAME_DATA				*frameData);

ODISTAT	MediaInit(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData);

ODISTAT	MediaReset(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				OPERATION_SCOPE		operationScope);

ODISTAT	MediaShutdown(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					shutdownType,
				OPERATION_SCOPE		operationScope);

void	   MediaSend(
				ECB						*ecb,
				CONFIG_TABLE			*configTable);

ODISTAT	MediaAddMulticast(
				DRIVER_DATA			   *driverData,
				NODE_ADDR				*McAddress);

ODISTAT	MediaDeleteMulticast(
				DRIVER_DATA			   *driverData,
				NODE_ADDR				*McAddress);

ODISTAT	MediaNodeOverride(
				FRAME_DATA				*frameData,
				MEON						mode);


ODISTAT	MediaAdjustNodeAddress(
				FRAME_DATA				*frameData);


ODISTAT	MediaSetLookAheadSize(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					size);

ODISTAT	MediaPromiscuousChange(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					PromiscuousState,
				UINT32					*PromiscuousMode);

ODISTAT	MediaRegisterMonitor(
				FRAME_DATA				*frameData,
				void						(*TXRMonRoutine)(TCB *),
				BOOLEAN					MonitorState);

ODISTAT	MediaGetParameters(
            CONFIG_TABLE      *configTable);

ODISTAT	MediaGetMulticastInfo(
             DRIVER_DATA    *driverData,
             ECB            *MulticastInfoECB);

ODISTAT  CTokenTSMUpdateMulticast(
				DRIVER_DATA			   *driverData);

UINT32   CMediaSend8022(
            SHARED_DATA *sharedData,
				ECB         *ecb,
				TCB         *tcb);
	
UINT32   CMediaSendSNAP(
            SHARED_DATA *sharedData,
				ECB         *ecb,
				TCB         *tcb);
	
TCB		*CTokenTSMGetNextSend(
				DRIVER_DATA			   *driverData,
				CONFIG_TABLE			**configTable,
				UINT32					*lengthToSend,
				void						**TCBPhysicalPtr);

void		CTokenTSMFastSendComplete(
				DRIVER_DATA			   *driverData,
				TCB						*tcb,
            UINT32               transmitStatus);

void		CTokenTSMSendComplete(
				DRIVER_DATA			   *driverData,
				TCB						*tcb,
            UINT32               transmitstatus);

RCB 		*CTokenTSMProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize);

RCB 		*CTokenTSMFastProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize);

RCB		*CTokenTSMGetRCB(
				DRIVER_DATA          *driverData,
				UINT8                *packetHdr,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32 					*startByte,
				UINT32 					*numBytes);

void		CTokenTSMFastRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb);

void		CTokenTSMFastRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus);

UINT32	CTokenTSMGetHSMIFLevel();

void		CTokenTSMRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb);

void		CTokenTSMRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus);

void		CTokenTSMECBRcvComplete(
				DRIVER_DATA				*driverData,
				ECB 						*ecb);

void		CTokenTSMPipelineRcvComplete(
				DRIVER_DATA				*driverData,
				ECB 						*ecb,
				UINT32					packetLength,
				UINT32					packetStatus);

ODISTAT CTokenTSMBuildTransmitControlBlock(
						DRIVER_DATA		*driverData,
						ECB				*ecb,
						TCB				**tcb,
						UINT32			*pktSize);


void CTokenTSMCancelPrioritySend(
							DRIVER_DATA		*driverData,
							ECB		  	*ecb);

ODISTAT	CTokenTSMGetConfigInfo (
         void 	*configInfo,
         UINT32 *nBytes);

#ifdef IAPX386
void	BuildASMStatStrings(
				StatTableEntry		   *tableEntry);
#endif


#define TOKENTSM_H
#endif /* TOKENTSM_H */

