/**************************************************************************\
*
*  Copyright Unpublished Work of Novell, Inc. All Rights Reserved
*
*  THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*  PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.  ACCESS
*  TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
*  NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
*  AND (II) ENTITIES OTHER THAN NOVELL WHO HAVE ENTERED INTO
*  APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS WORK MAY BE USED,
*  PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED, MODIFIED,
*  TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED, COMPILED,
*  LINKED,RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
*  CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
*  THE PRIOR WRITTEN CONSENT OF NOVELL.  ANY USE OR EXPLOITATION OF
*  THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
*  CRIMINAL AND CIVIL LIABILITY.
*
\*************************************************************************/
/**************************************************************************\
*
* Title:   		C Toke-Ring Topology Module
*
* Filename:		TOKENTSM.C
*
* ODI Spec Ver:	1.11
*
* Description:	This module is the C version of TOKENTSM.NLM.  It is
*               single sourced for C_ODI & Windows NT.  
*
* Modification History:
*
* 6-18-96 TNL 	Fix bug in CTokenTSMBuildTransmitControlBlock where
*				BitSwapAddressFlag not being placed in TCB_DriverWS[0].
*				SPD 129918.	
*
* 7-22-96 TNL 	Modified CTokenTSMProcessGetRCB to increment RxTooSmall counter
*				instead of RxMiscError in one case. SPD 131116.
*
* 8-09-96 TNL 	Modified CTokenTSMPipelineRcv Complete to check for
*				RxTooSmall. SPD 132063.
*
* 8-20-96 TNL	Modified TSM(Fast)SendComplete code to not increment Tx
*				statistics counters when transmitStatus is non-zero.
*				SPD #133048, 133062, and 133074.
*
* 8-29-96 WTT	Added CTokenTSMConfigTable and modified CInitializeTokenTSM
*				to set up the module version fields in the table.  Also added
*				the CTokenTSMGetConfigInfo API.
*
* 10-29-96 MPK	Added code to MediaShutdown, to diferentiate between a standard
*				Shutdown, and one caused by an HSM call to
*				CMSMShutdownMLID, who in turn calls MediaShutdown
*
* 11-11-96 KRA	Added code for remote multicast in MediaPromiscuousChange.
*
* 11-12-96 WTT	CTokenTSMProcessGetRCB was blowing away the
*				ECB_Fragment[0].FragmentLength field in the ECB when it was
*				building the look ahead buffer.
*				SPD #142351
*
* 11-15-96 WTT	Added code to CTokenTSMGetRCB check and make sure that an
*				ecb was returned in the lookahead structure when
*				CLSL_GetStackECB returned ODISTAT_SUCCESSFUL before tring to
*				use the pointer.
*				SPD #137523
*
* 11-19-96 KRA	In MediaReset and MediaShutdown, if there are ecbs queued up
*				and returned to the LSL, decrement the Qeueu depth counter.
*
* 12-05-96 WTT	Changed MediaAddMulticast, MediaDeleteMulticast and 
*				FindAddressInMCTable to conform to spec.  They were using 
*				UINT8 pointer and should have been using NODE_ADDR pointer 
*				for	the mcAddress.
*				SPD #143898
*
* 12-19-96 WTT	Reversed the change made to CEtherTSMProcessGetRCB on 11-12-96.
*				for SPD#142351.  According to specification and code the 
*				adjustment to the length field needs to be made for HSMs that 
*				pre-allocate RCBs.  To fix the side effects of doing this 
*				CLSL_ReturnECB will be changed to restore the fragment 
*				address and length fields before adding the ECB back to the 
*				ECB pool it came from.
*
* 01-02-97 WTT	Changed MediaSend to check shutdown bit for logical board 
*				instead of adapter.
*
* 01-03-96 WTT	Changed CTokenTSMProcessGetRCB and CTokenTSMGetRCB to check 
*				shutdown bit for logical board and set rcvStatus to
*				PAE_NOT_ENABLED_BIT if the logical board is shutdown.
*
* 01-07-97 MPK	Changed MediaAddMulticast, MediaDeleteMulticast and 
*				FindAddressInMCTable to access the the NODE_ADDR pointer
*               with proper casting.  A correction to work done 12-05-96
*
* 01-10-97 MPK	Changed CTokenTSMGetRCB and CTokenTsmProcessGetRCB so that
*		the case of a clear Functional/Non-Functional bit in the
*               header, will require the I/G and funct/Non-funt bits to be set.
*
* 01-14-97 MPK	Changed CTokenTSMBuildTransmitControlBlock so that ECB 
*			   	Drivers will return quicker, as they don't need this process.
*				However, I will provide Physical addresses to those ECB
*				Aware adapters requiring them.
*				SPD #146442
*
* 01-17-97 MPK	Changed CancelPrioritySEND, since BuildTransmitControlBlock is now
*			   	Responsible for making the ECB physical (if necessary,
*				the checking for Physical and return Physical is no
*				longer required in CancelPrioritySend, since the ECB will
*				never have been made physical before calling CancelPrioritySend.
*
* 03-07-97 MPK	Changed MediaNodeOverride to check for the proper range value of the 
*			   	NOdeAddress Override to make sure we are not over or undersize.
*				Also to report a message and reset to unknown value if so.
*				SPD 151819
*
* 07-29-97 WTT	With the recent changes to LSLC32 and CMSM for the SPDs listed 
*				below was able to remove the check to see if we were using 
*				a spare ECB and the subsquent call to CMSMReturnSpareECB in 
*				CTokenTSMProcessGetRCB.
*				SPD 162312, 146492
*
* 08-18-97 WTT	Added driverOption structure for LSB Keyword and changed 
*				MediaGetParameters to check the structure to see if LSB
*				was present on the command line.  This change was neccessary
*				due to changes changes made in the CMSM's parser code for
*				consistency accross platforms.
*				SPD 164110
*
* 08-29-97 WTT	Changed the code to use the new define NON_STACKID_RAW_SEND_MASK
*				when checking to see if we have a raw send.
*	
* 09-05-97 WTT	For priority packets changed CMediaSend8022 and CMediaSendSNAP 
*				to take the priority level from the ECB's StackID field and 
*				place it in the AC field of the MAC header.
*				SPD 162279
*
* 09-05-97 WTT	Fixed CTokenTSMProcessGetRCB, CTokenTSMGetRCB, and 
*				CTokenTSMECBRcvComplete to set the packets priority level
*				in LkAhd_PriorityLevel before calling CLSL_GetStackECB.
* 
* 09-05-97 WTT	The fix made initially for SPD 162279 did not take into
*				account the fact that the priority level defined in 
*				the ECB's StackID field is opposite of what is to
*				be placed in the AC field of the MAC header.  For example the
*				last three bits of a priority 7 (hightest priority) packet in 
*				the StackID field are all zeros and highest priority in the 
*				the AC field the bits are all ones.
*
* 09-10-97 WTT	CTokenTSMGetRCB failed to set ECB_RX_PRIORITY in LKAnd_DestType
*					for priority packets.
*
* 10-14-97 WTT	Changed CTokenTSMProcessGetRCB and CTokenTSMGetRCB so that 
*					MAC frames which the source address matches the adapters node 
*					address are not filtered out.  Before this change all frames 
*					in which the source address matches the adapters node address
*					were filtered out.  This prevented Applications from using the 
*					protocol stack to send Token-Ring MAC frames (such as Request
*					Ring Station Address) to the adapter, expecting a repy frame 
*					from the adapter's MAC layer.
*					SPD 167919
*
* 11-10-97 WTT	Changed CTokenTSMGetRCB to check for exclusive ECB_MACFRAME 
*					destination bit before setting ECB_RX_PRIORITY bit.
*					SPD 170028.
*
* 11-14-97 WTT	Changed CTokenTSMProcessGetRCB to check for exclusive ECB_MACFRAME 
*					destination bit before setting ECB_RX_PRIORITY bit.
*					SPD 171315.
* 
\**************************************************************************/

/* Fake type definition to keep .h files happy. _DRIVER_DATA_ is defined	*/
/* in the HSM and not accessed directly by the MSM or TSM.					*/

typedef	struct	_DRIVER_DATA_	DRIVER_DATA;

#define	_cdecl		

#include <portable.h>
#include <odi.h>
#include <odi_nbi.h>
#include	<parser.h>
#include <cmsm.h>
#include <tokentsm.h>
#include <cmsmstr.h>
#include <splat.h>
#include <tokentsm.txt>

/**************************************************************************\
**
**    Global declarations
**
\**************************************************************************/
MEON_STRING	CODISPEC[] = TOKENTSM_ODI_SPECVER_MSG;
MEON		**DriverMessages = NULL;

/* TokenDescriptionLen and TokenSNAPDescriptLen supplied for	*/
/*	CMSMOldParseDriverParameters.  Order is critical.				*/

static UINT8 Token8022Index       = TOKEN_INTERNALID;
static UINT8 TokenDescriptionLen  = TOKEN_DESCRIPT_LEN;
static MEON  TokenDescription  [] = { TOKENTSM_FRAME_TOKEN_RING_TXTMSG };
static UINT8 TokenSNAPIndex       = TSNAP_INTERNALID;
static UINT8 TokenSNAPDescriptLen = TSNAP_DESCRIPT_LEN;
static MEON  TokenSNAPDescript [] = { TOKENTSM_FRAME_SNAP_TXTMSG };

static MEON  MACToken[] = { TOKENTSM_MACTOK_TXTMSG };

MEON    *TokenTSMFrameDescriptTable[] =
{
	TokenDescription,
	TokenSNAPDescript
};

static UINT8 MinAddress[]  = { 1, 0, 0, 0, 0, 0 };
static UINT8 MaxAddress[]  = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE };

static TSM_CONFIG_LIMITS TokenTSMConfigLimits =
{
   (UINT8 *)&MinAddress,	/* MinNodeAddress */
   (UINT8 *)&MaxAddress,   /* MaxNodeAddress */
   0,                      /* MinRetries     */
   255,                    /* MaxRetries     */
   2,                      /* NumberFrames   */
};

static PROT_ID TokenTSMProtocolIDArray [] =
{
	0, 0, 0, 0, 0, 0xE0,						/* Token-Ring        */
	0, 0, 0, 0, 0x81, 0x37,             /* Token-Ring SNAP   */
};

static UINT8 TokenTSMIDArray         [] = { TOKEN_ID, TSNAP_ID };
static UINT8 TokenTSMHeaderSizeArray [] =
{
   sizeof (TOKEN_MEDIA_HEADER) - 5,
   sizeof (TOKEN_MEDIA_HEADER),
   0,
   0
};

UINT32   (*TokenTSMSendRoutineArray []) (ECB *, TCB *) =
{
	(UINT32 (*)())CMediaSend8022,             /* 802.2 over 802.5			*/
	(UINT32 (*)())CMediaSendSNAP,             /* SNAP  over 802.5			*/
	(UINT32 (*)())CMediaSend8022,             /* Error Case Initialized	*/
	(UINT32 (*)())CMediaSend8022              /* Error Case Initialized	*/
};

GROUP_ADDR_LIST_NODE	MCPlaceHolder;

MEDIA_DATA MediaDataTemplate =
{
	MAX_MULTICAST,				/* MaxMulticastAddresses		*/
	&MCPlaceHolder,			/* MulticastAddressList			*/
	0,								/* MulticastAddressCount		*/
   {0},							/* FunctionalAddressList      */
	0,								/* TCBHead							*/
	0,								/* TCBList							*/
	0,								/* NewStatsPtr						*/
	0,								/* PromiscuousMode				*/
	{0},							/* PromiscuousCounters			*/
	0,								/* RxStatus							*/
};

static TSM_PARM_BLOCK MediaParameters =
{
   sizeof (TSM_PARM_BLOCK),							/* MediaParameterSize       */
   &TokenTSMConfigLimits,								/* MediaConfigLimits        */
   (MEON **)&TokenTSMFrameDescriptTable,			/* MediaFrameDescriptTable  */
   (UINT8 (*)[])&TokenTSMProtocolIDArray,			/* MediaProtocolIDArray     */
   TokenTSMIDArray,										/* MediaIDArray             */
   TokenTSMHeaderSizeArray,							/* MediaHeaderSizeArray     */
   (UINT32 (**)())&TokenTSMSendRoutineArray,		/* MediaSendRoutineArray    */
	sizeof( MEDIA_DATA ),								/* MediaAdapterDataSpaceSize */
   &MediaDataTemplate,									/* MediaAdapterPtr          */
   MediaAdjust,											/* MediaAdjustPtr           */
   (ODISTAT (*)())MediaInit,							/* MediaInitPtr             */
   MediaReset,												/* MediaResetPtr            */
   MediaShutdown,											/* MediaShutdownPtr         */
   MediaSend,												/* MediaSendPtr             */
   (void *(*)())CTokenTSMGetNextSend,				/* MediaGetNextSendPtr      */
   MediaAddMulticast,									/* MediaAddMulticastPtr     */
   MediaDeleteMulticast,								/* MediaDeleteMulticastPtr  */
   MediaNodeOverride,									/* MediaNodeOverridePtr     */
   MediaAdjustNodeAddress,								/* MediaAdjustNodeOverridePtr*/
   MediaSetLookAheadSize,								/* MediaSetLookAheadSize    */
   MediaPromiscuousChange,								/* MediaPromiscuousChange   */
   MediaRegisterMonitor,								/* MediaRegisterMonitor		 */
   MediaGetParameters,									/* MediaGetParametersPtr	 */
   MediaGetMulticastInfo								/* MediaGetMulticastInfoPtr */
};

struct
{
   UINT32		OptionCount;
  	MEON_STRING	*StrOptVal[2];
} FrameOptionList = { 2, NULL};

DRIVER_OPTION  NodeOption =
{
   NULL,
   TOKENTSM_NODE_EQUAL_TXTMSG,
   0,
   0,
   0,
   CUSTOMPARAM,
   OPTIONALPARAM | STRINGPARAM,
   {'\0'}
};

DRIVER_OPTION	FrameOption =
{
	&NodeOption,
	TOKENTSM_FRAME_EQUAL_TXTMSG,
	(PARAMETER_OPTIONS *)&FrameOptionList,
	0,
	0,
	FRAMEPARAM,
	ENUMPARAM,
	{'\0'}
};

DRIVER_OPTION	LSBOption =
{
	&FrameOption,
	TOKENTSM_LSB_TXTMSG,
	0,
	0,
	0,
	CUSTOMPARAM,
	KEYWORDPARAM,
	{'\0'}
};


CTSM_CONFIG_TABLE CTokenTSMConfigTable =
{
	sizeof(CTSM_CONFIG_TABLE),	/* CTSMCFG_TableSize */
	CTSM_TABLE_MAJOR_VER,		/* CTSMCFG_TableMajorVersion */
	CTSM_TABLE_MINOR_VER,		/* CTSMCFG_TableMinorVersion */
	NULL,						/* CTSMCFG_ModuleMajorVersion - filled in at Init Time */
    NULL,						/* CTSMCFG_ModuleMinorVersion - filled in at Init Time */
    ODI_SPEC_MAJOR_VER,			/* CTSMCFG_ODISpecMajorVersion */
    ODI_SPEC_MINOR_VER,			/* CTSMCFG_ODISpecMinorVersion */
	NULL,						/* CTSMCFG_Reserved */
    MAX_PACKET_LENGTH,			/* CTSMCFG_MaxFrameSize - defined in TOKENTSM.H */
	NULL,						/* CTSMCFG_SystemFlags - filled in at Init Time */
};


/**************************************************************************\
**
**    Procedures
**
\**************************************************************************/

/**************************************************************************\
**
** PROC NAME:     MediaInit
**
**                This function initializes media specific variables of the
**                adapter data space.  It will build a v4.0 style statistics
**                table if the HSM is using a v3.0 table.  It will also
**                allocate memory for a multicast table, functional address
**                table, and a TCB pool.  This function is called by
**                CMSMRegisterMLID.
**
** Parameters:
**          driverData  -  Pointer to CHSMs adapter data space.
**          frameData   -  to get a Pointer to default configuration table.
**
** Return Value:
**				ODISTAT_SUCCESSFUL       - If media data space initialized
**          ODISTAT_OUT_OF_RESOURCES - If unable to allocate memory
**
**
\**************************************************************************/
ODISTAT	MediaInit(
						DRIVER_DATA 			*driverData,
						FRAME_DATA	   		*frameData)
{
   MEDIA_DATA						*mediaData;
   CONFIG_TABLE					*configTable;
   TCB								*tcbPtr;
	UINT32							i;
   UINT32							bufferSize;

#ifdef   IAPX386
		MSTATS							*statsTable;	/* Old ASM table */
		MLID_STATS_TABLE           *newStats;
		StatTableEntry					*tableEntry;
		UINT32							*customCounters, numCustomCounters;
		MEON								*customStrings;
		UINT32							bitMask, j;
#endif

	TokenDescriptionLen = TokenDescriptionLen;		/* resolve complier warnings.*/
	TokenSNAPDescriptLen = TokenSNAPDescriptLen;		/* resolve complier warnings.*/

	/* Setup Local Variables */
	configTable = &frameData->ConfigTable;

	/* Set the Maximum frame header size to SNAP + 18 for lookahead Data.*/
	DADSP_TO_CMSMADSP(driverData)->CMSMMaxFrameHeaderSize =
												MAX_MAC_HEADER + DEFAULT_LOOK_AHEAD_SIZE;
	mediaData = DADSP_TO_CMSMADSP(driverData)->CMSMMediaDataPtr;

#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
		/* Get Valid pointer to HSM Stat Table */
		statsTable = (MSTATS *) DADSP_TO_CMSMADSP (driverData)->CMSMStatsPtr;

	   /********************************************************************\
   	**
	  	**  We only Support version 3.00 of the Assembly Statistics Table
		**  Check the Statistics table Major Version for 3 as there are no 3.x
		**	  versions as of 9-26-95 (Assembly Spec Version 3.2 or 3.3)
	   **
	   \********************************************************************/
		if ( statsTable->MSTATS_MajorVer >= 0x03 )
		 {
			if ( statsTable->MSTATS_MajorVer == 0x03 )
			 {
				/**************************************************************\
	         **
   	      **  We have a valid OLD ASSEMBLY style Statistics Table.
      	   **  Set mask field for Valid 3.00 Table Stats.
         	**
	         \**************************************************************/
				statsTable->MSTATS_ValidMask0	&= STATISTICSMASK;

      	   /**************************************************************\
	         **
 				**  Allocate memory for Generic and, Media specific table entries.
 				**  We need 2 less than NumGenericCounters says because of the
         	**  64 bit counters in Assembly are 2 32 bit counters.
				**	 We need to remember to skip CounterMask1 which is not
				**  used in the C spec, but need to count it to have a skip.
      	   **
         	\**************************************************************/
	 			customCounters		= &statsTable->MSTATS_Generic.Gen_TotalTxPackets;
	 			customCounters   += statsTable->MSTATS_NumGenericCounters;
 				numCustomCounters = *(UINT16 *) customCounters;

				bufferSize = sizeof(MSTATS) + (sizeof(StatTableEntry) *
				 ((statsTable->MSTATS_NumGenericCounters - 2) + numCustomCounters));

				mediaData->NewStatsPtr =
							(MLID_STATS_TABLE *) CMSMAlloc( driverData, bufferSize );

				if (!mediaData->NewStatsPtr)
				{
					CMSMPrintString( configTable, MSG_TYPE_INIT_WARNING,
						  TOKENTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );

					return( ODISTAT_OUT_OF_RESOURCES );	/* Could not allocate Memory. */
				}

      	   /**************************************************************\
         	**
	         **  Overwrite StatsPtr with pointer to "C" style table.
				**	 As far as any one will know we are now a "C" driver...
      	   **
         	\**************************************************************/
	  			DADSP_TO_CMSMADSP(driverData)->CMSMStatsPtr = mediaData->NewStatsPtr;
   	      newStats = mediaData->NewStatsPtr;

      	   /**************************************************************\
         	**
	         **  Prepare the NEW "C" table header:
   	      **
	         \**************************************************************/
				newStats->MStatTableMajorVer  = 0x04;
				newStats->MStatTableMinorVer  = 0x00;
				tableEntry = (StatTableEntry *) (newStats + 1);
				newStats->MGenericCountsPtr   = tableEntry;
				newStats->MNumGenericCounters = MLID_Q_DEPTH + 1;

				/* Set up the Counter Strings */
				BuildASMStatStrings (tableEntry);

      	   /**************************************************************\
	        	**
	         **  Now prepare the generic and media counter fields.
   	      **
	         \**************************************************************/
				/* Which Stats are valid? */
				bitMask = statsTable->MSTATS_ValidMask0;

				for (i = 0, j = 0; i < statsTable->MSTATS_NumGenericCounters; ++i)
				 {
					/* TotalTxOKbyte,TotalRxOKbyte, UpStreamNode */
					if (i == 14 || i == 16 || i == 34)
					 {
						/*We have the high part of a 64 bit counter.*/
						(tableEntry -1)->StatUseFlag = ODI_STAT_UINT64;
					 }
					else if ( i == 32 )     /* CounterMask1 */
					 {
						/******************************************************
							Reset the Bit Shifter and the Valid Counter Bitmask
								for Assembly counters 33-36
						********************************************************/
						j = 0;
						bitMask = (UINT32)(UINT32*)(&statsTable->MSTATS_Generic.Gen_TotalTxPackets)[i];
					 }
					else
					 {
						/****************************************
							Check if the HSM uses this Counter.
							True => Not used
							False => UINT32 Stat Counter
						*****************************************/
						/* Force RxTooSmall and TxMiscCount to always be used	*/
						if( i == 7 || i == 8 )
							tableEntry->StatUseFlag = ODI_STAT_UINT32;
						else if (bitMask & (0x80000000 >> j))
         	         tableEntry->StatUseFlag = ODI_STAT_UNUSED;
						else
	                  tableEntry->StatUseFlag = ODI_STAT_UINT32;

						tableEntry->StatCounter
							= &(((void **)&statsTable->MSTATS_Generic.Gen_TotalTxPackets)[i]);
						++tableEntry;
						++j;
					 } /* end Else normal Counter setup. */
				 } /* end Prepare Generic and Media Counters. */

	         /**************************************************************\
   	      **
      	   **  Now initialize other media fields.
	         **
   	      \**************************************************************/
				newStats->MNumMediaCounters   = TOKENSTATSCOUNT;
				newStats->MMediaCountsPtr     = tableEntry - TOKENSTATSCOUNT;

	         /**************************************************************\
   	      **
      	   **  Now prepare the custom counters.
	         **
   	      \**************************************************************/
				customCounters = (UINT32 *) ((UINT8 *)customCounters + 2);
	
				/* Setup pointer to the Strings for the custom counters. */
				customStrings  = (UINT8 *) customCounters [numCustomCounters];
				customStrings += 2;

				newStats->MNumCustomCounters = numCustomCounters;
				newStats->MCustomCountersPtr = numCustomCounters ? tableEntry:NULL;

				for (i = 0; i < numCustomCounters; ++i)
				 {
					tableEntry->StatUseFlag = ODI_STAT_UINT32;
					tableEntry->StatCounter = customCounters;
					tableEntry->StatString  = customStrings;

				  /* Move to the end of the String */
					while (*customStrings != NULL)
							++customStrings;

					/* Point at the start of the next string. */
					++customStrings;

					/* Move to next Counter */
					++customCounters;
					++tableEntry;
				 } /* End Update Custom Strings  */
			 }	/* End Supported Old Style Assembly */
			else    /* Setup Generic and Media strings for C drivers */
			 {
  				/**************************************
					C version of Stats Table, needs
					TOPOLOGY specific components setup
				***************************************/
				newStats = DADSP_TO_CMSMADSP(driverData)->CMSMStatsPtr;
			
				/* build generic stats strings */
				tableEntry = newStats->MGenericCountsPtr;
      	   tableEntry = CMSMBuildGenericStatStrings( tableEntry );
	
				/* build media stats strings */
				tableEntry = newStats->MMediaCountsPtr;
   	      BuildTSMStatStrings (tableEntry);
			 }      /* End String changes and Fixups in TSM */
		 }	 /* Valid Stats Version */
		else
		 {
			/* Wrong version of Stats Table */
			/* Error out. */
			CMSMPrintString(	configTable, MSG_TYPE_INIT_WARNING,
									TOKENTSM_STATS_VERSION_ERROR_MSG,
 									NULL,	NULL);
	    	return( ODISTAT_OUT_OF_RESOURCES );
		 }	/* end wrong version of Stats Table */

#endif			/* IAPX386	*/

	/*Allocate multicast table*/
	if (DADSP_TO_CMSMADSP(driverData)->CMSMDPBlockPtr->DriverMaxMulticast)
	 {
		if ( (mediaData->MulticastAddressList = CMSMAlloc(driverData,
			mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE))) == NULL)
		 {
			CMSMPrintString( configTable, MSG_TYPE_INIT_WARNING,
								  TOKENTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );
			return( ODISTAT_OUT_OF_RESOURCES );
		 }

		CMSMmemset(mediaData->MulticastAddressList, NULL,
			(mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE)));
	 }	/* End Allocate Space for Multicast Table */

	/* Clear out the Functional Table. */
	CMSMmemset(&mediaData->FunctionalAddressList, NULL, sizeof(FUNCTIONAL_TABLE));

	/* Allocate and initialize memory for TCBs */
	bufferSize = sizeof(TCB) * DADSP_TO_CMSMADSP(driverData)->CMSMTxFreeCount;
	bufferSize += sizeof(TCB) * DADSP_TO_CMSMADSP(driverData)->CMSMPriorityTxFreeCount;

	if ( (mediaData->TCBHead = (TCB *)CMSMAlloc(driverData,bufferSize)) == NULL)
	 {
		CMSMPrintString( configTable, MSG_TYPE_INIT_ERROR,
							  TOKENTSM_MEMORY_ALLOC_ERROR_MSG, NULL, NULL );
  		return( ODISTAT_OUT_OF_RESOURCES );
	 }

	/* Initialize the Buffer */
	CMSMmemset( mediaData->TCBHead, 0, bufferSize );
	mediaData->TCBList = mediaData->TCBHead;
	tcbPtr = mediaData->TCBHead;

	/* Fill out fields in TCB and link into list. */
	for( i = DADSP_TO_CMSMADSP(driverData)->CMSMTxFreeCount +
				DADSP_TO_CMSMADSP(driverData)->CMSMPriorityTxFreeCount;
		  i!=0; i--)
	 {
		tcbPtr->TCB_DriverWS[0] = (UINT32)(tcbPtr + 1);

		COPY_ADDR( &tcbPtr->TCB_MediaHeader.TokenMedia.MH_Source,
					  &configTable->MLIDCFG_NodeAddress );
		++tcbPtr;
	 }	 /* End Filling in Node Address into TCBs */

	 /* Mark the Last TCB as the End */
	--tcbPtr;
	tcbPtr->TCB_DriverWS[0] = NULL;
	return( ODISTAT_SUCCESSFUL );

}	/*End MediaInit*/

#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
void	BuildASMStatStrings(	StatTableEntry	   *tableEntry )
 {
	/* Calling the CMSM for the generic statistics */
	tableEntry = CMSMBuildGenericStatStrings( tableEntry );

	/* now moving on to Topology specific statistics*/
	BuildTSMStatStrings( tableEntry );
   return;
}	/* End BuildASMStatStrings */

#endif		/* End ifdef IAPX386 */

void	BuildTSMStatStrings( StatTableEntry	   *tableEntry )
 {
	/* Token Ring Media Specific Statistics Counters */
	(tableEntry++)->StatString = TOKENTSM_STAT_AC_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_ABORT_DELIMITER_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_BURST_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_FRAME_COPY_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_FREQ_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_INTERN_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_LAST_RING_STATUS_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_LINE_EROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_LOST_FRAME_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_TOKEN_ERROR_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_UPSTREAM_NODE_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_LAST_RING_ID_MSG;
	(tableEntry++)->StatString = TOKENTSM_STAT_LAST_BEACON_TYPE_MSG;
   return;
 }	/* End BuildTSMStatStrings */

/**************************************************************************\
*
*	Name:
*		ODISTAT MediaAdjust(FRAME_DATA	*frameData);
*
*	Description:
*		This function initializes MLIDCFG_MaxFrameSize to its current value
*		or the maximum token packet size, whichever is smaller.  It then
*		must initialize MLIDCFG_BestDataSize and MLIDCFG_WorstDataSize
*		according to the MediaID.
*
*	Parameters in:
*		frameData -		Pointer to MSM/TSM Frame Data Space
*
* Return Value:
*		ODISTAT_SUCCESSFUL       - Frame Size values initialized.
*
\**************************************************************************/
ODISTAT	MediaAdjust( FRAME_DATA		*frameData )
{
	CONFIG_TABLE		*configTable;

	/* Initialize Locals */
	configTable = &frameData->ConfigTable;

	/* If the Frame Size is specified and Less than Max do nothing. */
	if( (configTable->MLIDCFG_MaxFrameSize) &&
		 (configTable->MLIDCFG_MaxFrameSize <= MAX_PACKET_LENGTH));
	else
		configTable->MLIDCFG_MaxFrameSize = MAX_PACKET_LENGTH;

	configTable->MLIDCFG_BestDataSize =
					configTable->MLIDCFG_MaxFrameSize - frameData->MediaHeaderSize;

	/* If Token-Ring Remember the Extra Byte for Type II frames */
   if (configTable->MLIDCFG_FrameID == TOKEN_ID)
		configTable->MLIDCFG_WorstDataSize = configTable->MLIDCFG_BestDataSize - SOURCE_MAX_SIZE - 1;
	else
		configTable->MLIDCFG_WorstDataSize = configTable->MLIDCFG_BestDataSize - SOURCE_MAX_SIZE;

  	/* Set the Raw Send bit in MLIDCFG_ModeFlags. */
	configTable->MLIDCFG_ModeFlags |= MM_RAW_SENDS_BIT;
	return( ODISTAT_SUCCESSFUL );

}	/* End MediaAdjust */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaReset(
*							PDRIVER_DS			driverData,
*							FRAME_DATA			*frameData,
*							OPERATION_SCOPE   operationScope
*							);
*
*	Description:
*		This function returns any sends queued up, and calls the HSM's
*		Driver Reset function.
*
*	Parameters in:
*		driverData -		Pointer to CHSMs Adapter Data Space.
*		frameData -			Pointer to MSM/TSM Frame Data Space.
*     operationScope -	OP_SCOPE_ADAPTER or OP_SCOPE_LOGICAL_BOARD
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			TSM/HSM reset successfully.
*		ODISTAT_FAIL					HSM's Driver Reset failed.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CTokenTsmReset from NT's tokentsm.c
*
************************************************************************/
ODISTAT	MediaReset(
							DRIVER_DATA		*driverData,
							FRAME_DATA		*frameData,
							OPERATION_SCOPE	operationScope)
{
	SHARED_DATA		   *sharedData;
	ECB					*ecb;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Local Variables. */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	statTable = sharedData->CMSMGenericStatsPtr;

	/* Check if any sends are outstanding. */
	while ((ecb = sharedData->CMSMSendListHead) != NULL)
	 {
		/* Return ECBs to LSL */
		CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
		(statTable->Gen_Qdepth)--;
		sharedData->CMSMSendListHead = ecb->ECB_NextLink;
		ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		CLSL_FastSendComplete(ecb);
	 }

	/* Initialize queue and exit */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
	ecb = (ECB *)&sharedData->CMSMSendListHead;
	sharedData->CMSMSendListTail = ecb;
	sharedData->CMSMStatusFlags &= ~TXQUEUED;
	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

	return( sharedData->CMSMDPBlockPtr->DriverResetPtr(
					driverData,
					&frameData->ConfigTable,
					operationScope) );

}	/* End MediaReset */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaShutdown(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData,
*									UINT32					shutdownType,
*									OPERATION_SCOPE		operationScope
*									);
*
*	Description:
*		This function returns any sends queued up, returns any memory
*		allocated if a permanent shutdown and calls the HSM's
*		Driver Shutdown function.
*
*	Parameters in:
*		driverData -		Pointer to CHSMs Adapter Data Space.
*		frameData -			Pointer to MSM/TSM frame data space.
*		shutdownType -		SHUTDOWN_PERMANENT or SHUTDOWN_PARTIAL
*
*     operationScope -	OP_SCOPE_ADAPTER or OP_SCOPE_LOGICAL_BOARD
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			TSM/HSM shutdown successfully.
*		ODISTAT_FAIL					HSM's Driver shutdown failed.
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*  Origin:	CEtherTsmShutdown in NT's ethertsm.c
*
************************************************************************/
ODISTAT	MediaShutdown(
							DRIVER_DATA				*driverData,
							FRAME_DATA				*frameData,
							UINT32					shutdownType,
							OPERATION_SCOPE		operationScope)
{
	SHARED_DATA		   *sharedData;
	MEDIA_DATA			*mediaData;
	ECB					*ecb;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Set up Local Variables */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData  = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/*Check if any sends are outstanding.*/
	while( (ecb = sharedData->CMSMSendListHead) != NULL )
	 {
		/*Return ECBs to LSL*/
		CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
		(statTable->Gen_Qdepth)--;
		sharedData->CMSMSendListHead = ecb->ECB_NextLink;
		ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		CLSL_FastSendComplete(ecb);
	 }

	/* Initialize queue and exit */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );
	ecb = (ECB *)&sharedData->CMSMSendListHead;
	sharedData->CMSMSendListTail = ecb;
	sharedData->CMSMStatusFlags &= ~TXQUEUED;
	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

	/*Check if permanent shutdown*/
	if (shutdownType == SHUTDOWN_PERMANENT)
	 {
		/*This is a permanent shutdown free any memory that was allocated*/
		/*by this module and exit.*/

#ifdef IAPX386				/* Only Intel x86 supports old assembly stats */
		/*Free the EXTRA Statistics Table ("C" or ASSEMBLY).*/
		if ( mediaData->NewStatsPtr )
       {
			CMSMFree(driverData, mediaData->NewStatsPtr);
			mediaData->NewStatsPtr = NULL;
		 }
#endif

		if ( mediaData->MulticastAddressList
			&& mediaData->MulticastAddressList != &MCPlaceHolder)
		 {
			CMSMFree(driverData, mediaData->MulticastAddressList);
			mediaData->MulticastAddressList = NULL;
		 }
		if ( mediaData->TCBList )
		{
			CMSMFree(driverData, mediaData->TCBList);
			mediaData->TCBList = NULL;
		}
	 }	/* end This is a Permanant ShutDown */

	    return( sharedData->CMSMDPBlockPtr->DriverShutdownPtr(
				driverData,
				&frameData->ConfigTable,
				shutdownType,
				operationScope) );

}	/* End MediaShutdown */

/**************************************************************************\
**
*	Name:
*		ODISTAT MediaNodeOverride(FRAME_DATA	*frameData,
*											MEON			mode);
**
**                This function will verify that the node override was of
**                the correct format and convert to canonical format if
**                necessary.
**
** Parameters:    IN    PMSM_FRAMEDATASPACE  frameData
**                IN    MEON                 mode
**
**                frameData  -  Pointer to default configuration table.
**                mode       -  'M' or 'm' if entered with msb (non-canonical) suffix
**                              'L' or 'l' if entered with lsb (canonical) suffix
**
** Return Value:  ODISTAT_SUCCESSFUL - Config Table NodeAddress in correct format
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaNodeOverride(
				FRAME_DATA				*frameData,
				MEON						mode)
{
	CONFIG_TABLE		*configTable;
	UINT8					addressbyte;
	UINT32			i,j;

	configTable = &frameData->ConfigTable;
	/*check if L option was used.*/
	if (mode == 'L' || mode == 'l')
	{
		/*Swap address to MSB mode.*/
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[0];
		configTable->MLIDCFG_NodeAddress.nodeAddress[0]
			= MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[1];
		configTable->MLIDCFG_NodeAddress.nodeAddress[1]
			= MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[2];
		configTable->MLIDCFG_NodeAddress.nodeAddress[2]
			= MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[3];
		configTable->MLIDCFG_NodeAddress.nodeAddress[3]
			= MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[4];
		configTable->MLIDCFG_NodeAddress.nodeAddress[4]
			= MSMBitSwapTable[addressbyte];
		addressbyte = configTable->MLIDCFG_NodeAddress.nodeAddress[5];
		configTable->MLIDCFG_NodeAddress.nodeAddress[5]
			= MSMBitSwapTable[addressbyte];
	}

/* MPK 970307  range checking of node address */
	/* see how long we are equal to the max address size */
	addressbyte = 1;	/* need this for flag purposes later */
	i=0;
	while ((i < 5) 
		&& 
	(configTable->MLIDCFG_NodeAddress.nodeAddress[i] == MaxAddress[5-i]))
	    i++;	/*over the entire address */

	/* see how long we are equal to the min address size */
	j=0;
	while ((j < 5) 
		&& 
	(configTable->MLIDCFG_NodeAddress.nodeAddress[j] == MinAddress[5-j]))
	     j++;	/*over the entire address */


	/* now: i = index where the node different from the max or is completely equal
	        j = index where the node different from the min or is completely equal
	   are we in range, or did we jump out?  */
	if (((UINT8 *)configTable->MLIDCFG_NodeAddress.nodeAddress[i] > (UINT8 *)MaxAddress[5-i])
					||
	    ((UINT8 *)configTable->MLIDCFG_NodeAddress.nodeAddress[j] < (UINT8 *)MinAddress[5-j]))
	  { 			/* we are either over or under range here */
	    addressbyte = 0;    /* flag that we can do no more */
	    CMSMPrintString( configTable,	
			MSG_TYPE_INIT_WARNING,
			TOKENTSM_INVALID_NODE_ERROR_MSG,	
			NULL,	
			NULL );
	    for (i=0;i<6;i++)	/*over the entire address reset to ff <no override> */
		configTable->MLIDCFG_NodeAddress.nodeAddress[i] = 0xFF;
	  }
/* END MPK 970307 */

	/*Check group and locally administered bits.*/
	if( (configTable->MLIDCFG_NodeAddress.nodeAddress[0] & GROUPBIT )
		 && (addressbyte))
	 {
		/*Clear group bit and warn user.*/
		configTable->MLIDCFG_NodeAddress.nodeAddress[0] &= ~GROUPBIT;

		CMSMPrintString( configTable,	MSG_TYPE_INIT_WARNING,
							  TOKENTSM_GRP_BIT_OVERRIDE_MSG,	NULL,	NULL );
	 }

	if(( !(configTable->MLIDCFG_NodeAddress.nodeAddress[0] & LOCALBIT) )
		 && (addressbyte))
	 {
		/*Set locally administered bit and warn user.*/
		configTable->MLIDCFG_NodeAddress.nodeAddress[0] |= LOCALBIT;
		CMSMPrintString( configTable, MSG_TYPE_INIT_WARNING,
							  TOKENTSM_LOCAL_BIT_OVERRIDE_MSG, NULL, NULL );
	 }
	return( ODISTAT_SUCCESSFUL );

}	/* End MediaNodeOverride	*/

/**************************************************************************\
**
** PROC NAME:     MediaAdjustNodeAddress
**
**                This function will copy the physical node address from the
**                adapter data space to the config table.
**
** Parameters:    IN    PMSM_FRAMEDATASPACE  frameData
**
**                frameData  -  Pointer to default configuration table.
**
** Return Value:  ODISTAT_SUCCESSFUL - Config Table Mode Flags LSB/MSB bits set
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaAdjustNodeAddress( FRAME_DATA	*frameData )
{
	DRIVER_DATA 			*driverData;
	SHARED_DATA	   		*sharedData;
	CONFIG_TABLE			*configTable;

	/* Set up the Locals */
	configTable = &frameData->ConfigTable;
	driverData  = frameData->DriverData;
	sharedData  = DADSP_TO_CMSMADSP( driverData );

	/* Set the Modeflags for Address settings. */
	frameData->ConfigTable.MLIDCFG_ModeFlags |= MM_PHYS_NODE_ADDR_BIT;
	frameData->ConfigTable.MLIDCFG_ModeFlags |= MM_NONCANONICAL_BIT;

	/*Check to see if BitSwap Flag is set and then adjust the address*/
	if( frameData->BitSwapAddressFlag )
	 {
		frameData->ConfigTable.MLIDCFG_ModeFlags &= ~MM_NONCANONICAL_BIT;

		/* We need to Swap the address */
      configTable->MLIDCFG_NodeAddress.nodeAddress [0] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [0]];
      configTable->MLIDCFG_NodeAddress.nodeAddress [1] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [1]];
      configTable->MLIDCFG_NodeAddress.nodeAddress [2] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [2]];
      configTable->MLIDCFG_NodeAddress.nodeAddress [3] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [3]];
      configTable->MLIDCFG_NodeAddress.nodeAddress [4] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [4]];
      configTable->MLIDCFG_NodeAddress.nodeAddress [5] =
         MSMBitSwapTable [sharedData->CMSMPhysNodeAddress [5]];
	 }
	else
		COPY_ADDR( configTable->MLIDCFG_NodeAddress.nodeAddress,
					  sharedData->CMSMPhysNodeAddress );

	return( ODISTAT_SUCCESSFUL );
}	/* End MediaAdjustNodeAddress */

/**************************************************************************\
**
*	Name:
*		ODISTAT MediaSetLookAheadSize(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData;
*									UINT32					size);
*
**                Update MLIDCFG_LookAheadSize to requested size, unless
**                size is greater than maximum look ahead size (128), or
**                MLIDCFG_LookAheadSize is already at least the requested
**                size.  We'll also need to update
**                DADSP_TO_CMSMADSP (adapterData)->CMSMMaxFrameHeaderSize.
**
** Parameters:    IN    PMSM_FRAMEDATASPACE  frameData
**                IN    void                 *driverData
**                IN    UINT32               size
**
**                frameData    -  Pointer to default configuration table.
**                driverData    -  Pointer to CHSMs Adapter Data Space
**                size         -  Requested look ahead size.
**
** Return Value:  ODISTAT_SUCCESSFUL    - Look Ahead Size successfully updated
**                ODISTAT_BAD_PARAMETER - size > 128.
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaSetLookAheadSize(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					size)
{
	SHARED_DATA			   *sharedData;
	CONFIG_TABLE			*configTable;
	ODISTAT					odiStat = ODISTAT_SUCCESSFUL;		/*assume successful status.*/
	UINT32					i;

	sharedData = DADSP_TO_CMSMADSP(driverData);

	if((size > frameData->ConfigTable.MLIDCFG_LookAheadSize) &&
		(size <= 128))
	 {
		/*Set size for all virtual boards associated with this physical board.*/
		for (i = 0; i < 2; ++i)
		{
			if (configTable = sharedData->CMSMVirtualBoardLink[i])
			{
				configTable->MLIDCFG_LookAheadSize = (UINT16)size;
				sharedData->CMSMMaxFrameHeaderSize = size + sizeof(TOKEN_MEDIA_HEADER) + SOURCE_MAX_SIZE;
	 		}
		}

		/*Call HSM if it has a SetLookAheadRoutine*/
		if (sharedData->CMSMDPBlockPtr->DriverRxLookAheadChangePtr)
		 {
			odiStat = sharedData->CMSMDPBlockPtr->DriverRxLookAheadChangePtr(
							driverData,
							&frameData->ConfigTable);
		 }
	 }
	else
	 {
		if( size > 128 )
			odiStat = ODISTAT_BAD_PARAMETER;
	 }
	return( odiStat );
}	/* End MediaSetLookAheadSize */

/**************************************************************************\
**
** PROC NAME:     MediaAddMulticast
**
**                This function adds the multicast address to the MLIDs
**                multicast address table.  If its a new address, the HSM's
**                DriverMulticastChange function will get called.  If the
**                multicast address has already added this address, we will
**                only increment its GRP_ADDR_COUNT Flag.
**
** Parameters:    IN    void                 *driverData
**                IN    NODE_ADDR            *mcAddress
**
**                driverData    -  Pointer to CHSMs Adapter Data Space
**                mcAddress    -  Pointer to multicast node address
**
** Return Value:  ODISTAT_SUCCESSFUL       - Multicast address successfully added
**                ODISTAT_BAD_PARAMETER    - Invalid multicast address
**                ODISTAT_OUT_OF_RESOURCES - Multicast table full
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaAddMulticast(
				DRIVER_DATA			*driverData,
				NODE_ADDR			*mcAddressIn )
{
	GROUP_ADDR_LIST_NODE			*mcEntry, *newMulticast;
	SHARED_DATA			         *sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;
	UINT32							i;
	NODE_ADDR						*mcAddress;
	NODE_ADDR						copyMcAddress;

	/* Set up Local variables */
	sharedData = DADSP_TO_CMSMADSP( driverData );
   mediaData  = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;
	COPY_ADDR( &copyMcAddress, mcAddressIn );
	mcAddress = &copyMcAddress;

	/*Check if valid Multicast Address.*/
	if ((mcAddress->nodeAddress[0] & MULTICASTBIT) == 0)
	 {
		/*Address is not a valid Multicast*/
		/*Swap it and check again*/
      mcAddress->nodeAddress[0] = MSMBitSwapTable[mcAddress->nodeAddress[0]];
      mcAddress->nodeAddress[1] = MSMBitSwapTable[mcAddress->nodeAddress[1]];
      mcAddress->nodeAddress[2] = MSMBitSwapTable[mcAddress->nodeAddress[2]];
      mcAddress->nodeAddress[3] = MSMBitSwapTable[mcAddress->nodeAddress[3]];
      mcAddress->nodeAddress[4] = MSMBitSwapTable[mcAddress->nodeAddress[4]];
      mcAddress->nodeAddress[5] = MSMBitSwapTable[mcAddress->nodeAddress[5]];

   	if ((mcAddress->nodeAddress[0] & MULTICASTBIT) == 0)
   		return( ODISTAT_BAD_PARAMETER );
	 } /* End Try Swapping Multicast Address. */
		
	/* Check if Functional or Group address. */
   if( (mcAddress->nodeAddress[2] & GROUPBIT) == 0 )
    {
		if( *(UINT16*)mcAddress->nodeAddress != 0x00C0 )  /* Remember it is in High-Low Order in memory */
   		return( ODISTAT_BAD_PARAMETER );

      /* We have a Functional address, so add it */
      for (i = 0; i < 32; ++i)
       {
         mediaData->FunctionalAddressList.FunctionalBit[i] +=
												  ((*(UINT32 *)(&mcAddress->nodeAddress[2]) >> i) & 1);
			if( (*(UINT32 *)(&mcAddress->nodeAddress[2]) >> i) & 1 )
				mediaData->FunctionalAddressList.ActiveFunctionalBits |= (1 << i);
       }
    }	/* Functional Address Check. */
   else
    {
      /* We have a Group address, so add it */
      /* See if table can hold address */
		if ( FindAddressInMCTable( sharedData,	mcAddress, &mcEntry) )
		 {
     		/* Address is not in Table */
     		/* Search for an empty slot. */

     		mediaData = sharedData->CMSMMediaDataPtr;
     		mcEntry = mediaData->MulticastAddressList;

     		for (i = 0; i < mediaData->MaxMulticastAddresses; ++i)
     		 {
	      	/*Check if entry is in use.*/
     			if (mcEntry->GRP_ADDR_COUNT == 0)
		      		break;
     			++mcEntry;
	       }

			if( i >= sharedData->CMSMDPBlockPtr->DriverMaxMulticast )
				return( ODISTAT_OUT_OF_RESOURCES );

     		if( i >= mediaData->MaxMulticastAddresses )
     		 {
	      	/*Table is full.*/
     			newMulticast = CMSMAlloc(driverData,
											(mediaData->MaxMulticastAddresses + 32)
											* sizeof(GROUP_ADDR_LIST_NODE));

      		if( newMulticast == NULL )
	      		return( ODISTAT_OUT_OF_RESOURCES );

     			CMSMmemset(newMulticast, 0,
								  ( (mediaData->MaxMulticastAddresses + 32)
								    * sizeof(GROUP_ADDR_LIST_NODE)) );

     			CMSMmemcpy(newMulticast, mediaData->MulticastAddressList,
								( mediaData->MaxMulticastAddresses
								 * sizeof(GROUP_ADDR_LIST_NODE)) );

	      	CMSMFree(driverData, mediaData->MulticastAddressList);
     			mediaData->MulticastAddressList = newMulticast;
	      	mediaData->MaxMulticastAddresses += 32;
     			return ( MediaAddMulticast(driverData, mcAddress) );
			 }	/* Table was full */
		 } /* Address was not in the table */

     	/*Address has been found or an empty location was returned.*/
     	if( !(mcEntry->GRP_ADDR_COUNT) )
     	 {
     		/*Add address to table*/
     		COPY_ADDR( mcEntry->GRP_ADDR.nodeAddress, (NODE_ADDR *)mcAddress->nodeAddress);
	      mediaData->MulticastAddressCount++;
     	 }

     	/*Increment Entry Used count.*/
     	mcEntry->GRP_ADDR_COUNT++;
	 } /* Group Address check */

	/*Inform driver of Multicast Change.*/
	if( (mediaData->PromiscuousMode == 0) ||
		 (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) )
		odiStat = CTokenTSMUpdateMulticast( driverData );

	return( odiStat );
}	/* End MediaAddMulticast */

/**************************************************************************\
**
*	Name:
*		ODISTAT MediaDeleteMulticast(
*									DRIVER_DATA		*driverData,
*									NODE_ADDR		*mcAddress);
**
**                This function deletes the multicast address from the MLIDs
**                multicast address table.  If another process has added this
**                multicast address, we will only decrement its GRP_ADDR_COUNT Flag.
**                If the GRP_ADDR_COUNT flag equals zero, no other process is
**                expecting this multicast address, the address will get
**                get deleted and the HSM's DriverMulticastChange function
**                will get called and the HSM's DriverMulticastChange Function
**                will get called.
**
*	Parameters in:
*		driverData						HSMs adapter data space
*		mcAddress						Pointer to multicast node address
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Instance of Multicast address deleted
*		ODISTAT_BAD_PARAMETER		Invalid multicast address
*		ODISTAT_ITEM_NOT_PRESENT	Multicast address not in table
*
*	Implied parameters:
*		NONE
*
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaDeleteMulticast(
				DRIVER_DATA			   *driverData,
				NODE_ADDR				*mcAddressIn)
{
	GROUP_ADDR_LIST_NODE		*mcEntry;
	SHARED_DATA			      *sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT						odiStat = ODISTAT_SUCCESSFUL;
	UINT32						count, i;
	NODE_ADDR					*mcAddress;
	NODE_ADDR					copyMcAddress;

	/* Set up Locals */
	sharedData = DADSP_TO_CMSMADSP( driverData );
   mediaData  = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;
	COPY_ADDR( &copyMcAddress, mcAddressIn );
	mcAddress = &copyMcAddress;

	/*Check if valid Multicast Address.*/
	if( (mcAddress->nodeAddress[0] & MULTICASTBIT) == 0 )
	 {
		/*Address is not a valid Multicast*/
		/*Swap it and check again*/
      mcAddress->nodeAddress[0] = MSMBitSwapTable[mcAddress->nodeAddress[0]];
      mcAddress->nodeAddress[1] = MSMBitSwapTable[mcAddress->nodeAddress[1]];
      mcAddress->nodeAddress[2] = MSMBitSwapTable[mcAddress->nodeAddress[2]];
      mcAddress->nodeAddress[3] = MSMBitSwapTable[mcAddress->nodeAddress[3]];
      mcAddress->nodeAddress[4] = MSMBitSwapTable[mcAddress->nodeAddress[4]];
      mcAddress->nodeAddress[5] = MSMBitSwapTable[mcAddress->nodeAddress[5]];

   	if( (mcAddress->nodeAddress[0] & MULTICASTBIT) == 0 )
   		return( ODISTAT_BAD_PARAMETER );
	 }
		
	/* Check if Functional or Group address. */
   if( (mcAddress->nodeAddress[2] & GROUPBIT) == 0 )
    {
		if( *(UINT16*)mcAddress->nodeAddress != 0x00C0 )  /* Remember it is in High-Low order */
   		return( ODISTAT_BAD_PARAMETER );

      /* We have a Functional address, so delete it */
      for( i = 0; i < 32; ++i )
       {
         if( mediaData->FunctionalAddressList.FunctionalBit[i] )
          {
            mediaData->FunctionalAddressList.FunctionalBit[i] -=
												 ((*(UINT32 *)(&mcAddress->nodeAddress[2]) >> i) & 1);
				if( mediaData->FunctionalAddressList.FunctionalBit[i] == 0 )
					mediaData->FunctionalAddressList.ActiveFunctionalBits &= ~( 1 << i);
          }	/* If there is a bit that is set then check it out. */
			else
				/* A Bit to delete wasn't active. */
				if( (*(UINT32 *)(&mcAddress->nodeAddress[2]) >> i) & 1 )
				 	return( ODISTAT_ITEM_NOT_PRESENT );				

      }/* Check Each entry until a request is made for a bit not set. */
    } /* looks like a functional Address */
   else
    {
   	/* See if Group Address Entry is in table */
		if( FindAddressInMCTable(sharedData, mcAddress, &mcEntry) )
		 {
	   	/*Address is not in Table*/
   		return( ODISTAT_ITEM_NOT_PRESENT );
		 }

   	/*Address has been found.*/
	   /*Decrement Entry Used count.*/
   	mcEntry->GRP_ADDR_COUNT--;

		if( mcEntry->GRP_ADDR_COUNT == 0 )
		 {
			/* Collapse Muticast Table */
			mediaData = sharedData->CMSMMediaDataPtr;
			count = (UINT32)((UINT8 *)mcEntry - (UINT8 *)mediaData->MulticastAddressList);
			count = (mediaData->MaxMulticastAddresses * sizeof(GROUP_ADDR_LIST_NODE)) - count;
			count -= sizeof(GROUP_ADDR_LIST_NODE);
			CMSMmemcpy(mcEntry, mcEntry + 1, count);

			/* Now Zero out the last Address in the table. */
			CMSMmemset(
					mediaData->MulticastAddressList + (mediaData->MaxMulticastAddresses - 1),
					0,	sizeof(GROUP_ADDR_LIST_NODE));

			mediaData->MulticastAddressCount--;

	 	 } /* Colapse the Table another address has been removed. */
    }	/* Check if Group Address in Table */

	/*	************************************************************
		Inform driver of Multicast Change if Not in Promsicuous Mode
  		Note, we don't track an if an actual change occured, but
		we tell the TOKEN HSM a change occured for each Add/Delete
		************************************************************* */
	if ( (mediaData->PromiscuousMode == 0) ||
		  (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) )

		odiStat = CTokenTSMUpdateMulticast( driverData );

	return( odiStat );
}	/* End MediaDeleteMulticast */

/**************************************************************************\
**
*	Name:
*		ODISTAT	FindAddressInMCTable(
*									SHARED_DATA		*sharedData,
*									NODE_ADDR		*mcAddress
*									GROUP_ADDR_LIST_NODE **mcEntry);
*
*	Description:
*        This function checks to see if a Group/Functional address has
*			been added to the Group/Functional address table.
*
*	Parameters in:
*		sharedData				MSM/TSM/HSM shared adapter data space
*		mcAddress				Pointer to multicast node address
*		mcEntry					Pointer to Where to place Pointer to Group_addr_list_Node.
*
*	Values returned:
*		If mcAddress is a Functional Address:
*							ODISTAT_SUCCESSFUL		Functional Address active
*															mcEntry = NULL
*							ODISTAT_ITEM_NOT_PRESENT	Functional Address Not Active
*															mcEntry = NULL
*
*		If mcAddress is a Group Address:
*							ODISTAT_SUCCESSFUL		Group Address Found,
*															mcEntry = GroupAddresslistnode.
*							ODISTAT_ITEM_NOT_PRESENT	Group Address Not in Table.
*															mcEntry = NULL
*
*	Implied parameters:
*		NONE
*
** See Also:
**
\**************************************************************************/
ODISTAT	FindAddressInMCTable(
				SHARED_DATA			   *sharedData,
				NODE_ADDR				*mcAddress,
				GROUP_ADDR_LIST_NODE **mcEntry )
{
	MEDIA_DATA						*mediaData;
	UINT32							i;
	UINT32							maxAddresses;

	/* Set up Locals */
	mediaData = sharedData->CMSMMediaDataPtr;

	/* Check if Functional or Group address. */
   if( (mcAddress->nodeAddress[2] & GROUPBIT) == 0)
    {
		*mcEntry = NULL;	  /* No Group Address */
      /* We might have a Functional address, so Check it */
		if( *(UINT16*)mcAddress->nodeAddress != 0x00C0 ) /* Remember it is in High-Low Order */
   		return( ODISTAT_BAD_PARAMETER );

		/* It really is Functional Address. */
		if ( mediaData->FunctionalAddressList.ActiveFunctionalBits &
			  (*(UINT32 *)(&mcAddress->nodeAddress[2])) )
			return( ODISTAT_SUCCESSFUL );
	 }
	else
	 {
		*mcEntry = mediaData->MulticastAddressList;
		maxAddresses = (sharedData->CMSMDPBlockPtr->DriverMaxMulticast <
							 mediaData->MaxMulticastAddresses) ?
						  	 sharedData->CMSMDPBlockPtr->DriverMaxMulticast :
						  	 mediaData->MaxMulticastAddresses;

		for (i = 0; i < maxAddresses; ++i)
		 {
			/*Check if entry is in use.*/
			if ((*mcEntry)->GRP_ADDR_COUNT != 0)
			 {
				/*Check to see if address matches*/
				if( NODE_EQUAL((*mcEntry)->GRP_ADDR.nodeAddress, mcAddress->nodeAddress) )
					return( ODISTAT_SUCCESSFUL );
			 }
			else
				break;	/* End of the Table */
			(*mcEntry)++;
		 }	/* Check Each entry */
	 } /* Check Group Address */

	/* Signal that GroupAddress not Found */
	*mcEntry = NULL;	
	return( ODISTAT_ITEM_NOT_PRESENT );

} /*End FindAddressInMCTable*/

/**************************************************************************\
**
** PROC NAME:     CTokenTSMUpdateMulticast
**
**                This function gives the current multicast address table to
**                the HSM's DriverMulticastChange function so that the HSM
**                can update its multicast registers.  It is called by
**                MediaAddMulticast and MediaDeleteMulticast.  It may also
**                be called by the HSM if it has just reset its adapter and
**                needs its multicast address registers updated.
**
** Parameters:    IN    void                 *driverData
**
**                driverData    -  Pointer to CHSMs Adapter Data Space
**
** Return Value:  ODISTAT_SUCCESSFUL - HSM's Multicast address registers updated
**
** See Also:
**
\**************************************************************************/
ODISTAT	CTokenTSMUpdateMulticast(
				DRIVER_DATA			*driverData)
{
	SHARED_DATA	   		      *sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;

	 /* Initialize Local Variables. */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	configTable = sharedData->CMSMDefaultVirtualBoard;

	if (sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr &&
			mediaData->PromiscuousMode)
    {
		odiStat = sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr(
						driverData,
						sharedData->CMSMDefaultVirtualBoard,
						mediaData->PromiscuousMode);

		if ( (configTable->MLIDCFG_Flags & MF_SOFT_FILT_GRP_BIT) &&
	    	  (sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr) )

  		 	odiStat = sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
	   					driverData,
		   				sharedData->CMSMDefaultVirtualBoard,
			   			mediaData->MulticastAddressList,
				   		mediaData->MulticastAddressCount,
					   	mediaData->FunctionalAddressList.ActiveFunctionalBits);

	 }
   else if (sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr)
    {
   	odiStat = sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
	   				driverData,
		   			sharedData->CMSMDefaultVirtualBoard,
			   		mediaData->MulticastAddressList,
				   	mediaData->MulticastAddressCount,
					   mediaData->FunctionalAddressList.ActiveFunctionalBits);
    }

	return( odiStat );
}	/*End	CTokenTsmUpdateMulticast*/

/**************************************************************************\
**
*	Name:
*		ODISTAT MediaPromiscuousChange(
*									DRIVER_DATA				*driverData,
*									FRAME_DATA				*frameData,
*									UINT32					promiscuousState,
*									UINT32					*promiscuousMode);
*
*	Description:
**                This function enables/disables promiscuous mode depending
**                on promiscuousState.
**
*	Parameters in:
*		frameData -				Pointer to Frame Data Space
*		driverData -			HSMs adapter data space
*		promiscuousState -	PROM_STATE_OFF to disable promiscuous mode
*									PROM_STATE_ON  to enable  promsicuous mode
*
*		promiscuousMode -		PROM_MODE_QUERY     to query promiscuous state
*									PROM_MODE_MAC       to enable all MAC reception
*									PROM_MODE_NON_MAC   to enable all Non-MAC reception
*									PROM_MODE_SMT		  to enable all SMT Frames (FDDI)
*									PROM_MODE_MACANDNON to enable MAC and Non-MAC
*	Parameters out:
*		promiscuousMode -		The current promiscuous mode will be returned
*									to the location pointed to by this field.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Promiscuous Mode enabled/disabled
*		ODISTAT_BAD_COMMAND			Promiscuous Mode not supported by MLID
*
*	Implied parameters:
*		NONE
*
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaPromiscuousChange(
				DRIVER_DATA			   *driverData,
				FRAME_DATA				*frameData,
				UINT32					promiscuousState,
				UINT32					*promiscuousMode)
{
	SHARED_DATA			         *sharedData;
	MEDIA_DATA						*mediaData;
	CONFIG_TABLE					*configTable;
	ODISTAT							odiStat = ODISTAT_SUCCESSFUL;
	UINT32							i;

	configTable = &frameData->ConfigTable;
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/*Check if promiscuous mode supported by this mlid.*/
	if ((configTable->MLIDCFG_ModeFlags & MM_PROMISCUOUS_BIT) &&
		sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr)
	{
		/*The MLID supports promiscuous mode*/
		if (*promiscuousMode != 0)
		{
			/*We need to change the PromiscuousMode*/
			if( promiscuousState )
			 {
				/*We are Enabling promiscuous mode*/
				for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
				 {
					mediaData->PromiscuousCounters[i] += ((*promiscuousMode >> i) & 0x01);
				 }
			 }
			else
			 {
				/*We are Disabling promiscuous mode*/
				for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
				 {
					if (mediaData->PromiscuousCounters[i])
						mediaData->PromiscuousCounters[i] -= ((*promiscuousMode >> i) & 0x01);
				 }
			 }

			/*Determine the new promiscuous Mode*/
         mediaData->PromiscuousMode = 0;
			for (i = 0; i < NUMBER_OF_PROMISCUOUS_COUNTERS; ++i)
			{
				if (mediaData->PromiscuousCounters[i])
				{
					mediaData->PromiscuousMode = (mediaData->PromiscuousMode | (1 << i));
				}
			}

			/* Check to see if the remote multicast bit is set. */
			if (mediaData->PromiscuousMode & PROM_MODE_RMC)
			{
				/* Find out if this driver supports remote multicase.*/
				if (sharedData->CMSMDPBlockPtr->DriverParameterSize <
					(UINT32)&(((DRIVER_PARM_BLOCK *)0)->HSMSpecVersionStringPtr))
				{
					/* This driver does not have the spec string so it must not */
					/* be able to support remote multicast. */
					mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
				}
				else
				{
					/* The driver has the spec string.  Check to see if it */
               /* supports remote multicast.  Must be at least cspec 1.11 */
					/* or asm spec 3.31. */
					if (configTable->MLIDCFG_ModeFlags & MM_C_HSM_BIT)
					{
						if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MAJOR_VERSION_POS]
							< '2')
						{
							if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MINOR_VERSION_TENS_POS]
								< '2')
							{
								if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[C_MINOR_VERSION_ONES_POS]
									< '1')
								{
									/* This is less than c spec 1.11. */
									mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
								}
							}
						}
					}
					else
					{
						if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MAJOR_VERSION_POS]
							< '4')
						{
							if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MINOR_VERSION_TENS_POS]
								< '4')
							{
								if (sharedData->CMSMDPBlockPtr->HSMSpecVersionStringPtr[A_MINOR_VERSION_ONES_POS]
									< '1')
								{
									/* This is less than a spec 3.31. */
									mediaData->PromiscuousMode |= PROM_MODE_NON_MAC;
								}
							}
						}
					}
				}
			}

			/*********************************/
			/*Inform the Stacks of a change*/
			/*********************************/
			CLSL_ControlStackFilter(
							configTable->MLIDCFG_BoardNumber,
							PSTK_PROMISCUOUS_STATE,
							DT_PROMISCUOUS,
							&mediaData->PromiscuousMode,
							NULL);
							
			/*Inform MLID of change*/
			odiStat = sharedData->CMSMDPBlockPtr->DriverPromiscuousChangePtr(
							driverData,
							configTable,
							mediaData->PromiscuousMode);

			/* If promiscuous mode is being disabled, make sure that the */
			/* the multicast addresses get updated too.  Disabling promiscuous */
			/* mode may destroy the multicast table on the card. */
         if( mediaData->PromiscuousMode == 0 )
			 {
				/* Check that multicast is supported. */
				if (sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr != NULL)
				 {
					/* Check that there are addresses to be updated. */
					if( mediaData->FunctionalAddressList.ActiveFunctionalBits ||
						 mediaData->MulticastAddressCount)
					 {
						odiStat =
							sharedData->CMSMDPBlockPtr->DriverMulticastChangePtr(
								driverData,
								sharedData->CMSMDefaultVirtualBoard,
								mediaData->MulticastAddressList,
								mediaData->MulticastAddressCount,
								mediaData->FunctionalAddressList.ActiveFunctionalBits);
					 }
				 }
			 }
		}
		/*Return the current promiscuous mode.*/
		*promiscuousMode = mediaData->PromiscuousMode;

		return( odiStat );
	}

	/*********************************************/
	/* Set Mode to 0 because we don't support it.*/
	/*********************************************/
	*promiscuousMode = 0;
	return( ODISTAT_BAD_COMMAND );

}	/* End MediaPromiscuousChange */

/**************************************************************************\
**
** PROC NAME:     MediaRegisterMonitor
**
**                This function allows a monitor protocol stack to enable/
**                disable its Transmit Monitor Routine.  It will be called
**                by the TSM after a packet has been transmitted with a
**                pointer to the TCB.  Only one transmit monitor may be
**                registered per logical board.
**
** Parameters:
**						IN   PMSM_FRAMEDATASPACE  frameData
**                IN   UINT32               TxMonRoutine
**                IN   BOOLEAN              MonitorState
**
**                frameData        - Pointer to Frame Data Space
**                TxMonRoutine     - Pointer to transmit monitor function
**                MonitorState     - TRUE if registering function
**                                   FALSE if deregistering function
**
** Return Value:  ODISTAT_SUCCESSFUL       - Routine registerd/deregistered successfully
**                ODISTAT_OUT_OF_RESOURCES - Another transmit routine already registered
**                ODISTAT_NO_SUCH_HANDLER  - Tried to disable another processes routine
**
** See Also:
**
\**************************************************************************/
ODISTAT	MediaRegisterMonitor(
				FRAME_DATA				*frameData,
				void						(*TXRMonRoutine)(TCB *),
				BOOLEAN					MonitorState)
{
	UINT32				status;

	for (;;)
	{
		if (MonitorState == TRUE)
		{
			/*We need to enable the Monitor.*/
			/*Check to see if already enabled.*/
			if (frameData->TransmitRoutine)
			{
				/*We are already enabled.*/
				status = ODISTAT_OUT_OF_RESOURCES;
				break;
			}
			frameData->TransmitRoutine = TXRMonRoutine;
			status = ODISTAT_SUCCESSFUL;
			break;
		}

		/*Disable the Monitor.*/
		if (TXRMonRoutine == frameData->TransmitRoutine)
		 {
			frameData->TransmitRoutine = NULL;
			status = ODISTAT_SUCCESSFUL;
			break;
		 }

		/*An imposter tried to disable the Monitor.*/
		status = ODISTAT_NO_SUCH_HANDLER;
		break;
	}
	return( status );

}	/* End MediaRegisterMonitor */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT MediaGetParamters(
*           CONFIG_TABLE      *configTable);
*
*	Description:
*		This function allows the TSM to parse for custom parameters
*     The TSM must parse for FRAME and NODE.
*
*	Parameters in:
*		hsmParameters        Pointer to the CHSMs parameterblock.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL			Routine fetched required parameters
*		ODISTAT_FAIL               Parameters could not be obtained.
*
*  Notes:
*
*
************************************************************************/
ODISTAT	MediaGetParameters(
            CONFIG_TABLE      *configTable)
{
   DRIVER_DATA *driverData;
   FRAME_DATA  *frameData;
   ODISTAT     status;
   UINT32      frameIndex;
	MEON_STRING *whiteSpace = TOKENTSM_WHITESPACE_MSG;
	MEON_STRING *s;

	frameData = CONFIG_TO_MSM(configTable);

   /* If MLIDCFG_FrameID is set To UNUSED This is virtual board loading. */
	/* The driverData was saved in MLIDCFG_FrameTypeString. */
	if((configTable->MLIDCFG_FrameID & 0xFF00) == (UINT16)(UNUSED & 0xFF00) )
		driverData = (DRIVER_DATA *)configTable->MLIDCFG_FrameTypeString;
	else driverData = NULL;

   if (driverData != NULL)
    {
      FrameOptionList.OptionCount = 0;

      for (frameIndex = 0; frameIndex < 2; ++frameIndex)
       {
         if( DADSP_TO_CMSMADSP(driverData)->CMSMVirtualBoardLink[frameIndex] == NULL )
         {
            FrameOptionList.StrOptVal[FrameOptionList.OptionCount]
               = TokenTSMFrameDescriptTable[frameIndex];
            ++FrameOptionList.OptionCount;
         }
       }
      FrameOption.Flags = DEFAULTPRESENT | ENUMPARAM | REQUIREDPARAM;
    }
   else
    {
      FrameOptionList.OptionCount = 2;

      for (frameIndex = 0; frameIndex < 2; ++frameIndex)
      {
         FrameOptionList.StrOptVal[frameIndex]
            = TokenTSMFrameDescriptTable[frameIndex];
      }
      FrameOption.Flags = STRINGPARAM;
    }

   status = CMSMParseCTSMParameters(configTable, &LSBOption);

   if (!(status == ODISTAT_SUCCESSFUL || status == ODISTAT_ITEM_NOT_PRESENT))
      return( status );
   else if (FrameOption.String[0] != '\0')
    {
		/* Place a NULL in the FrameOption.String so that the CMSMstrcmp will */
		/* compare the correct piece of the string. */
		s = FrameOption.String;
		s += CMSMstrcspn(FrameOption.String, whiteSpace);
		*s = '\0';

      for (frameIndex = 0; frameIndex < 2; ++frameIndex)
      {
         if (CMSMstrcmp(FrameOption.String, TokenTSMFrameDescriptTable[frameIndex]) == 0)
         {
            break;
         }

      }
      if (frameIndex >= 2)
         return ODISTAT_FAIL;

		/* Now lets check to see if LSB was entered. */

		if (LSBOption.Parameter1.Max)
		{
			/* We need to present addresses to the upper layers in LSB format. */

			frameData->BitSwapAddressFlag = 1;
		}

    }
   else
		frameIndex = 0; /* Use default frame. */

   configTable->MLIDCFG_FrameTypeString
      = TokenTSMFrameDescriptTable[frameIndex];
   frameData->InternalMediaID = frameIndex;
   frameData->MediaHeaderSize
      = frameData->MPBlock->MediaHeaderSizeArray[frameData->InternalMediaID];
   configTable->MLIDCFG_FrameID
      = frameData->MPBlock->MediaIDArray[frameData->InternalMediaID];

   /* Now lets check to see if the Node was entered. */
   if (NodeOption.String[0] != '\0')
    {
      /* We have a Node Address Override convert string to HEX. */
      CMSMASCIINodeToHex( configTable, NodeOption.String );
    }

   return( ODISTAT_SUCCESSFUL );
}	/* End MediaGetParameters */

/***********************************************************************\
*
*	Name:
*		ODISTAT MediaGetMulticastInfo(
*            struct _DRIVER_DATA_    *driverData,
*            ECB                     *MulticastInfoECB);
*
*   Description:  This function returns the Group(Multicast) and/or
*                 Functional addresses the MLID is using.
*
*   Input Parameters:
*
*            driverData        Pointer to the drivers data space.
*
*            MulticastInfoECB  Pointer to an ECB to return info in.
*
*   Values returned:
*
*     ODISTAT_SUCCESSFUL       Command was successfully executed.  The
*                              ECB is returned to the caller.
*
*     ODISTAT_RESPONSE_DELAYED The requested operation was successfully
*                              started but will complete asynchronously.
*                              MulticastInfoECB event service routine
*                              will be called with the cpmpletion code
*                              when the command has finished execution.
*						
*     ODISTAT_BAD_COMMAND      GetMulticastInfo is not supported by
*                              the MLID.
*
*     ODISTAT_OUT_OF_RESOURCES The buffer presented in the MulticastInfoECB
*                              to transfer the Group(Multicast) addresses
*                              used by the MLID into was insufficient.  The
*                              field MulticastInfoECB ECB_DataLength contains
*                              the size the buffer should be to transfer
*                              Group(Multicast) addresses used by the MLID.
*
*
*   Input/Output Parameters:
*
*				  MulticastInfoECB
*                      Pointer to an ECB in which to return Group
*                      (Multicast) and/or Functional address
*                      information.  The MulticastInfoECB contains a
*                      single fragment into which to copy Group
*                      (Multicast) address information:
*
*                      On Entry:
*
*                         ECB_FragmentCount
*                                   Is equal to 1.
*
*                         ECB_FragmentAddress
*                                   Points to the buffer to copy Group
*                                   (Multicast) address structures.
*
*                         ECB_FragmentLength
*                                   The size of the buffer to copy Group
*                                   (Multicast) address structures.
*
*
*                      On Exit:
*
*                         ECB_ImmediateAddress
*                                   Contains the Functional address currently
*                                   used by the MLID,  zero if the MLID does
*                                   (Multicast) address structures.
*                                   not use Functional addresses (e.g.,
*                                   Ethernet, FDDI), or the Functional address
*                                   is currently unused.
*
*                         ECB_DataLength
*                                   Contains the number of bytes transferred
*                                   into the buffer indicated by
*                                   ECB_FragmentAddress.  If the MLID does not
*                                   use Group addresses the ECB_DataLength
*                                   field will be set to zero.
*
*                                   If ODISTAT_OUT_OF_RESOURCES is returned.
*                                   This field will contain the size of the
*                                   buffer required to return the Multicast
*                                   address information.
*
*                         ECB_FragmentAddress
*                                   Points to the buffer passed in on entry.
*                                   The buffer has been filled in with a
*                                   series of Group (Multicast) address
*                                   structures used by the MLID.  If the MLID
*                                   does not use Group (Multicast) addresses
*                                   or there are no active the ECB_DataLength
*                                   field will be set to zero.  The address
*                                   structure is defined as follows:
*
*                                   typedef struct _GROUP_ADDR_LIST_NODE_
*                                   {
*                                       NODE_ADDR     GRP_ADDR;
*                                       UINT16        GRP_ADDR_COUNT;
*                                   }GROUP_ADDR_LIST_NODE;
*
*                         ECB_FragmentLength
*                                   Unchanged.
*
*
*   Notes:
*     The MulticastInfoECb's ESR will only be called if the function
*     returns ODISTAT_RESPONSE_DELAYED, any other return code implies
*     that the processing of the ECB is complete and the ECB contains
*     valid information (e.g., ODISTAT_OUT_OF_RESOURCES implies the
*     ECB_DATALength field contains the size of the buffer required
*     to return appropriate information.
*
*  Origin:	New routine.
*
************************************************************************/
ODISTAT	MediaGetMulticastInfo(
             struct _DRIVER_DATA_    *driverData,
             ECB                     *MulticastInfoECB)
{
	GROUP_ADDR_LIST_NODE	*mcEntry;
	SHARED_DATA		  		*sharedData;
	MEDIA_DATA				*mediaData;
	void						*dataPtr;
	UINT32					i, bufSizeNeeded;

	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* initialize fields */
	MulticastInfoECB->ECB_DataLength = 0;

    /* Set Functional address */
	 if ( mediaData->FunctionalAddressList.ActiveFunctionalBits )
	 	MulticastInfoECB->ECB_ImmediateAddress.nodeAddress[0] = 0xC0;
	 else
	 	MulticastInfoECB->ECB_ImmediateAddress.nodeAddress[0] = 0x00;

	 MulticastInfoECB->ECB_ImmediateAddress.nodeAddress[1] = 0x00;
	 *(UINT32 *)&(MulticastInfoECB->ECB_ImmediateAddress.nodeAddress[2]) =
						 		mediaData->FunctionalAddressList.ActiveFunctionalBits;

	/* Calculate the size of buffer needed. */
	for (i = 0, bufSizeNeeded = 0; i < mediaData->MaxMulticastAddresses; i++)
	 {
		/*Check if entry is in use.*/
		if (mcEntry->GRP_ADDR_COUNT != 0)
		 {
		 	bufSizeNeeded += sizeof(GROUP_ADDR_LIST_NODE);
		 }
		++mcEntry;
	 }
	
	/* Make sure there is enough room to copy data */
	if (MulticastInfoECB->ECB_Fragment[0].FragmentLength < bufSizeNeeded)
	{
		MulticastInfoECB->ECB_DataLength = bufSizeNeeded;
		return( ODISTAT_OUT_OF_RESOURCES );
	}

	/* Get data for MulticastInfoECB */
	dataPtr = MulticastInfoECB->ECB_Fragment[0].FragmentAddress;
	mcEntry = mediaData->MulticastAddressList;

	for (i = 0; i < mediaData->MaxMulticastAddresses; i++)
	{
		/*Check if entry is in use.*/
		if (mcEntry->GRP_ADDR_COUNT != 0)
		{
			/* Copy Entry into MulticastInfoECB */

			CMSMmemcpy(dataPtr, mcEntry,sizeof(GROUP_ADDR_LIST_NODE));
			dataPtr = (UINT8 *)dataPtr + sizeof(GROUP_ADDR_LIST_NODE);
		
  			/* Update amount of data copied. */
  			MulticastInfoECB->ECB_DataLength =
  				MulticastInfoECB->ECB_DataLength +
  				sizeof(GROUP_ADDR_LIST_NODE);
  		}
		++mcEntry;
	}
    return( ODISTAT_SUCCESSFUL );

}	/* End MediaGetMulticastInfo */

/**************************************************************************\
**
*	Name:
*		void	MediaSend(
*							ECB					*ecb,
*							CONFIG_TABLE		*configTable);
*
*	Description:
**   This function converts the ECB to a TCB, and passes the TCB
**   to the HSM's DriverSend routine.  Converting to TCB means
**   building a Media Access Header and verifying contiguous,
**   physically addressed fragments if the HSM is expecting it.
**
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		configTable -		Pointer to configuration table.
*
*	Values returned:
*
*	Implied parameters:
*		NONE
*
** See Also:
**
\**************************************************************************/
void	   MediaSend( ECB					*ecb,
						  CONFIG_TABLE		*configTable)
{
	DRIVER_DATA					*driverData;
	FRAME_DATA				   *frameData;
	SHARED_DATA 			   *sharedData;
	MEDIA_DATA					*mediaData;
	GENERIC_STATS				*statTable;
	TCB							*tcb;
   void			        	   *physTcb;	
	UINT32						pktSize = 0;
	UINT32						oldState;

		/* Initialize the Local Variables */
	frameData = CONFIG_TO_MSM(configTable);
	driverData = frameData->DriverData;
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/*Protect Data*/
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	/*Check if logical board is shutdown or if we already have */
	/*transmits waiting on the queue.*/

	if((sharedData->CMSMStatusFlags & TXQUEUED) ||
				(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
	 {
		/*Either the board is shutdown or */
		/*transmits are on the queue.*/

		if(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT)
		 {
			CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );

			/* The board is shutdown */
			/* Set the status and return the ECB. */
			ecb->ECB_Status = (UINT16)ODISTAT_MLID_SHUTDOWN;
			CLSL_FastSendComplete( ecb );
		 }
		else
		 {
			/* Transmits are on the queue. */
			/* Queue this send. */
			CMSMEnqueueSend( frameData, driverData, ecb );
			CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		 }
		return;
	 } /* End:Card is ShutDown or Tx Already Q'd */

	/* If we are not in a critical section we can send this */
	/* packet imediatly to the card. */
	if( sharedData->CMSMInCriticalSection )
	 {
		/*We are in a critical section*/
		/*Queue the send.*/
		CMSMEnqueueSend( frameData, driverData, ecb );
	 }
	else
	 {
		/* We are not in a critical section */
		/* Make sure the card has resources to process this send. */
		if( sharedData->CMSMTxFreeCount > 0 )
		 {
			/* We have resources. */
			/* This is here for DMA, or BusMaster boards */
			/* Check if DriverUsesDMA or DriverSupportsPhysFrags bits are Set. */
			/* If set get a Physically contiguous ECB */
			if ( sharedData->CMSMNeedContiguousECB )
				ecb = CMSMGetPhysicalECB( driverData, ecb );

			/* Disable the cards interrupts. */
			if( sharedData->CMSMAdapterIntDisabled == 0 )
				if( sharedData->CMSMDPBlockPtr->DriverDisableInterruptPtr )
				{
#ifdef TRACE_OUT
	CMSMOutMEON('S',0);
#endif
					sharedData->CMSMDPBlockPtr->DriverDisableInterruptPtr( driverData, FALSE );
				}

			++sharedData->CMSMAdapterIntDisabled;

			if(sharedData->CMSMDPBlockPtr->DriverSendWantsECBs != 0)
			 {
				/* check that the packet is not too big. */
				if( (ecb->ECB_DataLength <= configTable->MLIDCFG_BestDataSize) ||
					 ((( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK ) &&
					  (ecb->ECB_DataLength <= configTable->MLIDCFG_MaxFrameSize)) )
				 {
					/*Bump Queue Depth*/
					++(statTable->Gen_Qdepth);
					--(sharedData->CMSMTxFreeCount);
					++sharedData->CMSMInCriticalSection;

					CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

					/* Give the ECB to the HSM to process. */
					sharedData->CMSMDPBlockPtr->DriverSendPtr(
									driverData,
									configTable,
									(TCB *)ecb,
									pktSize,
									CMSMGetPhysical(ecb));

					/* Check to see if anything got queued up while we	*/
					/* were in DriverSend; if so service the queue now	*/
				   if (sharedData->CMSMStatusFlags & TXQUEUED)
				    {
				   	do
				       {
							/* CTokenTSMGetNextSend acquires his own protection. */
							/* No need to do it here. */
				      	tcb = CTokenTSMGetNextSend(
				                  driverData,
  					               &configTable,
					               &pktSize,
				                  &physTcb);

				         if( tcb )
							 {
				         	sharedData->CMSMDPBlockPtr->DriverSendPtr (
				                     driverData,
				                     configTable,
				                     tcb,
				                     pktSize,
				                     physTcb);
							 }
				     	 }
				      while (tcb && (sharedData->CMSMStatusFlags & TXQUEUED));
   				 }	/* End at least one TCB was Q'd */

					CMSMAcquireProtection(
						sharedData->CMSMProtectionObject,
						&oldState );

					--sharedData->CMSMInCriticalSection;
				 }
				else
				 {
					++(statTable->Gen_TxTooBig);
					/*If we are a DMA card then we need to return the ECB to the pool*/
					if (sharedData->CMSMNeedContiguousECB)
						ecb = CMSMReturnPhysicalECB(driverData, ecb);

					/*Set ECB status.*/
					ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
					CLSL_FastSendComplete(ecb);
				 }
			 }
			else
			 {
				/* Get a TCB and create the MAC Header. */
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];

				/* Put the BitSwap flag in the TCB */
				tcb->TCB_DriverWS[0] = frameData->BitSwapAddressFlag;

			/*	**********************************************************
				Due to Problem with Needing to checkin, Pass in FrameData
				as a cast through the ShareData Param, and it will recast
				in the MediaSend Routine.   _SROUTE_BUG_
				********************************************************** */
				pktSize = frameData->SendRoutine( (SHARED_DATA *)frameData, ecb,	tcb );
				ecb->ECB_DriverWorkspace.DWs_pval = (UINT32 *) frameData;

				if (sharedData->CMSMNeedContiguousECB)
				{
					tcb->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
					tcb->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&tcb->TCB_MediaHeader);
				}
						
				/* check that the packet is not to big. */
				if (pktSize	<= configTable->MLIDCFG_MaxFrameSize)
				 {
					/* Bump Queue Depth */
					--(sharedData->CMSMTxFreeCount);
					++(statTable->Gen_Qdepth);
					++sharedData->CMSMInCriticalSection;

					CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );

					/* Give the TCB to the HSM to process. */
					sharedData->CMSMDPBlockPtr->DriverSendPtr(
									driverData,
									configTable,
									tcb,
									pktSize,
									NULL);

					/* Check to see if anything got queued up while we	*/
					/* were in DriverSend; if so service the queue now	*/
				   if (sharedData->CMSMStatusFlags & TXQUEUED)
				    {
				   	do
				       {
							/* CTokenTSMGetNextSend acquires his own protection. */
							/* No need to do it here. */
				      	tcb = CTokenTSMGetNextSend(
				                  driverData,
  					               &configTable,
					               &pktSize,
				                  &physTcb);

				         if (tcb)
							 {
				         	sharedData->CMSMDPBlockPtr->DriverSendPtr (
				                     driverData,
				                     configTable,
				                     tcb,
				                     pktSize,
				                     physTcb);
							 }
				     	 }
				      while (tcb && (sharedData->CMSMStatusFlags & TXQUEUED));
   				} /* Something was Queued up. */

					CMSMAcquireProtection(
						sharedData->CMSMProtectionObject,
						&oldState );

					--sharedData->CMSMInCriticalSection;
				 }
				else
				 {
					++(statTable->Gen_TxTooBig);
					/* If we are a DMA card then we need to return the ECB to the pool */
					if( sharedData->CMSMNeedContiguousECB )
						ecb = CMSMReturnPhysicalECB(driverData, ecb);

         		/*Return TCB and link into TCB list.*/
         		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		         mediaData->TCBHead = tcb;

					/* Set ECB status. */
					ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
					CLSL_FastSendComplete( ecb );
				 } /* Packet turned out to be too big */
			 }	/* Driver Wanted TCB's */

			/* Reenable the cards interrupts. */
			--sharedData->CMSMAdapterIntDisabled;
			if( sharedData->CMSMAdapterIntDisabled == 0 )
				if( sharedData->CMSMDPBlockPtr->DriverEnableInterruptPtr )
				{
#ifdef TRACE_OUT
	CMSMOutMEON('s',0);
#endif
					sharedData->CMSMDPBlockPtr->DriverEnableInterruptPtr( driverData );
				}
		 }
		else
		 {
			/* We don't have any transmit resources available. */
			/* Queue the send. */
			CMSMEnqueueSend( frameData, driverData, ecb );
		 }
	 } /* Not in Critical Section, and Sent the Frame. */

	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

#ifdef OS_NIOS
	CLSL_ServiceEvents();
#endif

 	return;

}	/* End MediaSend */

/**************************************************************************\
**
*	Name:
*		UINT32	CMediaSend8022(
*******> This SHould be FRAME_DATA  *frameData,
******> _SROUTE_BUG_
*							SHARED_DATA			*sharedData,
*							ECB					*ecb,
*							TCB					*tcb);
*
*	Description:
*		This function constructs an 802.2 media header. It will fill
**                fill in all the TCB fields.
**
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
*
**
** Return Value:
**				PacketSize	Size of the packet to transmit.
**
** See Also:
**
\**************************************************************************/
UINT32 CMediaSend8022(
			/*	**********************************************************
				Due to Problem with Needing to checkin, Pass in FrameData
				as a cast through the ShareData Param, and it will recast
				in the MediaSend Routine.   _SROUTE_BUG_
				********************************************************** */
 /* 				FRAME_DATA *frameData, */
				SHARED_DATA *sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	TOKEN_MEDIA_HEADER	*machdr;
	UINT32					routeSize = NULL;
   UINT8       		   *pprotocolID;
   UINT8          		*pheader;
   UINT8          		loopcount, i;
	UINT8						*routeInfo, *pRouteInfo;

	FRAME_DATA 				*frameData;        /* _SROUTE_BUG_WORK_AROUND */
	frameData = (FRAME_DATA *)sharedData;

	/* set up fragment structure. */
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
	
	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		/*fill in MAC header.*/

		machdr = &(tcb->TCB_MediaHeader);
		machdr->MH_AccessControl = 00;

		/* If this is a priority packet set the priority bits in the AC field */

		if(((ecb->ECB_StackID & NON_STACKID_BIT) == NON_STACKID_BIT) &&
		 ((ecb->ECB_StackID & 0x0007) != 0x0007))
		{
			machdr->MH_AccessControl |= (UINT8)(((~ecb->ECB_StackID) & 0x0007) << 5);
		}

	    machdr->MH_FrameControl  = FC_LLC_FRAME;

		if( tcb->TCB_DriverWS[0] != 0 )
		{
         /*We need to Swap the address*/
         machdr->MH_Destination.nodeAddress[0]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[0]];
         machdr->MH_Destination.nodeAddress[1]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[1]];
         machdr->MH_Destination.nodeAddress[2]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[2]];
         machdr->MH_Destination.nodeAddress[3]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[3]];
         machdr->MH_Destination.nodeAddress[4]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[4]];
         machdr->MH_Destination.nodeAddress[5]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[5]];
       }
      else
       {
   		COPY_ADDR( &machdr->MH_Destination,	&ecb->ECB_ImmediateAddress	);
       }

		/* Now see if we need Routing Info for this address. */
		if( frameData->ConfigTable.MLIDCFG_SourceRouting )
			routeSize = frameData->ConfigTable.MLIDCFG_SourceRouting(
											ecb->ECB_BoardNumber,
											machdr->MH_Destination.nodeAddress,
											(void **)&pRouteInfo,
											TRUE );
		if( routeSize )
		 {
			/* We need to copy routeInfo to the MAC Header. */
			routeInfo = &machdr->MH_DSAP;
			for (i = 0; i < routeSize; ++i)
				routeInfo[i] = pRouteInfo[i];

			/* Now set the route bit in the Source address. */
         machdr->MH_Source.nodeAddress[0] |= SOURCE_ROUTING_BIT;

			/* Now adjust the pointer to the MAC Header. */
			pRouteInfo = (UINT8 *)machdr + routeSize;
			machdr = (TOKEN_MEDIA_HEADER *)pRouteInfo;

		 }	/* Source Routing was provided */
		else
		 {
			/* Make sure that the route bit is cleared. */
         machdr->MH_Source.nodeAddress[0] &= ~SOURCE_ROUTING_BIT;
		 }	/* No Source routing this time */

		/* Set the TCB HeaderLen (Nominal Header + Route bytes) */
      tcb->TCB_MediaHeaderLen =  SIZE_TOKEN + routeSize;

		/* PID[0] = Number of 802.2 bytes. 								*/
		/* If PID[0] = 0  then PID[5] = DSAP, SSAP, CTL0. 			*/
		/* if PID[0] = 1  then PID[4] = DSAP, SSAP 					*/
		/* if PID[0] = 2  then PID[3] = DSAP, SSAP, CTRL0 			*/
		/* if PID[0] = 3  then PID[2] = DSAP, SSAP, CTRL0, CTRL1 */
		/* if PID[0] = 4  then PID[1] = DSAP, SSAP, CTRL0, CTRL1, First byte of Data. */
		if(ecb->ECB_ProtocolID.protocolID[0] == 0)
		 {
			/* Nominal LLC Header */
			machdr->MH_DSAP  = ecb->ECB_ProtocolID.protocolID [5];
			machdr->MH_SSAP  = ecb->ECB_ProtocolID.protocolID [5];
			machdr->MH_Ctrl0 = 03;
		 }
		else
		 {
			/* explicit Header Given  -- Loop Count 0 Based */
         loopcount   = (UINT8)(ecb->ECB_ProtocolID.protocolID[0]);
			pprotocolID = &(ecb->ECB_ProtocolID.protocolID[5 - loopcount]);
			pheader     = &(machdr->MH_DSAP);

			for(i = 0; i <= loopcount; ++i )
				pheader[i] = pprotocolID[i];

			tcb->TCB_MediaHeaderLen += loopcount - 2;

       }/* End Explicit Header */
   } /* end Non-Raw Send Packet */

   tcb->TCB_DataLen = ecb->ECB_DataLength + tcb->TCB_MediaHeaderLen;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;
	return( tcb->TCB_DataLen );

}	/* End CTokenSend8022 */

/**************************************************************************\
**
*	Name:
*		UINT32 CMediaSendSNAP(
*******> This SHould be FRAME_DATA  *frameData,
******> _SROUTE_BUG_
*				SHARED_DATA *sharedData,
*				ECB 			*ecb,
*				TCB 			*tcb)
**
*	Description:
**                This function constructs a SNAP media header.  It will
**                fill in all the TCB fields.
**
*	Parameters in:
*		ecb -					Pointer to Event Control Block.
*		tcb -					Pointer to Transmit Control Block.
**
** Return Value:
**				PacketSize	Size of the packet to transmit.
**
** See Also:
**
\**************************************************************************/
UINT32 CMediaSendSNAP(
			/*	**********************************************************
				Due to Problem with Needing to checkin, Pass in FrameData
				as a cast through the ShareData Param, and it will recast
				in the MediaSend Routine.   _SROUTE_BUG_
				********************************************************** */
 /* 				FRAME_DATA *frameData, */
				SHARED_DATA *sharedData,
				ECB 			*ecb,
				TCB 			*tcb)
{
	TOKEN_MEDIA_HEADER	*machdr;
	UINT32			routeSize = 0, i;
	UINT8				*routeInfo, *pRouteInfo;

	FRAME_DATA 				*frameData;        /* _SROUTE_BUG_WORK_AROUND */
	frameData = (FRAME_DATA *)sharedData;


   machdr = &(tcb->TCB_MediaHeader);

	/*set up fragment structure.*/
	tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
	
	/******************************************************************
		Check if raw send.--
		Raw Send Priority StackID values increase with lessor priority
	*******************************************************************/
	if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) >= NON_STACKID_RAW_SEND_MASK )
	 {
		tcb->TCB_MediaHeaderLen = 0;
	 }
	else
	 {
		/* fill in MAC header. */

		machdr->MH_AccessControl = 00;

		/* If this is a priority packet set the priority bits in the AC field */

		if(((ecb->ECB_StackID & NON_STACKID_BIT) == NON_STACKID_BIT) &&
		 ((ecb->ECB_StackID & 0x0007) != 0x0007))
		{
			machdr->MH_AccessControl |= (UINT8)(((~ecb->ECB_StackID) & 0x0007) << 5);
		}

	    machdr->MH_FrameControl  = FC_LLC_FRAME;

      if (tcb->TCB_DriverWS[0] != 0)
       {
         /*We need to Swap the address*/
         machdr->MH_Destination.nodeAddress[0]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[0]];
         machdr->MH_Destination.nodeAddress[1]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[1]];
         machdr->MH_Destination.nodeAddress[2]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[2]];
         machdr->MH_Destination.nodeAddress[3]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[3]];
         machdr->MH_Destination.nodeAddress[4]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[4]];
         machdr->MH_Destination.nodeAddress[5]
            = MSMBitSwapTable[ecb->ECB_ImmediateAddress.nodeAddress[5]];
       }
      else
       {
   		COPY_ADDR( &machdr->MH_Destination, &ecb->ECB_ImmediateAddress	);
       }

		/* Now see if we need Routing Info for this address. */
		if( frameData->ConfigTable.MLIDCFG_SourceRouting )
			routeSize = frameData->ConfigTable.MLIDCFG_SourceRouting(
							ecb->ECB_BoardNumber,
							machdr->MH_Destination.nodeAddress,
							(void **)&pRouteInfo,
							TRUE);
		if( routeSize )
		 {
			/* We need to copy routeInfo to the MAC Header. */
			routeInfo = &machdr->MH_DSAP;
			for (i = 0; i < routeSize; ++i)
				routeInfo[i] = pRouteInfo[i];

			/* Now set the route bit in the Source address. */
         machdr->MH_Source.nodeAddress[0] |= SOURCE_ROUTING_BIT;

			/* Now adjust the pointer to the MAC Header. */
			pRouteInfo = (UINT8 *)machdr + routeSize;
			machdr = (TOKEN_MEDIA_HEADER *)pRouteInfo;
		 }
		else
		 {
			/* Make sure that the route bit is cleared. */
         machdr->MH_Source.nodeAddress[0] &= ~SOURCE_ROUTING_BIT;
		 }
		/*************************************************************
			Set these Values a byte at a time to maintain Portability
		***************************************************************/
		machdr->MH_DSAP = 0xAA;
		machdr->MH_SSAP = 0xAA;
		machdr->MH_Ctrl0 = 0x03;
		machdr->MH_SNAP[0] =  ecb->ECB_ProtocolID.protocolID[1];
		COPY_UINT32( &(machdr->MH_SNAP[1]),
						 &(ecb->ECB_ProtocolID.protocolID[2]) );
		tcb->TCB_MediaHeaderLen = SIZE_TOKENSNAP + routeSize;
	 }	/* End Non-Raw_Send packet */

   tcb->TCB_DataLen = ecb->ECB_DataLength + tcb->TCB_MediaHeaderLen;
	tcb->TCB_BoardNumber = ecb->ECB_BoardNumber;
	return( tcb->TCB_DataLen );
}	/* End CMediaSendSnap */

/**************************************************************************\
**
*	Name:
*		TCB	*CTokenTSMGetNextSend(
*							DRIVER_DATA			*driverData,
*							CONFIG_TABLE		**configTable,
*							UINT32				*pktSize,
*							void					**physicalEcb);
*
*	Description:
*         This function returns a TCB if the TSM has one queued up.
**
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*
*	Parameters out:
*		configTable - 			Pointer to configuration table
*		pktSize -				Pointer to hold packet size
*		physicalEcb -			Pointer to hold physical ECB
*
*	Values returned:
*		Pointer to TCB if we had one queued
*		Otherwise NULL
**
** See Also:
**
\**************************************************************************/
TCB		*CTokenTSMGetNextSend(
				DRIVER_DATA	   		*driverData,
				CONFIG_TABLE			**configTable,
				UINT32				   *pktSize,
				void					   **physicalEcb)
{
	FRAME_DATA					*frameData;
	SHARED_DATA			      *sharedData;
	MEDIA_DATA					*mediaData;
	TCB							*tcb = NULL;
	ECB							*ecb;
	UINT32						STATUS = ODISTAT_ITEM_NOT_PRESENT;
	UINT32						oldState;

	sharedData = DADSP_TO_CMSMADSP(driverData);

	/* Protect Our Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	if( (ecb = sharedData->CMSMSendListHead) != NULL )
	 {
		if( sharedData->CMSMTxFreeCount > 0 )
		 {
			STATUS = ODISTAT_SUCCESSFUL;

			/* Get ECB off of queue.  Check if more ECBs on queue. */
			--(sharedData->CMSMTxFreeCount);
			sharedData->CMSMSendListHead = ecb->ECB_NextLink;
			frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;
			*configTable = &frameData->ConfigTable;

			if( sharedData->CMSMSendListHead == NULL )
			 {
				/*No more ECBs on the queue. Set up queue tail.*/
				sharedData->CMSMSendListTail = (ECB *)&sharedData->CMSMSendListHead;
				sharedData->CMSMStatusFlags &= ~TXQUEUED;
			 }

			/* Check if we need to get physical addresses for DMA cards */
			if (sharedData->CMSMNeedContiguousECB)
				ecb = CMSMGetPhysicalECB( driverData, ecb );
			
			if( sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0 )
			 {
				mediaData = sharedData->CMSMMediaDataPtr;
				*physicalEcb = NULL;

				/* Get a TCB and call Media Send routine to fill it out. */
				tcb = mediaData->TCBHead;
				mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];

				/* Put the BitSwap flag in the TCB */
				tcb->TCB_DriverWS[0] = frameData->BitSwapAddressFlag;

		/*	**********************************************************
			Due to Problem with Needing to checkin, Pass in FrameData
			as a cast through the ShareData Param, and it will recast
			in the MediaSend Routine.   _SROUTE_BUG_
			********************************************************** */
				*pktSize = frameData->SendRoutine( (SHARED_DATA *)frameData, ecb, tcb);

				/* if the adapter needs physical frags then set the DriverWs + 8 */
				/* to point to the Media Header */
				if( sharedData->CMSMNeedContiguousECB )
				 {
					tcb->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
					tcb->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&tcb->TCB_MediaHeader);
				 }
			 }	/* Driver Wants TCBs */
			else
			 {
				*physicalEcb = CMSMGetPhysical( ecb );
				tcb = (TCB *)ecb;
			 } /* Driver Wants ECBs */
		 }	/* TxFreeCount Greater than 0 */
	 } /* end ECB on Q */

	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
	return( tcb );

}	/* End CTokenTSMGetNextSend */

/**************************************************************************\
**
*	Name:
*		void	*CTokenTSMFastSendComplete(
*							DRIVER_DATA			*driverData,
*                    TCB               *tcb,
                     UINT32            transmitStatus);
*
*	Description:
*		This function returns a TCB to the TSM. The TSM will extract the
*		ECB from the TCB, return the TCB to the TCB pool, and return
*		the ECB to the LSL. The LSL will give the ECB back to its owner
*		by calling its ESR routine.
*
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*		tcb - 					Pointer to Transmit Control block.
*     transmitStatus -     Flag used to indicate if packet was actually sent.
*									Non-zero means packet was undeliverable.
*
*	Values returned:
*
*	Implied parameters:
*
\**************************************************************************/
void		CTokenTSMFastSendComplete(
				DRIVER_DATA			   *driverData,
				TCB						*tcb,
            UINT32               transmitStatus)
{
	FRAME_DATA						*frameData;
	SHARED_DATA			         *sharedData;
	MEDIA_DATA						*mediaData;
	GENERIC_STATS					*statsTable;
	ECB								*ecb, *nullEcb = NULL;
	UINT32							count, oldState, i;

	/* Setup Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject, &oldState );

	/* Track the stats */
	statsTable = sharedData->CMSMGenericStatsPtr;
	--(statsTable->Gen_Qdepth);

	/* Is HSM ECB aware? */
 	if(sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0)
	 {
		/* HSM is not ECB aware. */
		/* Set count to hold the Total Bytes transmitted. */
		count = tcb->TCB_DataLen;

		/* Get ptr to ECB. */
		ecb = (ECB *)((UINT32)(tcb->TCB_FragBlockPtr) -
									(UINT32)(&(nullEcb->ECB_FragmentCount)));

		/* Get Frame Data Pointer */
		frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;

		/*If we are a DMA card then we need to return the ECB to the pool*/
		if( sharedData->CMSMNeedContiguousECB )
		{
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

			/* Adjust TCB to use logical fragment list */
			tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
		}

		/*call The Transmit monitor*/
		if( frameData->TransmitRoutine )
			frameData->TransmitRoutine( tcb );

		/*Return TCB and Link into TCB list.*/
		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		mediaData->TCBHead = tcb;

	 } /* End HSM wants TCBs */
	else
	 {
		/* HSM is ECB aware. CurTCB is really an ECB. */
		ecb = (ECB *)tcb;

		/* If we are a DMA card then we need to return the ECB to the pool */
		if( sharedData->CMSMNeedContiguousECB )
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

		/* Find correct FrameData */
		frameData = NULL;
		for (i = 0; i < 4; i++)
			if( (sharedData->CMSMVirtualBoardLink[i])->MLIDCFG_BoardNumber ==
					ecb->ECB_BoardNumber)
				{
					frameData = CONFIG_TO_MSM(sharedData->CMSMVirtualBoardLink[i]);
					break;
				}

		/* call The Transmit monitor */
		if(frameData->TransmitRoutine )
		 {
			/*Get a TCB and call Media Send routine to fill it out.*/
			tcb = mediaData->TCBHead;
			mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];

			/* Put the BitSwap flag in the TCB */
			tcb->TCB_DriverWS[0] = frameData->BitSwapAddressFlag;

			/*	**********************************************************
				Due to Problem with Needing to checkin, Pass in FrameData
				as a cast through the ShareData Param, and it will recast
				in the MediaSend Routine.   _SROUTE_BUG_
				********************************************************** */
			frameData->SendRoutine( (SHARED_DATA *)frameData, ecb, tcb );
			frameData->TransmitRoutine( tcb );

			/*Return TCB and Link into TCB list.*/
			tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
			mediaData->TCBHead = tcb;

		 } /* Call the TxMonitor */

		count = ecb->ECB_DataLength;

		/******************************************************************
			Check if Not raw send Add in Header bytes.--
			Raw Send Priority StackID values increase with lessor priority
		*******************************************************************/
		if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) < NON_STACKID_RAW_SEND_MASK )
		 {
			count += frameData->MediaHeaderSize;
			if( ecb->ECB_ProtocolID.protocolID[0] == 0x03 )
				++count;
		 }
	 } /* HSM wanted ECBs */

	/*Set ECB status.*/
	if (transmitStatus)
		ecb->ECB_Status = ODISTAT_CANCELED;
	else
	{
		++(statsTable->Gen_TotalTxPackets);

		statsTable->Gen_TotalTxOkByteLow += count;

		/* Check for overflow in the lower 32 bits. */
	   if (statsTable->Gen_TotalTxOkByteLow < count)
   	   statsTable->Gen_TotalTxOkByteHigh++;

		if (frameData->BitSwapAddressFlag)
			statsTable->Gen_TotalGroupAddrTx +=
						 (ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x01);
		else
			statsTable->Gen_TotalGroupAddrTx +=
						 ((ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x80) >> 7);

		ecb->ECB_Status = ODISTAT_SUCCESSFUL;
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
  	CLSL_FastSendComplete( ecb );
	return;

}	/* End CTokenTSMFastSendComplete */

/**************************************************************************\
**
*	Name:
*		void	*CTokenTSMSendComplete(
*							DRIVER_DATA			*driverData,
*							TCB					*tcb,
                     UINT32            transmitStatus);
*
*	Description:
*		This function returns a TCB to the TSM. The TSM will extract the
*		ECB from the TCB, return the TCB to the TCB pool, and return
*		the ECB to the LSL. The LSL will queue up the ECB until
* 		LSLServiceEvents is called. The HSM will have to initiate this
*		by calling MSMServiceEvents when it is safe for the LSL to give
*		the queued ECBs back to their owners by calling their ESR routines.
*
*	Parameters in:
*		driverData -			Pointer to HSMs adapter data space.
*		tcb - 					Pointer to Transmit Control block.
*     transmitStatus -     Flag used to indicate if packet was acutally sent.
*                          Non-zero means packet undeliverable.
*
*	Values returned:
*
*	Implied parameters:
*
*  Notes:
**
\**************************************************************************/
void		CTokenTSMSendComplete(
				DRIVER_DATA			   *driverData,
            TCB                  *tcb,
            UINT32               transmitStatus)
{
	FRAME_DATA						*frameData;
	SHARED_DATA			         *sharedData;
	MEDIA_DATA						*mediaData;
	GENERIC_STATS		         *statsTable;
	ECB								*ecb, *nullEcb = NULL;
	UINT32							count, oldState, i;

	/* Setup the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

	statsTable = sharedData->CMSMGenericStatsPtr;
	--(statsTable->Gen_Qdepth);

	/*Is HSM ECB aware?*/
	if( sharedData->CMSMDPBlockPtr->DriverSendWantsECBs == 0 )
	 {
		/* HSM is not ECB aware. */
		/* Set count to hold the Total Bytes transmitted. */
		count = tcb->TCB_DataLen;

		/*Get ptr to ECB.*/
		ecb = (ECB *)((UINT32)(tcb->TCB_FragBlockPtr) -
									(UINT32)(&(nullEcb->ECB_FragmentCount)));

		/* Get Frame Data Pointer */
		frameData = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;

		/*If we are a DMA card then we need to return the ECB to the pool*/
		if (sharedData->CMSMNeedContiguousECB)
		 {
			ecb = CMSMReturnPhysicalECB(driverData, ecb);

			/* Adjust TCB to use logical fragment list */
			tcb->TCB_FragBlockPtr = (TCB_FRAGMENT_BLOCK *)(&ecb->ECB_FragmentCount);
		 }

		/*call The Transmit monitor*/
		if( frameData->TransmitRoutine )
			frameData->TransmitRoutine( tcb );

		/*Return TCB and Link into TCB list.*/
		tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
		mediaData->TCBHead = tcb;
	 }	/* End HSM wanted TCBs */
	else
	 {
		/*HSM is ECB aware. CurTCB is really an ECB.*/
		ecb = (ECB *)tcb;

		/*If we are a DMA card then we need to return the ECB to the pool*/
		if( sharedData->CMSMNeedContiguousECB )
			ecb = CMSMReturnPhysicalECB( driverData, ecb );

		/* Find correct FrameData */
		frameData = NULL;
		for (i = 0; i < 4; i++)
			if( (sharedData->CMSMVirtualBoardLink[i])->MLIDCFG_BoardNumber ==
					ecb->ECB_BoardNumber)
				{
					frameData = CONFIG_TO_MSM(sharedData->CMSMVirtualBoardLink[i]);
					break;
				}

		/*call The Transmit monitor*/
		if( frameData->TransmitRoutine )
		 {
			/*Get a TCB and call Media Send routine to fill it out.*/
			tcb = mediaData->TCBHead;
			mediaData->TCBHead = (TCB *)tcb->TCB_DriverWS[0];

			/* Put the BitSwap flag in the TCB */
			tcb->TCB_DriverWS[0] = frameData->BitSwapAddressFlag;

			/*	**********************************************************
				Due to Problem with Needing to checkin, Pass in FrameData
				as a cast through the ShareData Param, and it will recast
				in the MediaSend Routine.   _SROUTE_BUG_
				********************************************************** */
			frameData->SendRoutine( (SHARED_DATA *)frameData, ecb, tcb );
			frameData->TransmitRoutine( tcb );

			/*Return TCB and Link into TCB list.*/
			tcb->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
			mediaData->TCBHead = tcb;
		 }	/* End call the TxMonitor */

		count = ecb->ECB_DataLength;

		/******************************************************************
			Check if Not raw send.--
			Raw Send Priority StackID values increase with lessor priority
		*******************************************************************/
		if(( ecb->ECB_StackID & NON_STACKID_RAW_SEND_MASK) < NON_STACKID_RAW_SEND_MASK )
		 {
			count += frameData->MediaHeaderSize;
			if( ecb->ECB_ProtocolID.protocolID[0] == 0x03 )
				++count;
		 }
	 }/* HSM Wants ECBs */

	if (transmitStatus)
		ecb->ECB_Status = ODISTAT_CANCELED;
	else
	{
		++(statsTable->Gen_TotalTxPackets);

		statsTable->Gen_TotalTxOkByteLow += count;

		/* Check for overflow in the lower 32 bits. */
	   if (statsTable->Gen_TotalTxOkByteLow < count)
   	   statsTable->Gen_TotalTxOkByteHigh++;

		if (frameData->BitSwapAddressFlag)
			statsTable->Gen_TotalGroupAddrTx +=
						 (ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x01);
		else
			statsTable->Gen_TotalGroupAddrTx +=
						 ((ecb->ECB_ImmediateAddress.nodeAddress[0] & 0x80) >> 7);

		ecb->ECB_Status = ODISTAT_SUCCESSFUL;
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
  	CLSL_SendComplete( ecb );
	return;

}	/* End CTokenTSMSendComplete */

/**************************************************************************\
**
*	Name:
*		RCB	*CTokenTSMFastProcessGetRCB(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						newRcbSize);
*
*	Description:
*		This routine is called by HSM's that have completely filled
*		in a pre-allocated RCB and is ready to be passed the LSL/
*		protocol stack. If this function can pass back a new RCB
*		to use, it will. This call will give the packet to the protocol
*		stack if one wants it and could take a while to return.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Receive Control Block
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		newRcbSize -		Size of new RCB to be returned
*
*	Values returned:
*		New Receive Control Block for HSM to reuse if we can get one,
*		otherwise NULL.
*
*	Implied parameters:
*		NONE
*
*  Notes:
**
\**************************************************************************/
RCB 		*CTokenTSMFastProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize)
{
	RCB * retRCB;

	retRCB = CTokenTSMProcessGetRCB(
					driverData,
					rcb,
					pktSize,
					rcvStatus,
					newRcbSize);

#ifdef OS_NIOS
	CLSL_ServiceEvents();
#endif
	return( retRCB );

}	/*End CTokenTSMFastProcessGetRCB*/

/**************************************************************************\
**
*	Name:
*		RCB	*CTokenTSMProcessGetRCB(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						newRcbSize);
*
*	Description:
*		This routine is called by HSM's that have completely filled
*		in a pre-allocated RCB and is ready to be passed the LSL/
*		protocol stack. If this function can pass back a new RCB
*		to use, it will.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Receive Control Block
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		newRcbSize -		Size of new RCB to be returned
*
*	Values returned:
*		New Receive Control Block for HSM to reuse if we can get one,
*		otherwise NULL.
*
*	Implied parameters:
*		NONE
*
*  Notes:
**
\**************************************************************************/
RCB *CTokenTSMProcessGetRCB(
				DRIVER_DATA          *driverData,
				RCB			 			*rcb,
				UINT32					pktSize,
				UINT32					rcvStatus,
				UINT32					newRcbSize)
{
	FRAME_DATA					*frameData = NULL;
	SHARED_DATA				   *sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statsTable;
	TOKEN_MEDIA_HEADER		*lookAheadData, *macData;
	GROUP_ADDR_LIST_NODE		*mcEntry;
	void							*physicalAddress;
	void							*routeInfo;
	UINT32						receptiontype = 0;
	UINT32						headerSize;
	UINT32						srFieldLen = 0;
	LOOKAHEAD					lkAheadStruc;
	
	/* Initialize Local Variables */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
	statsTable = sharedData->CMSMGenericStatsPtr;

	macData = (TOKEN_MEDIA_HEADER *)(rcb + 1);

     /* Set the value to the number of bytes to skip to the begining
        of the FC field. In some busmaster implementations,
        this value (in protocol work space) can be as high as 128 bytes
        for chips which have poor alignment capabilities.*/
	macData = (TOKEN_MEDIA_HEADER *)((UINT32)macData +
				(UINT32)((ECB *)rcb)->ECB_ProtocolWorkspace.PWs_i32val[0]);
  ((ECB *)rcb)->ECB_ProtocolWorkspace.PWs_i32val[0] = 0;

	/*Make sure this is NOT a packet that we sent*/
	if( !((*(UINT32*) &macData->MH_Source.nodeAddress[2] ^
         *(UINT32*) &sharedData->CMSMPhysNodeAddress [2]) ||
        ((*(UINT16*) &macData->MH_Source.nodeAddress[0] & ~SOURCE_ROUTING_BIT) ^
         *(UINT16*) &sharedData->CMSMPhysNodeAddress [0])) )
	{
   		if( (macData->MH_FrameControl & FC_FRAME_TYPE_BITS) != FC_MAC_FRAME )
		{
			/* This packet was sent by us and is non-MAC frame. */
			return( rcb );
		}
	}

	/* Check to see if MAC packet. */
   if( (macData->MH_FrameControl & FC_FRAME_TYPE_BITS) == FC_MAC_FRAME )
	 {
		/* This is a MAC Packet. */
		headerSize = SIZE_MACTOK;
      receptiontype = ECB_MACFRAME;
      configTable = sharedData->CMSMDefaultVirtualBoard;
  		frameData = CONFIG_TO_MSM(configTable);
		COPY_ADDR( &lkAheadStruc.LkAhd_ProtocolID, &MACToken );
	 }
	else
    {
			/* This is a NON-MAC Frame */
			/* Check if Source Routing included */
         if (macData->MH_Source.nodeAddress[0] & SOURCE_ROUTING_BIT)
          {
            /*This packet has Source Route Info*/
            lookAheadData = (TOKEN_MEDIA_HEADER *) ((UINT32) macData +
					               (UINT32) (macData->MH_DSAP & SOURCE_SIZE_MASK));
            headerSize = (UINT32) (macData->MH_DSAP & SOURCE_SIZE_MASK);
				srFieldLen = headerSize;
   			receptiontype |= DT_SOURCE_ROUTE;
          }  /* Sroute info included. */
         else
          {
            lookAheadData = macData;
            headerSize    = 0;
          } /* No Sroute */

      	/*Determine the frame type of the packet.*/
	      /*	TOKEN-RING:      AC FC DA, SA, RI, DSAP, SSAP, 0x03, data...*/
   	   /*	TOKEN-RING_SNAP: AC FC DA, SA, RI, 0xAA, 0xAA, 0x03, PID, data...*/
   		if (((*(UINT16*) &lookAheadData->MH_DSAP) == 0xAAAA) &&
            (lookAheadData->MH_Ctrl0  == 0x3))
	    {	
   			/* type is SNAP */	

			headerSize += SIZE_TOKENSNAP;

	   		/* Get frame data space and copy 5 bytes following */	
		   	/* DSAP, SSAP and Ctl0 to PID. */
		
			if(configTable = sharedData->CMSMVirtualBoardLink[TSNAP_INTERNALID])
			{
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
					frameData = CONFIG_TO_MSM(configTable);
				   	lkAheadStruc.LkAhd_ProtocolID.protocolID[0] = 0;
		            lkAheadStruc.LkAhd_ProtocolID.protocolID[1] = lookAheadData->MH_SNAP[0];
					COPY_UINT32( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[2]),
								 &(lookAheadData->MH_SNAP[1]) );
				}
			}/* End of frame loaded and not shutdown */
		}	/* Packet is Snap Type */

		else	
		{	
	   		/* type is TOKEN-RING (802.2 over 802.5) */	

			headerSize += SIZE_TOKEN;	/* Default to TYPE I. */


			if (!(lookAheadData->MH_Ctrl0 & S_OR_U_FORMAT) || !(lookAheadData->MH_Ctrl0 & U_FORMAT))	
  			{	
   				++headerSize;				/* Change to type 2	 */
	   			receptiontype |=  DT_8022_TYPE_II;	
			}	

			else
				receptiontype |= DT_8022_TYPE_I;

  	   		/* Get frame data space and copy DSAP field to PID. */	

			if(configTable = sharedData->CMSMVirtualBoardLink[TOKEN_INTERNALID])	
			{
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
		   			frameData = CONFIG_TO_MSM(configTable);
					*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
				   	lkAheadStruc.LkAhd_ProtocolID.protocolID[4]  = 0;
				   	lkAheadStruc.LkAhd_ProtocolID.protocolID[5]  = lookAheadData->MH_DSAP;
				}
			}/* End of frame loaded and not shutdown */
	   	} /* Packet is Type Token-Ring */
	
		if( frameData == NULL )
		{
			configTable = sharedData->CMSMDefaultVirtualBoard;
			frameData = CONFIG_TO_MSM(configTable);
			rcvStatus |= PAE_NOT_ENABLED_BIT;
			statsTable->Gen_RxMiscError++;

			/* zero out Protocol ID field */

			*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
			*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
       }

		/****************************************/
   	/* We have determined the frame type.	 */
		/* Check the destination address.		 */
		/* Check if multicast bit set.			 */
		/****************************************/
		if( macData->MH_Destination.nodeAddress[0] & MULTICASTBIT )
   	 {
         statsTable->Gen_TotalGroupAddrRx++;

	   	/*Check if BroadCast Remember it is in HIGH-Low Order */
  		  	if ( ((*(UINT16 *)&macData->MH_Destination.nodeAddress[0] == 0x00C0) ||
				   (*(UINT16*)&macData->MH_Destination.nodeAddress[0] == 0xFFFF)) &&				
				(*(UINT32 *)&(macData->MH_Destination.nodeAddress[2]) == BROADCAST) )	
				receptiontype |= ( ECB_BROADCAST | ECB_MULTICAST );
   		else
	   	 {
				/* if Functional bit is zero, then better be Group-functional */
				if ( (!((*(UINT16 *)(&macData->MH_Destination.nodeAddress[2])) & 0x0080))
							&&
				     ( *(UINT16*)macData->MH_Destination.nodeAddress != 0x00C0 ) )
					receptiontype |= ECB_GLOBALERROR;

				/* We don't need to filter...
					if Not in PromiscuousMode AND HardwareFiltering is ON. (Both Bits.) */
				if ( (!mediaData->PromiscuousMode) &&
					  (configTable->MLIDCFG_Flags & (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)) ==
					  (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT) )
				 {
					receptiontype |= ECB_MULTICAST;
				 }
				else
				 {
					/*Check if multicast is in our table.*/
		  			if ( FindAddressInMCTable( sharedData,
														(NODE_ADDR *)macData->MH_Destination.nodeAddress,
														&mcEntry) == ODISTAT_SUCCESSFUL )
			   		receptiontype |= ECB_MULTICAST;
				   else
					 {
						receptiontype |= ECB_MULTICASTREMOTE;

						/* If not in Promiscous Mode filter out the Packet */
 						if( !mediaData->PromiscuousMode )
							return( rcb );
					 } /* Remote Multicast, Drop if not in Promiscuous Mode. */
				 }	/* Need to check if in Table */
		 	 }	/* Packet has a Group Address --Not Broadcast*/
  	    } /* Packet is a Multicast type */
  		else
       {
      	if( mediaData->PromiscuousMode )
			 {
		   	if( ADDR_EQUAL( &macData->MH_Destination,
									 &configTable->MLIDCFG_NodeAddress) )
	   		  	receptiontype |= ECB_UNICASTDIRECT;
   		   else
					receptiontype |= ECB_UNICASTREMOTE;
			 }
 	   	else
   	   	receptiontype |= ECB_UNICASTDIRECT;
       }	/* Packet has Unicast Address */
	 }	/* End Non-MAC Frame */

  	/*Make sure that packet is not too small. */
	if( pktSize < headerSize )
	 {
		pktSize = headerSize + 3;
		rcvStatus |= PAE_MALFORMED_BIT;
      statsTable->Gen_RxTooSmall++;
	 }

   /* Make sure that packet is not to large */
	if( pktSize > configTable->MLIDCFG_MaxFrameSize)
	 {
		rcvStatus |= PAE_TOO_BIG_BIT;	
	   statsTable->Gen_RxTooBig++;
	 }

  	/* Check to see if route module loaded. */
   if ( configTable->MLIDCFG_SourceRouting )
	 {
  		/* Inform route module of This route. */
      routeInfo = &macData->MH_DSAP;
	   configTable->MLIDCFG_SourceRouting(
						configTable->MLIDCFG_BoardNumber,
  						macData->MH_Source.nodeAddress,
   					&routeInfo,
	   				FALSE);
	 }
	else		/* Watch out for missing Route.NLM	*/
	 {
		if (srFieldLen > 2)
			receptiontype = DT_SOURCE_ROUTE;		/* Make this bit exclusive	*/
	 }

   if( (UINT8)rcvStatus )
		receptiontype = ECB_GLOBALERROR;	/* This is an exclusive bit 	*/
	else
	++(statsTable->Gen_TotalRxPackets);	/* Bump counters for Good Packets */

	/*fill in the lookahead structure.*/
	pktSize -= headerSize;				/* set for size fields. */
	lkAheadStruc.LkAhd_MediaHeaderPtr            = (UINT8 *)macData;
	lkAheadStruc.LkAhd_MediaHeaderLen            = headerSize;
	lkAheadStruc.LkAhd_DataLookAheadPtr          = (UINT8 *)(macData) + headerSize;
	((ECB*)rcb)->ECB_Fragment[0].FragmentAddress = lkAheadStruc.LkAhd_DataLookAheadPtr;
	lkAheadStruc.LkAhd_DataLookAheadLen				  = pktSize;
	lkAheadStruc.LkAhd_FrameDataSize               = pktSize;
	((ECB*)rcb)->ECB_DataLength                    = pktSize;
	((ECB*)rcb)->ECB_DriverWorkspace.DWs_i16val[1] = pktSize;
	((ECB*)rcb)->ECB_Fragment[0].FragmentLength = pktSize;
	lkAheadStruc.LkAhd_BoardNumber	= configTable->MLIDCFG_BoardNumber;
	((ECB*)rcb)->ECB_BoardNumber     = configTable->MLIDCFG_BoardNumber;
   COPY_ADDR( &((ECB*)rcb)->ECB_ProtocolID, &lkAheadStruc.LkAhd_ProtocolID );
	lkAheadStruc.LkAhd_PadAlignBytes1            = 0;
	lkAheadStruc.LkAhd_PadAlignBytes2            = 0;
	lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
	lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
	lkAheadStruc.LkAhd_PriorityLevel = (UINT32)(macData->MH_AccessControl >> 5);

	if((lkAheadStruc.LkAhd_PriorityLevel != 0) && (!(receptiontype & (ECB_GLOBALERROR | ECB_MACFRAME))))
	{
		receptiontype |= ECB_RX_PRIORITY;
	}

	lkAheadStruc.LkAhd_Reserved = NULL;

	lkAheadStruc.LkAhd_DestType                  = receptiontype;
   ((ECB *)rcb)->ECB_DriverWorkspace.DWs_i16val[0]
									     = VALUE_TO_LOHI_UINT16((UINT16)receptiontype);

   /*Swap the Source address if needed*/
   if( frameData->BitSwapAddressFlag != 0 )
    {
      /*We need to swap the address*/
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[0]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[1] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[1]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[2] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[2]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[3] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[3]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[4] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[4]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[5] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[5]];

   	   /* Clear the Source Routing RII bit. */
		lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] &= ~0x01;
    }	/* Swap Address LSB Mode */
   else
	 {
		COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress, &macData->MH_Source );
		/* Clear the Source Routing RII bit. */
		lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] &= ~SOURCE_ROUTING_BIT;
		rcvStatus |= PA_NONCAN_ADDR_BIT;
	 } /* Don't Swap MSB Mode */

	COPY_ADDR( &((ECB*)rcb)->ECB_ImmediateAddress,
				  &lkAheadStruc.LkAhd_ImmediateAddress );

	lkAheadStruc.LkAhd_PktAttr      = rcvStatus;
	((ECB*)rcb)->ECB_PreviousLink   = (ECB*)rcvStatus;
	lkAheadStruc.LkAhd_PreFilledECB = (ECB*)rcb;

	/*add to count header size*/
	statsTable->Gen_TotalRxOkByteLow += lkAheadStruc.LkAhd_DataLookAheadLen +
													lkAheadStruc.LkAhd_MediaHeaderLen;

	/* Check for overflow in the lower 32 bits. */
   if (statsTable->Gen_TotalRxOkByteLow <
		  (lkAheadStruc.LkAhd_DataLookAheadLen + lkAheadStruc.LkAhd_MediaHeaderLen))
		statsTable->Gen_TotalRxOkByteHigh++;

	/*See if the packet was not Wanted.*/
	if( CLSL_GetStackECB(&lkAheadStruc) == ODISTAT_OUT_OF_RESOURCES )
		statsTable->Gen_NoECBs++;

	/* Get a new RCB to return to the HSM (The LSL always keeps it) */
	rcb = (RCB *)CMSMAllocateRCB(driverData, newRcbSize, &physicalAddress);
	return( rcb );

}	/*End CTokenTSMProcessGetRCB*/

/**************************************************************************\
**
*	Name:
*		RCB	*CTokenTSMGetRCB(
*					DRIVER_DATA					*driverData,
*					UINT8                   *packetHdr,
*					UINT32						pktSize,
*					UINT32						rcvStatus,
*					UINT32						*startByte,
*					UINT32						*numBytes);
*
*	Description:
*		This routine is called by HSM's that have read in at least the
*		look ahead portion of the received packet. If a protocol stack
*		wants the packet, an RCB is returned with a fragment list for
*		which points to the buffers that the HSM should place the rest
*		of the packet.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		packetHdr -	      first look ahead bytes of the packet
*		pktSize -			Total Size of packet including MAC header
*		rcvStatus -			Error Status of received packet
*		startByte -			Where TSM will store bytes how many bytes
*								from the beginning of the packet that the
*								HSM should skip before copying the remainder
*								of the packet.
*		numBytes -			Remaining number of bytes for HSM to copy
*								into fragment list.
*
*	Values returned:
*		RCB with fragment list to fill in if accepted
*		NULL if no protocol stack wanted it. HSM should discard packet.
*
*	Implied parameters:
*		NONE
*
*  Notes:
**
\**************************************************************************/
RCB		*CTokenTSMGetRCB(
				DRIVER_DATA          *driverData,
				UINT8                *packetHdr,
				UINT32 					pktSize,
				UINT32					rcvStatus,
				UINT32 					*startByte,
				UINT32 					*numBytes)
{
	FRAME_DATA				*frameData = NULL;
	SHARED_DATA	   		*sharedData;
	MEDIA_DATA				*mediaData;
	CONFIG_TABLE			*configTable;
   GENERIC_STATS        *statsTable;
	ECB						*ecb;
   FRAGMENT_STRUCT      *fragptr;
   TOKEN_MEDIA_HEADER	*macData, *lookAheadData;
	GROUP_ADDR_LIST_NODE	*mcEntry;
	void						*routeInfo;
	UINT32					receptiontype = 0;
   UINT32               fragCount;
   UINT32               sumOfFragBytes;
	UINT32					headerSize = 0;
	UINT32					srFieldLen = 0;
	LOOKAHEAD				lkAheadStruc;

		/* Initialize the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
   statsTable = sharedData->CMSMGenericStatsPtr;
   macData = (TOKEN_MEDIA_HEADER *)packetHdr;

	/*Make sure this is NOT a packet that we sent*/
	if( !((*(UINT32*) &macData->MH_Source.nodeAddress[2] ^
         *(UINT32*) &sharedData->CMSMPhysNodeAddress [2]) ||
        ((*(UINT16*) &macData->MH_Source.nodeAddress[0] & ~SOURCE_ROUTING_BIT) ^
         *(UINT16*) &sharedData->CMSMPhysNodeAddress [0])) )
		/* This packet was sent by us. */
	{
		if( (macData->MH_FrameControl & FC_FRAME_TYPE_BITS) != FC_MAC_FRAME )
		{
			/* This packet was sent by us and is a non-MAC frame. */						
			return( NULL );
		}
	}

   /* Check to see if MAC packet. */
   if( (macData->MH_FrameControl & FC_FRAME_TYPE_BITS) == FC_MAC_FRAME )
	 {
      /* This is a MAC packet. */
      headerSize = SIZE_MACTOK;
      receptiontype = ECB_MACFRAME;
      configTable = sharedData->CMSMDefaultVirtualBoard;
  		frameData = CONFIG_TO_MSM(configTable);
		COPY_ADDR( &lkAheadStruc.LkAhd_ProtocolID, &MACToken );
    }
	else
	 {
	 		/* This is a NON-MAC Frame */
         /*Check if Source Routing included*/
         if( macData->MH_Source.nodeAddress[0] & SOURCE_ROUTING_BIT )
          {
            /*This packet has Source Route Info*/
            lookAheadData = (TOKEN_MEDIA_HEADER *) ((UINT32) macData +
										(UINT32) (macData->MH_DSAP & SOURCE_SIZE_MASK));
            headerSize = (UINT32) (macData->MH_DSAP & SOURCE_SIZE_MASK);
				srFieldLen = headerSize;
   			receptiontype |= DT_SOURCE_ROUTE;
          }
         else
          {
            lookAheadData = macData;
				 /*  headerSize is initilized to 0; */
          }

		/* ************************************************************** */
       	/* Determine the frame type of the packet.								*/
        /*	TOKEN-RING:      AC FC DA, SA, RI, DSAP, SSAP, 0x03, data...	*/
		/*	TOKEN-RING_SNAP: AC FC DA, SA, RI, 0xAA, 0xAA, 0x03, PID, data.*/
		/* ************************************************************** */

     	if( ((*(UINT16*) &lookAheadData->MH_DSAP) == 0xAAAA) &&
             (lookAheadData->MH_Ctrl0  == 0x3) )
		{
			/*type is SNAP*/

            headerSize += SIZE_TOKENSNAP;

			/*Get frame data space and copy 5 bytes following DSAP, SSAP and Ctl0 to PID.*/	

	   	   	if( configTable = sharedData->CMSMVirtualBoardLink [TSNAP_INTERNALID] )
			{
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
		   	    	frameData = CONFIG_TO_MSM( configTable );
				   	lkAheadStruc.LkAhd_ProtocolID.protocolID[0] = 0;	
 					lkAheadStruc.LkAhd_ProtocolID.protocolID[1] = lookAheadData->MH_SNAP [0];	
					COPY_UINT32( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[2]),
							    &(lookAheadData->MH_SNAP [1]));
				}
			}/* End of frame loaded and not shutdown */
		}	/* Packet is Snap Type */

		else	
		{
			/* type is TOKEN-RING (802.2 over 802.5) */	

            headerSize += SIZE_TOKEN; /* Default to TYPE I */

			if (!(lookAheadData->MH_Ctrl0 & S_OR_U_FORMAT) || !(lookAheadData->MH_Ctrl0 & U_FORMAT))	
			{
	   	   		++headerSize;				/* Change to type 2	 */
				receptiontype |= DT_8022_TYPE_II;	
			}	

			else
				receptiontype |= DT_8022_TYPE_I;

  	   		/*Get frame data space and copy DSAP field to PID. */	

	   	   	if( configTable = sharedData->CMSMVirtualBoardLink[TOKEN_INTERNALID] )
			{
				if (!(configTable->MLIDCFG_SharingFlags & MS_SHUTDOWN_BIT))
				{
				   	frameData = CONFIG_TO_MSM( configTable );
					*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
					lkAheadStruc.LkAhd_ProtocolID.protocolID[4] = 0;
					lkAheadStruc.LkAhd_ProtocolID.protocolID[5] = lookAheadData->MH_DSAP;
				}
			}/* End of frame loaded and not shutdown */
		} /* Packet is Type Token-Ring */

		if( frameData == NULL )
	    {
        	configTable = sharedData->CMSMDefaultVirtualBoard;
			frameData = CONFIG_TO_MSM(configTable);
			rcvStatus |= PAE_NOT_ENABLED_BIT;
			statsTable->Gen_RxMiscError++;

			/* zero out Protocol ID field */

			*(UINT32*)(lkAheadStruc.LkAhd_ProtocolID.protocolID) = 0;
			*(UINT16*)&(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]) = 0;
		}

		/****************************************/
		/* We have determined the frame type.	 */
		/* Check the destination address.		 */
		/* Check if multicast bit set.			 */
		/****************************************/
	   if( macData->MH_Destination.nodeAddress[0] & MULTICASTBIT )
   	 {
  	   	statsTable->Gen_TotalGroupAddrRx++;

	   	/*Check if BroadCast Remember it is in High-Low Order */
  		  	if( ((*(UINT16 *)&(macData->MH_Destination.nodeAddress[0]) == 0x00C0) ||
				   (*(UINT16 *)&(macData->MH_Destination.nodeAddress[0]) == 0xFFFF)) &&				
				(*(UINT32 *)&(macData->MH_Destination.nodeAddress[2]) == BROADCAST) )	
				receptiontype |= ( ECB_BROADCAST | ECB_MULTICAST );
   		else
	   	 {

				/* if Functional bit is zero, then better be Group-functional */
				if ( (!((*(UINT16 *)(&macData->MH_Destination.nodeAddress[2])) & 0x0080))
							&&
				     ( *(UINT16*)macData->MH_Destination.nodeAddress != 0x00C0 ) )

					receptiontype |= ECB_GLOBALERROR;

				/* We don't need to filter...
					if Not in PromiscuousMode AND HardwareFiltering is ON. (Both Bits.) */
				if ( (!mediaData->PromiscuousMode) &&
					  (configTable->MLIDCFG_Flags & (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)) ==
					  (MF_SOFT_FILT_GRP_BIT | MF_GRP_ADDR_SUP_BIT)
					)
				 {
					receptiontype |= ECB_MULTICAST;
				 }
				else
				 {
					/*Check if multicast is in our table.*/
		  			if ( FindAddressInMCTable( sharedData,
														(NODE_ADDR *)macData->MH_Destination.nodeAddress,
														&mcEntry) == ODISTAT_SUCCESSFUL )
			   			receptiontype |= ECB_MULTICAST;
				   else
					 {
						receptiontype |= ECB_MULTICASTREMOTE;

						/* If not in Promiscous Mode filter out the Packet */
 						if( !mediaData->PromiscuousMode )
							return( (RCB *)NULL );
					 } /* Remote Multicast, Drop if not in Promiscuous Mode. */
				 }	/* Need to check if in Table */
		 	 }	/* Packet has a Group Address --Not Broadcast*/
  	    } /* Packet is a Multicast type */
  		else
       {
      	if( mediaData->PromiscuousMode )
			 {
		   	if( ADDR_EQUAL( &macData->MH_Destination,
									 &configTable->MLIDCFG_NodeAddress) )
	   		  	receptiontype |= ECB_UNICASTDIRECT;
   		   else
					receptiontype |= ECB_UNICASTREMOTE;
			 }
 	   	else
   	   	receptiontype |= ECB_UNICASTDIRECT;
       }	/* Packet has Unicast Address */
	 }	/* End Non-MAC Frame */

  	/*Make sure that packet is not too small. */
	if( headerSize > pktSize )
	 {
		pktSize = headerSize + 3;
		rcvStatus |= PAE_MALFORMED_BIT;
      statsTable->Gen_RxTooSmall++;
	 }

   /* Check to see if route module loaded. */
  	if( configTable->MLIDCFG_SourceRouting )
    {
	   /* Inform route module of This route. */
      routeInfo = &macData->MH_DSAP;
   					configTable->MLIDCFG_SourceRouting(
	   				configTable->MLIDCFG_BoardNumber,
		   			macData->MH_Source.nodeAddress,
			   		&routeInfo,
				   	FALSE);
  	 }
	else		/* Watch out for missing Route.NLM	*/
	 {
		if( srFieldLen > 2 )
			receptiontype = DT_SOURCE_ROUTE;		/* Make this bit exclusive	*/

	 }	/* End- Missing Route.NLM */

	/* Check for Other Errors. */
	if( pktSize != UNUSED )
	 {
	   /* Make sure that packet is not to large if it is not a pipelined adapter. */
		if( pktSize > configTable->MLIDCFG_MaxFrameSize )
		 {
			rcvStatus |= PAE_TOO_BIG_BIT;	
		   statsTable->Gen_RxTooBig++;
		 }

	   /* Only bump counters if not a pipelined adapter. */
	   /* bump Recieve count, add to count header size*/
		if( (UINT8)rcvStatus )
			receptiontype = ECB_GLOBALERROR;	/* This is an exclusive bit 	*/
		else
		{
		   ++(statsTable->Gen_TotalRxPackets);	/* Count No Error Packets */

	  	   /*add to count header size*/
  		   statsTable->Gen_TotalRxOkByteLow += pktSize; /*contains media hdr len*/

			/* Check for overflow in the lower 32 bits. */
	  	   if (statsTable->Gen_TotalRxOkByteLow < pktSize)
  		   	statsTable->Gen_TotalRxOkByteHigh++;
		}

		/* Start filling in the LookAhead Structure */
	   lkAheadStruc.LkAhd_FrameDataSize = pktSize - headerSize;
	   if( configTable->MLIDCFG_LookAheadSize < (pktSize - headerSize) )
			lkAheadStruc.LkAhd_DataLookAheadLen = configTable->MLIDCFG_LookAheadSize;
	   else
			lkAheadStruc.LkAhd_DataLookAheadLen = pktSize - headerSize;

	 } /* Not PipeLine Adapter */
	else
	 {
		if( (UINT8)rcvStatus )
			receptiontype = ECB_GLOBALERROR;	/* This is an exclusive bit 	*/
		/* Start filling in the LookAhead Structure */
	   lkAheadStruc.LkAhd_FrameDataSize = pktSize;
		lkAheadStruc.LkAhd_DataLookAheadLen = configTable->MLIDCFG_LookAheadSize;
	 } /* Pipeline Adpter */

	/*fill in the rest of the lookahead structure.*/
	lkAheadStruc.LkAhd_MediaHeaderPtr	= (UINT8 *)macData;
	lkAheadStruc.LkAhd_MediaHeaderLen	= headerSize;
  	lkAheadStruc.LkAhd_DataLookAheadPtr	= (UINT8 *)(macData) + headerSize;
	lkAheadStruc.LkAhd_BoardNumber = configTable->MLIDCFG_BoardNumber;
	lkAheadStruc.LkAhd_PadAlignBytes1 = 0;
	lkAheadStruc.LkAhd_PadAlignBytes2 = 0;
	lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
	lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
	lkAheadStruc.LkAhd_PriorityLevel = (UINT32)(macData->MH_AccessControl >> 5);

	if((lkAheadStruc.LkAhd_PriorityLevel != 0) && (!(receptiontype & (ECB_GLOBALERROR | ECB_MACFRAME))))
	{
		receptiontype |= ECB_RX_PRIORITY;
	}

	lkAheadStruc.LkAhd_Reserved = NULL;
	lkAheadStruc.LkAhd_PreFilledECB   = NULL;
	lkAheadStruc.LkAhd_DestType = receptiontype;

   /*Swap the Source address if needed*/
   if( frameData->BitSwapAddressFlag != 0 )
    {
      /*We need to swap the address*/
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[0]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[1] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[1]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[2] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[2]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[3] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[3]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[4] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[4]];
      lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[5] =
         MSMBitSwapTable [macData->MH_Source.nodeAddress[5]];

	   /* Clear the Source Routing RII bit. */
		lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] &= ~0x01;
    }	/* Swap Address LSB Mode */
   else
	 {
		COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress, &macData->MH_Source );
		/* Clear the Source Routing RII bit. */
		lkAheadStruc.LkAhd_ImmediateAddress.nodeAddress[0] &= ~SOURCE_ROUTING_BIT;
		rcvStatus |= PA_NONCAN_ADDR_BIT;
	 } /* Don't Swap MSB Mode */

	lkAheadStruc.LkAhd_PktAttr	 = rcvStatus;
	mediaData->RxStatus			 = rcvStatus;

	/****************************************/
	/* See if anyone wants the packet.      */
	/****************************************/

	if( CLSL_GetStackECB(&lkAheadStruc) == ODISTAT_SUCCESSFUL )
	{
		ecb = lkAheadStruc.LkAhd_ReturnedECB;

		if(ecb) /* Protocol Stack provied an ECB */
		{
			ecb->ECB_Status = (UINT16)ODISTAT_SUCCESSFUL;

  		   /* Calculate the sum of all the fragment buffers sizes. */
	  	   sumOfFragBytes = 0;
  		   fragCount = ecb->ECB_FragmentCount;
	  	   fragptr = ecb->ECB_Fragment;
  		   while( fragCount )
  	       {
				sumOfFragBytes += fragptr->FragmentLength;
	  		   fragptr++;
  			   fragCount--;
  	       }

	  		*startByte  = lkAheadStruc.LkAhd_FrameDataStartCopyOffset + headerSize;
  			*numBytes   = lkAheadStruc.LkAhd_FrameDataBytesWanted;
	  	   if (pktSize != UNUSED) /* The packet has been fully received. */
  		   {
			   /*******************************************************************/
	  		   /* Check if the number of bytes requested by the protocol stack is */
  			   /* greater than the actual number of bytes in the frame. */
			   /*******************************************************************/
	  		   if (*numBytes > (pktSize - *startByte))
  					   *numBytes = pktSize - *startByte;

	  		   /* Check for possible overflow. */
  			   if (*numBytes > sumOfFragBytes)
  			   {
  			   	*numBytes = sumOfFragBytes;
  			   	ecb->ECB_Status = (UINT16)ODISTAT_RX_OVERFLOW;
			   }

	  		   /*set Data length field in ECB.*/
  			   ecb->ECB_DataLength = *numBytes;
	    	   ecb->ECB_DriverWorkspace.DWs_i16val[0]
   						     = VALUE_TO_LOHI_UINT16((UINT16)receptiontype);
			   ecb->ECB_DriverWorkspace.DWs_i16val[1] = pktSize - headerSize;
			}	/* End Packet Fully received */

	  	   else /* pipelined */
  	       {
  		   		ecb->ECB_DataLength = 0;
	  		   	/* Check for possible overflow.  If the number of bytes wanted is */
  			   	/* <= the sumOfFragBytes either the protocol stack wants the packet */
  			   	/* and the packet fits perfectly in the fragments or the protocol */
  		   		/* wants only the number of bytes requested even if the frame data */
	  		    /* is greater than the nubmer of bytes wanted.  It is only overflow */
  			    /* if the number of bytes wanted is greater than the sumOfFragbytes.*/
  				if( *numBytes > sumOfFragBytes )
     		 	{
  	   				*numBytes = sumOfFragBytes;

	  				/* Set ECB_DataLength field bit 30 to protocol wants all data. */
	     			ecb->ECB_DataLength |= PSTK_WANTS_FRAME;
  			   	 }

	  		    /* Save bytes to skip in bits b23-b16 of the ECB DataLength field. */
				ecb->ECB_DataLength |= (((*startByte) << 16) & HSM_TOLD_TO_SKIP);

	     		/* Save bytes to copy in bits b15-b0 of the ECB DataLength field. */
  			   	ecb->ECB_DataLength |= ((*numBytes) & HSM_TOLD_TO_COPY);

	  		    /* Save the lkahd destination type in first part of DriverWorkspace */
  				ecb->ECB_DriverWorkspace.DWs_i16val[0]
												= VALUE_TO_LOHI_UINT16((UINT16)receptiontype);

	     		/* Save the media header length the second half of driver workspace */
  			   	ecb->ECB_DriverWorkspace.DWs_i16val[1] =
					            (UINT16)lkAheadStruc.LkAhd_MediaHeaderLen;
		   } /* end PipeLine Support */

	  	   /* Mark Ecb as having gone through look ahead. */
  		   ecb->ECB_DataLength |= HSM_HAS_BEEN_IN_GETRCB;
  			return( (RCB*)ecb );
		}

		else /* Protocol Stack got everything it needed from the lookahead */
		{
		   return( (RCB *)NULL );
		}
 	}	/* Packet Wanted */

	else  /* The packet was not wanted. */
    {
  	   ++(statsTable->Gen_NoECBs);

  	   /* If the frame came from a pipeline adapter and was not wanted, this */
  	   /* is the only chance we have to bump any stat counters. */

  	   if (pktSize == UNUSED)
  	    {
  		   /*bump Recieve count */

			if( !((UINT8)rcvStatus) )
  			   ++(statsTable->Gen_TotalRxPackets);

			/*add to the count the header size + data length */

			statsTable->Gen_TotalRxOkByteLow += (lkAheadStruc.LkAhd_MediaHeaderLen + lkAheadStruc.LkAhd_DataLookAheadLen);

			/* Check for overflow in the lower 32 bits. */

	        if ( statsTable->Gen_TotalRxOkByteLow < lkAheadStruc.LkAhd_MediaHeaderLen + lkAheadStruc.LkAhd_DataLookAheadLen)
				  	statsTable->Gen_TotalRxOkByteHigh++;

		 } /* End of Pipeline Adapter Counter Fest */
	 } /* No Ecb for this RCB */
   return( (RCB *)NULL );  /* Null tells the HSM to discard */

}	/*End CTokenTSMGetRCB*/

/**************************************************************************\
**
** PROC NAME:     CTokenTSMFastRcvComplete
**
**                This routine is called by HSM's that call CTokenTSMGetRCB
**                and have filled out the receive fragment list returned
**                by CTokenTSMGetRCB.
**
** Parameters:    IN     DRIVER_DATA         *driverData
**                IN     RCB                 *rcb
**
**                driverData       - Pointer to CHSMs Adapter Data Space
**                ecb             - Pointer to Event Control Block
**
** Return Value:  NONE
**
** See Also:
**
\**************************************************************************/
void		CTokenTSMFastRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb)
{
	SHARED_DATA			      *sharedData;
	MEDIA_DATA					*mediaData;
   ECB                     *ecb;

   ecb = (ECB *)rcb;

   /* Check if the ecb has already had a look ahead built for it. */
   if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
    {
		ecb->ECB_DataLength &= ~HSM_HAS_BEEN_IN_GETRCB; /* clear the bit */
	   sharedData = DADSP_TO_CMSMADSP(driverData);
   	mediaData = sharedData->CMSMMediaDataPtr;
	   ecb->ECB_PreviousLink = (ECB*)mediaData->RxStatus;

		/* ECB_Status was set in GetRCB */
	   /* ecb->ECB_Status = ODISTAT_SUCCESSFUL; */

	   CLSL_FastHoldEvent(ecb);
    }
	else	/* This ECB came from a ecb aware adapter. */
	 {
      CTokenTSMECBRcvComplete(driverData, ecb);
#ifdef OS_NIOS
		CLSL_ServiceEvents();
#endif
	 }

	return;
}	/*End	CTokenTSMFastRcvComplete*/

/**************************************************************************\
**
** PROC NAME:     CTokenTSMFastRcvCompleteStatus
**
**                This function is called by pipelined HSM's that call
**                CTokenTSMGetRCB with an unknown packet size/status.  Since
**                the packet has been fully received off of the wire, the
**                HSM can now give us the full size of the packet and the
**                error status.
**
** Parameters:    IN     DRIVER_DATA          *driverData
**                IN     RCB                  *rcb
**                IN     UINT32               packetLength
**                IN     UINT32               packetStatus
**
**                driverData      - Pointer to CHSMs Adapter Data Space
**                ecb             - Pointer to Event Control Block
**                packetLength    - The packet length
**                packetStatus    - The packet status
**
** Return Value:  NONE
**
** See Also:
**
\**************************************************************************/
void		CTokenTSMFastRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus)
{
   ECB                     *ecb;

	/* Set up Locals */
   ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
		/* Fix up all the necessary ECB fields to complete the receive. */
		CTokenTSMPipelineRcvComplete(driverData, ecb, packetLength, packetStatus);
		CLSL_FastHoldEvent(ecb);
	 }
	else
	 {
		/* This is an old style ecb aware adapter. */
		/* Setup for calling the nominal ECBRcvComplete Function */
		ecb->ECB_PreviousLink = (ECB *)(packetStatus);
		CTokenTSMECBRcvComplete(driverData, ecb);
#ifdef OS_NIOS
		CLSL_ServiceEvents();
#endif
	
    } /* Old style ecb aware adapter. */

	return;
}	/*End	CTokenTSMFastRcvCompleteStatus*/

/**************************************************************************\
**
** PROC NAME:     CTokenTSMRcvComplete
**
**                This function is called by HSM's that call CTokenTSMGetRCB
**                and have filled out the receive fragment list returned by
**                CTokenTSMGetRCB.
**
** Parameters:    IN     DRIVER_DATA          *driverData
**                IN     RCB                  *rcb
**
**                driverData       - Pointer to CHSMs Adapter Data Space
**                ecb             - Pointer to Event Control Block
**
** Return Value:  NONE
**
** See Also:
**
\**************************************************************************/
void		CTokenTSMRcvComplete(
				DRIVER_DATA          *driverData,
            RCB                  *rcb)
{
	SHARED_DATA			      *sharedData;
	MEDIA_DATA					*mediaData;
   ECB                     *ecb;

   ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if (ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB)
	 {
   	sharedData = DADSP_TO_CMSMADSP(driverData);
   	mediaData = sharedData->CMSMMediaDataPtr;

		ecb->ECB_DataLength &= ~HSM_HAS_BEEN_IN_GETRCB; /* clear the bit */
	   ecb->ECB_PreviousLink = (ECB*)mediaData->RxStatus;

		/* ECB_Status was set in GetRCB */
	   /* ecb->ECB_Status = ODISTAT_SUCCESSFUL; */

	   CLSL_HoldEvent(ecb);
    }
   else
    {
      CTokenTSMECBRcvComplete(driverData, ecb);
    }

	return;
}	/*End	CTokenTSMRcvComplete*/

/**************************************************************************\
**
** PROC NAME:     CTokenTSMRcvCompleteStatus
**
**                This routine is called by pipelined HSM's that call
**                CTokenTSMGetRCB with an unknown packet size/status.  Since
**                the packet has been fully received off of the wire, the
**                HSM can now give us the fully size of the packet and the
**                error status.
**
** Parameters:    IN     DRIVER_DATA          *driverData
**                IN     RCB                  *rcb
**                IN     UINT32               packetLength
**                IN     UINT32               packetStatus
**
**                driverData       - Pointer to CHSMs Adapter Data Space
**                ecb             - Pointer to Event Control Block
**                packetLength    - The packet length
**                packetStatus    - The packet status
**
** Return Value:  NONE
**
** See Also:
**
\**************************************************************************/
void		CTokenTSMRcvCompleteStatus(
				DRIVER_DATA          *driverData,
            RCB                  *rcb,
				UINT32					packetLength,
				UINT32					packetStatus)
{
   ECB                     *ecb;

	/* Setup Locals */
   ecb = (ECB *)rcb;

	/* Check if the ecb has already had a look ahead build for it. */
	if( ecb->ECB_DataLength & HSM_HAS_BEEN_IN_GETRCB )
	 {
		/* Fix up all the necessary ECB fields to complete the receive. */
		CTokenTSMPipelineRcvComplete( driverData, ecb, packetLength, packetStatus );
		CLSL_HoldEvent( ecb );
	 }
	else
	 {
		/* This is an old style ecb aware adapter. */
		/* Setup for calling the nominal ECBRcvComplete Function */
		ecb->ECB_PreviousLink = (ECB *)(packetStatus);
		CTokenTSMECBRcvComplete(driverData, ecb);

    } /* Old style ecb aware adapter. */

	return;
}	/*End	CTokenTSMRcvCompleteStatus*/

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CTokenTSMECBRcvComplete(
*					DRIVER_DATA					*driverData,
*					ECB							*ecb,
*
*	Description:
*		This routine is called from CTokenTSMFastRcvComplete of
*		CTokenTSMRcvComplete for an ECB aware adapter.  A lookahead has
*		not been created an presented to the LSL.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		ecb -					Pointer to a completely filled out ecb
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
*                                                                       *
************************************************************************/
void	CTokenTSMECBRcvComplete(
				DRIVER_DATA		*driverData,
				ECB 				*ecb)
{
	SHARED_DATA				*sharedData;
	GENERIC_STATS			*statsTable;
	MEDIA_DATA				*mediaData;
	TOKEN_MEDIA_HEADER   *macData;
	GROUP_ADDR_LIST_NODE	*mcEntry;
	LOOKAHEAD				lkAheadStruc;

	/* Set up locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData = sharedData->CMSMMediaDataPtr;
   statsTable = sharedData->CMSMGenericStatsPtr;

  	/*fill in the lookahead structure.*/
   lkAheadStruc.LkAhd_PreFilledECB = ecb;
	lkAheadStruc.LkAhd_MediaHeaderPtr = (UINT8 *)&(ecb->ECB_Fragment[1]);
	macData = (TOKEN_MEDIA_HEADER *)lkAheadStruc.LkAhd_MediaHeaderPtr;

   /* Check to see if physical fragments are being used. */
   if (((DADSP_TO_CMSMADSP(driverData)->CMSMDefaultVirtualBoard->MLIDCFG_ModeFlags) & MM_FRAGS_PHYS_BIT))
      lkAheadStruc.LkAhd_MediaHeaderLen =
	      (UINT32)(((UINT32)(ecb->ECB_Fragment[0]).FragmentAddress) -
           			((UINT32)CMSMGetPhysical(lkAheadStruc.LkAhd_MediaHeaderPtr)));
   else
      lkAheadStruc.LkAhd_MediaHeaderLen =
	      (UINT32)(((UINT32)(ecb->ECB_Fragment[0]).FragmentAddress) -
           			((UINT32)lkAheadStruc.LkAhd_MediaHeaderPtr));

   lkAheadStruc.LkAhd_DataLookAheadPtr =
			         (UINT8 *)((UINT32)(lkAheadStruc.LkAhd_MediaHeaderPtr) +
														lkAheadStruc.LkAhd_MediaHeaderLen);
   ecb->ECB_Fragment[0].FragmentAddress = lkAheadStruc.LkAhd_DataLookAheadPtr;
   lkAheadStruc.LkAhd_DataLookAheadLen = ecb->ECB_DataLength;
   lkAheadStruc.LkAhd_FrameDataSize = ecb->ECB_DataLength;
   lkAheadStruc.LkAhd_BoardNumber = ecb->ECB_BoardNumber;
   COPY_UINT32( &lkAheadStruc.LkAhd_ProtocolID,	&ecb->ECB_ProtocolID	);
   COPY_UINT16( &(lkAheadStruc.LkAhd_ProtocolID.protocolID[4]),
					 &(ecb->ECB_ProtocolID.protocolID[4]) );
   COPY_ADDR( &lkAheadStruc.LkAhd_ImmediateAddress,
				  &ecb->ECB_ImmediateAddress );
   lkAheadStruc.LkAhd_PadAlignBytes1 = 0;
   lkAheadStruc.LkAhd_PadAlignBytes2 = 0;
	lkAheadStruc.LkAhd_FrameDataStartCopyOffset = 0;
	lkAheadStruc.LkAhd_FrameDataBytesWanted = 0;
	lkAheadStruc.LkAhd_PriorityLevel = (UINT32)(macData->MH_AccessControl >> 5);
	lkAheadStruc.LkAhd_Reserved = NULL;
   lkAheadStruc.LkAhd_PktAttr = (UINT32)ecb->ECB_PreviousLink;
   lkAheadStruc.LkAhd_DestType =
							 GET_LOHI_UINT16(&ecb->ECB_DriverWorkspace.DWs_i8val[0]);

	/* Check to see if we need to filter/fix-up DestType bits	*/
	if ( (mediaData->PromiscuousMode) &&
		  (!(sharedData->CMSMDefaultVirtualBoard->MLIDCFG_Flags
		  	  & MF_SOFT_FILT_GRP_BIT)) &&
		  (!(lkAheadStruc.LkAhd_DestType & ECB_BROADCAST)) &&
		  (lkAheadStruc.LkAhd_DestType & (ECB_MULTICAST | ECB_MULTICASTREMOTE))
		)
	{
		/* We need to do additional filterering */
		if (FindAddressInMCTable(sharedData,
										 (NODE_ADDR *)macData->MH_Destination.nodeAddress,
										 &mcEntry) == ODISTAT_SUCCESSFUL)
		{
			lkAheadStruc.LkAhd_DestType |= ECB_MULTICAST;
			lkAheadStruc.LkAhd_DestType &= ~ECB_MULTICASTREMOTE;
		}
		else
		{
			lkAheadStruc.LkAhd_DestType |= ECB_MULTICASTREMOTE;
			lkAheadStruc.LkAhd_DestType &= ~ECB_MULTICAST;
		}
	}


   /*Bump Counters*/
	if( !((UINT16)ecb->ECB_PreviousLink)
			|| ((UINT16)ecb->ECB_PreviousLink == PA_NONCAN_ADDR_BIT) )
	   ++(statsTable->Gen_TotalRxPackets);

   /*add to count header size*/
   statsTable->Gen_TotalRxOkByteLow += ecb->ECB_DataLength +
     												lkAheadStruc.LkAhd_MediaHeaderLen;

  	/* Check for overflow in the lower 32 bits. */
	if (statsTable->Gen_TotalRxOkByteLow < (ecb->ECB_DataLength +
									         	lkAheadStruc.LkAhd_MediaHeaderLen))
	{
	  	statsTable->Gen_TotalRxOkByteHigh++;
	}

   statsTable->Gen_TotalGroupAddrRx +=
								ecb->ECB_DriverWorkspace.DWs_i8val[0] & ECB_MULTICAST;

	/* Check if Source Routing included */
	if (macData->MH_Source.nodeAddress[0] & SOURCE_ROUTING_BIT)
	 {
		/*This packet has Source Route Info*/
		lkAheadStruc.LkAhd_DestType |= DT_SOURCE_ROUTE; /* Set bit */

		/* Check to see if route module loaded. */
		if( !(sharedData->CMSMDefaultVirtualBoard->MLIDCFG_SourceRouting)
				 	&& ((macData->MH_DSAP & SOURCE_SIZE_MASK) > 2) )
			lkAheadStruc.LkAhd_DestType = DT_SOURCE_ROUTE; /* Exclusive bit case */

	 }  /* Sroute info included in Packet. */

	/* Update ECB DestType Field in DriverWorkspace */
	/* Save the lkahd destination type in first part of DriverWorkspace */
	ecb->ECB_DriverWorkspace.DWs_i16val[0] =
											 VALUE_TO_LOHI_UINT16((UINT16)lkAheadStruc.LkAhd_DestType);

	/* Update ECB fields to ensure set correctly */
	ecb->ECB_DriverWorkspace.DWs_i16val[1] = ecb->ECB_DataLength;

	if (CLSL_GetStackECB(&lkAheadStruc) == ODISTAT_OUT_OF_RESOURCES)
	   ++(statsTable->Gen_NoECBs);

	return;
} /* End CTokenTSMECBRcvComplete */

/***********************************************************************\
*                                                                       *
*	Name:
*		void	CTokenTSMPipelineRcvComplete(
*					DRIVER_DATA					*driverData,
*					RCB							*rcb,
*					UINT32						packetLength,
*					UINT32						packetStatus);
*
*	Description:
*		This routine is called by one of the RcvCompleteStatus routines for
*		ecbs coming from pipelined HSM's with an unknown packet size/status.
*		Since the packet has been fully received off of the wire, the HSM can
*		now give us the full size of the packet and the error status.
*
*	Parameters in:
*		driverData -		HSM adapter data space
*		rcb -					Pointer to receive control block
*		packetLength -		Length of entire packet, including media header
*		packetStatus -		Error status of received packet
*
*	Values returned:
*		NONE
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
************************************************************************/
void	CTokenTSMPipelineRcvComplete(
				DRIVER_DATA		*driverData,
				ECB 				*ecb,
				UINT32			packetLength,
				UINT32			packetStatus)
{
	SHARED_DATA					*sharedData;
 	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statsTable;
	UINT32						skipBytes;
	UINT32						bytesToCopy;

	/* Set up Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
  	mediaData = sharedData->CMSMMediaDataPtr;
	statsTable = sharedData->CMSMGenericStatsPtr;
   configTable = sharedData->CMSMDefaultVirtualBoard;

	/* Update Status from what we found before. */
   packetStatus |= mediaData->RxStatus;

	/* check for Packet Too Big. */
	if( packetLength > configTable->MLIDCFG_MaxFrameSize )
	 {
		packetStatus |= PAE_TOO_BIG_BIT;	
	   statsTable->Gen_RxTooBig++;
	 }

  	/* check for Packet Too Small. */
	if( ecb->ECB_DriverWorkspace.DWs_i16val[1] > packetLength )
	 {
		packetStatus |= PAE_MALFORMED_BIT;
      statsTable->Gen_RxTooSmall++;
	 }

	/* Save the total frame data size. */
	ecb->ECB_DriverWorkspace.DWs_i16val[1] = packetLength -
													ecb->ECB_DriverWorkspace.DWs_i16val[1];
	/*bump stat counts*/
   /* statsTable->Gen_TotalGroupAddrRx was incremented in GetRCB */
	/* count all Bytes received --even for Error Packets? */
	statsTable->Gen_TotalRxOkByteLow += packetLength;

  	/* Check for overflow in the lower 32 bits. */
	if (statsTable->Gen_TotalRxOkByteLow < packetLength)
		statsTable->Gen_TotalRxOkByteHigh++;

	/* Only count the Good packets received */
	if( !((UINT8)packetStatus) )
	 {
		++(statsTable->Gen_TotalRxPackets);
		ecb->ECB_Status = ODISTAT_SUCCESSFUL; /* assume success */

		/* Check for Possible Overflow */
		skipBytes = (ecb->ECB_DataLength & HSM_TOLD_TO_SKIP) >> 16;
		bytesToCopy = ecb->ECB_DataLength & HSM_TOLD_TO_COPY;

		if ((skipBytes + bytesToCopy) >= packetLength)
		 {
			if (skipBytes > packetLength)
				ecb->ECB_DataLength = 0;	/* packet too small */
			else
				ecb->ECB_DataLength = packetLength - skipBytes;
		 }	
		else /* may be an overflow packet */
		 {
			if (ecb->ECB_DataLength & PSTK_WANTS_FRAME)
			 {
				/* The protocol stack wanted the entire frame but did not supply */
				/* enough data buffer space. */
				ecb->ECB_Status = ODISTAT_RX_OVERFLOW;
			 }
			/* Clear bits 31 through 16. */
			ecb->ECB_DataLength &= HSM_TOLD_TO_COPY;
	 	 }	/* end of Maybe Overflow */
	 } /* Successful packets */
	else
	 {
		ecb->ECB_Status = ODISTAT_CANCELED; /* Indicate Failure */
		ecb->ECB_DriverWorkspace.DWs_i16val[0] = ECB_GLOBALERROR;

		/* Clear bits 31 through 16. */
		ecb->ECB_DataLength &= HSM_TOLD_TO_COPY;
		
	 }	/* End Error Packets */

	/* Set ECB packetStatus */
	ecb->ECB_PreviousLink = (ECB *)packetStatus;
	return;
} /* End CTokenTSMPipelineRcvComplete */

/**************************************************************************\
**
** PROC NAME:     CTokenTSMGetHSMIFLevel
**
**                This routine will return the TSM<->HSM interface version
**                level.  Current Version Level is 1.11.
**
** Parameters:    NONE
**
** Return Value:  HSM Interface Level
**
\**************************************************************************/
UINT32	CTokenTSMGetHSMIFLevel()
{
	return( 111 );
}	/*End CTokenTSMGetHsmIfLevel*/

/**************************************************************************\
**
*	Name:
*		ODISTAT CTokenTSMRegisterHSM(
*					DRIVER_PARM_BLOCK			*hsmParameters,
*					CONFIG_TABLE				**configTable)
*
*	Description:
*		This routine is called by HSM's at the beginning of DriverInit.
*		It must be called at init time.
*
*	Parameters in:
*		hsmParameters -	Pointer to the Adapter Data Space.
*		configFDSP -		Location to return config table.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL	If TSM/MSM registers HSM successfully
*
*	Implied parameters:
*		NONE
*
*  Notes:
*
** See Also:      <HSM>       DriverInit
**                TOKENGLUE   TokenTSMRegisterHSM
**                MSM         CMsmRegisterTsm
**
\**************************************************************************/
ODISTAT	CTokenTSMRegisterHSM(
				DRIVER_PARM_BLOCK	   *hsmParameters,
				CONFIG_TABLE			**configTable)
{
	return( CMSMRegisterTSM( hsmParameters, &MediaParameters, configTable) );
}

/**************************************************************************\
**
** PROC NAME:     CInitializeTokenTSM
**
**                This routine is called by the OS (definition file's START
**                routine) when it is being loaded.
**
** Parameters:    IN    struct LoadDefinitionStructure   *moduleHandle
**                IN    struct ScreenStruct              *screenID
**
**                moduleHandle   -  Pointer this modules handle
**                screenID       -  Pointer to output screen handle (console)
**
** Return Value:  0        = successful
**                non-zero = unsuccessful
**
** See Also:      <HSM>       DriverInit
**                TOKENGLUE   TokenTSMRegisterHSM
**                MSM         CMsmRegisterTsm
**
\**************************************************************************/
UINT32   CInitializeTokenTSM (
            struct _MODULE_HANDLE_     *moduleHandle,
            SCREEN_HANDLE     *screenID)
{
	UINT32		messageCount=0;
	UINT32		languageID;
	UINT8		*helpFile;
	UINT32      majorVersion, minorVersion;

   screenID       = screenID;                /* stop compiler warnings */

   Token8022Index = Token8022Index;
   TokenSNAPIndex = TokenSNAPIndex;

	/* Initialize fields in the TSM's Configuration Table */

	CMSMGetNLMVersionInfo ( moduleHandle, &majorVersion,
   	                     &minorVersion, NULL, NULL, NULL, NULL, NULL);
   	CTokenTSMConfigTable.CTSMCFG_ModuleMajorVersion = majorVersion;
   	CTokenTSMConfigTable.CTSMCFG_ModuleMinorVersion = minorVersion;

#ifdef SVR
	CTokenTSMConfigTable.CTSMCFG_SystemFlags = CTokenTSMConfigTable.CTSMCFG_SystemFlags | CTSM_CFG_SERVER_BIT;
#else
	CTokenTSMConfigTable.CTSMCFG_SystemFlags = CTokenTSMConfigTable.CTSMCFG_SystemFlags | CTSM_CFG_CLIENT_BIT;
#endif


	return CMSMReturnMessageInformation (
					moduleHandle,
					&DriverMessages,
					&messageCount,
					&languageID,
					&helpFile);
}

/**************************************************************************\
**
** PROC NAME:     CRemoveTokenTSM
**
**                This routine is called by the OS when it is being
**                unloaded.  It calls UnRegisterEventNotification with
**                ExitToDOSID as the parameter and returns.
**
** Parameters:    NONE
**
** Return Value:  NONE
**
** See Also:      <HSM>       DriverInit
**                TOKENGLUE   TokenTSMRegisterHSM
**                MSM         CMsmRegisterTsm
**
\**************************************************************************/
void     CRemoveTokenTSM (
            void)
{
   return;
}	/* End CRemoveTokenTSM */

/***********************************************************************\
*                                                                       *
*	Name:
*		ODISTAT CTokenTSMBuildTransmitControlBlock(
*							DRIVER_DATA		*driverData,
*							ECB				*ecb,
*							TCB				**tcb,
*						UINT32			*pktSize);
*
*	Description:
*			This routine is called by the CHSM when it is ready to send a
*	priority Packet that has been queued by a call to DriverPriorityQueueSupport.
*	The CHSM calls this routine to convert an ECB to a TCB.
*	
*
*	Parameters in:
*		driverData	-		CHSM adapter data space
*		ecb			-		Pointer to a Transmit ECB
*
*	Parameters returned:
*		tcb			-     Pointer to a Pointer to the TCB to send.
*		pktSize		-		Padded Length to send.
*								Will be equal to DataLen in Token.
*
*	Values returned:
*		ODISTAT_SUCCESSFUL
*							TCB pointer is valid. The CHSM should transmit the TCB.
*
*		ODISTAT_OUT_OF_RESOURCES
*							A TCB was not available.  The CHSM must not call this
*							routine with more outstanding TCB's than it set in the
*							CMSMPriorityTxFreeCount variable. The ECB is returned
*							to the CHSM.  The CHSM must either call this function
*							again after a TCB resource is available, or return the
*							ECB via the <CTSM>CancelPrioritySend.
*
*		ODISTAT_PACKET_UNDELIVERABLE
*							A TCB was available, but the ECB created a packet that
*							was too large for the media.  The ECB was returned to
*							the LSL and a TCB was not allocated.
*
*
*  Notes:
*			The CHSM should be aware of the number of TCBs available to the MLID
*			for Priority Sends.  The CTSM allocates a number of TCBs based on
*			the sum of the CMSMTxFreeCount and the CMSMPriorityTxFreeCount.
*			The CHSM must not have more outstanding PriorityTCB's than the CHSM
*			set in CMSMPriorityTxFreeCount during DriverInit. If the CHSM makes
*			this call when there are no TCB's available, an
*			ODISTAT_OUT_OF_RESOURCES is returned.
*
*			The CHSM need not do size checking on the resultant TCB.  If the
*			packet generated is too large for the Media, this function returns
*			ODISTAT_PACKET_UNDELIVERABLE after it returns the ECB to the LSL
*			and does not return a TCB to the CHSM.
*
*			The CHSM should not change the CMSMTxFreeCount for any TCB obtained
*			for a Priority transmit.  An internal counter for Priority support
*			resources should be maintained by the CHSM.
*
*			If the HSM needs to cancel this ECB it should use
*			CTokenTSMSendComplete or CTokenTSMFastSendComplete.
*
************************************************************************/
ODISTAT CTokenTSMBuildTransmitControlBlock(
						DRIVER_DATA		*driverData,
						ECB				*ecb,
						TCB				**tcb,
						UINT32			*pktSize)
{
	FRAME_DATA					*frameData;
	SHARED_DATA					*sharedData;
	MEDIA_DATA					*mediaData;
	CONFIG_TABLE				*configTable;
	GENERIC_STATS				*statTable;
	UINT32						oldState;

	/* Set Up Local variables */
	frameData  = (FRAME_DATA *)ecb->ECB_DriverWorkspace.DWs_pval;
	configTable = &frameData->ConfigTable;
	sharedData = DADSP_TO_CMSMADSP(driverData);
	mediaData  = sharedData->CMSMMediaDataPtr;
	statTable = sharedData->CMSMGenericStatsPtr;

	/* are we ECB aware? */
	if (sharedData->CMSMDPBlockPtr->DriverSendWantsECBs & 1)
	{  /* Yes, so not much to do */

		/* Does CHSM want physical addresses? */

	    if (sharedData->CMSMNeedContiguousECB)
		{
			/* Protect Data */
			CMSMAcquireProtection( sharedData->CMSMProtectionObject, &oldState );

			*tcb = (TCB *)CMSMGetPhysicalECB (driverData, ecb);

			CMSMReleaseProtection( sharedData->CMSMProtectionObject, oldState );
		}
	    else *tcb = (TCB *) ecb;

	    /* All done withe the ECB aware case */
	    return (ODISTAT_SUCCESSFUL);
	}    	/* End ECB Aware case */

	/* Protect Data */
	CMSMAcquireProtection( sharedData->CMSMProtectionObject,	&oldState );

		/* Do we have a free TCB? */
	if( mediaData->TCBHead )
	{

		/* Does CHSM want physical addresses? */

	    if (sharedData->CMSMNeedContiguousECB)
		{
			ecb = CMSMGetPhysicalECB (driverData, ecb);
		}

		/* Get a TCB and create the MAC Header. */
		*tcb = mediaData->TCBHead;
		mediaData->TCBHead = (TCB *)((*tcb)->TCB_DriverWS[0]);

		/* Put the BitSwap flag in the TCB */
		(*tcb)->TCB_DriverWS[0] = frameData->BitSwapAddressFlag;

		/***********************************************************
			Due to Problem with Needing to checkin, Pass in FrameData
			as a cast through the ShareData Param, and it will recast
			in the MediaSend Routine.   _SROUTE_BUG_
		********************************************************** */
		*pktSize = frameData->SendRoutine( (SHARED_DATA *)frameData, ecb, *tcb );

		/* Check if Need ???? */
		if (sharedData->CMSMNeedContiguousECB)
		{
			(*tcb)->TCB_DriverWS[1] = (UINT32)ecb->ECB_PreviousLink;
			(*tcb)->TCB_DriverWS[2] = (UINT32)CMSMGetPhysical(&((*tcb)->TCB_MediaHeader) );
		}
						
		/* check that the packet is not to big. */
		if( *pktSize <= configTable->MLIDCFG_MaxFrameSize )
		{
			/* Bump Queue Depth */
			++(statTable->Gen_Qdepth);
		}
		else
		{
     		/* Return TCB and link into TCB list. */
	    	(*tcb)->TCB_DriverWS[0] = (UINT32)mediaData->TCBHead;
   	    	mediaData->TCBHead = *tcb;
			++(statTable->Gen_TxTooBig);

			/* Set ECB status. */
			ecb->ECB_Status = (UINT16)ODISTAT_BAD_PARAMETER;
			CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );

			CLSL_FastSendComplete( ecb );
			*tcb = NULL;
			return( ODISTAT_PACKET_UNDELIVERABLE );
		} /* Packet turned out to be too big */
	} /* A free Resource was available. */
	else
	{
		CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
		*tcb = NULL;
		return( ODISTAT_OUT_OF_RESOURCES );
	}

	CMSMReleaseProtection( sharedData->CMSMProtectionObject,	oldState );
	return( ODISTAT_SUCCESSFUL );
}	/* BuildTransmitControlBlock */

/***********************************************************************\
*                                                                       *
*   Name:
*       void CTokenTSMCancelPrioritySend(
*									 DRIVER_DATA		*driverData,
*                           ECB             *ecb);
*
*   Description:
*           This routine is called by the CHSM when it is canceling an ECB
*           originally accepted to transmit via DriverPriorityQueueSupport
*           The CHSM calls this routine to cancel/return an ECB that has
*           not been Sent.
*
*
*   Parameters in:
*       ecb         -       Pointer to a Transmit ECB
*
*   Values returned:
*       None
*
*  Notes:
*
*
************************************************************************/
void CTokenTSMCancelPrioritySend(
									 DRIVER_DATA		*driverData,
                            ECB             *ecb)
{
	SHARED_DATA					*sharedData;
	GENERIC_STATS				*statTable;

	/* Setup the Locals */
	sharedData = DADSP_TO_CMSMADSP(driverData);
	statTable = sharedData->CMSMGenericStatsPtr;

	++(statTable->Gen_TxMiscError);
    ecb->ECB_Status = (UINT16) ODISTAT_CANCELED;
    CLSL_FastSendComplete( ecb );
    return;

} /* End CTokenTSMCancelPrioritySend */


/***********************************************************************\
*
*   Name:
*      ODISTAT     CTokenTSMGetConfigInfo(
*                  		void 	*configInfo,
*                  		UINT32  *nBytes);
*
*   Description:
*                 	Allows a C HSM to get the configuration information
*					for the C TSM, including specification and module
*					versions.
*
*   Parameters in:
*					configInfo
*                    	A pointer to a buffer used to receive the returned
*                    	configuration information.  The caller needs to be
*						sure that the buffer is at least nBytes bytes long.
*
*                   nBytes
*                       Pointer to the requested number of bytes to be returned into the
*						buffer.
*
*   Values returned:
*					ODISTAT_SUCCESSFUL
*						The configuration information of nBytes was
*						successfully returned in the buffer.
*
*					ODISTAT_BAD_PARAMETER
*
*						The nBytes requested was larger than the actual
*						configuration information available.
*
*   Implied parameters:
*                   nBytes
*                       Pointer to the number of bytes returned in the
*						buffer.
*
*   Notes:
*					The configuration information is returned in the format
*					defined by CTSM_CONFIG_TABLE.
*
************************************************************************/

ODISTAT	CTokenTSMGetConfigInfo (
         void 	*configInfo,
         UINT32 *nBytes)
{
	UINT32	configTableSize;

	configTableSize = sizeof(CTSM_CONFIG_TABLE);

	if (*nBytes > configTableSize)
	{
		CMSMmemcpy(configInfo, &CTokenTSMConfigTable, configTableSize);

		*nBytes = configTableSize;

		return ( ODISTAT_BAD_PARAMETER );
	}

	CMSMmemcpy(configInfo, &CTokenTSMConfigTable, *nBytes);

	return ( ODISTAT_SUCCESSFUL );

}  /* End CTokenTSMGetConfigInfo */
